#!/usr/bin/env python
# -*- coding: utf-8 -*-
#This script will build all the neccessary dlls from models and
#move them to the target directory.
#Usage as noted below

import sys
import os
import ConfigParser
from shutil import move
from subprocess import list2cmdline
import logging
logging.basicConfig(level=logging.INFO)

msg = "Running from %s" % os.getcwd()
logging.info(msg)
if len(sys.argv) != 5:
    msg = "DLL creation and move needs four parameters: "\
            "modelroot, simoroot, pythonexec and dll.ini"
    logging.error(msg)
    raise AttributeError('bad parameters!')
simo_root, model_root, py_exe, dll_ini = sys.argv[1:]

modelpath = os.path.join(model_root, 'Finland', 'c')
targetpath = os.path.join(simo_root, 'simulator', 'models', 'prediction')

dll_ini = os.path.abspath(dll_ini)

config = ConfigParser.RawConfigParser()
config.read(dll_ini)

def _fix_path(path, filename=None):
    path = os.path.relpath(os.path.join('..', '..', path, 'c'))
    if not filename:
        return path
    else:
        return os.path.join(path, filename)

msg = "Compiling files in %s" % modelpath
logging.info(msg)
#We move to the modelpath, which hosts dll_batch.py that does the
#actual compiling work for us. Look there (and make_dll.py) for further info.
sys_id = str.lower(sys.platform[:3])
if sys_id == 'win':
    os.chdir(modelpath)
    os.system(list2cmdline((py_exe, 'dll_batch.py', dll_ini)))
else:
    os.chdir(modelpath)
    os.system(' '.join((py_exe, 'dll_batch.py', dll_ini)))

used_paths = set()
for section in config.sections():
    data = dict(config.items(section))
    path = data.get('path')
    if path:
        used_paths.add(_fix_path(path))
    else:
        used_paths.add(_fix_path('Finland'))
    
msg = "Compiling complete\n"
msg += '##################################################################'
logging.info(msg)

msg = "Moving compiled files to %s" % targetpath
logging.info(msg)
for path in used_paths:
    files = os.listdir(path)
    
    for file in files:
        if file.endswith('dll'):
            pfile = os.path.join(path, file)
            if os.path.lexists(os.path.join(targetpath, file)):
                os.remove(os.path.join(targetpath, file))
            msg = "moving %s" % pfile
            logging.info(msg)
            move(pfile, targetpath)
    
msg = "Move complete!\n"
msg += '##################################################################'
logging.info(msg)

