#!/usr/bin/env python
# encoding: utf-8
"""
This will run the main program. The output will be
redirected to output.test which is tested against output.cmp
*output_tester.py*

Usage: output_tester.py exec program

Copyright (c) 2009 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""
import subprocess
import os, sys
import time
sys.path.append('dev/utils')
import compare
from ConfigParser import ConfigParser
import logging
logging.basicConfig(level=logging.INFO)

OUTPUT = 'dev/utils/output.test'
CMP = 'dev/utils/output.cmp'
MAIN_MOD = 'src/runner.py'
MAIN_INI = 'simulator/test/runner_with_warnings.ini'
EXEC_PATH = 'bin/python'
if not os.path.exists(EXEC_PATH):
    EXEC_PATH = 'python'

def run_sim():
    """
    run simulation with test ini
    """
    msg = 'Running program... (this will take several minutes...)'
    logging.info(msg)
    exe = EXEC_PATH + ' ' + MAIN_MOD + ' ' + MAIN_INI + ' 1>%s 2>&1' % OUTPUT
    p = subprocess.Popen(exe, shell=True)
    start = time.time()
    # Drop time taken while waiting
    while True:
        result = p.poll()
        if result is None:
            time.sleep(10)
            msg = 'Time spent: %.1f min' % round((time.time()-start)/60.0, 1)
            logging.info(msg)
            sys.stderr.flush()
        else:
            msg = 'Time spent: %.1f min' % round((time.time()-start)/60.0, 1)
            break
    if result > 0:
        msg = 'Program run failed!'
        logging.error(msg)
        sys.exit(1)
    msg = 'Program run successfully!'
    logging.info(msg)

def compare_logs():
    """
    compare simulation run logs
    """
    msg = 'Comparing runtime logs...'
    logging.info(msg)
    nose_exec = 'nosetests'
    if os.path.exists('bin/root_test'):
        nose_exec = 'bin/root_test'
    log_res = os.system('%s --with-doctest --doctest-extension=cmp %s' \
                        % (nose_exec, CMP))
    msg = 'Run logs exited with %d' % log_res
    logging.info(msg)
    return log_res

def compare_output_files(output_dir, outputs):
    """
    compare different output type result files
    """
    bl_res = True
    or_res = True
    ag_res = True
    ex_res = True
    msg = 'Comparing output results...'
    logging.info(msg)
    if 'by_level' in outputs:
        msg = 'By level:'
        logging.info(msg)
        bl_res = compare.main(os.path.join(output_dir, outputs['by_level']),
                         'dev/utils/output_test_files/by_level.txt',
                         'by_level')
        msg = 'matches ... %s' % bl_res
        logging.info(msg)
    if 'operation_result' in outputs:
        msg = 'Operation results:'
        logging.info(msg)
        or_res = compare.main(os.path.join(output_dir,
                                           outputs['operation_result']),
                   'dev/utils/output_test_files/operation_result.txt',
                   'opres')
        msg = 'matches ... %s' % or_res
        logging.info(msg)
    if 'aggregation' in outputs:
        msg = 'Aggregation:'
        logging.info(msg)
        ag_res = compare.main(os.path.join(output_dir, outputs['aggregation']),
                   'dev/utils/output_test_files/aggregation.txt',
                   'aggr')
        msg = 'matches ... %s' % ag_res
        logging.info(msg)
    if 'expression' in outputs:
        msg = 'Expression:'
        logging.info(msg)
        ex_res = compare.main(os.path.join(output_dir, outputs['expression']),
                   'dev/utils/output_test_files/expression.txt',
                   'expres')
        msg = 'matches ... %s' % ex_res
        logging.info(msg)
    return bl_res, or_res, ag_res, ex_res

def main(output_dir, outputs):
    run_sim()
    log_res = compare_logs()
    out_res = compare_output_files(output_dir, outputs)
    bl_res, or_res, ag_res, ex_res = out_res
    if log_res == 0 and bl_res and or_res and ag_res and ex_res:
        sys.exit(0)
    else:
        sys.exit(1)

if __name__ == '__main__':
    cp = ConfigParser()
    cp.read(MAIN_INI)
    base = cp.get('program_execution', 'base_folder')
    output_dir = cp.get('output', 'output_dir')
    output_dir = os.path.join(base, output_dir)
    output_formats = cp.get('output', 'output_format').split(';')
    output_files = cp.get('output', 'output_filename').split(';')
    outputs = dict(zip(output_formats, output_files))
    main(output_dir, outputs)
