#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys
sys.path.append('src')
from simo import __version__

install_requires = ['spyne<2.10', 'cmd2<0.6.7', 'redis', 'celery',
                    'minimock', 'psycopg2',
                    'pysqlite', 'guppy', 'cython', 'pygooglechart', 'lxml',
                    'numpy<1.9.0', 'zope.interface', 'pyparsing<2.0',
                    'python-dateutil<2.0', 'interlude']

# Get text from README.txt
readme_text = file('README.txt', 'rb').read()

setup(name          = 'SIMO',
      version       = __version__,
      description   = 'Simulation and optimisation framework for natural '\
                      'resource planning',
      license       = 'GPL2',
      keywords      = 'simulation optimization',
      author        = 'Simosol Oy',
      author_email  = 'info@simosol.fi',
      maintainer    = 'Simosol Oy',
      maintainer_email  = 'info@simosol.fi',
      url   = 'http://www.simo-project.org',
      long_description = readme_text,

      #package structure
      packages=['']+find_packages('src', 'simulator'),
      package_dir={'':'src'},
      install_requires = install_requires,
      entry_points=("""
          [console_scripts]
          runner=runner:main
          builder=builder:main
          logger=logger:main
          f2py=numpy.f2py:main
          celeryd=celery.bin.celeryd:main
          server=simoserver.simoserver:main
          server_celeryd=simoserver.run_celeryd:main
          server_control=simoserver.simoctrl:main
          """),
      classifiers   = [
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GPL License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Scientific/Engineering :: Simulation',
      ],
     )

