import os
import traceback
import pdb
import logging
import time
from optparse import OptionParser
import numpy
import threading
import celery

base = os.getcwd()
import celery_tasks
os.chdir(base)
import runner
import runner2

def store_run(runner_obj, ids, res, done, id_states, group, group_count,
              logger):
    done.add(ids)
    tname = threading.currentThread().name
    logger.debug(str(res.state))
    datarows, logrows = res.result #wait(0.1)
    logger.info('%s: Done on %s' % (tname, str(ids),))
    logger.info('%s: Storing group %d of %d to database...' % (tname, group+1,
                                                               group_count))
    try:
        runner_obj.result_db.store_datarows(datarows)
        id_states[ids] = 'success'
    except StandardError, e:
        logger.exception('%s: Failed storing datarows for %s!' \
                         % (tname, str(ids),))
        id_states[ids] = 'failed'
    try:
        runner_obj.log_db.store_datarows(logrows)
    except StandardError, e:
        logger.exception('%s: Failed storing logrows!' % (tname,))
    logger.info('%s: Finished storing %s' % (tname, str(ids),))
    res.forget()

def main_memory(inifile=None):
    """
    Function: main

    Runner for SIMO computation. Reads initialization file and does data
    import, simulation, optimization and reporting according to the
    initialization definitions.
    """
    #NB: set Numpy to ignore warnings
    try:
        numpy.seterr(all='ignore')
    except:
        pass

    logger = logging.getLogger('celery_runner')
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler()
    formatter = logging.Formatter("[%(asctime)s: %(levelname)s/%(name)s] "\
                                  "%(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    handler = logging.FileHandler('celery_runner.log')
    handler.setLevel(logging.INFO)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.propagate = False



    inilogger = runner._get_ini_logger()
    # NOTE: do_wipe means that we HAVE to kill the workers, they will break
    # due to not knowing the db state afterwards.
    runner_obj = runner2.get_runner(True, inifile, None, do_wipe=True)
    cont = True
    try:
        if runner_obj.input_db.db_class == 'POSTGRESQL':
            runner_obj.input_db.db.execute(\
                                    'SET LOCAL synchronous_commit TO OFF;')
        if cont and runner_obj.cfg['program_execution.simulate']:
            main_level = runner_obj.ctrl.main_level
            
            ids = [str(i) for i in range(210)]
            dpath = runner_obj.cfg['import.data_dir']
            datafiles = runner_obj.cfg['import.data_file']
            opath = runner_obj.cfg['import.operations.operation_dir']
            ofile = runner_obj.cfg['import.operations.operation_file']
            f = open(os.path.join(dpath, datafiles[0]))
            try:
                data_string = f.read()
            finally:
                f.close()
            data_list = data_string.split('\n')
            split_index = 0
            id_indexes = [str(r*5) for r in range(1, 42)]
            data_strings = []
            id_ind = 0
            split_root = 0
            for i, row in enumerate(data_list):
                split_index = i
                if id_ind == len(id_indexes):
                    continue
                if row.startswith(id_indexes[id_ind]):
                    data_strings.append('\n'.join(data_list\
                                                  [split_root:split_index]))
                    print 'Added', data_strings[-1]
                    split_root = split_index
                    id_ind += 1
            data_strings.append('\n'.join(data_list[split_root:split_index]))
            print 'Added', data_strings[-1]
#            data_string1 = '\n'.join(data_list[:split_index])
#            data_string2 = '\n'.join(data_list[split_index:])
            f = open(os.path.join(opath, ofile))
            try:
                op_string = f.read()
            finally:
                f.close()
            #cont = runner_obj.simulate()
            ress = []
            print id_indexes
            print data_strings
            for i in range(41):
                ress.append([tuple(ids[i*5:i*5+5]), 
                             celery_tasks.run_data.delay([data_strings[i]],
                                                         op_string,
                                                         clear_old=True)])
#            ress.append([tuple(ids[:100]), 
#                         celery_tasks.run_data.delay([data_string1],
#                                                     op_string,
#                                                     clear_old=True)])
#            ress.append([tuple(ids[100:]), 
#                         celery_tasks.run_data.delay([data_string2], '',
#                                                     clear_old=True)])
            done = set()
            id_states = {}
            group_count = len(ress)
            last_done_count = 0
            cooler = time.time()
            simstart = time.time()
            try:
                while True:
                    done_count = len(done)
                    if done_count == group_count \
                        and threading.activeCount()==1:
                        # make sure only main thread is running
                        break
                    if last_done_count != done_count:
                        logger.debug('Finished %d during last loop' \
                                    % (done_count-last_done_count,))
                        last_done_count = done_count
                    drops = set()
                    # small cooldown so we don't blaze at 100% processor...
                    time.sleep(1.0)
                    for i, ids_res in enumerate(ress):
                        ids, res = ids_res
                        if ids in done:
                            continue
                        if time.time()-cooler > 60:
                            done_count = len(done)
                            remaining = (group_count - done_count)
                            average = (time.time()-simstart)/max(done_count, 1)
                            logger.info('Checking group %d of %d '\
                                        '(%d remaining, '\
                                        'current average %.1fs per group)' \
                                        % (i+1, group_count, remaining,
                                           average))
                            cooler = time.time()
                        if res.state == 'SUCCESS':
                            if threading.activeCount() > 2:
                                continue # limit thread count, handle next time
                            thread = threading.Thread(target=store_run,
                                                      args=(runner_obj, ids,
                                                            res, done,
                                                            id_states, i,
                                                            group_count,
                                                            logger))
                            thread.name = 'Thread-%d' % threading.activeCount()
                            thread.start()

                        elif res.status == 'FAILURE':
                            done.add(ids)
                            id_states[ids] = 'failed'
                            logger.error('Failed on %s got %s' \
                                         % (str(ids), str(res.result)))
                                                                    #str(ex)))
                            logger.error(res.traceback)
                            res.forget()
            finally:
                for ids, res in ress:
                    if ids in done:
                        continue
                    logger.warning('Revoking un-finished id group task %s' \
                                   % (str(ids),))
                    res.revoke()
                    res.forget()

            done_count = len(done)
            average = (time.time()-simstart)/max(done_count, 1)
            logger.info('Done with simulating the set of %d in %.1f min, '\
                        'avg %.fs per group; '\
                        'getting run end states...' \
                        % (group_count, (time.time()-simstart)/60.0, average))
            logger.info(str(id_states.items()))

        if cont and runner_obj.cfg['program_execution.optimize']:
            runner_obj.optimize()
        if runner_obj.input_db.db_class == 'POSTGRESQL':
            runner_obj.input_db.db.execute(\
                                    'SET LOCAL synchronous_commit TO ON;')
        if cont and runner_obj.cfg['program_execution.output_data']:
            runner_obj.output_data()
    finally:
        runner_obj.close_databases()

def main(inifile=None):
    """
    Function: main

    Runner for SIMO computation. Reads initialization file and does data
    import, simulation, optimization and reporting according to the
    initialization definitions.
    """
    #NB: set Numpy to ignore warnings
    try:
        numpy.seterr(all='ignore')
    except:
        pass

    logger = logging.getLogger('celery_runner')
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler()
    formatter = logging.Formatter("[%(asctime)s: %(levelname)s/%(name)s] "\
                                  "%(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    handler = logging.FileHandler('celery_runner.log')
    handler.setLevel(logging.INFO)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.propagate = False



    inilogger = runner._get_ini_logger()
    # NOTE: do_wipe means that we HAVE to kill the workers, they will break
    # due to not knowing the db state afterwards.
    runner_obj = runner2.get_runner(True, inifile, None, do_wipe=True)
    cont = True
    try:
        if runner_obj.input_db.db_class == 'POSTGRESQL':
            runner_obj.input_db.db.execute(\
                                    'SET LOCAL synchronous_commit TO OFF;')
        if runner_obj.cfg['program_execution.import_data']:
            start = time.time()
            cont = runner_obj.import_data()
            logger.info('Finished import in %s min' \
                        % ((time.time()-start)/60,))
        if cont and runner_obj.cfg['program_execution.simulate']:
            # dropping all indices from the result db
            # TODO: enabling indices when?
            ind_set = ('link', 'branching', 'price', 'ops', 'geom')
            runner_obj.result_db.drop_indices(ind_set)

            main_level = runner_obj.ctrl.main_level
            ids = runner_obj.input_db.get_data_from_level('data', ('id',),
                                                          main_level,
                                                          distinct=True)
            ids = [i[0] for i in ids]
            id_list = runner_obj.cfg.get('simulation.id_list')
            ind_seq = runner_obj.cfg.get('simulation.index_sequence')
            if id_list:
                ids = set(ids)
                id_list = set(id_list)
                ids = ids.intersection(id_list)
            elif ind_seq is not None:
                if ind_seq[1] is None:
                    ind_seq = (ind_seq[0], len(ids) + 1)
                if ind_seq[1] > len(ids):
                    ind_seq[1] = len(ids)
                ids = ids[ind_seq[0]:ind_seq[1]+1]
            #cont = runner_obj.simulate()
            ress = []
            id_groups = [[]]
            for id in ids:
                print 'adding %s to %s' % (id, id_groups[-1])
                id_groups[-1].append(id)
                # split to max_unit_count sized id groups
                if len(id_groups[-1]) >= \
                                runner_obj.cfg['simulation.max_unit_count']:
                    id_groups.append([])
            if len(id_groups[-1]) == 0:
                id_groups.pop()
            for ids in id_groups:
                ress.append([tuple(ids), celery_tasks.run_ids.delay(ids,
                                                             clear_old=True)])
            done = set()
            id_states = {}
            group_count = len(ress)
            last_done_count = 0
            cooler = time.time()
            simstart = time.time()
            try:
                while True:
                    done_count = len(done)
                    if done_count == group_count \
                        and threading.activeCount()==1:
                        # make sure only main thread is running
                        break
                    if last_done_count != done_count:
                        logger.debug('Finished %d during last loop' \
                                    % (done_count-last_done_count,))
                        last_done_count = done_count
                    drops = set()
                    # small cooldown so we don't blaze at 100% processor...
                    time.sleep(1.0)
                    for i, ids_res in enumerate(ress):
                        ids, res = ids_res
                        if ids in done:
                            continue
                        if time.time()-cooler > 60:
                            done_count = len(done)
                            remaining = (group_count - done_count)
                            average = (time.time()-simstart)/max(done_count, 1)
                            logger.info('Checking group %d of %d '\
                                        '(%d remaining, '\
                                        'current average %.1fs per group)' \
                                        % (i+1, group_count, remaining,
                                           average))
                            cooler = time.time()
                        if res.state == 'SUCCESS':
                            if threading.activeCount() > 2:
                                continue # limit thread count, handle next time
                            thread = threading.Thread(target=store_run,
                                                      args=(runner_obj, ids,
                                                            res, done,
                                                            id_states, i,
                                                            group_count,
                                                            logger))
                            thread.name = 'Thread-%d' % threading.activeCount()
                            thread.start()

                        elif res.status == 'FAILURE':
                            done.add(ids)
                            id_states[ids] = 'failed'
                            logger.error('Failed on %s got %s' \
                                         % (str(ids), str(res.result)))
                                                                    #str(ex)))
                            logger.error(res.traceback)
                            res.forget()
            finally:
                for ids, res in ress:
                    if ids in done:
                        continue
                    logger.warning('Revoking un-finished id group task %s' \
                                   % (str(ids),))
                    res.revoke()
                    res.forget()

            done_count = len(done)
            average = (time.time()-simstart)/max(done_count, 1)
            logger.info('Done with simulating the set of %d in %.1f min, '\
                        'avg %.fs per group; '\
                        'getting run end states...' \
                        % (group_count, (time.time()-simstart)/60.0, average))
            logger.info(str(id_states.items()))

        if cont and runner_obj.cfg['program_execution.optimize']:
            runner_obj.optimize()
        if runner_obj.input_db.db_class == 'POSTGRESQL':
            runner_obj.input_db.db.execute(\
                                    'SET LOCAL synchronous_commit TO ON;')
        if cont and runner_obj.cfg['program_execution.output_data']:
            runner_obj.output_data()
    finally:
        runner_obj.close_databases()

if __name__=='__main__':
    usage = "usage: %prog [options] ini_file\n"\
            "    ini_file -- text file containing the settings for the "\
            "program run\n"
    parser = OptionParser(usage)
    parser.add_option('-d', '--debug', dest='debug', action='store_true',
                      help='run post mortem debug mode enabled')
    options, args = parser.parse_args()
    if len(args) == 0:
        parser.error('Ini file not given')
    inifile = args[0]
    if os.path.isfile(inifile) == 0:
        parser.error('Configuration file %s is not a valid file' % inifile)
    inifile = os.path.join(base, inifile)
    try:
        start = time.time()
        main_memory(inifile)
        print 'Finished run in %.2f min' % ((time.time()-start)/60.0)
    except Exception, e:
        type, value, tb = sys.exc_info()
        traceback.print_exc()
        if options.debug:
            pdb.post_mortem(tb)
