// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  CalibratedtreegrowthmodelsLibrary.cpp
//
//	Calibrated tree growth model library - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "TreegrowthmodelLibrary.h"
#include "SimoUtils.h"
#include "CalibratedtreegrowthmodelsLibrary.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>

int H_dom_growth_calib_JKK (double SP, double A, double H_gM, double TS, double LAKE, double PLANT, double SC,
						double PUB, double PALU, double HUMUS, double SEA, double ALT,
						double SCRUB, double WASTE, double STONY, double RDF,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double TS_constant = 1250;
    //Plant dummy is set to 0 for all the birch stands, so regeneration information will not be used
    PLANT=0;
	double H_dom_potential, H_dom_a5y_potential;
	double i_Hdom5;
	//Calculate stratum's dominant height
	//First calculate H_dom at the beginning of the period
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Dominant_height_pine_with_bias_corr_Hynynenym(A, TS_constant, ALT, LAKE, SEA, SC, STONY, PALU, HUMUS, RDF, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Dominant_height_spruce_with_bias_corr_Hynynenym(A, TS_constant, LAKE, SEA, SC, STONY, PALU, HUMUS, ALT, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_birch_with_bias_corr_Hynynenym(A, TS_constant, LAKE, PLANT, SC, SP, PUB, PALU, HUMUS, SEA, ALT, SCRUB, WASTE, STONY,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
    H_dom_potential = *modelresult;

	if (ret == 0)
		return ret;

	//Calculate H_dom after 5 years
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Dominant_height_after_5_years_pine_with_bias_corr_Hynynenym(A, TS_constant, ALT, LAKE, SEA, SC, STONY, PALU, HUMUS, RDF, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Dominant_height_after_5_years_spruce_with_bias_corr_Hynynenym(A, TS_constant, LAKE, SEA, SC, STONY, PALU, HUMUS, ALT, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_after_5_years_birch_with_bias_corr_Hynynenym(A, TS_constant, LAKE, PLANT, SC, SP, PUB, PALU, HUMUS, SEA, ALT, SCRUB, WASTE, STONY,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	H_dom_a5y_potential = *modelresult;

	if (ret == 0)
		return ret;

	//Calculate H_dom growth during 5 years
	i_Hdom5 = H_dom_a5y_potential - H_dom_potential;

	//Restrict Hdom growth if it's too high
	if (SP==3||SP==5)
	{
            if (H_gM>30)
               i_Hdom5 = i_Hdom5*0.2;
            else if (H_gM>26)
               i_Hdom5 = i_Hdom5*0.5;
            else
                i_Hdom5 = i_Hdom5*0.8;
            if (i_Hdom5>5)
               i_Hdom5 = 5;
    }
	else if (SP==4||SP==6||SP==7||SP==9)
    {
            if (H_gM>28)
               i_Hdom5 = i_Hdom5*0.1;
            else if (H_gM>24)
               i_Hdom5 = i_Hdom5*0.5;
            else
                i_Hdom5 = i_Hdom5*0.8;
            if (i_Hdom5>3.5)
               i_Hdom5 = 3.5;
    }
        //Restrict Hdom growth in SCRUB and WASTE lands
        if (SCRUB==1)
        {	
        	i_Hdom5 = 0.65*i_Hdom5;
        }
        if (WASTE==1)
        {	
        	i_Hdom5 = 0.5*i_Hdom5;
        }

	*nres = 1;
	*modelresult = i_Hdom5;
	return ret;

}

int Grow_ba_peatland_calib_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT,
						double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA,
						double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING,
						double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb,
						double BA_max_pb, double MANAGEMENT_REGION, double growth_bending_age, double Age, double i_ba_peat_factor,
						double decrease_factor, int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double TS_constant = 1250;
    double hdom_ref, hdom_ts, hdom1, hdom2, hdom3;
    double f1,f2,f3,f4,f5,f6,f7,f8;

    //Calculate Hdom limits for growth
    if (SP==1||SP==8)
    {
    	if (SC <= 2)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 1.86; //1.85
		f2 = 1.3; //1.28
		f3 = 1.17; //1.13
		f4 = 0.68; //0.67
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 2.02; //1.92,1.95,2.0
		f2 = 1.25; //1.14,1.17,1.2
		f3 = 1.09; //1.0,1.02,1.07
		f4 = 0.51; //-,0.45,0.5
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 4)
	{
		hdom_ref = 53.24-0.154*TS_constant+0.000173*pow(TS_constant,2)-0.00000005554*pow(TS_constant,3);
		hdom_ts = 53.24-0.154*TS+0.000173*pow(TS,2)-0.00000005554*pow(TS,3);
		hdom1 = hdom_ts * (11/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 2.09;
		f2 = 2.15;
		f3 = 1.14;
		f4 = 0.7;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 5)
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.66-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 1.94;
		f2 = 1.25;
		f3 = 0.69;
		f4 = 0.53;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.66-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 1.6;
		f2 = 1.0;
		f3 = 0.5;
		f4 = 0.4;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 2)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (17/hdom_ref);
		hdom3 = hdom_ts * (21/hdom_ref);
		f1 = 1.0;  //0.85,0.92
		f2 = 0.75; //0.6,0.65
		f3 = 0.36;
		f4 = 0.28;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (14/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 0.9;  //0.76,0.83
		f2 = 0.68; //0.59,0.64
		f3 = 0.4; //-,0.36
		f4 = 0.24;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (19/hdom_ref);
		f1 = 0.74;
		f2 = 0.59;
		f3 = 0.41;
		f4 = 0.28;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 3)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 124-0.3788*TS_constant+0.0003934*pow(TS_constant,2)-0.000000122*pow(TS_constant,3);
		hdom_ts = 124-0.3788*TS+0.0003934*pow(TS,2)-0.000000122*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		f1 = 1.9;
		f2 = 0.9;
		f3 = 0.58;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
	else if (SC == 3)
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8.5/hdom_ref);
		hdom2 = hdom_ts * (15/hdom_ref);
		f1 = 1.8;
		f2 = 0.9;
		f3 = 0.4;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
	else
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		f1 = 1.2;
		f2 = 0.7;
		f3 = 0.4;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
    }
    else
    {
    	if (SC <= 2)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 1.76;
		f2 = 1.02;
		f3 = 0.75;
		f4 = 0.42;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 1.68;
		f2 = 0.83;
		f3 = 0.68;
		f4 = 0.4;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 1.85;
		f2 = 1.05;
		f3 = 0.78;
		f4 = 0.38;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }


	//Calculate tree basal area growth on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_peatland_pine_with_bias_corr_Hynynenym(d, BA, BA_L, SC, TS_constant, PDR, FUSC, SUBGROUP, ALT, SEA, SCRUB, UNDRAINED, WASTE,
		      SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 4)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 5)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_peatland_spruce_with_bias_corr_Hynynenym(d, BA_prosNs, BA_L, SC, TS_constant, PDR, H_dom, ALT, LAKE, UNDRAINED, WASTE,
		      SUBGROUP, SCRUB, SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.25;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//White birch
	else if (SP == 3)
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS_constant, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
		      SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_sb)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_sb*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_sb*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_sb*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Downy birch and other deciduous species
	else
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS_constant, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
							SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.25;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_pb)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_pb*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_pb*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_pb*0.85)
			*modelresult = *modelresult*0.6;
	}

	*modelresult = *modelresult*rectFactor;

	if (ret == 0)
		return ret;

	*nres = 1;
	return ret;

}

int Tree_height_before_growing_d_peatland_calib_JKK (double d, double D_gM, double BA, double SP, double LAT, double SC,
						double SINCE_THINNING, double ALT, double BA_prosb, double TS, double Hdom,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    //Calculate Hdom limits for growth
    double TS_constant = 1250;
    double hdom_ref, hdom_ts, hdom1;
    if (SP==1||SP==8)
    {
    	if (SC <= 3)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (16/hdom_ref);
	}
	else if (SC == 4)
	{
		hdom_ref = 53.24-0.154*TS_constant+0.000173*pow(TS_constant,2)-0.00000005554*pow(TS_constant,3);
		hdom_ts = 53.24-0.154*TS+0.000173*pow(TS,2)-0.00000005554*pow(TS,3);
		hdom1 = hdom_ts * (14/hdom_ref);
	}
	else
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.7-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (13/hdom_ref);
	}
    }
    else if (SP == 2)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (18/hdom_ref);
	}
	else if (SC == 3)
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (16.5/hdom_ref);
	}
	else
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (15/hdom_ref);
	}
    }

	//Calculate tree height on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		if (Hdom >= hdom1)
		{
			ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 0.75);
		}
		else
		{
			ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
		}
	}
	//Norway spruce
	else if (SP==2)
	{
		if (Hdom >= hdom1)
		{
			ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 0.75);
		}
		else
		{
			ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}
	//Birches and other deciduous species
	else
	{
		ret = Height_peatland_birch_Hynynenym(d, D_gM, BA, LAT, ALT, BA_prosb, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1.2);
	}

	if (ret == 0)
		return ret;

	*nres = 1;
	return ret;

}

int Tree_height_after_growing_d_peatland_calib_JKK (double d, double h, double h_before, double D_gM, double BA, double SP, double LAT, double SC,
						double SINCE_THINNING, double ALT, double BA_prosb, double TS, double Hdom,
						int *nres, struct Tree_h_peatland *h_peatland, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    //Calculate Hdom limits for growth
    double TS_constant = 1250;
    double hdom_ref, hdom_ts, hdom1;
    if (SP==1||SP==8)
    {
    	if (SC <= 3)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (16/hdom_ref);
	}
	else if (SC == 4)
	{
		hdom_ref = 53.24-0.154*TS_constant+0.000173*pow(TS_constant,2)-0.00000005554*pow(TS_constant,3);
		hdom_ts = 53.24-0.154*TS+0.000173*pow(TS,2)-0.00000005554*pow(TS,3);
		hdom1 = hdom_ts * (14/hdom_ref);
	}
	else
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.7-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (13/hdom_ref);
	}
    }
    else if (SP == 2)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (18/hdom_ref);
	}
	else if (SC == 3)
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (16.5/hdom_ref);
	}
	else
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (15/hdom_ref);
	}
    }

    double *h_temp = (double *) malloc(1*sizeof(double));
	//Calculate tree height on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		if (Hdom >= hdom1)
		{
			ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 0.75);
		}
		else
		{
			ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}
	//Norway spruce
	else if (SP==2)
	{
		if (Hdom >= hdom1)
		{
			ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 0.75);
		}
		else
		{
			ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}
	//Birches and other deciduous species
	else
	{
		ret = Height_peatland_birch_Hynynenym(d, D_gM, BA, LAT, ALT, BA_prosb, SINCE_THINNING, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1.2);
	}

	if (ret == 0)
	{
        free(h_temp);
		return ret;
    }

	double h_after, i_h;
	h_after = *h_temp;
	i_h = h_after - h_before;

	//Restrict if result is too small
	if (i_h<=0.01)
	{
         i_h = 0.01;
    }

	//Update h
	h = h + i_h;

	*nres = 1;
	h_peatland[0].i_h=i_h;
	h_peatland[0].h=h;

	free(h_temp);

	return ret;

}

int Grow_h_mineral_soil_calib_JKK (double d, double cr, double SP, double i_Hdom5,
						double CR_dom, double RDF_L, double PLANT, double D_dom, double h,
						double h_max_Sp, double h_max_Ns, double h_max_sb, double h_max_pb,
						double SC, double H_dom, double growth_bending_age, double Age, double TS, double i_h_factor,
						double decrease_factor, int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

    double TS_constant = 1250;
    //Plant dummy is set to 0 for all the birch stands, so regeneration information will not be used
    PLANT=0;
    double hdom_ref, hdom_ts, hdom1, hdom2, hdom3;
    double f1,f2,f3,f4,f5,f6,f7,f8;

    //Calculate Hdom limits for growth
    if (SP==1||SP==8)
    {
    	if (SC <= 2)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 1.15; //1.08,-
		f2 = 1.13; //1.1,-
		f3 = 0.85;  //0.75,0.8
		f4 = 0.62; //-,0.58
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 1.17; //1.03,1.13
		f2 = 1.12; //1.05,1.08
		f3 = 0.85;  //0.8,-
		f4 = 0.57;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 4)
	{
		hdom_ref = 53.24-0.154*TS_constant+0.000173*pow(TS_constant,2)-0.00000005554*pow(TS_constant,3);
		hdom_ts = 53.24-0.154*TS+0.000173*pow(TS,2)-0.00000005554*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (19/hdom_ref);
		f1 = 1.05;
		f2 = 1.02;
		f3 = 0.84;
		f4 = 0.5;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;

	}
	else if (SC == 5)
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.7-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (11/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);  //19
		f1 = 0.86;
		f2 = 0.88;
		f3 = 0.53;
		f4 = 0.43;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.7-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref); //15
		f1 = 0.77;
		f2 = 0.53;
		f3 = 0.4;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
    }
    else if (SP == 2)
    {
    	if (SC <= 2)
    	{
		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (25/hdom_ref); //23
		f1 = 1.0; //0.92,0.95
		f2 = 1.28; //1.18,1.23
		f3 = 0.85; // 0.75,0.77
		f4 = 0.5; // -,0.56
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (15/hdom_ref);
		hdom3 = hdom_ts * (21/hdom_ref);
		f1 = 1.03;
		f2 = 1.17;
		f3 = 0.81;
		f4 = 0.53;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (14/hdom_ref);
		hdom2 = hdom_ts * (19/hdom_ref);
		f1 = 0.92;
		f2 = 0.81;
		f3 = 0.46;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
    }
    else if (SP == 3)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 124-0.3788*TS_constant+0.0003934*pow(TS_constant,2)-0.000000122*pow(TS_constant,3);
		hdom_ts = 124-0.3788*TS+0.0003934*pow(TS,2)-0.000000122*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (17/hdom_ref);
		hdom3 = hdom_ts * (24/hdom_ref);
		f1 = 0.98; //-,0.87,0.94
		f2 = 1.05; //0.97,0.99
		f3 = 0.84; //0.75,0.83
		f4 = 0.68; //0.65,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8.5/hdom_ref);
		hdom2 = hdom_ts * (15/hdom_ref);
		hdom3 = hdom_ts * (22/hdom_ref);
		f1 = 0.99;  //-,0.9, 0.95
		f2 = 1.08; //1.0,1.03
		f3 = 0.82; //0.75,0.79
		f4 = 0.68; //0.65,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 0.8;
		f2 = 0.88;
		f3 = 0.68;
		f4 = 0.61;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else
    {
    	if (SC <= 2)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 1.0; //0.85,0.87,0.9,1.05
		f2 = 1.02; //0.88,0.94,1.0,1.05
		f3 = 0.94; //0.8,0.85,0.9,0.95
		f4 = 0.8; //0.65,0.67,0.75,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 0.89; //0.83,0.85,0.9
		f2 = 1.02; //0.93,0.99,1.05
		f3 = 0.89; //0.83,0.88,0.9
		f4 = 0.74;  //0.7,0.72,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 0.87;
		f2 = 0.98;
		f3 = 0.84;
		f4 = 0.66;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }

	//Calculate tree height growth on mineral soil
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Height_growth_pine_Hynynenym(d, D_dom, i_Hdom5, cr, CR_dom, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_h_factor);
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 4)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 5)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		//Brakes for the height growth
		if (h>h_max_Sp)
			*modelresult = *modelresult*0.01;
		else if (h>h_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (h>h_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (h>h_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Height_growth_spruce_Hynynenym(d, D_dom, i_Hdom5, cr, CR_dom, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_h_factor);
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		//Brakes for the height growth
		if (h>h_max_Ns)
			*modelresult = *modelresult*0.01;
		else if (h>h_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (h>h_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (h>h_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//White birch
	else if (SP == 3)
	{
		ret = Height_growth_birch_Hynynenym(d, D_dom, i_Hdom5, PLANT, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_h_factor);
		if (SC <=2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC ==3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		//Brakes for the height growth
			if (h>h_max_sb)
				*modelresult = *modelresult*0.01;
			else if (h>h_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (h>h_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (h>h_max_sb*0.85)
				*modelresult = *modelresult*0.6;

	}
	//Downy birch and other deciduous species
	else
	{
		ret = Height_growth_birch_Hynynenym(d, D_dom, i_Hdom5, PLANT, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_h_factor);
		if (H_dom < hdom1)
			*modelresult = *modelresult*f1;
		else if (H_dom >= hdom1 && H_dom < hdom2)
			*modelresult = *modelresult*f2;
		else if (H_dom >= hdom2 && H_dom < hdom3)
		        *modelresult = *modelresult*f3;
		else if (H_dom >= hdom3 && Age < growth_bending_age)
			*modelresult = *modelresult*f4;
		else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
			*modelresult = *modelresult*f5;
		else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
			*modelresult = *modelresult*f6;
		else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
			*modelresult = *modelresult*f7;
		else if (Age >= growth_bending_age + 15)
			*modelresult = *modelresult*f8;
		//Brakes for the height growth
		if (h>h_max_pb)
			*modelresult = *modelresult*0.01;
		else if (h>h_max_pb*0.95)
			*modelresult = *modelresult*0.1;
		else if (h>h_max_pb*0.9)
			*modelresult = *modelresult*0.3;
		else if (h>h_max_pb*0.85)
			*modelresult = *modelresult*0.6;

	}

	*modelresult = *modelresult*rectFactor;

	if (ret == 0)
		return ret;

	*nres = 1;
	//Restrict if result is impossible
	if (*modelresult<=0.01)
	{
         	*modelresult = 0.01;
    	}

	return ret;

}

int Grow_ba_mineral_soil_calib_JKK (double d, double cr, double h, double SP, double H_dom, double RDF, double RDF_L, double PLANT, double RDF_Sp,
						double RDF_Ns, double RDF_decid, double SI_50, double TS, double SC, double D_dom,
						double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING, double BA,
						double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double growth_bending_age, double Age, double i_ba_factor,
						double decrease_factor, int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double TS_constant = 1250;
    //Plant dummy is set to 0 for all the birch stands, so regeneration information will not be used
    PLANT=0;

    double hdom_ref, hdom_ts, hdom1, hdom2, hdom3, hdom4, hdom5;
    double f1,f2,f3,f4,f5,f6,f7,f8,f9,f10;

    //Calculate Hdom limits for growth
    if (SP==1||SP==8)
    {
    	if (SC <= 2)
	{
		hdom_ref = 49.86-0.1498*TS_constant+0.0001789*pow(TS_constant,2)-0.00000005865*pow(TS_constant,3);
		hdom_ts =  49.86-0.1498*TS+0.0001789*pow(TS,2)-0.00000005865*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (21/hdom_ref);
		f1 = 1.1; //0.89,0.95,-
		f2 = 1.2; //1.15,1.2,-
		f3 = 0.9; //0.82,0.85,0.87
		f4 = 0.58;//-,-,0.56
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 47.21-0.1445*TS_constant+0.0001749*pow(TS_constant,2)-0.00000005788*pow(TS_constant,3);
		hdom_ts = 47.21-0.1445*TS+0.0001749*pow(TS,2)-0.00000005788*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		hdom4 = hdom_ts * (21/hdom_ref);
		hdom5 = hdom_ts * (24/hdom_ref);
		f1 = 0.95; //0.75,0.85
		f2 = 1.3;  //1.1,1.2
		f3 = 1.2; //1.04,1.14
		f4 = 0.8;  //0.75,-
		f5 = 0.68; //-,-,0.63
		f6 = 0.52; 
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
		f9 = decrease_factor*f8;
		f10 = decrease_factor*f9;
	}
	else if (SC == 4)
	{
		hdom_ref = 45.81-0.136*TS_constant+0.0001615*pow(TS_constant,2)-0.0000000542*pow(TS_constant,3);
		hdom_ts = 45.81-0.136*TS+0.0001615*pow(TS,2)-0.0000000542*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (14.2/hdom_ref);
		hdom4 = hdom_ts * (17/hdom_ref);
		hdom5 = hdom_ts * (20/hdom_ref);
		f1 = 0.9;  //0.7,0.8
		f2 = 1.3;  //1.1,1.2
		f3 = 1.22; //1.06,1.16
		f4 = 1.06; //0.95,1.0
		f5 = 0.82; //-,-,0.79
		f6 = 0.63;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
		f9 = decrease_factor*f8;
		f10 = decrease_factor*f9;
	}
	else if (SC == 5)
	{
		hdom_ref = 6.775+0.001018*TS_constant+0.000007821*pow(TS_constant,2)-0.000000001807*pow(TS_constant,3);
		hdom_ts = 6.775+0.001018*TS+0.000007821*pow(TS,2)-0.000000001807*pow(TS,3);
		hdom1 = hdom_ts * (8.3/hdom_ref);
		hdom2 = hdom_ts * (13.2/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		hdom4 = hdom_ts * (18/hdom_ref); //19
		f1 = 0.65;
		f2 = 1.12;
		f3 = 0.73;
		f4 = 0.59;
		f5 = 0.42;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
		f9 = decrease_factor*f8;
	}
	else
	{
		hdom_ref = 24.99-0.05942*TS_constant+0.0000731*pow(TS_constant,2)-0.00000002478*pow(TS_constant,3);
		hdom_ts =  24.99-0.05942*TS+0.0000731*pow(TS,2)-0.00000002478*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (14/hdom_ref); //15
		f1 = 0.6;
		f2 = 0.8;
		f3 = 0.5;
		f4 = 0.3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 2)
    {
    	if (SC == 1)
    	{
		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (24/hdom_ref); //22
		f1 = 0.9; //-,-,0.8,-
		f2 = 0.95; //0.75,0.8,0.9,-
		f3 = 0.8; //0.63,0.68,0.73,0.75,0.77
		f4 = 0.57;  //-,-,-,0.5,0.55
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 2)
    	{
		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (24/hdom_ref); //22
		f1 = 0.8; //-,0.68,0.75,-
		f2 = 1.13; //0.92,0.97,1.08,1.18
		f3 = 0.69; //0.63,0.68,0.68,0.7
		f4 = 0.5;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 20.25-0.1039*TS_constant+0.0001701*pow(TS_constant,2)-0.00000006534*pow(TS_constant,3);
		hdom_ts = 20.25-0.1039*TS+0.0001701*pow(TS,2)-0.00000006534*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (14.5/hdom_ref);
		hdom3 = hdom_ts * (21/hdom_ref);
		f1 = 0.82; //0.73,0.78,-
		f2 = 1.05;  //1.0,-,1.1
		f3 = 0.79; //0.77,-,0.8
		f4 = 0.5;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 12.16-0.06517*TS_constant+0.0001089*pow(TS_constant,2)-0.00000004203*pow(TS_constant,3);
		hdom_ts = 12.16-0.06517*TS+0.0001089*pow(TS,2)-0.00000004203*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (17/hdom_ref);
		hdom3 = hdom_ts * (19/hdom_ref);
		f1 = 0.73;
		f2 = 0.7;
		f3 = 0.6;
		f4 = 0.4;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 3)
    {
    	if (SC <= 2)
    	{
		hdom_ref = 117.7-0.3611*TS_constant+0.0003786*pow(TS_constant,2)-0.0000001185*pow(TS_constant,3);
		hdom_ts = 117.7-0.3611*TS+0.0003786*pow(TS,2)-0.0000001185*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (16.3/hdom_ref);
		hdom3 = hdom_ts * (24/hdom_ref);
		f1 = 1.0;
		f2 = 0.98; //1.08,1.01
		f3 = 0.98;
		f4 = 0.78;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}

	else if (SC == 3)
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8.5/hdom_ref);
		hdom2 = hdom_ts * (15/hdom_ref);
		hdom3 = hdom_ts * (22/hdom_ref);
		f1 = 0.87;
		f2 = 1.02; //0.97
		f3 = 0.82;  //0.8
		f4 = 0.7;  //0.67
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 0.6;
		f2 = 0.62;
		f3 = 0.5;
		f4 = 0.41;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else
    {
    	if (SC <= 2)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref); //18,19
		f1 = 1.07;  //-,1.0,1.05,-,-
		f2 = 1.33; //1.03,1.08,1.13,1.2,1.26
		f3 = 1.17; //0.95,1.0,1.02,1.07,1.14
		f4 = 0.92; //-,-,0.9,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 0.94;  //-,-,0.92,-
		f2 = 1.09;  //1.0,-,1.05,1.07
		f3 = 1.05;  //0.9,0.95,1.0,1.01
		f4 = 0.75;   //-,0.7,-,-
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 0.75;
		f2 = 0.93;
		f3 = 0.67;
		f4 = 0.57;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }

	//Calculate tree basal area growth on mineral soil
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_pine_with_bias_corr_Hynynenym(d, cr, SI_50, RDF_L, RDF_Sp, RDF_Ns, RDF_decid, TS_constant, H_dom, SC, h,
	              ALT, SEA, SCRUB, WASTE, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && H_dom < hdom4)
				*modelresult = *modelresult*f4;
			else if (H_dom >= hdom4 && H_dom < hdom5)
				*modelresult = *modelresult*f5;
			else if (H_dom >= hdom5 && Age < growth_bending_age)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f8;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f9;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f10;
		}
		else if (SC == 4)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && H_dom < hdom4)
				*modelresult = *modelresult*f4;
			else if (H_dom >= hdom4 && H_dom < hdom5)
				*modelresult = *modelresult*f5;
			else if (H_dom >= hdom5 && Age < growth_bending_age)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f8;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f9;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f10;
		}
		else if (SC == 5)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && H_dom < hdom4)
				*modelresult = *modelresult*f4;
			else if (H_dom >= hdom4 && Age < growth_bending_age)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f8;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f9;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.45;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.45;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.45;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.45;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.45;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.45;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.45;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.45;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.35;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.35;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.35;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.35;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.35;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.35;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.35;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.35;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_spruce_with_bias_corr_Hynynenym(d, cr, SI_50, RDF_L, RDF_Sp, RDF_Ns, RDF_decid, TS_constant, H_dom, SC, h,
						ALT, SCRUB, WASTE, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC == 1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.45;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.45;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.45;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.45;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.45;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.45;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.45;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.45;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.35;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.35;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.35;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.35;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.35;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.35;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.35;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.35;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//White birch
	//Bias correction not used because it causes problems
	else if (SP == 3)
	{
		ret = Basal_area_growth_birch_Hynynenym(d, cr, SI_50, RDF_L, RDF, TS_constant, H_dom, PLANT, SP, h,
	                        SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.45;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.45;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.45;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.45;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.45;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.45;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.45;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.45;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.35;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.35;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.35;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.35;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.35;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.35;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.35;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.35;
		}
                //Brakes for the diameter growth
                	if (BA>BA_max_sb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_sb*0.85)
				*modelresult = *modelresult*0.6;
	}
	//Downy birch and other deciduous species
	//Bias correction not used because it causes problems
	else
	{
		ret = Basal_area_growth_birch_Hynynenym(d, cr, SI_50, RDF_L, RDF, TS_constant, H_dom, PLANT, SP, h,
	                        SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.45;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.45;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.45;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.45;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.45;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.45;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.45;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.45;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.35;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.35;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.35;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.35;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.35;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.35;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.35;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.35;
		}
                //Brakes for the diameter growth
    			if (BA>BA_max_pb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_pb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_pb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_pb*0.85)
				*modelresult = *modelresult*0.6;
	}

	if (ret == 0)
		return ret;

	*modelresult = *modelresult*rectFactor;

	*nres = 1;
	return ret;

}

int Growth_factors_calib_JKK (double SP, double TS,
				int *nres, struct Growth_factors *grow, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    	int ret = 1;
    	double i_ba, i_ba_peat, i_h;

	if (SP == 1 || SP == 8)
	{

		i_ba = 1.353-0.004538*TS+0.000005634*pow(TS,2)+0.000000002376*pow(TS,3)-0.000000000006213*pow(TS,4)+2.31E-015*pow(TS,5);
		i_ba_peat = 2.218-0.01468*TS+0.00004145*pow(TS,2)-0.00000005531*pow(TS,3)+0.00000000003558*pow(TS,4)-8.677E-015*pow(TS,5);
		i_h = 0.752-0.001509*TS+0.000002155*pow(TS,2)-0.0000000003778*pow(TS,3)-0.0000000000001801*pow(TS,4);
	}
	else if (SP == 2)
	{
		i_ba = 1.05-0.00234*TS+0.00000447*pow(TS,2)-0.0000000031*pow(TS,3)+0.000000000000801*pow(TS,4);
		i_ba_peat = 0.1491+0.001741*TS-0.000003837*pow(TS,2)+0.000000003918*pow(TS,3)-0.000000000001222*pow(TS,4);
		i_h = 0.5282+0.0001635*TS-0.0000006812*pow(TS,2)+0.000000001206*pow(TS,3)-0.0000000000004214*pow(TS,4);
	}
	else if (SP == 3)
	{
		i_ba = -5.626+0.04113*TS-0.000108*pow(TS,2)+0.0000001349*pow(TS,3)-0.00000000007966*pow(TS,4)+0.00000000000001844*pow(TS,5)-3.2E-019*pow(TS,6);
		i_ba_peat = 0.787-0.0007511*TS+0.000001252*pow(TS,2)-0.0000000004116*pow(TS,3);
		i_h = 0.8737-0.0125*TS+0.00006471*pow(TS,2)-0.0000001484*pow(TS,3)+0.0000000001698*pow(TS,4)-0.00000000000009358*pow(TS,5)+1.98E-017*pow(TS,6);
	}
	else
	{
		i_ba = 0.6336-0.0001593*TS+0.000000359*pow(TS,2)-0.000000000003321*pow(TS,3);
		i_ba_peat = 0.3522+0.0009567*TS-0.000002337*pow(TS,2)+0.000000002721*pow(TS,3)-0.0000000000009052*pow(TS,4);
		i_h = 0.5588-0.0003074*TS+0.0000004766*pow(TS,2)+0.00000000003253*pow(TS,3);
		
	}

	*nres = 1;

	grow[0].i_ba_factor = i_ba;
	grow[0].i_ba_peat_factor = i_ba_peat;
	grow[0].i_h_factor = i_h;

	return ret;

}

int Grow_ba_peatland_calib_fixed_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT,
						double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA,
						double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING,
						double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb,
						double BA_max_pb, double MANAGEMENT_REGION, double growth_bending_age, double Age, double i_ba_peat_factor,
						double decrease_factor, int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double TS_constant = 1250;
    double hdom_ref, hdom_ts, hdom1, hdom2, hdom3;
    double f1,f2,f3,f4,f5,f6,f7,f8;

    //Calculate Hdom limits for growth
    if (SP==1||SP==8)
    {
    	if (SC <= 2)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 1.86; //1.85
		f2 = 1.3; //1.28
		f3 = 1.17; //1.13
		f4 = 0.68; //0.67
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 59.04-0.1774*TS_constant+0.0002035*pow(TS_constant,2)-0.00000006572*pow(TS_constant,3);
		hdom_ts = 59.04-0.1774*TS+0.0002035*pow(TS,2)-0.00000006572*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 2.02; //1.92,1.95,2.0
		f2 = 1.25; //1.14,1.17,1.2
		f3 = 1.09; //1.0,1.02,1.07
		f4 = 0.51; //-,0.45,0.5
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 4)
	{
		hdom_ref = 53.24-0.154*TS_constant+0.000173*pow(TS_constant,2)-0.00000005554*pow(TS_constant,3);
		hdom_ts = 53.24-0.154*TS+0.000173*pow(TS,2)-0.00000005554*pow(TS,3);
		hdom1 = hdom_ts * (11/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 2.09;
		f2 = 2.15;
		f3 = 1.14;
		f4 = 0.7;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 5)
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.66-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 1.94;
		f2 = 1.25;
		f3 = 0.69;
		f4 = 0.53;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 29.66-0.0721*TS_constant+0.00008348*pow(TS_constant,2)-0.0000000273*pow(TS_constant,3);
		hdom_ts = 29.66-0.0721*TS+0.00008348*pow(TS,2)-0.0000000273*pow(TS,3);
		hdom1 = hdom_ts * (10/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 1.6;
		f2 = 1.0;
		f3 = 0.5;
		f4 = 0.4;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 2)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 23.23-0.1213*TS_constant+0.0002004*pow(TS_constant,2)-0.00000007726*pow(TS_constant,3);
		hdom_ts = 23.23-0.1213*TS+0.0002004*pow(TS,2)-0.00000007726*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (17/hdom_ref);
		hdom3 = hdom_ts * (21/hdom_ref);
		f1 = 1.0;  //0.85,0.92
		f2 = 0.75; //0.6,0.65
		f3 = 0.36;
		f4 = 0.28;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (14/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		hdom3 = hdom_ts * (20/hdom_ref);
		f1 = 0.9;  //0.76,0.83
		f2 = 0.68; //0.59,0.64
		f3 = 0.4; //-,0.36
		f4 = 0.24;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
	{
		hdom_ref = 25.16-0.1188*TS_constant+0.0001844*pow(TS_constant,2)-0.00000006961*pow(TS_constant,3);
		hdom_ts = 25.16-0.1188*TS+0.0001844*pow(TS,2)-0.00000006961*pow(TS,3);
		hdom1 = hdom_ts * (12/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (19/hdom_ref);
		f1 = 0.74;
		f2 = 0.59;
		f3 = 0.41;
		f4 = 0.28;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }
    else if (SP == 3)
    {
    	if (SC <= 2)
    	{
    		hdom_ref = 124-0.3788*TS_constant+0.0003934*pow(TS_constant,2)-0.000000122*pow(TS_constant,3);
		hdom_ts = 124-0.3788*TS+0.0003934*pow(TS,2)-0.000000122*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (16/hdom_ref);
		f1 = 1.9;
		f2 = 0.9;
		f3 = 0.58;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
	else if (SC == 3)
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8.5/hdom_ref);
		hdom2 = hdom_ts * (15/hdom_ref);
		f1 = 1.8;
		f2 = 0.9;
		f3 = 0.4;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
	else
	{
		hdom_ref = 117.9-0.3609*TS_constant+0.0003754*pow(TS_constant,2)-0.0000001165*pow(TS_constant,3);
		hdom_ts = 117.9-0.3609*TS+0.0003754*pow(TS,2)-0.0000001165*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		f1 = 1.2;
		f2 = 0.7;
		f3 = 0.4;
		f4 = decrease_factor*f3;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
	}
    }
    else
    {
    	if (SC <= 2)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (14/hdom_ref);
		hdom3 = hdom_ts * (18/hdom_ref);
		f1 = 1.76;
		f2 = 1.02;
		f3 = 0.75;
		f4 = 0.42;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else if (SC == 3)
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (9/hdom_ref);
		hdom2 = hdom_ts * (13/hdom_ref);
		hdom3 = hdom_ts * (16/hdom_ref);
		f1 = 1.68;
		f2 = 0.83;
		f3 = 0.68;
		f4 = 0.4;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
	else
    	{
		hdom_ref = 64.89-0.1847*TS_constant+0.0001892*pow(TS_constant,2)-0.00000005591*pow(TS_constant,3);
		hdom_ts = 64.89-0.1847*TS+0.0001892*pow(TS,2)-0.00000005591*pow(TS,3);
		hdom1 = hdom_ts * (8/hdom_ref);
		hdom2 = hdom_ts * (12/hdom_ref);
		hdom3 = hdom_ts * (15/hdom_ref);
		f1 = 1.85;
		f2 = 1.05;
		f3 = 0.78;
		f4 = 0.38;
		f5 = decrease_factor*f4;
		f6 = decrease_factor*f5;
		f7 = decrease_factor*f6;
		f8 = decrease_factor*f7;
	}
    }


	//Calculate tree basal area growth on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_peatland_pine_with_bias_corr_fixed_Hynynenym(d, BA, BA_L, SC, TS_constant, PDR, FUSC, SUBGROUP, ALT, SEA, SCRUB, UNDRAINED, WASTE,
		      SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 4)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 5)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_peatland_spruce_with_bias_corr_fixed_Hynynenym(d, BA_prosNs, BA_L, SC, TS_constant, PDR, H_dom, ALT, LAKE, UNDRAINED, WASTE,
		      SUBGROUP, SCRUB, SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.25;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//White birch
	else if (SP == 3)
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_fixed_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS_constant, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
		      SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && Age < growth_bending_age)
			        *modelresult = *modelresult*f3*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f7;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_sb)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_sb*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_sb*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_sb*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Downy birch and other deciduous species
	else
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_fixed_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS_constant, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
							SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, i_ba_peat_factor);
		if (SCRUB == 0 && WASTE==0)
		{
		if (SC <= 2)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else if (SC == 3)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		else
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8;
		}
		}
		else if (SCRUB==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.4;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.4;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.4;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.4;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.4;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.4;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.4;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.4;
		}
		else if (WASTE==1)
		{
			if (H_dom < hdom1)
				*modelresult = *modelresult*f1*0.25;
			else if (H_dom >= hdom1 && H_dom < hdom2)
				*modelresult = *modelresult*f2*0.25;
			else if (H_dom >= hdom2 && H_dom < hdom3)
			        *modelresult = *modelresult*f3*0.25;
			else if (H_dom >= hdom3 && Age < growth_bending_age)
				*modelresult = *modelresult*f4*0.25;
			else if (Age >= growth_bending_age && Age < growth_bending_age + 5)
				*modelresult = *modelresult*f5*0.25;
			else if (Age >= growth_bending_age + 5 && Age < growth_bending_age + 10)
				*modelresult = *modelresult*f6*0.25;
			else if (Age >= growth_bending_age + 10 && Age < growth_bending_age + 15)
				*modelresult = *modelresult*f7*0.25;
			else if (Age >= growth_bending_age + 15)
				*modelresult = *modelresult*f8*0.25;
		}
		//Brakes for the diameter growth
		if (BA>BA_max_pb)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_pb*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_pb*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_pb*0.85)
			*modelresult = *modelresult*0.6;
	}

	*modelresult = *modelresult*rectFactor;

	if (ret == 0)
		return ret;

	*nres = 1;
	return ret;

}
