// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  CalibratedtreegrowthmodelsLibrary.cpp
//
//	Calibrated tree growth model library - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029

struct Thinning_limits_and_passed_limit {
	double Thinning_limit;
	double Remainlimit_upper;
	double Remainlimit_lower;
	double JUST_THINNED;
};

struct Growth_factors {
	double i_ba_factor;
	double i_ba_peat_factor;
	double i_h_factor;
};

//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult, warnings and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int H_dom_growth_calib_JKK (double SP, double A, double H_gM, double TS, double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY, double RDF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_peatland_calib_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION, double growth_bending_age, double Age, double i_ba_peat_factor, double decrease_factor, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_height_before_growing_d_peatland_calib_JKK (double d, double D_gM, double BA, double SP, double LAT, double SC, double SINCE_THINNING, double ALT, double BA_prosb, double TS, double Hdom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_height_after_growing_d_peatland_calib_JKK (double d, double h, double h_before, double D_gM, double BA, double SP, double LAT, double SC, double SINCE_THINNING, double ALT, double BA_prosb, double TS, double Hdom, int *nres, struct Tree_h_peatland *h_peatland, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_h_mineral_soil_calib_JKK (double d, double cr, double SP, double i_Hdom5, double CR_dom, double RDF_L, double PLANT, double D_dom, double h, double h_max_Sp, double h_max_Ns, double h_max_sb, double h_max_pb, double SC, double H_dom, double growth_bending_age, double Age, double TS, double i_h_factor, double decrease_factor, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_mineral_soil_calib_JKK (double d, double cr, double h, double SP, double H_dom, double RDF, double RDF_L, double PLANT, double RDF_Sp, double RDF_Ns, double RDF_decid, double SI_50, double TS, double SC, double D_dom, double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING, double BA, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double growth_bending_age, double Age, double i_ba_factor, double decrease_factor, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Growth_factors_calib_JKK (double SP, double TS, int *nres, struct Growth_factors *grow, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_peatland_calib_fixed_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION, double growth_bending_age, double Age, double i_ba_peat_factor, double decrease_factor, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif
