// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  ClimatechangemodelLibrary.cpp
//
// 	Tree level growth Models - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "ClimatechangemodelLibrary.h"
#include "SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


int Diameter_and_height_growth_climate_change_Matalaym (double d, double h, double i_v, double i_h, double RDF_L, double SP, double RDF, double TS, double SC, double T_elevation, double CO2_elevation,
									int *nres, struct Tree_d_h_climate_change *d_h_cc, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if ((2+1.25*d)<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "(2+1.25*d)<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	int SC2, SC4;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC2 = 1;
				SC4 = 0;
				break;
		case 2:
				SC2 = 1;
				SC4 = 0;
				break;
		case 3:
				SC2 = 0;
				SC4 = 0;
				break;
		case 4:
				SC2 = 0;
				SC4 = 1;
				break;
		case 5:
				SC2 = 0;
				SC4 = 1;
				break;
		case 6:
				SC2 = 0;
				SC4 = 1;
				break;
		default:
				SC2 = 0;
				SC4 = 1;
				break;
	};
	

	*nres = 1;
	double rse_v, se, rse_h, v;
	if (SP==1 || SP==8)
	{
		rse_v = (0.0446+0.0036*pow(TS/1000,2)+0.0001*SC2+(-0.0001)*SC4)*T_elevation+
			(0.0015+0.0001*TS/1000+0.000011*SC2+(-0.000013)*SC4)*CO2_elevation+
			(0.0154+(-0.0314)*TS/1000+0.0134*pow(TS/1000,2))*pow(T_elevation,2)+
			(-0.000003)*pow(CO2_elevation,2)+
			(0.0001+(-0.000014)*TS/1000)*T_elevation*CO2_elevation+
			(-0.0001)*pow(T_elevation,3)+
			0.0000000026*pow(CO2_elevation,3)+
			(-0.0000019)*pow(T_elevation,2)*CO2_elevation+
			(-0.00000011)*T_elevation*pow(CO2_elevation,2)+
			((-0.013)*T_elevation+0.0002*CO2_elevation+0.002*pow(T_elevation,2)+
			(-0.00000022)*pow(CO2_elevation,2)+0.0000088*T_elevation*CO2_elevation+
			(-0.0002)*pow(T_elevation,3))*RDF_L+
			((-0.0037)*T_elevation+0.0001*CO2_elevation+0.0000015*pow(T_elevation,2)*CO2_elevation)*RDF;
		se = 0.569-0.00022*TS+0.167*RDF_L-0.101*rse_v;
		rse_h = rse_v*se;
		if (h<3)
			{
				//Tomppo's volume model for little tree
				v = (exp(-4.3115-0.027792+0.81849*log(h)+2.34597*log(2+1.25*d)))/1000;
			}
			else
			{
				//Laasasenaho's volume model
				v = (exp(-3.2890 +1.9995*log(d) +2.1395*log(h)-1.1411*log(h-1.3) -0.05694))/1000;
			}
	}
	else if (SP==2)
	{
		rse_v = (0.0168+0.0321*pow(TS/1000,2))*T_elevation+
			(0.0013+(-0.00004)*TS/1000+0.000013*SC2)*CO2_elevation+
			(0.01+(-0.0149)*TS/1000+0.0025*pow(TS/1000,2))*pow(T_elevation,2)+
			(-0.0000022)*pow(CO2_elevation,2)+
			(0.0001)*T_elevation*CO2_elevation+
			0.0000000022*pow(CO2_elevation,3)+
			(-0.00000014)*T_elevation*pow(CO2_elevation,2)+
			((-0.0647)*T_elevation+0.0025*CO2_elevation+0.0078*pow(T_elevation,2)+
			(-0.000004)*pow(CO2_elevation,2)+0.0001*T_elevation*CO2_elevation+
			(-0.0012)*pow(T_elevation,3)+0.0000000031*pow(CO2_elevation,3))*RDF_L+
			((-0.0102)*T_elevation+0.0005*CO2_elevation+(-0.00000048)*pow(CO2_elevation,2)+0.000025*T_elevation*CO2_elevation)*RDF;
		se = 1.282-0.00079*TS-0.092*RDF_L-0.04*rse_v;
		rse_h = rse_v*se;
		if (h<3)
			{
				//Tomppo's volume model for little tree
				v = (exp(-3.9171+1.04666*log(h)+1.971*log(2+1.25*d)))/1000;
			}
			else
			{
				//Laasasenaho's volume model
				v = (exp(-3.7544 +1.8960*log(d) +2.8979*log(h)-1.6020*log(h-1.3) -0.15654))/1000;
			}
	}
	else //deciduous species
	{
		rse_v = (0.1258+(-0.0655)*pow(TS/1000,2))*T_elevation+
			(0.0013+0.0002*TS/1000)*CO2_elevation+
			(0.0434+(-0.1164)*TS/1000+0.0629*pow(TS/1000,2))*pow(T_elevation,2)+
			(-0.0000026)*pow(CO2_elevation,2)+
			(0.0001+(-0.000029)*TS/1000)*T_elevation*CO2_elevation+
			0.0004*pow(T_elevation,3)+
			0.0000000023*pow(CO2_elevation,3)+
			(-0.0000012)*pow(T_elevation,2)*CO2_elevation+
			(-0.000000096)*T_elevation*pow(CO2_elevation,2)+
			((-0.0068)*T_elevation+0.0001*CO2_elevation+(-0.000000097)*pow(CO2_elevation,2)+
			0.0000049*T_elevation*CO2_elevation+0.000044*pow(T_elevation,3))*RDF_L+
			(0.0001*CO2_elevation+(-0.0024)*pow(T_elevation,2)+(-0.000000084)*pow(CO2_elevation,2)+
			0.0003*pow(T_elevation,3)+0.0000011*pow(T_elevation,2)*CO2_elevation)*RDF;
		se = 0.922-0.00029*TS-0.067*RDF_L-0.016*rse_v;
		rse_h = rse_v*se;
		if (h<4)
			{
				//Tomppo's volume model for little tree
				v = (exp(-4.92615-0.031845+0.787837*log(h)+2.56275*log(2+1.25*d)))/1000;
			}
			else
			{
				//Laasasenaho's volume model
				v = (exp(-4.4759 +2.0851*log(d) +4.0691*log(h)-2.7375*log(h-1.3) -0.26622))/1000;
			}			
	}
	
	double iv_cc, ih_cc, id_cc, v_cc, h_cc, d_cc;
	iv_cc = (i_v*(1+rse_v)) - i_v;
	ih_cc = (i_h*(1+rse_h)) - i_h;
	h_cc = h + ih_cc;
	v_cc = v + iv_cc;
	//calculate approximation of d_cc and id_cc
	if (SP==1 || SP==8)
	{
		if (h<3)
			{
				//Transformation of Tomppo's volume model for little tree
				d_cc = ((exp((log(1000*v_cc)+4.3115+0.027792-0.81849*log(h_cc))/2.34597))-2)/1.25;
			}
			else
			{
				//Transformation of Laasasenaho's volume model
				d_cc = exp((log(1000*v_cc)-2.1395*log(h_cc)+1.1411*log(h_cc-1.3)+3.289+0.05694)/1.9995);
			}
		id_cc = d_cc - d;
	}
	else if (SP==2)
	{
		if (h<3)
			{
				//Transformation of Tomppo's volume model for little tree
				d_cc = ((exp((log(1000*v_cc)+3.9171-1.04666*log(h_cc))/1.971))-2)/1.25;
			}
			else
			{
				//Transformation of Laasasenaho's volume model
				d_cc = exp((log(1000*v_cc)-2.8979*log(h_cc)+1.602*log(h_cc-1.3)+3.7544+0.15654)/1.896);
			}
		id_cc = d_cc - d;
	}
	else //deciduous species
	{
		if (h<4)
			{
				//Transformation of Tomppo's volume model for little tree
				d_cc = ((exp((log(1000*v_cc)+4.92615+0.031845-0.787873*log(h_cc))/2.56275))-2)/1.25;
			}
			else
			{
				//Transformation of Laasasenaho's volume model
				d_cc = exp((log(1000*v_cc)-4.0691*log(h_cc)+2.7375*log(h_cc-1.3)+4.4759+0.26622)/2.0851);
			}
		id_cc = d_cc - d;
	}

	d_h_cc[0].ih_cc=ih_cc;
	d_h_cc[0].id_cc=id_cc;
	
	return ret;

}

int T_elevation_JKK (double base_year_cc, double T_elev_in_100_years, double year,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	*nres = 1;
	double years_since_base = year - base_year_cc;

	*modelresult = years_since_base * T_elev_in_100_years/100;
	return ret;

}

int CO2_elevation_JKK (double base_year_cc, double CO2_elev_in_100_years, double year,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	*nres = 1;
	double years_since_base = year - base_year_cc;

	*modelresult = years_since_base * CO2_elev_in_100_years/100;
	return ret;

}
