//////////////////////////////////////////////////////////////////////////////
//
//  DistributionmodelLibrary.h
//
//    DistributionModels - Header file
//
//    Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029

struct diamclass {
    double dbh;
    double n;
};

struct heightclass {
    double h;
    double d;
    double n;
    double seedling_below13;
    double seedling_to_mature13;
};

struct diamclass_seedtree {
    double d;
    double h;
    double n;
};

struct heightclass_weibull {
    double h;
    double n;
};

struct heightclass_weibull_with_diameters {
    double d;
    double h;
    double n;
};

struct deadtreeclass {
    double sp;
    double d;
    double since_death;
    double snag;
    double decay_class;
    double n;
    double v;
    double density;
    double bm_aboveground_total;
    double v_ad;
    double density_ad;
    double bm_total_ad;
};

//    Conventional function declarations used when the library is linked
//    using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult, warnings and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Diameter_distribution_pine_percent_point_KangasMaltamo ( double BA, double DgM, double Age, double N, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_percent_point_KangasMaltamo ( double BA, double BA_total, double DgM, double Age, double N, double DIAM_CLASS_WIDTH, double SC, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_percent_point_KangasMaltamo ( double BA, double DgM, double Age, double N, double DIAM_CLASS_WIDTH, double SC, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_Weibull_Kilkki ( double BA, double D_gM, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_Weibull_only_N_measured_Kilkki ( double BA, double D_gM, double N, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_Weibull_Mykkanen ( double BA, double D_gM, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_Weibull_only_N_measured_Mykkanen ( double BA, double D_gM, double N, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_Weibull_Siipilehto (double BA, double D_gM, double A, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_Weibull_only_N_measured_Siipilehto (double BA, double D_gM, double A, double N, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_grey_alder_Weibull_Karkiym (double BA, double D_gM, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_grey_alder_Weibull_only_N_measured_Karkiym (double BA, double D_gM, double N, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_Weibull_integer_class_Kilkki ( double BA, double D_gM, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_Weibull_integer_class_Mykkanen ( double BA, double D_gM, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_Weibull_integer_class_Siipilehto (double BA, double D_gM, double A, double DIAM_CLASS_WIDTH,    double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_grey_alder_Weibull_integer_class_Karkiym (double BA, double D_gM, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_with_stem_number_JohnsonSB_Siipilehto ( double BA, double D_gM, double N, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_with_stem_number_JohnsonSB_Siipilehto ( double BA, double D_gM, double N, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_with_stem_number_JohnsonSB_Siipilehto ( double BA, double D_gM, double N, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_JohnsonSB_Siipilehto ( double D_gM, double A, double H_gM, double BA, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_JohnsonSB_Siipilehto ( double D_gM, double A, double H_gM, double BA, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_birch_JohnsonSB_Siipilehto ( double D_gM, double A, double BA, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Normal_height_distribution (double H_gM, double N, double HEIGHT_CLASS_NUM, double CULT, double max_class_count, int *nres, struct heightclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Normal_height_distribution_with_diameters (double H_gM, double CULT, double N, double HEIGHT_CLASS_NUM, double SP, double D_gM, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct heightclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_seedtree_JKK (double N, double D_gM, double H_gM, double MAX_CLASS_COUNT, int *nres, struct diamclass_seedtree *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Fix_D_gM_for_JohnsonSB_pine ( double D_gM, double A, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_distribution_young_pine_Weibull_Siipilehto (double H_gM, double H_dom, double N, double HEIGHT_CLASS_WIDTH, double max_class_count, int *nres, struct heightclass_weibull *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_distribution_young_pine_with_diameters_Weibull_Siipilehto (double H_gM, double H_dom, double N, double D_gM, double HEIGHT_CLASS_WIDTH, double CLASS_N_MIN,    double max_class_count, int *nres, struct heightclass_weibull_with_diameters *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_distribution_young_plantation_spruce_Valkonen (double N, double H_ari, double HEIGHT_CLASS_WIDTH, double MAX_CLASS_COUNT, int *nres, struct heightclass_weibull *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_distribution_understorey_spruce_Valkonen (double N, double H_ari, double HEIGHT_CLASS_WIDTH, double MAX_CLASS_COUNT, int *nres, struct heightclass_weibull *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_JKK (double BA, double BA_total, double D_gM, double H_gM, double Age, double A, double N, double SP, double N_measured, double PEAT, double SC, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, double D_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_only_N_measured_JKK (double BA, double D_gM, double Age, double A, double N, double SP, double N_measured, double SC, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_weibull_JKK (double BA, double D_gM, double Age, double A, double SP, double SC, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors,    int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_weibull_integer_class_JKK (double BA, double D_gM, double Age, double A, double SP, double SC, double DIAM_CLASS_WIDTH, double MAX_CLASS_COUNT, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors,    int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_spruce_Weibull_MELA_Kilkki ( double BA, double D_gM, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_pine_Weibull_MELA_Mykkanen ( double BA, double D_gM, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_MELA_JKK (double BA, double D_gM, double SP, double CLASS_N_MIN, int *nres, struct diamclass *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_distribution_young_pine_Siipilehto (double N, double H_gM, double H_dom, double HEIGHT_CLASS_WIDTH, double max_class_count, int *nres, struct heightclass_weibull *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_distribution_little_ba_and_n_JKK (double N, double D_gM, double H_gM, double BA, double BA_measured, double MAX_CLASS_COUNT, int *nres, struct diamclass_seedtree *dist, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif
