// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  SeedlingmodelLibrary.h
//
//	Seedling models library - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029


struct new_species {
	double SP;
	double N;
	double natural_ingrowth;
};

struct new_origspecies {
	double SP;
	double ORIG_SP;
	double N;
	double natural_ingrowth;
};


//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult, warnings and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Initial_age_for_a_new_tree_pine_Hynynenym (double TS, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Initial_age_for_a_new_tree_spruce_Hynynenym (double TS, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Initial_age_for_a_new_tree_deciduous_Hynynenym (double TS, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_for_a_seedling_Hynynenym (double th, double LSS, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Time_to_reach_breast_height_natural_regeneration_Hynynenym (double th_ref, double TS, double BA, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Time_to_reach_breast_height_artificial_regeneration_Hynynenym (double th_ref, double TS, double BA, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_for_a_seedling_reached_breast_height_Hynynenym (double d_href, double BA, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_for_a_seedling_reached_breast_height_Hynynenym (double d_href, double BA, double H_gM, double time_step, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_seedling_pine_Tomppoym (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_seedling_spruce_Tomppoym (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_seedling_birch_Tomppoym (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_small_tree_pine_Tomppoym (double d, double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_small_tree_spruce_Tomppoym (double d, double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_small_tree_birch_Tomppoym (double d, double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_pine_seed_tree_Niemisto (double d, double age, double T_s, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_young_stand_pine_Hagglund (double i_Hdom2_5a, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_young_stand_pine_Varmola (double H_dom, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Time_to_reach_breast_height_young_stand_pine_Varmola (double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Time_to_reach_breast_height_with_TS_young_stand_pine_Varmola (double SI_100, double TS, double SEED, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_young_stand_pine_Varmola (double BA_u, double SI_100, double N, double SEED, double H_domc, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_after_cleaning_young_stand_pine_Varmola (double H_dom, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_after_breast_height_young_stand_pine_Varmola (double H_dom, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Bark_from_BA_young_stand_pine_Varmola (double BA, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Bark_from_BAu_young_stand_pine_Varmola (double BA_u, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_young_stand_pine_Varmola (double BA, double H_dom, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_young_stand_pine_Varmola (double H_dom, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_young_stand_pine_Varmola (double H_gM, double N, double BA, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_birch_in_gabs_of_planted_spruce_stand_Valkonen (double h, double SI_25, double pub, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_young_plantation_spruce_Valkonen (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_young_plantation_spruce_Valkonen (double H_dom, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_young_plantation_spruce_Valkonen (double h, double H_dom, double i_Hdom5, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_young_plantation_spruce_Valkonen (double d, double BA, double SI_25, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Probability_a_tree_to_die_young_plantation_spruce_Valkonen (double h, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_released_understorey_spruce_Valkonen (double h, double BA, double T_r, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_in_two_storey_spruce_birch_stand_spruce_Valkonen (double h, double BA_b, double N_b, double H_g, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_in_two_storey_spruce_birch_stand_spruce_Valkonen (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_in_two_storey_spruce_birch_stand_white_birch_Valkonen (double h, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_in_two_storey_spruce_birch_stand_pubescent_birch_Valkonen (double h, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_in_two_storey_spruce_birch_stand_white_birch_Valkonen (double d, double SI_50, double BA_b, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_in_two_storey_spruce_birch_stand_pubescent_birch_Valkonen (double d, double SI_50, double BA_b, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Bark_in_two_storey_spruce_birch_stand_birch_Valkonen (double d, double cr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_in_pine_birch_mixed_stand_pine_Valkonen (double h, double H_dom, double CI_Sp, double CI_b, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_in_pine_birch_mixed_stand_birch_Valkonen (double h, double CI, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_in_pine_birch_mixed_stand_pine_Valkonen (double h, double H_dom, double i_Hdom, double CI_b, double SCC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_in_pine_birch_mixed_stand_birch_Valkonen (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_in_pine_birch_mixed_stand_pine_Valkonen (double d, double h, double H_dom, double SI_100, double BA, double CI_Sp, double CI_b, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_in_pine_birch_mixed_stand_birch_Valkonen (double d, double BA, double CI, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Max_branch_diameter_in_pine_birch_mixed_stand_pine_Valkonen (double h, double H_dom, double SI_100, double BA, double CI_Sp, double CI_b, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Median_height_seedling_in_pine_seed_tree_stand_pine_Valkonen (double IPOT_tr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Maximum_height_seedling_in_pine_seed_tree_stand_pine_Valkonen (double H_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_seedling_in_pine_seed_tree_stand_pine_Valkonen (double h, double IPOT_tr, double IPOT_se, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_seedling_in_pine_seed_tree_stand_pine_Valkonen (double h, double SI_100, double IPOT_tr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_seedling_in_pine_seed_tree_stand_pine_Valkonen (double d, double SI_100, double IPOT_tr, double IPOT_se, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Max_branch_diameter_seedling_in_pine_seed_tree_stand_pine_Valkonen (double h, double H_dom, double SI_100, double IPOT_tr, double IPOT_se, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Annual_height_growth_seedling_JKK (double th, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_small_tree_pine_Ojansuu (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_small_tree_spruce_Ojansuu (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_small_tree_birch_Ojansuu (double h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_percentage_for_seedling_variables_JKK (double Age, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Seedling_limits_JKK (double SC, double SP, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_tree_species_Hynynenym (double SC, double PEAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_species_natural_Hynynenym (double SC, double PEAT, double i_N5, double SP, double CULT, double random, double MAXCLASSCOUNT, int *nres, struct new_species *new_sp, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_species_artificial_Hynynenym (double SC, double PEAT, double i_N5, double MAIN_SP, double CULT, double random, int *nres, struct new_species *new_sp, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_species_JKK (double SC, double PEAT, double i_N5, double MAIN_SP, int *nres, struct new_species *new_sp, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_species_fixed_JKK (double i_N5, double REGEN_SP, int *nres, struct new_species *new_sp, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_new_trees_Hynynenym (double BA, double N, double N_ref, double SCRUB, double WASTE, double SOILPREP, double Two_st, double Age, double EXTERIOR_PORTION_OF_STAND_EFFECT_REGENERATION, double AREA, double SP, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_new_trees_JKK (double N, double N_ref, double SCRUB, double WASTE, double SOILPREP, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Average_number_of_trees_natural_regeneration_Hynynenym (double PEAT, double SC, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Average_number_of_trees_artificial_regeneration_Hynynenym (double PEAT, double SC, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Average_reference_time_to_reach_breast_height_Hynynenym (double SC, double SP, double SCRUB, double WASTE, double PEAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Average_reference_diameter_at_breast_height_Hynynenym (double SC, double SP, double SCRUB, double WASTE, double PEAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_young_stand_pine_Huuskonen (double BA, double N, double PLANT, double SEED, double SC, double TS, double PALU, double ALT, double LAKE, double SEA, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_diameter_young_stand_pine_Huuskonen (double BA, double N, double PLANT, double TS, double LAKE, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_young_stand_pine_Huuskonen (double H_dom, double N, double Age, double PLANT, double SEED, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_young_stand_pine_Huuskonen (double H_dom, double BA, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_young_stand_pine_Huuskonen (double PLANT, double D_dom, double H_dom, double BA, double N, double H_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_young_stand_pine_Huuskonen (double PLANT, double D_dom, double BA, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_small_tree_pine_Siipilehto (double h, double D_gM, double H_gM, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Initial_age_JKK (double SP, double TS, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_young_pine_Siipilehto (double h, double D_gM, double H_gM, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_seedling_spruce_uneven_aged_stand_Eerikainen (double H_gM, double BA, double SC, double SINCE_THINNING, double removal_proportion, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_new_trees_uneven_aged_stand_JKK (double N_ref, double SCRUB, double WASTE, double SOILPREP, double random, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_species_natural_after_uneven_aged_harvest_JKK (double Nref_Sp, double Nref_Ns, double Nref_b, int *nres, struct new_origspecies *new_origsp, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif