// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  DynamicstandmodelLibrary.cpp
//
//	Dynamic stand models library - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "StandgrowthmodelLibrary.h"
#include "SimoUtils.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>



int Basal_area_growth_under_bark_pine_VuokilaValiaho (double BA_u, double Age, double H_dom, double SI_100, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {
	int ret = 1;
	double a0 = pow(H_dom,0.3)*pow(BA_u,0.4)*pow(Age,(0.8+SI_100/175));

	//Check for fatal errors
	char errorStr[200] = "";
	if (a0<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a0<=0(a0=",a0);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	double res = -0.89537 + 0.0019059 * pow(SI_100, 2) + 730.26 / a0 + 363780 / pow(a0, 3);
	*modelresult = res * rectFactor * SC_factor;
	return ret;
}


int Basal_area_growth_under_bark_spruce_VuokilaValiaho (double BA_u, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Restrict the values of explaining variables to avoid problems arise from combination of the input variables
    //Otherwise the model can produce negative results
    double BA_u_orig = BA_u;

	if (BA_u<0.5)
	   BA_u = 0.5;
    if (H_dom<4)
       H_dom = 4;

	double a0 = pow(H_dom,0.5)*pow(BA_u,0.6)*pow(Age,0.9);

	//Check for fatal errors
	char errorStr[200] = "";
	if (a0<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a0<=0(a0=",a0);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = -0.099776+3256.7/a0-31736/pow(a0,2);

	//Correct the result if original BA_u was very low
	if (BA_u_orig<0.01)
	    res = 1000;
    else if (BA_u_orig<0.1)
        res = 400;
    else if (BA_u_orig<0.4)
        res = 200;

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Basal_area_growth_under_bark_pubescent_birch_Saramaki (double V_u, double Age_s, double H_dom, double LGP,double SC,
                     double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

    // siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	double res = -23.93375+0.00181*(215-V_u)*(100-Age_s)+4.37571*pow((26-H_dom)/10,8)*pow((100-Age_s)/100,10)+0.20845*LGP;
	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Basal_area_growth_pubescent_birch_Mielikainen (double BA, double A, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
    double BA_orig = BA;

	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	//Restrictions of modeling data
	if (BA<1)
	   BA = 1;
	if (A<8)
	   A = 8;

	double res = exp(6.071-1.0491*log(A)-0.3182*log(BA));

	//Correct the result if original BA was very low
	if (BA_orig<0.01)
	    res = 1000;
    else if (BA_orig<0.1)
        res = 400;
    else if (BA_orig<0.4)
        res = 200;

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Basal_area_growth_white_birch_Mielikainen (double BA, double A, double BA_prosb, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (BA_prosb<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA_prosb<=0(BA_prosb=",BA_prosb);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Restrictions of modeling data
	if(A<15)
		A=15;
	if(BA<3)
		BA=3;

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(6.816-0.8409*log(A)-0.3634*log(BA)-0.3331*log(BA_prosb));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Basal_area_growth_spruce_white_birch_mixed_stand_Mielikainen (double BA, double A, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(6.177-0.8441*log(A)-0.5989*log(BA));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Basal_area_growth_spruce_pubescent_birch_mixed_stand_Mielikainen (double BA, double A, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(6.645-1.0261*log(A)-0.5163*log(BA));
	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_white_birch_Oikarinen (double Age, double V, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	*modelresult = (14832/pow(Age,2.2)+3140.5/V);
	return ret;

}


int Volume_growth_white_birch_Mielikainen (double V, double A, double BA_prosb, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (BA_prosb<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA_prosb<=0(BA_prosb=",BA_prosb);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(7.267-0.9317*log(A)-0.2324*log(V)-0.2354*log(BA_prosb));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}



int Volume_growth_pubescent_birch_Mielikainen (double V, double A, double SI_50, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	//According to Mielikinen pubescent birch's site index in this formula should actually be site index for spruce at age of 100 years. Why?
	//It can be estimated to SI_50+10! If this is used, the model will overestimate volume growth badly.
	//double res =exp(3.601-0.8436*log(A)-0.2606*log(V)+0.7597*log(SI_50+10));

	double res =exp(3.601-0.8436*log(A)-0.2606*log(V)+0.7597*log(SI_50));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_white_birch_mixed_stand_Mielikainen (double V, double A, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(6.862-0.6477*log(A)-0.5577*log(V));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_pubescent_birch_mixed_stand_Mielikainen (double V, double A, double SI_100, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (SI_100<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_100<=0(SI_100=",SI_100);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(4.061-0.8555*log(A)-0.3735*log(V)+0.7963*log(SI_100));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_pine_with_dgm_NyyssonenMielikainen (double V, double Age, double D_gM, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
	double SC2_4;

	if(SC==2||SC==3||SC==4) {
		SC2_4=1;

	} else {
		SC2_4=0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(-0.7702-0.09667*pow(log(Age),2)+1.2503*pow(V,(pow((1/V),0.3)))+0.1796*pow(log(D_gM),8)/10000+0.1817*(SC2_4));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_with_dgm_NyyssonenMielikainen (double V, double Age, double D_gM, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
	double SC2_dummy;

	if (SC == 2) {
		SC2_dummy = 1;
	} else {
		SC2_dummy = 0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(8.839-1.2749*log(Age)-0.5948*log(V)+0.00309*pow((log(Age)*log(V)),2)-0.01193*log(Age)*pow(V,2)/100000-0.0006095*pow(log(D_gM),5)+0.1009*(SC2_dummy));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_pine_NyyssonenMielikainen (double V, double Age, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
	double SC2_4;

	if(SC==2||SC==3||SC==4) {
		SC2_4=1;

	} else {
		SC2_4=0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(-0.7632-0.1181*pow(log(Age),2)+1.3516*pow(V,(pow((1/V),0.3)))+0.9116*(SC2_4));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_NyyssonenMielikainen (double V, double Age, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(9.7669-1.5813*log(Age)-0.5730*log(V)+0.003315*pow((log(Age)*log(V)),2)-0.1177*log(Age)*pow(V,2)/100000);

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_pine_entire_country_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(2.77432+1.02982*(5.69-log(Age))-0.06296*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_entire_country_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(10.86168-2.71727*pow(log(Age),0.65)-0.00164*pow(log(V*H_dom),3));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_birch_entire_country_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(9.28134-3.28948*pow(log(Age),0.6)+0.74753*log(V*H_dom)-0.07086*pow(log(V*H_dom),2));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_pine_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(2.95091+1.05986*pow((5.69-log(Age)),0.9)-0.071*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Volume_growth_spruce_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(9.25266-1.99553*pow(log(Age),0.65)-0.00216*pow(log(V*H_dom),3));

	*modelresult = res * rectFactor * SC_factor;
	return ret;
}


int Volume_growth_birch_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(10.6552-2.5083*pow(log(Age),0.6)-0.07567*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_northern_Finland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(2.83315+0.84866*pow((5.69-log(Age)),1.25)-0.0575*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_spruce_northern_Finland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(12.85558-3.22716*pow(log(Age),0.65)-0.16904*log(V*H_dom));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_Kainuu_Pohjois_Pohjanmaa_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(3.68602+0.625*pow((5.69-log(Age)),1.25)-0.0832*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_spruce_Kainuu_Pohjois_Pohjanmaa_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(12.20508-2.68005*pow(log(Age),0.65)-0.28188*log(V*H_dom));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_Lapland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(2.79139+0.81442*pow((5.69-log(Age)),1.25)-0.0572*pow(log(V*H_dom),1.5));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_spruce_Lapland_Gustavsen (double V, double Age, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(11.2276-2.7552*pow(log(Age),0.65)-0.14934*log(V*H_dom));
        *modelresult = res * rectFactor * SC_factor;
        return ret;


}


int Volume_growth_white_birch2_Mielikainen (double V, double A, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(-27.77-10.22*log(A)+35.84*sqrt(log(A))-0.3418*log(V));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_Mielikainen (double V, double A, double V_prosb, double SI_100, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (V_prosb<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_prosb<0(V_prosb=",V_prosb);
	}
	if (SI_100<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_100<=0(SI_100=",SI_100);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res = exp(7.989-0.8873*log(A)-0.1935*log(V)-0.00666*V_prosb+0.04838*sqrt(V_prosb)-0.678*log(SI_100));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_white_birch_in_mixed_stand_Mielikainen (double V, double A, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(8.657-1.129*log(A)-0.5336*log(V));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_white_birch_mixed_stand_Mielikainen (double V, double A, double V_prosb, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	//Valsta have defined a volume limit fuction to avoid too large stand volumes
	//Same kind of limit functions are needed also for V & V models!
	double limit =273.42+0.9308*A;

	if(V>limit)
		V=limit;

	double res =exp(7.050-0.8732*log(A)-0.4187*log(V)-0.5154*V_prosb*pow(A,2)/1000000-1.403*pow((50-V_prosb),2)/pow(A,3));


	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_pine_pubescent_birch_mixed_stand_Mielikainen (double V, double A, double V_prosb, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (A<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "A<=0(A=",A);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(5.762-0.7563*log(A)-0.2546*log(V)-0.5137*V_prosb*pow(A,2)/1000000-0.2887*pow((50-V_prosb),2)/pow(A,3));

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_yield_class1_Gustavsenym (double V, double N, double DS, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(3.7791+0.003*log(V)*log(N)-0.0052*DS+pow(0.2941,2)/2);

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_yield_class2_Gustavsenym (double V, double N, double T_dr, double TS, double P_t, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(-1.7749+0.1977*log(V)+0.2317*log(N)+0.1515*log(T_dr+1)+0.0017*TS-0.0096*P_t+pow(0.4899,2)/2);

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_yield_class3_Gustavsenym (double V, double N, double T_dr, double TS, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(-1.2682+0.2248*log(V)+0.1666*log(N)+0.2024*log(T_dr+1)+0.0013*TS+pow(0.6427,2)/2);

	*modelresult = res * rectFactor * SC_factor;
        return ret;

}


int Volume_growth_yield_class4_Gustavsenym (double V, double N, double T_dr, double TS, double P_t, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =exp(-8.2926+0.3606*log(V)+0.4932*log(N)+0.1777*log(T_dr+1)+0.0044*TS-0.013*P_t+pow(0.9346,2)/2);

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_pine_VuokilaValiaho (double Age, double H_dom, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Restrictions of modeling data
	if (Age<12)
	   Age = 12;

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	double res =-0.40006+434.52/(pow(H_dom,0.4)*pow(Age,1.1))-124.51/pow((pow(H_dom,0.4)*pow(Age,1.1)),2);
	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_spruce_VuokilaValiaho (double Age, double H_dom, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =-0.41018+616.40/(pow(H_dom,0.55)*pow(Age,1.05))-3592.9/pow((pow(H_dom,0.55)*pow(Age,1.05)),2);

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_white_birch_Oikarinen (double Age, double H_dom, double SC,
 					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	double res = -1.103+3177.4/pow((Age+4.5),1.5)+802.8/pow(H_dom,1.8)-3.99*pow(((H_dom+34.7)/Age),1.2);

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_pubescent_birch_Saramaki (double Age_s, double H_dom, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

    // siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;
	double res = 3.04105+0.11580*(100-Age_s)+1.11130*pow(((26-H_dom)/10),5)-2.01025*pow(((26-H_dom)/10),3);

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_pine_Gustavsen (double Age, double H_dom, double SI_100, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (SI_100<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_100<=0(SI_100=",SI_100);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =H_dom*log(SI_100/128.229)*exp(4.70248/pow(Age,0.47692)-4.70248/pow(100,0.47692))*(-2.24269/pow(Age,1.47692));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}


int Dominant_height_growth_spruce_Gustavsen (double Age, double H_dom, double SI_100, double SC,
					   double SC1, double SC2, double SC3, double SC4, double SC5, double SC6,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (SI_100<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_100<=0(SI_100=",SI_100);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	// siteclass dependent "calibration" factor
    double SC_factor = 1.0;
    int SCi = (int) SC;
    switch(SCi) {
        case 1: SC_factor = SC1;
        case 2: SC_factor = SC2;
        case 3: SC_factor = SC3;
        case 4: SC_factor = SC4;
        case 5: SC_factor = SC5;
        case 6: SC_factor = SC6;
    }

	*nres = 1;

	double res =H_dom*log(SI_100/147.481)*exp(4.64631/pow(Age,0.29981)-4.64631/pow(100,0.29981))*(-1.39299/pow(Age,1.29981));

	*modelresult = res * rectFactor * SC_factor;
	return ret;

}
