// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  DynamicstandmodelLibrary.h
//
//	Dynamic stand models library - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

struct stemdata {
	double d;
	double cum_h;
	double cum_v;
};

//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult, warnings and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Basal_area_growth_under_bark_pine_VuokilaValiaho (double BA_u, double Age, double H_dom, double SI_100, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_under_bark_spruce_VuokilaValiaho (double BA_u, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_under_bark_pubescent_birch_Saramaki (double V_u, double Age_s, double H_dom, double LGP, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_pubescent_birch_Mielikainen (double BA, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_white_birch_Mielikainen (double BA, double A, double BA_prosb, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_spruce_white_birch_mixed_stand_Mielikainen (double BA, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_spruce_pubescent_birch_mixed_stand_Mielikainen (double BA, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_white_birch_Oikarinen (double Age, double V, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_white_birch_Mielikainen (double BA, double A, double BA_prosb, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pubescent_birch_Mielikainen (double V, double A, double SI_50, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_white_birch_mixed_stand_Mielikainen (double V, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_pubescent_birch_mixed_stand_Mielikainen (double V, double A, double SI_100, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_with_dgm_NyyssonenMielikainen (double V, double Age, double D_gM, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_with_dgm_NyyssonenMielikainen (double V, double Age, double D_gM, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_NyyssonenMielikainen (double V, double Age, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_NyyssonenMielikainen (double V, double Age, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_entire_country_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_entire_country_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_birch_entire_country_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_birch_southern_Finland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_northern_Finland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_northern_Finland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_Kainuu_Pohjois_Pohjanmaa_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_Kainuu_Pohjois_Pohjanmaa_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_Lapland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_spruce_Lapland_Gustavsen (double V, double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_white_birch2_Mielikainen (double V, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_Mielikainen (double V, double A, double V_prosb, double SI_100, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_white_birch_in_mixed_stand_Mielikainen (double V, double A, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_white_birch_mixed_stand_Mielikainen (double V, double A, double V_prosb, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_pine_pubescent_birch_mixed_stand_Mielikainen (double V, double A, double V_prosb, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_yield_class1_Gustavsenym (double V, double N, double DS, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_yield_class2_Gustavsenym (double V, double N, double T_dr, double TS, double P_t, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_yield_class3_Gustavsenym (double V, double N, double T_dr, double TS, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_growth_yield_class4_Gustavsenym (double V, double N, double T_dr, double TS, double P_t, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_pine_VuokilaValiaho (double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_spruce_VuokilaValiaho (double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_white_birch_Oikarinen (double Age, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_pubescent_birch_Saramaki (double Age_s, double H_dom, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_pine_Gustavsen (double Age, double H_dom, double SI_100, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_growth_spruce_Gustavsen (double Age, double H_dom, double SI_100, double SC, double SC1, double SC2, double SC3, double SC4, double SC5, double SC6, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif