// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  StandLibrary.cpp
//
//	Stand model library - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029


struct RDF_values {
	double RDF_Sp;
	double RDF_Ns;
	double RDF_sb;
	double RDF_pb;
	double RDF_oc;
	double RDF_od;
	double RDF_decid;
};

struct regen_limits {
	double regen_dgm;
	double regen_age;
};

struct carbon_cwl_components {
	double Carbon_A_cwl;
	double Carbon_W_cwl;
	double Carbon_E_cwl;
	double Carbon_N_cwl;
	double Carbon_dimension_cwl;
};

struct carbon_fwl_components {
	double Carbon_A_fwl;
	double Carbon_W_fwl;
	double Carbon_E_fwl;
	double Carbon_N_fwl;
	double Carbon_dimension_fwl;
};

struct carbon_nwl_components {
	double Carbon_A_nwl;
	double Carbon_W_nwl;
	double Carbon_E_nwl;
	double Carbon_N_nwl;
};

struct carbon_soil_components {
	double Carbon_A_cwl_soil;
	double Carbon_W_cwl_soil;
	double Carbon_E_cwl_soil;
	double Carbon_N_cwl_soil;
	double Carbon_H_cwl_soil;
	double Carbon_A_fwl_soil;
	double Carbon_W_fwl_soil;
	double Carbon_E_fwl_soil;
	double Carbon_N_fwl_soil;
	double Carbon_H_fwl_soil;
	double Carbon_A_nwl_soil;
	double Carbon_W_nwl_soil;
	double Carbon_E_nwl_soil;
	double Carbon_N_nwl_soil;
	double Carbon_H_nwl_soil;
};

struct Value_growth {
	double value_growth_abs;
	double value_growth_rel;
	double mean_value_growth_abs;
	double mean_value_growth_rel;
	double value_growth_rel_1;
	double value_growth_rel_2;
	double value_growth_rel_3;
	double value_growth_rel_4;
	double value_growth_rel_5;
	double value_growth_abs_1;
	double value_growth_abs_2;
	double value_growth_abs_3;
	double value_growth_abs_4;
	double value_growth_abs_5;
};

struct PVs {
	double PV;
	double PV_land;
	double PV_1;
	double PV_2;
	double PV_3;
	double PV_4;
	double PV_5;
	double PV_6;
	double PV_7;
	double PV_land_1;
	double PV_land_2;
	double PV_land_3;
	double PV_land_4;
	double PV_land_5;
	double PV_land_6;
	double PV_land_7;
};

struct Carbon_flux {
	double Carbon_flux_natural_removal;
	double Carbon_flux_natural_removal_energy;
};

struct Statutory_regen_limits {
	double reg_dgm;
	double reg_age;
};

struct Mean_growth {
	double mean_i_Vm3;
	double i_Vm3_1;
	double i_Vm3_2;
	double i_Vm3_3;
	double i_Vm3_4;
	double i_Vm3_5;
};

//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult, warnings and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Volume_BAHdomF (double BA, double H_dom, double F, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_of_bark_spruce_VuokilaValiaho (double BA, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_under_bark_pine_VuokilaValiahoLappi (double BA, double Age, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_of_bark_based_on_BA_u_spruce_VuokilaValiaho (double BA_u, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_of_bark_based_on_BA_u_pine_VuokilaValiaho (double BA_u, double Age, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Yield_yield_class1_Gustavsenym (double V, double N, double T_dr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Yield_yield_class2_Gustavsenym (double V, double N, double T_dr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Yield_yield_class3_Gustavsenym (double V, double N, double T_dr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Yield_yield_class4_Gustavsenym (double V, double N, double T_dr, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Form_factor_pine_VuokilaValiaho (double Age, double H_dom, double BA, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Form_factor_spruce_VuokilaValiaho (double BA, double N, double SI_100, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Form_factor_white_birch_Oikarinen (double D_g, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_white_birch_Oikarinen (double V, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_pubescent_birch_Saramaki (double V_u, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_pine_Nissinen (double Age, double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_spruce_Nissinen (double Age, double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_birch_Nissinen (double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_trees_per_hectare_white_birch_Oikarinen (double V, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Proportion_of_stems_removed_in_thinning_pine_VuokilaValiaho (double BA, double N, double V, double RV, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Proportion_of_stems_removed_in_thinning_spruce_VuokilaValiaho (double BA, double N, double V, double H_dom, double RV, double THIN1, double THIN2, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_pine_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_spruce_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_birch_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_pine_peatland_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_spruce_peatland_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_birch_peatland_entire_country_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_pine_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_spruce_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_birch_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_pine_peatland_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_spruce_peatland_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_birch_peatland_Nuutinen (double Age, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_white_birch_Oikarinen (double BA, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_pubescent_birch_Saramaki (double Age, double V_u, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_pine_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_spruce_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_birch_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_pine_peat_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_spruce_peat_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_birch_peat_Nuutinen (double Age, double H_gM, double TS, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_pine_Nissinen (double Age, double D_ari, double TREAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_spruce_Nissinen (double Age, double D_ari, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_birch_Nissinen (double D_ari, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_pine_Hynynenym (double TS, double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_spruce_Hynynenym (double TS, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_birch_Hynynenym (double TS, double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Need_for_ditching_Hokkaym (double LAT, double SINCE_DRAINAGE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_at_stump_height_from_D_gM_Laasasenaho (double D_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_pine_JKK (double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_spruce_JKK (double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_birch_JKK (double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Proportion_of_bark_birch_from_stand_basal_area_JKK (double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_under_bark_birch_JKK (double BA, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_pine_Gustavsen (double Age, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_spruce_Gustavsen (double Age, double H_dom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Two_storey_forest_Hynynenym (double Upper_sum, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Upper_storey_stratum_Hynynenym (double H_gM, double H_ari, double N_stratum, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Upper_storey_stratum_laser_JKK (double H_gM, double H_ari, double N_stratum, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_pine_JKK (double H_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_spruce_JKK (double H_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_birch_JKK (double H_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_trees_JKK (double BA, double D_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Number_of_trees_Niemisto (double BA, double D_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_Niemisto (double N, double D_gM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_of_stratum_Niemisto (double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_JKK (double Age, double SC, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_MELA_JKK (double Age, double t_h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_MELA_tree_JKK (double Age, double t_h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_stump_height_JKK (double Age, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_pubescent_birch_JKK (double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_before_growing_JKK (double BA, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_JKK (double BA, double BA_old, double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_diameter_after_thinning_JKK (double D_gM, double DgM_theor, int *nres, double *modelresult, char *errors,	int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_white_birch_JKK (double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_spruce_JKK (double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_pine_JKK (double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Regeneration_limits_Tapio (double lower_dgm, double upper_dgm, double lower_age, double upper_age, double REGEN_DGM_LEVEL, double REGEN_AGE_LEVEL, int *nres, struct regen_limits *regen, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Regeneration_statutory_limit_JKK (double orig_dgm, double REGENERATION_LIMIT_SUBTRACTION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Regeneration_statutory_limit_in_protection_forest_area_JKK (double regen_dgm, double regen_age, double Municipal_id, int *nres, struct Statutory_regen_limits *limits, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Regeneration_limits_old_SF_Tapio (double SC, double SP, double D_gM, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Regeneration_limits_old_NF_Tapio (double SC, double SP, double D_gM, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_removed_from_logging_roads (double N, double road_width, double road_distance, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_of_stratum_JKK (double D_gM, double N, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_check_JKK (double D_gM, double H_gM, double PEAT, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_seedling_MELA_JKK (double Age, double t_h, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Development_class_JKK (double Age, double H_gM, double D_gM, double SP, double Age_difference, double BA, double N, double Two_st, double MAIN_GROUP, double regen_dgm, double regen_age, double OLD_DEVEL_CLASS, double dgm_upper, double Hdom, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_class_JKK (double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_increase_m3_stratum_JKK (double V, double V_old, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_increase_percent_stratum_JKK (double V, double V_old, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_increase_m3_stand_JKK (double V, double V_old, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Volume_increase_percent_stand_JKK (double V, double V_old, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int V_value_JKK (double stand_value, double value_growth_abs, double value_land, double IR, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Indicating_percent_JKK (double stand_value, double value_growth_abs, double value_land, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Soil_carbon_Liski (double TS, double SC, double MAIN_GROUP, int *nres, struct carbon_soil_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_cwl_Liski (double BM_stems, double BM_stumps, double D_gM, double SP, double natural_removal, double V, double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double time_step, int *nres, struct carbon_cwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_fwl_Liski (double D_gM, double BM_branches, double BM_roots, double SP, double natural_removal, double V, double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double time_step, int *nres, struct carbon_fwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_nwl_Liski (double MANAGEMENT_REGION, double MAIN_SP, double D_gM, double BM_foliage, double BM_stump_bark, double BM_stem_bark, double BM_fine_roots, double SP, double natural_removal, double V, double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double PEAT, double BMASS_COLLECT_PROP_grass, double time_step, int *nres, struct carbon_nwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_cwl_after_harvest_Liski (double Carbon_A_cwl, double Carbon_W_cwl, double Carbon_E_cwl, double Carbon_N_cwl, double Carbon_dimension_cwl, double Harvest_BM_stumps_to_soil, double Harvest_BM_scrapstems_to_soil, double SP, double carbon_in_biomass, int *nres, struct carbon_cwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_fwl_after_harvest_Liski (double Carbon_A_fwl, double Carbon_W_fwl, double Carbon_E_fwl, double Carbon_N_fwl, double Carbon_dimension_fwl, double Harvest_BM_branches_to_soil, double Harvest_BM_roots_to_soil, double Clearing_BM, double Tending_BM, double SP, double carbon_in_biomass, int *nres, struct carbon_fwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_JKK (double D_gM, double H_gM, double PEAT, double SP, double Age, double TS, double SC, double NF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_JKK (double H_gM, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_comp_unit_JKK (double H_gM, double MAIN_SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_JKK (double Age, double D_gM, double N, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_age_JKK (double D_gM, double SP, double t_href, double H_gM, double TS, double SC, double PEAT, double SCRUB, double WASTE, double NF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_JKK (double SP, double TS, double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, double PLANT, double PUB, double PEAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Value_growth_JKK (double stand_value, double stand_value_old, double value_growth_rel_1, double value_growth_rel_2, double value_growth_rel_3, double value_growth_rel_4, double value_growth_rel_5, double value_growth_abs_1, double value_growth_abs_2, double value_growth_abs_3, double value_growth_abs_4, double value_growth_abs_5, double timeStep, double Age, int *nres, struct Value_growth *value_growth, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Value_growth_standsimulator_JKK (double stand_value, double stand_value_old, double value_growth_rel_1, double value_growth_rel_2, double value_growth_rel_3, double value_growth_rel_4, double value_growth_rel_5, double value_growth_abs_1, double value_growth_abs_2, double value_growth_abs_3, double value_growth_abs_4, double value_growth_abs_5, double timeStep, double Age, int *nres, struct Value_growth *value_growth, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_aggr_JKK (double Age, double A, double t_href, double D_gM, double SC, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Update_age_at_breast_height_JKK (double Age, double A, double t_href, double D_gM, double SC, double timeStep, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Productive_value_JKK (double TS, double SC, double D_g, double H_gM, double BA, double Age, double N, double BA_Sp, double BA_Ns, double BA_b, double BA_od, double N_Sp, double N_Ns, double N_b, double N_od, double PEAT, double MAIN_SP, double MAIN_GROUP, double IR, double log_price_Sp, double log_price_Ns, double log_price_b, double log_price_od, double pulp_price_Sp, double pulp_price_Ns, double pulp_price_b, double pulp_price_od, double seedtree_stand, double Peatland_reduction, int *nres, struct PVs *pv_values, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_flux_tending_energy_JKK (double carbon_in_biomass, double mwh_to_kg_c, double Tending_BM, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_flux_natural_removal_JKK (double BM_stems, double BM_branches, double BM_foliage, double natural_removal, double V, double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double mwh_to_kg_c,	int *nres, struct Carbon_flux *values, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_increment_JKK (double SI_50, double climate_change_scenario, double time_step, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_present_state_JKK (double SI_50, double climate_change_scenario, double year, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_nwl_agri_Liski (double Municipal_id, int *nres, struct carbon_nwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Carbon_nwl_agri_plants_Liski (double Municipal_id, double cult_plant, int *nres, struct carbon_nwl_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Soil_carbon_JKK (double County_id, double SC, double Age, int *nres, struct carbon_soil_components *components, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_volume_increase_m3_stratum_JKK (double V, double V_old, double i_V_1, double i_V_2, double i_V_3, double i_V_4, double i_V_5, double timeStep, int *nres, struct Mean_growth *mean_growth, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif