// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  ThinningLibrary.cpp
//
//	Thinning limit model library - Source file
//
//	Here all model functions must be defined

#include <stdio.h>
#include "ThinningmodelLibrary.h"
#include "SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


static void Thinning_spruce_OMT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0625*pow(H_dom,2)+3.2357*H_dom-7.8786;
	limit[0].Remainlimit_upper = -0.0621*pow(H_dom,2)+3.3846*H_dom-16.176;
	limit[0].Remainlimit_lower = -0.0507*pow(H_dom,2)+2.7676*H_dom-13.222;

}


static void Thinning_spruce_MT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0518*pow(H_dom,2)+2.5893*H_dom-2.1343;
	limit[0].Remainlimit_upper = -0.0406*pow(H_dom,2)+2.2773*H_dom-6.9929;
	limit[0].Remainlimit_lower = -0.0332*pow(H_dom,2)+1.8632*H_dom-5.7214;

}


static void Thinning_white_birch_OMT_MT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0268*pow(H_dom,2)+1.4164*H_dom+3.02;
	limit[0].Remainlimit_upper = -0.0423*pow(H_dom,2)+2.3321*H_dom-13.645;
	limit[0].Remainlimit_lower = -0.0335*pow(H_dom,2)+1.8723*H_dom-10.913;

}


static void Thinning_pine_MT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0469*pow(H_dom,2)+2.057*H_dom+7.8364;
	limit[0].Remainlimit_upper = -0.055*pow(H_dom,2)+2.6243*H_dom-6.3329;
	limit[0].Remainlimit_lower = -0.045*pow(H_dom,2)+2.1471*H_dom-5.1814;

}


static void Thinning_pine_VT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0446*pow(H_dom,2)+1.9021*H_dom+7.1829;
	limit[0].Remainlimit_upper = -0.0615*pow(H_dom,2)+2.6649*H_dom-6.7571;
	limit[0].Remainlimit_lower = -0.0504*pow(H_dom,2)+2.1804*H_dom-5.5286;

}


static void Thinning_pine_CT_southern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.1375*pow(H_dom,2)+5.2*H_dom-25.75;
	limit[0].Remainlimit_upper = -0.0521*pow(H_dom,2)+2.442*H_dom-9.4369;
	limit[0].Remainlimit_lower = -0.0425*pow(H_dom,2)+1.993*H_dom-7.686;

}


static void Thinning_pubescent_birch_northern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0421*pow(H_dom,2)+2.1066*H_dom-4.5354;
	limit[0].Remainlimit_upper = -0.0403*pow(H_dom,2)+1.8256*H_dom-4.4157;
	limit[0].Remainlimit_lower = -0.0329*pow(H_dom,2)+1.4937*H_dom-3.6129;

}


static void Thinning_spruce_OMT_MT_northern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0321*pow(H_dom,2)+1.6071*H_dom+6.8029;
	limit[0].Remainlimit_upper = -0.0419*pow(H_dom,2)+2.1777*H_dom-5.4371;
	limit[0].Remainlimit_lower = -0.0343*pow(H_dom,2)+1.7818*H_dom-4.4486;

}


static void Thinning_pine_MT_northern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0214*pow(H_dom,2)+1.1314*H_dom+10.769;
	limit[0].Remainlimit_upper = -0.0543*pow(H_dom,2)+2.5248*H_dom-7.0714;
	limit[0].Remainlimit_lower = -0.0445*pow(H_dom,2)+2.0657*H_dom-5.7857;

}


static void Thinning_pine_VT_northern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0268*pow(H_dom,2)+1.2993*H_dom+8.2657;
	limit[0].Remainlimit_upper = -0.055*pow(H_dom,2)+2.5946*H_dom-9.2757;
	limit[0].Remainlimit_lower = -0.045*pow(H_dom,2)+2.1232*H_dom-7.5957;

}


static void Thinning_pine_CT_northern_Finland_old_Tapio (double H_dom, struct Thinning_limits *limit)
{

	limit[0].Thinning_limit = -0.0125*pow(H_dom,2)+0.625*H_dom+12.8;
	limit[0].Remainlimit_upper = -0.0561*pow(H_dom,2)+2.453*H_dom-9.0314;
	limit[0].Remainlimit_lower = -0.0457*pow(H_dom,2)+2.002*H_dom-7.3543;

}


static void Thinning_spruce_OMT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0139*pow(H_dom,3)-0.9048*pow(H_dom,2)+19.647*H_dom-105.31;
	double Thinning_limit_lower = 0.0058*pow(H_dom,3)-0.4325*pow(H_dom,2)+10.624*H_dom-52.865;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0005*pow(H_dom,3)-0.057*pow(H_dom,2)+3.2722*H_dom-12.826;
	limit[0].Remainlimit_lower = -0.0005*pow(H_dom,3)-0.057*pow(H_dom,2)+3.2722*H_dom-16.826;

}


static void Thinning_spruce_and_pine_MT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper =  0.0074*pow(H_dom,3)-0.4992*pow(H_dom,2)+11.164*H_dom-50.86;
	double Thinning_limit_lower = 0.0029*pow(H_dom,3)-0.2444*pow(H_dom,2)+6.4297*H_dom-25.979;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0044*pow(H_dom,3)-0.2904*pow(H_dom,2)+6.4108*H_dom-25.335;
	limit[0].Remainlimit_lower = 0.0058*pow(H_dom,3)-0.3639*pow(H_dom,2)+7.6808*H_dom-36.473;

}


static void Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0093*pow(H_dom,3)-0.5794*pow(H_dom,2)+12.241*H_dom-63.397;
	double Thinning_limit_lower = 0.0042*pow(H_dom,3)-0.275*pow(H_dom,2)+6.3333*H_dom-29.12;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0571*pow(H_dom,2)+2.6286*H_dom-13.711;
	limit[0].Remainlimit_lower = -0.0607*pow(H_dom,2)+2.7429*H_dom-16.577;

}


static void Thinning_pine_VT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0174*pow(H_dom,3)-1.0774*pow(H_dom,2)+22.121*H_dom-120.26;
	double Thinning_limit_lower = 0.001*pow(H_dom,3)-0.1607*pow(H_dom,2)+5.1887*H_dom-21.557;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0043*pow(H_dom,3)-0.2761*pow(H_dom,2)+5.9848*H_dom-23.461;
	limit[0].Remainlimit_lower = 0.0032*pow(H_dom,3)-0.2071*pow(H_dom,2)+4.5872*H_dom-18.416;

}


static void Thinning_pine_CT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0104*pow(H_dom,3)-0.6839*pow(H_dom,2)+14.719*H_dom-78.134;
	double Thinning_limit_lower = -0.0125*pow(H_dom,3)+0.4643*pow(H_dom,2)-4.2071*H_dom+21.383;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0021*pow(H_dom,3)-0.1929*pow(H_dom,2)+5.0631*H_dom-22.789;
	limit[0].Remainlimit_lower = -0.001*pow(H_dom,3)-0.0375*pow(H_dom,2)+2.4792*H_dom-12.28;

}


static void Thinning_spruce_OMT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.003*pow(H_dom,3)-0.2481*pow(H_dom,2)+6.6604*H_dom-25.268;
	double Thinning_limit_lower = -0.0022*pow(H_dom,3)+0.0153*pow(H_dom,2)+2.2044*H_dom-3.4397;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0014*pow(H_dom,3)-0.0042*pow(H_dom,2)+2.2496*H_dom-6.9905;
	limit[0].Remainlimit_lower = -0.0028*pow(H_dom,3)+0.0676*pow(H_dom,2)+1.0403*H_dom-4.381;

}


static void Thinning_spruce_and_pine_MT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0071*pow(H_dom,3)-0.4507*pow(H_dom,2)+9.7255*H_dom-40.879;
	double Thinning_limit_lower = -0.001*pow(H_dom,3)-0.0205*pow(H_dom,2)+2.281*H_dom-2.0343;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0013*pow(H_dom,3)-0.1145*pow(H_dom,2)+3.3358*H_dom-9.8492;
	limit[0].Remainlimit_lower = 0.0034*pow(H_dom,3)-0.2289*pow(H_dom,2)+5.2443*H_dom-22.864;

}


static void Thinning_pine_VT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0138*pow(H_dom,3)-0.8216*pow(H_dom,2)+16.35*H_dom-81.678;
	double Thinning_limit_lower = 0.0046*pow(H_dom,3)-0.32*pow(H_dom,2)+7.3025*H_dom-31.084;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0003*pow(H_dom,3)-0.0515*pow(H_dom,2)+2.5401*H_dom-7.3048;
	limit[0].Remainlimit_lower = -0.0012*pow(H_dom,3)+0.0019*pow(H_dom,2)+1.3985*H_dom-3.3825;

}


static void Thinning_pine_CT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1286*pow(H_dom,2)+4.8643*H_dom-21.866;
	double Thinning_limit_lower = -0.0063*pow(H_dom,3)+0.2054*pow(H_dom,2)-1.0214*H_dom+9.2686;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0031*pow(H_dom,3)+0.1*pow(H_dom,2)-0.3375*H_dom+7.74;
	limit[0].Remainlimit_lower = 0.001*pow(H_dom,3)-0.0964*pow(H_dom,2)+2.6815*H_dom-10.294;

}


static void Thinning_pubescent_birch_middle_and_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper =  0.0063*pow(H_dom,3)-0.35*pow(H_dom,2)+7.225*H_dom-31.1;
	double Thinning_limit_lower = 0.0104*pow(H_dom,3)-0.5*pow(H_dom,2)+8.7083*H_dom-36.5;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0063*pow(H_dom,3)-0.3125*pow(H_dom,2)+5.65*H_dom-21.9;
	limit[0].Remainlimit_lower = -0.0083*pow(H_dom,3)+0.3375*pow(H_dom,2)-3.9417*H_dom+22.1;

}


static void Thinning_spruce_OMT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1*pow(H_dom,2)+4.08*H_dom-12.54;
	double Thinning_limit_lower = -0.0893*pow(H_dom,2)+3.5071*H_dom-9.3429;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.1013*pow(H_dom,2)+3.8302*H_dom-15.186;
	limit[0].Remainlimit_lower = -0.0996*pow(H_dom,2)+3.768*H_dom-17.643;

}


static void Thinning_pine_and_spruce_MT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1045*pow(H_dom,2)+4.0639*H_dom-13.386;
	double Thinning_limit_lower = -0.0714*pow(H_dom,2)+2.8857*H_dom-6.0143;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0558*pow(H_dom,2)+2.267*H_dom-4.0143;
	limit[0].Remainlimit_lower = -0.0621*pow(H_dom,2)+2.4659*H_dom-8.4857;

}


static void Thinning_pine_VT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.0857*pow(H_dom,2)+3.43*H_dom-10.114;
	double Thinning_limit_lower = -0.0612*pow(H_dom,2)+2.5791*H_dom-5.7286;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0897*pow(H_dom,2)+3.3891*H_dom-14.8;
	limit[0].Remainlimit_lower = -0.0714*pow(H_dom,2)+2.6971*H_dom-11.486;

}


static void Thinning_pine_CT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1036*pow(H_dom,2)+3.7*H_dom-11.891;
	double Thinning_limit_lower = -0.0982*pow(H_dom,2)+3.385*H_dom-11.014;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0893*pow(H_dom,2)+3.13*H_dom-12.486;
	limit[0].Remainlimit_lower = -0.0536*pow(H_dom,2)+2.03*H_dom-7.1714;

}


static void Thinning_spruce_OMT_southern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0139*pow(H_dom,3)-0.9048*pow(H_dom,2)+19.647*H_dom-105.31;
	double Thinning_limit_lower = 0.0058*pow(H_dom,3)-0.4325*pow(H_dom,2)+10.624*H_dom-52.865;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0005*pow(H_dom,3)-0.057*pow(H_dom,2)+3.2722*H_dom-12.826;
	limit[0].Remainlimit_lower = -0.0023*pow(H_dom,3)+0.0556*pow(H_dom,2)+0.7791*H_dom-3.3651;

}


static void Thinning_spruce_and_pine_MT_southern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper =  0.0074*pow(H_dom,3)-0.4992*pow(H_dom,2)+11.164*H_dom-50.86;
	double Thinning_limit_lower = 0.0029*pow(H_dom,3)-0.2444*pow(H_dom,2)+6.4297*H_dom-25.979;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0044*pow(H_dom,3)-0.2904*pow(H_dom,2)+6.4108*H_dom-25.335;
	limit[0].Remainlimit_lower = -0.0069*pow(H_dom,3)+0.2738*pow(H_dom,2)-2.5198*H_dom+11.7619;

}


static void Thinning_white_birch_OMT_MT_southern_and_middle_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0093*pow(H_dom,3)-0.5794*pow(H_dom,2)+12.241*H_dom-63.397;
	double Thinning_limit_lower = 0.0042*pow(H_dom,3)-0.275*pow(H_dom,2)+6.3333*H_dom-29.12;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0571*pow(H_dom,2)+2.6286*H_dom-13.711;
	limit[0].Remainlimit_lower = 0.3312-0.39998*H_dom+0.102*pow(H_dom,2)-0.002683*pow(H_dom,3);

}


static void Thinning_pine_VT_southern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0174*pow(H_dom,3)-1.0774*pow(H_dom,2)+22.121*H_dom-120.26;
	double Thinning_limit_lower = 0.001*pow(H_dom,3)-0.1607*pow(H_dom,2)+5.1887*H_dom-21.557;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0043*pow(H_dom,3)-0.2761*pow(H_dom,2)+5.9848*H_dom-23.461;
	limit[0].Remainlimit_lower = 0.0023*pow(H_dom,3)-0.1806*pow(H_dom,2)+4.4709*H_dom-22.6349;

}


static void Thinning_pine_CT_southern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0104*pow(H_dom,3)-0.6839*pow(H_dom,2)+14.719*H_dom-78.134;
	double Thinning_limit_lower = -0.0125*pow(H_dom,3)+0.4643*pow(H_dom,2)-4.2071*H_dom+21.383;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0021*pow(H_dom,3)-0.1929*pow(H_dom,2)+5.0631*H_dom-22.789;
	limit[0].Remainlimit_lower = 0.0035*pow(H_dom,3)-0.2173*pow(H_dom,2)+4.4921*H_dom-20.6667;

}


static void Thinning_spruce_OMT_middle_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.003*pow(H_dom,3)-0.2481*pow(H_dom,2)+6.6604*H_dom-25.268;
	double Thinning_limit_lower = -0.0022*pow(H_dom,3)+0.0153*pow(H_dom,2)+2.2044*H_dom-3.4397;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0014*pow(H_dom,3)-0.0042*pow(H_dom,2)+2.2496*H_dom-6.9905;
	limit[0].Remainlimit_lower = -0.0023*pow(H_dom,3)+0.0556*pow(H_dom,2)+0.7791*H_dom-3.3651;

}


static void Thinning_spruce_and_pine_MT_middle_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0071*pow(H_dom,3)-0.4507*pow(H_dom,2)+9.7255*H_dom-40.879;
	double Thinning_limit_lower = -0.001*pow(H_dom,3)-0.0205*pow(H_dom,2)+2.281*H_dom-2.0343;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0013*pow(H_dom,3)-0.1145*pow(H_dom,2)+3.3358*H_dom-9.8492;
	limit[0].Remainlimit_lower = -0.0804*pow(H_dom,2)+3.232*H_dom-18.29;

}


static void Thinning_pine_VT_middle_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = 0.0138*pow(H_dom,3)-0.8216*pow(H_dom,2)+16.35*H_dom-81.678;
	double Thinning_limit_lower = 0.0046*pow(H_dom,3)-0.32*pow(H_dom,2)+7.3025*H_dom-31.084;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0003*pow(H_dom,3)-0.0515*pow(H_dom,2)+2.5401*H_dom-7.3048;
	limit[0].Remainlimit_lower = 0.0013*pow(H_dom,3)-0.1806*pow(H_dom,2)+4.4709*H_dom-22.6349;

}


static void Thinning_pine_CT_middle_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = -0.1286*pow(H_dom,2)+4.8643*H_dom-21.866;
	double Thinning_limit_lower = -0.0063*pow(H_dom,3)+0.2054*pow(H_dom,2)-1.0214*H_dom+9.2686;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0031*pow(H_dom,3)+0.1*pow(H_dom,2)-0.3375*H_dom+7.74;
	limit[0].Remainlimit_lower = 0.0035*pow(H_dom,3)-0.2173*pow(H_dom,2)+4.4921*H_dom-20.6667;

}


static void Thinning_pubescent_birch_middle_and_northern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper =  0.0063*pow(H_dom,3)-0.35*pow(H_dom,2)+7.225*H_dom-31.1;
	double Thinning_limit_lower = 0.0104*pow(H_dom,3)-0.5*pow(H_dom,2)+8.7083*H_dom-36.5;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0063*pow(H_dom,3)-0.3125*pow(H_dom,2)+5.65*H_dom-21.9;
	limit[0].Remainlimit_lower = -0.0116*pow(H_dom,3)+0.5546*pow(H_dom,2)-8.2295*H_dom+45.746;

}


static void Thinning_spruce_OMT_northern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = -0.1*pow(H_dom,2)+4.08*H_dom-12.54;
	double Thinning_limit_lower = -0.0893*pow(H_dom,2)+3.5071*H_dom-9.3429;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.1013*pow(H_dom,2)+3.8302*H_dom-15.186;
	limit[0].Remainlimit_lower = -0.0804*pow(H_dom,2)+3.232*H_dom-18.29;

}


static void Thinning_pine_and_spruce_MT_northern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = -0.1045*pow(H_dom,2)+4.0639*H_dom-13.386;
	double Thinning_limit_lower = -0.0714*pow(H_dom,2)+2.8857*H_dom-6.0143;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0558*pow(H_dom,2)+2.267*H_dom-4.0143;
	limit[0].Remainlimit_lower = 0.0069*pow(H_dom,3)-0.4345*pow(H_dom,2)+8.9841*H_dom-49.3333;

}


static void Thinning_pine_VT_northern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = -0.0857*pow(H_dom,2)+3.43*H_dom-10.114;
	double Thinning_limit_lower = -0.0612*pow(H_dom,2)+25791*H_dom-5.7286;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0897*pow(H_dom,2)+3.3891*H_dom-14.8;
	limit[0].Remainlimit_lower = -15.5281+3.58327*H_dom-0.1588*pow(H_dom,2)+0.002315*pow(H_dom,3);

}


static void Thinning_pine_CT_northern_Finland_statutory (double H_dom, double THINNING_LEVEL, struct Thinning_statutory_limits *limit)
{

	double Thinning_limit_upper = -0.1036*pow(H_dom,2)+3.7*H_dom-11.891;
	double Thinning_limit_lower = -0.0982*pow(H_dom,2)+3.385*H_dom-11.014;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0893*pow(H_dom,2)+3.13*H_dom-12.486;
	limit[0].Remainlimit_lower = 0.0116*pow(H_dom,3)-0.6349*pow(H_dom,2)+11.4616*H_dom-59.0317;

}

static void Thinning_pine_MT_PEAT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper =  0.009028*pow(H_dom,3)-0.565774*pow(H_dom,2)+12.024008*H_dom-55.590476;
	double Thinning_limit_lower = 0.005324*pow(H_dom,3)-0.376*pow(H_dom,2)+8.804*H_dom-40.2;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0010417*pow(H_dom,3)-0.0946429*pow(H_dom,2)+2.6494048*H_dom-1.9714286;
	limit[0].Remainlimit_lower = 0.0006944*pow(H_dom,3)-0.0747024*pow(H_dom,2)+2.2394841*H_dom-3.5190476;

}


static void Thinning_pine_VT_PEAT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.008333*pow(H_dom,3)-0.540179*pow(H_dom,2)+11.754167*H_dom-56.657143;
	double Thinning_limit_lower = 0.007755*pow(H_dom,3)-0.521825*pow(H_dom,2)+11.586442*H_dom-58.769841;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.001389*pow(H_dom,3)-0.121726*pow(H_dom,2)+3.259325*H_dom-7.509524;
	limit[0].Remainlimit_lower = -0.0002315*pow(H_dom,3)-0.0248016*pow(H_dom,2)+1.3386243*H_dom+0.8920635;

}


static void Thinning_pine_CT_PEAT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.009144*pow(H_dom,3)-0.580159*pow(H_dom,2)+12.24041*H_dom-59.665079;
	double Thinning_limit_lower = 0.011343*pow(H_dom,3)-0.701687*pow(H_dom,2)+14.448479*H_dom-75.125397;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0008102*pow(H_dom,3)-0.0864087*pow(H_dom,2)+2.5148148*H_dom-4.1507937;
	limit[0].Remainlimit_lower = 0.003241*pow(H_dom,3)-0.207242*pow(H_dom,2)+4.403902*H_dom-16.288889;

}


static void Thinning_pine_MT_PEAT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.009028*pow(H_dom,3)-0.565774*pow(H_dom,2)+12.024008*H_dom-55.590476;
	double Thinning_limit_lower = 0.005324*pow(H_dom,3)-0.376*pow(H_dom,2)+8.804*H_dom-40.2;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0010417*pow(H_dom,3)-0.0946429*pow(H_dom,2)+2.6494048*H_dom-1.9714286;
	limit[0].Remainlimit_lower = 0.0006944*pow(H_dom,3)-0.0747024*pow(H_dom,2)+2.2394841*H_dom-3.5190476;

}


static void Thinning_pine_VT_PEAT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.008333*pow(H_dom,3)-0.540179*pow(H_dom,2)+11.754167*H_dom-56.657143;
	double Thinning_limit_lower = 0.007755*pow(H_dom,3)-0.521825*pow(H_dom,2)+11.586442*H_dom-58.769841;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.001389*pow(H_dom,3)-0.121726*pow(H_dom,2)+3.259325*H_dom-7.509524;
	limit[0].Remainlimit_lower = -0.0002315*pow(H_dom,3)-0.0248016*pow(H_dom,2)+1.3386243*H_dom+0.8920635;

}


static void Thinning_pine_CT_PEAT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.009144*pow(H_dom,3)-0.580159*pow(H_dom,2)+12.24041*H_dom-59.665079;
	double Thinning_limit_lower = 0.011343*pow(H_dom,3)-0.701687*pow(H_dom,2)+14.448479*H_dom-75.125397;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0008102*pow(H_dom,3)-0.0864087*pow(H_dom,2)+2.5148148*H_dom-4.1507937;
	limit[0].Remainlimit_lower = 0.003241*pow(H_dom,3)-0.207242*pow(H_dom,2)+4.403902*H_dom-16.288889;

}


static void Thinning_pine_MT_PEAT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.009375*pow(H_dom,3)-0.604464*pow(H_dom,2)+13.05*H_dom-65.228571;
	double Thinning_limit_lower = 0.009606*pow(H_dom,3)-0.624306*pow(H_dom,2)+13.457804*H_dom-70.220635;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.005324*pow(H_dom,3)-0.341171*pow(H_dom,2)+7.29914*H_dom-32.131746;
	limit[0].Remainlimit_lower = 0.002431*pow(H_dom,3)-0.174405*pow(H_dom,2)+4.110516*H_dom-15.995238;

}


static void Thinning_pine_VT_PEAT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.014352*pow(H_dom,3)-0.867659*pow(H_dom,2)+17.558862*H_dom-92.307937;
	double Thinning_limit_lower = 0.011111*pow(H_dom,3)-0.694345*pow(H_dom,2)+14.408532*H_dom-75.161905;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.004167*pow(H_dom,3)-0.270536*pow(H_dom,2)+5.831548*H_dom-23.228571;
	limit[0].Remainlimit_lower = 0.002546*pow(H_dom,3)-0.1736*pow(H_dom,2)+3.911*H_dom-14.53;

}


static void Thinning_pine_CT_PEAT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.006366*pow(H_dom,3)-0.410813*pow(H_dom,2)+8.769974*H_dom-38.61746;
	double Thinning_limit_lower = 0.00625*pow(H_dom,3)-0.372321*pow(H_dom,2)+7.355357*H_dom-27.971429;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.008102*pow(H_dom,3)-0.4659*pow(H_dom,2)+8.873*H_dom-40.38;
	limit[0].Remainlimit_lower = 0.003009*pow(H_dom,3)-0.191865*pow(H_dom,2)+4.040741*H_dom-15.096825;

}

static void Thinning_pine_MT_PEAT_most_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.005208*pow(H_dom,3)-0.382143*pow(H_dom,2)+9.079167*H_dom-42.985714;
	double Thinning_limit_lower = 0.001273*pow(H_dom,3)-0.1359*pow(H_dom,2)+4.014*H_dom-11.29;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.002199*pow(H_dom,3)+0.071528*pow(H_dom,2)-0.150926*H_dom+11.088889;
	limit[0].Remainlimit_lower = 0.0005787*pow(H_dom,3)-0.0710317*pow(H_dom,2)+2.1873677*H_dom-5.015873;

}


static void Thinning_pine_VT_PEAT_most_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.01435*pow(H_dom,3)-0.8891*pow(H_dom,2)+18.37*H_dom-100.6;
	double Thinning_limit_lower = 0.01019*pow(H_dom,3)-0.6364*pow(H_dom,2)+13.26*H_dom-69.11;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.003472*pow(H_dom,3)-0.240476*pow(H_dom,2)+5.459921*H_dom-22.72381;
	limit[0].Remainlimit_lower = 0.0008102*pow(H_dom,3)-0.0881944*pow(H_dom,2)+2.5969577*H_dom-9.1936508;

}

int Thinning_limits_JKK (double H_dom, double BA, double MAIN_SP, double MANAGEMENT_REGION,
                        double SC, double TS, double PEAT, double THINNING_LEVEL,
						int *nres, struct Thinning_limits *limits, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double thinning_limit=0;
    double remain_up=0;
    double remain_low=0;
    
    // Assign H_dom to 10 to be able to calculate remaining limits also for short trees  
    if (H_dom<=10)
    	H_dom = 10.01;
    
	//Calculate thinning limits and define whether comp_unit is ready for thinning
	//Southern Finland
	if (MANAGEMENT_REGION==1)
	{
	        // Mineral soil
	        if (PEAT==0)
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    			if (SC<=3)
	    			{
	    				Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			else if (SC==4)
	    			{
	    				Thinning_pine_VT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//CT, ClT
	    			else
	    			{
	    				Thinning_pine_CT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Spruce dominated stand
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	           	}
	        }
	        // Peatland
	        else
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    			if (SC<=3)
	    			{
	    				Thinning_pine_MT_PEAT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			else if (SC==4)
	    			{
	    				Thinning_pine_VT_PEAT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//CT, ClT
	    			else
	    			{
	    				Thinning_pine_CT_PEAT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Spruce dominated stand (the same models as on mineral soil) 
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	            	}
	        }
        
	}
	//Middle Finland
	else if (MANAGEMENT_REGION==2)
	{
	        // Mineral soil
	        if (PEAT==0)
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    			if (SC<=3)
	    			{
	    				Thinning_spruce_and_pine_MT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			else if (SC==4)
	    			{
	    				Thinning_pine_VT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//CT, ClT
	    			else
	    			{
	    				Thinning_pine_CT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Spruce dominated stand
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_spruce_and_pine_MT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	            	}
	        }
	        // Peatland
	        else
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    			if (SC<=3)
	    			{
	    				Thinning_pine_MT_PEAT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			else if (SC==4)
	    			{
	    				Thinning_pine_VT_PEAT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//CT, ClT
	    			else
	    			{
	    				Thinning_pine_CT_PEAT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Spruce dominated stand (the same models as on mineral soil)
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_spruce_and_pine_MT_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_pubescent_birch_middle_and_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	            	}
	        }		
	}
	//Northern Finland (MANAGEMENT_REGION==3)
	else
	{
	        // Mineral soil
	        if (PEAT==0)
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    			if (SC<=3)
	    			{
	    				Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			else if (SC==4)
	    			{
	    				Thinning_pine_VT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
			     	}
	    			//CT, ClT
	    			else
	    			{
	    				Thinning_pine_CT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Spruce dominated stand
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_pubescent_birch_middle_and_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	            	}
	        }
	        // Peatland 
	        else
	        {
	    		//Pine dominated stands
	    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    		{
	    		    if (SC<=3)
	    		    {
		                    if (TS >= 900)
		                    {
		    		    	Thinning_pine_MT_PEAT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
		                    }
		                    else
		                    {
		                        Thinning_pine_MT_PEAT_most_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
		                    }
		            }
	    		    else if (SC==4)
	    		    {
	                    	    if (TS >= 900)
	                	    {
	    				    Thinning_pine_VT_PEAT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
			            }
	                    	    else
	                    	    {
	                    	    	Thinning_pine_VT_PEAT_most_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	                    	    }
	                    }
	    		    //CT, ClT
	    		    else
	    		    {
	    			    Thinning_pine_CT_PEAT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		    }
	    		}
	    		//Spruce dominated stand
	    		else if (MAIN_SP==2&&H_dom>10)
	    		{
	    			if (SC<=2)
	    			{
	    				Thinning_spruce_OMT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    			//MT, VT, CT, ClT
	    			else
	    			{
	    				Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    			}
	    		}
	    		//Birch or other deciduous dominated stand
	    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    		{
	    			//OMaT, OMT, MT, VT, CT, ClT
	    			Thinning_pubescent_birch_middle_and_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//Too little H_dom
	    		else
	    		{
	               		thinning_limit = 99;
	           	}
	        }
	}
	
	if (thinning_limit==0)
	{
	   thinning_limit = limits[0].Thinning_limit;
	   remain_up = limits[0].Remainlimit_upper;
	   remain_low = limits[0].Remainlimit_lower;
    	}
    
	*nres = 1;
	limits[0].Thinning_limit = thinning_limit;
	limits[0].Remainlimit_upper = remain_up;
	limits[0].Remainlimit_lower = remain_low;
	limits[0].JUST_THINNED = 0;
	
	*nres = 1;
	return ret;

}

int Thinning_statutory_limits_JKK (double H_dom, double BA, double MAIN_SP, double MANAGEMENT_REGION,
                        double SC, double TS, double PEAT, double Municipal_id, double THINNING_LEVEL,
						int *nres, struct Thinning_statutory_limits *limits, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double thinning_limit=0;
    double remain_up=0;
    double remain_low=0;
    int protection_forest_area = 0;
    double statutory_N;
	//Calculate thinning limits and statutory minimum tree number after thinning for stand's dominant height less than 12 m
	//Southern Finland
	if (MANAGEMENT_REGION==1)
        {
    		//Pine dominated stands
    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
    		{
    			if (SC<=3)
    			{
    				Thinning_spruce_and_pine_MT_southern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			else if (SC==4)
    			{
    				Thinning_pine_VT_southern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			//CT, ClT
    			else
    			{
    				Thinning_pine_CT_southern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2&&H_dom>10)
    		{
    			if (SC<=2)
    			{
    				Thinning_spruce_OMT_southern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			//MT, VT, CT, ClT
    			else
    			{
    				Thinning_spruce_and_pine_MT_southern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Birch or other deciduous dominated stand
    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
    		{
    			//OMaT, OMT, MT, VT, CT, ClT
    			Thinning_white_birch_OMT_MT_southern_and_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    		}
    		//Too little H_dom
    		else
    		{
               		thinning_limit = 99;
            	}
    }
    //Middle Finland
	else if (MANAGEMENT_REGION==2)
	{
    		//Pine dominated stands
    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
    		{
    			if (SC<=3)
    			{
    				Thinning_spruce_and_pine_MT_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			else if (SC==4)
    			{
    				Thinning_pine_VT_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			//CT, ClT
    			else
    			{
    				Thinning_pine_CT_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2&&H_dom>10)
    		{
    			if (SC<=2)
    			{
    				Thinning_spruce_OMT_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			//MT, VT, CT, ClT
    			else
    			{
    				Thinning_spruce_and_pine_MT_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Birch or other deciduous dominated stand
    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
    		{
    			//OMaT, OMT, MT, VT, CT, ClT
    			Thinning_white_birch_OMT_MT_southern_and_middle_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    		}
    		//Too little H_dom
    		else
    		{
               		thinning_limit = 99;
            	}	
	}
	//Northern Finland (MANAGEMENT_REGION==3)
	else
	{
		if (Municipal_id==47||Municipal_id==148||Municipal_id==261||Municipal_id==498||Municipal_id==732||Municipal_id==742||Municipal_id==758||Municipal_id==890)
			protection_forest_area = 1;
    		//Pine dominated stands
    		if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
    		{
    			if (SC<=3)
    			{
    				Thinning_pine_and_spruce_MT_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			else if (SC==4)
    			{
    				Thinning_pine_VT_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
		        }
    			//CT, ClT
    			else
    			{
    				Thinning_pine_CT_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2&&H_dom>10)
    		{
    			if (SC<=2)
    			{
    				Thinning_spruce_OMT_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    			//MT, VT, CT, ClT
    			else
    			{
    				Thinning_pine_and_spruce_MT_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    			}
    		}
    		//Birch or other deciduous dominated stand
    		else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
    		{
    			//OMaT, OMT, MT, VT, CT, ClT
    			Thinning_pubescent_birch_middle_and_northern_Finland_statutory(H_dom, THINNING_LEVEL, limits);
    		}
    		//Too little H_dom
    		else
    		{
               		thinning_limit = 99;
            	}
	}
	
	//Calculate statutory minimum tree number after thinning for stand's dominant height less than 12 m
	if (MANAGEMENT_REGION==1)
        {
    		//Pine dominated stands
    		if (MAIN_SP==1||MAIN_SP==8)
    		{
    			if (SC<=4)
    			{	
    				statutory_N = 800;
    			}
    			//CT, ClT
    			else
    			{
    				statutory_N = 700;
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2)
    		{
    			statutory_N = 800;
    		}
    		//Birch
    		else if (MAIN_SP==3)
    		{
    			statutory_N = 600;
    		}
    		//Birch or other deciduous dominated stand
    		else
    		{
    			statutory_N = 700;
    		}
    	}
    	//Middle Finland
	else if (MANAGEMENT_REGION==2)
	{
    		//Pine dominated stands
    		if (MAIN_SP==1||MAIN_SP==8)
    		{
    			if (SC==4)
    			{	
    				statutory_N = 800;
    			}
    			//MT, CT, ClT
    			else
    			{
    				statutory_N = 700;
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2)
    		{
    			if (SC<=2)
    			{	
    				statutory_N = 800;
    			}
    			//MT, VT, CT, ClT
    			else
    			{
    				statutory_N = 700;
    			}
    		}
    		//Birch
    		else if (MAIN_SP==3)
    		{
    			statutory_N = 600;
    		}
    		//Birch or other deciduous dominated stand
    		else
    		{
    			statutory_N = 700;
    		}	
	}
	//Northern Finland (MANAGEMENT_REGION==3)
	else
	{
		//Pine dominated stands
    		if (MAIN_SP==1||MAIN_SP==8)
    		{
    			if (SC<=4)
    			{	
    				statutory_N = 700;
    			}
    			//CT, ClT
    			else
    			{
    				statutory_N = 600;
    			}
    		}
    		//Spruce dominated stand
    		else if (MAIN_SP==2)
    		{
    			statutory_N = 700;
    		}
    		//Birch
    		else if (MAIN_SP==3)
    		{
    			statutory_N = 500;
    		}
    		//Birch or other deciduous dominated stand
    		else
    		{
    			statutory_N = 600;
    		}
	}
	
	if (thinning_limit==0)
	{
		if (PEAT==1 && protection_forest_area == 1)
		{
			thinning_limit = limits[0].Thinning_limit;
	   		remain_up = limits[0].Remainlimit_upper;
	   		remain_low = 0.9*(limits[0].Remainlimit_lower)-1;
	   		statutory_N = statutory_N - 100;
		}
		else if (PEAT==1 && protection_forest_area == 0)
		{
			thinning_limit = limits[0].Thinning_limit;
	   		remain_up = limits[0].Remainlimit_upper;
	   		remain_low = 0.9*(limits[0].Remainlimit_lower);
		}
		else if (PEAT==0 && protection_forest_area == 1)
		{
			thinning_limit = limits[0].Thinning_limit;
	   		remain_up = limits[0].Remainlimit_upper;
	   		remain_low = (limits[0].Remainlimit_lower)-1;
	   		statutory_N = statutory_N - 100;
		}
		else
		{
		   thinning_limit = limits[0].Thinning_limit;
		   remain_up = limits[0].Remainlimit_upper;
		   remain_low = limits[0].Remainlimit_lower;
	    	}
    	}
    
	*nres = 1;
	limits[0].Thinning_limit = thinning_limit;
	limits[0].Remainlimit_upper = remain_up;
	limits[0].Remainlimit_lower = remain_low;
	limits[0].JUST_THINNED = 0;
	limits[0].statutory_N = statutory_N;
	
	*nres = 1;
	return ret;

}
