// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  TreegrowthmodelLibrary.cpp
//
// 	Tree level growth Models - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029

struct Tree_h_peatland {
	double i_h;
	double h;
};

//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Height_growth_pine_Hynynenym (double d, double D_domvmi, double i_Hdom5, double cr, double CR_domvmi, double RDF_L, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_spruce_Hynynenym (double d, double D_domvmi, double i_Hdom5, double cr, double CR_domvmi, double RDF_L, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_birch_Hynynenym (double d, double D_domvmi, double i_Hdom5, double PLANT, double RDF_L, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_pine_Hynynenym (double d, double D_gM, double BA, double LAT, double SC, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_spruce_Hynynenym (double d, double D_gM, double BA, double LAT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_birch_Hynynenym (double d, double D_gM, double BA, double LAT, double ALT, double BA_prosb, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_pine_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_spruce_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_birch_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF, double TS, double H_domvmi, double PLANT, double SP, double h, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_pine_Hynynenym (double d, double BA, double BAL, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_spruce_Hynynenym (double d, double BA_prosNs, double BAL, double SC, double TS, double PDR, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_birch_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BAL, double SC, double TS, double PDR, double SEA, double SUBGROUP, double SINCE_THINNING, double SINCE_DRAINAGE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_pine_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h, double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_spruce_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h, double ALT, double SCRUB, double WASTE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_birch_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF, double TS, double H_domvmi, double PLANT, double SP, double h, double SC, double LAKE, double WASTE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_pine_with_bias_corr_Hynynenym (double d, double BA, double BA_L, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_spruce_with_bias_corr_Hynynenym (double d, double BA_prosNs, double BA_L, double SC, double TS, double PDR, double H_domvmi, double ALT, double LAKE, double UNDRAINED, double WASTE, double SUBGROUP, double SCRUB, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BA_L, double SC, double TS, double PDR, double SEA, double SUBGROUP, double ALT, double SCRUB, double UNDRAINED, double SP, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_pine_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double SC, double TS, double ALT, double SEA, double SCRUB, double UNDRAINED, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_spruce_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double TS, double ALT, double LAKE, double SC, double SCRUB, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_peatland_birch_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double ALT, double BA_prosb, double TS, double LAKE, double SC, double SCRUB, double UNDRAINED, double SP, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_peatland_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_peatland_spruce_with_mineralsoil_model_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION, double cr, double h, double RDF_L, double RDF_Sp, double RDF_Ns, double RDF_decid, double SI_50, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_h_mineral_soil_JKK (double d, double cr, double SP, double i_Hdom5, double CR_dom, double RDF_L, double PLANT, double D_dom, double h, double h_max_Sp, double h_max_Ns, double h_max_sb, double h_max_pb, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Grow_ba_mineral_soil_JKK (double d, double cr, double h, double SP, double H_dom, double RDF, double RDF_L, double PLANT, double RDF_Sp, double RDF_Ns, double RDF_decid, double SI_50, double TS, double SC, double D_dom, double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING, double BA, double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_height_before_growing_d_peatland_JKK (double d, double D_gM, double BA, double SP, double LAT, double SC, double SINCE_THINNING, double ALT, double BA_prosb, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_height_after_growing_d_peatland_JKK (double d, double h, double h_before, double D_gM, double BA, double SP, double LAT, double SC, double SINCE_THINNING, double ALT, double BA_prosb, int *nres, struct Tree_h_peatland *h_peatland, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int H_dom_growth_JKK (double SP, double A, double H_gM, double TS, double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY, double RDF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_pine_Hynynenym (double A, double TS, double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_pine_Hynynenym (double A, double TS,	double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_spruce_Hynynenym (double A, double TS, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_spruce_Hynynenym (double A, double TS,	double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, int *nres, double *modelresult, char *errors,	int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_birch_Hynynenym (double A, double TS, double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_birch_Hynynenym (double A, double TS, double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS, int *nres, double *modelresult, char *errors,	int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_pine_with_bias_corr_Hynynenym (double A, double TS,	double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, double SCRUB, double WASTE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_spruce_with_bias_corr_Hynynenym (double A, double TS, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double ALT, double SCRUB, double WASTE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_birch_with_bias_corr_Hynynenym (double A, double TS, double LAKE, double PLANT, double SC, double SP, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_pine_with_bias_corr_Hynynenym (double A, double TS, double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, double SCRUB, double WASTE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_spruce_with_bias_corr_Hynynenym (double A, double TS, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double ALT, double SCRUB, double WASTE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Dominant_height_after_5_years_birch_with_bias_corr_Hynynenym (double A, double TS, double LAKE, double PLANT, double SC, double SP, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_pine_with_bias_corr_fixed_Hynynenym (double d, double BA, double BA_L, double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double SCRUB, double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_spruce_with_bias_corr_fixed_Hynynenym (double d, double BA_prosNs, double BA_L, double SC, double TS, double PDR, double H_domvmi, double ALT, double LAKE, double UNDRAINED, double WASTE, double SUBGROUP, double SCRUB, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_peatland_birch_with_bias_corr_fixed_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BA_L, double SC, double TS, double PDR, double SEA, double SUBGROUP, double ALT, double SCRUB, double UNDRAINED, double SP, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif