import sys, os

python_include = os.path.join(os.path.dirname(sys.executable), 'include')
python_libs = os.path.join(os.path.dirname(sys.executable), 'libs')
python_PC = os.path.join(os.path.dirname(sys.executable), 'PC')
python_PCbuild = os.path.join(os.path.dirname(sys.executable), 'PCbuild')
python_lib = os.path.join(os.path.dirname(sys.executable), 
                          'libs\\python%d%d.lib' % sys.version_info[:2])

def make_dll(filenames):
    """This function will create a single dll from the given files.
    IMPORTANT: The first filename is for the .dll, the following ones are
    for the actual C-code (each name will get .c, .h and .o appended
    automagically)
    
    filenames - (path, name) combinations where path is Finland/Sweden/...
    """
    used_paths = set()
    def _fix_path(path, filename=None):
        path = os.path.relpath(os.path.join('..', '..', path, 'c'))
        used_paths.add(path)
        if not filename:
            return path
        else:
            return os.path.join(path, filename)
    
    sys_id = str.lower(sys.platform[:3])
    posix_source = ''
    if sys_id=='lin':
        #posix_source = '-D_POSIX_SOURCE=1'
        pass
    else:
        posix_source = '-D_POSIX_SOURCE=1'
    if sys_id=='win':
        fpic = ''
    else:
        fpic = '-fPIC'
    dllname = '.'.join((filenames[0][1], 'dll'))
    dllpath = _fix_path(filenames[0][0])
    dll_file = os.path.join(dllpath, dllname)
    input = ''
    if sys_id=='win' and dllname=='ModelaggregationsModels.dll':
        dmode = '-DMODE=2'
    elif sys_id=='win':
        dmode = '-DMODE=1'
    else:
        dmode = '-DMODE=3'
    if sys.version_info[:2] <= (2, 5) or sys_id != 'win':
        for filename in filenames[1:]:
            print "Compiling %s..." % (filename,)
            ffilename = _fix_path(*filename)
            to_run = ' '.join(('gcc', fpic, '-c', dmode, posix_source,
                               '%s.h' % ffilename, 
                               '%s.c' % ffilename))
            print to_run
            os.system(to_run)
            input=' '.join((input, '%s.o' % filename[1]))
        print "output =", dll_file
        print "input =", input
        print "Creating dll..."
        os.system(' '.join(('gcc', '-shared', '-dynamiclib', '-o',
                            ''.join((dll_file, input)) )) )
    else:
        is_64bit = sys.maxsize > 2**32
        dll_file = os.path.join(dllpath, filenames[0][1])
        
        #for filename in filenames[1:]:
        print "Compiling %s..." % (dllname,)
        to_run = ' '.join(('cl', '/c', dmode, '/O2', '/MD', 
                           '/W3', '/GS-', 
                           '/DNDEBUG', 
                           '/D_CRT_SECURE_NO_WARNINGS', '-I'+python_include,
                           '-I'+python_PC,
                           ' '.join(['/Tp%s.c' % _fix_path(*filename) \
                                     for filename in filenames[1:]])))
        print to_run
        os.system(to_run)
        machine_param = '/MACHINE:X64'
        if not is_64bit:
            machine_param = ''
        to_run = ' '.join(('link', machine_param, '/DLL', '/nologo', '/INCREMENTAL:NO',
                           '/LIBPATH:'+python_libs, 
                           '/LIBPATH:'+python_PCbuild,
                           ' '.join(['%s.obj' % filename[1] \
                                     for filename in filenames[1:]]),
                           '/OUT:'+dll_file+'.dll', 
                           '/IMPLIB:'+dll_file+'.lib',
                           '/MANIFESTFILE:'+dll_file+'.dll.manifest',
                           ))
        print to_run
        os.system(to_run)
        to_run = ' '.join(('mt', '-nologo', '-manifest', 
                           dll_file+'.dll.manifest',
                           '-outputresource:'+dll_file+'.dll;2'))
        print to_run
        os.system(to_run)
        
    
    print "Removing lingering files..."
    for path in used_paths:
        tmpfiles = [file for file in os.listdir(path) if file.endswith('.o') or
                    file.endswith('.gch') or file.endswith('.exp') or
                    file.endswith('.obj') or file.endswith('.lib') or
                    file.endswith('.exe') or file.endswith('.manifest')]
        for file in tmpfiles:
            os.remove(os.path.join(path, file))
