// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  HeurekamodelLibrary.cpp
//
// 	Heureka Models - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "HeurekamodelLibrary.h"
#include "../../Finland/c/SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


int Basal_area_growth_stand_Elfving (double Age, double BA_propCo, double BA_propSp, double BA_propNs, double BA_propB, double BA, double N, 
		double VEGTYPE, double PEAT, double SOILMOIST, double SI, double TS, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int MOIST, WET, DIKAT;
	double BJDKYL, VEG;
	MOIST=0;
	WET=0;
	BJDKYL=0;
	DIKAT=0;
	VEG=0;
	
	if (SOILMOIST==4)
	{
		MOIST=1;
	}
	else if (SOILMOIST==5)
	{
		WET=1;
	}
	
	BJDKYL=BA_propB*exp(-0.01*(TS-300));
	
	if (VEGTYPE==1)
	{
		VEG=4;
	}
	else if (VEGTYPE==2||VEGTYPE==5||VEGTYPE==8)
	{
		VEG=2.5;
	}
	else if (VEGTYPE==3||VEGTYPE==6)
	{
		VEG=2;
	}
	else if (VEGTYPE==4||VEGTYPE==7)
	{
		VEG=3;
	}
	else if (VEGTYPE==9)
	{
		VEG=1.5;
	}
	else if (VEGTYPE==10||VEGTYPE==11||VEGTYPE==15)
	{
		VEG=-3;
	}
	else if (VEGTYPE==12)
	{
		VEG=1;
	}
	else if (VEGTYPE==14||VEGTYPE==17)
	{
		VEG=-0.5;
	}
	else if (VEGTYPE==16)
	{
		VEG=-5;
	}
	else if (VEGTYPE==18)
	{
		VEG=-1;
	}
	
	double THIN_0_10 = 0;
	double THIN_11_25 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;
	if(SINCE_THINNING >= 11 &&SINCE_THINNING <= 30)
		THIN_11_25 = 1;
	

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(0.366-0.5842*log(Age)+8.374*(BA_propCo/Age)-0.0237*(BA_propSp*VEG)-0.3192*pow(BA_propB,2)-
			10.8034*BJDKYL+0.5002*log(BA)-0.00632*BA+1.376*N/(N+80)+0.0627*VEG-0.0244*PEAT-0.0498*MOIST
			-0.1807*WET+0.0109*SI+0.0542*DIKAT+0.1396*THIN_0_10+0.0567*THIN_11_25-0.06*BA_propSp-0.03*BA_propNs);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_pine_Elfving (double d, double BA_L, double D_gM, double A, double OST, double BA,  double BA_propother, 
		double GOTLAND, double TS, double SI, double VEGTYPE, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int RICH, FERTRIS, KANTEFF, LNGREL, DELAD;
	KANTEFF=0;
	LNGREL=0;
	FERTRIS=0;
	RICH=0;
	DELAD=0;
	
	if (VEGTYPE <=8)
	{
		RICH=1;
	}
	if (VEGTYPE <12)
	{
		FERTRIS=1;
	}
	
	double THIN_0_10 = 0;
	double THIN_11_25 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;
	if(SINCE_THINNING >= 11 &&SINCE_THINNING <= 30)
		THIN_11_25 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(3.4176+1.0149*log(d+1)-0.3902*BA_L/(d+1)-0.7731*log(A+20)+0.2218*OST+0.1843*D_gM/10-0.3145*log(BA+3)+
		0.1391*pow(BA_propother,2)-0.0844*GOTLAND+0.1178*pow((0.001*TS),2)+1.089*SI/10-0.2164*pow(SI,2)/100+0.1011*RICH+
		0.279*FERTRIS+0.1245*THIN_0_10+0.0451*THIN_11_25+0.0487*DELAD+0.1368*KANTEFF+0.0842*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}
	
int Basal_area_growth_spruce_Elfving (double d, double BA_L, double D_gM, double SUM_d, double N, double A, double OST, 
		double BA, double BA_propother, double GOTLAND, double TS, double SI, double VEGTYPE, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int RICH, FERTRIS, KANTEFF, LNGREL, DELAD;
	KANTEFF=0;
	LNGREL=0;
	FERTRIS=0;
	RICH=0;
	DELAD=0;
	
	if (VEGTYPE <=8)
	{
		RICH=1;
	}
	if (VEGTYPE <12)
	{
		FERTRIS=1;
	}
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;
	
	double SUM_d2=pow(SUM_d,2);

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (D_gM<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(3.436+1.5163*log(d+1)-0.152*d/10-0.4024*BA_L/(d+1)+0.1625*BA_L/(d+1)*BA_propother+0.4702*BA_L/(d+1)*pow(((D_gM-pow((SUM_d2/N),0.5))/D_gM),3)-
	0.7789*log(A+20)+0.4034*OST+0.1914*pow(D_gM,2)/1000-0.2342*log(BA+3)+0.1754*pow(BA_propother,2)-0.3264*GOTLAND-0.6923*(0.001*TS)+0.2568*pow((0.001*TS),2)+
	0.2903*SI/10+0.1965*RICH+0.4034*FERTRIS+0.1309*THIN_0_10+0.0561*DELAD+0.1126*KANTEFF+0.077*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_birch_Elfving (double d, double BA_L, double A, double OST, double BA, double BA_propother, double TS, 
		double DISTCOAST, double VEGTYPE, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int RICH, FERTRIS, KANTEFF, LNGREL;
	KANTEFF=0;
	LNGREL=0;
	FERTRIS=0;
	RICH=0;
	
	if (VEGTYPE <=8)
	{
		RICH=1;
	}
	if (VEGTYPE <12)
	{
		FERTRIS=1;
	}
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (DISTCOAST<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "DISTCOAST<=0(DISTCOAST=",DISTCOAST);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(5.9648+1.2217*log(d+1)-0.3998*BA_L/(d+1)-0.9226*log(A+20)+0.4772*OST-0.209*log(BA+3)-0.5821*pow(BA_propother,0.5)-0.5386*(0.001*TS)-
		0.4505*(1/(0.001*TS-0.3)+0.8801*(1/(DISTCOAST+3))+0.3439*RICH+0.3844*FERTRIS+0.1814*THIN_0_10+0.2258*KANTEFF+0.1321*LNGREL));
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_aspen_Elfving (double d, double BA_L, double A, double BA, double BA_propother, double TS, double VEGTYPE,
		double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int RICH, LNGREL;
	LNGREL=0;
	RICH=0;
	
	if (VEGTYPE <=8)
	{
		RICH=1;
	}
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(0.9945+1.9071*log(d+1)-0.3313*d/10-0.304*BA_L/(d+1)-0.4058*log(A+20)-0.1981*log(BA+3)-0.5967*pow(BA_propother,0.5)+
		0.4408*(0.001*TS)+0.4759*RICH+0.2143*THIN_0_10+0.2427*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_beech_Elfving (double d, double BA_L, double A, double BA, double BA_propother, double LAT, double SI,
		double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LNGREL, DELAD;
	LNGREL=0;
	DELAD=0;
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;
	
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(1.7005+2.5823*log(d+1)-0.3758*d/10-0.2079*BA_L/(d+1)-0.4478*log(A+20)-0.5348*log(BA+3)-0.9304*pow(BA_propother,0.5)-
		0.1906*(LAT-50)+0.3055*SI/10+0.22*THIN_0_10+0.2009*DELAD+0.2669*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_oak_Elfving (double d, double BA_L, double BA, double BA_propother, double GOTLAND, double ALT, double VEGTYPE,
		double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int RICH, KANTEFF, LNGREL;
	KANTEFF=0;
	LNGREL=0;
	RICH=0;
	
	if (VEGTYPE <=8)
	{
		RICH=1;
	}
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(1.9047+1.3115*log(d+1)-0.264*BA_L/(d+1)-0.5056*log(BA+3)-0.6001*pow(BA_propother,0.5)-0.4615*GOTLAND+0.3833*ALT/100-
		0.1938*pow((ALT/100),2)+0.2635*RICH+0.1034*THIN_0_10+0.3551*KANTEFF+0.1897*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_noble_Elfving (double d, double BA_L, double A, double BA, double GOTLAND, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LNGREL, ORT;
	LNGREL=0;
	ORT=0;
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(2.3316+0.825*log(d+1)-0.2877*BA_L/(d+1)-0.401*log(BA+3)-0.3809*GOTLAND+0.9397*ORT+0.241*THIN_0_10+0.4676*LNGREL);
		
	*modelresult = res * rectFactor;
	return ret;
}

int Basal_area_growth_trivial_Elfving (double d, double BA_L, double A, double BA, double SI, double SINCE_THINNING,
		int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int ORT;
	ORT=0;
	
	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 10)
		THIN_0_10 = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	
	double res = exp(2.1108+0.9418*log(d+1)-0.2599*BA_L/(d+1)-0.3026*log(A+20)-0.228*log(BA+3)+0.2595*SI/10+0.4392*ORT+0.1561*THIN_0_10);
		
	*modelresult = res * rectFactor;
	return ret;

}

int BA_growth_Elfving (double d, double BA_L, double D_gM, double SUM_d, double N, double A, double OST, 
		double BA, double BA_propother, double GOTLAND, double TS, double SI, double VEGTYPE, double DISTCOAST, 
		double LAT, double ALT,	double SP, double SINCE_THINNING, int *nres, double *modelresult, char *errors,
		int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    
	//Calculate tree basal area growth
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_pine_Elfving(d, BA_L, D_gM, A, OST, BA,  BA_propother, GOTLAND, TS, SI, VEGTYPE, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_spruce_Elfving (d, BA_L, D_gM, SUM_d, N, A, OST, BA, BA_propother, GOTLAND, TS, SI, VEGTYPE, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches
	else if (SP==3)
	{
		ret = Basal_area_growth_birch_Elfving(d, BA_L, A, OST, BA, BA_propother, TS, DISTCOAST, VEGTYPE, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Aspen
	else if (SP==4)
	{
		ret = Basal_area_growth_aspen_Elfving(d, BA_L, A, BA, BA_propother, TS, VEGTYPE, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}//Oak
	else if (SP==5)
	{
		ret = Basal_area_growth_oak_Elfving(d, BA_L, BA, BA_propother, GOTLAND, ALT, VEGTYPE, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}//Beech
	else if (SP==6)
	{
		ret = Basal_area_growth_beech_Elfving(d, BA_L, A, BA, BA_propother, LAT, SI, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}//Other noble deciduos
	else if (SP==7)
	{
		ret = Basal_area_growth_noble_Elfving(d, BA_L, A, BA, GOTLAND, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}//Other trivial deciduos
	else
	{
		ret = Basal_area_growth_trivial_Elfving(d, BA_L, A, BA, SI, SINCE_THINNING,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}

	if (ret == 0)
		return ret;

	*nres = 1;	
	return ret;

}

int Height_growth_Elfving (double h, double age, double a, double d13, double BA_L, double SP, double BA_propNs,
			double time_step, int *nres, struct Tree_h *height, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double d, r, beta, b2, h_new, i_h, i_hrel;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d13<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d13<=0(d13=",d13);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	if (SP==1||SP==8)
	{
		beta = 7395.6;
		b2 = -1.78;
		d = beta*pow(25,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		i_hrel = 4.05-0.471*(BA_L/(d13+1))-15.35/((BA_L/(d13+1))+5);
		h_new = (h+d+r)/(2+(4*beta*pow((age+time_step),b2))/(h-d+r));
		i_h = (h_new-h)*i_hrel/time_step;
		h_new = h+i_h*time_step;
		
	}
	else if (SP==2)
	{
		beta = 1495.3;
		b2 = -1.6;
		d = beta*pow(10,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow((age-3),b2)),0.5);
		i_hrel = 1/(1+exp(10.81-112.8/((0.6+0.4*BA_propNs)*BA_L/(d13+1)+7.5)));
		h_new = (h+d+r)/(2+(4*beta*pow(((age-3)+time_step),b2))/(h-d+r));
		i_h = (h_new-h)*i_hrel/time_step;
		h_new = h+i_h*time_step;
	}
	else if (SP==4)
	{
		//aspen
		beta = 693.2;
		b2 = -0.9771;
		d = beta*pow(7,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((age+time_step),b2))/(h-d+r));
		i_h = (h_new-h)/time_step;
	}
	else if (SP==5)
	{
		//oak
		beta = 8841.4;
		b2 = -1.4317;
		d = beta*pow(1000,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((age+time_step),b2))/(h-d+r));
		i_h = (h_new-h)/time_step;
	}
	else if (SP==6)
	{
		//beech
		beta = 4239.3;
		b2 = -1.7753;
		d = beta*pow(15,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((age+time_step),b2))/(h-d+r));
		i_h = (h_new-h)/time_step;
	}
	else
	//SP==3,7,9
	{
		beta = 394;
		b2 = -1.387;
		d = beta*pow(7,b2);
		r = pow((pow(((h-1.3)-d),2)+4*beta*(h-1.3)*pow(age,b2)),0.5);
		h_new = 1.3+((h-1.3)+d+r)/(2+(4*beta*pow((age+time_step),b2))/((h-1.3)-d+r));
		i_h = (h_new-h)/time_step;
	}
	
	/*larch, how to define larch species? LARCH-dummy=1 and SP=1?
	{
		beta = 1529;
		b2 = -1.3451;
		d = beta*pow(17.97,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((age+time_step),b2))/(h-d+r));
		i_h = (h_new-h)/time_step;
	}*/
	
	*nres = 1;
	
	height[0].i_h=i_h;
	height[0].h=h_new;

	return ret;

}

int Height_Soderberg (double d, double a, double SI, double BA, double MAX_d, double LAT, double ALT, 
			double BA_propSp, double BA_propNs, double BA_propB, double BA_propBe, double DISTCOAST, 
			double REGION, double SP, double Calibration_height, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double DIV, MAR, COAST, KONT, DELYTA, SOST, REGION5, SI_spruce, SI_pine;

	DIV=0;
	MAR=0;
	COAST=0;
	KONT=0;
	DELYTA=0;
	SOST=0;		//Can be defined based on county if county is known
	REGION5=0;	//Can be defined based on county if county is known
	
	if(DISTCOAST<50)
		COAST=1;
	
	SI_spruce=SI;
	SI_pine=SI;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (MAX_d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "MAX_d<=0(MAX_d=",MAX_d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res;
	
	if (SP==1||SP==8)
	{
		if (REGION==1)
		{
			//SS_pine
			res = (exp(14.687-328.24/((d*10)+50)+10788/pow(((d*10)+50),2)-10.143/(a+10)+114.51/pow((a+10),2)-0.0030959*a-0.04905*(d*10)/a+
				0.0013824*(SI_spruce*10)+0.0017327*(SI_pine*10)+0.0095731*BA-0.000095444*pow(BA,2)+0.2106*(d*10)/(MAX_d*10)-
				0.21919*pow(((d*10)/(MAX_d*10)),2)-0.3043*LAT+0.0026409*pow(LAT,2)-0.000059174*ALT+0.14785*BA_propSp+0.24375*BA_propNs+
				0.12387*BA_propB-0.02554*MAR-0.060927*DELYTA+0.0151*SOST-0.042302*REGION5+0.00819))/10;
		}
		else if (REGION==2)
		{
			//MS_pine
			res = (exp(-81.303-285.36/((d*10)+50)+7523.3/pow(((d*10)+50),2)-10.295/(a+10)+119.34/pow((a+10),2)-0.0018296*a-0.040078*(d*10)/a+
				0.0011538*(SI_spruce*10)+0.0013061*(SI_pine*10)+0.011572*BA-0.00011765*pow(BA,2)-0.24016*(d*10)/(MAX_d*10)+
				0.063405*pow(((d*10)/(MAX_d*10)),2)+2.8478*LAT-0.023177*pow(LAT,2)-0.0000004414*pow(ALT,2)+0.029354*BA_propNs-0.03701*COAST-
				0.053233*DIV-0.050755*MAR+0.00673))/10;
		}
		else 
		//REGION==3
		{
			//NS_pine
			res = (exp(-8.2981-279.95/((d*10)+50)+6781.6/pow(((d*10)+50),2)-8.0059/(a+10)+100.56/pow((a+10),2)-0.001899*a-0.066679*(d*10)/a+
				0.0014751*(SI_spruce*10)+0.0014142*(SI_pine*10)+0.0011473*BA-0.00012937*pow(BA,2)-0.15662*(d*10)/(MAX_d*10)+
				0.02292*pow(((d*10)/(MAX_d*10)),2)+0.45084*LAT-0.0035449*pow(LAT,2)-0.0000002026*pow(ALT,2)+0.12172*BA_propSp+0.0639*BA_propNs-
				0.026745*COAST-0.047547*DIV-0.016487*MAR+0.00819))/10;
		}
	}
	else if (SP==2)
	{
		if (REGION==1)
		{
			//SS_spruce
			res = (exp(6.0404-330.48/((d*10)+50)+7301.5/pow(((d*10)+50),2)-2.4683/(a+10)+96.352/pow((a+10),2)-0.0020097*a-0.049498*(d*10)/a+
				0.00092562*(SI_spruce*10)+0.0010289*(SI_pine*10)+0.0077592*BA-0.000068334*pow(BA,2)+0.38474*(d*10)/(MAX_d*10)-
				0.3426*pow(((d*10)/(MAX_d*10)),2)+0.00033536*ALT-0.0000010375*pow(ALT,2)+0.024412*BA_propSp+0.13087*BA_propNs-0.043084*DELYTA+
				0.020959*KONT-0.016755*MAR+0.018975*SOST+0.00769))/10;
		}
		else
		//REGION==2or3
		{
			//MandNS_spruce
			res = (exp(6.1585-289.27/((d*10)+50)+5276.2/pow(((d*10)+50),2)-4.4821/(a+10)+136/pow((a+10),2)-0.00108*a-0.038071*(d*10)/a+
				0.0007429*(SI_spruce*10)+0.00058052*(SI_pine*10)+0.011361*BA-0.00010922*pow(BA,2)+0.28029*(d*10)/(MAX_d*10)-
				0.2406*pow(((d*10)/(MAX_d*10)),2)-0.000071695*pow(LAT,2)+0.00011744*ALT-0.00000056669*pow(ALT,2)+0.095561*BA_propSp+
				0.087584*BA_propNs-0.041899*COAST+0.013342*KONT-0.03918*DELYTA+0.00832))/10;
		}
	}
	else if (SP==3)
	{
		if (REGION==1)
		{
			//SS_birch
			res = (exp(4.6194-216.24/((d*10)+50)+4768.3/pow(((d*10)+50),2)-14.308/pow((a+10),2)-0.010377*(d*10)/a+0.00057275*(SI_spruce*10)+
				0.00070769*(SI_pine*10)+0.012258*BA-0.00014025*pow(BA,2)+0.15315*(d*10)/(MAX_d*10)-0.2544*pow(((d*10)/(MAX_d*10)),2)+
				0.016736*LAT-0.0000035533*LAT*ALT-0.046067*BA_propSp+0.077659*BA_propNs+0.081706*BA_propB-0.069576*DELYTA-
				0.059341*MAR-0.041456*REGION5+0.01638))/10;
		}
		else
		//REGION==2or3
		{
			//Mand_NS_birch
			res = (exp(6.4427-229.19/((d*10)+50)+4898.6/pow(((d*10)+50),2)+5.2925/(a+10)-0.039008*(d*10)/a+0.00096299*(SI_spruce*10)+
				0.00083824*(SI_pine*10)+0.016511*BA-0.00017611*pow(BA,2)-0.087603*(d*10)/(MAX_d*10)+0.025864*pow(((d*10)/(MAX_d*10)),2)-
				0.013252*LAT-0.00010123*ALT-0.0000004613*pow(ALT,2)-0.040542*BA_propSp-0.056039*BA_propNs-0.049758*DELYTA+0.01008))/10;
		}
	}
	else if (SP==5)
	{
		//oak
		res = (exp(5.6224-247.48/((d*10)+50)+6119.9/pow(((d*10)+50),2)-0.0016569*a-0.016744/(a+10)+0.00099719*(SI_spruce*10)+
			0.0011754*(SI_pine*10)+0.01096*BA-0.00010344*pow(BA,2)-0.11226*pow(((d*10)/(MAX_d*10)),2)-0.00023866*ALT+0.16577*BA_propBe-
			0.10511*DELYTA-0.054901*REGION5+0.01462))/10;
	}
	else if (SP==6)
	{
		//beech
		res = (exp(5.6648-128.84/((d*10)+50)-21.199/(a+10)+210.53/pow((a+10),2)+0.00015372*(SI_spruce*10)+0.009706*BA-
			0.00009029*pow(BA,2)-0.39544*(d*10)/(MAX_d*10)+0.29474*pow(((d*10)/(MAX_d*10)),2)-0.000001728*pow(ALT,2)+0.115103*BA_propBe-
			0.13339*DELYTA+0.097831*REGION5+0.0114))/10;
	}
	else
	{
		if (REGION==1)
		{
			//SS_other_dedic
			res = (exp(3.9136-217.37/((d*10)+50)+6636/pow(((d*10)+50),2)-39.007/pow((a+10),2)+0.00080244*(SI_spruce*10)+0.00076595*(SI_pine*10)+
				0.010224*BA-0.000094866*pow(BA,2)+0.26833*(d*10)/(MAX_d*10)-0.30654*pow(((d*10)/(MAX_d*10)),2)+0.024865*LAT+0.093169*BA_propNs-
				0.029612*MAR-0.067029*DELYTA+0.01462))/10;
		}
		else
		//REGION==2or3
		{
			//MandNS_other_decid
			res = (exp(6.3675-109.49/((d*10)+50)-20.419/(a+10)+229.66/pow((a+10),2)-0.0021025*a+0.0011182*(SI_spruce*10)+
				0.0012554*(SI_pine*10)+0.017302*BA-0.00016525*pow(BA,2)+0.64313*(d*10)/(MAX_d*10)-0.34368*pow(((d*10)/(MAX_d*10)),2)-
				0.023565*LAT+0.10443*BA_propSp+0.11788*BA_propNs+0.02636*MAR-0.053057*DELYTA+0.13323*COAST+0.01462))/10;
		}
	}

	//Calibrate height. Calibration height is 1, if calibration is not used
	*modelresult = res * Calibration_height;
	return ret;

}

int Age_at_breast_height_pine_Elfving (double d, double D_gM, double Age, double BA,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = exp(-1.6957+0.2982*log(d)-0.00801*d+0.0726*d/D_gM+1.1587*log(Age)-0.0007699*Age+0.031*log(BA));


	*modelresult = res * rectFactor;
	return ret;

}

int Age_at_breast_height_spruce_Elfving (double d, double D_gM, double Age, double BA,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = exp(-1.3834+0.379*log(d)-0.01003*d+0.1283*d/D_gM+1.0201*log(Age)+0.0175*log(BA));


	*modelresult = res * rectFactor;
	return ret;

}

int Age_at_breast_height_birch_Elfving (double d, double D_gM, double Age, double BA, double SI, double VEGTYPE,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double RICH;

	RICH=0;

	if (VEGTYPE <=8)
	{
		RICH=1;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = exp(0.006+0.5207*log(d)-0.00793*d-0.0814*d/D_gM+0.6701*log(Age)-0.01053*SI-0.099*RICH+0.0394*log(BA+1));


	*modelresult = res * rectFactor;
	return ret;

}

int Age_at_breast_height_other_deciduous_Elfving (double d, double D_gM, double Age,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double SLOV, LJUSLOV;

	SLOV=0;
	LJUSLOV=0;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = exp(0.138+0.6469*log(d)-0.265*d/D_gM+0.5053*log(Age)+0.1939*SLOV-0.1787*LJUSLOV);


	*modelresult = res * rectFactor;
	return ret;

}

int Age_Elfving (double d, double D_gM, double Age, double BA, double SI, double VEGTYPE, double SP,
			double t_href, double Calibration_age, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Calculate age at breast height
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Age_at_breast_height_pine_Elfving (d, D_gM, Age, BA,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Age_at_breast_height_spruce_Elfving (d, D_gM, Age, BA,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches
	else if (SP==3)
	{
		ret = Age_at_breast_height_birch_Elfving (d, D_gM, Age, BA, SI, VEGTYPE,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Other deciduous
	else
	{
		ret = Age_at_breast_height_other_deciduous_Elfving (d, D_gM, Age,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}

	if (ret == 0)
		return ret;
		
	//Calibrate age. Calibration age is 1, if calibration is not used
	//Calculate biological age
	double age = *modelresult + t_href;
	age = age*Calibration_age;

	*nres = 1;
	*modelresult = age;
	return ret;

}

int Temperature_sum_Elfving (double LAT, double ALT,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	*nres = 1;
	double res = 4835-57.6*LAT-0.9*ALT;


	*modelresult = res * rectFactor;
	return ret;

}

int Time_to_reach_breast_height_Elfving (double LAT, double SI, double SP,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double Ns;
	Ns = 0;
	
	if (SP==2)
	{
		Ns = 1;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (SI<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI<=0(SI=",SI);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res;
	
	if (SP <= 2 || SP == 8)
	{ 
		res = 37-0.605*LAT-1121/SI+21.92*LAT/SI+29.5*Ns/SI;
	}
	else
	{
		res = 150/SI;
	}


	*modelresult = res * rectFactor;
	return ret;

}

int Mean_height_Hugin (double Age, double SI, double SP, double H_gM, double time_step, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	*nres = 1;
	
	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	double h_est, hnew_est, res;
	
	if (SP==1)
	{
		//pine
		h_est = SI/(exp(7+(-0.57-0.05*SI)*log(Age)+(-0.28+0.0094*SI)*pow(log(Age),2))+1);
		hnew_est = SI/(exp(7+(-0.57-0.05*SI)*log(Age+time_step)+(-0.28+0.0094*SI)*pow(log(Age+time_step),2))+1);
	}
	else if (SP==2)
	{
		//spruce
		h_est = SI/(exp((6.27+12.1/SI)+(-0.262-0.0575*SI+0.00088*pow(SI,2))*log(Age)+
			(-0.323-0.134*(-0.262-0.0575*SI+0.00088*pow(SI,2)))*pow(log(Age),2))+1);
		hnew_est = SI/(exp((6.27+12.1/SI)+(-0.262-0.0575*SI+0.00088*pow(SI,2))*log(Age+time_step)+
			(-0.323-0.134*(-0.262-0.0575*SI+0.00088*pow(SI,2)))*pow(log(Age+time_step),2))+1);
	}
	else if (SP==3)
	{
		//pend birch
		h_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age))+1);
		hnew_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age+time_step))+1);
	}
	else if (SP==4)
	{
		//aspen
		h_est = SI/(exp((10.024-0.1664*SI)+(-4.093+0.1605*SI-0.0025*pow(SI,2))*log(Age))+1);
		hnew_est = SI/(exp((10.024-0.1664*SI)+(-4.093+0.1605*SI-0.0025*pow(SI,2))*log(Age+time_step))+1);
	}
	else if (SP==5)
	{
		//oak
		h_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age))+1);
		hnew_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age+time_step))+1);
	}
	else if (SP==6)
	{
		//beech
		h_est = SI/(exp((6.27+12.1/SI)+(-0.262-0.0575*SI+0.00088*pow(SI,2))*log(Age)+
			(-0.323-0.134*(-0.262-0.0575*SI+0.00088*pow(SI,2)))*pow(log(Age),2))+1);
		hnew_est = SI/(exp((6.27+12.1/SI)+(-0.262-0.0575*SI+0.00088*pow(SI,2))*log(Age+time_step)+
			(-0.323-0.134*(-0.262-0.0575*SI+0.00088*pow(SI,2)))*pow(log(Age+time_step),2))+1);
	}
	else if (SP==8)
	{
		//contorta
		h_est = SI/(exp(7+(-0.57-0.05*SI)*log(Age)+(-0.28+0.0094*SI)*pow(log(Age),2))+1);
		hnew_est = SI/(exp(7+(-0.57-0.05*SI)*log(Age+time_step)+(-0.28+0.0094*SI)*pow(log(Age+time_step),2))+1);
	}
	else
	{
		//pub birch
		h_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age))+1);
		hnew_est = SI/(exp((6.836+0.03165*SI-0.002757*pow(SI,2))+(-2.694+0.4937*(6.836+0.03165*SI-0.002757*pow(SI,2))-
			0.05331*pow((6.836+0.03165*SI-0.002757*pow(SI,2)),2))*log(Age+time_step))+1);
	}
	
	res = H_gM + (hnew_est-h_est);

	*modelresult = res * rectFactor;
	return ret;

}

int Age_at_breast_height_Hugin (double h, double H_ari, double SI, double SP, 
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;
    double PEB,SPROUT;
    PEB=0;
    SPROUT=0;
    
    if (SP==3)
    {
    	PEB=1;
    }
    
    //Check for fatal errors
	char errorStr[200] = "";
	if (h*10-12<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h*10-12<=0(h*10-12=",h*10-12);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res;
	
	if (SP==1||SP==8)
	{
		//pine
		res = 2.548*log(h*10-12)+68.23*pow((h*10/(SI*10)),2)-0.003*h*10*SI*10/10+
			24.54*pow((H_ari*10/(SI*10)),2)+0.572*(h*10-H_ari*10)/(H_ari*10);
	}
	else if (SP==2)
	{
		//spruce
		res = 2.643*log(h*10-12)+78.64*pow((h*10/(SI*10)),2)-0.002*h*10*SI*10/10+
			28.86*pow((H_ari*10/(SI*10)),2)+0.51*(h*10-H_ari*10)/(H_ari*10);
	}
	else
	{
		//birch
		res = 1.988*log(h*10-12)+61.89*pow((h*10/(SI*10)),2)-1.49*(H_ari*10/(SI*10))-
			15.72*PEB*pow((h*10/(SI*10)),2)+0.014*SPROUT*(h*10-50);
	}

	*modelresult = res * rectFactor;
	return ret;

}

int Diameter_at_breast_height_Hugin (double h, double MAX_h, double SUM_h2, double SUMh2_propdecid, 
			double CULT, double DISTCOAST, double LAT, double ALT, double SI, 
			double VEGTYPE, double SP, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
    	double HD, CF, NAT, ROJ, AR, AK, GRASS, SHRUB, STSK, LDEL;
    	NAT=0;
    	ROJ=1;
    	AR=3;
    	AK=0;
    	GRASS=0;
    	SHRUB=0;
    	STSK=0;
    	LDEL=0;
    	
    	HD=MAX_h-h;
    	if (HD<0.1)
    	{
    		HD=0.1;
    	}
    	CF=1+10*HD*(0.00001*SUM_h2+0.1);
    	LDEL=SUMh2_propdecid*1.5708;
    	if (CULT==0)
    	{
    		NAT=1;
    	}
    	if (DISTCOAST<5)
    	{
    		AK=1;
    	}
    	if (VEGTYPE<=9)
	{
		GRASS=1;
	}
	if (VEGTYPE <12)
	{
		SHRUB=1;
	}
	
	//Check for fatal errors
	char errorStr[200] = "";
	if (h-1.1<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h-1.1<=0(h-1.1=",h-1.1);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res;
	
	if (SP==1||SP==8)
	{
		//pine
		res = sqrt(exp(2.098*log(h-1)-0.1473*0.00001*SUM_h2-0.176*log(0.00001*SUM_h2+0.1)-0.098*log(CF)+0.136*sin(LDEL)-
			0.312*NAT+0.0759*NAT*pow(h,0.5)-0.171*ROJ/(1+AR)+0.022*ROJ*log(10+AR)+0.014*LAT-0.0466*(ALT/100)+
			0.0126*pow((ALT/100),2)+0.096*AK+0.088*GRASS+0.261));
	}
	else if (SP==2)
	{
		//spruce
		res = sqrt(exp(1.868*log(h-1)+0.0147*pow((h-1),2)-0.0525*0.00001*SUM_h2-0.109*log(0.00001*SUM_h2+0.1)-0.058*log(CF)+
			0.081*sin(LDEL)-0.052*NAT-0.093*ROJ/(1+AR)+0.013*ROJ*log(10+AR)+0.014*LAT+0.068*(ALT/100)+
			0.0045*pow((ALT/100),2)+0.103*AK-0.053*SHRUB+0.186));
	}
	else
	{
		//birch
		res = sqrt(exp(2.25*log(h-1.1)-0.0166*0.00001*SUM_h2-0.119*log(CF)-0.071*ROJ/(1+AR)+0.033*ROJ*log(10+AR)-
			0.00029*STSK*pow(h,2)+0.12*AK-0.0096*SI+0.867));
	}

	*modelresult = res * rectFactor;
	return ret;

}

int Site_index_Elfving (double h, double age, double a, double d13, double BA_L, double SP, 
			double time_step, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	double d, r, beta, b2, h_new, i_h, i_hrel;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d13<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d13<=0(d13=",d13);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	if (SP==1||SP==8)
	{
		beta = 7395.6;
		b2 = -1.78;
		d = beta*pow(25,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		i_hrel = 4.05-0.471*(BA_L/(d13+1))-15.35/((BA_L/(d13+1))+5);
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
		i_h = (h_new-h)*i_hrel;
		h_new = h+i_h;
		
	}
	else if (SP==2)
	{
		beta = 1495.3;
		b2 = -1.6;
		d = beta*pow(10,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow((age-3),b2)),0.5);
		i_hrel = 1/(1+exp(10.81-112.8/(BA_L/(d13+1)+7.5)));
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
		i_h = (h_new-h)*i_hrel;
		h_new = h+i_h;
	}
	else if (SP==4)
	{
		//aspen
		beta = 693.2;
		b2 = -0.9771;
		d = beta*pow(7,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
	}
	else if (SP==5)
	{
		//oak
		beta = 8841.4;
		b2 = -1.4317;
		d = beta*pow(1000,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
	}
	else if (SP==6)
	{
		//beech
		beta = 4239.3;
		b2 = -1.7753;
		d = beta*pow(15,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
	}
	else
	//SP==3,7,9
	{
		beta = 394;
		b2 = -1.387;
		d = beta*pow(7,b2);
		r = pow((pow(((h-1.3)-d),2)+4*beta*(h-1.3)*pow(age,b2)),0.5);
		h_new = 1.3+((h-1.3)+d+r)/(2+(4*beta*pow((100),b2))/((h-1.3)-d+r));
	}
	
	/*larch, how to define larch species? LARCH-dummy=1 and SP=1?
	{
		beta = 1529;
		b2 = -1.3451;
		d = beta*pow(17.97,b2);
		r = pow((pow((h-d),2)+4*beta*h*pow(age,b2)),0.5);
		h_new = (h+d+r)/(2+(4*beta*pow((100),b2))/(h-d+r));
	}*/
	
	*nres = 1;
	
	*modelresult = h_new * rectFactor;
	return ret;
}

int Tree_volume_heureka_JKK (double d, double h, double SP, double REGION, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	*nres = 1;
	
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	double res;
	
	if (SP==1)
	{
		//pine
		if (REGION==1||REGION==2)
		{
			// Southern Sweden
			res = pow(10,(-1.38903+1.84493*log10(d)+0.06563*log10(d+20)+2.02122*log10(h)-1.01095*log10(h-1.3)));
		}
		else
		{
			// Northern Sweden
			res = pow(10,(-1.20914+1.9474*log10(d)-0.05947*log10(d+20)+1.40958*log10(h)-0.4581*log10(h-1.3)));
		}
	}
	else if (SP==2)
	{
		//spruce
		if (REGION==1||REGION==2)
		{
			// Southern Sweden
			res = pow(10,(-1.02039+2.00128*log10(d)-0.47473*log10(d+20)+2.87138*log10(h)-1.61803*log10(h-1.3)));
		}
		else
		{
			// Northern Sweden
			res = pow(10,(-0.79783+2.07157*log10(d)-0.73882*log10(d+20)+3.16332*log10(h)-1.82622*log10(h-1.3)));
		}
	}
	else if (SP==4)
	{
		//aspen
		res = 0.01548*pow(d,2)+0.03255*pow(d,2)*h-0.000047*pow(d,2)*pow(h,2)-0.01333*d*h+0.004859*d*pow(h,2);
	}
	else if (SP==5||SP==7)
	{
		//oak
		if (h>=10)
		{
			res = 0.03522*pow(d,2)*h+0.08772*d*h-0.04905*pow(d,2);
		}
		else
		{
			res = 0.03522*pow(d,2)*h+0.08772*d*h-0.04905*pow(d,2)+pow((1-h/10),2)*(0.01682*pow(d,2)*h+
			0.01108*d*h-0.02167*d*pow(h,2)+0.04905*d);
		}
	}
	else if (SP==6)
	{
		//beech
		res = 0.01275*pow(d,2)*h+0.12368*pow(d,2)+0.0004701*pow(d,2)*pow(h,2)+0.00622*d*pow(h,2);
	}
	else if (SP==8)
	{
		//contorta
		res = 0.98*(0.1121*pow(d,2)+0.0287*pow(d,2)*h-0.000061*pow(d,2)*pow(h,2)-0.09176*d*h+0.01249*d*pow(h,2));
	}
	//else if (SP==larch)
	//{
	//	//larch
	//	res = 0.04801*pow(d,2)*h+0.08886*pow(d,2)-0.01012*pow(d,3)-0.08406*d*h+0.1972*h;
	//}
	else
	{
		//birch and other deciduous species
		if (REGION==1||REGION==2)
		{
			// Southern Sweden
			res = pow(10,(-0.89359+2.27954*log10(d)-1.18672*log10(d+20)+7.07362*log10(h)-5.45175*log10(h-1.3)));
		}
		else
		{
			// Northern Sweden
			res = pow(10,(-0.44224+2.4758*log10(d)-1.40854*log10(d+20)+5.16863*log10(h)-3.77147*log10(h-1.3)));
		}
	}
	//change the unit from dm3 to m3
	res=res/1000;

	*modelresult = res * rectFactor;
	return ret;

}

int Calibration_age_heureka_JKK (double D_gM, double Age, double BA, double SI, double VEGTYPE, double SP,
			double t_href, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;
    
    double d=D_gM;

	//Calculate age at breast height
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Age_at_breast_height_pine_Elfving (d, D_gM, Age, BA,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Age_at_breast_height_spruce_Elfving (d, D_gM, Age, BA,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches
	else if (SP==3)
	{
		ret = Age_at_breast_height_birch_Elfving (d, D_gM, Age, BA, SI, VEGTYPE,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Other deciduous
	else
	{
		ret = Age_at_breast_height_other_deciduous_Elfving (d, D_gM, Age,
				nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	
	if (ret == 0)
		return ret;
	
	*modelresult = Age/(*modelresult+t_href);
	return ret;

}

int Calibration_height_heureka_JKK (double H_gM, double D_gM, double A, double SI, double BA, double MAX_d, double LAT, double ALT, 
			double BA_propSp, double BA_propNs, double BA_propB, double BA_propBe, double DISTCOAST, 
			double REGION, double SP, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;
    
        double d=D_gM;
        double a=A;
	double DIV, MAR, COAST, KONT, DELYTA, SOST, REGION5, SI_spruce, SI_pine;

	DIV=0;
	MAR=0;
	COAST=0;
	KONT=0;
	DELYTA=0;
	SOST=0;		//Can be defined based on county if county is known
	REGION5=0;	//Can be defined based on county if county is known
	
	if(DISTCOAST<50)
		COAST=1;
	
	SI_spruce=SI;
	SI_pine=SI;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (MAX_d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "MAX_d<=0(MAX_d=",MAX_d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res;
	
	if (SP==1||SP==8)
	{
		if (REGION==1)
		{
			//SS_pine
			res = (exp(14.687-328.24/((d*10)+50)+10788/pow(((d*10)+50),2)-10.143/(a+10)+114.51/pow((a+10),2)-0.0030959*a-0.04905*(d*10)/a+
				0.0013824*(SI_spruce*10)+0.0017327*(SI_pine*10)+0.0095731*BA-0.000095444*pow(BA,2)+0.2106*(d*10)/(MAX_d*10)-
				0.21919*pow(((d*10)/(MAX_d*10)),2)-0.3043*LAT+0.0026409*pow(LAT,2)-0.000059174*ALT+0.14785*BA_propSp+0.24375*BA_propNs+
				0.12387*BA_propB-0.02554*MAR-0.060927*DELYTA+0.0151*SOST-0.042302*REGION5+0.00819))/10;
		}
		else if (REGION==2)
		{
			//MS_pine
			res = (exp(-81.303-285.36/((d*10)+50)+7523.3/pow(((d*10)+50),2)-10.295/(a+10)+119.34/pow((a+10),2)-0.0018296*a-0.040078*(d*10)/a+
				0.0011538*(SI_spruce*10)+0.0013061*(SI_pine*10)+0.011572*BA-0.00011765*pow(BA,2)-0.24016*(d*10)/(MAX_d*10)+
				0.063405*pow(((d*10)/(MAX_d*10)),2)+2.8478*LAT-0.023177*pow(LAT,2)-0.0000004414*pow(ALT,2)+0.029354*BA_propNs-0.03701*COAST-
				0.053233*DIV-0.050755*MAR+0.00673))/10;
		}
		else 
		//REGION==3
		{
			//NS_pine
			res = (exp(-8.2981-279.95/((d*10)+50)+6781.6/pow(((d*10)+50),2)-8.0059/(a+10)+100.56/pow((a+10),2)-0.001899*a-0.066679*(d*10)/a+
				0.0014751*(SI_spruce*10)+0.0014142*(SI_pine*10)+0.0011473*BA-0.00012937*pow(BA,2)-0.15662*(d*10)/(MAX_d*10)+
				0.02292*pow(((d*10)/(MAX_d*10)),2)+0.45084*LAT-0.0035449*pow(LAT,2)-0.0000002026*pow(ALT,2)+0.12172*BA_propSp+0.0639*BA_propNs-
				0.026745*COAST-0.047547*DIV-0.016487*MAR+0.00819))/10;
		}
	}
	else if (SP==2)
	{
		if (REGION==1)
		{
			//SS_spruce
			res = (exp(6.0404-330.48/((d*10)+50)+7301.5/pow(((d*10)+50),2)-2.4683/(a+10)+96.352/pow((a+10),2)-0.0020097*a-0.049498*(d*10)/a+
				0.00092562*(SI_spruce*10)+0.0010289*(SI_pine*10)+0.0077592*BA-0.000068334*pow(BA,2)+0.38474*(d*10)/(MAX_d*10)-
				0.3426*pow(((d*10)/(MAX_d*10)),2)+0.00033536*ALT-0.0000010375*pow(ALT,2)+0.024412*BA_propSp+0.13087*BA_propNs-0.043084*DELYTA+
				0.020959*KONT-0.016755*MAR+0.018975*SOST+0.00769))/10;
		}
		else
		//REGION==2or3
		{
			//MandNS_spruce
			res = (exp(6.1585-289.27/((d*10)+50)+5276.2/pow(((d*10)+50),2)-4.4821/(a+10)+136/pow((a+10),2)-0.00108*a-0.038071*(d*10)/a+
				0.0007429*(SI_spruce*10)+0.00058052*(SI_pine*10)+0.011361*BA-0.00010922*pow(BA,2)+0.28029*(d*10)/(MAX_d*10)-
				0.2406*pow(((d*10)/(MAX_d*10)),2)-0.000071695*pow(LAT,2)+0.00011744*ALT-0.00000056669*pow(ALT,2)+0.095561*BA_propSp+
				0.087584*BA_propNs-0.041899*COAST+0.013342*KONT-0.03918*DELYTA+0.00832))/10;
		}
	}
	else if (SP==3)
	{
		if (REGION==1)
		{
			//SS_birch
			res = (exp(4.6194-216.24/((d*10)+50)+4768.3/pow(((d*10)+50),2)-14.308/pow((a+10),2)-0.010377*(d*10)/a+0.00057275*(SI_spruce*10)+
				0.00070769*(SI_pine*10)+0.012258*BA-0.00014025*pow(BA,2)+0.15315*(d*10)/(MAX_d*10)-0.2544*pow(((d*10)/(MAX_d*10)),2)+
				0.016736*LAT-0.0000035533*LAT*ALT-0.046067*BA_propSp+0.077659*BA_propNs+0.081706*BA_propB-0.069576*DELYTA-
				0.059341*MAR-0.041456*REGION5+0.01638))/10;
		}
		else
		//REGION==2or3
		{
			//Mand_NS_birch
			res = (exp(6.4427-229.19/((d*10)+50)+4898.6/pow(((d*10)+50),2)+5.2925/(a+10)-0.039008*(d*10)/a+0.00096299*(SI_spruce*10)+
				0.00083824*(SI_pine*10)+0.016511*BA-0.00017611*pow(BA,2)-0.087603*(d*10)/(MAX_d*10)+0.025864*pow(((d*10)/(MAX_d*10)),2)-
				0.013252*LAT-0.00010123*ALT-0.0000004613*pow(ALT,2)-0.040542*BA_propSp-0.056039*BA_propNs-0.049758*DELYTA+0.01008))/10;
		}
	}
	else if (SP==5)
	{
		//oak
		res = (exp(5.6224-247.48/((d*10)+50)+6119.9/pow(((d*10)+50),2)-0.0016569*a-0.016744/(a+10)+0.00099719*(SI_spruce*10)+
			0.0011754*(SI_pine*10)+0.01096*BA-0.00010344*pow(BA,2)-0.11226*pow(((d*10)/(MAX_d*10)),2)-0.00023866*ALT+0.16577*BA_propBe-
			0.10511*DELYTA-0.054901*REGION5+0.01462))/10;
	}
	else if (SP==6)
	{
		//beech
		res = (exp(5.6648-128.84/((d*10)+50)-21.199/(a+10)+210.53/pow((a+10),2)+0.00015372*(SI_spruce*10)+0.009706*BA-
			0.00009029*pow(BA,2)-0.39544*(d*10)/(MAX_d*10)+0.29474*pow(((d*10)/(MAX_d*10)),2)-0.000001728*pow(ALT,2)+0.115103*BA_propBe-
			0.13339*DELYTA+0.097831*REGION5+0.0114))/10;
	}
	else
	{
		if (REGION==1)
		{
			//SS_other_dedic
			res = (exp(3.9136-217.37/((d*10)+50)+6636/pow(((d*10)+50),2)-39.007/pow((a+10),2)+0.00080244*(SI_spruce*10)+0.00076595*(SI_pine*10)+
				0.010224*BA-0.000094866*pow(BA,2)+0.26833*(d*10)/(MAX_d*10)-0.30654*pow(((d*10)/(MAX_d*10)),2)+0.024865*LAT+0.093169*BA_propNs-
				0.029612*MAR-0.067029*DELYTA+0.01462))/10;
		}
		else
		//REGION==2or3
		{
			//MandNS_other_decid
			res = (exp(6.3675-109.49/((d*10)+50)-20.419/(a+10)+229.66/pow((a+10),2)-0.0021025*a+0.0011182*(SI_spruce*10)+
				0.0012554*(SI_pine*10)+0.017302*BA-0.00016525*pow(BA,2)+0.64313*(d*10)/(MAX_d*10)-0.34368*pow(((d*10)/(MAX_d*10)),2)-
				0.023565*LAT+0.10443*BA_propSp+0.11788*BA_propNs+0.02636*MAR-0.053057*DELYTA+0.13323*COAST+0.01462))/10;
		}
	}

	*modelresult = H_gM / res;
	return ret;

}

int Basal_area_coefficient_JKK (double i_BA, double SUM_iba, int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;
    
        double calib_coeff;

	calib_coeff=1;

	*nres = 1;

	*modelresult = i_BA / (SUM_iba/10000);
	return ret;

}

static void Thinning_spruce_OMT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0139*pow(H_dom,3)-0.9048*pow(H_dom,2)+19.647*H_dom-105.31;
	double Thinning_limit_lower = 0.0058*pow(H_dom,3)-0.4325*pow(H_dom,2)+10.624*H_dom-52.865;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0005*pow(H_dom,3)-0.057*pow(H_dom,2)+3.2722*H_dom-12.826;
	limit[0].Remainlimit_lower = -0.0005*pow(H_dom,3)-0.057*pow(H_dom,2)+3.2722*H_dom-16.826;

}


static void Thinning_spruce_and_pine_MT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper =  0.0074*pow(H_dom,3)-0.4992*pow(H_dom,2)+11.164*H_dom-50.86;
	double Thinning_limit_lower = 0.0029*pow(H_dom,3)-0.2444*pow(H_dom,2)+6.4297*H_dom-25.979;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0044*pow(H_dom,3)-0.2904*pow(H_dom,2)+6.4108*H_dom-25.335;
	limit[0].Remainlimit_lower = 0.0058*pow(H_dom,3)-0.3639*pow(H_dom,2)+7.6808*H_dom-36.473;

}


static void Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0093*pow(H_dom,3)-0.5794*pow(H_dom,2)+12.241*H_dom-63.397;
	double Thinning_limit_lower = 0.0042*pow(H_dom,3)-0.275*pow(H_dom,2)+6.3333*H_dom-29.12;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0571*pow(H_dom,2)+2.6286*H_dom-13.711;
	limit[0].Remainlimit_lower = -0.0607*pow(H_dom,2)+2.7429*H_dom-16.577;

}


static void Thinning_pine_VT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0174*pow(H_dom,3)-1.0774*pow(H_dom,2)+22.121*H_dom-120.26;
	double Thinning_limit_lower = 0.001*pow(H_dom,3)-0.1607*pow(H_dom,2)+5.1887*H_dom-21.557;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0043*pow(H_dom,3)-0.2761*pow(H_dom,2)+5.9848*H_dom-23.461;
	limit[0].Remainlimit_lower = 0.0032*pow(H_dom,3)-0.2071*pow(H_dom,2)+4.5872*H_dom-18.416;

}


static void Thinning_pine_CT_southern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0104*pow(H_dom,3)-0.6839*pow(H_dom,2)+14.719*H_dom-78.134;
	double Thinning_limit_lower = -0.0125*pow(H_dom,3)+0.4643*pow(H_dom,2)-4.2071*H_dom+21.383;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0021*pow(H_dom,3)-0.1929*pow(H_dom,2)+5.0631*H_dom-22.789;
	limit[0].Remainlimit_lower = -0.001*pow(H_dom,3)-0.0375*pow(H_dom,2)+2.4792*H_dom-12.28;

}


static void Thinning_spruce_OMT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.003*pow(H_dom,3)-0.2481*pow(H_dom,2)+6.6604*H_dom-25.268;
	double Thinning_limit_lower = -0.0022*pow(H_dom,3)+0.0153*pow(H_dom,2)+2.2044*H_dom-3.4397;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0014*pow(H_dom,3)-0.0042*pow(H_dom,2)+2.2496*H_dom-6.9905;
	limit[0].Remainlimit_lower = -0.0028*pow(H_dom,3)+0.0676*pow(H_dom,2)+1.0403*H_dom-4.381;

}


static void Thinning_spruce_and_pine_MT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0071*pow(H_dom,3)-0.4507*pow(H_dom,2)+9.7255*H_dom-40.879;
	double Thinning_limit_lower = -0.001*pow(H_dom,3)-0.0205*pow(H_dom,2)+2.281*H_dom-2.0343;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0013*pow(H_dom,3)-0.1145*pow(H_dom,2)+3.3358*H_dom-9.8492;
	limit[0].Remainlimit_lower = 0.0034*pow(H_dom,3)-0.2289*pow(H_dom,2)+5.2443*H_dom-22.864;

}


static void Thinning_pine_VT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = 0.0138*pow(H_dom,3)-0.8216*pow(H_dom,2)+16.35*H_dom-81.678;
	double Thinning_limit_lower = 0.0046*pow(H_dom,3)-0.32*pow(H_dom,2)+7.3025*H_dom-31.084;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0003*pow(H_dom,3)-0.0515*pow(H_dom,2)+2.5401*H_dom-7.3048;
	limit[0].Remainlimit_lower = -0.0012*pow(H_dom,3)+0.0019*pow(H_dom,2)+1.3985*H_dom-3.3825;

}


static void Thinning_pine_CT_middle_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1286*pow(H_dom,2)+4.8643*H_dom-21.866;
	double Thinning_limit_lower = -0.0063*pow(H_dom,3)+0.2054*pow(H_dom,2)-1.0214*H_dom+9.2686;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0031*pow(H_dom,3)+0.1*pow(H_dom,2)-0.3375*H_dom+7.74;
	limit[0].Remainlimit_lower = 0.001*pow(H_dom,3)-0.0964*pow(H_dom,2)+2.6815*H_dom-10.294;

}


static void Thinning_pubescent_birch_middle_and_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper =  0.0063*pow(H_dom,3)-0.35*pow(H_dom,2)+7.225*H_dom-31.1;
	double Thinning_limit_lower = 0.0104*pow(H_dom,3)-0.5*pow(H_dom,2)+8.7083*H_dom-36.5;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = 0.0063*pow(H_dom,3)-0.3125*pow(H_dom,2)+5.65*H_dom-21.9;
	limit[0].Remainlimit_lower = -0.0083*pow(H_dom,3)+0.3375*pow(H_dom,2)-3.9417*H_dom+22.1;

}


static void Thinning_spruce_OMT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1*pow(H_dom,2)+4.08*H_dom-12.54;
	double Thinning_limit_lower = -0.0893*pow(H_dom,2)+3.5071*H_dom-9.3429;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.1013*pow(H_dom,2)+3.8302*H_dom-15.186;
	limit[0].Remainlimit_lower = -0.0996*pow(H_dom,2)+3.768*H_dom-17.643;

}


static void Thinning_pine_and_spruce_MT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1045*pow(H_dom,2)+4.0639*H_dom-13.386;
	double Thinning_limit_lower = -0.0714*pow(H_dom,2)+2.8857*H_dom-6.0143;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0558*pow(H_dom,2)+2.267*H_dom-4.0143;
	limit[0].Remainlimit_lower = -0.0621*pow(H_dom,2)+2.4659*H_dom-8.4857;

}


static void Thinning_pine_VT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.0857*pow(H_dom,2)+3.43*H_dom-10.114;
	double Thinning_limit_lower = -0.0612*pow(H_dom,2)+2.5791*H_dom-5.7286;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0897*pow(H_dom,2)+3.3891*H_dom-14.8;
	limit[0].Remainlimit_lower = -0.0714*pow(H_dom,2)+2.6971*H_dom-11.486;

}


static void Thinning_pine_CT_northern_Finland_Tapio (double H_dom, double THINNING_LEVEL, struct Thinning_limits *limit)
{

	double Thinning_limit_upper = -0.1036*pow(H_dom,2)+3.7*H_dom-11.891;
	double Thinning_limit_lower = -0.0982*pow(H_dom,2)+3.385*H_dom-11.014;
	double difference = Thinning_limit_upper - Thinning_limit_lower;
	if (difference<0)
		difference = 0;

	limit[0].Thinning_limit = Thinning_limit_lower + (difference*THINNING_LEVEL);
	limit[0].Remainlimit_upper = -0.0893*pow(H_dom,2)+3.13*H_dom-12.486;
	limit[0].Remainlimit_lower = -0.0536*pow(H_dom,2)+2.03*H_dom-7.1714;

}



int Thinning_limits_heureka_JKK (double H_dom, double MAIN_SP, double REGION, double SI, double THINNING_LEVEL,
						int *nres, struct Thinning_limits *limits, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    //According to Bj�rn Elfving Finnish thinning models can be used. Heureka models are from year 1985.
    
    int ret = 1;
    double thinning_limit=0;
    double remain_up=0;
    double remain_low=0;
    
    // Assign H_dom to 10 to be able to calculate remaining limits also for short trees  
    if (H_dom<=10)
    	H_dom = 10.01;
    
	//Calculate thinning limits and define whether comp_unit is ready for thinning
	//Southern Sweden
	if (REGION==1||REGION==2)
	{
		//Pine dominated stands SI 16-28
	    	if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    	{
	    		if (SI>25)
	    		//OMT,MT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=25 && SI>23)
	    		//MT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=23 && SI>21)
	    		//VT
	    		{
	    			Thinning_pine_VT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=21 && SI>19)
	    		//CT regen_age=120 ja regen_dgm=24
	    		{
	    			Thinning_pine_CT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=19 && SI>17)
	    		//CT regen_age=120 ja regen_dgm=23
	    		{
	    			Thinning_pine_CT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//SI<17 
	    		else
	    		//ClT regen_age=130 ja regen_dgm=22
	    		{
	    			Thinning_pine_CT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    	}
	    	//Spruce dominated stand 18-36
	    	else if (MAIN_SP==2&&H_dom>10)
	    	{
	    		if (SI>31)
	    		//OMT ja OMaT
	    		{
	    			Thinning_spruce_OMT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=31 && SI>27)
	    		//OMT
	    		{
	    			Thinning_spruce_OMT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=27 && SI>23)
	    		//MT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=23 && SI>19)
	    		//VT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//SI<19
	    		else
	    		//CT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    	}
	    	//Birch or other deciduous dominated stand
	    	else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    	{
	    		//all deciduous species and SI-values with pine model
	    		Thinning_white_birch_OMT_MT_southern_and_middle_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    	}
	    	//Too little H_dom
	    	else
	    	{
	  		thinning_limit = 99;
          	}
	}
	//Northern Sweden (REGION==3)
	else
	{
	        //Pine dominated stands SI 16-28
	    	if ((MAIN_SP==1||MAIN_SP==8)&&H_dom>10)
	    	{
	    		if (SI>25)
	    		//OMT,MT
	    		{
	    			Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=25 && SI>23)
	    		//MT
	    		{
	    			Thinning_spruce_and_pine_MT_southern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=23 && SI>21)
	    		//VT
	    		{
	    			Thinning_pine_VT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=21 && SI>19)
	    		//CT
	    		{
	    			Thinning_pine_CT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=19 && SI>17)
	    		//CT
	    		{
	    			Thinning_pine_CT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//SI<17 regen_age=130 ja regen_dgm=22
	    		else
	    		//ClT
	    		{
	    			Thinning_pine_CT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    	}
	    	//Spruce dominated stand 18-36
	    	else if (MAIN_SP==2&&H_dom>10)
	    	{
	    		if (SI>31)
	    		//OMT ja OMaT
	    		{
	    			Thinning_spruce_OMT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=31 && SI>27)
	    		//MT
	    		{
	    			Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=27 && SI>23)
	    		//MT
	    		{
	    			Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		else if (SI<=23 && SI>19)
	    		//VT
	    		{
	    			Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    		//SI<19
	    		else
	    		//CT
	    		{
	    			Thinning_pine_and_spruce_MT_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    		}
	    	}
	    	//Birch or other deciduous dominated stand
	    	else if ((MAIN_SP==3||MAIN_SP==4||MAIN_SP==5||MAIN_SP==6||MAIN_SP==7||MAIN_SP==9)&&H_dom>10)
	    	{
	    		//all deciduous species and SI-values with pine model
	    		Thinning_pubescent_birch_middle_and_northern_Finland_Tapio(H_dom, THINNING_LEVEL, limits);
	    	}
	    	//Too little H_dom
	    	else
	    	{
	  		thinning_limit = 99;
          	}
	}
	
	if (thinning_limit==0)
	{
	   thinning_limit = limits[0].Thinning_limit;
	   remain_up = limits[0].Remainlimit_upper;
	   remain_low = limits[0].Remainlimit_lower;
    	}
    
	*nres = 1;
	limits[0].Thinning_limit = thinning_limit;
	limits[0].Remainlimit_upper = remain_up;
	limits[0].Remainlimit_lower = remain_low;
	limits[0].JUST_THINNED = 0;
	
	*nres = 1;
	return ret;

}
