// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  HeurekamodelLibrary.cpp
//
// 	Heureka Models - Header file
//
//	Here all model functions must be defined

#if MODE==1
# define DLLEXPORT __declspec (dllexport)
#elif MODE==2
# define DLLEXPORT
#elif MODE==3
# define DLLEXPORT
#endif

#define PI 3.1415926535897932384626433832795029

struct Tree_h {
	double i_h;
	double h;
};

struct Thinning_limits {
	double Thinning_limit;
	double Remainlimit_upper;
	double Remainlimit_lower;
	double JUST_THINNED;
};

//	Conventional function declarations used when the library is linked
//	using the .LIB file / tjp 4.5.2005

//  All the model calls follow the same rules for the input parameters. Each
//  model is called with the actual model parameters, plus a set of pointers
//  which the model function sets to contain the number of results, the
//  actual modelresult and errors. The last three parameters
//  are the error checking mode, allowed risk level and the rectification
//  factor for the model. This way the models can be called easily from python
//  and C/C++ implementation. / AM 2.9.2005

//Undecorate C++ functions
#ifdef __cplusplus
extern "C" {
#endif

DLLEXPORT int Basal_area_growth_pine_Elfving (double d, double BAL, double D_gM, double A, double OST, double BA,  double BA_propother, double GOTLAND, double TS, double SI, double VEGTYPE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_spruce_Elfving (double d, double BAL, double D_gM, double SUM_d, double N, double A, double OST, double BA, double BA_propother, double GOTLAND, double TS, double SI, double VEGTYPE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_birch_Elfving (double d, double BAL, double A, double OST, double BA, double BA_propother, double TS, double DISTCOAST, double VEGTYPE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_aspen_Elfving (double d, double BAL, double A, double BA, double BA_propother, double TS, double VEGTYPE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_beech_Elfving (double d, double BAL, double A, double BA, double BA_propother, double LAT, double SI, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_oak_Elfving (double d, double BAL, double BA, double BA_propother, double GOTLAND, double ALT, double VEGTYPE, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_noble_Elfving (double d, double BAL, double A, double BA, double GOTLAND, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_trivial_Elfving (double d, double BAL, double A, double BA, double SI, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int BA_growth_Elfving (double d, double BAL, double D_gM, double SUM_d, double N, double A, double OST, double BA, double BA_propother, double GOTLAND, double TS, double SI, double VEGTYPE, double DISTCOAST, double LAT, double ALT, double SP, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_growth_Elfving (double h, double age, double a, double d, double BAL, double SP, double BA_propNs, double time_step, int *nres, struct Tree_h *height, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Height_Soderberg (double d, double a, double SI, double BA, double MAX_d, double LAT, double ALT, double BA_propSp, double BA_propNs, double BA_propB, double BA_propBe, double DISTCOAST, double REGION, double SP, double Calibration_height, int *nres, double *modelresult, char *errors,int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_pine_Elfving (double d, double D_gM, double Age, double BA, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_spruce_Elfving (double d, double D_gM, double Age, double BA, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_birch_Elfving (double d, double D_gM, double Age, double BA, double SI, double VEGTYPE, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_other_deciduous_Elfving (double d, double D_gM, double Age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_Elfving (double d, double D_gM, double Age, double BA, double SI, double VEGTYPE, double SP, double t_href, double Calibration_age, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Temperature_sum_Elfving (double LAT, double ALT, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Time_to_reach_breast_height_Elfving (double LAT, double SI, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Mean_height_Hugin (double Age, double SI, double SP, double H_gM, double time_step, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Age_at_breast_height_Hugin (double h, double H_ari, double SI, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Diameter_at_breast_height_Hugin (double h, double MAX_h, double SUM_h2, double SUMh2_propdecid, double CULT, double DISTCOAST, double LAT, double ALT, double SI, double VEGTYPE, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Site_index_Elfving (double h, double age, double a, double d, double BAL, double SP, double time_step, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Tree_volume_heureka_JKK (double d, double h, double SP, double REGION, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Calibration_age_heureka_JKK (double D_gM, double Age, double BA, double SI, double VEGTYPE, double SP, double t_href, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Calibration_height_heureka_JKK (double H_gM, double D_gM, double A, double SI, double BA, double MAX_d, double LAT, double ALT, double BA_propSp, double BA_propNs, double BA_propB, double BA_propBe, double DISTCOAST, double REGION, double SP, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_growth_stand_Elfving (double Age, double BA_propCo, double BA_propSp, double BA_propNs, double BA_propB, double BA,  double N, double VEGTYPE, double PEAT, double SOILMOIST, double SI, double TS, double SINCE_THINNING, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Basal_area_coefficient_JKK (double i_BA, double SUM_iba, int *nres, double *modelresult, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);
DLLEXPORT int Thinning_limits_heureka_JKK (double H_dom, double MAIN_SP, double REGION, double SI, double THINNING_LEVEL, int *nres, struct Thinning_limits *limits, char *errors, int errorCheckMode, double allowedRiskLevel, double rectFactor);

#ifdef __cplusplus
}
#endif