#!/usr/bin/env python
# encoding: utf-8
"""
*operation2modelchains.py*

Process the data conversion XML definitions for data import

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""
from simo.builder.xmlobject import XmlObject

class OperationMappingDef(XmlObject):
    """Data conversion definitions"""

    def __init__(self, typedef):
        super(OperationMappingDef, self).__init__(typedef)
        self.from_module = 'operation mapping'

    def xml_to_obj(self, xmlname, root, lexicon):
        """operation mapping XML definition to object instance"""
        self.lexicon = lexicon
        ns = self.elem_namespace(root.tag)
        cm = OperationMapping(ns, root, self)
        self.obj[xmlname] = cm
        self.lexicon = None


class OperationMapping(object):
    """Data conversion definition for data import"""
    def __init__(self, ns, root, validator):
        super(OperationMapping, self).__init__()
        self.operation_mapping = {}
        self.forced_op_chains = set()
        for op in root.findall(ns+'operation'):
            # Get current operation name
            fromelem = op.find(ns+'from')
            opname = fromelem.find(ns+'name').text
            self.operation_mapping[opname] = []
            # Get current operation name to modelchains mapping
            toelem = op.find(ns+'to')
            for chain in toelem.findall(ns+'model_chain'):
                chainname = chain.text
                self.operation_mapping[opname].append(chainname)
                self.forced_op_chains.update([chainname])
