#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO lexicon variable

Classes for storing lexicon level definition

*lexiconlevel.py*

Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""

from simo.builder.lexicon.lexiconvariable import LexiconVariable

class LexiconLevel(object):
    """ Class for storing a single data level in lexicon """
    def __init__(self, ns, elem, validator):
        """Initialize lexicon level object"""
        super(LexiconLevel, self).__init__()
        self._ns = ns
        self._validator = validator
        self._name = elem.find(self._ns + "name").text
        self._type = 'static'
        if elem.find(self._ns + "type") is not None:
            self._type = elem.find(self._ns + "type").text
        if elem.find(self._ns + "geom_type") is not None:
            self._geom_type = elem.find(self._ns + "geom_type").text
        else:
            self._geom_type = None
        self._variables = {}
        self._numerical = self._parse_vars(elem.find(self._ns + "num_vars"))
        self._categorical = self._parse_vars(elem.find(self._ns + "cat_vars"))
        self._textual = self._parse_vars(elem.find(self._ns + "text_vars"))
        self._validator = None

    def _parse_vars(self, elem):
        """ Parse single level variables from lexicon """
        if elem is None: return []
        variables = []
        for velem in elem:
            var = LexiconVariable(self._ns, velem, self._validator)
            self._variables[var.name] = var
            variables.append(var.name)
        return set(variables)

    def name():
        doc = "Level name"
        def fget(self):
            return self._name
        return locals()
    name = property(**name())

    def type():
        doc = "Level type"
        def fget(self):
            return self._type
        return locals()
    type = property(**type())

    def geom_type():
        doc = "Level geometry type"
        def fget(self):
            return self._geom_type
        return locals()
    geom_type = property(**geom_type())

    def numerical():
        doc = "Numerical variables"
        def fget(self):
            return self._numerical
        return locals()
    numerical = property(**numerical())

    def categorical():
        doc = "Categorical variables"
        def fget(self):
            return self._categorical
        return locals()
    categorical = property(**categorical())
    
    def textual():
        doc = "Textual variables"
        def fget(self):
            return self._textual
        return locals()
    textual = property(**textual())

    def variables():
        doc = "Get variable index as a dictionary"
        def fget(self):
            return self._variables
        return locals()
    variables = property(**variables())


def main():
    pass

if __name__ == '__main__':
    main()
