#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO lexicon variable

Classes for storing lexicon variable definition

*lexiconvariable.py*

Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""

class LexiconVariable(object):
    """ Class for storing a single variable in lexicon """
    def __init__(self, ns, elem, validator):
        """Parse an xml snippet containing variable definition and construct
        a variable instance."""
        super(LexiconVariable, self).__init__()
        self._name = None
        self._unit = None
        self._minvalue = None
        self._maxvalue = None
        self._desc = None
        self._values = None
        self._name = elem.find(ns + 'name').text
        unit = elem.find(ns + 'unit')
        minvalue = elem.find(ns + 'min_value')
        maxvalue = elem.find(ns + 'max_value')
        if unit is not None:
            self._unit = unit.text
        if minvalue is not None:
            if minvalue.text is not None:
                self._minvalue = float(minvalue.text)
        if maxvalue is not None:
            if maxvalue.text is not None:
                self._maxvalue = float(maxvalue.text)
        self._desc = elem.find(ns + 'description').text
        if self._desc is None:
            errormsg = "No description given for variable '%s'" % self._name
            validator.add_error(errormsg)
        self._values = self._parse_values(elem.find(ns + 'values'))

    def _parse_values(self, elem):
        """ Parse categorical variable enumerated values """
        if elem is None: return None
        values = {}
        for velem in elem:
            values[float(velem[0].text)] = velem[1].text
        return values

    def name():
        doc = "Variable name"
        def fget(self):
            return self._name
        return locals()
    name = property(**name())

    def unit():
        doc = "Variable unit"
        def fget(self):
            return self._unit
        return locals()
    unit = property(**unit())

    def minimum():
        doc = "Variable minimum value"
        def fget(self):
            return self._minvalue
        return locals()
    minimum = property(**minimum())

    def maximum():
        doc = "Variable maximum value"
        def fget(self):
            return self._maxvalue
        return locals()
    maximum = property(**maximum())

    def desc():
        doc = "Variable description"
        def fget(self):
            return self._desc
        return locals()
    desc = property(**desc())

    def values():
        doc = "Categorical variable values"
        def fget(self):
            return self._values
        return locals()
    values = property(**values())


def main():
    pass

if __name__ == '__main__':
    main()
