#!/usr/bin/env python
# encoding: utf-8
"""
*cashflowmodel.py*

Process the cash flow model XML definitions

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

from simo.builder.modelbase.model import Model
from simo.builder import names


class CashFlowModel(Model):
    """SIMO cash flow model definition
    """
    def __init__(self, ns, elem, validator, dirs):
        super(CashFlowModel, self).__init__()
        self._validator = validator
        self._type = names.CASH_FLOW
        self._name = self._get_val(elem, ns+'name')
        self._lib = self._get_val(elem, ns+'implemented_at')
        self._lang = self._get_val(elem, ns+'implemented_in').lower()
        ref_values = elem.find(ns+'reference_values')
        self.table_names = []
        for name_elem in ref_values.find(ns+'cash_flow_tables'):
            self.table_names.append(name_elem.text)
        self._date = self._get_val(ref_values, ns+'date')
        self._dirs = dirs
        self.variables = []
        self.table = None
        # Not all models have variables, so we need to check
        if elem.find(ns+'variables') is not None:
            for velem in elem.find(ns+'variables'):
                name = velem.find(ns+'name').text
                level = velem.find(ns+'level').text
                levind, varind = validator.variable_ind(level, name)
                self.variables.append((levind, varind))
        validator.add_model(self._type, self._name)
        self._validator = None

    def _get_val(self, elem, tag):
        item = elem.find(tag)
        if item is not None:
            return item.text
        return None


#class CashFlowModelParam(object):
#    """Variable value for cashflow model, defined in modelchain"""
#    def __init__(self, ns, elem, task, model):
#        super(CashFlowModelParam, self).__init__()
#        self.variables = []
#        self._ns = ns
#        self._validator = task.validator
#        varelem = elem.find(ns+'variables')
#        if varelem is not None:
#            for var in varelem:
#                self._parse_var(var)
#        self._validator = None
#
#    def _parse_var(self, elem):
#        """"""
#        level = elem[0].find(self._ns + 'level').text
#        var = elem[0].find(self._ns + 'name').text
#        vardef = self._validator.variable_ind(level, var)
#        self.variables.append({'type': 'variable', 'vardef': vardef})


def main():
   pass

if __name__ == '__main__':
   main()

