#!/usr/bin/env python
# encoding: utf-8
"""
*parametertable.py*

Process the parameter table XML definitions

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

import re
import copy
import numpy

import simo.utils.utils as utils

from simo.builder.modelbase.table import table2array
from simo.builder.modelbase.table import table2dict
from simo.builder.modelbase.table import parse_classifiers
from simo.builder.modelbase.table import parse_tables
from simo.builder import names


class ParameterTable(object):
    """Parameter table definition"""
    def __init__(self, ns, elem, validator, dirs):
        super(ParameterTable, self).__init__()
        self.name = elem.get('name')
        self.type = names.PARAMETER_TABLE
        self.classifiers = None
        self.classifier_order = None
        self.Nclassifiers = None
        self.targets = None
        self.Ntargets = None
        self.arrays = None
        self.Narrays = None
        self.active_array = None
        # parse parameter table classifier variables
        celem = elem.find(ns + 'classifiers')
        self.classifiers, dummy, dummy, self.classifier_order, self.keycount = \
            parse_classifiers(ns, celem, validator)
        self.Nclassifiers = len(self.classifiers)
        # parse parameter table target variables
        trgelem = elem.find(ns + 'targets')
        self.targets = [ParamTableTarget(ns, e[0], validator) for e in trgelem]
        self.Ntargets = len(self.targets)
        # parse parameter table arrays
        tabelem = elem.find(ns + 'table')
        self.arrays, self.Narrays = parse_tables([tabelem])
        validator.add_model(names.PARAMETER_TABLE, self.name)


class ParamTableTarget(object):
    def __init__(self, ns, elem, validator):
        """docstring for __init__"""
        super(ParamTableTarget, self).__init__()
        self.type = None
        self.variable = None
        self.level = None
        self.length = 1
        if validator.elem_name(elem.tag) == 'data':
            self.type = 'variable'
            name, level = elem[0].text, elem[1].text
            self.level, self.variable = validator.variable_ind(level, name)
        elif validator.elem_name(elem.tag) == 'operation_parameter':
            self.type = 'operation_parameter'
            self.variable, self.length = elem[0].text, int(elem[1].text)


class ParameterTableParam(object):
    """docstring for ParameterTableParam"""
    def __init__(self):
        super(ParameterTableParam, self).__init__()


def main():
    pass

if __name__ == '__main__':
    main()
