#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO Aggregation output definition

...

*aggrdef.py*

Copyright (c) 2009 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""

from datetime import timedelta
from datetime import date
import time
from lxml import etree

from simo.builder.xmlobject import XmlObject

class AggregationOutputDef(XmlObject):
    """Optimization task definition"""
    
    def __init__(self, typedef):
        """docstring for __init__"""
        super(AggregationOutputDef, self).__init__(typedef)
        self.from_module = "aggregation output"

    def xml_to_obj(self, xmlname, root, lexicon):
        """lexicon XML definition to object instance"""
        self.lexicon = lexicon
        ns = self.elem_namespace(root.tag)
        opt = AggregationOutput(ns, root, self)
        self.obj[xmlname] = opt


class AggregationOutput(object):
    """Optimization task"""
    
    def __init__(self, ns, root, validator):
        """docstring for __init__"""
        self._ns = ns
        self._validator = validator
        self.data = []
        self._parse(root)
        self._validator = None

    def _parse(self, root):
        init_date = date(\
                    *time.strptime(root.find(self._ns+'initial_date').text, 
                                   "%Y-%m-%d")[0:3])
        discount_rate = float(root.find(self._ns+'discount_rate').text)
        rp_element = root.find(self._ns+'reporting_periods')
        reporting_periods = []
        reporting_variables = []
        result = {'discount_rate': discount_rate,
                  'reporting_periods': reporting_periods,
                  'reporting_variables': reporting_variables,
                  'initial_date': init_date}
        for element in rp_element.iter(self._ns+'period'):
            length = element[0].text
            unit = element[1].text
            reporting_periods.append({'length': length, 
                                      'unit': unit})
        reports = root.find(self._ns+'reports')
        for element in reports.iter(self._ns+'report'):
            expression = element.find(self._ns+'expression').text
            proportion = (element.find(self._ns+'proportion').text in ('true', 1))
            grouping = {}
            grp_elems = element.find(self._ns+'group_by')
            if grp_elems is not None:
                for grp_elem in grp_elems.iter(self._ns+'grouping'):
                    level = grp_elem.find(self._ns+'level').text
                    var = grp_elem.find(self._ns+'variable').text
                    if level not in grouping:
                        grouping[level] = [var]
                    else:
                        grouping[level].append(var)
            reporting_variables.append({'expression':expression,
                                        'proportion':proportion,
                                        'grouping':grouping})
        self.data = result


def main():
    pass

if __name__ == '__main__':
    main()