#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO Expression output definition

...

*aggrdef.py*

Copyright (c) 2009 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""

from datetime import timedelta
from datetime import date
import time
from lxml import etree

from simo.builder.xmlobject import XmlObject

class ExpressionOutputDef(XmlObject):
    """Optimization task definition"""
    
    def __init__(self, typedef):
        """docstring for __init__"""
        super(ExpressionOutputDef, self).__init__(typedef)
        self.from_module = "aggregation output"

    def xml_to_obj(self, xmlname, root, lexicon):
        """lexicon XML definition to object instance"""
        self.lexicon = lexicon
        ns = self.elem_namespace(root.tag)
        opt = ExpressionOutput(ns, root, self)
        self.obj[xmlname] = opt


class ExpressionOutput(object):
    """Optimization task"""
    
    def __init__(self, ns, root, validator):
        """docstring for __init__"""
        self._ns = ns
        self._validator = validator
        self.data = []
        self._parse(root)
        self._validator = None

    def _parse(self, root):
        init_date = date(\
                    *time.strptime(root.find(self._ns+'initial_date').text, 
                                   "%Y-%m-%d")[0:3])
        discount_rate = float(root.find(self._ns+'discount_rate').text)
        reporting_variables = []
        result = {'discount_rate': discount_rate,
                  'expression_variables': reporting_variables,
                  'initial_date': init_date}
        
        reports = root.find(self._ns+'expressions')
        for element in reports.iter(self._ns+'item'):
            expression = element.find(self._ns+'expression').text.strip()
            label = element.find(self._ns+'label').text.strip()
            reporting_variables.append({'expression': expression,
                                        'label': label})
        self.data = result


def main():
    pass

if __name__ == '__main__':
    main()