#!/usr/bin/env python
# encoding: utf-8
"""
*OutputConstraint.py*

Process the data conversion XML definitions for data import

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""
from simo.builder.xmlobject import XmlObject

class OutputConstraintDef(XmlObject):
    """Data conversion definitions"""

    def __init__(self, typedef):
        super(OutputConstraintDef, self).__init__(typedef)
        self.from_module = 'output constraint'

    def xml_to_obj(self, xmlname, root, lexicon):
        """output constraint XML definition to object instance"""
        self.lexicon = lexicon
        ns = self.elem_namespace(root.tag)
        cm = OutputConstraint(ns, root, self)
        self.obj[xmlname] = cm
        self.lexicon = None

class OutputConstraint(object):
    """Defines the structure of the result database"""
    def __init__(self, ns, root, validator):
        super(OutputConstraint, self).__init__()
        self.variables={}
        for level in root.findall(ns+'data_level'):
            lname = level[0].text
            lind = validator.level_ind(lname)
            varnames = level[1].text.split()
            if len(varnames) == 1:
                if varnames[0].strip() == '*':
                    self.variables[(lname, lind)] = None
                    continue 
            varinds = []
            for vname in varnames:
                #NB! Checking for variable being active has been disabled for
                # now, as output contraints are not really used. Enable it
                # if output constraints will be activated!
                #varind = validator.variable_ind(lname, vname, True)
                varind = validator.variable_ind(lname, vname, False)
                varinds.append(varind[1])
            self.variables[(lname, lind)] = zip(varnames, varinds)

def main():
    import doctest
    doctest.testfile('./test/outputconstraint-py.rst')

if __name__ == '__main__':
    main()