#!/usr/bin/env python
# encoding: utf-8
"""
*lexicontranslationtable.py*

Process the lexicon translation table XML definition

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

from simo.builder.xmlobject import XmlObject

class LexiconTransTableDef(XmlObject):

    """Lexicon translation for SIMO"""
    def __init__(self, typedef):
        super(LexiconTransTableDef, self).__init__(typedef)

    def xml_to_obj(self, xmlname, root, lexicon):
        """
        Construct lexicon translations tables from the parsed translation
        table xml. Data level names and variable names get their own
        translation tables
        """
        # Iterate through data level names
        ns = self.elem_namespace(root.tag)
        lt = LexiconTransTable(ns, root)
        self.obj[xmlname] = lt

class LexiconTransTable(object):
    """Lexicon message translation definitions"""
    def __init__(self, ns, elem):
        super(LexiconTransTable, self).__init__()
        self.level_table = {}
        self.var_table = {}
        self.value_table = {}
        for dl in elem:
            transelem = dl.find(ns+'name')
            self.__get_mappings(transelem, ns, self.level_table)
            vars = dl.find(ns+'variables')
            #Since there may be no variables, we have to check.
            if vars is not None:
                for var in vars:
                    transelem = var.find(ns+'name')
                    self.__get_mappings(transelem, ns, self.var_table)
                    values = var.find(ns+'values')
                    if values is not None: #same here, could be None
                        for value in values:
                            self.__get_mappings(value, ns, self.value_table)

    def __get_mappings(self, transelem, ns, resobj, variable=None):
            fromdata = "'" + transelem.find(ns+'from').text + "'"
            for todata in transelem.find(ns+'to'):
                lcode = todata.get('name').lower()
                toname = unicode("'" + todata.text + "'")
                if variable is None:
                    try:
                        resobj[fromdata]
                    except:
                        resobj[fromdata] = {}
                    resobj[fromdata][lcode] = toname
                else:
                    try:
                        resobj[variable][fromdata]
                    except:
                        resobj[variable][fromdata] = {}
                    resobj[variable][fromdata][lcode] = toname

def main():
    pass

if __name__ == '__main__':
    main()
