#!/usr/bin/env python
# encoding: utf-8
"""
*messagetranslationtable.py*

Process the message translation table XML definition


Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public 
License 2.0.
"""

import re
from simo.builder.xmlobject import XmlObject

class MsgTransTableDef(XmlObject):
    """Log message translation for SIMO"""

    def __init__(self, typedef):
        super(MsgTransTableDef, self).__init__(typedef)

    def xml_to_obj(self, xmlname, root, lexicon):
        """
        Construct message translation table from the parsed translation table 
        xml
        """
        # Iterate through log levels
        ns = self.elem_namespace(root.tag)
        mt = MsgTransTable(ns, root)
        self.obj[xmlname] = mt
    
class MsgTransTable(object):
    """Log message translation definitions"""
    def __init__(self, ns, elem):
        super(MsgTransTable, self).__init__()
        self.translation = {}#defaultdict(dict)
        for loggerlevel in elem:            
            loglevel = loggerlevel.get('name')
            # Construct translations for different loggers
            # original message -> messages keyed by language identifiers
            for logger in loggerlevel:
                logname = logger.get('name')
                msgs = []
                msgmaps = {}
                for msg in logger:
                    frommsg = msg.find(ns+'from').text
                    frompatt = re.compile(frommsg)
                    msgs.append((frompatt, frommsg))
                    ldict = {}
                    for lang in msg.find(ns+'to'):
                        lcode = lang.get('name').lower()
                        tomsg = unicode(lang.text)
                        ldict[lcode] = tomsg
                    msgmaps[frommsg] = ldict
                try:
                    self.translation[loglevel]
                except:
                    self.translation[loglevel] = {}
                self.translation[loglevel][logname] = (msgs, msgmaps)
        
def main():
    pass

if __name__ == '__main__':
    main()
