#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
TODO:

"""

# these are for running the doctest from src/simo
import sys
#sys.path.append('..')
#DEBUG for main()
sys.path.append('../../..')

#import sqlite3
import logging
import time
import datetime
import numpy
import math
import hashlib
import os
from collections import defaultdict
from copy import copy, deepcopy
from simo.simulation.model.forcedoperation import ForcedOperation

try:
    import psycopg2
except:
    pass
else:
    import psycopg2.extensions
    psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
    psycopg2.extensions.register_type(psycopg2.extensions.FLOAT)

try:
    from pysqlite2 import dbapi2 as sqlite3
except:
    pass

global cx_Oracle
cx_Oracle = None

SRID = 3067

DEBUG = False

DATA_LINK_INSERT_SQL = \
    'INSERT INTO %sdata_link (data_level, id, iteration, branch, data_date, '\
        'data_id) VALUES (?, ?, ?, ?, ?, ?)'

# data_link table
DL_ID_COL = 1
DL_ITER_COL = 2
DL_BRANCH_COL = 3
DL_DATE_COL = 4
DL_DATA_ID_COL = 5
# level tables
DATA_ID_COL = 0
ID_COL = 1
ORIG_ID_COL = 2
PID_COL = 3
DATA_COL = 4
# op_link
OP_ID_COL = 0
OP_ITERR_COL = 1
OP_BRANCH_COL = 2
OP_DATE_COL = 3
OP_OP_ID_COL = 4
OP_OP_NAME_COL = 7
OPRES_OP_ID_COL = 0

OP2SQL = {'gt':'>', 'ge':'>=', 'eq':'=', 'ue':'!=', 'le':'<=', 'lt':'<',
          'in':'IN'}
ANDOR2SQL = {'and':'AND', 'or':'OR', None:''}

SQLITEID = 'id INTEGER PRIMARY KEY'
POSTGRESQLID = 'id SERIAL PRIMARY KEY'
ORACLEID = 'id NUMBER PRIMARY KEY'

POSTGRESQL_RETRIES = 5

if hasattr(sys, 'frozen'):
    try:
        filepath = os.environ['_MEIPASS2']
    except KeyError:
        try:
            filepath = os.path.dirname(__loader__.archive)
        except:
            try:
                filepath = os.path.dirname(sys.executable)
            except:
                filepath = os.path.abspath(os.path.dirname(__file__))
                if DEBUG:
                    print 'Could not find custom file path while frozen.'
                    print 'Using: ', filepath
else:
    filepath = os.path.abspath(os.path.dirname(__file__))

class SimpleLogger(object):
    def __init__(self):
        super(SimpleLogger, self).__init__()
        logger = logging.getLogger('SimpleLogger')
        logger.setLevel(logging.DEBUG)
        if not logger.handlers:
            handler = logging.StreamHandler()
            formatter = logging.Formatter("[%(asctime)s: "\
                                          "%(levelname)s/%(name)s] "\
                                          "%(message)s")
            handler.setFormatter(formatter)
            logger.addHandler(handler)
        logger.propagate = False
        self.logger = logger

    def log_message(self, lname, mtype, message, sim_id=None):
        logger = self.logger
        logger.name = lname
        if mtype=='critical':
            logger.critical(message)
        elif mtype=='error':
            logger.error(message)
        elif mtype=='warning':
            logger.warning(message)
        elif mtype=='info':
            logger.info(message)
        elif mtype=='debug':
            logger.debug(message)
        elif mtype == 'exception':
            logger.exception(message)

class DBInterface(object):
    def __init__(self):
        super(DBInterface, self).__init__()

    def open(self):
        raise Exception('Not implemented')

    def close(self):
        raise Exception('Not implemented')

    def execute(self, sql, data=None, control=False, cur=None):
        """
        sql -- the sql to execute
        data -- the data required for the sql to be executed
        cur -- if passed a cursor created elsewhere is used, it won't be closed
        control howto:
        control defaults to None.
        If control is ['skip_existing', data]: if the data is already in the
                                               db do not execute the sql
        """
        raise Exception('Not implemented')

    def get(self, sql, data=None, control=False):
        """
        The only difference to execute is that there is no autocommit, and the
        results are returned in a list of row tuples.
        """
        raise Exception('Not implemented')

    def executemany(self, sql, datas):
        raise Exception('Not implemented')

    def table_exists(self):
        raise Exception('Not implemented')

    def index_exists(self):
        raise Exception('Not implemented')

class Oracle(DBInterface):
    def __init__(self, db_name='default', db_host='localhost:1521/orcl',
                 user='test', pw='test',
                 wipe=False, spatial=None, no_exec=False, return_dict=False,
                 logger=None, add='', threaded=False):
        super(Oracle, self).__init__()
        global cx_Oracle
        if not cx_Oracle:
            # Use unicode NLS_LANG for cx_Oracle if no value preset.
            if not 'NLS_LANG' in os.environ:
                os.environ['NLS_LANG'] = '.AL32UTF8'
            import cx_Oracle

        self._add = add
        self._create_concurrently = ''
        self.opened = False
        if logger is None:
            logger = SimpleLogger()
        self.logger = logger
        # TODO: make fixes to ensure spatial works.
        if spatial:
            raise Exception('SPATIAL CURRENTLY NOT SUPPORTED FOR ORACLE!')
        self.store = []
        self.no_exec = no_exec
        self.return_dict = return_dict
        if DEBUG:
            self.debug_data = set()
            self.execute = self.debug_execute
            self.get = self.debug_get
        self.spatial = spatial
        self.user = user
        self.db_name = db_name
        c_str = '%s/%s@%s' % (user, pw, db_host)
        self.c_str = c_str
        self.threaded = threaded
        self.open()
        if wipe and not no_exec:
            self.drop_tables(user, db_name)

    def commit(self):
        if not self.no_exec:
            self.conn.commit()

    def create_autoid(self, table):
        tseq = table[:26]+'_seq'
        ttrig = table[:26]+'_trg'
        exists = self.get('SELECT count(*) '\
                          'FROM ALL_SEQUENCES '\
                          'WHERE SEQUENCE_OWNER=? AND SEQUENCE_NAME=?',
                          (self.user.upper(), (tseq).upper()))
        if exists[0][0] > 0:
            return # already have these, move along.
        self.execute('CREATE SEQUENCE %s '\
                     'START WITH 1 '\
                     'INCREMENT BY 1' % (tseq,))
        sql = '''CREATE OR REPLACE TRIGGER %s
                 BEFORE INSERT ON %s
                 FOR EACH ROW
                 DECLARE
                     max_id NUMBER;
                     cur_seq NUMBER;
                 BEGIN
                     IF :NEW.id IS NULL THEN
                         -- No ID passed, get one from the sequence
                         SELECT %s.NEXTVAL INTO :NEW.id FROM dual;
                     ELSE
                         -- ID was set via insert, so update the sequence
                         SELECT GREATEST(nvl(max(id),0), :NEW.id)
                         INTO max_id FROM %s;
                         SELECT %s.nextval INTO cur_seq FROM dual;
                         WHILE cur_seq < max_id
                         LOOP
                             SELECT %s.nextval INTO cur_seq FROM dual;
                         END LOOP;
                     END IF;
                 END;''' % (ttrig, table, tseq, table, tseq, tseq)
        self.execute(sql)
        self.commit()

    def debug_execute(self, sql, data=None, control=False):
        cur = self.conn.cursor()
        drop = self._do_i_skip(cur, control)
        if drop:
            cur.close()
            return

        if data is not None:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql, data)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';', data)
        else:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';')
        cur.close()

    def debug_get(self, sql, data=None, control=False):
        cur = self.conn.cursor()
        if data is not None:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql, data)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';', data)
        else:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';')
        try:
            return cur.fetchall()
        except:
            return []
        cur.close()

    def drop_tables(self, owner, prefix=''):
        """
        This will drop all tables from the given database using the db_name
        if db_class == ORACLE.
        """
        owner = owner.upper()
        prefix = prefix.upper()
        sql = 'SELECT TABLE_NAME '\
              'FROM USER_TABLES '\
              'WHERE TABLE_NAME LIKE ?'
        tables = self.get(sql, (prefix+'%',))
        for table in tables:
            table = table[0]
            try:
                self.execute('DROP TABLE %s CASCADE CONSTRAINTS PURGE' % table)
            except cx_Oracle.Error:
                self.logger.log_message('db', 'exception',
                                        'Failed to drop: %s' % table)
        tables = self.get('SELECT SEQUENCE_NAME '\
                          'FROM USER_SEQUENCES '\
                          'WHERE SEQUENCE_NAME LIKE ?',
                          ((prefix+'%'),))
        for table in tables:
            table = table[0]
            try:
                self.execute('DROP SEQUENCE %s' % table)
            except cx_Oracle.Error:
                self.logger.log_message('db', 'exception',
                                        'Failed to drop: %s' % table)
        tables = self.get('SELECT TRIGGER_NAME '\
                          'FROM USER_TRIGGERS '\
                          'WHERE TRIGGER_NAME LIKE ?',
                          ((prefix+'%'),))
        for table in tables:
            table = table[0]
            try:
                self.execute('DROP TRIGGER %s' % table)
            except cx_Oracle.Error:
                self.logger.log_message('db', 'exception',
                                        'Failed to drop: %s' % table)
        tables = self.get('SELECT INDEX_NAME '\
                          'FROM USER_INDEXES '\
                          'WHERE INDEX_NAME LIKE ?',
                          ((prefix+'%'),))
        for table in tables:
            table = table[0]
            try:
                self.execute('DROP INDEX %s' % table)
            except cx_Oracle.Error:
                self.logger.log_message('db', 'exception',
                                        'Failed to drop: %s' % table)

    def executemany(self, sql, datas):
        cur = self.conn.cursor()
        try:
            i = 0
            while True:
                try:
                    ind = sql.index('?')
                    sql = sql[:ind] + ':v%d' % i + sql[ind+1:]
                    i += 1
                except ValueError:
                    break
            new_datas = []
            for data in datas:
                datadict = dict([('v%d' % i, d) \
                                 for i, d in enumerate(data)])
                new_datas.append(datadict)
            cur.executemany(sql, new_datas)
        except cx_Oracle.Error, e:
            msg = 'Oracle error message: %s' % str(e)
            msg += '\n'+str(sql[:e.args[0].offset])
            msg += '\nPROBLEM HERE: %s' % str(sql[e.args[0].offset:])
            if new_datas:
                msg += '\nRun with %s' % str(new_datas)
            self.logger.log_message('db', 'exception', msg)
            raise
        finally:
            cur.close()

    def execute(self, sql, data=None, control=False):
        if sql:
            if (sql.strip().startswith('CREATE TABLE') or
                sql.strip().startswith('CREATE UNLOGGED TABLE') or
                sql.strip().startswith('ALTER TABLE')):
                sql = sql.replace('TEXT', 'NVARCHAR2(1000)')
                sql = sql.replace('DOUBLE PRECISION', 'BINARY_FLOAT')
            if isinstance(sql, unicode):
                sql = str(sql)
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        drop = self._do_i_skip(cur, control)
        if drop:
            cur.close()
            return
        try:
            if data is not None:
                i = 0
                while True:
                    try:
                        ind = sql.index('?')
                        sql = sql[:ind] + ':v%d' % i + sql[ind+1:]
                        i += 1
                    except ValueError:
                        break
                datadict = dict([('v%d' % i, d) \
                                 for i, d in enumerate(data)])
                cur.execute(sql, datadict)
            else:
                cur.execute(sql)
        except cx_Oracle.Error, e:
            msg = 'Oracle error message: %s' % str(e)
            msg += '\n'+str(sql[:e.args[0].offset])
            msg += '\nPROBLEM HERE: %s' % str(sql[e.args[0].offset:])
            if data:
                msg += '\nRun with %s' % str(datadict)
            self.logger.log_message('db', 'exception', msg)
            raise
        finally:
            cur.close()

    def get(self, sql, data=None, control=False, wkb=None):
        if sql:
            if (sql.strip().startswith('CREATE TABLE') or
                sql.strip().startswith('CREATE UNLOGGED TABLE') or
                sql.strip().startswith('ALTER TABLE')):
                sql = sql.replace('TEXT', 'NVARCHAR2(1000)')
            if isinstance(sql, unicode):
                sql = str(sql)
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        datadict = None
        if data:
            i = 0
            while True:
                try:
                    ind = sql.index('?')
                    sql = sql[:ind] + ':v%d' % i + sql[ind+1:]
                    i += 1
                except ValueError:
                    break
            datadict = dict([('v%d' % i, d) for i, d in enumerate(data)])
        if wkb is not None:
            if data is None:
                datadict = {}
            ora_wkb = cur.var(cx_Oracle.BLOB, len(wkb))
            ora_wkb.setvalue(0, wkb)
            datadict.update({'wkb': ora_wkb})
        try:
            if datadict is not None:
                cur.execute(sql, datadict)
            else:
                cur.execute(sql)
            try:
                return cur.fetchall()
            except:
                return []
        except cx_Oracle.Error, e:
            msg = 'Oracle error message: %s' % str(e)
            msg += '\n'+str(sql[:e.args[0].offset])
            msg += '\nPROBLEM HERE: %s' % str(sql[e.args[0].offset:])
            if datadict:
                msg += '\nRun with %s' % str(datadict)
            self.logger.log_message('db', 'exception', msg)
            raise
        finally:
            cur.close()

    def open(self):
        self.opened = True
        for i in range(5):
            try:
                self.conn = cx_Oracle.connect(self.c_str, threaded=self.threaded)
                break
            except cx_Oracle.DatabaseError, e:
                if 'Connect timeout' in str(e) and i < 4:
                    self.logger.log_message('db', 'warning',
                        'Connection timed out, retrying %d of %d' % (i+1, 4))
                else:
                    raise
        self.conn.autocommit = False

    def close(self):
        self.opened = False
        try:
            self.conn.commit()
        except:
            pass
        try:
            self.conn.close()
        except cx_Oracle.Error:
            pass
        if DEBUG:
            for i in self.debug_data:
                self.logger.log_message('db', 'debug',
                                        '--------------------------')
                for j in i:
                    self.logger.log_message('db', 'debug', str(j))

    def clear_store(self):
        self.store = []

    def table_exists(self, name):
        if self.no_exec:
            return True
        return len(self.get("SELECT table_name FROM user_tables "\
                            "WHERE table_name='%s'" % (name.upper(),))) > 0

    def index_exists(self, name):
        if self.no_exec:
            return True
        return len(self.get("SELECT index_name FROM user_indexes "\
                            "WHERE index_name='%s'" % (name.upper(),))) > 0

    def add_geom(self, table, geom_type):
        sql = "ALTER TABLE %s ADD COLUMN(geom MDSYS.SDO_GEOMETRY)" % table
        self.execute(sql)
        sql = 'INSERT INTO USER_SDO_GEOM_METADATA '\
                '(TABLE_NAME, COLUMN_NAME, DIMINFO, SRID) '\
                'VALUES (?, "geom", '\
                'SDO_DIM_ARRAY(SDO_DIM_ELEMENT("X", ?, ?, 0.005), '\
                'SDO_DIM_ELEMENT("Y", ?, ?, 0.005)), ?)'
        srid, diminfo = self.spatial
        minx, miny, maxx, maxy = diminfo
        data = (table, minx, maxx, miny, maxy, srid)
        self.execute(sql, data)

    def add_geom_index(self, table):
        """
        Adds a spatial index on the geometry column

        table -- table to add the spatial index to
        """
        sql = 'CREATE INDEX %s_geom_index ON %s (geom) '\
                'INDEXTYPE IS MDSYS.SPATIAL_INDEX' % (table, table)
        self.execute(sql)
        return sql

    def vacuum_analyze(self, table=None):
        """
        No VACUUM for Oracle
        """
        pass

    def _do_i_skip(self, cur, control):
        if control and control[0] == 'skip_existing':
            lev_name, uid, data_id = control[1]
            if data_id is None:
                check_sql = 'SELECT id FROM %s%s WHERE id=:id and rownum=1'\
                            % (self._add, lev_name)
                sql_data = {'id':uid,}
            else:
                check_sql = 'SELECT id FROM %s%s WHERE id=:id AND '\
                            'data_id=:data_id and rownum=1'\
                        % (self._add, lev_name)
                sql_data = {'id':uid, 'data_id':data_id}
            cur = self.conn.cursor()
            cur.execute(check_sql, sql_data)
            res = cur.fetchone()
            cur.close()
            if res:
                return True
        return False


class PostgreSQL(DBInterface):

    def __init__(self, db_name='default', db_host='localhost', db_port=5432,
                 user='test', pw='test', wipe=False, spatial=None,
                 no_exec=False, return_dict=False, db_database='SIMO',
                 db_schema=None, logger=None, add='', threaded=False,
                 unlogged=False):
        super(PostgreSQL, self).__init__()
        self.opened = False
        if logger is None:
            logger = SimpleLogger()
        self.logger = logger
        self.store = []
        self.no_exec = no_exec
        self.return_dict = return_dict
        schema = db_schema
        if not schema:
            schema = user
        self.schema = schema
        self.db_database = db_database
        if DEBUG:
            self.debug_data = set()
            self.execute = self.debug_execute
            self.get = self.debug_get
        if db_name in ('postgres', 'template_postgis'):
            raise Exception('Database name must not be postgres or '\
                            'template_postgis!')
            #it's the default, you need it to create more databases
        self.db_name = db_name
        self.db_host = db_host
        self.db_port = db_port
        self.user = user
        self.pw = pw
        if spatial is None:
            spatial = (SRID, None) # TODO: remove on non-spatial => spatial resolve.
        self._add = add
        self._create_concurrently = ''#'CONCURRENTLY '
        if unlogged:
            self.unlogged = ' UNLOGGED'
#        wipe_on_open = False
        if not self.no_exec:
            try:
                # Verify that you can connect to the database
#                #TODO: ini option for core database name!
#                #PROBLEM: What to do with the non-spatial db
#                #         => spatial db problem?
#                #         current bypass => forced spatial.
                self.params = "user='%s' password='%s' "\
                              "host='%s' port='%d' dbname='%s'" % \
                              (user, pw, db_host, db_port, db_database)
                self.open()
                self.close()
            except psycopg2.Error, e:
                if not 'database "%s" does not exist' % db_database in str(e):
                    raise
                # Database didn't exist, try to create the database
                self.params = "user='%s' password='%s' "\
                              "host='%s' port='%d' dbname='postgres'" % \
                              (user, pw, db_host, db_port)
                self.open()
                i_l = self.conn.isolation_level
                self.conn.set_isolation_level(0)
                if spatial is None:
                    self.execute('CREATE DATABASE "%s"' % db_database)
                else:
                    self.execute('CREATE DATABASE "%s" '\
                                 'TEMPLATE=template_postgis' % db_database)
                self.conn.set_isolation_level(i_l)
                self.close()
        self.spatial = spatial
        self.params = "user='%s' password='%s' "\
                      "host='%s' port='%d' dbname='%s'" % \
                      (user, pw, db_host, db_port, db_database)
        self.open()
        if wipe and not no_exec:
            self.drop_tables(user, db_name)

    def set_schema(self):
        """Create and/or select the active schema"""
        if ' ' in self.schema or '.' in self.schema or \
           ';' in self.schema or '"' in self.schema or \
           "'" in self.schema:
            raise Exception('Bad schema name: %s' % self.schema)
        sql = 'SELECT 1 '\
              'FROM information_schema.schemata '\
              'WHERE schema_name=?'
        exists = self.get(sql, (self.schema,))
        if not exists:
            self.execute('CREATE SCHEMA "%s"' % self.schema)
        sql = 'SET search_path TO "%s",public' % self.schema
        self.execute(sql)

    def commit(self):
        if not self.no_exec:
            self.conn.commit()

    def debug_execute(self, sql, data=None, control=None):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        drop = self._do_i_skip(cur, control)
        if drop:
            cur.close()
            return

        if data is not None:
            sql = sql.replace('?', '%s')
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql+';', data)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';', data)
        else:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql+';')
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';')
            cur.close()

    def debug_get(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        if data is not None:
            sql = sql.replace('?', '%s')
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql+';', data)
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';', data)
        else:
            if 'CREATE' not in sql and 'DROP' not in sql:
                cur.execute('EXPLAIN '+sql+';')
                self.debug_data.add(tuple([sql]+cur.fetchall()))
            cur.execute(sql+';')
        try:
            return cur.fetchall()
        except:
            pass
        cur.close()

    def drop_tables(self, owner, prefix=''):
        """
        This will drop all tables from the given database using the db_name
        if db_class == POSTGRESQL.
        """
        sql = 'SELECT tablename '\
              'FROM pg_catalog.pg_tables '\
              'WHERE tableowner=? AND '\
                    'tablename LIKE ?'
        tables = self.get(sql, (owner, prefix+'%'))
        for table in tables:
            table = table[0]
            try:
                self.execute('DROP TABLE %s CASCADE' % table)
            except psycopg2.Error:
                self.logger.log_message('db', 'exception',
                                        'Failed to drop: %s' % table)


    def executemany(self, sql, data):
        cur = self.conn.cursor()
        sql = sql.replace('?', '%s')
        try:
            cur.executemany(sql, data)
        except:
            self.commit()
            raise
        finally:
            cur.close()

    def execute(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        drop = self._do_i_skip(cur, control)
        if drop:
            cur.close()
            return
        try:
            if data is not None:
                sql = sql.replace('?', '%s')
                cur.execute(sql+';', data)
            else:
                cur.execute(sql+';')
        except:
            self.commit()
            raise
        finally:
            cur.close()

    def get(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append([(sql, control), [data]])
            return
        cur = self.conn.cursor()
        try:
            if data is not None:
                sql = sql.replace('?', '%s')
                cur.execute(sql+';', data)
            else:
                cur.execute(sql+';')
            try:
                return cur.fetchall()
            except:
                return []
        except:
            self.commit()
            raise
        finally:
            cur.close()

    def open(self):
        self.opened = True
        if self.no_exec:
            return
        conn_err = 'could not connect to server: Connection refused'
        log_msg = 'Unable to connect to server at %s:%d (attempt %d/%d)'

        for i in range(POSTGRESQL_RETRIES):
            try:
                self.conn = psycopg2.connect(self.params)
                break
            except psycopg2.OperationalError, e:
                if not str(e).startswith(conn_err):
                    raise
                if (i+1) >= POSTGRESQL_RETRIES:
                    raise
                else:
                    self.logger.log_message('db', 'warning', log_msg % \
                                            (self.db_host, self.db_port, i+1,
                                             POSTGRESQL_RETRIES))
                    time.sleep(1)
        # schema settings are lost between connections
        self.set_schema()
        #ac = psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
        #self.conn.set_isolation_level(ac)

    def close(self):
        self.opened = False
        if self.no_exec:
            return
        try:
            self.conn.commit()
        except:
            pass
        try:
            self.conn.close()
        except psycopg2.InterfaceError:
            pass
        if DEBUG:
            for i in self.debug_data:
                self.logger.log_message('db', 'debug', '--------------------------')
                for j in i:
                    self.logger.log_message('db', 'debug', str(j))

    def table_exists(self, name):
        if self.no_exec:
            return True
        sql = "SELECT tablename FROM pg_catalog.pg_tables "\
              "WHERE tablename='%s' and schemaname='%s'"\
              % (name.lower(), self.schema)
        res = self.get(sql)
        return len(res) > 0

    def index_exists(self, name):
        if self.no_exec:
            return True
        sql = "SELECT indexname FROM pg_catalog.pg_indexes "\
              "WHERE indexname='%s' and schemaname='%s'"\
              % (name.lower(), self.schema)
        res = self.get(sql)
        return len(res) > 0

    def add_geom(self, table, geom_type):
        """
        Add a spatial column to the table

        table -- table to add the spatial column (geom) to
        geom_type -- type of the geometry (f.e. POLYGON)
        """
        sql = "SELECT AddGeometryColumn(?, 'geom', ?, ?, 2)"
        srid = self.spatial[0]
        self.execute(sql, (table, srid, geom_type))
        self.commit()

    def add_geom_index(self, table):
        """
        Adds a spatial index on the geometry column

        table -- table to add the spatial index to
        """
        i_l = self.conn.isolation_level
        self.conn.set_isolation_level(0)
        try:
            sql = 'CREATE INDEX %s_geom_index ON %s '\
                    'USING GIST (geom)' % (table, table)
            self.execute(sql)
            self.commit()
        finally:
            self.conn.set_isolation_level(i_l)
        return sql

    def vacuum_analyze(self, table=None):
        if self.no_exec:
            return
        self.conn.commit()
        i_l = self.conn.isolation_level
        self.conn.set_isolation_level(0)
        cur = self.conn.cursor()
        try:
            if not table:
                cur.execute('VACUUM ANALYZE;')
            else:
                cur.execute('VACUUM ANALYZE "%s"' % (table,))
        finally:
            cur.close()
            self.conn.commit()
        self.conn.set_isolation_level(i_l)

    def clear_store(self):
        self.store = []

    def _do_i_skip(self, cur, control):
        if control and control[0] == 'skip_existing':
            lev_name, uid, data_id = control[1]
            if data_id is None:
                check_sql = 'SELECT id FROM %s%s WHERE id=%%s LIMIT 1'\
                            % (self._add, lev_name)
                sql_data = (uid, )
            else:
                check_sql = 'SELECT id FROM %s%s WHERE id=%%s AND '\
                            'data_id=%%s LIMIT 1' % (self._add, lev_name)
                sql_data = (uid, data_id)
            cur = self.conn.cursor()
            cur.execute(check_sql, sql_data)
            res = cur.fetchone()
            cur.close()
            if res:
                return True
        return False


class SQLiteDB(DBInterface):
    def __init__(self, db_path='default.db', wipe=False, memory=False,
                 commit=True, spatial=None, no_exec=False, return_dict=False,
                 logger=None, add='', threaded=False):
        super(SQLiteDB, self).__init__()
        self.opened = False
        if logger is None:
            logger = SimpleLogger()
        self.logger = logger
        self.no_exec = no_exec
        self.return_dict = return_dict
        self.store = []
        if DEBUG:
            self.debug_data = set()
            self.get = self.debug_get
            self.execute = self.debug_execute
        self._commit = commit
        if not memory and not no_exec:
            if wipe:
                import os
                try:
                    os.remove(db_path)
                except OSError:
                    pass
            self.dbfile = db_path
        else:
            self.dbfile = ':memory:'
        self.db_name = self.dbfile
        if not no_exec:
            self.open()
        self.spatial = spatial
        if spatial and not no_exec:
            self.load_spatialite(True)
        self._add = add
        self._create_concurrently = ''

    def open(self):
        self.opened = True
        if self.no_exec:
            return
        # NB! check_same_thread set to false to enable concurrent access to
        # the same stem cache db, a bit risky, but...
        try:
            self.conn = sqlite3.connect(self.dbfile,
                  check_same_thread=False,
                  detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)
        except sqlite3.OperationalError, e:
            if str(e) == 'unable to open database file':
                msg = 'Unable to open database file "%s". Verify '\
                      'that it is in a folder that is writeable by you or, '\
                      'if the file exists, that the file is writable by you!' \
                      % (self.dbfile,)
                self.logger.log_message('SQLiteDB', 'error', msg)
            raise e
        self.conn.commit()
        orig_isolation = self.conn.isolation_level
        self.conn.isolation_level = None
        try:
            self.conn.execute('PRAGMA journal_mode=OFF')
            self.conn.execute('PRAGMA synchronous=OFF')
            self.conn.execute('PRAGMA cache_size=50000')
            self.conn.execute('PRAGMA temp_store=2')
        finally:
            self.conn.isolation_level = orig_isolation
        self.conn.commit()

    def commit(self):
        if not self.no_exec:
            self.conn.commit()

    def executemany(self, sql, data):
        self.conn.executemany(sql, data)

    def execute(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append(((sql, control), [data]))
            return
        drop = self._do_i_skip(control)
        if drop:
            return
        if data is not None:
            self.conn.execute(sql, data)
        else:
            self.conn.execute(sql)

    def debug_execute(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append(((sql, control), [data]))
            return
        drop = self._do_i_skip(control)
        if drop:
            return

        if data is not None:
            self.debug_data.add(tuple([sql]+\
                self.conn.execute('EXPLAIN QUERY PLAN '+sql, data).fetchall()))
            self.conn.execute(sql, data)
        else:
            self.debug_data.add(tuple([sql]+\
                self.conn.execute('EXPLAIN QUERY PLAN '+sql).fetchall()))
            self.conn.execute(sql)

    def get(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append(((sql, control), [data]))
            return
        if data is not None:
            return self.conn.execute(sql, data).fetchall()
        else:
            return self.conn.execute(sql).fetchall()

    def debug_get(self, sql, data=None, control=False):
        if self.no_exec:
            if len(self.store) > 0 and self.store[-1][0] == (sql, control):
                self.store[-1][1].append(data)
            else:
                self.store.append(((sql, control), [data]))
            return
        if data is not None:
            self.debug_data.add(tuple([sql]+\
                self.conn.execute('EXPLAIN QUERY PLAN '+sql, data).fetchall()))
            return self.conn.execute(sql, data).fetchall()
        else:
            self.debug_data.add(tuple([sql]+\
                self.conn.execute('EXPLAIN QUERY PLAN '+sql).fetchall()))
            return self.conn.execute(sql).fetchall()

    def close(self):
        self.opened = False
        if self.no_exec:
            return
        if self._commit:
            try:
                self.conn.commit()
            except sqlite3.ProgrammingError:
                pass
        self.conn.close()
        if DEBUG:
            for i in self.debug_data:
                self.logger.log_message('db', 'debug', '-'*50)
                for j in i:
                    self.logger.log_message('db', 'debug', str(j))

    def enable_load_extension(self, bool):
        if self.no_exec:
            return
        self.conn.enable_load_extension(bool)

    def load_extension(self, absfile):
        if self.no_exec:
            return
        self.conn.load_extension(absfile)

    def executescript(self, script):
        if self.no_exec:
            return
        self.conn.executescript(script)

    def table_exists(self, name):
        if self.no_exec:
            return True
        return len(self.get("SELECT name FROM sqlite_master "\
                            "WHERE type='table' AND name=?", (name,))) > 0

    def index_exists(self, name):
        if self.no_exec:
            return True
        return len(self.get("SELECT name FROM sqlite_master "\
                            "WHERE type='index' AND name=?", (name,))) > 0

    def add_geom(self, table, geom_type):
        """
        Add a spatial column to the table

        table -- table to add the spatial column (geom) to
        geom_type -- type of the geometry (f.e. POLYGON)
        srid -- SRID the data is in
        dim_info -- not used in postgresql
        """
        sql = "SELECT AddGeometryColumn(?, 'geom', ?, ?, 2)"
        srid = self.spatial[0]
        self.execute(sql, (table, srid, geom_type))

    def add_geom_index(self, table):
        """
        Adds a spatial index on the geometry column

        table -- table to add the spatial index to
        """
        sql = "SELECT CreateSpatialIndex(?, 'geom')"
        self.execute(sql, (table,))
        return sql

    def load_spatialite(self, first=False):
        """
        Get spatialite up and running
        """
        if self.no_exec:
            return
        self.enable_load_extension(True)
        if sys.platform.startswith('darwin'):
            libfile = 'libspatialite.dylib'
        elif sys.platform.startswith('linux'):
            libfile = 'libspatialite.so'
        else:
            libfile = 'libspatialite-2.dll'
        absfile = os.path.join(filepath, libfile)
        try: #try to load the buildout file
            self.load_extension(absfile)
        except Exception, e: #fallback to system lib
            if not sys.platform.startswith('win'):
                self.logger.log_message('db', 'warning',
                                        'Failed to load spatialite (%s), '\
                                        'falling back to system library' \
                                        % (str(e),))
            self.load_extension(libfile)
        if first:
            try:
                self.execute('SELECT count(*) FROM spatial_ref_sys')
            except sqlite3.OperationalError, e:
                if str(e) != 'no such table: spatial_ref_sys':
                    raise
                self.execute('SELECT InitSpatialMetaData()')


    def vacuum_analyze(self, table=None):
        if self.no_exec:
            return
        self.conn.commit()
        orig_isolation = self.conn.isolation_level
        self.conn.isolation_level = None
        try:
            if not table:
                self.execute('VACUUM ANALYZE;')
            else:
                self.execute('VACUUM ANALYZE "%s"', table)
            self.conn.commit()
        finally:
            self.conn.isolation_level = orig_isolation

    def clear_store(self):
        del self.store
        self.store = []

    def _do_i_skip(self, control):
        if control and control[0] == 'skip_existing':
            lev_name, uid, data_id = control[1]
            if data_id is None:
                check_sql = 'SELECT id FROM %s%s WHERE id=? LIMIT 1'\
                            % (self._add, lev_name)
                sql_data = (uid, )
            else:
                check_sql = 'SELECT id FROM %s%s WHERE id=? AND '\
                            'data_id=? LIMIT 1' % (self._add, lev_name)
                sql_data = (uid, data_id)
            res = self.conn.execute(check_sql, sql_data).fetchone()
            if res:
                return True
        return False


class CoreDB(object):
    def __init__(self, db_name='default.db', wipe=False,
                 memory=False, db_host='localhost', db_port=5432,
                 user='test', pw='test', db_class='SQLITE', spatial=None,
                 no_exec=False, return_dict=False, db_database='SIMO',
                 db_schema='', logger=None, threaded=False, unlogged=False,
                 force_prefix=False):
        super(CoreDB, self).__init__()
        self.logger = logger
        self.db_name = db_name
        self.db_class = db_class
        self._add = ''
        self.unlogged = ''
        self.threaded = threaded
        self.no_exec = no_exec
        self.return_dict = return_dict
        self.static_ids_written = {}
        self.data_id_store = defaultdict(list)
        self.op_id_store = defaultdict(list)
        self.binfo_store = defaultdict(list)
        self.indices = defaultdict(dict)
        if db_class == 'SQLITE':
            if (memory or force_prefix) and db_name != 'default.db':
                self._add = os.path.basename(db_name).replace('.', '_') + '_'
            self.db = SQLiteDB(db_name, wipe, memory, spatial=spatial,
                               no_exec=no_exec, return_dict=return_dict,
                               logger=logger, add=self._add, threaded=threaded)
        elif db_class == 'POSTGRESQL':
            self._add = db_name + '_'
            self.db = PostgreSQL(db_name, db_host, db_port, user, pw,
                                 wipe, spatial, no_exec=no_exec,
                                 return_dict=return_dict,
                                 db_database=db_database, db_schema=db_schema,
                                 logger=logger, add=self._add,
                                 threaded=threaded, unlogged=unlogged)
        elif db_class == 'ORACLE':
            self._add = db_name + '_'
            self.db = Oracle(db_name, db_host, user, pw,
                             wipe=wipe, spatial=spatial,
                             no_exec=no_exec, return_dict=return_dict,
                             logger=logger, add=self._add,
                             threaded=threaded)
        else:
            raise Exception('db_class not in (SQLITE, POSTGRESQL)')
        self.open = self.db.open
        self.close = self.db.close
        if self.logger is None:
            self.logger = self.db.logger


    def _create_any_trigger(self, parent, child, parent_keys, child_keys):
        parent = self._add + parent
        child = self._add + child
        links = '\n    AND '.join(['%s = OLD.%s' % (ck, pk) \
                                   for ck, pk in zip(child_keys, parent_keys)])
        if self.db_class == 'POSTGRESQL':
            exists = self.db.get('SELECT count(*) '\
                                 'FROM pg_trigger '\
                                 'WHERE tgname=?',
                                 ('on_delete_%s_to_%s' % (parent, child),))
            if exists[0][0]:
                return
            sql = """
            BEGIN WORK;
            LOCK TABLE pg_proc IN EXCLUSIVE MODE;
            CREATE OR REPLACE FUNCTION delete_%s_to_%s() RETURNS trigger AS $$
            DECLARE
                -- nothing to declare
            BEGIN
                DELETE FROM %s WHERE %s;
                RETURN OLD;
            END;
            $$ LANGUAGE plpgsql;
            COMMIT WORK;""" % (parent, child, child, links)
            try:
                self.db.execute(sql)
            except:
                pass
            try:
                self.db.execute('BEGIN WORK;\n' \
                                'LOCK TABLE pg_trigger ' \
                                'IN EXCLUSIVE MODE;\n' \
                                'CREATE TRIGGER on_delete_%s_to_%s ' \
                                'BEFORE DELETE ON %s ' \
                                'FOR EACH ROW ' \
                                'EXECUTE PROCEDURE delete_%s_to_%s();\n' \
                                'COMMIT WORK;' \
                                % (parent, child, parent, parent, child))
            except:
                pass
        elif self.db_class == 'SQLITE' or self.db_class == 'ORACLE':
            sql = 'CREATE TRIGGER IF NOT EXISTS on_delete_%s_to_%s\n'\
                  'BEFORE DELETE ON %s\n'\
                  'FOR EACH ROW BEGIN\n'\
                  '    DELETE FROM %s\n'\
                  '    WHERE %s;\n'\
                  'END;'\
                  % (parent, child,
                     parent, child, links)
            if self.db_class == 'ORACLE':
                sql = 'CREATE OR REPLACE TRIGGER on_delete_%s\n'\
                      'BEFORE DELETE ON %s\n'\
                      'FOR EACH ROW BEGIN\n'\
                      '    DELETE FROM %s\n'\
                      '    WHERE %s;\n'\
                      'END;'\
                      % (parent.replace('operation', 'op', 1),
                         parent, child, links)
                sql = sql.replace('OLD.', ':old.')
            self.db.execute(sql)
        self.db.commit()

###############################################################################

class DataDB(CoreDB):
    def __init__(self, db_type, sim_level, content_def, hierarchy,
                 level_meta, opres_def, cf_cfiers, logger,
                 db_name='default.db', make_new_db=False, memory=False,
                 spatial=None, constraints=None,
                 db_host='localhost', db_port=5432, user='test',
                 pw='test', db_class='SQLITE', create_branch_desc=False,
                 track_solution=False, no_exec=False, return_dict=False,
                 db_database='SIMO', db_schema='', threaded=False,
                 unlogged=False, force_prefix=False):
        """
        If make_new_db == True, the old database will be destroyed
        db_class in (SQLITE, POSTGRESQL)
        """
        super(DataDB, self).__init__(db_name, make_new_db, memory, db_host,
                                     db_port, user, pw, db_class, spatial,
                                     no_exec, return_dict, db_database,
                                     db_schema, logger, threaded, unlogged,
                                     force_prefix)
        self.db_type = db_type
        self.sim_ordinal = sim_level[0]
        if spatial is not None:
            self.is_spatial = True
        else:
            self.is_spatial = False
        self.logger = logger
        self._complained_about = set([])
        self.content_def = content_def
        self.hierarchy = self._get_constrained_hierarchy(hierarchy,
                                                         constraints)
        self.opres_def = list(opres_def)
        self._oplink_insert_sql = ''
        self._opres_insert_sql = ''
        if not self.no_exec:
            self._create_meta_tables()
            # ensure meta tables are up to date
            self._set_meta_tables(level_meta)
            self._add_branching_table()
        self.parents = set([])
        self.ind2date = {}
        self.has_geom = {}
        self.old_keys = {}
        self.price_tracker = set([])
        self.table_def = {}
        self._level_name = {}
        self._level_ind = {}
        self._id_iter_branch = {}
        self._iter_branch = set([])
        self._constraints = constraints
        self._all_text_vars = set([])
        self.create_branch_desc = create_branch_desc
        for defkey in content_def.keys():
            self._level_name[defkey[1]] = defkey[0]
            self._level_ind[defkey[0]] = defkey[1]
        self._main_level = self._level_ind[sim_level[1]]

        order = []
        for key, val in hierarchy.items():
            order.append((val['ordinal'], key))
        order.sort()
        order = [o[1] for o in order]
        self.level_order = order
        if not self.no_exec:
            self._create_data_link()
        for item in order:
            for defkey in content_def.keys():
                if defkey[1] != item:
                    continue
                # table for this level generated at all?
                if constraints is not None and defkey[1] > 0 and \
                    defkey[1] not in constraints:
                    continue
                self._create_tabledef(defkey, levels=level_meta)
                if not self.no_exec:
                    self._create_table(defkey[0])
        if not no_exec:

            #NB! cascading delete triggers are notorious for performance on
            # big dbs
            # cascading batch delete implemented in drop_id(s) instead

            #for lname, tdef in self.table_def.items():
            #    self._create_trigger(lname, tdef)
            self._create_op_res_table(self.opres_def)
            #self._create_any_trigger('op_link', 'op_res',
            #                         ('op_id',), ('op_id',))
            self._create_timber_price_table(cf_cfiers)
            if create_branch_desc:
                self._create_branch_desc_table()
            if track_solution:
                self._create_optimal_table()

    def add_data_from_matrix(self, ddate, matrix, ind2id, links, blocked,
                             terminated, main_level, text_data, update=False,
                             start_iter=0):
        """
        From the base level down, writes the data from the matrix using the
        matrix index mapping stored in the table def

        NB! in update mode the geometry and text attribute content is lost
        below base level. When not updating, they are lost from all levels
        """
        links = self._constrain_links(links)
        self._update = update
        self._main_level = main_level
        if update:
            self._remove_old(links, ind2id.ids, blocked, terminated)
        for iter_branch, self._relatives in links.items():
            if self._main_level not in self._relatives[(0, 0)]:
                # just an empty branch, nothing to do here
                continue
            orig_iter, branch = iter_branch
            iter_branch = (start_iter + orig_iter, branch)
            self._orig_iter = int(orig_iter)
            self._iterr = int(iter_branch[0])
            self._branch = int(iter_branch[1])
            self._ids = ind2id.ids[(orig_iter, self._branch)]
            self._inds = ind2id.inds[(orig_iter, self._branch)]
            base_obj = self._relatives[(0,0)][self._main_level]
            for oind in base_obj:
                bind = (orig_iter, branch, oind)
                if bind not in blocked:
                    self._write_matrix_obj(oind, ddate, matrix, text_data)
        #self.db.commit()

    def add_opres(self, levelname, data):
        """
        data is a list of dictionaries, each containing date, id, iteration,
        branch, values, op_name and op_group. values is a list of dictionaries
        containing variable, value pairs [{var: val, ...}, ...]
        """
        add = self._add
        errs = []
        hashes = set([])
        for item in data:
            obj_id = item['id']
            orig_op_id = item['op_id']
            iteration = item['iteration']
            branch = item['branch']
            ddate = item['date']
            op_name = item['op_name']
            op_group = item['op_group']
            op_type = item['op_type']
            notes = item.get('notes', '')
            materials = item.get('materials', list())
            counter = 0
            unique_hash = False
            while not unique_hash:
                op_hash = self._get_op_hash(obj_id, iteration, branch, ddate,
                                            op_name, counter)
                if op_hash not in hashes:
                    unique_hash = True
                hashes.add(op_hash)
                counter += 1
            if counter > 1:
                msg = "More than one %s operation for the same year" % op_name
                self.logger.log_message('db', 'warning', msg)
            op_id = self._encode_hash(op_hash)
            if not orig_op_id:
                orig_op_id = op_id
            # first the object, iteration, branch, date, op_id data table
            row_sql = 'INSERT INTO %sop_link (id, iteration, branch, '\
                      'op_date, op_id, o_op_id, op_level, op_name, op_group, '\
                      'op_type) '\
                      'VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)'
            row_sql = row_sql % add
            row_data = (obj_id, int(iteration), int(branch),
                        ddate, op_id, orig_op_id, levelname,
                        op_name, op_group, op_type)
            op_id_key = (levelname, obj_id, int(iteration), int(branch))
            op_data = (ddate, op_id, orig_op_id, op_name, op_group, op_type)
            self.op_id_store[(op_id_key)].append(op_data)
            try:
                self.db.execute(row_sql, row_data)
            except sqlite3.IntegrityError:
                err = 'Operation already in db: %s, %s, %s' \
                        % (obj_id, ddate, op_name)
                raise Exception(err)
            except psycopg2.IntegrityError:
                err = 'Operation already in db: %s, %s, %s' \
                        % (obj_id, ddate, op_name)
                raise Exception(err)

            # Insert notes (if any)
            if notes:
                note_sql = 'INSERT INTO %sop_note (op_id, notes) VALUES (?, ?)'
                note_sql = note_sql % add
                self.db.execute(note_sql, (op_id, notes))

            # Insert materials (if any)
            res_sql = 'INSERT INTO %sop_material (op_id, material) ' \
                      'VALUES (?, ?)' % add
            for material in materials:
                self.db.execute(res_sql, (op_id, material))

            # and then the actual operation results
            for values in item['values']:
                vals, d_date = values
                for v_dict in vals:
                    tuples = v_dict.items()
                    for i, t in enumerate(tuples):
                        if t[0] != 'cash_flow':
                            tuples[i] = ('"%s"' % t[0], t[1])

                    sql = 'INSERT INTO %sop_res (op_id' % add
                    data = []
                    for var, val in tuples:
                        sql += ', %s' % (var, )
                        if var.strip('"') in self._all_text_vars:
                            data.append(val)
                        else:
                            data.append(float(val))
                    sql += ') VALUES (' + \
                           ', '.join(['?'] * (len(tuples) + 1)) +')'
                    data = tuple([op_id] + data)
                    self.db.execute(sql, data)
        #self.db.commit()
        return errs


    def add_data_from_dictionary(self, data, iterr, branch, update=False):
        """
        For data levels in the dictionary writes the data for the given dates
        into level tables. If is an update, old data is removed first.
        """
        add = self._add
        lnames = data.keys()
        sorted = []
        for lname in lnames:
            lind = self._level_ind[lname]
            my_ordinal = self.hierarchy[lind]['ordinal']
            sorted.append((my_ordinal, lind, lname))
        sorted.sort()
        dli_sql = DATA_LINK_INSERT_SQL % add
        llname = add + lname
        do_skip = False
        did_cache = {}
        for my_ordinal, lind, lname in sorted:
            leveldata = data[lname]
            if my_ordinal == self.sim_ordinal:
                create_data_id = True
            else:
                create_data_id = False
                if my_ordinal < self.sim_ordinal:
                    no_data_id = True
                    do_skip = True
                else:
                    no_data_id = False

            # insert each date worths of data
            for ddate, info in leveldata:
                if isinstance(ddate, datetime.datetime):
                    ddate = datetime.date(year=ddate.year, month=ddate.month,
                                          day=ddate.day)
                if create_data_id:
                    data_hash = self._get_data_hash(lind, info['id'], iterr,
                                                    branch, ddate)
                    data_id = self._encode_hash(data_hash)
                    did_key = (lind, info['id'], iterr, branch, ddate)
                    did_cache[did_key] = data_id
                    if not update:
                        sql_data = (lname, info['id'], iterr, branch, ddate,
                                    data_id)
                        self.db.execute(dli_sql, sql_data)
                        did_key = (lname, info['id'], iterr, branch)
                        self.data_id_store[did_key].append((ddate, data_id))
                elif no_data_id:
                    data_id = None
                    ddate = None
                else:
                    if not 'parent level' in info:
                        plind = self.hierarchy[lind]['parent']
                    else:
                        plind = info['parent level']
                    pid = info['parent id']
                    did_key = (plind, pid, iterr, branch, ddate)
                    if did_key in did_cache:
                        data_id = did_cache[did_key]
                    else:
                        data_id = self._get_data_id_from_parent(plind, pid,
                                                       iterr, branch, ddate)
                    did_key = (lind, info['id'], iterr, branch, ddate)
                    did_cache[did_key] = data_id
                    if not data_id:
                        err_msg = "No parent %s (on level %s) found for "\
                                "object %s on level %s"\
                                % (pid, plind, info['oid'], lname)
                        self.logger.log_message('db', 'error', err_msg)
                    ddate = None
                sql_key_data = [data_id, info['id'], info['oid'],
                                info['parent id']]

                # other attributes for updates
                if update:
                    other_attr = self.table_def[lname]['other attr name']
                    oa = ', '.join(['"' + attr + '"' for attr in other_attr])
                    oa_sql = 'SELECT %s FROM %s WHERE data_id=?' % (oa, llname)
                    geom_sql = 'SELECT AsText(geom), SRID(geom) '\
                               'FROM %s WHERE data_id=?' \
                               % llname
                    # store geom + text data that's not in data dictionary
                    if len(other_attr)>0:
                        other_val = self.db.get(oa_sql, (data_id,))[0]
                    else:
                        other_val = None
                    if self.table_def[lname]['geom type'] is not None:
                        geom = self.db.get(geom_sql, (data_id,))[0]
                        self._geom = geom

                # put data into db
                vals, geom = self._map_attr_to_table(lname, info['values'])
                if update:
                    sql = self.table_def[lname]['update sql']
                    sql_data = vals + [data_id, info['id']]
                elif geom is not None and self.db_class != 'SQLITE':
                    sql = self.table_def[lname]['geom insert sql']
                    sql_data = sql_key_data + vals + [geom, self.db.spatial[0]]
                else:
                    sql = self.table_def[lname]['insert sql']
                    sql_data = sql_key_data + vals
                ctrl = None
                if do_skip:
                    ctrl = ['skip_existing', (lname, info['id'], data_id)]
                self.db.execute(sql, sql_data, ctrl)

                if geom is not None and (self.db_class == 'SQLITE' or update):
                    sql = self.table_def[lname]['geom update sql']
                    if self.db_class in ('SQLITE', 'ORACLE'):
                        sql_data = (geom[0], geom[1], data_id, info['id'])
                    elif self.db_class == 'POSTGRESQL':
                        sql_data = (geom[0], data_id, info['id'])
                    self.db.execute(sql, sql_data)

                # put other data back in case of update
                if update and other_val:
                    oa = self.table_def[lname]['other attr name']
                    pairs = []
                    for ind in range(len(oa)):
                        if other_val[ind] is None:
                            val = 'Null'
                        else:
                            val = "'" + other_val[ind] + "'"
                        pairs.append('"'+oa[ind]+'"' + '=' + val)
                    other_attr = ', '.join(pairs)
                    sql = 'UPDATE %s SET %s WHERE data_id=? AND id=?' \
                            % (llname, other_attr)
                    self.db.execute(sql, (data_id, info['id']))
        #self.db.commit()

    def _get_data_id_from_parent(self, plind, pid, iterr, branch, ddate):
        """
        Queries the data_id from the parent, resolving the correct one using
        iteration, branch and date if several data_ids are found

        plind -- parent level index
        pid -- parent object id
        iterr -- iteration
        branch -- branch
        ddate -- data date
        """
        add = self._add
        data_id = None
        did_sql = 'SELECT data_id FROM %s WHERE id=?'
        plname = add + self._level_name[plind]
        res = self.db.get(did_sql % plname, (pid,))
        if len(res) > 1:
            dids = "', '".join([v[0] for v in res])
            sql = "SELECT data_date, data_id FROM %sdata_link "\
                    "WHERE iteration=? AND branch=? AND data_id IN ('%s')"\
                    % (add, dids)
            ddates = self.db.get(sql, (iterr, branch))
            for hit in ddates:
                if hit[0] == ddate:
                    data_id = hit[1]
        elif res:
            data_id = res[0][0]
        return data_id

    def add_branching_info(self, lind, binfo, ind2id, start_iter=0):
        """
        Add branching info to db
        """
        try:
            for key, val in binfo.items():
                iter_ind, child_branch, obj_ind = key
                par_branch, op_grp, op, bname, bgname, bdate = val
                uid, oid = ind2id[(iter_ind, par_branch)][(lind, obj_ind)]
                sql = 'INSERT INTO %sBRANCHING_INFO VALUES '\
                      '(?, ?, ?, ?, ?, ?, ?, ?, ?)' % self._add
                data = (uid, int(start_iter + iter_ind), int(par_branch),
                        int(child_branch), bdate, op_grp, op, bname, bgname)
                self.db.execute(sql, data)
                self.binfo_store[uid].append(data[1:])

                # if to and from branches are the same, no need to copy data
                # as the to branch has already been created in the simulation
                if int(child_branch) == int(par_branch):
                    continue
                self._duplicate_old_branch(uid, lind,
                                           int(start_iter+iter_ind),
                                           int(par_branch), int(child_branch),
                                           bdate)
        except sqlite3.Error, e:
            err_msg = ' '.join(("An error occurred with branching:",
                                 e.args[0]))
            self.logger.log_message('db', 'error', err_msg)
        except psycopg2.Error, e:
            err_msg = ' '.join(("An error occurred with branching:",
                                 e.args[0]))
            self.logger.log_message('db', 'error', err_msg)
        #self.db.commit()

    def clear_data(self):
        """
        Empties all level tables
        """
        add = self._add
        for defkey in self.content_def.keys():
            level_name = add + defkey[0]
            sql = "DELETE FROM %s" % level_name
            try:
                self.db.execute(sql)
            except sqlite3.OperationalError:
                pass
            except psycopg2.ProgrammingError:
                pass
        sql = "DELETE FROM %sdata_link" % add
        self.db.execute(sql)
        sql = "DELETE FROM %sop_res" % add
        self.db.execute(sql)
        sql = "DELETE FROM %sop_link" % add
        self.db.execute(sql)
        sql = "DELETE FROM %sbranching_info" % add
        self.db.execute(sql)
        if self.db.table_exists('%sbranch_desc' % add):
            sql = "DELETE FROM %sbranch_desc" % add
            self.db.execute(sql)
        self.db.commit()
        self.db.vacuum_analyze()

    def copy_to_db(self, key_list, main_level_name, db, iteration,
                   skip_weight_splits=None):
        """
        For the given keys of (iteration, object id, branch) copies the
        data to a new db from the given level. Level is taken as base level,
        below which all the children from different levels are copied as well.

        key_list -- list of (iteration, uid, orig_uid, branch, ...) tuples
        main_level_name -- main (simulation) level name
        db -- result (target) database object
        iteration -- current iteration
        skip_weight_splits -- numerical columns which should NOT be multiplied
                              with the split weights
        """
        # If we have split some stands (using LP solver), we need to weight
        # the numerical operation results using the split weights. However,
        # some of the numerical results can be in reality categorical, such as
        # tree species or timber assortment. User must define these in the
        # optimization ini file, and we must skip the weighting of these.
        weight_columns = None
        if skip_weight_splits:
            # Get all columns from operation result table and drop the ones
            # that are in skip_weight_splits listing
            weight_columns = []
            for column in self._get_table_columns('op_res'):
                do_weight = True
                if column in skip_weight_splits:
                    do_weight = False
                weight_columns.append(do_weight)
        add = self._add
        add2 = db._add
        level_order = [(val['ordinal'], self._level_name[key]) \
                       for key, val in self.hierarchy.items()]
        level_order.sort()
        for lev in level_order:
            if lev[0] < self.sim_ordinal and iteration == 0:
                # copy static level data only during first iteration
                self._copy_level_data(db, lev[1])
        ops_copy = set([])
        for keys in key_list:
            iterr, main_uid, orig_uid, branch, weight, copy_only_ops = keys
            iterr = int(iterr)
            branch = int(branch)
            sql = 'SELECT DISTINCT data_id FROM %sdata_link '\
                  'WHERE data_level=? AND id=? AND iteration=? AND branch=?'\
                  % (add,)
            query_data = (main_level_name, main_uid, iterr, branch)
            data_ids = self.db.get(sql, query_data)
            data_ids = "', '".join([v[0] for v in data_ids])
            sql = "SELECT * FROM %sdata_link "\
                  "WHERE data_id IN ('%s') AND iteration=? AND branch=?"\
                   % (add, data_ids)
            insert_sql = 'INSERT INTO %sdata_link VALUES (?, ?, ?, ?, ?, ?)'\
                          % add2
            res = self.db.get(sql, (iterr, branch))
            for row in res:
                db.db.execute(insert_sql, row)
            # replicate level tables
            for lev in level_order:
                if lev[0] >= self.sim_ordinal:
                    if not copy_only_ops:
                        self._copy_level_data(db, lev[1], data_ids)
            if not main_uid in ops_copy:
                if copy_only_ops:
                    main_uid = orig_uid
                self._copy_operation_data(\
                    db, main_uid, iterr, branch, weight=weight,
                    weight_columns=weight_columns)
                ops_copy.add(main_uid)
        self._copy_timber_prices(db)
        self.db.commit()

    def drop_id(self, cid):
        """
        This method will drop any and all data to the given id from the
        database.

        cid -- main level id to remove
        """
        if self.no_exec:
            self.db.execute(None, control=('drop_ids', (cid,)))
            return
        add = self._add
        # get data_ids to remove
        sql = 'SELECT DISTINCT data_id FROM %sdata_link WHERE id=?' % (add, )
        res = self.db.get(sql, (cid, ))
        self._delete_with_data_id(res)

        self.db.execute('DELETE FROM %sdata_link WHERE id=?' % add, (cid,))

        # get op_ids to remove
        sql = 'SELECT DISTINCT op_id FROM %sop_link WHERE id=?' % (add, )
        res = self.db.get(sql, (cid, ))
        self._delete_with_op_id(res)

        self.db.execute('DELETE FROM %sop_link WHERE id=?' % add, (cid,))

        if self.db.table_exists('%sBRANCHING_INFO' % add):
            sql = 'DELETE FROM %sBRANCHING_INFO WHERE id=?' % add
            self.db.execute(sql, (cid,))

        if self.db.table_exists('%sbranch_desc' % add):
            sql = 'DELETE FROM %sbranch_desc WHERE id=?' % add
            self.db.execute(sql, (cid,))

        if self.db.table_exists('%soptimal' % add):
            sql = 'DELETE FROM %soptimal WHERE id=?' % add
            self.db.execute(sql, (cid,))

        self.db.commit()

    def drop_ids(self, all_ids):
        """
        This method will drop any and all data to the given id from the
        database (in batches of 1000 for oracle compatibility).
        """
        if self.no_exec:
            self.db.execute(None, control=('drop_ids', all_ids))
            return
        all_ids = tuple(all_ids)
        add = self._add
        sets = int(math.ceil(len(all_ids)/1000.0))
        for i in range(sets):
            ids = all_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            id_str = ', '.join('?'*len(ids))

            # get data_ids to remove
            sql = 'SELECT DISTINCT data_id FROM %sdata_link WHERE id in (%s)'\
                    % (add, id_str)
            res = self.db.get(sql, ids)
            self._delete_with_data_id(res)

            self.db.execute('DELETE FROM %sdata_link WHERE id in (%s)' \
                            % (add, id_str), ids)

            # get op_ids to remove
            sql = 'SELECT op_id FROM %sop_link WHERE id in (%s)'\
                    % (add, id_str)
            res = self.db.get(sql, ids)
            self._delete_with_op_id(res)

            self.db.execute('DELETE FROM %sop_link WHERE id in (%s)' \
                            % (add, id_str), ids)

            if self.db.table_exists('%sBRANCHING_INFO' % add):
                sql = 'DELETE FROM %sBRANCHING_INFO WHERE id in (%s)' \
                      % (add, id_str)
                self.db.execute(sql, ids)

            if self.db.table_exists('%sbranch_desc' % add):
                sql = 'DELETE FROM %sbranch_desc WHERE id in (%s)' \
                      % (add, id_str)
                self.db.execute(sql, ids)

            if self.db.table_exists('%soptimal' % add):
                sql = 'DELETE FROM %soptimal WHERE id in (%s)' % (add, id_str)
                self.db.execute(sql, ids)

        self.db.commit()

    def drop_data_ids(self, data_ids):
        """
        This will drop all data from data levels matching the given data ids.
        """
        data_ids = tuple(data_ids)
        self._delete_with_data_id([(d,) for d in data_ids])
        add = self._add
        sets = int(math.ceil(len(data_ids)/1000.0))
        for i in range(sets):
            ids = data_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            id_str = ', '.join('?'*len(ids))

            # get data_ids to remove
            sql = 'DELETE FROM %sdata_link WHERE data_id in (%s)'\
                    % (add, id_str)
            self.db.get(sql, ids)
        self.db.commit()

    def drop_op_ids(self, op_ids):
        """
        This will drop all ops from operations matching the given op ids.
        """
        op_ids = tuple(op_ids)
        self._delete_with_op_id([(o,) for o in op_ids])
        add = self._add
        sets = int(math.ceil(len(op_ids)/1000.0))
        for i in range(sets):
            ids = op_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            id_str = ', '.join('?'*len(ids))

            # get op_ids to remove
            sql = 'DELETE FROM %sop_link WHERE op_id in (%s)'\
                    % (add, id_str)
            self.db.get(sql, ids)
        self.db.commit()

    def drop_indices(self, ind_types):
        """
        Drop indices of the given index types from the db

        ind_types -- index types ('link', 'branching', 'price', 'ops', 'geom')
        """
        for i_t in ind_types:
            for index in self.indices[i_t]:
                if self.db.index_exists(index):
                    sql = 'DROP INDEX %s' % index
                    self.db.execute(sql)
        self.db.commit()

    def enable_indices(self, ind_types):
        """
        Recreate indices of the given index types to the db

        ind_types -- index types ('link', 'meta', 'geom', 'ops')
        """
        for i_t in ind_types:
            for name, sql in self.indices[i_t].items():
                if not self.db.index_exists(name):
                    self.db.execute(sql)
        self.db.commit()

    def fill_branch_desc_table(self, ids, start_iter=0):
        """
        Fill out the branch description table
        """
        add = self._add
        for uid in ids:
            data = self.binfo_store[uid]
            for row in data:
                iterr, from_b, to_b, bdate, op_grp, op, bname, bgname = row
                if iterr >= start_iter:
                    desc = self._get_branch_desc(uid, iterr, to_b)
                sql = 'INSERT INTO %sbranch_desc VALUES (?, ?, ?, ?, ?)' % add
                data = (iterr, uid, to_b, desc, bgname)
                self.db.execute(sql, data)

    def fill_dictionary(self, lname, uid):
        """
        Returns a dictionary in the format that add_data_from_dictionary
        uses, containing all the iterations, branches and dates of the given
        level and oid
        datadict = {levelname: [(ddate, {'id':id, 'parent id': pid,
                                            'values': [('key', value),
                                                       ('key', value)]}),
                                ...],
                      ...}
        """
        add = self._add
        res = None
        sql = 'SELECT * FROM %sdata_link WHERE data_level=? AND id=?' % add
        val_sql = 'SELECT * FROM %s%s WHERE data_id=?' % (add, lname)
        link_data = self.db.get(sql, (lname, uid))
        res = {lname:[]}
        for row in link_data:
            data_id = row[DL_DATA_ID_COL]
            iterr = row[DL_ITER_COL]
            branch = row[DL_BRANCH_COL]
            ddate = row[DL_DATE_COL]
            data = self._get_star_data(lname, val_sql, (data_id, ))
            if data != []:
                attrs = self.table_def[lname]['float attr name']
                to_remove = self.table_def[lname]['to remove']
                for row in data:
                    oid = row[ORIG_ID_COL]
                    pid = row[PID_COL]
                    vals = list(row[DATA_COL:])
                    for ind in to_remove:
                        vals.pop(ind)
                    tuples = zip(attrs, vals)
                    entry = (ddate,
                             {'id': uid, 'oid':oid, 'parent id': pid,
                              'values': tuples, 'iteration': iterr,
                              'branch': branch})
                    res[lname].append(entry)
        return res

    def fill_matrix(self, handler, lind, full_ids, iterr=0, branch=0):
        """
        Fills a data matrix with data for the objects identified by the level
        and a set of object ids (unique and original). The matrix is filled
        for all data levels and objects linked to from the given level and
        ids in top down order; i.e., simulation level first.
        All lineages are filled as well.
        """
        add = self._add
        self._handler = handler
        self._iterr = iterr
        self._branch = branch
        try:
            my_lineage = self.hierarchy[lind]['lineage']
            self._filled_parents = {}
            lname = self._level_name[lind]
            sql = 'SELECT data_id, data_date FROM %sdata_link '\
                  'WHERE data_level=? AND id=? '\
                    'AND iteration=? AND branch=?' % add
            for obj_id in full_ids:
                self._no_data_levels = set([])
                self._data_exists = set([])
                sql_data = (lname, obj_id[0], iterr, branch)
                # only a single data_id is expected to exist for the object
                result = self.db.get(sql, sql_data)[0]
                data_id = result[0]
                ddate = result[1]
                # ensure we have a date object, not datetime object
                ddate = datetime.date(ddate.year, ddate.month, ddate.day)
                tind = self._fill_parents(lind, obj_id, data_id, ddate,
                                          my_lineage)
                tind = self._fill_sim_level(tind, lind, obj_id, data_id, ddate)
                self._fill_children(tind, lind, obj_id, data_id, ddate,
                                    my_lineage)
        finally:
            del self._handler

    def get_all_child_levels(self, lind):
        """Get all the child levels below current level"""
        return self._get_all_child_levels(self.hierarchy[lind])

    def get_branching_info(self, id, iterr, to_branch=None, ddate=None):
        add = self._add
        iterr = int(iterr)
        sql = 'SELECT * FROM %sBRANCHING_INFO WHERE id=? AND iteration=?' % add
        keys = [id, iterr]
        if to_branch is not None:
            to_branch = int(to_branch)
            sql += ' AND to_branch=?'
            keys.append(to_branch)
        if ddate is not None:
            sql += ' AND data_date=?'
            keys.append(ddate)
        data = self.db.get(sql, keys)
        return data

    def get_child_ids(self, lind, oid, iterr=0, branch=0, ddate=None,
                      child_levs=None, data_id=None):
        """
        Recursively gets child ids for the given object and optionally for the
        given date
        """
        self._iterr = int(iterr)
        self._branch = int(branch)
        ids = []
        ids = self._get_children_id(lind, oid, ids, ddate=ddate,
                                    child_levs=child_levs, data_id=data_id)
        result = defaultdict(list)
        for lname, cid in ids:
            lind = self._level_ind[lname]
            result[lind].append(cid)
        return result

    def get_child_levels(self, lind):
        return self._get_child_levels(self.hierarchy[lind])

    def get_raw_data_from_levels(self, level_headers, required=None,
                                 distinct=False):
        """level_headers and required are lists of level, variable tuples
        (local copies are made to avoid influencing caller.)"""
        level_headers = [l for l in level_headers]
        required = [r for r in required]
        add = self._add
        if len(level_headers) == 0:
            return list([])
        if distinct:
            distinct = 'DISTINCT '
        else:
            distinct = ''
        requirestr = ''
        requirestr = ''
        if required is not None:
            for i, (l, v) in enumerate(required):
                if l == 'data_link':
                    if v in ('id', 'data_id', 'iteration', 'branch',
                             'data_date'):
                        required[i] = (l, '"%s"' % v)
                elif l == 'op_link':
                    if v not in ('id', 'iteration', 'branch', 'op_date',
                                 'op_id', 'op_level', 'op_name', 'op_group'):
                        required[i] = (l, '"%s"' % v)
                elif l == 'op_res':
                    if v not in ('op_res', 'op_id', 'cash_flow'):
                        required[i] = (l, '"%s"' % v)
                else:
                    if v not in ('id', 'oid', 'pid', 'data_id'):
                        required[i] = (l, '"%s"' % v)
            requirestr = ' AND '.join(['%s IS NOT NULL' % \
                                       '.'.join((add + l, v)) \
                                       for l, v in required])
        for i, (l, v) in enumerate(level_headers):
            if l == 'data_link':
                if v in ('id', 'data_id', 'iteration', 'branch',
                         'data_date'):
                    level_headers[i] = (l, '"%s"' % v)
            elif l == 'op_link':
                if v not in ('id', 'iteration', 'branch', 'op_date',
                             'op_id', 'op_level', 'op_name', 'op_group'):
                    level_headers[i] = (l, '"%s"' % v)
            elif l == 'op_res':
                if v not in ('op_res', 'op_id', 'cash_flow'):
                    level_headers[i] = (l, '"%s"' % v)
            else:
                if v not in ('id', 'oid', 'pid', 'data_id'):
                    level_headers[i] = (l, '"%s"' % v)
        levels = [lh[0] for lh in level_headers]
        main_level = levels[0]
        levels = set(levels)
        try:
            join_string = self._get_join_strings(main_level, levels)
            header = ', '.join(['.'.join((add + l, v)) \
                                 for l, v in level_headers])
            if requirestr!='':
                sql = 'SELECT %s%s FROM %s%s WHERE %s'\
                    % (distinct, header, add + main_level,
                       join_string, requirestr)
            else:
                sql = 'SELECT %s%s FROM %s%s' % (distinct, header,
                                                 add + main_level,
                                                 join_string)
            result = self.db.get(sql)
            return result
        except sqlite3.OperationalError:
            pass

    def get_data_from_level(self, from_data, headers, level, constraints=None,
                            dates=None, rsort=None, required=None,
                            distinct=False, group_by_headers=[],
                            no_join=False,
                            limit2branches={'include': [], 'exclude': []}):
        """
        Return a generator for the requested data

        from_data -- 'data', 'op_res' or 'combined'; where the data is pulled from
        headers -- for 'data' and 'op' a list of headers we want returned.
                   for op the returned non-key values (f.e. Volume)
                   are summed up.
                   for 'combined' these must be ('data', 'column') or
                   ('op', 'column') pairs to differentiate the source.
        level -- level (number/name) of the data for data and combined types.
        constraints -- a list of constraints as (col, op, val<, and/or>)
                       for data and op f.e.
                       [('id', 'in', ('tree1-1-1', 'tree1-2-1'), 'and'),
                        ('branch', 'gt', 0)]
                       where op is in OP2SQL constant and
                       and/or is in ANDOR2SQL constant.
                       For combined, add 'data' or 'op' for the column in the
                       beginning of the tuple. f.e. [('op', 'id', 'eq', '1')]
        dates -- a tuple of (startdate, enddate) (inclusive).
                 If either is None, that side is not limited
        rsort -- columns the result should be sorted by. For combined, these
                 must be ('data/op', 'column') tuples.
        required -- columns that must not be Null to be included in the results
                    For combined, ('data/op', 'column') tuples.
        distinct -- Do we want unique results?
        group_by_headers -- additional columns for 'op' and 'combined'.
                            These are added to the returned values and the
                            results are grouped by them. (Do NOT add data/op
                            for the combined. This has no effect for
                            from_data: 'data')
        no_join -- do not do an explicit JOIN on the data.
                   No effect outside of from_data: 'data'
        """
        add = self._add

        do_limit_branches = False
        if len(limit2branches['include']) > 0 or \
           len(limit2branches['exclude']) > 0:
            do_limit_branches = True

        #self.logger.log_message('db', 'debug', str(headers))
        #self.logger.log_message('db', 'debug', str(group_by_headers))
        if from_data == 'data':
            levname = self._convert_to_level_name(level)
            bad_headers = self._check_header(levname, headers, from_data)
            if len(bad_headers) > 0:
                msg = "Column(s) '%s' do not exist on level '%s' "\
                      "(Check output constraints)" % \
                      ("', '".join(bad_headers), levname)
                if not msg in self._complained_about:
                    self._complained_about.add(msg)
                    self.logger.log_message('db', 'error', msg)
            if no_join:
                levname = '%s l' % (add + levname,)
            else:
                levname = '%sdata_link d JOIN %s l '\
                          'ON d.data_id=l.data_id' \
                          % (add, add + levname, )
            if do_limit_branches:
                levname += ' LEFT OUTER JOIN %sbranch_desc b '\
                           'ON d.id=b.id AND d.branch=b.branch' % (add, )
        elif from_data == 'combined':
            levname = self._convert_to_level_name(level)
            levname = '%sdata_link d JOIN %s l '\
                      'ON d.data_id=l.data_id' \
                      % (add, add + levname, )
            levname += ' JOIN %sop_link olink '\
                       'ON (d.id=olink.id '\
                       'AND d.iteration=olink.iteration '\
                       'AND d.branch=olink.branch '\
                       'AND d.data_date=olink.op_date) '\
                       'JOIN %sop_res ores '\
                       'ON olink.op_id=ores.op_id' \
                       % (add, add,)
            if do_limit_branches:
                levname += ' LEFT OUTER JOIN %sbranch_desc b '\
                           'ON d.id=b.id AND d.branch=b.branch' % (add, )
        else:
            levname = '%sop_link JOIN %sop_res '\
                      'ON %sop_link.op_id=%sop_res.op_id' \
                      % (add, add, add, add)
            if do_limit_branches:
                levname += ' LEFT OUTER JOIN %sbranch_desc b '\
                           'ON %sop_link.id=b.id and %sop_link.branch='\
                           'b.branch' % (add, add, add)

        if distinct:
            distinct = 'DISTINCT '
        else:
            distinct = ''

        wherestr = ''
        where_data = []
        if constraints is not None:
            res = self._create_constraints(from_data, constraints)
            wherestr, where_data = res

        datestr = ''
        if dates is not None and dates is not (None, None):
            start, end = dates
            if start is not None:
                if from_data == 'data':
                    datestr += ' AND d.data_date>=?'
                elif from_data == 'combined':
                    datestr += ' AND olink.op_date>=?'
                else:
                    datestr += ' AND %sop_link.op_date>=?' % add
                where_data.append(start)
            if end is not None:
                if from_data == 'data':
                    datestr += ' AND d.data_date<=?'
                elif from_data == 'combined':
                    datestr += ' AND olink.op_date<=?'
                else:
                    datestr += ' AND %sop_link.op_date<=?' % add
                where_data.append(end)
            if wherestr != '':
                wherestr += datestr
            else:
                wherestr = datestr[5:]

        if do_limit_branches:
            limitstr = ''
            limit_incl = limit2branches['include']
            limit_excl = limit2branches['exclude']
            if limit_incl:
                limitstr += " AND (((b.branch_desc LIKE '%%%s%%'" %\
                        (limit_incl[0], )
                for branch_p in limit_incl[1:]:
                    limitstr += " OR b.branch_desc LIKE '%%%s%%'" %\
                            (branch_p, )
            if limit_excl:
                if limitstr != '':
                    limitstr += ") AND NOT (b.branch_desc LIKE '%%%s%%'" %\
                            (limit_excl[0], )
                else:
                    limitstr += " AND ((NOT(b.branch_desc LIKE '%%%s%%'" %\
                            (limit_excl[0], )
                for branch_p in limit_excl[1:]:
                    limitstr += " OR b.branch_desc LIKE '%%%s%%'" %\
                            (branch_p, )

            # always include the 0 branch
            if from_data == 'data':
                limitstr += ')) OR d.branch=0)'
            elif from_data == 'combined':
                limitstr += ')) OR olink.branch=0)'
            else:
                limitstr += ')) OR %sop_link.branch=0)' % add

            if wherestr != '':
                wherestr += limitstr
            else:
                wherestr = limitstr[5:]

        requirestr = ''
        if required is not None:
            if from_data == 'data':
                for i, r in enumerate(required):
                    if r not in ('id', 'data_id', 'iteration',
                                 'branch', 'data_date'):
                        required[i] = '"%s"' % r

                requirestr = ' AND '.join(['l.%s IS NOT NULL' % r \
                                           for r in required])
            elif from_data == 'combined':
                for i, r in enumerate(required):
                    if r[0] == 'data':
                        if r[1] not in ('id', 'data_id', 'iteration',
                                        'branch', 'data_date'):
                            requirestr += ' AND l."%s" IS NOT NULL' % r[1]
                        else:
                            requirestr += ' AND d.%s IS NOT NULL' % r[1]
                    else:
                        if r[1] not in ('op_id', 'op_name',
                                        'op_date', 'cash_flow'):
                            requirestr += ' AND ores."%s" IS NOT NULL' % r[1]
                        else:
                            requirestr += ' AND olink.%s IS NOT NULL' % r[1]
                if requirestr:
                    requirestr = requirestr[5:]
            else:
                for i, r in enumerate(required):
                    if r not in ('op_id', 'op_name',
                                 'op_date', 'cash_flow'):
                        required[i] = '"%s"' % r
                requirestr = ' AND '.join(['%sop_res.%s IS NOT NULL' \
                                           % (add, r) for r in required])
            if wherestr != '' and requirestr != '':
                wherestr += ' AND ' + requirestr
            else:
                wherestr = requirestr

        sortstr = ''
        if rsort is not None:
            rsorter = ''
            if isinstance(rsort, str):
                rsort = [rsort]
            if from_data == 'data':
                for r in rsort:
                    if r in ('iteration', 'branch',
                             'data_date'):
                        rsorter += ', d.%s' % r
                    elif r in ('data_id', 'id', 'oid', 'pid'):
                        rsorter += ', l.%s' % r
                    else:
                        rsorter += ', l."%s"' % r
            elif from_data == 'combined':
                for r in rsort:
                    if r[0] == 'data':
                        if r[1] in ('iteration', 'branch',
                                 'data_date'):
                            rsorter += ', d.%s' % (r[1],)
                        elif r[1] in ('data_id', 'id', 'oid', 'pid'):
                            rsorter += ', l.%s' % (r[1],)
                        else:
                            rsorter += ', l."%s"' % (r[1],)
                    else:
                        if r[1] in ('id', 'iteration', 'branch', 'op_date',
                                    'op_id', 'op_level', 'op_name',
                                    'op_group'):
                            rsorter += ', olink.%s' % (r[1],)
                        elif r[1] in ('op_id', 'cash_flow'):
                            rsorter += ', ores.%s' % (r[1],)
                        else:
                            rsorter += ', ores."%s"' % (r[1],)
            else:
                for r in rsort:
                    if r in ('id', 'iteration', 'branch', 'op_date', 'op_id',
                             'op_level', 'op_name', 'op_group'):
                        rsorter += ', %sop_link.%s' % (add, r)
                    elif r in ('op_id', 'cash_flow'):
                        rsorter += ', %sop_res.%s' % (add, r)
                    else:
                        rsorter += ', %sop_res."%s"' % (add, r)
            rsort = rsorter[2:]
            sortstr = ' ORDER BY %s' % rsort

        group_by = ''
        data_link_groupers = set(['data_level', 'oid', 'id', 'pid',
                                  'iteration', 'branch', 'data_date'])
        op_link_groupers = set(['id', 'iteration', 'branch', 'op_date',
                                'op_id', 'op_level', 'op_name',
                                'op_group'])
        if from_data == 'data':
            to_join = []
            for h in headers:
                if h in data_link_groupers:
                    if h in ('oid', 'id', 'pid'):
                        to_join.append('l.%s' % h)
                    else:
                        to_join.append('d.%s' % h)
                else:
                    to_join.append('l."%s"' % h)
            header = ', '.join(to_join)
        elif from_data == 'combined':
            to_join = []
            shorten = False
            for h in headers:
                if h[0] == 'data':
                    if h[1] in data_link_groupers:
                        if h[1] in ('oid', 'id', 'pid'):
                            to_join.append('l.%s' % h[1])
                        else:
                            to_join.append('d.%s' % h[1])
                    else:
                        to_join.append('l."%s"' % h[1])
            header = ', '.join(to_join)
            if not header:
                shorten = True
            for h in headers:
                if h[0] == 'op':
                    if h[1] in op_link_groupers:
                        header += ', olink.%s' % (h[1],)
                        group_by += ', olink.%s' % (h[1],)
                    elif h in ('op_id', 'cash_flow'):
                        header += ', sum(ores.%s)' % (h[1],)
                    else:
                        header += ', sum(ores."%s")' % (h[1],)
            for h in group_by_headers:
                if h in op_link_groupers:
                    header += ', olink.%s' % (h,)
                    group_by += ', olink.%s' % (h,)
                elif h in ('op_id', 'cash_flow'):
                    header += ', ores.%s' % (h,)
                    group_by += ', ores.%s' % (h,)
                else:
                    header += ', ores."%s"' % (h,)
                    group_by += ', ores."%s"' % (h,)
            if shorten:
                header = header[2:]
            if group_by:
                group_by = ' GROUP BY ' + group_by[2:]
        else:
            header = ''
            for h in headers:
                if h in op_link_groupers:
                    header += ', %sop_link.%s' % (add, h)
                    group_by += ', %sop_link.%s' % (add, h)
                elif h in ('op_id', 'cash_flow'):
                    header += ', sum(%sop_res.%s)' % (add, h)
                else:
                    header += ', sum(%sop_res."%s")' % (add, h)
            for h in group_by_headers:
                if h in op_link_groupers:
                    header += ', %sop_link.%s' % (add, h)
                    group_by += ', %sop_link.%s' % (add, h)
                elif h in ('op_id', 'cash_flow'):
                    header += ', %sop_res.%s' % (add, h)
                    group_by += ', %sop_res.%s' % (add, h)
                else:
                    header += ', %sop_res."%s"' % (add, h)
                    group_by += ', %sop_res."%s"' % (add, h)
            header = header[2:]
            group_by = ' GROUP BY ' + group_by[2:]
        if wherestr != '':
            sql = 'SELECT %s%s FROM %s WHERE %s%s'\
                        % (distinct, header, levname, wherestr, group_by)
        else:
            sql = 'SELECT %s%s FROM %s%s' % (distinct, header, levname,
                                             group_by)
        sql += sortstr
        #self.logger.log_message('db', 'debug', sql + str(where_data))
        try:
            iterator = self.db.get(sql, where_data)
        except Exception, e:
            msg = sql + ' ' + str(where_data)+\
                            ' failed due to '+str(e)
            self.logger.log_message('db', 'error', msg)
            iterator = []
        for item in iterator:
            yield item

    def get_all_dates(self, dates=None):
        """
        All main simulation level data dates
        """
        add = self._add
        sql = 'SELECT DISTINCT data_date FROM %sdata_link' % add
        result = self.db.get(sql)
        return [item[0] for item in result]

    def get_dates(self, from_data, dates=None):
        """
        Get all dates from data or operations, where date is constraint to
        the given pair.

        from_data -- data or operation
        dates -- None, (Start, None) or (Start, End)
        """
        add = self._add
        if from_data=='data':
            link = '%sdata_link' % add
            date_pre = 'data_'
        else:
            link = '%sop_link' % add
            date_pre = 'op_'
        if dates is not None:
            start, end = dates
            sql = 'SELECT DISTINCT %sdate '\
                  'FROM %s WHERE %sdate >= ? ' % (date_pre, link, date_pre)
            if end is not None:
                sql += 'AND %sdate <= ?' % date_pre
                result = self.db.get(sql, (start, end))
            else:
                result = self.db.get(sql, (start,))
        else:
            sql = 'SELECT DISTINCT %sdate FROM %s' % (date_pre, link,)
            result = self.db.get(sql)
        return [item[0] for item in result]

    def get_dates_by_id_and_iter(self, level, dates=None):
        """
        Returns a group of id, iteration, date rows from the given level,
        constrained by the given dates.

        level -- name/index of the level we want the data from
        dates -- None or (Start, End)
        """
        add = self._add
        levname = self._convert_to_level_name(level)
        if dates is not None:
            sql = 'SELECT DISTINCT id, iteration, data_date FROM %sdata_link' \
                  ' WHERE data_level=? AND data_date BETWEEN ? AND ?' % add
            result = self.db.get(sql, (add + levname, dates[0], dates[1]))
        else:
            sql = 'SELECT DISTINCT id, iteration, data_date FROM %sdata_link '\
                    'WHERE data_level=?' % add
            result = self.db.get(sql, (add + levname,))
        result_dict = {}
        for item in result:
            key = (item[0], item[1])
            if not key in result_dict.keys():
                result_dict[key] = []
            result_dict[key].append(item[2])
        return result_dict

    def get_ids(self, from_data, level, both=False, id_list=None):
        """
        Get ids from data or operations, from the given level.

        from_data -- data or operation
        level -- name of the level we want the data from
        both -- return id AND oid if from_data == data
        id_list -- list of oids
        """
        add = self._add
        levname = self._convert_to_level_name(level)
        levname = levname
        where = ''
        if from_data == 'data':
            if id_list:
                where = "WHERE oid IN ('%s') " % ("', '".join(id_list),)
            if both:
                sql = 'SELECT DISTINCT id, oid FROM %s %s'\
                        'ORDER BY id' % (add + levname, where)
                result = self.db.get(sql)
            else:
                sql = 'SELECT DISTINCT id FROM %s %s'\
                        'ORDER BY id' % (add + levname, where)
                data = self.db.get(sql)
                result = [item[0] for item in data]
        else: # operation result
            sql = 'SELECT DISTINCT id FROM %sop_link '\
                  "WHERE op_level='%s'" % (add, levname)
            data = self.db.get(sql)
            result = [item[0] for item in data]
        return result

    def get_level_ind(self, lname):
        """Convert level name into level index"""
        return self._level_ind[lname]

    def get_level_var_headers(self, level):
        """Get the headers of the level variables"""
        for key, val in self.content_def.items():
            if level in key:
                return [hd[0] for hd in val]

    def get_max_iter(self, from_data):
        """
        Get the highest iteration in the data or operations

        from_data -- data or operation
        """
        add = self._add
        if from_data=='data':
            sql = 'SELECT MAX(iteration) FROM %sdata_link'
        else: # operation result
            sql = 'SELECT MAX(iteration) FROM %sop_link'
        return self.db.get(sql % add)[0][0]

    def get_max_branch(self, from_data):
        """
        Get the highest branch in the data or operations

        from_data -- data or operation
        """
        add = self._add
        if from_data=='data':
            sql = 'SELECT MAX(branch) FROM %sdata_link'
        else: # operation result
            sql = 'SELECT MAX(branch) FROM %sop_link'
        return self.db.get(sql % add)[0][0]

    def get_opres_def_headers(self):
        """Returns the operation result variable headers"""
        return self.opres_def

    def get_main_level(self):
        """Returns main level indice"""
        return self._main_level

    def row_count(self, level):
        """Return number of items a level has"""
        res = self.db.get('SELECT count(*) FROM %s' % (self._add+level,))
        return res[0][0]

    def has_parent(self, level):
        """Return does the level have a parent level"""
        level_ind = self._level_ind[level]
        return self.hierarchy[level_ind]['parent'] is not None

    def is_child_level(self, level, child):
        """Return is the child a child of the level"""
        level_ind = self._level_ind[level]
        child_ind = self._level_ind[child]
        return child_ind in self.hierarchy[level_ind]['children']

    def is_parent_level(self, level, parent):
        """Return is the parent the parent of the level"""
        level_ind = self._level_ind[level]
        parent_ind = self._level_ind[parent]
        return parent_ind == self.hierarchy[level_ind]['parent']

    def prepare_db(self, omatrix):
        """
        Makes sure the proper indices exist and that their content is up to
        date
        """
        db = self.db
        add = self._add
        cc = self.db._create_concurrently
        
        omatrix._add_info('  creating indexes...')
        timer = time.time()
        
        # Create indexes for data link, op link and op result tables
        items = [('%s%sdatalink_idx' % (cc, add), 
                  '%sdata_link' % add, 
                  ['id', 'branch', 'data_date', 'data_id']),
                 ('%s%soplink_idx' % (cc, add), 
                  '%sop_link' % add, 
                  ['id', 'branch', 'op_date', 'op_id']),
                 ('%s%sop_idx' % (cc, add), 
                  '%sop_res' % add, 
                  ['op_id',]),]
        
        # Create indexes for the actual data levels, except simulation level
        for level, index in self._level_ind.items():
            if index > 0:
                items.append(('%s%s%s_idx' % (cc, add, level), 
                              '%s%s' % (add, level),
                              ['data_id']))
        
        # Create the indexes for the tables defined above
        for name, level, cols in items:
            
            # Drop the old index if it exists
            sql = 'DROP INDEX IF EXISTS %s' % (name,)
            db.execute(sql)
            
            # Create the index
            sql = 'CREATE INDEX %s ON %s (%s)' % (name, level, ', '.join(cols))
            try:
                db.execute(sql)
            except Exception:
                # Silently pass; this could happen when the data hierarchy has
                # data levels that are not stored in result db, like tree-level
                # data commonly isn't.
                continue
                            
            if self.db_class != 'ORACLE':
                sql = 'ANALYZE %s' % level
                db.execute(sql)
        
        self.db.commit()
        
        omatrix._add_info('  indexes created in %0.1f sec' % \
                          (time.time() - timer))

    def store_solution(self, keys, run_id, use_orig_uid=False):
        """
        Store a solution to the optimal table, removing all old data under
        the given run_id. keys is an array of iteration, id, branch comboes
        """
        add = self._add
        sql = 'DELETE FROM %soptimal WHERE run_id=?' % add
        self.db.execute(sql, (run_id,))
        sql = 'INSERT INTO %soptimal (run_id, iteration, id, branch, weight) '\
              'VALUES (?, ?, ?, ?, ?)' % add
        for row in keys:
            iteration, uid, origuid, branch, weight, copy_only_ops = row
            iteration = int(iteration)
            branch = int(branch)
            if use_orig_uid:
                uid = origuid
            self.db.execute(sql, (run_id, iteration, uid, branch, weight))
        self.db.commit()

    def store_datarows(self, datarows):
        """
        Dump the given datarows (as stored by no_exec database)
        into the database.
        """
        for row in datarows:
            sql_control, data = row
            sql, control = sql_control
            if not control:
                self.db.executemany(sql, data)
            elif control[0] == 'drop_ids':
                self.drop_ids(control[1])
            elif control[0] == 'skip_existing':
                lev_name, uid, data_id = control[1]
                if self.db_class == 'ORACLE':
                    check_sql = 'SELECT id FROM %s%s '\
                            'WHERE id=? AND data_id=? AND rownum=1'\
                            % (self._add, lev_name)
                else:
                    check_sql = 'SELECT id FROM %s%s '\
                            'WHERE id=? AND data_id=? LIMIT 1'\
                            % (self._add, lev_name)
                res = self.db.get(check_sql, (uid, data_id))
                if res:
                    continue
                for item in data:
                    self.db.execute(sql, item, control)
            else:
                allowed = ', '.join(('False', 'skip_existing', 'drop_ids'))
                err = 'Invalid control name %s. Expected one of (%s)' \
                      % (control[0], allowed)
                raise Exception(err)
        self.db.commit()

    def _add_branching_table(self):
        """
        Create the branching table
        """
        add = self._add
        cc = self.db._create_concurrently
        if not self.db.table_exists('%sBRANCHING_INFO' % add):
            sql = '''CREATE%s TABLE %sBRANCHING_INFO
                           (id TEXT NOT NULL,
                           iteration INTEGER,
                           from_branch INTEGER,
                           to_branch INTEGER,
                           data_date DATE,
                           op_name TEXT NOT NULL,
                           op_group TEXT NOT NULL,
                           branch_name TEXT NOT NULL,
                           branch_group_name TEXT NOT NULL,
                           CONSTRAINT %spk_oift PRIMARY KEY
                              (id, iteration, from_branch,
                               to_branch, data_date))'''\
                            % (self.unlogged, add, add)
            self.db.execute(sql)
        if not self.db.index_exists('%sB_INFO_index1'\
                                    % add):
            sql = 'CREATE INDEX %s%sB_INFO_index1 ON %sBRANCHING_INFO '\
                  '(data_date, iteration, id)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['branching']['%sB_INFO_index1' % add] = sql

        self.db.commit()

    def _add_categorical_data(self, level, data):
        """
        Add data to the META_CAT table

        level -- what level is the data added to
        data -- an object with name, minimum, maximum and desc
        """
        if self.no_exec:
            return
        add = self._add

        sql = 'SELECT id FROM %sMETA_CAT WHERE data_level=? AND name=?' % add
        meta_rows = self.db.get(sql, (add + level, data.name))
        # If the data exists, update rather than potentially swapping ids
        if meta_rows:
            sql = 'UPDATE %sMETA_CAT ' \
                  'SET min_value=?, max_value=?, descr=? ' \
                  'WHERE data_level=? AND name=?' % add
            sql_data = (data.minimum, data.maximum, data.desc,
                        add + level, data.name)
            self.db.execute(sql, sql_data)
            lastid = meta_rows[0][0]
        else:
            if self.db_class == 'ORACLE':
                lastid = self.db.get('SELECT %sMETA_CAT_seq.NEXTVAL '
                                     'FROM dual' % add)[0][0]
                sql = 'INSERT INTO %sMETA_CAT ' \
                      '(id, data_level, name, min_value, max_value, descr) ' \
                      'VALUES (%d, ?, ?, ?, ?, ?)' % (add, lastid)
            else:
                sql = 'INSERT INTO %sMETA_CAT ' \
                      '(data_level, name, min_value, max_value, descr) ' \
                      'VALUES (?, ?, ?, ?, ?)' % add
            sql_data = (add + level, data.name, data.minimum, data.maximum,
                        data.desc)
            self.db.get(sql, sql_data)
            if self.db_class == 'SQLITE':
                lastid = self.db.get('select last_insert_rowid()')[0][0]
            elif self.db_class == 'POSTGRESQL':
                sql = "SELECT CURRVAL(pg_get_serial_sequence('%sMETA_CAT', " \
                      "'id'))" % add
                lastid = self.db.get(sql)[0][0]
        self.db.commit() #end transaction.
        self._add_categorical_value(lastid, data.values)
        self.db.commit()

    def _add_categorical_value(self, pid, data):
        """
        Add a value to the categorical data

        pid -- "parent" id, the row id in META_CAT
        data -- a set of value, description pairs
        """
        add = self._add
        for value, desc in data.items():
            sql = 'SELECT descr FROM %sMETA_CAT_VAL ' \
                  'WHERE pid=? AND value=?' % add
            rows = self.db.get(sql, (pid, value,))
            if rows:
                odesc = rows[0][0]
                if desc == odesc:
                    # Already exists and matches, no changes needed
                    continue
                # Exists but doesn't match, update description
                sql = 'UPDATE %sMETA_CAT_VAL SET descr=? ' \
                      'WHERE pid=? AND value=?' % add
                self.db.execute(sql, (desc, pid, value))
            else:
                # New value, add to table
                sql = 'INSERT INTO %sMETA_CAT_VAL (pid, value, descr) '\
                      'VALUES (?, ?, ?)' % add
                self.db.execute(sql, (pid, value, desc))

    def _add_numerical_data(self, level, data):
        """
        Add data to the numerical table

        level -- name of the level the value is on.
        data -- object with name, unit, minimum, maximum and desc
        """
        if self.no_exec:
            return
        add = self._add
        # Verify that this data does not exist:
        sql = 'SELECT * FROM %sMETA_NUM ' \
              'WHERE data_level=? AND name=?' % add
        num_rows = self.db.get(sql, (add + level, data.name))
        if num_rows:
            sql = 'UPDATE %sMETA_NUM ' \
                  'SET unit=?, min_value=?, max_value=?, descr=? ' \
                  'WHERE data_level=? AND name=?' % add
            sql_data = (data.unit, data.minimum, data.maximum, data.desc,
                        add + level, data.name)
            self.db.execute(sql, sql_data)
        else:
            sql_data = (add + level, data.name, data.unit, data.minimum,
                        data.maximum, data.desc)
            sql = 'INSERT INTO %sMETA_NUM'\
                  ' (data_level, name, unit, min_value, max_value, descr)'\
                  ' VALUES (?, ?, ?, ?, ?, ?)' % add
            self.db.execute(sql, sql_data)

    def _check_header(self, level, header, from_data):
        """
        Return columns that are NOT on the level.

        level -- name of the level
        header -- a list of header items
        """
        result = []
        for item in header:
            if item in ('iteration', 'branch', 'data_date', 'op_date'):
                continue
            if item not in self.table_def[level]['columns']:
                result.append(item)
        return result

    def _collect_all_id(self, lind, uid, data_id):
        """
        For the given object collect ids of parents and children

        lind -- index of the level
        uid -- unique id on the level
        data_id -- data_id we're interested in
        """
        parents = []
        parents = self._get_parent(lind, uid, parents)
        parents.sort(reverse=True)
        for parent in parents:
            pl_ind, pid = parent[1], parent[3]
            self._all_id.append(pid)
            plin = self.hierarchy[pl_ind]['lineage']
            other_lin = plin.difference(self._lineages)
            for lin in other_lin:
                cid = self._collect_all_child_id(pl_ind, pid, lineage=lin)
                self._all_id += cid
            self._lineages.update(other_lin)
        cid = self._collect_all_child_id(lind, uid, data_id)
        self._all_id += cid

    def _collect_all_child_id(self, lind, uid, lineage=None):
        """
        Collect all children's ids

        lind -- index of the level
        uid -- unique id of the object we're interested in
        lineage -- If lineage matters, restrict the ids to this lineage
        """
        cids = []
        kids = self._get_children_id(lind, uid, cids, lineage=lineage)
        child_id = [child[1] for child in kids]
        return child_id

    def _convert_to_level_name(self, level):
        """
        Whether level is the index or the actual level name, return the name
        of the level
        """
        if isinstance(level, int):
            levname = self._level_name[level]
        else:
            levname = level
        return levname

    def _constrain_links(self, links):
        """
        Constrain the given links by the constraints stored to this object
        in initialization
        """
        root_level = self._get_root_level()
        constraints = self._constraints
        if constraints is None:
            return links
        result = {}
        for iter_branch, ibdata in links.items():
            result[iter_branch] = {}
            for level_ind, lidata in ibdata.items():
                if level_ind[0] == root_level or level_ind[0] in constraints:
                    new_data = {}
                    result[iter_branch][level_ind] = new_data
                    for level, inds in lidata.items():
                        if level == root_level or level in constraints:
                            new_data[level] = inds
        return result

    def _copy_level_data(self, db, lname, data_ids=None, new_id=False):
        """
        Copy object data to another db or to a new object

        db -- database to copy the data to
        lname -- name of the level copying the data from/to
        data_ids -- ids to copy
        new_id -- make a new unique id for the object being copied
        """
        add = self._add
        if data_ids:
            sql = "SELECT * FROM %s WHERE data_id in ('%s')" \
                    % (add + lname, data_ids)
        else:
            sql = 'SELECT * FROM %s' % (add + lname, )
        obj_data = self._get_star_data(lname, sql)
        if len(obj_data) > 0:
            # get the insert sql from the target db
            sql = db.table_def[lname]['insert sql']
            for row in obj_data:
                if new_id:
                    new_uid = self._get_new_uid(lname, row[ID_COL])
                    lind = self.get_level_ind(lname)
                    plname = self._level_name[self.hierarchy[lind]['parent']]
                    pid = row[PID_COL]
                    new_pid = self._new_uid[plname].get(pid, pid)
                    row = list(row)
                    row[ID_COL] = new_uid
                    row[ORIG_ID_COL] = new_uid
                    row[PID_COL] = new_pid
                db.db.execute(sql, row)
                uid = row[ID_COL]
                data_id = row[DATA_ID_COL]
                if self.table_def[lname]['geom type'] is not None:
                    self._update_geom(db, lname, uid, data_id)

    def _copy_operation_data(self, db, main_oid, iterr, branch, new_id=None,
                             weight=1., weight_columns=None):
        """
        Copy operation data to another db for the given main level object,
        or duplicate operation data from the main_oid to the new_id

        db -- database to copy the operations to
        main_oid -- object from which we copy the operations from
        iterr -- iteration we're interested in
        branch -- branch we're interested in
        new_id -- copy operations to this id instead
        weight -- weight for the current branch, float 0..1
        weight_columns -- list with a boolean value for each op_res column
                          indicating whether the column should be weighted
        """
        cur = db.db.conn.cursor()
        add = self._add
        new_op_ids = {}
        sql = 'SELECT * FROM %sop_link '\
              'WHERE id=? AND iteration=? AND branch=?' % add
        sql_data = [main_oid, iterr, branch]
        obj_data = self.db.get(sql, sql_data)
        if len(obj_data)>0:
            op_ids = set([])
            for row in obj_data:
                op_ids.add(row[OP_OP_ID_COL])
                if new_id:
                    iterr = row[OP_ITERR_COL]
                    branch = row[OP_BRANCH_COL]
                    ddate = row[OP_DATE_COL]
                    op_name = row[OP_OP_NAME_COL]
                    op_hash = self._get_op_hash(new_id, iterr, branch, ddate,
                                                op_name)
                    op_id = self._encode_hash(op_hash)
                    new_op_ids[row[OP_OP_ID_COL]] = op_id
                    row = list(row)
                    row[OP_ID_COL] = new_id
                    row[OP_OP_ID_COL] = op_id
                db.db.execute(db._op_link_insert_sql, row)
            sql = "SELECT * FROM %sop_res WHERE op_id in ('%s')" \
                   % (add, "', '".join(op_ids))
            op_data = self.db.get(sql)
            for row in op_data:
                if new_id:
                    row = list(row)
                    row[OPRES_OP_ID_COL] = new_op_ids[row[OPRES_OP_ID_COL]]
                if weight != 1. and weight_columns is not None:
                    # If weight is something else than 1, it means that the
                    # computing unit has been split and the operation results
                    # should be multiplied with the weight, except for the
                    # columns defined in optimization ini's skip_weight_splits
                    # option
                    row = list(row)
                    for k, value in enumerate(row):
                        if isinstance(value, float) and weight_columns[k]:
                            row[k] = float(value) * weight
                    row = tuple(row)
                db.db.execute(db._opres_insert_sql, row)
        cur.close()

    def _copy_timber_prices(self, db):
        """
        Copy timber prices to the given database
        """
        add = self._add
        add2 = db._add
        sql = 'SELECT * from %stimber_price' % add
        data = self.db.get(sql)
        if data!=[]:
            vals = len(data[0])
            sql = 'INSERT INTO %stimber_price VALUES (' % add2
            sql += ', '.join(vals * ['?']) + ')'
            for row in data:
                db.db.execute(sql, row)

    def _get_new_uid(self, lname, uid):
        """
        Creates a new unique identifier from the old uid

        lname -- level name
        uid -- unique identifier
        """
        add = self._add
        if uid in self._new_uid[lname]:
            return self._new_uid[lname][uid]
        new_id_sql = 'SELECT count(id) FROM %s WHERE id=?' % add + lname
        count = 1
        new_id = uid + '_%d' % count
        hits = self.db.get(new_id_sql, (new_id,))
        while hits[0][0] > 0:
            count += 1
            new_id = uid + '_%d' % count
            hits = self.db.get(new_id_sql, (new_id,))
        self._new_uid[lname][uid] = new_id
        return new_id

    def _create_branch_desc_table(self):
        """
        Create the branch description table.
        """
        add = self._add
        if not self.db.table_exists('%sbranch_desc' % add):
            # TODO: branch_desc back to NOT NULL when 0 branch actully
            # has a description of its 'non-branching' operation
            sql = "CREATE%s TABLE %sbranch_desc (\n"\
                  "    iteration INTEGER NOT NULL,\n"\
                  "    id TEXT NOT NULL,\n"\
                  "    branch INTEGER NOT NULL,\n"\
                  "    branch_desc TEXT,\n"\
                  "    branching_group TEXT NOT NULL, \n"\
                  "    CONSTRAINT %sbranch_desc_pk PRIMARY KEY\n"\
                  "        (id, iteration, branch))"\
                  % (self.unlogged, add, add)
            self.db.execute(sql)
            self.db.commit()

    def _create_constraints(self, from_data, constraints):
        add = self._add
        result = ''
        vals = []
        for i, constraint in enumerate(constraints):
            if from_data != 'combined':
                constraint = ('',) + tuple(constraint)
            if len(constraint) < 4:
                self.logger.log_message('db', 'error', 'invalid constraint %s'\
                                        % str(constraint))
                continue
            elif  len(constraints) > i+1 and len(constraint) < 5:
                self.logger.log_message('db', 'error',
                            'missing concatenation operand from %s index %d' \
                            % (str(constraints), i))
                return result, vals
            lev, var, op, val = constraint[:4]
            # TODO: DEBUG: FIXME: NOTE: IS THIS NEEDED?!?
            # is_level = var == 'data_level' or var == 'op_level'
            if from_data == 'op_res':
                if var in ('id', 'iteration', 'branch', 'op_date', 'op_id',
                           'op_level', 'op_name', 'op_group'):
                    var = '%sop_link.%s' % (add, var)
                else:
                    var = '%sop_res."%s"' % (add, var)
            elif from_data == 'combined':
                if lev == 'op':
                    if var in ('id', 'iteration', 'branch', 'op_date', 'op_id',
                               'op_level', 'op_name', 'op_group'):
                        var = 'olink.%s' % (var,)
                    else:
                        var = 'ores."%s"' % (var,)
                else:
                    if var in ('iteration', 'branch', 'data_date'):
                        var = 'd.%s' % var
                    elif var in ('id', 'oid', 'pid'):
                        var = 'l.%s' % var
                    else:
                        var = 'l."%s"' % var
            else:
                if var in ('iteration', 'branch', 'data_date'):
                    var = 'd.%s' % var
                elif var in ('id', 'oid', 'pid'):
                    var = 'l.%s' % var
                else:
                    var = 'l."%s"' % var
            andor = None
            if len(constraint) > 4:
                andor = constraint[4]

            if op not in OP2SQL:
                self.logger.log_message('db', 'error',
                                        'Bad operation (%s) not in (%s)' \
                                        % (op, ', '.join(OP2SQL.keys())))
                continue

            if andor not in ('and', 'or', None):
                self.logger.log_message('db', 'error',
                                'Concatenation operand (%s) not in (and, or)'\
                                % (andor, 'and, or'))
                continue

            if op == 'in':
                #if not is_level:
                const = ','.join(['?'] * len(val))
                #else:
                #    const = ','.join([add + '?'] * len(val))
                const = '('+const+')'
            elif not (isinstance(val, list) or isinstance(val, tuple)):
                const = '?'
                #if is_level:
                #    const = add + const
            else:
                self.logger.log_message('db', 'error',
                                        'Too many values for operation '\
                                        'type (%s)' % op)
                continue

            andor = ANDOR2SQL[andor]
            op = OP2SQL[op]

            if isinstance(val, list):
                vals += val
            elif isinstance(val, tuple):
                vals += list(val)
            else:
                vals.append(val)

            consts = '%s %s %s %s' % (var, op, const, andor)
            if result=='':
                result = consts
            else:
                result = ' '.join((result, consts))
        return result, vals

    def _create_data_link(self):
        """
        Creates the master table for data. Level tables linked to this
        """
        add = self._add
        #cc = self.db._create_concurrently
        sql = '''CREATE%s TABLE %sdata_link
                          (data_level TEXT NOT NULL,
                           id TEXT NOT NULL,
                           iteration INTEGER NOT NULL,
                           branch INTEGER NOT NULL,
                           data_date DATE NOT NULL,
                           data_id TEXT NOT NULL,
                           CONSTRAINT %sdata_link_pk PRIMARY KEY
                              (id, iteration, branch, data_date)
                           )''' % (self.unlogged, add, add)
        if not self.db.table_exists('%sdata_link' % add):
            self.db.execute(sql)
            self.db.commit()

    def _create_meta_tables(self):
        """
        Sets up tables for storing the numerical and categorical attribute
        definitions
        """
        created = False
        add = self._add
        if self.db_class == 'SQLITE':
            id = SQLITEID
        elif self.db_class == 'POSTGRESQL':
            id = POSTGRESQLID
        elif self.db_class == 'ORACLE':
            id = ORACLEID
        if not self.db.table_exists('%sMETA_NUM' % add):
            created = True
            sql = '''CREATE%s TABLE %sMETA_NUM
                           (%s,
                           data_level TEXT NOT NULL,
                           name TEXT NOT NULL,
                           unit TEXT,
                           min_value DOUBLE PRECISION,
                           max_value DOUBLE PRECISION,
                           descr TEXT NOT NULL,
                           CONSTRAINT %suq_mn_levname
                           UNIQUE (data_level, name))
                           ''' % (self.unlogged, add, id, add)
            self.db.execute(sql)
            if self.db_class == 'ORACLE':
                self.db.create_autoid('%sMETA_NUM' % add)
        if not self.db.table_exists('%sMETA_CAT' % add):
            created = True
            sql = '''CREATE%s TABLE %sMETA_CAT
                           (%s,
                           data_level TEXT NOT NULL,
                           name TEXT NOT NULL,
                           min_value DOUBLE PRECISION,
                           max_value DOUBLE PRECISION,
                           descr TEXT NOT NULL,
                           CONSTRAINT %suq_mc_levname UNIQUE (data_level, name))
                           ''' % (self.unlogged, add, id, add)
            self.db.execute(sql)
            if self.db_class == 'ORACLE':
                self.db.create_autoid('%sMETA_CAT' % add)
        if not self.db.table_exists('%sMETA_CAT_VAL' % add):
            created = True
            sql = '''CREATE%s TABLE %sMETA_CAT_VAL
                           (%s,
                           pid INTEGER NOT NULL,
                           value DOUBLE PRECISION NOT NULL,
                           descr TEXT NOT NULL,
                           CONSTRAINT %sfk_pid FOREIGN KEY (pid)
                                REFERENCES %sMETA_CAT (id)
                                ON DELETE CASCADE,
                           CONSTRAINT %suq_pidval UNIQUE (pid, value))
                           ''' % (self.unlogged, add, id, add, add, add)
            self.db.execute(sql)
            if self.db_class == 'ORACLE':
                self.db.create_autoid('%sMETA_CAT_VAL' % add)
        self.db.commit()
        return created

    def _create_op_res_table(self, opres_def):
        add = self._add
        #cc = self.db._create_concurrently
        if not self.db.table_exists('%sop_link' % add):
            sql = '''CREATE%s TABLE %sop_link
                          (id TEXT NOT NULL,
                           iteration INTEGER NOT NULL,
                           branch INTEGER NOT NULL,
                           op_date DATE NOT NULL,
                           op_id TEXT NOT NULL,
                           o_op_id TEXT,
                           op_level TEXT NOT NULL,
                           op_name TEXT NOT NULL,
                           op_group TEXT NOT NULL,
                           op_type TEXT NOT NULL,
                           CONSTRAINT %sop_link_pk PRIMARY KEY
                              (id, iteration, branch, op_id))'''\
                        % (self.unlogged, add, add)
            self.db.execute(sql)
        if not self.db.table_exists('%sop_note' % add):
            sql = '''CREATE%s TABLE %sop_note
                        (op_id TEXT NOT NULL,
                         notes TEXT)''' % (self.unlogged, add)
            self.db.execute(sql)
            sql = 'CREATE INDEX %sop_note_op_id_idx ON %sop_note(op_id)'\
                    % (add, add)
            self.db.execute(sql)
            self.indices['link']['%sop_note_op_id_idx' % (add,)] = sql
        if not self.db.table_exists('%sop_material' % add):
            sql = '''CREATE%s TABLE %sop_material
                        (op_id TEXT NOT NULL,
                         material TEXT)''' % (self.unlogged, add)
            self.db.execute(sql)
            sql = 'CREATE INDEX %sop_mat_op_id_idx ON %sop_material(op_id)'\
                    % (add, add)
            self.db.execute(sql)
            self.indices['link']['%sop_mat_op_id_idx' % (add,)] = sql
        if not self.db.table_exists('%sop_res' % add):
            sql = '''CREATE%s TABLE %sop_res
                          (op_id TEXT NOT NULL,
                           cash_flow DOUBLE PRECISION,'''\
                        % (self.unlogged, add)
            for var in opres_def:
                if var in self._all_text_vars:
                    # NB! We are relying only on var names here for typing, so
                    # same name vars at different levels are not really
                    # handled properly
                    sql += '\n"%s" TEXT,' % (var, )
                else:
                    sql += '\n"%s" DOUBLE PRECISION,' % (var, )
            sql = sql[:-1] + ')'
            self.db.execute(sql)
            sql = 'CREATE INDEX %sop_res_op_id_idx ON %sop_res(op_id)'\
                    % (add, add)
            self.db.execute(sql)
            self.indices['link']['%sop_res_op_id_idx' % (add,)] = sql
        else:
            # Ensure columns match to the expected
            metadata = self._get_table_columns('op_res')
            low_opres = [o.lower() for o in opres_def]
            low_mdata = [m.lower() for m in metadata]
            rem_columns = []
            if metadata!=[]:
                changed = False
                dropped = False
                for lvar, var in zip(low_opres, opres_def):
                    if lvar not in low_mdata:
                        changed = True
                        self.db.execute('ALTER TABLE %sop_res '\
                                        'ADD COLUMN "%s" DOUBLE PRECISION' %\
                                        (add, var,))
                for lvar, var in zip(low_mdata, metadata):
                    if lvar not in ('op_id', 'cash_flow'):
                        if lvar not in low_opres:
                            dropped = True
                            rem_columns.append(var)
                if changed or dropped:
                    msg = 'Operation result columns have changed!'
                    self.logger.log_message(__name__, 'warning', msg)
                if self.db_class != 'SQLITE':
                    for col in rem_columns:
                        self.db.execute('ALTER TABLE %sop_res '\
                                        'DROP COLUMN "%s"' % (add, col,))
                elif rem_columns:
                    msg = 'Operation result change resulted to a '\
                          'drop request for column(s) (%s). '\
                          "Columns can't be dropped in SQLite! "\
                          'This should only cause increased database size, '\
                          'however.' % (', '.join(rem_columns),)

                    # TODO: HACK: FIXME: this is a hack for a situation where
                    # the simulated op_res structure differs from optimized
                    # (different simulation controls)
                    self.opres_def += rem_columns

                self.db.commit()

        sql = 'INSERT INTO %sop_link (id, iteration, branch, op_date, op_id, '\
              'o_op_id, op_level, op_name, op_group, op_type) VALUES '\
              '(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)' % add
        self._op_link_insert_sql = sql
        column_count = 2 + len(opres_def)
        self._opres_insert_sql = 'INSERT INTO %sop_res VALUES (%s)' \
                                 % (add, ', '.join(['?'] * column_count))
        opres_def.append('cash_flow')
        self.db.commit()

    def _create_optimal_table(self):
        """
        Create the optimal solutions table.
        """
        add = self._add
        if not self.db.table_exists('%soptimal' % add):
            sql = "CREATE%s TABLE %soptimal (\n"\
                  "    run_id TEXT NOT NULL,\n"\
                  "    iteration INTEGER NOT NULL,\n"\
                  "    id TEXT NOT NULL,\n"\
                  "    branch INTEGER NOT NULL,\n"\
                  "    weight DOUBLE PRECISION NOT NULL,\n"\
                  '    CONSTRAINT %soptimal_pk PRIMARY KEY\n'\
                  '        (run_id, id, iteration, branch))'\
                  % (self.unlogged, add, add)
            self.db.execute(sql)
            self.db.commit()

    def _create_timber_price_table(self, cf_cfiers):
        add = self._add
        cc = self.db._create_concurrently
        metadata = self._get_table_columns('timber_price')
        low_cfiers = [c.lower() for c in cf_cfiers]
        low_mdata = [m.lower() for m in metadata]
        rem_columns = []
        if metadata!=[]:
            changed = False
            dropped = False
            for lvar, var in zip(low_cfiers, cf_cfiers):
                if lvar not in low_mdata:
                    changed = True
                    self.db.execute('ALTER TABLE %stimber_price '\
                                    'ADD COLUMN "%s" DOUBLE PRECISION' %\
                                    (add, var,))
            for lvar, var in zip(low_mdata, metadata):
                if lvar not in ('id', 'iteration', 'branch', 'data_date',
                               'price_table', 'price'):
                    if lvar not in low_cfiers:
                        dropped = True
                        rem_columns.append(var)
            if changed or dropped:
                msg = 'Timber price classifiers have changed!'
                self.logger.log_message(__name__, 'warning', msg)
            if self.db_class == 'POSTGRESQL':
                for col in rem_columns:
                    self.db.execute('ALTER TABLE %stimber_price '\
                                    'DROP COLUMN "%s"' % (add, col,))
            elif rem_columns:
                msg = 'Timber price classifier change resulted to a '\
                      'drop request for column(s) (%s). '\
                      "Columns can't be dropped in SQLite! "\
                      'This should only cause increased database size, '\
                      'however.' % (', '.join(rem_columns),)
                self.logger.log_message(__name__, 'warning', msg)
                self.db.execute('DROP INDEX %stbr_prc_pk_idx' % add)
                sql = 'CREATE UNIQUE INDEX %stbr_prc_pk_idx '\
                      'ON timber_price (id, iteration, branch, data_date, ' \
                      % add
                if len(cf_cfiers)==0:
                    sql += 'price_table)'
                else:
                    sql += ', '.join(['"'+c+'"' for c in cf_cfiers]) \
                           + ', price_table)'
                self.db.execute(sql)
                # rebuild table index
                self.db.execute('ANALYZE %stimber_price' % add)
            self.db.commit()


        if not self.db.table_exists('%stimber_price' % add):
            sql = '''CREATE%s TABLE %stimber_price
                           (id TEXT NOT NULL,
                           iteration INTEGER NOT NULL,
                           branch INTEGER NOT NULL,
                           data_date DATE NOT NULL,'''\
                        % (self.unlogged, add)
            sql += '\n'.join(['"'+var+'"'+' DOUBLE PRECISION,' for var in \
                              cf_cfiers])
            sql += '''     price_table TEXT NOT NULL,
                           price DOUBLE PRECISION)'''
            self.db.execute(sql)
            sql = 'CREATE UNIQUE INDEX %stbr_prc_pk_idx '\
                  'ON %stimber_price '\
                  '(id, iteration, branch, data_date, ' % (add, add)
            if len(cf_cfiers)==0:
                sql += 'price_table)'
            else:
                sql += ', '.join(['"'+c+'"' for c in cf_cfiers]) \
                       + ', price_table)'
            self.db.execute(sql)
        if not self.db.index_exists('%stbr_price_date_idx' % add):
            sql = 'CREATE INDEX %s%stbr_price_date_idx '\
                  'ON %stimber_price (data_date)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['price']['%stbr_price_date_idx' % add] = sql

        if not self.db.index_exists('%stbr_prc_id_idx' % add):
            sql = 'CREATE INDEX %s%stbr_prc_id_idx ON %stimber_price (id)' \
                  % (cc, add, add)
            self.db.execute(sql)
            self.indices['price']['%stbr_prc_id_idx' % add] = sql

        if not self.db.index_exists('%stbr_prc_br_idx' % add):
            sql = 'CREATE INDEX %s%stbr_prc_br_idx ON %stimber_price (branch)'\
                    % (cc, add, add)
            self.db.execute(sql)
            self.indices['price']['%stbr_prc_br_idx' % add] = sql

        if not self.db.index_exists('%stbr_prc_it_idx' % add):
            sql = 'CREATE INDEX %s%stbr_prc_it_idx '\
                    'ON %stimber_price (iteration)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['price']['%stbr_prc_it_idx' % add] = sql

        self.db.commit()

    def _create_sql_templates(self, tdef, lname):
        """
        Creates sql templates for variable substitution for the data level
        """
        add = self._add
        lname = add + lname
        float_attr = tdef['float attr name']
        all_attr = tdef['attr name']
        insert_base = 'INSERT INTO %s (%%s) VALUES (%%s)' % lname
        update_base = 'UPDATE %s SET %%s WHERE data_id=? AND id=?' % lname

        attr_s = 'data_id, id, oid, pid'
        upd_s = ''
        create_s = ''
        ins_val = '?, ?, ?, ?'
        f_attr_s = attr_s
        f_ins_val = ins_val
        for attr in all_attr:
            attr_s += ', ' + '"'+attr+'"'
            ins_val += ', ?'
            upd_s += '"'+attr+'"'+'=?,'
            if attr in float_attr:
                create_s += ' "' + attr + '" DOUBLE PRECISION,\n'
            else:
                # geometry column added separately at create_table
                create_s += ' "' + attr + '" TEXT,\n'

        if tdef['geom type'] is not None:
            if self.db_class == 'SQLITE':
                geom_update = 'UPDATE %s SET geom=GeomFromText(?, ?) '\
                                 'WHERE data_id=? AND id=?' % lname
            elif self.db_class == 'POSTGRESQL':
                geom_update = 'UPDATE %s SET geom=? '\
                                 'WHERE data_id=? AND id=?' % lname
                g_attr_s = attr_s + ', "geom"'
                if 'multi' in tdef['geom type'].lower():
                    g_ins_val = ins_val + ', ST_Multi(ST_GeomFromText(?, ?))'
                else:
                    g_ins_val = ins_val + ', ST_GeomFromText(?, ?)'
            elif self.db_class == 'ORACLE':
                geom_update = 'UPDATE %s SET geom=SDO_UTIL.'\
                        'FROM_WKTGEOMETRY(?, ?)'\
                        ' WHERE data_id=? AND id=?' % lname
                g_attr_s = attr_s + ', "geom"'
                g_ins_val = ins_val + ', SDO_UTIL.FROM_WKTGEOMETRY(?, ?)'
            else:
                geom_update = ''
                g_attr_s = ''
                g_ins_val = ''
                msg = 'Unsupported database for geometry data. Supported are:'\
                        ' Sqlite, Postgresql and Oracle'
                self.logger.log_message(__name__, 'error', msg)
        else:
            geom_update = ''
            g_attr_s = ''
            g_ins_val = ''

        for attr in float_attr:
            f_attr_s += ', ' + '"'+attr+'"'
            f_ins_val += ', ?'
        upd_s = upd_s[:-1]
        tdef['insert sql'] = insert_base % (attr_s, ins_val)
        tdef['update sql'] = update_base % (upd_s,)
        tdef['float insert sql'] = insert_base % (f_attr_s, f_ins_val)
        if self.db_class != 'SQLITE':
            tdef['geom insert sql'] = insert_base % (g_attr_s, g_ins_val)
        else:
            tdef['geom insert sql'] = None
        tdef['geom update sql'] = geom_update
        tdef['create attr'] = create_s

    def _create_table(self, lname):
        """
        Based on the table definitions, creates new tables in the db. For
        existing tables checks that the schema hasn't changed. If it has and
        the the db is not an input db, raises an error
        """
        add = self._add
        cc = self.db._create_concurrently
        llname = add + lname
        lind = self._level_ind[lname]
        tdef = self.table_def[lname]
        def_cols = set(tdef['columns'])
        msg = 'The following columns exceeded max length of 30: '
        bad_cols = []
        for col in def_cols:
            if len(col) > 30:
                bad_cols.append(col)
        if bad_cols:
            msg = msg + ', '.join(bad_cols)
            self.logger.log_message(__name__, 'error', msg)
            raise Exception(msg)
        self.has_geom[lname] = False
        if tdef['geom type']:
            self.has_geom[lname] = True
        metadata = self._get_table_columns(lname)
        if metadata!=[]:
            meta_cols = set(metadata)
            def_cols = set(tdef['columns'])
            columns = set([m.lower() for m in meta_cols])
            low_cols = set([c.lower() for c in def_cols])
            add_columns = low_cols - columns
            rem_columns = columns - low_cols
            if not add_columns and not rem_columns:
                return # no changes.
            temp_cols = set()
            for lcol in add_columns:
                for col in def_cols:
                    if col.lower() == lcol:
                        temp_cols.add(col)
            add_columns = temp_cols
            temp_cols = set()
            for lcol in rem_columns:
                for col in meta_cols:
                    if col.lower() == lcol:
                        temp_cols.add(col)
            rem_columns = temp_cols
            msg = 'Lexicon has changed! Updating %s.' % lname
            self.logger.log_message(__name__, 'warning', msg)
            msg = "We can't alter column- or geometry types in this "\
                  'fashion. If you need those changed, wipe the existing '\
                  'database.'
            self.logger.log_message(__name__, 'warning', msg)
            if self.db_class == 'POSTGRESQL':
                for col in rem_columns:
                    if col == 'geom':
                        self.db.execute('SELECT DropGeometryColumn(?, ?)',
                                        (llname, col))
                    else:
                        self.db.execute('ALTER TABLE %s DROP COLUMN "%s"' \
                                        % (llname, col))
            elif rem_columns:
                msg = 'Lexicon change resulted to a drop request for '\
                      "column(s) (%s). Columns can't be dropped in "\
                      'SQLite! This should only cause increased database '\
                      'size, however.' % (', '.join(rem_columns),)
                self.logger.log_message(__name__, 'warning', msg)

            float_attrs = tdef['float attr name']
            for col in add_columns:
                if col != 'geom':
                    # geometry column handled separately
                    datatype = 'TEXT'
                    if col in float_attrs:
                        datatype = 'DOUBLE PRECISION'
                    self.db.execute('ALTER TABLE %s ADD "%s" %s' \
                                    % (llname, col, datatype))
            self.db.commit()
            return # we're done with the changes.
        if tdef['level'] == 0:
            sql = 'CREATE%s TABLE %s\n'\
                  '    (data_id TEXT,\n'\
                  '    id TEXT NOT NULL,\n'\
                  '    oid TEXT NOT NULL,\n'\
                  '    pid TEXT,\n'\
                  '    %s'\
                  '    CONSTRAINT %s_pk_id PRIMARY KEY (id))'\
                  % (self.unlogged, llname, tdef['create attr'], llname)
        elif self.hierarchy[lind]['ordinal'] < self.sim_ordinal:
            sql = 'CREATE%s TABLE %s\n'\
                  '    (data_id TEXT,\n'\
                  '    id TEXT NOT NULL,\n'\
                  '    oid TEXT NOT NULL,\n'\
                  '    pid TEXT NOT NULL,\n'\
                  '    %s'\
                  '    CONSTRAINT %s_pk_id PRIMARY KEY (id))'\
                  % (self.unlogged, llname, tdef['create attr'], llname,)
        else:
            sql = 'CREATE%s TABLE %s\n'\
                  '   (data_id TEXT NOT NULL,\n'\
                  '   id TEXT NOT NULL,\n'\
                  '   oid TEXT NOT NULL,\n'\
                  '   pid TEXT NOT NULL,\n'\
                  '   %s'\
                  '   CONSTRAINT %s_pk_id PRIMARY KEY (id, data_id))'\
                  % (self.unlogged, llname, tdef['create attr'], llname,)
        self.db.execute(sql)
        self.db.commit()
        sql = 'CREATE INDEX %s%s_did_index ON %s (data_id)'\
                % (cc, llname, llname)
        self.db.execute(sql)
        self.indices['link']['%s_did_index' % llname] = sql

        if tdef['level'] > 1: # useless at root and root+1
            sql = 'CREATE INDEX %s%s_pid_index ON %s (pid)'\
                    % (cc, llname, llname)
            self.db.execute(sql)
            self.indices['link']['%s_pid_index' % llname] = sql

        if tdef['geom type']:
            self.db.add_geom(llname, tdef['geom type'])
            idx_sql = self.db.add_geom_index(llname)
            self.indices['geom']['%s_geom_index' % llname] = idx_sql
        self.db.commit()

    def _create_tabledef(self, defkey, content_def=None, levels=None):
        """
        Creates table definitions for each data level in content definition.
        Table definition is stored keyed with the level name and stores the
        level index, parent level name, attribute names and corresponding
        matrix indices (for all attributes and for float type attributes),
        optional geometry type of the level, a set of
        attributes names that are of type TEXT, and list of indices to remove
        from data when only float type vars are wanted
        """
        lname, lind = defkey
        lconstraints = None
        constraints = self._constraints
        if self.db_type == 'write' and constraints is not None \
            and self.hierarchy[lind]['ordinal'] > 0:
            lconstraints = constraints[lind]
        if content_def is None:
            content_def = self.content_def
        self.table_def[lname] = {}
        tdef = self.table_def[lname]
        tdef['level'] = lind
        if tdef['level']>0:
            parent_level = self._get_parent_level(self.hierarchy[lind])
            tdef['parent name'] = self._level_name[parent_level]
        attr_name, attr_ind = self._split_content_def(content_def[defkey],
                                                      lconstraints)
        tdef['attr name'] = attr_name
        tdef['float attr name'] = copy(attr_name)
        tdef['other attr name'] = []
        tdef['attr ind'] = attr_ind
        tdef['float attr ind'] = self._copy(attr_ind)
        tdef['text vars'] = set()
        tdef['to remove'] = []
        tdef['geom type'] = None
        keys = [u'id', u'oid', u'pid', u'data_id']
        if levels is not None:
            inds = []
            if self.is_spatial:
                geom_type = levels[defkey[0]].geom_type
                if geom_type is not None:
                    tdef['geom type'] = geom_type
                    keys.append(u'geom')
            tdef['text vars'] = levels[defkey[0]].textual
            self._all_text_vars.update(levels[defkey[0]].textual)
            for tvar in tdef['text vars']:
                try:
                    inds.append(tdef['attr name'].index(tvar))
                except ValueError:
                    pass
            inds.sort(reverse=True)
            tdef['to remove'] = inds
            for ind in inds:
                tdef['other attr name'].append(tdef['attr name'][ind])
                tdef['float attr name'].pop(ind)
                tdef['float attr ind'].pop(ind)
        tdef['columns'] = set(keys + tdef['attr name'])
        # Because the database column order might not be that of the matrix
        tdef['matrix column string'] = \
                    '"' + '", "'.join([u'data_id', u'id', u'oid', u'pid'] + \
                                      self.table_def[lname]['attr name']) + '"'
        self._create_sql_templates(tdef, lname)

    def _create_trigger(self, lname, tdef):
        if tdef['level']==0:
            return
        # Create kill trigger
        self._create_any_trigger(tdef['parent name'], lname,
                                 ('id', 'data_id'), ('pid', 'data_id'))
        lind = self._level_ind[lname]
        if self.hierarchy[lind]['ordinal'] >= self.sim_ordinal:
            # Don't make a cascading delete on the levels above the
            # simulation level (f.e. simulation and estate)
            self._create_any_trigger('data_link', lname,
                                     ('data_id',), ('data_id',))

    def _delete_with_data_id(self, sql_res):
        """
        Delete data from all data levels from main level onwards with the
        data_id query result set

        sql_res -- query result with data_ids to delete
        """
        add = self._add
        ml = self._main_level
        levels = [ml] + list(self.get_all_child_levels(ml))
        data_ids = []
        for row in sql_res:
            data_ids.append(row[0])
        sets = int(math.ceil(len(data_ids)/1000.))
        for i in range(sets):
            ids = data_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            did_str = ', '.join('?'*len(ids))
            for lev in levels:
                lname = self._level_name[lev]
                sql = 'DELETE FROM %s%s WHERE data_id in (%s)'\
                        % (add, lname, did_str)
                self.db.execute(sql, ids)

    def _delete_with_op_id(self, sql_res):
        """
        Delete data from op_res with the op_id query result set

        sql_res -- query result with op_ids to delete
        """
        add = self._add
        op_ids = []
        for row in sql_res:
            op_ids.append(row[0])
        sets = int(math.ceil(len(op_ids)/1000.))
        for i in range(sets):
            ids = op_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            oid_str = ', '.join('?'*len(ids))
            sql = 'DELETE FROM %sop_res WHERE op_id in (%s)'\
                    % (add, oid_str)
            self.db.execute(sql, ids)

    def _duplicate_old_branch(self, obj_id, lind, iterr, from_branch,
                              to_branch, bdate):
        """
        Copies the data for an existing branch to the new branch that was
        generated from the old one
        """
        # else, copy data and operation links from the 'from' to 'to' branch
        add = self._add
        lname = self._level_name[lind]

        # data_link
        cand_dids = self.data_id_store[(lname, obj_id, iterr, from_branch)]
        dids = []
        for did in cand_dids:
            if did[0] < bdate:
                dids.append(did)
        insert_sql = 'INSERT INTO %sdata_link VALUES (?, ?, ?, ?, ?, ?)' % add
        for did in dids:
            ddate, data_id = did
            data = (lname, obj_id, iterr, to_branch, ddate, data_id)
            self.db.execute(insert_sql, data)
            did_key = (lname, obj_id, iterr, to_branch)
            self.data_id_store[did_key].append((ddate, data_id))

        # op_link
        cand_opids = self.op_id_store[(lname, obj_id, iterr, from_branch)]
        opids = []
        for opid in cand_opids:
            if opid[0] < bdate:
                opids.append(opid)
        qmarks = '?, ?, ?, ?, ?, ?, ?, ?, ?, ?'
        op_insert_sql = 'INSERT INTO %sop_link VALUES (%s)' % (add, qmarks)
        # id, iteration, branch, op_date, op_id, o_op_id, op_level, op_name,
        # op_group, op_type
        for opid in opids:
            ddate, op_id, orig_op_id, op_name, op_group, op_type = opid
            data = (obj_id, iterr, to_branch, ddate, op_id, orig_op_id, lname,
                    op_name, op_group, op_type)
            self.db.execute(op_insert_sql, data)
            op_id_key = (lname, obj_id, iterr, to_branch)
            op_data = (ddate, op_id, orig_op_id, op_name, op_group, op_type)
            self.op_id_store[(op_id_key)].append(op_data)

    def _fill_children(self, tind, pl_ind, full_pid, data_id, ddate,
                       lineage=None):
        """
        Gets the children for the given object and adds those into the
        matrix managed by the handler. Recurses into children of children
        Note: relies on Python list sorting and db's ORDER BY having the same
        sorting order
        """
        add = self._add
        add_obj = self._handler.add_objects
        children = self._get_children(pl_ind, full_pid[0], data_id, lineage)
        for cl_ind, child_ids in children:
            if len(child_ids)==0:
                continue
            child_ids.sort()
            pind = tind[0, 2]
            num = len(child_ids)
            tind = add_obj(self._iterr, self._branch, cl_ind, num, pl_ind,
                           pind, child_ids)
            cl_name = self._level_name[cl_ind]
            sql = 'SELECT * FROM %s WHERE data_id=? AND pid=? ORDER BY id'\
                    % (add + cl_name,)
            data = self._get_star_data(cl_name, sql, (data_id, full_pid[0],))
            attrs = self.table_def[cl_name]['float attr ind']
            to_remove = self.table_def[cl_name]['to remove']
            self._set_matrix_data(cl_ind, tind, attrs, data, to_remove)
            self._set_text_data(cl_ind, tind, to_remove, data, '_fill_children')
            for ind, child_id in enumerate(child_ids):
                obj_tind = tind[ind,:]
                obj_tind.shape = (1, len(obj_tind))
                self._fill_children(obj_tind, cl_ind, child_id, data_id, ddate)

    def _fill_sim_level(self, tind, lind, obj_id, data_id, ddate):
        """
        At or below simulation level object data to matrix
        """
        add = self._add
        lname = self._level_name[lind]
        pl_ind = self.hierarchy[lind]['parent']
        if pl_ind in self._no_data_levels:
            pl_ind = self._hop_to_parent(pl_ind)
        pind = tind[0, 2]
        tind = self._handler.add_objects(self._iterr, self._branch, lind, 1,
                                         pl_ind, pind, [obj_id, ])
        sql = 'SELECT * FROM %s WHERE data_id=? AND id=?'  % (add + lname)
        sql_data = (data_id, obj_id[0])
        data = self._get_star_data(lname, sql, sql_data)
        attrs = self.table_def[lname]['float attr ind']
        to_remove = self.table_def[lname]['to remove']
        self._handler.set_date(tind, ddate, True)
        self._set_matrix_data(lind, tind, attrs, data, to_remove)
        self._set_text_data(lind, tind, to_remove, data, '_fill_sim_level')
        return tind

    def _fill_parents(self, lind, obj_id, data_id, ddate, orig_lin):
        """
        Finds out the parent objects for the given object and writes their data
        into the matrix managed by the handler in top down order
        If there are roots for other lineages in the parents, those lineages
        are written as well
        """
        add = self._add
        add_obj = self._handler.add_objects
        parents = []
        parents = self._get_parent(lind, obj_id[0], parents)
        parents.sort()
        tind = None
        for ordinal, lind, lname, uid in parents:
            oid_sql = 'SELECT DISTINCT oid FROM %s WHERE id=?' % (add + lname)
            if (lname, uid) not in self._filled_parents:
                if ordinal==0:
                    pl_ind, pind = None, None
                else:
                    pl_ind = self.hierarchy[lind]['parent']
                    # TODO: this is a bug that will surface when there are
                    # multiple levels above simulation level
                    pind = tind[0, 2]
                oid = self.db.get(oid_sql, (uid, ))[0][0]
                tind = add_obj(self._iterr, self._branch, lind, 1, pl_ind,
                               pind, [(uid, oid)])
                self._filled_parents[(lname, uid)] = tind
                sql = 'SELECT * FROM %s WHERE id=?' % (add + lname)
                data = self._get_star_data(lname, sql, (uid, ))
                attrs = self.table_def[lname]['float attr ind']
                to_remove = self.table_def[lname]['to remove']
                self._set_matrix_data(lind, tind, attrs, data, to_remove)
                self._set_text_data(lind, tind, to_remove, data, '_fill_parents')
                # test for other lineage
                my_lin = self.hierarchy[lind]['lineage']
                other_lin = my_lin.difference(orig_lin)
                for lineage in other_lin:
                    self._fill_children(tind, lind, (uid, oid), data_id, ddate,
                                        lineage)
            else:
                tind = self._filled_parents[(lname, uid)]
        # tind is from whatever is the nearest parent in the tree.
        return tind

    def _flatten(self, l, ltypes=(list, tuple)):
        """
        Flattens a nested list/tuple into an unnested list/tuple
        """
        ltype = type(l)
        l = list(l)
        i = 0
        while i < len(l):
            while isinstance(l[i], ltypes):
                if not l[i]:
                    l.pop(i)
                    i -= 1
                    break
                else:
                    l[i:i + 1] = l[i]
            i += 1
        return ltype(l)

    def _get_branch_desc(self, uid, iteration, to_branch):
        """
        Recursively get the description to a specific id, iteration, branch
        combination
        """
        if to_branch == 0:
            return ''
        data = self.binfo_store[uid]
        hits = 0
        for row in data:
            iterr, from_b, to_b, bdate, op_grp, op, bname, bgname = row
            if iterr == iteration and to_b == to_branch:
                from_branch = from_b
                desc = bname
                hits += 1
        if hits != 1:
            raise Exception('BRANCHING_INFO integrity is broken!')
        return '|'.join((self._get_branch_desc(uid, iteration, from_branch),
                         desc)).strip('| ')

    def _get_children(self, lind, uid, data_id, lineage=None):
        """
        For the given level and object, get all of its immediate children when
        no lineage is defined. Only from the given lineage if it is defined.
        """
        add = self._add
        children = []
        clev = self.hierarchy[lind]['children']
        if clev is None:
            return children
        clevs = []
        for i in range(lind, len(self.hierarchy)):
            clev = self.hierarchy[i]['children']
            if clev is None:
                continue
            for cl_ind in clev:
                if cl_ind not in self._data_exists:
                    if cl_ind not in self._level_name:
                        # this can happen in the case that a level is defined
                        # in the lexicon, but is not used in the simulation
                        # for some reason
                        continue
                    lname = self._level_name[cl_ind]
                    sql = 'SELECT id FROM %s WHERE data_id=?' % (add + lname)
                    ids = list(self.db.get(sql, (data_id,)))
                    if len(ids) == 0:
                        self._no_data_levels.add(cl_ind)
                        continue
                    self._data_exists.add(cl_ind)
                clevs.append(cl_ind)
            if len(clevs) > 0:
                break
        for cl_ind in clevs:
            if lineage is not None:
                clin = self.hierarchy[cl_ind]['lineage']
                if isinstance(lineage, int):
                    if lineage not in clin:
                        continue
                elif len(lineage & clin) == 0:
                    continue
            lname = self._level_name[cl_ind]
            sql = 'SELECT id, oid FROM %s '\
                  'WHERE pid=? AND data_id=?' % (add + lname)
            cids = list(self.db.get(sql, (uid, data_id,)))
            children.append((cl_ind, cids))
        return children

    def _get_children_id(self, lind, oid, ids=None, ddate=None, lineage=None,
                         child_levs=None, data_id=None):
        """
        Queries the database for all children of the given object on all
        levels below the given level

        NOTE: If data_id is given, ignore ddate iteration and branch.
        """
        add = self._add
        if ids is None:
            ids = list([])
        clev = self.hierarchy[lind]['children']
        if clev is None:
            return ids
        if child_levs is not None:
            clev = set(clev) & set(child_levs)
        for lev in clev:
            if lineage is not None:
                clin = self.hierarchy[lev]['lineage']
                if not lineage in clin:
                    continue
            lname = self._level_name[lev]
            if data_id is not None:
                sql = 'SELECT DISTINCT id FROM %s '\
                      'WHERE data_id=? AND pid=?' % (add + lname,)
                sql_data = (data_id, oid)
            elif ddate is None:
                sql = 'SELECT DISTINCT l.id FROM %s l '\
                      'JOIN %sdata_link d '\
                        'ON l.data_id=d.data_id WHERE d.iteration=? AND '\
                        'd.branch=? AND l.pid=?' % (add + lname, add)
                sql_data = (self._iterr, self._branch, oid)
            else:
                sql = 'SELECT DISTINCT l.id FROM %s l '\
                      'JOIN %sdata_link d '\
                        'ON l.data_id=d.data_id WHERE d.iteration=? AND '\
                        'd.branch=? AND l.pid=? AND d.data_date=?' \
                        % (add + lname, add)
                sql_data = (self._iterr, self._branch, oid, ddate)
            children = self.db.get(sql, sql_data)
            for child in children:
                ids.append((lname, child[0]))
                self._get_children_id(lev, child[0], ids,
                                      child_levs=child_levs,
                                      data_id=data_id)
        return ids

    def _get_all_child_levels(self, current_hierarchy):
        """Recursively gets child levels starting from the given hierarchy"""
        children = current_hierarchy['children']
        result = set(children) if children is not None else set()
        if len(result) == 0:
            return result
        for child in children:
            result = result.union(self._get_all_child_levels(
                                                        self.hierarchy[child]))
        return result

    def _get_constrained_hierarchy(self, hierarchy, constraints):
        if constraints is None:
            return self._deepcopy(hierarchy)
        result = {}
        for key, data in hierarchy.items():
            data = self._deepcopy(data)
            if key not in constraints and data['ordinal'] > 0:
                continue
            result[key] = data
            if data['children'] is None:
                continue
            children = []
            for child in data['children']:
                if child in constraints:
                    children.append(child)
            data['children'] = children
        return result

    def _get_child_levels(self, current_hierarchy):
        return current_hierarchy['children']

    def _get_join_strings(self, main_level, levels):
        add = self._add
        if len(levels) == 1:
            return ''
        if 'op_link' in levels and 'op_res' in levels:
            levels = [l for l in levels if l != 'op_link']
        if 'op_res' in levels and len(levels) > 2:
            raise Exception('Can not handle op_res with '\
                            'more than one join level (got %s)' % (levels,))
        if main_level == 'op_res':
            for level in levels:
                if level != main_level:
                    return ' JOIN %s ON op_link.id=%s.id'\
                           ' JOIN op_link ON op_res.op_id=op_link.op_id'\
                           .replace('op_link', add + 'op_link')\
                           .replace('op_res', add + 'op_res')\
                           % (add + level, add + level)
        if main_level == 'op_link':
            for level in levels:
                if level != main_level:
                    return ' JOIN %s ON op_link.id=%s.id'\
                           ' JOIN op_res ON op_res.op_id=op_link.op_id'\
                           .replace('op_link', add + 'op_link')\
                           .replace('op_res', add + 'op_res')\
                           % (add + level, add + level)
        if 'op_res' in levels:
            return ' JOIN op_link ON op_link.id=%s.id'\
                   ' JOIN op_res ON op_res.op_id=op_link.op_id'\
                   .replace('op_link', add + 'op_link')\
                   .replace('op_res', add + 'op_res') % (add + main_level,)
        level_hierarchy = [l for i,l in sorted(self._level_name.items())]
        started = False
        main_hit = False
        join_levels = []
        count = 0
        last_level = ''
        for level in level_hierarchy:
            if level in levels:
                started = True
            if started:
                count += 1
                if level == main_level:
                    main_hit = True
                if last_level != '':
                    join_levels.append([last_level, level, main_hit])
                last_level = level
            if count == len(levels):
                break
        result = ''
        for last, level, main_hit in join_levels:
            last = add + last
            level = add + level
            if main_hit:
                result += ' JOIN %s ON %s.id=%s.pid' \
                            % (last, last, level)
            else:
                result += ' JOIN %s ON %s.id=%s.pid' \
                            % (level, last, level)
        return result

    def _get_parent(self, lind, oid, parents):
        """
        For the given level and object, get all of its parents from all parent
        levels
        """
        add = self._add
        lname = self._level_name[lind]
        if self.db_class != 'ORACLE':
            sql = 'SELECT pid FROM %s where id=? LIMIT 1' % (add + lname)
        else:
            sql = 'SELECT pid FROM %s where id=? AND rownum=1' % (add + lname)
        pid = self.db.get(sql, (oid, ))[0]
        if pid is None:
            # for result dbs where the content from main level up is not there
            for ind in range(len(parents)):
                if parents[ind][2]==lname:
                    parents.pop(ind)
                    break
            return parents
        else:
            pid = pid[0]
        plev = self.hierarchy[lind]['parent']
        if plev is None:
            return parents
        else:
            plname = self._level_name[plev]
            ordinal = self.hierarchy[plev]['ordinal']
            parents.append((ordinal, plev, plname, pid))
            parents = self._get_parent(plev, pid, parents)
        return parents

    def _get_parent_id(self, lind, oind):
        """
        Based on the object identification, gets the parent id
        """
        if lind==0:
            return None
        plev = self.hierarchy[lind]['parent']
        pind = self._relatives[(lind, oind)][plev]
        if len(pind)!=1:
            err_msg = 'Item at (iteration, branch, level, index)=%s, %s, %s,'\
                ' %s has no or multiple parents' \
                % (self._iterr, self._branch, lind, oind)
            self.logger.log_message('db', 'error', err_msg)
            raise IndexError(err_msg)
        puid, poid = self._ids[(plev, pind[0])]
        return puid

    def _get_parent_level(self, my_hierarchy):
        return my_hierarchy['parent']

    def _get_star_data(self, lname, sql, sql_data=None, fetch_one=False):
        """
        Queries the data without the geometry column and returns it in the
        order a simo matrix expects it.
        """
        # NOTE: the matrix column string does NOT have the geometry column.
        sql = sql.replace('*', self.table_def[lname]['matrix column string'])
        if fetch_one:
            data = self.db.get(sql, sql_data)
            if len(data) == 0:
                data = []
            elif data[0] is None:
                data = []
            else:
                data = [data[0]]
        else:
            data = self.db.get(sql, sql_data)
        return data

    def _get_table_columns(self, table_name):
        """Get the columns of a table. this WILL add the prefix, so don't
        put that to the table_name!"""
        add = self._add
        if self.db_class == 'SQLITE':
            sql = 'PRAGMA table_info("%s")' % (add + table_name,)
            metadata = self.db.get(sql)
            metadata = [m[1] for m in metadata]
        elif self.db_class == 'POSTGRESQL':
            sql = "SELECT column_name FROM information_schema.columns "\
                  "WHERE table_name = '%s' AND table_schema='%s'"\
                  % (add + table_name, self.db.schema)
            metadata = self.db.get(sql)
            metadata = [m[0] for m in metadata]
        elif self.db_class == 'ORACLE':
            sql = 'SELECT COLUMN_NAME '\
                  'FROM USER_TAB_COLUMNS '\
                  "WHERE TABLE_NAME = '%s'" % ((add + table_name).upper(),)
            metadata = self.db.get(sql)
            metadata = [m[0] for m in metadata]
        return metadata

    def _get_year_branches(self, main_oid, branch, iteration, max_date):
        add = self._add
        sql = 'SELECT id, from_branch, to_branch, data_date '\
              'FROM %sBRANCHING_INFO '\
              'WHERE id=? and iteration=? and data_date<=?'\
              ' ORDER BY data_date DESC, to_branch DESC' % add
        branching = self.db.get(sql, (main_oid, iteration, max_date))
        result = []
        result.append(['branch=? AND data_date>=? AND data_date<=?',
                       [branch, max_date]])
        for id, from_branch, to_branch, bdate in branching:
            if to_branch == branch:
                result[-1][1].insert(1, bdate) #add lower limit
                result.append(['branch=? AND data_date>=? AND data_date<?',
                               [from_branch, bdate]])
                branch = from_branch
        if len(result) > 1:
            result[-1][0] = 'branch=? AND data_date<?'
        elif len(result) > 0:
            result[-1][0] = 'branch=? AND data_date<=?'
        return result

    def _get_data_hash(self, lind, uid, iterr, branch, ddate):
        """
        Construct a hash string from data object key

        lind -- level ind as integer
        uid -- unique id as string
        iterr -- iteration as integer
        branch -- branch as integer
        ddate -- data date as datetime object
        """
        return "%d%si%db%d%s" % (lind, uid, iterr, branch, str(ddate))

    def _get_op_hash(self, uid, iterr, branch, ddate, op_name, counter=0):
        """
        Construct a hash string from operation key

        uid -- unique id as string
        iterr -- iteration as integer
        branch -- branch as integer
        ddate -- data date as datetime object
        op_name -- operation name as string
        counter -- optional counter argument
        """
        return "%si%db%d%s%s%d" % (uid, iterr, branch, str(ddate), op_name,
                                   counter)

    def _encode_hash(self, hash):
        """
        Compute the md5 hash of a  string

        hash -- hash string
        """
        return hashlib.md5(hash.encode('utf8')).hexdigest()

    def _hop_to_parent(self, lind):
        """
        Finds the first parent level that actually has objects in it

        lind -- level index
        """
        pl_ind = self.hierarchy[lind]['parent']
        if pl_ind in self._no_data_levels:
            pl_ind = self._hop_to_parent(pl_ind)
        return pl_ind

    def _insert_matrix_data(self, data_id, lind, oind, pid, matrix, text_data):
        """
        Gets the sql template from the table definition and inserts the
        values from the matrix to it, as well as the static text data
        """
        add = self._add
        orig_iter = self._orig_iter
        branch = self._branch
        try:
            uid, oid = self._ids[(lind, oind)]
        except:
            raise Exception(str(self._ids)+' '+str((lind, oind)))
        level = self._convert_to_level_name(lind)
        tdef = self.table_def[level]
        data = matrix[orig_iter, branch, lind, oind,
                      tdef['float attr ind']]
        data = numpy.asarray(data, dtype=float)
        cond = (numpy.isnan(data)) | (numpy.isinf(data))
        if True in cond:
            data = numpy.array(data, dtype=object)
            data[cond] = None
        data = list(data)
        #for i, d in enumerate(data):
        #    if d is not None:
        #        data[i] = float(d)
        #    else:
        #        data[i] = None
        sql_data = [data_id, uid, oid, pid] + list(data)
        sql = tdef['float insert sql']

        my_ordinal = self.hierarchy[lind]['ordinal']
        if my_ordinal < self.sim_ordinal:
            control = ['skip_existing', (level, uid, data_id)]
            # No data_ids for parents.
            sql_data[0] = None
        else:
            control = None
        sql = self._inject_text_data(lind, oind, text_data, sql, sql_data)
        self.db.execute(sql, sql_data, control=control)
        # put other attributes back to base level object if it's updated
        if self._update and lind==self._main_level:
            ov = self._other_attr.get(uid)
            if ov is not None:
                oa = self.table_def[level]['other attr name']
                pairs = []
                for ind in range(len(oa)):
                    if ov[ind] is None:
                        val = 'Null'
                    else:
                        val = "'" + ov[ind] + "'"
                    pairs.append('"'+oa[ind]+'"' + '=' + val)
                other_attr = ', '.join(pairs)
                sql = 'UPDATE %s SET %s WHERE data_id=? AND id=?' \
                        % (add + level, other_attr)
                self.db.execute(sql, (data_id, uid))
            geom = self._geom.get(uid)
            if geom is not None:
                sql = self.table_def[level]['geom update sql']
                if self.db_class in ('SQLITE', 'ORACLE'):
                    sql_data = (geom[0], geom[1], data_id, uid)
                elif self.db_class == 'POSTGRESQL':
                    sql_data = (geom[0], data_id, uid)
                self.db.execute(sql, sql_data)

    def _inject_text_data(self, lind, oind, text_data, sql, sql_data):
        """
        Modify the data insert sql and data so that the text values are also
        inserted

        lind -- level index
        oind -- object index
        text_data -- static text data for objects
        sql -- insert sql to modify
        sql_data -- insert data to modify
        """
        level_data = text_data.get(lind)
        if level_data is None:
            return sql
        obj_data = level_data.get(oind)
        if obj_data is None:
            return sql
        # text data is a list of (attribute name, value) tuples
        lname = self._level_name[lind]
        my_attrs = self.table_def[lname]['attr name']
        for attr, val in obj_data.iteritems():
            if attr not in my_attrs:
                continue
            with_this = ', "%s") VALUES' % (attr,)
            sql = sql.replace(') VALUES', with_this)
            sql = sql.replace(', ?)', ', ?, ?)')
            sql_data.append(val)
        return sql

    def _map_attr_to_table(self, lname, attr):
        """
        Sorts the data dictionary attribute tuples of (name, value) into
        correct order and constructs a value list matching that order
        """
        tdef_attr = self.table_def[lname]['attr name']
        data = dict(attr)
        vals = []
        geom = None
        for a in tdef_attr:
            if a == 'geom':
                continue
            vals.append(data.get(a))
        if self.has_geom[lname]:
            geom = data.get('geom', None)
        return vals, geom

    def _remove_old(self, links, ind2id, blocked, terminated):
        """
        Removes all the data for the main level objects and their child
        objects found in new data.
        """
        add = self._add
        self._other_attr = {}
        self._geom = {}
        level = self._convert_to_level_name(self._main_level)
        di_sql = 'SELECT DISTINCT data_id FROM %sdata_link WHERE id=? AND '\
                'iteration=? AND branch=?' % add
        dl_sql = 'DELETE FROM %sdata_link WHERE id=? AND iteration=? AND '\
                'branch=?' % add
        other_attr = self.table_def[level]['other attr name']
        oa = ', '.join(['"' + attr + '"' for attr in other_attr])
        oa_sql = 'SELECT %s FROM %s WHERE data_id=?' % (oa, (add + level))
        geom_sql = 'SELECT AsText(geom), SRID(geom) from %s WHERE data_id=?' \
                % level
        # skip objects that were blocked but not those that were terminated
        # (due to too many simulation errors)
        skip = blocked.difference(terminated)
        for iter_branch, relatives in links.items():
            iterr, branch = iter_branch
            base_obj = relatives[(0,0)][self._main_level]
            for oind in base_obj:
                # do not remove blocked objects (as the blocked objects won't
                # get new values anymore)
                if (iterr, branch, oind) in skip:
                    continue
                objkey = (self._main_level, oind)
                uid, oid = ind2id[iter_branch][objkey]
                # affected data_ids
                sql_data = (uid, int(iterr), int(branch))
                res = self.db.get(di_sql, sql_data)
                if not res:
                    # NB: For some reason object was not in skip, but still
                    # isn't in the database
                    continue
                # store geom + text data that's not in matrix
                # NB! Only geared up to treat single values (i.e. not
                # differences between branches)
                if len(other_attr) > 0:
                    other_val = self.db.get(oa_sql, res[0])[0]
                    self._other_attr[uid] = other_val
                if self.table_def[level]['geom type'] is not None:
                    geom = self.db.get(geom_sql, res[0])[0]
                    self._geom[uid] = geom
                # remove old row(s)
                self._delete_with_data_id(res)
                self.db.execute(dl_sql, sql_data)

    def _set_matrix_data(self, lind, tind, attrs, data, to_remove):
        """
        Writes the queried data into the matrix after stripping the key columns
        from the data
        """
        values = []
        for row in data:
            vals = list(row[DATA_COL:])
            for ind in to_remove:
                vals.pop(ind)
            if vals!=[]:
                values.append(vals)
        if len(values)>0:
            unturned_values = numpy.array(values, dtype=float)
            turned_values = unturned_values.transpose()
            self._handler.set_value(lind, tind, attrs, turned_values)

    def _set_text_data(self, lind, tind, attr_ind, data, call_from=None):
        """
        Store text data for the objects in the internal text data store for simulation

        lname -- level name
        lind -- level index
        tind -- object indices
        attr_ind -- text attribute indices
        data -- all data for the objects
        """
        lname = self._level_name[lind]
        attr_name = self.table_def[lname]['attr name']
        values = {}
        for row in data:
            vals = list(row[DATA_COL:])
            for ind in attr_ind:
                values[attr_name[ind]] = vals[ind]
        if len(values)>0:
            if lind not in self._handler.text_data:
                self._handler.text_data[lind] = {}
            for oind in tind[:, 2]:
                self._handler.text_data[lind][oind] = values

    def _set_meta_tables(self, level_meta):
        """
        Writes numerical and categorical variable definitions to the meta
        tables
        """
        for level, ldata in level_meta.items():
            for item in ldata.numerical:
                self._add_numerical_data(level, ldata.variables[item])
            for item in ldata.categorical:
                self._add_categorical_data(level, ldata.variables[item])
        self.db.commit()

    def _split_content_def(self, cdef, lconst=None):
        """
        Returns the content def tuples of (attribute name, attribute index)
        split into two lists for names and indices
        """
        name = []
        ind = []
        for adef in cdef:
            if lconst is not None and adef[1] not in lconst:
                continue
            name.append(unicode(adef[0]))
            ind.append(adef[1])
        return name, ind

    def store_timber_prices(self, sim, target, table):
        """
        Write the timber prices in the cash flow table into db for the
        active targets
        """
        add = self._add
        sim_data = sim.data
        sql = 'INSERT INTO %stimber_price (id, iteration, '\
                                          'branch, data_date, ' % add
        sql += ', '.join(['"'+c+'"' for c in table.db_cfiers])
        sql += ', price_table, price) VALUES (?, ?, ?, ?'
        sql += len(table.db_cfiers) * ', ?' + ', ?, ?)'
        if target is None:
            for ind in range(len(sim_data.active_tind)):
                tind = sim_data.active_tind[ind][0:3]
                self._store_prices_for_unit(sql, tind, sim_data, table)
        else:
            tind = sim_data.active_tind[target][0:3]
            self._store_prices_for_unit(sql, tind, sim_data, table)

    def _store_prices_for_unit(self, sql, tind, sim_data, table):
        l_ind = sim_data._main_level
        iterr, branch, obj_ind = tind
        obj_ind, ddate = self._get_main_obj_and_data_date(tind, sim_data)
        uid, oid = sim_data.ind2id.ids[(iterr, branch)][(l_ind, obj_ind)]
        for cfiers, val in table.active_reduced.iteritems():
            main_key = (uid, int(iterr), int(branch), ddate) + cfiers + \
                       (table.name, )
            if main_key not in self.price_tracker:
                self.price_tracker.add(main_key)
                main_key = list(main_key)
                for i, j in enumerate(main_key):
                    if isinstance(j, float):
                        main_key[i] = float(j)
                data = main_key + [float(val)]
                self.db.execute(sql, data)

    def _get_main_obj_and_data_date(self, tind, sim_data):
        """
        Gets the date for the timber prices for the object being stored
        """
        l_ind = sim_data.active_level
        iterr, branch, obj_ind = tind
        if l_ind!=sim_data._main_level:
            links = sim_data.linkage.links[(iterr, branch)][(l_ind, obj_ind)]
            obj_ind = links[sim_data._main_level][0]
        return obj_ind, sim_data.matrix.dmx[obj_ind]

    def _update_geom(self, db, lname, oid, data_id):
        """
        Updates the geometry for the given object from the existing geometry
        in this db to the given db
        """
        add = self._add
        sql = 'SELECT AsText(geom), SRID(geom) from %s WHERE id=? AND '\
                'data_id=?' % (add + lname)
        sql_data = (oid, data_id)
        geom = self.db.get(sql, sql_data)[0]
        try:
            if geom is not None and geom!=(None, None):
                # get the update sql from the target database
                up_sql = db.table_def[lname]['geom update sql']
                if self.db_class in ('SQLITE', 'ORACLE'):
                    up_data = (geom[0], geom[1], data_id, oid)
                elif self.db_class == 'POSTGRESQL':
                    up_data = (geom[0], data_id, oid)
                db.db.execute(up_sql, up_data)
        except Exception, e:
            err_msg = 'update geom (%s, %s, %s, %s, %s) failed with: \n' % \
                       (str(lname), str(oid), data_id) + str(e)
            self.logger.log_message('db', 'error', err_msg)

    def _write_matrix_obj(self, oind, data_dates, matrix, text_data):
        """
        For the given base level object, writes the data from the matrix using
        the matrix index mapping stored in the table def.
        """
        add = self._add
        lind = self._main_level
        my_lin = self.hierarchy[lind]['lineage']
        ddate = data_dates[oind]
        pid = self._get_parent_id(lind, oind)
        obj_key = (lind, oind)
        try:
            uid, oid = self._ids[(lind, oind)]
        except:
            raise Exception(str(self._ids)+' '+str((lind, oind)))
        data_hash = self._get_data_hash(lind, uid, self._iterr, self._branch,
                                        ddate)
        data_id = self._encode_hash(data_hash)
        lname = self._convert_to_level_name(lind)
        data = (lname, uid, self._iterr, self._branch, ddate, data_id)
        self.db.execute(DATA_LINK_INSERT_SQL % add, data)
        did_key = (lname, uid, self._iterr, self._branch)
        self.data_id_store[did_key].append((ddate, data_id))
        # Add parents TODO: This should only be done _once_ for all objects!
        for plind, pinds in sorted(self._relatives[obj_key].items()):
            p_ord = self.hierarchy[plind]['ordinal']
            p_lin = self.hierarchy[plind]['lineage']
            if  p_ord < self.sim_ordinal and p_lin.intersection(my_lin):
                for pind in pinds:
                    if not (plind, pind) in self.parents:
                        self.parents.add((plind, pind))
                    ppid = self._get_parent_id(plind, pind)
                    if not plind in self.static_ids_written:
                        self.static_ids_written[plind] = set([])
                    if not ppid in self.static_ids_written[plind]:
                        self.static_ids_written[plind].add(ppid)
                        self._insert_matrix_data(None, plind, pind, ppid,
                                                 matrix, text_data)
        # Add object
        self._insert_matrix_data(data_id, lind, oind, pid, matrix, text_data)
        # Add children
        for clind, cinds in sorted(self._relatives[obj_key].items()):
            c_ord = self.hierarchy[clind]['ordinal']
            c_lin = self.hierarchy[clind]['lineage']
            if  c_ord > self.sim_ordinal and c_lin.intersection(my_lin):
                for cind in cinds:
                    cpid = self._get_parent_id(clind, cind)
                    self._insert_matrix_data(data_id, clind, cind, cpid,
                                             matrix, text_data)

    def _get_root_level(self):
        """Returns the level that has ordinal 0"""
        for level, data in self.hierarchy.items():
            if data['ordinal'] == 0:
                return level

    def _copy(self, data):
        return copy(data)

    def _deepcopy(self, data):
        return deepcopy(data)


###############################################################################

class LoggerDB(CoreDB):

    def __init__(self, db_name='default.db', wipe=False, memory=False,
                 db_host='localhost', db_port=5432, user='test',
                 pw='test', db_class='SQLITE', no_exec=False,
                 db_database='SIMO',
                 db_schema='', threaded=False, force_prefix=False):
        super(LoggerDB, self).__init__(db_name, wipe, memory, db_host,
                                       db_port, user, pw, db_class,
                                       no_exec=no_exec,
                                       db_database=db_database,
                                       db_schema=db_schema,
                                       threaded=threaded,
                                       force_prefix=force_prefix)
        self._create_log_table()

    def add_log_message(self, rid, level, module, timestamp, sim_id, message):
        add = self._add
        self.db.execute("""INSERT INTO %slog VALUES
                       (?, ?, ?, ?, ?, ?)""" % add,
                       (rid, level, module, timestamp, sim_id, message))
        self.db.commit()

    def clear_data(self):
        """
        Clear the entire log
        """
        add = self._add
        sql = "DELETE FROM %slog" % add
        self.db.execute(sql)
        self.db.commit()

    def drop_id(self, id):
        """
        Clear from the log all rows where sim_id is the given id
        """
        sql = 'DELETE FROM %slog WHERE sim_id=?' % self._add
        self.db.execute(sql, (id,))


    def drop_ids(self, all_ids):
        """
        This method will drop any and all data to the given id from the
        database (in batches of 1000 for oracle compatibility).
        """
        if isinstance(all_ids, set):
            all_ids = tuple(all_ids)
        sets = int(math.ceil(len(all_ids)/1000.))
        for i in range(sets):
            ids = all_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            id_str = ', '.join('?'*len(ids))
            sql = 'DELETE FROM %slog WHERE sim_id in (%s)' \
                  % (self._add, id_str)
            self.db.execute(sql, (ids,))
        self.db.commit()

    def drop_old(self, date, level=None, idless_only=False):
        """
        Clear old log data from the database
        date -- clear data from <= this date
        level -- clear data from <= this level
        idless_only -- clear data only if there is no id associated with it
        """
        level_list = ['debug', 'info', 'warning', 'error', 'critical']
        sql_insert = ''
        if idless_only:
            sql_insert += "sim_id IS NULL AND "
        if level:
            if level not in level_list:
                self.logger.error('Invalid log level to drop (%s)' % (level,))
                return
            lv_ind = level_list.index(level)
            levels = "', '".join(level_list[:lv_ind+1])
            sql_insert += "log_level IN ('%s') AND " % levels
        sql = 'DELETE FROM %slog '\
              'WHERE %stime<=?' % (self._add, sql_insert)
        self.db.execute(sql, (date,))


    def get_log_messages(self, run_id=None, sim_id=None, level=None,
                         module=None, start_time=None, end_time=None,
                         empty_log=False):
        """
        level, module and run_id are tuples of level names used to constrain
        the log output. start_time and end_time are datetime instances to
        constraint the log output"""
        add = self._add
        exec_start = "SELECT time, module, log_level, run_id, sim_id, message "
        if empty_log:
            exec_start = "DELETE "
        strings = ['time BETWEEN ? AND ?']
        log_times = self._get_times()
        if len(log_times)==0 and not empty_log:
            return []
        first_time = log_times[0]
        last_time = log_times[-1] or log_times[0]
        start_time = start_time or first_time
        end_time = end_time or last_time
        if sim_id is not None:
            strings.append("sim_id IN ('%s')" % ("','".join(sim_id),))
        if level is not None:
            strings.append("log_level IN ('%s')" % ("','".join(level),))
        if module is not None:
            strings.append("module IN ('%s')" % ("','".join(module),))
        if run_id is not None:
            strings.append("run_id IN ('%s')" % ("','".join(run_id),))
        if level is not None or module is not None or run_id is not None:
            executable = exec_start + "FROM %slog WHERE %s" % \
                       (add, ' AND '.join(strings))
        else:
            executable = exec_start + \
                        "FROM %slog WHERE time BETWEEN ? AND ?" % add

        result = self.db.get(executable, (start_time, end_time))
        if not empty_log:
            return result

    def store_datarows(self, datarows):
        for row in datarows:
            sql_control, data = row
            sql, control = sql_control
            if not control:
                self.db.executemany(sql, data)
            elif control[0] == 'skip_existing':
                for item in data:
                    self.db.execute(sql, item, control)
            else:
                err = 'Invalid control name %s. Expected "skip_existing"' \
                      % (control[0], )
                raise Exception(err)
        self.db.commit()

    def _create_log_table(self):
        add = self._add
        if not self.db.table_exists('%slog' % add):
            sql = """
                CREATE TABLE %slog(
                run_id TEXT,
                log_level TEXT,
                module TEXT,
                time TIMESTAMP,
                sim_id TEXT,
                message TEXT)""" % (add, )
            if self.db_class == 'ORACLE':
                sql = sql[:-5] + 'NCLOB)'
            self.db.execute(sql)
            self.db.commit()

    def _get_times(self):
        add = self._add
        times = self.db.get('SELECT DISTINCT time FROM %slog' % add)
        times = [time[0] for time in times]
        times.sort()
        return times

###############################################################################

class OperationDB(CoreDB):
    def __init__(self, db_name='default.db', wipe=False, memory=False,
                 db_host='localhost', db_port=5432, user='test',
                 pw='test', db_class='SQLITE', db_database='SIMO',
                 db_schema='', extra_columns=[], threaded=False,
                 force_prefix=False):
        super(OperationDB, self).__init__(db_name, wipe, memory,
                                          db_host, db_port, user, pw, db_class,
                                          db_database=db_database,
                                          db_schema=db_schema,
                                          threaded=threaded,
                                          force_prefix=force_prefix)
        self._extra_columns = extra_columns
        self._create_tables()

    def _create_tables(self):
        add = self._add
        cc = self.db._create_concurrently
        if not self.db.table_exists('%sforced_operation' % add):
            extstr = ""
            if self._extra_columns:
                cols = [(" " * 16 + '"' + v[1] + '"' + " DOUBLE PRECISION") \
                        for v in self._extra_columns]
                rows = ",\n".join(cols)
                extstr += '\n%s,' % rows
            sql = '''CREATE TABLE %sforced_operation
                (op_id INTEGER NOT NULL,
                unit_id TEXT NOT NULL,
                op_level TEXT NOT NULL,
                iteration INTEGER NOT NULL,
                branch INTEGER NOT NULL,
                name TEXT NOT NULL,
                timingtype TEXT NOT NULL,
                timestep INTEGER,
                op_date DATE,
                stepunit TEXT,%s
                CONSTRAINT pk_forced_operation
                    PRIMARY KEY (op_id, unit_id))'''\
                % (add, extstr)
            self.db.execute(sql)
            self.db.commit()

            sql = 'CREATE INDEX %s%sforced_op_index_1 '\
                  'ON %sforced_operation (unit_id)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['ops']['%sforced_op_index_1' % add] = sql

        if not self.db.table_exists('%soperation_chain' % add):
            if self.db_class == 'SQLITE':
                id = SQLITEID
            elif self.db_class == 'POSTGRESQL':
                id = POSTGRESQLID
            elif self.db_class == 'ORACLE':
                id = ORACLEID
            sql = '''CREATE TABLE %soperation_chain
                (%s,
                op_id INTEGER NOT NULL,
                pid TEXT NOT NULL,
                chain_ordinal INTEGER NOT NULL,
                chain TEXT NOT NULL,
                CONSTRAINT fk_pid FOREIGN KEY (op_id, pid) REFERENCES
                %sforced_operation (op_id, unit_id)
                ON DELETE CASCADE)''' % (add, id, add)
            self.db.execute(sql)
            self.db.commit()

            sql = 'CREATE INDEX %s%sop_chain_index_1 '\
                  'ON %soperation_chain (op_id)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['ops']['%sop_chain_index_1' % add] = sql

            sql = 'CREATE INDEX %s%sop_chain_index_2 '\
                  'ON %soperation_chain (pid)' % (cc, add, add)
            self.db.execute(sql)
            self.indices['ops']['%sop_chain_index_2' % add] = sql

            if self.db_class == 'ORACLE':
                self.db.create_autoid('%soperation_chain' % add)
            self._create_any_trigger('forced_operation', 'operation_chain',
                                     ('op_id', 'unit_id'), ('op_id', 'pid'))


    def drop_id(self, id):
        """
        This method will drop any and all data to the given id from the
        database.
        """
        add = self._add
        self.db.execute('DELETE FROM %sforced_operation '\
                        'WHERE unit_id=?' % add, (id,))
        self.db.commit()

    def drop_ids(self, all_ids):
        """
        This method will drop any and all data to the given id from the
        database (in batches of 1000 for oracle compatibility).
        """
        if isinstance(all_ids, set):
            all_ids = tuple(all_ids)
        sets = int(math.ceil(len(all_ids)/1000.))
        for i in range(sets):
            ids = all_ids[i*1000:(i+1)*1000]
            if not ids:
                break
            id_str = ', '.join('?'*len(ids))
            self.db.execute('DELETE FROM %sforced_operation '\
                            'WHERE unit_id in (%s)' % (self._add, id_str), ids)
        self.db.commit()

    def clear_data(self):
        add = self._add
        sql = "DELETE FROM %soperation_chain" % add
        self.db.execute(sql)
        sql = "DELETE FROM %sforced_operation" % add
        self.db.execute(sql)
        self.db.commit()

    def add_operation(self, unit_id, level, iteration, branch, name,
                      timingtype, timestep, date, stepunit, chains,
                      op_id=None, ext_col_vals=None):
        add = self._add
        if op_id is None:
            sql = "SELECT MAX(op_id) FROM %sforced_operation WHERE unit_id=?" \
                  % add
            max_id = self.db.get(sql, (unit_id,))[0][0]
            if max_id is None:
                op_id = 1
            else:
                op_id = max_id + 1
        ext_locs, ext_sql, ext_vals = "", "", []
        if ext_col_vals:
            ext_cols = ext_col_vals.keys()
            ext_locs += ", ?" * len(ext_cols)
            ext_vals = [ext_col_vals[col] for col in ext_cols]
            ext_sql = ", " + ", ".join(['"%s"' % c for c in ext_cols])
        sql = """INSERT INTO %sforced_operation (op_id, unit_id, op_level,
            iteration, branch, name, timingtype, timestep, op_date,
            stepunit%s) """ % (add, ext_sql)
        sql += """VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?%s)""" % ext_locs
        data = tuple([op_id, unit_id, add + level, int(iteration), int(branch),
                      name, timingtype, timestep, date, stepunit] + ext_vals)
        self.db.execute(sql, data)
        chain_ord = 0
        for chain in chains:
            sql = """INSERT INTO %soperation_chain (op_id, pid,
                                                    chain_ordinal, chain)
                     VALUES (?, ?, ?, ?)""" % add
            self.db.execute(sql, (op_id, unit_id, chain_ord, chain))
            chain_ord += 1
        self.db.commit()

    def fill_dictionary(self, levelname, full_ids):
        add = self._add
        ids_string = "','".join([ids[0] for ids in full_ids])
        sql = "SELECT * FROM %sforced_operation WHERE op_level='%s' AND "\
            "unit_id IN ('%s')" % (add, add + levelname, ids_string)
        iterator = self.db.get(sql)
        result = {}
        for item in iterator:
            sql = "SELECT chain FROM %soperation_chain WHERE op_id=? AND "\
                "pid=? ORDER BY chain_ordinal" % add
            iter2 = self.db.get(sql, (item[0], item[1]))
            chains = [chain[0] for chain in iter2]
            # collect the data values (from operation variables and parameters)
            # into a dictionary and pass it to the ForcedOperation instance
            data_values = {}
            if self._extra_columns:
                for i, variable in enumerate(self._extra_columns):
                    value = item[i + 10]
                    if value is not None:
                        data_values[variable] = value
            forced_op = ForcedOperation(item[1], item[0], item[2], item[5],
                                        chains, item[6], item[7], item[8],
                                        item[9], data_values)

            key = (item[3], item[4], item[1])
            if key not in result.keys():
                result[key] = []
            result[key].append(forced_op)
        return result


def main():
    from test import mock_db
    hierarchy = mock_db.hierarchy
    content_def = mock_db.base_cd
    levels = mock_db.base_levels
    opres_def = mock_db.opres_def
    cf_cfiers = mock_db.cf_cfiers
    test = DataDB('write', content_def, hierarchy, levels, opres_def,
            cf_cfiers, mock_db.MockLogger(), 'test_mock.db',
            make_new_db=True, memory=False, spatial=(3067, None))
    test.add_data_from_matrix(mock_db.matrixdates, mock_db.datamatrix,
                    mock_db.Ind2Id(), mock_db.links, [], mock_db.main_level,
                    mock_db.text_data)
    print test.db.get('SELECT * FROM comp_unit')
#    polygon = GeoTypes.OGpolygonFromOGLines(
#                GeoTypes.OGlinearRingFromOGPoints(
#                    GeoTypes.OGpointFromValues(3,4),
#                    GeoTypes.OGpointFromValues(4,4),
#                    GeoTypes.OGpointFromValues(4,3),
#                    GeoTypes.OGpointFromValues(3,4),
#                    ),
#                GeoTypes.OGlinearRingFromOGPoints(
#                    GeoTypes.OGpointFromValues(3,4),
#                    GeoTypes.OGpointFromValues(4,4),
#                    GeoTypes.OGpointFromValues(4,3),
#                    GeoTypes.OGpointFromValues(3,4),
#                    )
#                )
#    multipoly = GeoTypes.OGmultiPolygonFromOGPolygons(polygon)
#    multipoly._srid = SRID
#    multipoly.default_srid = SRID
#    test.db.execute("INSERT INTO comp_unit (id, oid, iteration, branch, "\
#                    "pid, date, geom) VALUES ('rar', 'arr', 0, 0, 'estate1', "\
#                    "'0001-01-01', ?)", (multipoly,))
#    print test.db.get('SELECT * FROM comp_unit')
    return
    import doctest
    doctest.testfile('test/db-py.rst')

if __name__ == '__main__':
    main()


