#!/usr/bin/env python
# -*- coding: utf-8 -*-
import numpy
from datetime import date
from collections import defaultdict
import sys
sys.path.append('../../..')
sys.path.append('../../../..')

class MockLogger(object):
    def __init__(self):
        self.reporter = ''
        self.log = ''

    def log_message(self, name, type, errormsg):
        self.reporter = name
        self.log = errormsg
        print ', '.join([name, type, errormsg])

from simo.builder.lexicon.lexicon import LexiconDef
try:
    tdf = open('../../simulator/xml/schemas/Typedefs_SIMO.xsd')
except:
    try:
        tdf = open('../../../../simulator/xml/schemas/Typedefs_SIMO.xsd')
    except:
        tdf = open('../../../../../simulator/xml/schemas/Typedefs_SIMO.xsd')
typedef = tdf.read()
tdf.close()
try:
    sf = open('../../simulator/xml/schemas/lexicon.xsd')
except:
    try:
        sf = open('../../../../simulator/xml/schemas/lexicon.xsd')
    except:
        sf = open('../../../../../simulator/xml/schemas/lexicon.xsd')
schema = sf.read()
sf.close()
base_xml = '''<rootlevel xmlns="http://www.simo-project.org/simo"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.simo-project.org/simo
    ../schemas/lexicon.xsd">
    <name>simulation</name>
    <num_vars>
        <variable>
            <name>DIAM_CLASS_WIDTH</name>
            <description>Width of a single class...</description>
        </variable>
    </num_vars>
    <cat_vars/>
    <sublevels>
        <sublevel>
            <name>sample_estate</name>
            <type>static</type>
            <num_vars/>
            <cat_vars/>
            <text_vars>
                <variable>
                    <name>EstateName</name>
                    <description>Name of estate</description>
                </variable>
            </text_vars>
            <sublevels>
                <sublevel>
                    <name>sample_esplot</name>
                    <type>static</type>
                    <geom_type>POLYGON</geom_type>
                    <num_vars>
                        <variable>
                            <name>BA</name>
                            <description>Basal area...</description>
                        </variable>
                    </num_vars>
                    <cat_vars/>
                </sublevel>
            </sublevels>
        </sublevel>
        <sublevel>
            <name>estate</name>
            <type>static</type>
            <num_vars/>
            <cat_vars/>
            <text_vars>
                <variable>
                    <name>EstateName</name>
                    <description>Name of estate</description>
                </variable>
            </text_vars>
            <sublevels>
                <sublevel>
                    <name>comp_unit</name>
                    <type>static</type>
                    <geom_type>MULTIPOLYGON</geom_type>
                    <num_vars/>
                    <cat_vars>
                        <variable>
                            <name>SC</name>
                            <description>Site class...</description>
                            <values>
                                <enum>
                                    <value>1</value>
                                    <description>Very good</description>
                                </enum>
                            </values>
                        </variable>
                    </cat_vars>
                    <text_vars>
                        <variable>
                            <name>StandLabel</name>
                            <description>Id text for the stand</description>
                        </variable>
                    </text_vars>
                    <sublevels>
                        <sublevel>
                            <name>stratum</name>
                            <type>dynamic</type>
                            <num_vars>
                                <variable>
                                    <name>N</name>
                                    <description>Number of stems...</description>
                                </variable>
                            </num_vars>
                            <cat_vars>
                                <variable>
                                    <name>SP</name>
                                    <description>Tree species...</description>
                                    <values>
                                        <enum>
                                            <value>1</value>
                                            <description>Pine</description>
                                        </enum>
                                        <enum>
                                            <value>2</value>
                                            <description>Spruce</description>
                                        </enum>
                                    </values>
                                </variable>
                            </cat_vars>
                            <sublevels>
                                <sublevel>
                                    <name>tree</name>
                                    <type>dynamic</type>
                                    <geom_type>POINT</geom_type>
                                    <num_vars>
                                        <variable>
                                            <name>d</name>
                                            <description>Diameter...</description>
                                        </variable>
                                    </num_vars>
                                    <cat_vars/>
                                </sublevel>
                            </sublevels>
                        </sublevel>
                        <sublevel>
                            <name>sample_plot</name>
                            <type>static</type>
                            <geom_type>POLYGON</geom_type>
                            <num_vars>
                                <variable>
                                    <name>BA</name>
                                    <description>Basal area...</description>
                                </variable>
                            </num_vars>
                            <cat_vars/>
                            <sublevels>
                                <sublevel>
                                    <name>sample_tree</name>
                                    <type>dynamic</type>
                                    <num_vars>
                                        <variable>
                                            <name>h</name>
                                            <description>Height...</description>
                                        </variable>
                                    </num_vars>
                                    <cat_vars/>
                                </sublevel>
                            </sublevels>
                        </sublevel>
                    </sublevels>
                </sublevel>
            </sublevels>
        </sublevel>
    </sublevels>
</rootlevel>'''

aug_xml = '''<rootlevel xmlns="http://www.simo-project.org/simo"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.simo-project.org/simo
    ../schemas/lexicon.xsd">
    <name>simulation</name>
    <num_vars>
        <variable>
            <name>DIAM_CLASS_WIDTH</name>
            <description>Width of a single class...</description>
        </variable>
    </num_vars>
    <cat_vars/>
    <sublevels>
        <sublevel>
            <name>sample_estate</name>
            <type>static</type>
            <num_vars/>
            <cat_vars/>
            <text_vars>
                <variable>
                    <name>EstateName</name>
                    <description>Name of estate</description>
                </variable>
            </text_vars>
            <sublevels>
                <sublevel>
                    <name>sample_esplot</name>
                    <type>static</type>
                    <geom_type>POLYGON</geom_type>
                    <num_vars>
                        <variable>
                            <name>BA</name>
                            <description>Basal area...</description>
                        </variable>
                    </num_vars>
                    <cat_vars/>
                </sublevel>
            </sublevels>
        </sublevel>
        <sublevel>
            <name>estate</name>
            <type>static</type>
            <num_vars/>
            <cat_vars/>
            <text_vars>
                <variable>
                    <name>EstateName</name>
                    <description>Name of estate</description>
                </variable>
            </text_vars>
            <sublevels>
                <sublevel>
                    <name>comp_unit</name>
                    <type>static</type>
                    <geom_type>MULTIPOLYGON</geom_type>
                    <num_vars>
                        <variable>
                            <name>BA</name>
                            <description>Basal area...</description>
                        </variable>
                    </num_vars>
                    <cat_vars>
                        <variable>
                            <name>SC</name>
                            <description>Site class...</description>
                            <values>
                                <enum>
                                    <value>1</value>
                                    <description>Very good</description>
                                </enum>
                            </values>
                        </variable>
                        <variable>
                            <name>AnotherCatVar</name>
                            <description>For testing</description>
                            <values>
                                <enum>
                                    <value>1</value>
                                    <description>Very good</description>
                                </enum>
                            </values>
                        </variable>
                    </cat_vars>
                    <text_vars>
                        <variable>
                            <name>StandLabel</name>
                            <description>Id text for the stand</description>
                        </variable>
                        <variable>
                            <name>AnotherTextVar</name>
                            <description>For testing</description>
                        </variable>
                    </text_vars>
                    <sublevels>
                        <sublevel>
                            <name>stratum</name>
                            <type>dynamic</type>
                            <num_vars>
                                <variable>
                                    <name>N</name>
                                    <description>Number of stems...</description>
                                </variable>
                            </num_vars>
                            <cat_vars>
                                <variable>
                                    <name>SP</name>
                                    <description>Tree species...</description>
                                    <values>
                                        <enum>
                                            <value>1</value>
                                            <description>Pine</description>
                                        </enum>
                                        <enum>
                                            <value>2</value>
                                            <description>Spruce</description>
                                        </enum>
                                    </values>
                                </variable>
                            </cat_vars>
                            <sublevels>
                                <sublevel>
                                    <name>tree</name>
                                    <type>dynamic</type>
                                    <geom_type>POINT</geom_type>
                                    <num_vars>
                                        <variable>
                                            <name>d</name>
                                            <description>Diameter...</description>
                                        </variable>
                                    </num_vars>
                                    <cat_vars/>
                                </sublevel>
                            </sublevels>
                        </sublevel>
                        <sublevel>
                            <name>sample_plot</name>
                            <type>static</type>
                            <geom_type>POLYGON</geom_type>
                            <num_vars>
                                <variable>
                                    <name>BA</name>
                                    <description>Basal area...</description>
                                </variable>
                            </num_vars>
                            <cat_vars/>
                            <sublevels>
                                <sublevel>
                                    <name>sample_tree</name>
                                    <type>dynamic</type>
                                    <num_vars>
                                        <variable>
                                            <name>h</name>
                                            <description>Height...</description>
                                        </variable>
                                    </num_vars>
                                    <cat_vars/>
                                </sublevel>
                            </sublevels>
                        </sublevel>
                    </sublevels>
                </sublevel>
            </sublevels>
        </sublevel>
    </sublevels>
</rootlevel>'''

ldef = LexiconDef(typedef)
ldef.schema = schema
ldef.xml = ('Base lexicon', base_xml)
# augmented lexicon has BA added for comp_unit
ldef.xml = ('Augmented lexicon', aug_xml)
base_lex = ldef.obj["Base lexicon"]
aug_lex = ldef.obj["Augmented lexicon"]
base_levels = base_lex.levels
aug_levels = aug_lex.levels

base_cd= {('simulation', 0): [('DIAM_CLASS_WIDTH', 0)],
               ('estate', 1): [('EstateName', 0)],
               ('comp_unit', 2): [('SC', 0), ('StandLabel', 1)],
               ('stratum', 3): [('N', 0), ('SP', 1)],
               ('tree', 4): [('d', 0)],
               ('sample_plot', 5): [('BA', 0)],
               ('sample_tree', 6): [('h', 0)],
               ('sample_estate', 7): [('EstateName', 0)],
               ('sample_esplot', 8): [('BA', 0)],
               }

aug_cd = {('simulation', 0): [('DIAM_CLASS_WIDTH', 0)],
               ('estate', 1): [('EstateName', 0)],
               ('comp_unit', 2): [('BA',  0), ('SC', 1), ('AnotherCatVar', 2),
                                   ('StandLabel', 3), ('AnotherTextVar', 4)],
               ('stratum', 3): [('N', 0), ('SP', 1)],
               ('tree', 4): [('d', 0)],
               ('sample_plot', 5): [('BA', 0)],
               ('sample_tree', 6): [('h', 0)],
               ('sample_estate', 7): [('EstateName', 0)],
               ('sample_esplot', 8): [('BA', 0)],
               }

hierarchy = {0:{'ordinal':0, 'lineage':set([0, 1, 2]), 'parent':None,
                'children':[1, 7]},
             1:{'ordinal':1, 'lineage':set([0, 1]), 'parent':0,
                'children':[2]},
             2:{'ordinal':2, 'lineage':set([0,1]), 'parent':1,
                'children':[3, 5]},
             3:{'ordinal':3, 'lineage':set([0]), 'parent':2, 'children':[4]},
             4:{'ordinal':4, 'lineage':set([0]), 'parent':3, 'children':None},
             5:{'ordinal':3, 'lineage':set([1]), 'parent':2, 'children':[6]},
             6:{'ordinal':4, 'lineage':set([1]), 'parent':5, 'children':None},
             7:{'ordinal':1, 'lineage':set([2]), 'parent':0, 'children':[8]},
             8:{'ordinal':2, 'lineage':set([2]), 'parent':7, 'children':None}}

opres_def = ['SP', 'assortment', 'Volume']
cf_cfiers = set(['SP', 'assortment'])

# =============================================

# This data comes in, when data should be written in tables

matrixdates = [date(1,1,1), date(1,2,2)]
up_dates = [date(2009,1,1), date(2009,2,2)]
main_level = 2

datamatrix = numpy.array(
    [#iteration
# iteration 0
       [#branch
# branch 0
            [#data level #object[attribute, attribute
                #simulation
                [[1.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #comp_unit
                [[10.,  1., 2.], [11.,  1., 2.], [ 0.,  0., 0.],
                 [ 0.,  0., 0.], [ 0.,  0., 0.]],
                #stratum
                [[100., 1.,  0.], [110., 1.,  0.], [120., 1.,  0.],
                 [130., 1.,  0.], [0., 0.,  0.]],
                #tree
                [[11.,  0.,  0.], [12., 0.,  0.], [13., 0.,  0.],
                 [14., 0.,  0.], [15., 0.,  0.]],
                #sample_plot
                [[21., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_tree
                [[31., 0.,  0.], [32., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #sample_esplot
                [[27., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
            ],
# branch 1
            [#data level #object[attribute, attribute
                #simulation
                [[1.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #comp_unit
                [[20.,  2., 3.], [21.,  2., 3.], [ 0.,  0., 0.],
                 [ 0.,  0., 0.], [ 0.,  0., 0.]],
                #stratum
                [[200., 2.,  0.], [210., 2.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #tree
                [[21.,  0.,  0.], [22., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_plot
                [[31., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_tree
                [[41., 0.,  0.], [42., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #sample_esplot
                [[37., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
            ],
       ],
# iteration 1
       [#branch
# branch 0
            [#data level #object[attribute, attribute
                #simulation
                [[ 1.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #comp_unit
                [[ 90.,  9., 8.], [ 91.,  9., 8.], [ 0.,  0., 0.],
                 [ 0.,  0., 0.], [ 0.,  0., 0.]],
                #stratum
                [[900., 9.,  0.], [910., 9.,  0.], [920., 9.,  0.],
                 [930., 9.,  0.], [0., 0.,  0.]],
                #tree
                [[91.,  0.,  0.], [92., 0.,  0.], [93., 0.,  0.],
                 [94., 0.,  0.], [95., 0.,  0.]],
                #sample_plot
                [[91., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_tree
                [[91., 0.,  0.], [92., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
                #sample_estate
                [[ 0.,  0.,  0.], [ 0.,  0.,  0.], [ 0.,  0.,  0.],
                 [ 0.,  0.,  0.], [ 0.,  0.,  0.]],
                #sample_esplot
                [[97., 0.,  0.], [0., 0.,  0.], [0., 0.,  0.],
                 [0., 0.,  0.], [0., 0.,  0.]],
            ],
            [#data level #object[attribute, attribute
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]],
                [[0., 0., 0.], [0., 0., 0.], [0., 0., 0.],
                 [0., 0., 0.], [0., 0., 0.]]
            ],
        ]
    ])


# This is needed to find out the index of the object at the parent
# data level to get the foreign key from ind2id (obviously not for the
# root level objects
#keys: (iteration, branch), (from data level, object), to data level
links = {(0,0):{(0,0):{1:[0], 2:[0,1], 3:[0,1,2,3],
                       4:[0,1,2,3,4], 5:[0], 6:[0,1], 7:[0], 8:[0]},
                (1,0):{0:[0], 2:[0,1], 3:[0,1,2,3], 4:[0,1,2,3,4], 5:[0],
                       6:[0,1]},
                (2,0):{0:[0], 1:[0], 3:[0,1], 4:[0,1], 5:[0], 6:[0,1]},
                (2,1):{0:[0], 1:[0], 3:[2,3], 4:[2,3,4], 5:[], 6:[]},
                (3,0):{0:[0], 1:[0], 2:[0], 4:[0]},
                (3,1):{0:[0], 1:[0], 2:[0], 4:[1]},
                (3,2):{0:[0], 1:[0], 2:[1], 4:[2]},
                (3,3):{0:[0], 1:[0], 2:[1], 4:[3,4]},
                (4,0):{0:[0], 1:[0], 2:[0], 3:[0]},
                (4,1):{0:[0], 1:[0], 2:[0], 3:[1]},
                (4,2):{0:[0], 1:[0], 2:[1], 3:[2]},
                (4,3):{0:[0], 1:[0], 2:[1], 3:[3]},
                (4,4):{0:[0], 1:[0], 2:[1], 3:[3]},
                (5,0):{0:[0], 1:[0], 2:[0], 6:[0,1]},
                (6,0):{0:[0], 1:[0], 2:[0], 5:[0]},
                (6,1):{0:[0], 1:[0], 2:[0], 5:[0]},
                (7,0):{0:[0], 8:[0]},
                (8,0):{0:[0], 7:[0]}
                },
        (0,1):{(0,0):{1:[0], 2:[0,1], 3:[0,1],
                       4:[0,1], 5:[0], 6:[0,1], 7:[0], 8:[0]},
                (1,0):{0:[0], 2:[0,1], 3:[0,1], 4:[0,1], 5:[0],
                       6:[0,1]},
                (2,0):{0:[0], 1:[0], 3:[0,1], 4:[0,1], 5:[0], 6:[0,1]},
                (2,1):{0:[0], 1:[0], 3:[], 4:[], 5:[], 6:[]},
                (3,0):{0:[0], 1:[0], 2:[0], 4:[0]},
                (3,1):{0:[0], 1:[0], 2:[0], 4:[1]},
                (4,0):{0:[0], 1:[0], 2:[0], 3:[0]},
                (4,1):{0:[0], 1:[0], 2:[0], 3:[1]},
                (5,0):{0:[0], 1:[0], 2:[0], 6:[0,1]},
                (6,0):{0:[0], 1:[0], 2:[0], 5:[0]},
                (6,1):{0:[0], 1:[0], 2:[0], 5:[0]},
                (7,0):{0:[0], 8:[0]},
                (8,0):{0:[0], 7:[0]}
                },
        (1,0):{(0,0):{1:[0], 2:[0,1], 3:[0,1,2,3],
                       4:[0,1,2,3,4], 5:[0], 6:[0,1], 7:[0], 8:[0]},
                (1,0):{0:[0], 2:[0,1], 3:[0,1,2,3], 4:[0,1,2,3,4], 5:[0],
                       6:[0,1]},
                (2,0):{0:[0], 1:[0], 3:[0,1], 4:[0,1], 5:[0], 6:[0,1]},
                (2,1):{0:[0], 1:[0], 3:[2,3], 4:[2,3,4], 5:[], 6:[]},
                (3,0):{0:[0], 1:[0], 2:[0], 4:[0]},
                (3,1):{0:[0], 1:[0], 2:[0], 4:[1]},
                (3,2):{0:[0], 1:[0], 2:[1], 4:[2]},
                (3,3):{0:[0], 1:[0], 2:[1], 4:[3,4]},
                (4,0):{0:[0], 1:[0], 2:[0], 3:[0]},
                (4,1):{0:[0], 1:[0], 2:[0], 3:[1]},
                (4,2):{0:[0], 1:[0], 2:[1], 3:[2]},
                (4,3):{0:[0], 1:[0], 2:[1], 3:[3]},
                (4,4):{0:[0], 1:[0], 2:[1], 3:[3]},
                (5,0):{0:[0], 1:[0], 2:[0], 6:[0,1]},
                (6,0):{0:[0], 1:[0], 2:[0], 5:[0]},
                (6,1):{0:[0], 1:[0], 2:[0], 5:[0]},
                (7,0):{0:[0], 8:[0]},
                (8,0):{0:[0], 7:[0]}
                }
        }

# This is used to get the primary and foreign keys
# keys: (iteration, branch), (data level, object)
ind2id = {(0,0):{(0,0):('sim1', 'o-sim1'),
                 (1,0):('estate1','o-estate1'),
                 (2,0):('stand1','o-stand1'),
                 (2,1):('stand2','o-stand2'),
                 (3,0):('stratum1-1','o-stratum1-1'),
                 (3,1):('stratum1-2','o-stratum1-2'),
                 (3,2):('stratum2-1','o-stratum2-1'),
                 (3,3):('stratum2-2','o-stratum2-2'),
                 (4,0):('tree1-1-1','o-tree1-1-1'),
                 (4,1):('tree1-2-1','o-tree1-2-1'),
                 (4,2):('tree2-1-1','o-tree2-1-1'),
                 (4,3):('tree2-2-1','o-tree2-2-1'),
                 (4,4):('tree2-2-2','o-tree2-2-2'),
                 (5,0):('plot1','o-plot1'),
                 (6,0):('sample_tree1-1','o-sample_tree1-1'),
                 (6,1):('sample_tree1-2','o-sample_tree1-2'),
                 (7,0):('sample_estate1','o-sample_estate1'),
                 (8,0):('sample_esplot1-1','o-sample_esplot1-1'),
                 },
         (0,1):{(0,0):('sim1','o-sim1'),
                (1,0):('estate1','o-estate1'),
                (2,0):('stand1','o-stand1'),
                (2,1):('stand2','o-stand2'),
                (3,0):('stratum1-1','o-stratum1-1'),
                (3,1):('stratum1-2','o-stratum1-2'),
                (4,0):('tree1-1-1','o-tree1-1-1'),
                (4,1):('tree1-2-1','o-tree1-2-1'),
                (5,0):('plot1','o-plot1'),
                (6,0):('sample_tree1-1','o-sample_tree1-1'),
                (6,1):('sample_tree1-2','o-sample_tree1-2'),
                (7,0):('sample_estate1','o-sample_estate1'),
                (8,0):('sample_esplot1-1','o-sample_esplot1-1'),
                },
         (1,0):{(0,0):('sim1','o-sim1'),
                (1,0):('estate1','o-estate1'),
                (2,0):('stand1','o-stand1'),
                (2,1):('stand2','o-stand2'),
                (3,0):('stratum1-1','o-stratum1-1'),
                (3,1):('stratum1-2','o-stratum1-2'),
                (3,2):('stratum2-1','o-stratum2-1'),
                (3,3):('stratum2-2','o-stratum2-2'),
                (4,0):('tree1-1-1','o-tree1-1-1'),
                (4,1):('tree1-2-1','o-tree1-2-1'),
                (4,2):('tree2-1-1','o-tree2-1-1'),
                (4,3):('tree2-2-1','o-tree2-2-1'),
                (4,4):('tree2-2-2','o-tree2-2-2'),
                (5,0):('plot1','o-plot1'),
                (6,0):('sample_tree1-1','o-sample_tree1-1'),
                (6,1):('sample_tree1-2','o-sample_tree1-2'),
                (7,0):('sample_estate1','o-sample_estate1'),
                (8,0):('sample_esplot1-1','o-sample_esplot1-1'),
                }
         }

id2ind = {(0,0):{(0,'sim1'):0,
                 (1,'estate1'):0,
                 (2,'stand1'):0, (2,'stand2'):1,
                 (3,'stratum1-1'):0, (3,'stratum1-2'):1, (3,'stratum2-1'):2,
                 (3,'stratum2-2'):3,
                 (4,'tree1-1-1'):0, (4,'tree1-2-1'):1, (4,'tree2-1-1'):2,
                 (4,'tree2-2-1'):3, (4,'tree2-2-2'):4,
                 (5,'plot1'):0,
                 (6,'sample_tree1-1'):0, (6,'sample_tree1-2'):1,
                 (7,'sample_estate1'):0,
                 (8,'sample_esplot1-1'):0,
                 },
          (0,1):{(0,'sim1'):0,
                 (1,'estate1'):0,
                 (2,'stand1'):0, (2,'stand2'):1,
                 (3,'stratum1-1'):0, (3,'stratum1-2'):1,
                 (4,'tree1-1-1'):0, (4,'tree1-2-1'):1,
                 (5,'plot1'):0,
                 (6,'sample_tree1-1'):0, (6,'sample_tree1-2'):1,
                 (7,'sample_estate1'):0,
                 (8,'sample_esplot1-1'):0,
                 },
          (1,0):{(0,'sim1'):0,
                 (1,'estate1'):0,
                 (2,'stand1'):0, (2,'stand2'):1,
                 (3,'stratum1-1'):0, (3,'stratum1-2'):1, (3,'stratum2-1'):2,
                 (3,'stratum2-2'):3,
                 (4,'tree1-1-1'):0, (4,'tree1-2-1'):1, (4,'tree2-1-1'):2,
                 (4,'tree2-2-1'):3, (4,'tree2-2-2'):4,
                 (5,'plot1'):0,
                 (6,'sample_tree1-1'):0, (6,'sample_tree1-2'):1,
                 (7,'sample_estate1'):0,
                 (8,'sample_esplot1-1'):0,
                 },
        }

# Data for add_data_from_dictionary
simulation = \
['simulation', ('DIAM_CLASS_WIDTH',),
[
(u'sim1', u'o-sim1', 0, 1, None, date(1, 1, 1), 1.0),
(u'sim1', u'o-sim1', 0, 1, None, date(1, 2, 2), 1.0),
(u'sim1', u'o-sim1', 1, 0, None, date(1, 1, 1), 1.0),
(u'sim1', u'o-sim1', 1, 0, None, date(1, 2, 2), 1.0),
(u'sim1', u'o-sim1', 0, 0, None, date(1, 1, 1), 1.0),
(u'sim1', u'o-sim1', 0, 0, None, date(1, 2, 2), 1.0),
]
]

estate = \
['estate', ('EstateName',),
[
(u'estate1', u'o-estate1', 0, 0, u'sim1', date(1, 1, 1), None),
(u'estate1', u'o-estate1', 0, 0, u'sim1', date(1, 2, 2), None),
(u'estate1', u'o-estate1', 0, 1, u'sim1', date(1, 1, 1), None),
(u'estate1', u'o-estate1', 0, 1, u'sim1', date(1, 2, 2), None),
(u'estate1', u'o-estate1', 1, 0, u'sim1', date(1, 1, 1), None),
(u'estate1', u'o-estate1', 1, 0, u'sim1', date(1, 2, 2), None),
]
]

comp_unit = \
['comp_unit', ('BA', 'SC', 'StandLabel'),
[
(u'stand1', u'o-stand1', 0, 0, u'estate1', date(1, 1, 1), 10.0, 1.0, 'one'),
(u'stand2', u'o-stand2', 0, 0, u'estate1', date(1, 2, 2), 11.0, 1.0, 'two'),
(u'stand1', u'o-stand1', 0, 1, u'estate1', date(1, 1, 1), 20.0, 2.0, 'three'),
(u'stand2', u'o-stand2', 0, 1, u'estate1', date(1, 2, 2), 21.0, 2.0, 'four'),
(u'stand1', u'o-stand1', 1, 0, u'estate1', date(1, 1, 1), 90.0, 9.0, 'five\''),
(u'stand2', u'o-stand2', 1, 0, u'estate1', date(1, 2, 2), 91.0, 9.0,
                                                            u'six#$!"\'"äö'),
]
]

stratum = \
['stratum', ('N', 'SP'),
[
(u'stratum1-1', u'o-stratum1-1', 0, 0, u'stand1', date(1, 1, 1), 100.0, 1.0),
(u'stratum1-2', u'o-stratum1-2', 0, 0, u'stand1', date(1, 1, 1), 110.0, 1.0),
(u'stratum2-1', u'o-stratum2-1', 0, 0, u'stand2', date(1, 2, 2), 120.0, 1.0),
(u'stratum2-2', u'o-stratum2-2', 0, 0, u'stand2', date(1, 2, 2), 130.0, 1.0),
(u'stratum1-1', u'o-stratum1-1', 0, 1, u'stand1', date(1, 1, 1), 200.0, 2.0),
(u'stratum1-2', u'o-stratum1-2', 0, 1, u'stand1', date(1, 1, 1), 210.0, 2.0),
(u'stratum1-1', u'o-stratum1-1', 1, 0, u'stand1', date(1, 1, 1), 900.0, 9.0),
(u'stratum1-2', u'o-stratum1-2', 1, 0, u'stand1', date(1, 1, 1), 910.0, 9.0),
(u'stratum2-1', u'o-stratum2-1', 1, 0, u'stand2', date(1, 2, 2), 920.0, 9.0),
(u'stratum2-2', u'o-stratum2-2', 1, 0, u'stand2', date(1, 2, 2), 930.0, 9.0),
]
]

tree = \
['tree', ('d', ),
[
(u'tree1-1-1', u'o-tree1-1-1', 0, 0, u'stratum1-1', date(1, 1, 1), 11.0),
(u'tree1-2-1', u'o-tree1-2-1', 0, 0, u'stratum1-2', date(1, 1, 1), 12.0),
(u'tree2-1-1', u'o-tree2-1-1', 0, 0, u'stratum2-1', date(1, 2, 2), 13.0),
(u'tree2-2-1', u'o-tree2-2-1', 0, 0, u'stratum2-2', date(1, 2, 2), 14.0),
(u'tree2-2-2', u'o-tree2-2-2', 0, 0, u'stratum2-2', date(1, 2, 2), 15.0),
(u'tree1-1-1', u'o-tree1-1-1', 0, 1, u'stratum1-1', date(1, 1, 1), 21.0),
(u'tree1-2-1', u'o-tree1-2-1', 0, 1, u'stratum1-2', date(1, 1, 1), 22.0),
(u'tree1-1-1', u'o-tree1-1-1', 1, 0, u'stratum1-1', date(1, 1, 1), 91.0),
(u'tree1-2-1', u'o-tree1-2-1', 1, 0, u'stratum1-2', date(1, 1, 1), 92.0),
(u'tree2-1-1', u'o-tree2-1-1', 1, 0, u'stratum2-1', date(1, 2, 2), 93.0),
(u'tree2-2-1', u'o-tree2-2-1', 1, 0, u'stratum2-2', date(1, 2, 2), 94.0),
(u'tree2-2-2', u'o-tree2-2-2', 1, 0, u'stratum2-2', date(1, 2, 2), 95.0),
]
]

sample_plot = \
['sample_plot', ('BA',),
[
(u'plot1', u'o-plot1', 0, 0, u'stand1', date(1, 1, 1), 21.0),
(u'plot1', u'o-plot1', 0, 1, u'stand1', date(1, 1, 1), 31.0),
(u'plot1', u'o-plot1', 1, 0, u'stand1', date(1, 1, 1), 91.0),
]
]

sample_tree = \
['sample_tree', ('h', ),
[
(u'sample_tree1-1', u'o-sample_tree1-1', 0, 0, u'plot1', date(1, 1, 1), 31.0),
(u'sample_tree1-2', u'o-sample_tree1-2', 0, 0, u'plot1', date(1, 1, 1), 32.0),
(u'sample_tree1-1', u'o-sample_tree1-1', 0, 1, u'plot1', date(1, 1, 1), 41.0),
(u'sample_tree1-2', u'o-sample_tree1-2', 0, 1, u'plot1', date(1, 1, 1), 42.0),
(u'sample_tree1-1', u'o-sample_tree1-1', 1, 0, u'plot1', date(1, 1, 1), 91.0),
(u'sample_tree1-2', u'o-sample_tree1-2', 1, 0, u'plot1', date(1, 1, 1), 92.0),
]
]

sample_estate = \
['sample_estate', ('EstateName',),
[
(u'sample_estate1', u'o-sample_estate1', 0, 0, u'sim1', date(1, 1, 1), None),
(u'sample_estate1', u'o-sample_estate1', 0, 0, u'sim1', date(1, 2, 2), None),
(u'sample_estate1', u'o-sample_estate1', 0, 1, u'sim1', date(1, 1, 1), None),
(u'sample_estate1', u'o-sample_estate1', 0, 1, u'sim1', date(1, 2, 2), None),
(u'sample_estate1', u'o-sample_estate1', 1, 0, u'sim1', date(1, 1, 1), None),
(u'sample_estate1', u'o-sample_estate1', 1, 0, u'sim1', date(1, 2, 2), None),
]
]

sample_esplot = \
['sample_esplot', ('BA', ),
[
(u'sample_esplot1-1', u'o-sample_esplot1-1', 0, 0, u'sample_estate1',
 date(1, 1, 1), 27.0, None),
(u'sample_esplot1-1', u'o-sample_esplot1-1', 0, 0, u'sample_estate1',
 date(1, 2, 2), 27.0, None),
(u'sample_esplot1-1', u'o-sample_esplot1-1', 0, 1, u'sample_estate1',
 date(1, 1, 1), 37.0, None),
(u'sample_esplot1-1', u'o-sample_esplot1-1', 0, 1, u'sample_estate1',
 date(1, 2, 2), 37.0, None),
(u'sample_esplot1-1', u'o-sample_esplot1-1', 1, 0, u'sample_estate1',
 date(1, 1, 1), 97.0, None),
(u'sample_esplot1-1', u'o-sample_esplot1-1', 1, 0, u'sample_estate1',
 date(1, 2, 2), 97.0, None),
]
]

ib = defaultdict(dict)
levels = [simulation, estate, comp_unit, stratum, tree, sample_plot,
          sample_tree, sample_estate, sample_esplot]
for lev in levels:
    level, vars = lev[:2]
    for obj in lev[2]:
        id, oid, iterr, branch, pid, ddate = obj[:6]
        vals = zip(vars, obj[6:])
        unit = (ddate, {'id':id, 'oid':oid, 'parent id': pid, 'values': vals})
        if level not in ib[(iterr, branch)]:
            ib[(iterr, branch)][level] = []
        ib[(iterr, branch)][level].append(unit)
data_dict = dict(ib)


class Ind2Id(object):
    ids = ind2id
    inds = id2ind

    def get_id(self, iter, branch, level, obj):
        print 'Called Ind2Id.get_id(%d, %d, %d, %d)' \
                % (iter, branch, level, obj)
        print 'return', ind2id[iter, branch][level, obj]
        return ind2id[iter, branch][level, obj]

    def get_ind(self, iter, branch, level, obj):
        print 'Called Ind2Id.get_ind(%d, %d, %d, %s)'\
                % (iter, branch, level, obj)
        if (level, obj) not in id2ind[iter, branch]:
            print 'Bad (level, obj): (', level,',', obj,')'
            return None
        print 'return', id2ind[iter, branch][level, obj]
        return id2ind[iter, branch][level, obj]


# Matrix handler mock for fill_matrix
i2i = Ind2Id()

text_data = {2: {0: ('StandLabel', 'kuvio1 #€@"\'')}}

class Handler(object):
    ind2id = i2i
    text_data = text_data
    mx = []
    dmx = []
    added_ids = []

    def add_objects(self, iter, branch, lind, num, pl_ind, pind, obj_id):
        self.added_ids.append(obj_id)
        tind = numpy.array(3 * num * [0], dtype=int)
        tind.shape = (num, 3)
        for ind in range(num):
            tind[ind, 2] = ind
        return tind

    def set_date(self, tind, ddate, init):
        self.dmx.append((tind, ddate))

    def set_value(self, lind, tind, attrs, turned_values):
        self.mx.append((lind, tind[:,2], attrs, turned_values))

handler = Handler()

if __name__ == '__main__':
    pass
