#!/usr/bin/env python
# encoding: utf-8
"""
*mock4importdata.py*

Mock objects for importdata testing

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""
from datetime import date
from minimock import Mock
import StringIO

##############################################################################
# for __init__
##############################################################################
inputdb = Mock('DataDB')
inputdb.get_data_from_level.mock_returns = [(date(2009, 5, 6),),
                                            (date(2009, 1, 1),),
                                            (date(2009, 12, 31),)]
global count
count = -1
def row_count(level):
    global count
    count += 1
    return count
inputdb.row_count.mock_returns_func = row_count
comp_unit = Mock('LevelDef')
comp_unit.level_name = 'comp_unit'
comp_unit.rowtype_value = [1, 4]
comp_unit.id_pos = [0]
comp_unit.id_delimiter = None
comp_unit.linkid_pos = None
comp_unit.rowtype_pos = 1
comp_unit.new_object_row = 1
comp_unit.date_pos = 3

stratum = Mock('LevelDef')
stratum.level_name = 'stratum'
stratum.rowtype_value = [2]
stratum.id_pos = [0]
stratum.id_delimiter = None
stratum.linkid_pos = ('comp_unit', 1)
stratum.rowtype_pos = 1
stratum.new_object_row = None
stratum.date_pos = None

main_group = Mock('Mapping')
main_group.var_type = 'categorical'
main_group.from_data_type = 'int'
v1 = Mock('ValueConversion')
v1.var_ind = (1, 1)
v1.to_var = 'MAIN_GROUP'
v1.to_val = 1
v1_1 = Mock('ValueConversion')
v1_1.var_ind = (1, 5)
v1_1.to_var = 'SOMETHING_ELSE'
v1_1.to_val = 99
v2 = Mock('ValueConversion')
v2.var_ind = (1, 1)
v2.to_var = 'MAIN_GROUP'
v2.to_val = 4
main_group.map = {1: [v1, v1_1], 2: [v2]}
dev_class = Mock('Mapping')
dev_class.var_type = 'categorical'
dev_class.from_data_type = 'int'
vd1 = Mock('ValueConversion')
vd1.var_ind = (1, 1)
vd1.to_var = 'DEV_CLASS'
vd1.to_val = 1
vd2 = Mock('ValueConversion')
vd2.var_ind = (1, 1)
vd2.to_var = 'DEV_CLASS'
vd2.to_val = 2
dev_class.map = {1: [vd1], 2: [vd2]}
inventory_date = Mock('Mapping')
inventory_date.var_type = 'date'
v3 = Mock('ValueConversion')
v3.var_ind = (1, 2)
v3.to_var = 'Inventory_date'
v3.epoch_year = 'gregorian'
inventory_date.map = [v3]
v4 = Mock('ValueConversion')
v4.var_ind = (2, 1)
v4.to_var = 'BA'
v4.conv_fact = 10.0
basal_area = Mock('Mapping')
basal_area.var_type = 'numerical'
basal_area.map = [v4]
v5 = Mock('ValueConversion')
v5.var_ind = (2, 2)
v5.to_var = 'BT'
basal_text = Mock('Mapping')
basal_text.var_type = 'text'
basal_text.map = [v5]
v6 = Mock('ValueConversion')
v6.var_ind = (2, 3)
v6.to_var = 'ORIG_DC'
v6.conv_fact = 1.0
orig_dc = Mock('Mapping')
orig_dc.var_type = 'numerical'
orig_dc.map = [v6]

mapping = Mock('ConversionMapping')
mapping.level_list = [comp_unit, stratum]
mapping.none_val = ["''", "-1"]
mapping.object_rejection = {'MAIN_GROUP': [{'oper': 'in',
                            'criteria': [4, 5, 6, 7, 8]}]}
mapping.comment_prefix = ['!', '#']
mapping.mapping = {1: {2: [main_group],
                       3: [inventory_date],},
                   4: {2: [dev_class, orig_dc]},
                   2: {2: [basal_area],
                       3: [basal_text],}}
mapping.defaults = {'comp_unit': {'USE_RESTRICTION_SILVIC': {'value':'0', 'varind':(1, 6)},
                                  'USE_RESTRICTION_HARVEST': {'value':'0', 'varind':(1, 5)}}}

importdate = date(2009, 3, 13)

logger = Mock('Logger')
logname = 'testlog'

lexicon = Mock('Lexicon')
lexicon.level_ind = {'simulation':0, 'comp_unit':1, 'stratum':2, 'tree':3}
lexicon.level_name = {0:'simulation', 1:'comp_unit', 2:'stratum', 3:'tree'}
lexicon.hierarchy = {0:{'parent':None}, 1:{'parent':0}, 2:{'parent':1},
                     3:{'parent':2}}

##############################################################################
# for import_data
##############################################################################
inlinestr = """stand1 4 1
stand1 1 1 6.1.2009
stratum1_1 2 20.0 test
# comment in the text content
stratum1_2 2 2.2 piece
stand2 4 2
stand2 1 2 31.12.2009
stratum2_1 2 3.1 of
stratum2_2 2 0.1 text"""
inline = StringIO.StringIO(inlinestr)

standstr = """stand1 1 1 1.1.2009
stand2 1 1 31.12.2009"""
stratumstr = """stratum1_1 stand1 20.0 oh
stratum1_2 stand1 2.2 which
! Another comment
stratum2_1 stand2 3.1 is
stratum2_2 stand2 0.1 infact"""
by_level = [StringIO.StringIO(standstr), StringIO.StringIO(stratumstr)]


inlinestr2 = """stand1 1 1 6.1.2009
stratum1_1 2 20.0 pretty
# comment in the text content
stratum1_2 2 2.2 frekin
stand2 1 1 31.12.2009
-1 2 3.1 hmm
-1 2 0.1 wait"""
inline2 = StringIO.StringIO(inlinestr2)

inlinestr3 = """stand1;1;1;6.1.2009
stratum1_1;2;20.0;awesome
stratum1_2;2;2.2;do
stand2;1;1;31.12.2009
stratum2_1;2;3.1;you
stratum2_2;2;0.1;agree"""
inline3 = StringIO.StringIO(inlinestr3)

# by_level; with single level at a time using provided data date
by_level2 = [StringIO.StringIO(stratumstr)]
