#!/usr/bin/env python
# encoding: utf-8
"""
*mock4importops.py*

Mock objects for importops testing

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""
from minimock import Mock
from datetime import date
import StringIO

##############################################################################
# for __init__
##############################################################################
inputdb = Mock('DataDB')
inputdb.fill_dictionary.mock_returns = {'comp_unit':
            [(date(2009, 3, 27),
              {'id':'stand1', 'parent id': 'sim1',
               'values': [('AREA', 2.1), ('SC', 1.2), ('DEV_CLASS', 2.3)]}),]}
fopsdb = Mock('OperationDB')
separator = None
opdb = Mock('operation_DataDB')
opdb.conn.execute.mock_returns = [('stand1', 0, date(2009, 3, 27),
                                   'comp_unit', 'clearcut',)]
regen_sp = Mock('Mapping')
regen_sp.var_type = 'categorical'
regen_sp.from_data_type = 'int'
v1 = Mock('ValueConversion')
v1.var_ind = (1, 1)
v1.level = 'comp_unit'
v1.to_var = 'REGEN_SP'
v1.to_val = 3
v2 = Mock('ValueConversion')
v2.var_ind = (1, 1)
v2.to_var = 'REGEN_SP'
v2.to_val = 4
regen_sp.map = {1: [v1], 2: [v2]}

op_area = Mock('Mapping')
op_area.var_type = 'numerical'
op_area.from_data_type = 'float'
v3 = Mock('ValueConversion')
v3.level = 'comp_unit'
v3.to_val = None
v3.to_var = 'REGEN_SP'
v3.conv_fact = 1.0

op2mc = Mock('Operation2Modelchain')
op2mc.operation_mapping = \
        {'first_thinning': ['Calculate thinning limits'],
         'clearcut': ['Cut', 'Cut some more']}
opconv = Mock('OperationConversion')
opconv.op2mc.operation_mapping = \
        {'first_thinning': ['Calculate thinning limits'],
         'clearcut': ['Cut', 'Cut some more']}
opconv.operation_conv.operation_level = 'comp_unit'
opconv.operation_conv.no_operation_indicator = '99'
opconv.operation_conv.none_value_indicator = '0'
opconv.operation_conv.comment_prefix = None
opconv.operation_conv.id_rowpos = [0]
opconv.operation_conv.id_delim = '-'
opconv.operation_conv.op_id_rowpos = None
opconv.operation_conv.type_rowpos = 2
opconv.operation_conv.operation_rowpos = 3
opconv.operation_conv.types = {'1': 'harvest', '2': 'silviculture'}
op1 = Mock('OperationDef')
op1.type = 'harvest'
op1.from_name = '2'
op1.to_name = 'clearcut'
op1.parameters = None
op2 = Mock('OperationDef')
op2.type = 'harvest'
op2.from_name = '21'
op2.to_name = 'first_thinning'
p1 = Mock('Parameter')
p1.value = 500.0
p1.variable = 'TARGET_N'
p1.level = 'comp_unit'
op2.parameters = [p1]
opconv.operation_conv.operations = {('1', '2'): op1, ('1', '21'): op2}
opconv.operation_conv.timing_rowpos = 5
timing = Mock('OperationTiming')
timing.type = 'step'
timing.date_conversion = None
timing.step_conversion = {'1': 2, '2': 7}
timing.step_unit = 'year'
opconv.operation_conv.timing = timing
opconv.operation_conv.variables = {7: regen_sp}
opconv.variables = {4: op_area}

logger = Mock('Logger')
logname = 'testlog'

##############################################################################
# for import_data
##############################################################################
textstr1 = """
stand1 0 1 2 0 1 0 1
stand2 0 1 21 0 2 0 0
stand3 0 1 99 0 0 0 0"""
textops = StringIO.StringIO(textstr1)

textstr2 = """
stand1 0 1 2 0 16.5.2009 0 1 op1
stand2 0 1 21 0 2009-05-06 0 0 op2"""
textops2 = StringIO.StringIO(textstr2)

