#!/usr/bin/env python
# encoding: utf-8
"""
*ind2id.py*

object id handler for data matrix

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

from collections import defaultdict

class Ind2Id(object):
    """Ids for all objects in the data matrix"""
    def __init__(self):
        super(Ind2Id, self).__init__()
        self.ids = defaultdict(dict)
        self.inds = defaultdict(dict)
        self._object_count = defaultdict(int)

    def add_id(self, iteration, branch, l_ind, ind, obj_id, parlev=None,
               parind=None):
        """
        Add id for a new object

        obj_id is a tuple of the unique id and original id
        """
        ibkey = (iteration, branch)
        indkey = (l_ind,ind)
        if obj_id == (None, None):
            # if object id is None, parent id must be defined
            if l_ind > 0 and (parlev is None or parind is None):
                raise Exception, "No parent l_ind and indice!"
            # construct unique object id from parent object id, l_ind and
            # object count: "<parent_id>-<l_ind>-object_number"
            par_uid, par_oid = self.ids[ibkey][(parlev, parind)]
            while True:
                oid = "%s-%d" % (par_uid, self._object_count[(l_ind, par_uid)])
                uid = oid
                if self.get_ind(iteration, branch, l_ind, oid) is None:
                    break #Didn't exist, so it's ok
                #Key already exists, make a new one...
                self._object_count[(l_ind, par_uid)] += 1
            self._object_count[(l_ind, par_uid)] += 1
        else:
            uid, oid = obj_id
        self.ids[ibkey][indkey] = (uid, oid)
        idkey = (l_ind, uid)
        self.inds[ibkey][idkey] = ind

    def get_id(self, iteration, branch, l_ind, ind):
        """Get object id
        """
        if (l_ind, ind) in self.ids[(iteration, branch)]:
            return self.ids[(iteration, branch)][(l_ind, ind)]
        else:
            return None

    def get_ind(self, iteration, branch, l_ind, uid):
        """Get object ind
        """
        if (l_ind, uid) in self.inds[(iteration, branch)]:
            return self.inds[(iteration, branch)][(l_ind, uid)]
        else:
            return None

    def remove_id(self, iterr, branch, l_ind, ind):
        """Remove object id and ind"""
        ib_key = (iterr, branch)
        lo_key = (l_ind, ind)
        uid, oid = self.ids[ib_key][lo_key]
        lid_key = (l_ind, uid)
        del self.ids[ib_key][lo_key]
        del self.inds[ib_key][lid_key]

    def remove_a_bunch(self, iterr, branch, removed):
        """Remove object id and ind for a group of removed objects"""
        ib_key = (iterr, branch)
        for l_ind, inds in removed.items():
            for ind in inds:
                lo_key = (l_ind, ind)
                uid, oid = self.ids[ib_key][lo_key]
                lid_key = (l_ind, uid)
                del self.ids[ib_key][lo_key]
                del self.inds[ib_key][lid_key]

def main():
    pass

if __name__ == '__main__':
    main()

