#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO metaheuristic threshold accepting (TA) optimization algorithm

...

*threshold_accepting.py*

Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.
"""

# these are for running the doctest from src/simo
#import sys
#sys.path.append('..')

import copy
import numpy
from optimizer import Optimizer


class ThresholdAccepting(Optimizer):
    """Threshold accepting optimization algorithm implementation
    """
    def _process_arguments(self, args):
        """Process keyword arguments
        """
        self._abort = False
        if not 'maxiterations' in args:
            self._add_error('Parameter "maxiterations" missing!')
            self._abort = True
            return
        self._max_iterations = int(args["maxiterations"])

    def optimize(self):
        """Run TA algorithm
        """
        if self._abort:
            errormsg = 'Invalid optimizer initialization ... abort!'
            self._add_error(errormsg)
            return
        # optimization is done for a single iteration at a time
        # NB: here 'iteration' refers to simulation iteration (as in Monte
        # Carlo type of simulation) and 'optimization_round' or 'round' to
        # a single round in iterative heuristic optimization
        for iteration in range(self._num_of_iterations):
            self._stat_logger.start(iteration, self._branches)
            self._finished = False
            if not self._generate_random_solution(iteration):
                return
            for i in range(self._max_iterations):
                self._stat_logger.add_round(iteration, iunit,
                                            self._elite_solution,
                                            self._elite_solution_value)

                if self._abort:
                    errormsg = 'Optimization aborted due to too strict '\
                               'constraints!'
                    self.add_error(errormsg)
                    break
            self._stat_logger.stop(iteration)
            self._store_solution(iteration)
        self._store_results()


def main():
    import doctest
    doctest.testfile('./test/threshold_accepting-py.rst')

if __name__ == '__main__':
    main()
