import os
from datetime import datetime
import operator

from inline import OutputInline
from by_level import OutputByLevel
from branching import OutputBranching
from opres import OutputOpres
from aggr import OutputAggr
from expression import OutputExpr
from smt import OutputSMT

class Out(object):

    sep = ';'

    def __init__(self, datadb, data_type, id_list, main_level,
                 result_type, output_formats, output_dir,
                 output_filenames, logger,
                 output_constraint=None, default_decimal_places=1,
                 archiving=False, result_padding=True,
                 aggregation_def=None, expression_def=None,
                 opres_vars=None, group_by_vars=None, full_date=False,
                 dates=None):
        super(Out, self).__init__()
        logname = 'output'
        self.datadb = datadb
        self.datadb.db.vacuum_analyze()
        self.data_type = data_type
        self.id_list = id_list
        self.main_level = main_level
        self.result_type = result_type
        self.output_formats = output_formats
        self.output_filenames = output_filenames
        self.output_constraint = output_constraint
        self.default_decimal_places = default_decimal_places
        self.archiving = archiving
        self.result_padding = result_padding
        self.aggregation_def = aggregation_def
        self.expression_def = expression_def
        self.opres_vars = opres_vars
        self.group_by_vars = group_by_vars
        for i in range(len(output_formats)):
            filename = os.path.join(output_dir, output_filenames[i])
            if output_formats[i] == 'inlined':
                runcl = OutputInline
                logger.log_message(logname, 'info', 'Writing inlined')
            elif output_formats[i] == 'by_level':
                runcl = OutputByLevel
                logger.log_message(logname, 'info', 'Writing by_level')
            elif output_formats[i] == 'operation_result':
                runcl = OutputOpres
                logger.log_message(logname, 'info', 'Writing operation_result')
            elif output_formats[i] == 'aggregation':
                runcl = OutputAggr
                logger.log_message(logname, 'info', 'Writing aggregation')
            elif output_formats[i] == 'expression':
                runcl = OutputExpr
                logger.log_message(logname, 'info', 'Writing expression')
            elif output_formats[i] == 'branching_graph':
                runcl = OutputBranching
                logger.log_message(logname, 'info', 'Writing branching_graph')
            elif output_formats[i] == 'smt':
                runcl = OutputSMT
                logger.log_message(logname, 'info', 'Writing smt')
            starttime = datetime.now()
            runcl(datadb, data_type, id_list, main_level, result_type,
                  filename, logger, output_constraint, default_decimal_places,
                  archiving, result_padding, aggregation_def, expression_def,
                  opres_vars, group_by_vars, full_date, dates)
            logger.log_message(logname, 'info', 'finished in '+str(datetime.now() - starttime))
