#!/usr/bin/env python
# encoding: utf-8
"""
*caller.py*

Model caller super class

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""


class Caller(object):
    """Super class of all model and table caller classes"""
    def __init__(self, callertype):
        """Initialize caller object
        """
        super(Caller, self).__init__()
        self._type = callertype
        self._model = None
        self._params = None
        self._sim = None
        self._depth = None

    def execute(self, model, params, sim, depth):
        """Placeholder for subclass implementations
        """
        self._model = model
        self._params = params
        self._sim = sim
        self._depth = depth
        self._target_indices = {}

    def cleanup(self):
        del self._model
        del self._params
        del self._sim
        del self._depth
        del self._target_indices

    def _get_targets(self, level, depth, datalevel=None, throughlevel=None,
                     get_tind_from_data=False, order=None):
        """Get target index for the current level-depth-datalevel combination
        """
        if datalevel is not None and \
            throughlevel is None and \
            datalevel == level:
            datalevel = None
        key = (level, depth, datalevel, throughlevel)
        if self._target_indices.has_key(key) and not get_tind_from_data:
            tind, torem, active = self._target_indices[key]
            self._sim.data.set_active(active)
        else:
            tind, torem = self._sim.data.get_tind(level, depth, datalevel,
                                                  throughlevel, True, None,
                                                  order)
            active = self._sim.data.get_active()
            self._target_indices[key] = (tind, torem, active)
        return tind, torem

    def add_error(self, errorstr, level, tind):
        """Log error message"""
        msg = "%s caller: %s" % (self._type, errorstr)
        self._sim.add_error(msg, level, tind)

    def add_warning(self, warnstr, level, tind):
        """Log error message"""
        msg = "%s caller: %s" % (self._type, warnstr)
        self._sim.add_warning(msg, level, tind)


def main():
    pass

if __name__ == '__main__':
    main()
