#!/usr/bin/env python
# encoding: utf-8
"""
*mocks4operationcaller.py*

Mock objects for testing operationcaller

Copyright (c) 2009 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""
# import modules needed for test dummies
import numpy
import datetime

from lxml import etree
from minimock import Mock

from simo.builder.modelbase.operationmodel import OperationModel
from simo.builder.modelbase.cashflowtable import CashFlowTable
from simo.builder.modelbase.parametertable import ParameterTable
from simo.builder.modelbase.table import TableLabel, TableClassifier
from simo.builder.modelbase.operationmodel import Classifier

dummynotextlevel = Mock('Level')
dummynotextlevel.textual = []

dummytextlevel = Mock('Level')
dummytextlevel.textual = ['stand_number']

# construct dummy lexicon
dummylexicon = Mock('Lexicon')
dummylexicon.get_level_name.mock_returns = 'comp_unit'
dummylexicon.level_ind = {'comp_unit': 1, 'stratum': 2, 'tree': 3,
                          'dead_tree': 4}
dummylexicon.levels = {'simulation': dummynotextlevel,
                       'comp_unit': dummytextlevel,
                       'stratum': dummynotextlevel,
                       'dead_tree': dummynotextlevel}
dummylexicon.hierarchy = {0: {'ordinal': 0, 'lineage': set([0, 1]),
                              'children': [1], 'parent': None},
                          1: {'ordinal': 1, 'lineage': set([0, 1]),
                              'children': [2, 4], 'parent': 0},
                          2: {'ordinal': 2, 'lineage': set([0]),
                              'children': [3], 'parent': 1},
                          3: {'ordinal': 3, 'lineage': set([0]),
                              'children': None, 'parent': 2},
                          4: {'ordinal': 2, 'lineage': set([1]),
                              'children': None, 'parent': 1}}


dummyopxml = u'''<operation>
            <name>thinning</name>
            <type>
                <model>
                    <implemented_at>harvest.py</implemented_at>
                    <implemented_in>python</implemented_in>
                    <cash_flow_handler>model</cash_flow_handler>
                    <interaction>on</interaction>
                </model>
            </type>
            <description>...</description>
            <data>
                <group>
                    <level>comp_unit</level>
                    <variables>
                        <variable>
                            <name>D_gM</name>
                        </variable>
                    </variables>
                </group>
            </data>
            <variables>
                <variable>
                    <name>STEM_DIVISOR</name>
                    <level>simulation</level>
                </variable>
            </variables>
            <parameters>
                <parameter>
                    <name>target_density</name>
                    <optional>false</optional>
                </parameter>
            </parameters>
            <parameter_tables>
                <parameter_table>thinning_preference_order</parameter_table>
            </parameter_tables>
            <operation_result_weight>
                <name>AREA</name>
                <level>comp_unit</level>
            </operation_result_weight>
            <results>
                <type>
                    <operation_result/>
                    <variables>
                        <variable>
                            <name>Volume</name>
                            <classifiers>
                                <classifier>
                                    <variable>SP</variable>
                                    <level>stratum</level>
                                </classifier>
                                <classifier>
                                    <cashflow_classifier>
                                        <variable>assortment</variable>
                                    </cashflow_classifier>
                                </classifier>
                                <classifier>
                                    <variable>stand_number</variable>
                                    <level>comp_unit</level>
                                </classifier>
                            </classifiers>
                        </variable>
                        <variable type="date_array">
                            <name>C_flow</name>
                        </variable>
                    </variables>
                </type>
            </results>
        </operation>'''

dummycfxml = u'''<cash_flow_table name="timber_prices">
          <timber_price>true</timber_price>
          <base>
             <classifiers>
                <classifier>
                   <variable>SP</variable>
                   <level>stratum</level>
                </classifier>
                <classifier>
                   <variable>assortment</variable>
                   <level>within-operation</level>
                   <labels>
                      <label>
                         <value>1</value>
                         <name>log</name>
                      </label>
                      <label>
                         <value>2</value>
                         <name>pulp</name>
                      </label>
                   </labels>
                </classifier>
             </classifiers>
             <tables>
                 <table>
                     <cash_flow>1 1 1 160 370 55</cash_flow>
                     <cash_flow>2 1 1 160 400 56</cash_flow>
                 </table>
              </tables>
          </base>
        </cash_flow_table>'''

dummyptxml = u'''<parameter_table name="thinning_preference_order"
                                  type="operation_table"
                                  desc="Thinning preference order...">
        <classifiers>
            <classifier>
                <variable>MAIN_SP</variable>
                <level>comp_unit</level>
            </classifier>
            <classifier>
                <variable>SC</variable>
                <level>comp_unit</level>
            </classifier>
        </classifiers>
        <targets>
            <target>
                <operation_parameter>
                    <parameter>SP</parameter>
                    <length>9</length>
                </operation_parameter>
            </target>
        </targets>
        <table>
            <value>1 1 2 3 5 1 4 6 7 8 9</value>
            <value>1 2 2 3 5 1 4 6 7 8 9</value>
            <value>1 3 2 3 5 1 4 6 7 8 9</value>
            <value>2 1 2 3 5 1 4 6 7 8 9</value>
            <value>2 2 2 3 5 1 4 6 7 8 9</value>
            <value>2 3 2 3 5 1 4 6 7 8 9</value>
            <value>3 1 3 2 5 1 4 6 7 8 9</value>
            <value>3 2 3 2 5 1 4 6 7 8 9</value>
            <value>3 3 3 2 5 1 4 6 7 8 9</value>
        </table>
    </parameter_table>'''


class DummyValidator:
    lexicon = dummylexicon

    def elem_name(self, text):
        return text

    def variable_ind(self, level, variable):
        if level == 'simulation':
            return (0, 1)
        elif level == 'comp_unit':
            return (1, 1)
        else:
            return (2, 3)


class DummyData(object):
    text_data = {1: {0: {u'stand_number': 'kuvio1'},
                     1: {u'stand_number': 'kuvio2'},
                     2: {u'stand_number': 'kuvio3'},
                     }}

    def get_value(self, tind, attrs):
        if tind[-1, -1] == 0:
            n = len(tind)
        else:
            n = tind[-1, -1]
        if isinstance(attrs, int):
            natt = attrs
            arr = numpy.array(range(1, n+1), dtype=float)
        else:
            natt = len(attrs)
            arr = numpy.array([range(1, n+1) for i in range(natt)],
                              dtype=float)
        return arr, None

    def get_tind(self, level, depth, dlevel=None, tlevel=None, vals4all=True,
                 valuelevel=None, order=None):
        if dlevel is None or (level == 1 and dlevel <= 1):
            return numpy.array([[0, 0, 0, 0, 0],
                                [0, 0, 1, 0, 0],
                                [0, 0, 2, 0, 0]], dtype=int), set([])
        if dlevel != level:
            return numpy.array([[0, 0, 0, 0, 3],
                                [0, 0, 1, 3, 5],
                                [0, 0, 2, 5, 9]], dtype=int), set([])

    def get_active(self):
        return None

    def set_active(self, active):
        pass

    def get_date(self, tind):
        dd = datetime.date
        dt = datetime.date(2000, 1, 1)
        return numpy.array([dt for i in range(len(tind[:, 0]))], dtype=dd)

    def block_children(self, depth, level, childlevel):
        return {}


# construct dummy validator
validator = DummyValidator()
validator.add_error = Mock('add_error')
validator.add_model = Mock('add_model')

# construct dummy parameter table and cashflow table
elem = etree.fromstring(dummyptxml)
validator.model_type = 'parameter_table'
dummypttable = ParameterTable('', elem, validator, 'dummydir')
validator.model_type = 'cash_flow_table'
elem = etree.fromstring(dummycfxml)
dummycftable = CashFlowTable('', elem, validator, 'dummydir')

# construct dummy OperationModelParams
dummyparams = Mock('Parameters')
dummyparams.parameters = [0.5]
dummyparams.parameter_tables = [dummypttable]
dummyparams.cash_flow_table = [dummycftable]
dummyparams.cash_flow_model = None
dummyparams.simulation_effects = None
dummyparams.memory_level = 1
dummyparams.name = 'dummy_db_name'
dummyparams.group = 'dummy_db_group'
dummyparams.notes = 'dummy notes'
dummyparams.materials = ['dummy material 1', 'dummy material 2']

# construct dummy operation arg
dummyarg = Mock('OperationArg')
dummyarg.name = 'dummymodel'
dummyarg.group = 'dummygroup'
dummyarg.db_name = 'dummy_db_name'
dummyarg.db_group = 'dummy_db_group'
dummyarg.notes = 'dummy notes'
dummyarg.materials = ['dummy material 1', 'dummy material 2']
dummyarg.get_results.mock_returns = [
    {'volume': {(1, 1): 3.3, (1, 2): 3.6, (2, 1): 3.9, (2, 2): 4.2},
     'income': {(1, 1): 33, (1, 2): 36, (2, 1): 39, (2, 2): 42}},
    {'volume': {(1, 3): 5.3, (2, 3): 5.6},
     'income': {(1, 3): 53, (2, 3): 56}},
    {'volume': {(1, 4): 6.3, (1, 5): 6.6, (2, 4): 6.9, (2, 5): 7.2},
     'income': {(1, 4): 63, (1, 5): 66, (2, 4): 69, (2, 5): 72}}]
dummyarg.get_cash_flow.mock_returns = numpy.array([1., 2., 3.])
dummyarg.get_weights.mock_returns = numpy.array([3., 2., 1.])
dummyarg.drop_target = numpy.zeros(3, dtype=bool)

# construct another dummy operation arg
dummyarg2 = Mock('OperationArg')
dummyarg2.name = 'dummymodel2'
dummyarg2.group = 'dummygroup2'
dummyarg2.db_name = 'dummy_db_name2'
dummyarg2.db_group = 'dummy_db_group2'
dummyarg2.notes = 'dummy notes2'
dummyarg2.materials = ['dummy material 21', 'dummy material 22']
dummyarg2.target_index = numpy.array([[0, 0, 1, 0, 0],
                                      [0, 0, 2, 0, 0],
                                      [0, 0, 3, 0, 0]])
dummyarg2.num_of_targets = 3
dummyarg2.num_of_results = numpy.array([[2, 1, 3]], dtype=int)
dummyarg2.result_variables = [('operation_result', 'C_flow', None,
                               'date_array', True)]
dummyarg2.get_results.mock_returns = [
    {'C_flow': [(1000, 0), (1100, 1), (1500, 5), (1700, 7)]},
    {'C_flow': [(2200, 2), (2400, 4), (2800, 8), (2900, 9)]},
    {'C_flow': [(3100, 1), (3500, 5), (4000, 10), (5000, 20)]}]
dummyarg2.get_cash_flow.mock_returns = numpy.array([1., 2., 3.])
dummyarg2.get_weights.mock_returns = numpy.array([3., 2., 1.])
dummyarg2.drop_target = numpy.zeros(3, dtype=bool)


# construct dummy data
dummydata = DummyData()
dummydata.set_value = Mock('set_value')
dummydata.del_objects = Mock('del_objects')
dummydata.get_id = Mock('get_id')
dummydata.get_id.mock_returns = [(v, v) for v in
                                 ["UID000%d" % i for i in range(20)]]

broken_dummydata = DummyData()
broken_dummydata.set_value = Mock('set_value')
broken_dummydata.del_objects = Mock('del_objects')
broken_dummydata.get_id = Mock('get_id')
broken_dummydata.get_id.mock_returns = [(v, v) for v in
                                        ["UID000%d" % i for i in range(20)]]


def broken_get_value(tind, ind):
    return None, ([('errormessage', (0,))], tind, (0,))
broken_dummydata.get_value = broken_get_value

# construct dummy simulator with dummy data, cashflow table and parameter
# table objects
# dummy cashflow and paremeter tables are parsed from xml
dummysim = Mock('Simulator')
dummysim.level = 1
dummysim.main_level = 1
dummysim.period = 5
dummysim.value_fixing = False
dummysim.tables_cash_flow = {}
dummysim.tables_operation = {}
dummysim.data = dummydata
dummysim.price_cache = Mock('PriceCache')
dummysim.stem_cache = Mock('StemCache')
dummysim.lexicon = dummylexicon
dummysim.timespan.time_step = 5
dummysim.timespan.unit = 'year'

# construct dummy operation model by parsing it from xml and add a dummy
# operation model, that returns 1
elem = etree.fromstring(dummyopxml)
dummymodel = OperationModel('', elem, validator, '', 'thinning')
dummymodel._v_func = lambda a, b, c, d, e, f, g, h, i, j, k, l: 1
dummymodel.index = 1
key = ('SP', 'assortment', 'stand_number')
dummymodel.result_classifiers[key][1].labels = {1: 'log', 2: 'pulp'}
dummymodel.text_classifiers[('SP', 'assortment', 'stand_number')] = \
        [('comp_unit', 'stand_number')]


# construct dummy classifiers
cls1 = Classifier('cashflow', 'assortment', None, None)
cls1.labels = {1: 'log', 2: 'pulp'}
cls2 = Classifier('variable', 'SP', 0, 2)
dummyclassifiers = [cls1, cls2]

# construct dummy table
dummytable = Mock('Table')
tcls1 = TableClassifier('stratum', 'SP', (2, 1), None, 0, None)
tcls2 = TableClassifier('within-operation',
                        'assortment',
                        None,
                        [TableLabel('log', 1), TableLabel('pulp', 2)],
                        1, None)
dummytable.classifiers = {'SP': tcls1, 'assortment': tcls2}


def main():
    pass

if __name__ == '__main__':
    main()
