#!/usr/bin/env python
# encoding: utf-8
"""
*mocks4predictioncaller.py*

Mock objects for parameter table caller testing

Copyright (c) 2014 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

import numpy
from minimock import Mock
from simo.builder.modelbase.table import Array, TableClassifier


def func(arg, i):
    """Dummy prediction model function"""
    if arg.max_res_objs > 1:
        arg.num_of_res_objs[i] = 4 + i
    else:
        arg.num_of_res_objs[i] = 1
    nv, nr = arg.num_of_res_vars, arg.num_of_res_objs[i]
    vals = [[i+0.1*j+0.01*k for k in range(nr)] for j in range(nv)]
    arg.mem[:nv*nr,i] = numpy.ravel(numpy.array(vals, dtype=float))
    return 1

array = Array(numpy.array([[  1.,    1.,    0.7,    0.75,  15., ],
 [  1.,    2.,    0.7,    0.75,  15., ],
 [  1.,    3.,    0.7,    0.75,  15., ],
 [  1.,    4.,    0.,    0.75,  15., ],
 [  1.,    5.,    0.,    0.,   15., ],
 [  1.,    6.,    0.,    0.,   15., ],
 [  1.,    7.,    0.,    0.,   15., ],
 [  1.,    8.,    0.,    0.,   15., ],
 [  2.,    1.,    0.7,    0.75,  15., ],
 [  2.,    2.,    0.7,    0.75,  15., ],
 [  2.,    3.,    0.7,    0.75,  15., ],
 [  2.,    4.,    0.,    0.,   15., ],
 [  2.,    5.,    0.,    0.,   15., ],
 [  2.,    6.,    0.,    0.,   15., ],
 [  2.,    7.,    0.,    0.,   15., ],
 [  2.,    8.,    0.,    0.,   15., ],
 [  3.,    1.,    0.7,    0.75,  15., ],
 [  3.,    2.,    0.7,    0.75,  15., ],
 [  3.,    3.,    0.7,    0.75,  15., ],
 [  3.,    4.,    0.,    0.,   15., ],
 [  3.,    5.,    0.,    0.,   15., ],
 [  3.,    6.,    0.,    0.,   15., ],
 [  3.,    7.,    0.,    0.,   15., ],
 [  3.,    8.,    0.,    0.,   15., ],
 [  4.,    1.,    0.,    0.,   15., ],
 [  4.,    2.,    0.,    0.,   15., ],
 [  4.,    3.,    0.,    0.,   15., ],
 [  4.,    4.,    0.,    0.,   15., ],
 [  4.,    5.,    0.,    0.,   15., ],
 [  4.,    6.,    0.,    0.,   15., ],
 [  4.,    7.,    0.,    0.,   15., ],
 [  4.,    8.,    0.,    0.,   15., ],
 [  5.,    1.,    0.,    0.,   15., ],
 [  5.,    2.,    0.,    0.,   15., ],
 [  5.,    3.,    0.,    0.,   15., ],
 [  5.,    4.,    0.,    0.,   15., ],
 [  5.,    5.,    0.,    0.,   15., ],
 [  5.,    6.,    0.,    0.,   15., ],
 [  5.,    7.,    0.,    0.,   15., ],
 [  5.,    8.,    0.,    0.,   15., ]], dtype=float),
{'3:1:0.7:0.75': 15.0, '2:2:0.7:0.75': 15.0, '3:5:0:0': 15.0, '5:5:0:0': 15.0,
 '3:4:0:0': 15.0, '5:7:0:0': 15.0, '1:6:0:0': 15.0, '1:1:0.7:0.75': 15.0,
 '2:7:0:0': 15.0, '2:5:0:0': 15.0, '1:8:0:0': 15.0, '1:4:0:0.75': 15.0,
 '4:3:0:0': 15.0, '3:2:0.7:0.75': 15.0, '4:1:0:0': 15.0, '5:3:0:0': 15.0,
 '2:6:0:0': 15.0, '3:8:0:0': 15.0, '2:4:0:0': 15.0, '5:4:0:0': 15.0,
 '5:6:0:0': 15.0, '4:5:0:0': 15.0, '4:4:0:0': 15.0, '4:7:0:0': 15.0,
 '4:6:0:0': 15.0, '4:2:0:0': 15.0, '1:5:0:0': 15.0, '1:2:0.7:0.75': 15.0,
 '5:8:0:0': 15.0, '3:6:0:0': 15.0, '3:7:0:0': 15.0, '5:2:0:0': 15.0,
 '2:8:0:0': 15.0, '2:3:0.7:0.75': 15.0, '2:1:0.7:0.75': 15.0,
 '3:3:0.7:0.75': 15.0, '1:3:0.7:0.75': 15.0, '5:1:0:0': 15.0, '1:7:0:0': 15.0,
 '4:8:0:0': 15.0},
None, None)

SC = TableClassifier(level=1, name='SC', ind=10, labels=None, order=1,
                     variable_type=None)
SUB_GROUP = TableClassifier(level=1, name='SUB_GROUP', ind=9, labels=None,
                            order=0, variable_type=None)

paramtable = Mock('ParameterTable')
paramtable.classifier_order = ['SUB_GROUP', 'SC']
paramtable.Nclassifiers = 2
paramtable.name = 'biomass_collection_proportions'
paramtable.arrays = [array]
paramtable.classifiers = {'SC': SC, 'SUB_GROUP': SUB_GROUP}
paramtable.Ntargets = 3
paramtable.active_array = None
paramtable.keycount = 2
paramtable.type = 'parameter_table'
paramtable.targets = [Mock('ParamTableTarget'),
                      Mock('ParamTableTarget'),
                      Mock('ParamTableTarget')]
paramtable.Narrays = 1


# mock result variable objects
rv1 = Mock('Resvar')
rv1.variable = 1
rv1. cumulation = 'annual'
rv2 = Mock('Resvar')
rv2.variable = 1
rv2. cumulation = 'annual'
rv3 = Mock('Resvar')
rv3.variable = 1
rv3. cumulation = 'annual'
rv4 = Mock('Resvar')
rv4.variable = 1
rv4. cumulation = 'annual'

# mock data handler object
data = Mock('Data')
tind = numpy.array([[ 0, 0, 0, 0, 0],
 [ 0, 0, 1,  0, 0],
 [ 0, 0, 2,  0, 0],
 [ 0, 0, 3,  0, 0],
 [ 0, 0, 4,  0, 0],
 [ 0, 0, 5,  0, 0],
 [ 0, 0, 6,  0, 0],
 [ 0, 0, 7,  0, 0],
 [ 0, 0, 8,  0, 0],
 [ 0, 0, 9,  0, 0],
 [ 0, 0, 10, 0, 0],
 [ 0, 0, 11,  0, 0],
 [ 0, 0, 12,  0, 0],
 [ 0, 0, 13,  0, 0],
 [ 0, 0, 14,  0, 0],
 [ 0, 0, 15,  0, 0],
 [ 0, 0, 16,  0, 0],
 [ 0, 0, 17,  0, 0],
 [ 0, 0, 18,  0, 0],
 [ 0, 0, 19,  0, 0],
 [ 0, 0, 20, 0, 0],
 [ 0, 0, 21,  0, 0],
 [ 0, 0, 22,  0, 0],
 [ 0, 0, 23,  0, 0],
 [ 0, 0, 24,  0, 0],
 [ 0, 0, 25,  0, 0],
 [ 0, 0, 26,  0, 0],
 [ 0, 0, 27,  0, 0],
 [ 0, 0, 28,  0, 0],
 [ 0, 0, 29,  0, 0]], dtype=int)
data.get_tind.mock_returns = tind, set([])
values = numpy.array([[ 1., 1., 1., 1., 1., 2., 1., 1., 2., 1., 2., 2., 1., 
                        2., 1., 2., 1., 1., 1., 1., 2., 2., 1., 1., 1., 1., 
                        1., 1., 1., 1.]], dtype=float)
values2 = numpy.array([[ 3., 3., 3., 3., 4., 3., 3., 3., 3., 3., 3., 3., 4.,
                         3., 4., 3., 3., 3., 3., 3., 4., 3., 4., 2., 4., 3.,
                         4., 3., 3., 3.]], dtype=float)
data.get_value.mock_returns = values, None

# mock simulator object
sim = Mock('Simulator')
sim.level = 1
sim.chain_index = 0
sim.memory_models = []
sim.value_fixing = False
sim._max_res_objs = 60
sim.data = data

def main():
    pass

if __name__ == '__main__':
    main()
