#!/usr/bin/env python
# encoding: utf-8
"""
*mocks4predictioncaller.py*

Mock objects for prediction caller testing

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

import numpy
from minimock import Mock


def func(arg, i):
    """Dummy prediction model function"""
    if arg.max_res_objs > 1:
        arg.num_of_res_objs[i] = 4 + i
    else:
        arg.num_of_res_objs[i] = 1
    nv, nr = arg.num_of_res_vars, arg.num_of_res_objs[i]
    vals = [[i+0.1*j+0.01*k for k in range(nr)] for j in range(nv)]
    arg.mem[:nv*nr,i] = numpy.ravel(numpy.array(vals, dtype=float))
    return 1

# mock result variable objects
rv1 = Mock('Resvar')
rv1.variable = 1
rv1. cumulation = 'annual'
rv2 = Mock('Resvar')
rv2.variable = 1
rv2. cumulation = 'annual'
rv3 = Mock('Resvar')
rv3.variable = 1
rv3. cumulation = 'annual'
rv4 = Mock('Resvar')
rv4.variable = 1
rv4. cumulation = 'annual'

# mock data handler object
data = Mock('Data')
tind = numpy.array([[0,0,0,0,0],
                    [0,0,1,0,0],
                    [0,1,0,0,0]], dtype=int)
data.get_tind.mock_returns = tind, set([])
values = numpy.array([[1.1, 2.1, 3.1],
                      [1.2, 2.2, 3.2]], dtype=float)
data.get_value.mock_returns = values, None

# mock simulator object
sim = Mock('Simulator')
sim.level = 1
sim.chain_index = 0
sim.memory_models = []
sim.value_fixing = True
sim._max_res_objs = 60
sim.data = data

# prediction model parameters
params = Mock('Parameters')
params.rect_factor = 1.0
params.risk_level = 1
params.parameters = [5.0, 7.5]

# prediction model mock object
model = Mock('PredictionModel')
model.name = 'dummymodel'
model.language = 'python'
model.n_vars = 4
model.vars = variables = {0: {'index':[0,1], 'order':[0,2], 'limits': None},
                          1: {'index':[2,3], 'order':[1,3], 'limits': None}}
model.n_params = 2
model.param_names = ['param1', 'param2']
model.param_limits = [None, None]
model.result_level = 1
model.result_vars = [rv1, rv2, rv3, rv4]
model._v_func = func
model._v_mlib = None


def main():
    pass

if __name__ == '__main__':
    main()
