import os, sys, ctypes
  
def get_available_ram():
    """Will return the amount of RAM in the OS""" 
    if sys.platform.startswith('dar'):  
        return _osxRam()
    elif sys.platform.startswith('lin'):
        return _linuxRam()  
    elif os.name == "nt":  
        return _windowsRam()  
    else:  
        raise OSError("I only work with Win, Linux or OSX")
  
def _windowsRam():  
    """Uses Windows API to check RAM in this OS"""  
    kernel32 = ctypes.windll.kernel32  
    c_ulong = ctypes.c_ulong  
    class MEMORYSTATUS(ctypes.Structure):  
        _fields_ = [  
            ("dwLength", c_ulong),  
            ("dwMemoryLoad", c_ulong),  
            ("dwTotalPhys", c_ulong),  
            ("dwAvailPhys", c_ulong),  
            ("dwTotalPageFile", c_ulong),  
            ("dwAvailPageFile", c_ulong),  
            ("dwTotalVirtual", c_ulong),  
            ("dwAvailVirtual", c_ulong)  
        ]  
    memoryStatus = MEMORYSTATUS()  
    memoryStatus.dwLength = ctypes.sizeof(MEMORYSTATUS)  
    kernel32.GlobalMemoryStatus(ctypes.byref(memoryStatus))  
  
    return int(memoryStatus.dwTotalPhys/1024**2)  
  
def _osxRam():  
    """Returns the RAM of a osx system (that has top)"""
    output = os.popen("top -l 1").readlines()
    memline = ''
    for line in output:
        if line.startswith("PhysMem"):
            memline = line
            break
    
    memline = memline.strip().split(', ')[-2:]
    totalMemory = []
    for i in memline:
        try:
            totalMemory.append(int(i.split('M')[0]))
        except ValueError: #wasn't reported in megs
            totalMemory.append(int(i.split('K')[0])/1000)
    return totalMemory[0]+totalMemory[1]  

def _linuxRam():  
    """Returns the RAM of a linux system"""  
    totalMemory = os.popen("free -m").readlines()[1].split()[1]  
    return int(totalMemory)  
