#!/usr/bin/env python
# encoding: utf-8
"""
*aggregationarg.py*

Aggregation model call argument container

Copyright (c) 2008 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

import numpy
from collections import deque

class AggregationArg(object):
    """Aggregation model argument container"""
    def __init__(self, ind, tind, torem, scalar, deterministic=False):
        """docstring for __init__"""
        super(AggregationArg, self).__init__()
        self.operands = 0
        self.scalar = scalar
        self.target_variable = ind
        self.remove_targets = torem
        self.use_nan_funcs = False
        self.values = deque()
        self.weights = deque()
        self.target_index = tind
        self.oper_target_index = deque()
        self.oper_data_level = deque()
        self.success = True
        self.errors = deque()
        self.results = None
        self.deterministic = deterministic

    def add_operand(self, datalevel, tind, rem_targets, values, weights,
                    failed_ind):
        """Add operand parent mapping, values, and weights
        """
        if len(rem_targets) != 0:
            self.remove_targets.update(rem_targets)
        self.operands += 1
        self.oper_target_index.append(tind)
        self.oper_data_level.append(datalevel)
        self.values.append(values)
        self.weights.append(weights)
        if failed_ind is not None:
            # aggregation models can have conditions, which can cause some of
            # the values to be NaNs
            self.use_nan_funcs = True

    def handle_missing(self):
        """
        Set the aggregation function to handle NaNs
        """
        if len(self.remove_targets)>0:
            self.use_nan_funcs = True

def main():
    pass

if __name__ == '__main__':
    main()
