#!/usr/bin/env python
# encoding: utf-8
"""
Title: SIMO forced operations

Module for definining forced operations.

*forcedoperation.py*

Usage:

Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

"""

class ForcedOperation(object):
    """
    Class for defining a forced operation. Forced operations are used normally
    in updating simulations when information of actual past operations is
    available.
    """
    def __init__(self, uid, op_id, level, name, chains, timingtype, timestep,
                 date, stepunit, data_values=None):
        """
        Initialize ForcedOperation object. Operation name, group, level etc.
        must be validated against lexicon and operation library before
        initializing a forced operation.

        Actual operations are implemented in model chains instead of individual
        operation models. The model chain names etc. have to be checked in the
        simulation initialization.

        id -- operation unit id as string
        op_id -- operation identifier
        level -- operation level name as string
        name -- operation name as string
        chains -- forced operation model chain names in a sequence of strings
        timingtype -- operation timing type, 'date' or 'step'
        timestep -- operation time step as int
        date -- operation date as datetime object
        stepunit -- time step unit as string
        data_values -- dictionary containing variable and parameter values
                       associated with the operation
        """
        self.id = uid
        self.op_id = op_id
        self.level = level
        self.name = name
        self.timing_type = timingtype
        self.time_step = timestep
        self.date = date
        self.step_unit = stepunit
        self.chain_names = chains
        self.chain_indices = None
        self.data_values = data_values


def main():
    pass

if __name__ == '__main__':
    main()

