#!/usr/bin/env python
# encoding: utf-8
"""
*mocks4sim.py*

Mock objects for testing simulator

Copyright (c) 2009 Simosol Oy. Distributed under the GNU General Public
License 2.0.
"""

import numpy as np
import datetime as dt
from minimock import Mock
import simo.builder.names as names

# construc mock logger
logger = Mock('Logger')

# construct mock lexicon and set attributes
lexicon = Mock('Lexicon')
lexicon.max_num_of_vars = 5
lexicon.hierarchy = {
    0: {'ordinal':0, 'lineage':set([0]), 'parent': None},
    1: {'ordinal':1, 'lineage':set([0]), 'parent': 0},
    2: {'ordinal':2, 'lineage':set([0]), 'parent': 1}}
lexicon.get_variable_ind.mock_returns = (1,0)

# construct mock modelchain and set attributes
modelchain = Mock('ModelChain')
modelchain.tasks = []
mcc = Mock('ModelChainCollection')
mcc.chains = [modelchain]

# construct mock timespan and set attributes
timespan = Mock('Timespan')
timespan.ending.type = 'steps'
timespan.ending.target = 1
timespan.save_steps_to_db = 'all'
timespan.chain_collections = {'init': [mcc],
                   'simulation': [mcc],
                   'operation': [mcc]}
timespan.forced_operations = False
timespan.unit = 'year'
timespan.time_step = 5

# construct mock simulation control and set attributes
ctrl = Mock('Ctrl')
ctrl.main_level = 1
ctrl.time_spans = [timespan]
ctrl.initial_date = dt.date(2000,1,1)
ctrl.growth_season_start_month = 6
ctrl.growth_season_start_day = 1
ctrl.growth_season_end_month = 7
ctrl.growth_season_end_day = 15
ctrl.all_branching_groups = {'bgroup1': [[(None, {'btask1': ['cond1']})], True]}
ctrl.built_ins = {names.TIME_STEP: (0, 2), names.YEAR: (1, 3), 
                  names.MONTH: (1, 4), names.DAY: (1, 5), 
                  names.ITERATION: (0, 3)}
ctrl.init_variables = None
ctrl.max_model_chain_depth = 1

# construct mock evaluator and set attributes
evaluator = Mock('Evaluator')
evaluator.evaluate.mock_returns = np.array([True, False, False], dtype=bool)

# construct mock databases
input_db = Mock('InputDB')
def get_ids(*args, **kwargs):
    ids = list([(u'1',u'1'), (u'2',u'2'), (u'3',u'3'),
                (u'4',u'4'), (u'5',u'5'), ('6','6')])
    if not 'id_list' in kwargs or not kwargs['id_list']:
        return ids
    else:
        return [i for i in ids if i[1] in kwargs['id_list']]
input_db.get_ids.mock_returns_func = get_ids
data_db = Mock('DataDB')
data_db.add_opres.mock_returns = []
opres_db = Mock('OpresDB')
operation_db = Mock('OperationDB')
cursor = Mock('MockDBCursor')
cursor.fetchone.mock_returns = (10, )
operation_db.conn.execute.mock_returns = cursor

# construct mock data
data = Mock('Handler')
data.get_tind.mock_returns = (np.array([[0,0,0,0,0],
                                        [0,0,1,0,0],
                                        [0,0,2,0,0]], dtype=int),
                              None)
data.get_date.mock_returns = np.zeros(3, dtype=dt.date)
data.get_value.mock_returns = np.array([5.,5.,5.], dtype=float), None
data.brancher.stop.mock_returns = []

fake_handler = Mock('HandlerCreator')
def return_data(*args, **kwargs):
    return data
fake_handler.Handler.mock_returns_func = return_data
