// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  StaticstandLibrary.cpp
//
//	Stand model library - Source file
//
//	Here all model functions must be defined

#include <stdio.h>
#include "StandmodelLibrary.h"
#include "TreemodelLibrary.h"
#include "SeedlingmodelLibrary.h"
#include "SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


int Volume_BAHdomF (double BA, double H_dom, double F,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;
	double res = BA * H_dom * F;
	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_of_bark_spruce_VuokilaValiaho (double BA, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=", BA);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =pow(10,-0.15681+0.78270*log10(BA)-0.33235*log10(H_dom));

	//Correction for old and very thin forests (seed tree stands). Bark percentage should't ever be more than 35 %.

	if(res<=0||res>0.35*BA)
		res=0.35*BA;


	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_under_bark_pine_VuokilaValiahoLappi (double BA, double Age, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	double res =(BA-(0.086641*H_dom-0.013987*Age+0.67175))/1.15393;

	//Correction for old and very thin forests (seed tree stands). Bark percentage should't ever be more than 35 %.

	if(res<0.65*BA)
		res=0.65*BA;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_of_bark_based_on_BA_u_spruce_VuokilaValiaho (double BA_u, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA_u<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA_u<=0(BA_u=",BA_u);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =pow(10,0.080806+0.62909*log10(BA_u)-0.30883*log10(H_dom));

	//Correction for old and very thin forests (seed tree stands). Bark percentage should't ever be more than 35 %.

	if(res>0.538*BA_u)
		res=0.538*BA_u;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_of_bark_based_on_BA_u_pine_VuokilaValiaho (double BA_u, double Age, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	double res =0.67175+0.086641*H_dom-0.013987*Age+0.15393*BA_u;

	//Correction for old and very thin forests (seed tree stands). Bark percentage should't ever be more than 35 %.

	if(res>0.538*BA_u)
		res=0.538*BA_u;

	*modelresult = res * rectFactor;
	return ret;

}


int Yield_yield_class1_Gustavsenym (double V, double N, double T_dr,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.4035+0.8854*log(V)+0.1698*log(T_dr+1)-0.1233*log(N)+pow(0.7889,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Yield_yield_class2_Gustavsenym (double V, double N, double T_dr,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.6369+0.8754*log(V)+0.1373*log(T_dr+1)-0.133*log(N)+pow(1.0017,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Yield_yield_class3_Gustavsenym (double V, double N, double T_dr,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.57+0.8049*log(V)+0.1498*log(T_dr+1)-0.1069*log(N)+pow(1.1403,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Yield_yield_class4_Gustavsenym (double V, double N, double T_dr,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (T_dr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "T_dr<=0(T_dr=",T_dr);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.401+0.8421*log(V)+0.1502*log(T_dr+1)-0.0979*log(N)+pow(0.9511,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Form_factor_pine_VuokilaValiaho (double Age, double H_dom, double BA,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =0.49372-0.001628*H_dom+4.4746/(Age*BA);

	*modelresult = res * rectFactor;
	return ret;

}


int Form_factor_spruce_VuokilaValiaho (double BA, double N, double SI_100,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (SI_100<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_100<=0(SI_100=",SI_100);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(-0.042807*log(BA)-0.049259*log(N)-0.087706*log(SI_100));

	*modelresult = res * rectFactor;
	return ret;

}


int Form_factor_white_birch_Oikarinen (double D_g, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (D_g<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_g<=0(D_g=",D_g);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =0.43433+4.8749/(D_g*H_dom);

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_white_birch_Oikarinen (double V, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =2.2958*(V/H_dom);

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_pubescent_birch_Saramaki (double V_u, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (V_u<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_u<=0(V_u=",V_u);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.38769+0.86069*log(V_u)-0.90798*log(H_dom));

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_pine_Nissinen (double Age, double D_gM, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_ari<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_ari<=0(D_ari=",D_ari);
	}
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(-8.92+0.987*log(N)+0.0007*Age+1.855*log(D_ari)+pow(0.0641,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_spruce_Nissinen (double Age, double D_gM, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	double res = exp(-8.842+0.981*log(N)+0.00183*Age+1.851*log(D_ari)+pow(0.1032,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_birch_Nissinen (double D_gM, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	//Check for fatal errors
	char errorStr[200] = "";
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (D_ari<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_ari<=0(D_ari=",D_ari);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res = exp(-9.086+0.999*log(N)+1.887*log(D_ari)+pow(0.069,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Number_of_trees_per_hectare_white_birch_Oikarinen (double V, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =(136186.22*pow(V,1.23737))/pow(H_dom,3.80738);

	*modelresult = res * rectFactor;
	return ret;

}


int Proportion_of_stems_removed_in_thinning_pine_VuokilaValiaho (double BA, double N, double V, double RV,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =-0.25627+1.132*RV+0.26511*(N/BA)-0.86598*(N/V);

	*modelresult = res * rectFactor;
	return ret;

}


int Proportion_of_stems_removed_in_thinning_spruce_VuokilaValiaho (double BA, double N, double V,
						double H_dom, double RV, double THIN1, double THIN2,
					    int *nres, double *modelresult, char *errors,
					    int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.1842*RV+0.97946*H_dom+0.42868*(N/BA)-1.1804*(N/V)-22.134*THIN1-25.383*THIN2;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_pine_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4, SC5, SC6;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4 = 1;
			SC5 = 0;
			SC6 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 1;
			SC6 = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC6
		case 7:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC6
		case 8:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.63148-22.2704*(1/Age)-531.224*(1/TS)+0.121626*(SC1_2)-0.10922*(SC4)-0.2337*(SC5)-0.34533*(SC6)+pow(0.25509,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_spruce_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.05171-32.6151*(1/Age)-833.769*(1/TS)+0.13436*(SC1_2)+0.13087*(SC4_8)+pow(0.17061,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_birch_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.41969-30.4989*(1/Age)-1242.52*(1/TS)+0.09317*(SC1_2)-0.128522*(SC4_8)+pow(0.2677,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_pine_peatland_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_3, SC5_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 2:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 3:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 6:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 7:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 8:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.99489-31.90071*(1/Age)-1193.892*(1/TS)+0.15903*(SC1_3)-0.2714*(SC5_8)+pow(0.25137,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_spruce_peatland_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.04996-36.3124*(1/Age)-995.022*(1/TS)+0.12632*(SC1_2)-0.14404*(SC4_8)+pow(0.23947,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_birch_peatland_entire_country_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.90435-21.9695*(1/Age)-1014.01*(1/TS)+0.02373*(SC1_2)-0.07576*(SC4_8)+pow(0.21367,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_pine_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4, SC5, SC6;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4 = 1;
			SC5 = 0;
			SC6 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 1;
			SC6 = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC6
		case 7:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC6
		case 8:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.688644-22.4445*(1/Age)-543.591*(1/TS)+0.090541*(SC1_2)-0.144356*(SC4)-0.30466*(SC5)-0.416493*(SC6)+pow(0.23264,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_pine_JKK (double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<0)
	{
		constructErrorMessage (errorStr, "H_dom<0(H_dom=",H_dom);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_dom<=20.04)
	     *modelresult =0.9228*H_dom;
	else
		 *modelresult =H_dom-1.5;
	return ret;

}


int Mean_height_spruce_JKK (double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<0)
	{
		constructErrorMessage (errorStr, "H_dom<0(H_dom=",H_dom);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_dom<=22.06)
	     *modelresult =0.8884*H_dom;
	else
		 *modelresult =H_dom-2.4;
	return ret;

}


int Mean_height_birch_JKK (double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_dom<0)
	{
		constructErrorMessage (errorStr, "H_dom<0(H_dom=",H_dom);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_dom<=20.09)
	     *modelresult =0.9055*H_dom;
	else
		 *modelresult =H_dom-1.8;
	return ret;

}


int Dominant_height_pine_JKK (double H_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<0)
	{
		constructErrorMessage (errorStr, "H_gM<0(H_gM=",H_gM);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_gM<=17.93)
	     *modelresult =H_gM/0.9228;
	else
		 *modelresult =H_gM+1.5;
	return ret;

}


int Dominant_height_spruce_JKK (double H_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<0)
	{
		constructErrorMessage (errorStr, "H_gM<0(H_gM=",H_gM);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_gM<=19.1)
	     *modelresult = H_gM / 0.8884;
	else
		 *modelresult = H_gM + 2.4;
	return ret;
}


int Dominant_height_birch_JKK (double H_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<0)
	{
		constructErrorMessage (errorStr, "H_gM<0(H_gM=",H_gM);
		ret = 0;
	}
	if (ret == 0) {
                strcat(errors, errorStr);
                return 0;
        }

	*nres = 1;

	if(H_gM<=17.24)
	     *modelresult =H_gM/0.9055;
	else
		 *modelresult =H_gM+1.8;
	return ret;

}


int Mean_height_spruce_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.049161-31.5081*(1/Age)-840.636*(1/TS)+0.128494*(SC1_2)-0.23874*(SC4_8)+pow(0.15176,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_birch_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.469634-18.6961*(1/Age)-1451.94*(1/TS)-0.073275*(SC1_2)-0.015160*(SC4_8)+pow(0.19272,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_pine_peatland_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_3, SC5_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 2:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 3:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 6:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 7:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 8:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.82328-27.7346*(1/Age)-1083.369*(1/TS)+0.217054*(SC1_3)-0.26347*(SC5_8)+pow(0.22493,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_spruce_peatland_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(4.17346-34.8561*(1/Age)-1154.287*(1/TS)+0.152379*(SC1_2)-0.263315*(SC4_8)+pow(0.22699,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_birch_peatland_Nuutinen (double Age, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(3.75690-21.7536*(1/Age)-846.292*(1/TS)+0.038484*(SC1_2)-0.051519*(SC4_8)+pow(0.21267,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_white_birch_Oikarinen (double BA, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =0.56286+101.18*sqrt((4*BA)/(PI*N));

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_pubescent_birch_Saramaki (double Age, double V_u, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double D_gu;

	//Calculate first mean diameter under bark
	D_gu=-2.0946+0.0378*Age-0.0244*V_u+1.0102*H_dom;

	*nres = 1;

	double res =(1.1020-0.0019*D_gu-0.0002*H_dom)*D_gu;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_pine_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC3, SC5, SC6;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC3 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5 = 0;
			SC6 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 1;
			SC6 = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC6
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC6
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 0;
			SC6 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5 = 0;
			SC6 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.14978+0.61808*log(H_gM-1.3)+0.003635*Age-0.00000415*pow(Age,2)+0.0000000195*pow(TS,2)+0.16384*(SC1_2)+0.052283*(SC3)-0.013817*(SC5)+0.026128*(SC6)+pow(0.16459,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_spruce_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.92736+0.85904*log(H_gM-1.3)+0.00000414*pow(Age,2)-0.000000133*pow(TS,2)-0.051414*(SC1_2)+0.063884*(SC4_8)+pow(0.14178,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_birch_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.63947+0.632109*log(H_gM-1.3)+0.011601*Age-0.0000447*pow(Age,2)+0.00000000721*pow(TS,2)+0.091265*(SC1_2)-0.013628*(SC4_8)+pow(0.18154,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_pine_peat_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_3, SC5_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 2:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 3:
			SC1_3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 6:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 7:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		case 8:
			SC1_3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_3 = 0;
			SC5_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.03648+0.746814*log(H_gM-1.3)+0.00000711*pow(Age,2)-0.00000000201*pow(TS,2)-0.012102*(SC1_3)+0.02827*(SC5_8)+pow(0.14659,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_spruce_peat_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.96037+0.811069*log(H_gM-1.3)+0.00000468*pow(Age,2)-0.000000101*pow(TS,2)+0.057116*(SC1_2)-0.015813*(SC4_8)+pow(0.12213,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_birch_peat_Nuutinen (double Age, double H_gM, double TS, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4_8;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.48591+0.78193*log(H_gM-1.3)+0.008532*Age-0.0000337*pow(Age,2)-0.000000124*pow(TS,2)+0.064278*(SC1_2)+0.069132*(SC4_8)+pow(0.15993,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_pine_Nissinen (double Age, double D_ari, double TREAT,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	double res =1.798+1.061*D_ari-0.357*TREAT-0.011*Age;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_spruce_Nissinen (double Age, double D_ari,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	double res =0.82+0.00652*Age+1.067*D_ari;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_birch_Nissinen (double D_ari,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (D_ari<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_ari<=0(D_ari=",D_ari);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.603+0.812*log(D_ari)+pow(0.1215,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Site_index_pine_Hynynenym (double TS, double ALT,
					double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC2, SC3, SC5;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC2 = 0;
			SC3 = 0;
			SC5 = 0;
			break;
		case 2:
			SC2 = 1;
			SC3 = 0;
			SC5 = 0;
			break;
		case 3:
			SC2 = 0;
			SC3 = 1;
			SC5 = 0;
			break;
		case 4:
			SC2 = 0;
			SC3 = 0;
			SC5 = 0;
			break;
		case 5:
			SC2 = 0;
			SC3 = 0;
			SC5 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC2 = 0;
			SC3 = 0;
			SC5 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC2 = 0;
			SC3 = 0;
			SC5 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC2 = 0;
			SC3 = 0;
			SC5 = 1;
			break;
		// defaults to SC==4 or SC==1, sensible?
		default:
			SC2 = 0;
			SC3 = 0;
			SC5 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation:
	/*if(TS>1340)
		TS=1340;*/

	*nres = 1;

		//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	double res =exp(3.578+1.453*(TS/1000)+0.0875*(ALT/100)+0.1647*LAKE-0.4558*SEA+0.1385*SC2+0.0782*SC3-0.1363*SC5-0.1013*STONY-0.0365*PALU-
			0.1718*HUMUS+0.1308*sqrt(RDF)-6.054*pow(50,-0.2)+pow(0.178,2)/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Site_index_spruce_Hynynenym (double TS, double LAKE,
					double SEA, double SC, double STONY, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1, SC2, SC4;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			break;
		// SC 5 treated here as SC4
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			break;
	}

	//To avoid extrapolation:
	/*if(TS>1340)
		TS=1340;*/

	*nres = 1;

		//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
    double res =exp(3.418+1.48*(TS/1000)+0.08331*LAKE-0.7855*SEA+0.2112*SC1+0.1874*SC2-0.2005*SC4-0.0434*STONY-0.0892*PALU-0.1631*HUMUS
			-6.337*pow(50,-0.25)+pow(0.176,2)/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Site_index_birch_Hynynenym (double TS, double LAKE,
					double PLANT, double SC, double PUB, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1, SC2;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC1 = 0;
			SC2 = 0;
			break;
		// defaults to SC==3 or SC=4 or SC=5, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			break;
	}

	//To avoid extrapolation:
	/*if(TS>1290)
		TS=1290;*/

	*nres = 1;

		//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
    double res =exp(3.155+0.6528*(TS/1000)-0.05292*LAKE+0.03088*SC1+0.01246*SC2+0.1871*PALU-0.445*HUMUS
			+0.3229*PLANT-0.175*PUB-6.569*pow(50,-0.5)+pow(0.0745,2)/2)+1.3;
	* modelresult = res * rectFactor;
	return ret;

}

int Need_for_ditching_Hokkaym (double LAT, double SINCE_DRAINAGE,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since drainage dummy variables

	double DR_0_5, DR_6_10, DR_11_25;
	DR_0_5 = 0;
	DR_6_10 = 0;
	DR_11_25 = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 5 &&SINCE_DRAINAGE <= 10)
		DR_6_10 = 1;
	if(SINCE_DRAINAGE > 10 &&SINCE_DRAINAGE <= 25)
		DR_11_25 = 1;

	*nres = 1;
	double res =1/(1+exp(13.5728-(0.00201*LAT-3.2307*DR_0_5-2.513*DR_6_10-1.3532*DR_11_25)));

	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_at_stump_height_from_D_gM_Laasasenaho (double D_gM,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	double res;
	if (D_gM>0)
		res = 2+1.25*D_gM;
	else
		res = 0;
	*modelresult = res * rectFactor;
	return ret;

}


int Proportion_of_bark_birch_from_stand_basal_area_JKK (double H_dom, int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if(H_dom<3)
	     *modelresult =35;
	if(H_dom>=3 && H_dom<5)
		 *modelresult =25;
	if(H_dom>=5 && H_dom<8)
		 *modelresult =20;
	if(H_dom>=8 && H_dom<12)
		 *modelresult =17;
	if(H_dom>=12)
	     *modelresult =13;
	return ret;

}


int Basal_area_under_bark_birch_JKK (double BA, double H_dom, int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
	double bark_percent;


	if(H_dom<3)
	     bark_percent =35;
	if(H_dom>=3 && H_dom<5)
		 bark_percent =25;
	if(H_dom>=5 && H_dom<8)
		 bark_percent =20;
	if(H_dom>=8 && H_dom<12)
		 bark_percent =17;
	if(H_dom>=12)
	     bark_percent =13;

	*nres = 1;

	*modelresult =BA-BA*(bark_percent/100);

	return ret;

}


int Two_storey_forest_Hynynenym (double Upper_sum,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	if(Upper_sum>=1)
		*modelresult =1;
	else
		*modelresult =0;
	return ret;

}


int Upper_storey_stratum_Hynynenym (double H_gM, double H_ari, double N_stratum,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	if(N_stratum==1)
		*modelresult =0;
	else if(H_gM>4){
		if(H_gM>1.2*H_ari)
			*modelresult =1;
		else
			*modelresult =0;
	}
	else
	    *modelresult =0;

	return ret;

}

int Upper_storey_stratum_laser_JKK (double H_gM, double H_ari, double N_stratum,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	if(N_stratum==1)
		*modelresult =0;
	else if(H_gM>8){
		if(H_gM>1.2*H_ari)
			*modelresult =1;
		else
			*modelresult =0;
	}
	else
	    *modelresult =0;

	return ret;

}


int Site_index_pine_Gustavsen (double Age, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res = 128.229*exp((log(H_dom)-log(128.229))/exp(4.70248/pow(Age,0.47692)-4.70248/pow(100,0.47692)));

	*modelresult = res * rectFactor;
	return ret;

}


int Site_index_spruce_Gustavsen (double Age, double H_dom,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (H_dom<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_dom<=0(H_dom=",H_dom);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =147.481*exp((log(H_dom)-log(147.481))/exp(4.64631/pow(Age,0.29981)-4.64631/pow(100,0.29981)));

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_of_stratum_JKK (double D_gM, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;
	D_gM = D_gM / 100;

	*modelresult = N * PI * pow(D_gM / 2, 2);
	return ret;

}


int Basal_area_of_stratum_Niemisto (double D_gM, double N,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	*modelresult = exp((log(N)-9.29-0.0033*pow(D_ari,2)+8.963*(acos(sqrt((40-D_ari)/40))))/0.952);
	return ret;

}


static void Productive_value_of_pine_Pukkala (double TS, double SC, double D_g, double BA, double Age, double IR, double Price_log, double Price_pulp, double H_gM, struct PVs * pv_Sp) {

	double pv,pv_land,pv1,pv2,pv3,pv4,pv5,pv6,pv7,pv_land1,pv_land2,pv_land3,pv_land4,pv_land5,pv_land6,pv_land7;

	int SC3, SC4, SC5;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 2:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 3:
			SC3 = 1;
			SC4 = 0;
			SC5 = 0;
			break;
		case 4:
			SC3 = 0;
			SC4 = 1;
			SC5 = 0;
			break;
		case 5:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// defaults to SC==1 or 2, sensible?
		default:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
	};

        // Pukkala's constraints
	if(IR>5)
        	IR=5;
        if(Age>100)
		Age=100;
	double TS_land = TS;
	if(TS_land<600)
		TS_land=600;
		
	//Calculate productive value for bare land
	pv_land = pow(-6.802+0.043*TS_land+18.333*IR-71.989*log(IR)+0.592*Price_log+1.384*Price_pulp-0.14*Price_log*IR
			-0.143*Price_pulp*IR-10.057*SC3-18.036*SC4-26.655*SC5,2);
	pv_land1 = pow(-6.802+0.043*TS_land+18.333*1-71.989*log(1.)+0.592*Price_log+1.384*Price_pulp-0.14*Price_log*1
			-0.143*Price_pulp*1-10.057*SC3-18.036*SC4-26.655*SC5,2);
	pv_land2 = pow(-6.802+0.043*TS_land+18.333*2-71.989*log(2.)+0.592*Price_log+1.384*Price_pulp-0.14*Price_log*2
			-0.143*Price_pulp*2-10.057*SC3-18.036*SC4-26.655*SC5,2);
	pv_land3 = pow(-6.802+0.043*TS_land+18.333*3-71.989*log(3.)+0.592*Price_log+1.384*Price_pulp-0.14*Price_log*3
			-0.143*Price_pulp*3-10.057*SC3-18.036*SC4-26.655*SC5,2);
	pv_land4 = pow(-6.802+0.043*TS_land+18.333*4-71.989*log(4.)+0.592*Price_log+1.384*Price_pulp-0.14*Price_log*4
			-0.143*Price_pulp*4-10.057*SC3-18.036*SC4-26.655*SC5,2);
	pv_land5 = 0.6*pv_land4;
	pv_land6 = 0.4*pv_land4;
	pv_land7 = 0.25*pv_land4;
	// Check logicality of values
	if (pv_land2>pv_land1)
		pv_land2 = pv_land1;
	if (pv_land3>pv_land2)
		pv_land3 = pv_land2;
	if (pv_land4>pv_land3)
		pv_land4 = pv_land3;
	if (pv_land5>pv_land4)
		pv_land5 = pv_land4;
	if (pv_land6>pv_land5)
		pv_land6 = pv_land5;
	if (pv_land7>pv_land6)
		pv_land7 = pv_land6;	

	if (H_gM>=3 && BA>0)
    	{
		//Calculate productive value for growing stock
		pv = pow(10.384+0.0418*TS-0.000797*TS*D_g+12.611*IR-54.066*log(IR)+0.46*log(IR)*D_g+0.000331*Price_log*TS+1.524*Price_pulp
			-0.287*Price_log*log(IR)-0.425*Price_pulp*log(IR)+0.599*D_g+0.0239*Price_log*D_g-0.00961*Price_pulp*D_g+1.621*BA
			-4.751*log(BA)-0.058*Age-29.786*D_g/Age-5.325*SC3-12.152*SC4-16.186*SC5,2);
		pv1 = pow(10.384+0.0418*TS-0.000797*TS*D_g+12.611*1-54.066*log(1.)+0.46*log(1.)*D_g+0.000331*Price_log*TS+1.524*Price_pulp
			-0.287*Price_log*log(1.)-0.425*Price_pulp*log(1.)+0.599*D_g+0.0239*Price_log*D_g-0.00961*Price_pulp*D_g+1.621*BA
			-4.751*log(BA)-0.058*Age-29.786*D_g/Age-5.325*SC3-12.152*SC4-16.186*SC5,2);
		pv2 = pow(10.384+0.0418*TS-0.000797*TS*D_g+12.611*2-54.066*log(2.)+0.46*log(2.)*D_g+0.000331*Price_log*TS+1.524*Price_pulp
			-0.287*Price_log*log(2.)-0.425*Price_pulp*log(2.)+0.599*D_g+0.0239*Price_log*D_g-0.00961*Price_pulp*D_g+1.621*BA
			-4.751*log(BA)-0.058*Age-29.786*D_g/Age-5.325*SC3-12.152*SC4-16.186*SC5,2);
		pv3 = pow(10.384+0.0418*TS-0.000797*TS*D_g+12.611*3-54.066*log(3.)+0.46*log(3.)*D_g+0.000331*Price_log*TS+1.524*Price_pulp
			-0.287*Price_log*log(3.)-0.425*Price_pulp*log(3.)+0.599*D_g+0.0239*Price_log*D_g-0.00961*Price_pulp*D_g+1.621*BA
			-4.751*log(BA)-0.058*Age-29.786*D_g/Age-5.325*SC3-12.152*SC4-16.186*SC5,2);
		pv4 = pow(10.384+0.0418*TS-0.000797*TS*D_g+12.611*4-54.066*log(4.)+0.46*log(4.)*D_g+0.000331*Price_log*TS+1.524*Price_pulp
			-0.287*Price_log*log(4.)-0.425*Price_pulp*log(4.)+0.599*D_g+0.0239*Price_log*D_g-0.00961*Price_pulp*D_g+1.621*BA
			-4.751*log(BA)-0.058*Age-29.786*D_g/Age-5.325*SC3-12.152*SC4-16.186*SC5,2);
		pv5 = 0.6*pv4;
		pv6 = 0.4*pv4;
		pv7 = 0.25*pv4;
		// Check logicality of values
		if (pv2>pv1)
			pv2 = pv1;
		if (pv3>pv2)
			pv3 = pv2;
		if (pv4>pv3)
			pv4 = pv3;
		if (pv5>pv4)
			pv5 = pv4;
		if (pv6>pv5)
			pv6 = pv5;
		if (pv7>pv6)
			pv7 = pv6;
	}
    	else
    	//If hgm lt 3 m bare land productive value is used
    	{
	        pv=pv_land;
	        pv1=pv_land1;
	        pv2=pv_land2;
	        pv3=pv_land3;
	        pv4=pv_land4;
	        pv5=pv_land5;
	        pv6=pv_land6;
	  	pv7=pv_land7;
    	}
	
	pv_Sp[0].PV = pv;
	pv_Sp[0].PV_land = pv_land;
	pv_Sp[0].PV_1 = pv1;
	pv_Sp[0].PV_2 = pv2;
	pv_Sp[0].PV_3 = pv3;
	pv_Sp[0].PV_4 = pv4;
	pv_Sp[0].PV_5 = pv5;
	pv_Sp[0].PV_6 = pv6;
	pv_Sp[0].PV_7 = pv7;
	pv_Sp[0].PV_land_1 = pv_land1;
	pv_Sp[0].PV_land_2 = pv_land2;
	pv_Sp[0].PV_land_3 = pv_land3;
	pv_Sp[0].PV_land_4 = pv_land4;
	pv_Sp[0].PV_land_5 = pv_land5;
	pv_Sp[0].PV_land_6 = pv_land6;
	pv_Sp[0].PV_land_7 = pv_land7;

}


static void Productive_value_of_spruce_Pukkala (double TS, double SC, double D_g, double BA, double Age,double IR, double Price_log, double Price_pulp, double H_gM, struct PVs *pv_Ns) {

	double pv,pv_land,pv1,pv2,pv3,pv4,pv5,pv6,pv7,pv_land1,pv_land2,pv_land3,pv_land4,pv_land5,pv_land6,pv_land7;

	int SC3, SC4;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC3 = 0;
			SC4 = 0;
			break;
		case 2:
			SC3 = 0;
			SC4 = 0;
			break;
		case 3:
			SC3 = 1;
			SC4 = 0;
			break;
		case 4:
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 5 not present in original publication, treated here as SC4
		case 5:
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC3 = 0;
			SC4 = 1;
			break;
		// defaults to SC==1 or 2, sensible?
		default:
			SC3 = 0;
			SC4 = 0;
			break;
	};

	// Pukkala's constraints
	if(IR>5)
        	IR=5;
        if(Age>100)
		Age=100;
	double TS_land = TS;
	if(TS_land<600)
		TS_land=600;
		
	//Calculate productive value for bare land
	if (936+118*IR-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land = pow(-71.947+0.105*TS_land+29.405*IR-97.095*log(IR)+1.069*Price_log+1.058*Price_pulp-0.241*Price_log*IR
			-0.156*Price_pulp*IR-18.946*SC3-32.78*SC4,2);
	}
	else
	{
		pv_land = 0;
	}
	if (936+118*1-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land1 = pow(-71.947+0.105*TS_land+29.405*1-97.095*log(1.)+1.069*Price_log+1.058*Price_pulp-0.241*Price_log*1
			-0.156*Price_pulp*1-18.946*SC3-32.78*SC4,2);
	}
	else
	{
		pv_land1 = 0;
	}
	if (936+118*2-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land2 = pow(-71.947+0.105*TS_land+29.405*2-97.095*log(2.)+1.069*Price_log+1.058*Price_pulp-0.241*Price_log*2
			-0.156*Price_pulp*2-18.946*SC3-32.78*SC4,2);
	}
	else
	{
		pv_land2 = 0;
	}
	if (936+118*3-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land3 = pow(-71.947+0.105*TS_land+29.405*3-97.095*log(3.)+1.069*Price_log+1.058*Price_pulp-0.241*Price_log*3
			-0.156*Price_pulp*3-18.946*SC3-32.78*SC4,2);
	}
	else
	{
		pv_land3 = 0;
	}
	if (936+118*4-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land4 = pow(-71.947+0.105*TS_land+29.405*4-97.095*log(4.)+1.069*Price_log+1.058*Price_pulp-0.241*Price_log*4
			-0.156*Price_pulp*4-18.946*SC3-32.78*SC4,2);
	}
	else
	{
		pv_land4 = 0;
	}
	if (936+118*5-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{	
		pv_land5 = 0.6*pv_land4;;
	}
	else
	{
		pv_land5 = 0;
	}
	if (936+118*6-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land6 = 0.4*pv_land4;
	}
	else
	{
		pv_land6 = 0;
	}
	if (936+118*7-44*Price_pulp+604*SC3+1157*SC4<=TS)
	{
		pv_land7 = 0.25*pv_land4;
	}
	else
	{
		pv_land7 = 0;
	}
	// Check logicality of values
	if (pv_land2>pv_land1)
		pv_land2 = pv_land1;
	if (pv_land3>pv_land2)
		pv_land3 = pv_land2;
	if (pv_land4>pv_land3)
		pv_land4 = pv_land3;
	if (pv_land5>pv_land4)
		pv_land5 = pv_land4;
	if (pv_land6>pv_land5)
		pv_land6 = pv_land5;
	if (pv_land7>pv_land6)
		pv_land7 = pv_land6;
	
	if (H_gM>=3 && BA>0)
    	{
		//Calculate productive value for growing stock
		
		pv = pow(-34.371+0.0891*TS-0.00244*TS*D_g+13.108*IR-57.87*log(IR)+0.643*log(IR)*D_g+0.000662*Price_log*TS
			+1.365*Price_pulp-0.414*Price_log*log(IR)-0.441*Price_pulp*log(IR)+0.0205*Price_log*D_g
			-0.00565*Price_pulp*D_g+1.377*BA+0.577*Age-21.782*D_g/Age-17.356*SC3-29.99*SC4,2);
		pv1 = pow(-34.371+0.0891*TS-0.00244*TS*D_g+13.108*1-57.87*log(1.)+0.643*log(1.)*D_g+0.000662*Price_log*TS
			+1.365*Price_pulp-0.414*Price_log*log(1.)-0.441*Price_pulp*log(1.)+0.0205*Price_log*D_g
			-0.00565*Price_pulp*D_g+1.377*BA+0.577*Age-21.782*D_g/Age-17.356*SC3-29.99*SC4,2);
		pv2 = pow(-34.371+0.0891*TS-0.00244*TS*D_g+13.108*2-57.87*log(2.)+0.643*log(2.)*D_g+0.000662*Price_log*TS
			+1.365*Price_pulp-0.414*Price_log*log(2.)-0.441*Price_pulp*log(2.)+0.0205*Price_log*D_g
			-0.00565*Price_pulp*D_g+1.377*BA+0.577*Age-21.782*D_g/Age-17.356*SC3-29.99*SC4,2);
		pv3 = pow(-34.371+0.0891*TS-0.00244*TS*D_g+13.108*3-57.87*log(3.)+0.643*log(3.)*D_g+0.000662*Price_log*TS
			+1.365*Price_pulp-0.414*Price_log*log(3.)-0.441*Price_pulp*log(3.)+0.0205*Price_log*D_g
			-0.00565*Price_pulp*D_g+1.377*BA+0.577*Age-21.782*D_g/Age-17.356*SC3-29.99*SC4,2);
		pv4 = pow(-34.371+0.0891*TS-0.00244*TS*D_g+13.108*4-57.87*log(4.)+0.643*log(4.)*D_g+0.000662*Price_log*TS
			+1.365*Price_pulp-0.414*Price_log*log(4.)-0.441*Price_pulp*log(4.)+0.0205*Price_log*D_g
			-0.00565*Price_pulp*D_g+1.377*BA+0.577*Age-21.782*D_g/Age-17.356*SC3-29.99*SC4,2);
		pv5 = 0.6*pv4;
		pv6 = 0.4*pv4;
		pv7 = 0.25*pv4;
		// Check logicality of values
		if (pv2>pv1)
			pv2 = pv1;
		if (pv3>pv2)
			pv3 = pv2;
		if (pv4>pv3)
			pv4 = pv3;
		if (pv5>pv4)
			pv5 = pv4;
		if (pv6>pv5)
			pv6 = pv5;
		if (pv7>pv6)
			pv7 = pv6;
	}
    	else
    	//If hgm lt 3 m bare land productive value is used
    	{
	        pv=pv_land;
	        pv1=pv_land1;
	        pv2=pv_land2;
	        pv3=pv_land3;
	        pv4=pv_land4;
	        pv5=pv_land5;
	        pv6=pv_land6;
	  	pv7=pv_land7;
    	}
	
	pv_Ns[0].PV = pv;
	pv_Ns[0].PV_land = pv_land;
	pv_Ns[0].PV_1 = pv1;
	pv_Ns[0].PV_2 = pv2;
	pv_Ns[0].PV_3 = pv3;
	pv_Ns[0].PV_4 = pv4;
	pv_Ns[0].PV_5 = pv5;
	pv_Ns[0].PV_6 = pv6;
	pv_Ns[0].PV_7 = pv7;
	pv_Ns[0].PV_land_1 = pv_land1;
	pv_Ns[0].PV_land_2 = pv_land2;
	pv_Ns[0].PV_land_3 = pv_land3;
	pv_Ns[0].PV_land_4 = pv_land4;
	pv_Ns[0].PV_land_5 = pv_land5;
	pv_Ns[0].PV_land_6 = pv_land6;
	pv_Ns[0].PV_land_7 = pv_land7;
}


static void Productive_value_of_white_birch_Pukkala (double TS, double SC, double D_g, double BA, double Age, double N, double IR, double Price_log, double Price_pulp, double H_gM, struct PVs *pv_b) {

	double pv,pv_land,pv1,pv2,pv3,pv4,pv5,pv6,pv7,pv_land1,pv_land2,pv_land3,pv_land4,pv_land5,pv_land6,pv_land7;

	int SC3;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC3 = 0;
			break;
		case 2:
			SC3 = 0;
			break;
		case 3:
			SC3 = 1;
			break;
		// SC 4 not present in original publication, treated here as SC3
		case 4:
			SC3 = 1;
			break;
		// SC 5 not present in original publication, treated here as SC3
		case 5:
			SC3 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC3 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC3 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC3 = 1;
			break;
		// defaults to SC==1 or 2, sensible?
		default:
			SC3 = 0;
			break;
	};

       // Pukkala's constraints
	if(IR>5)
        	IR=5;
        if(Age>100)
		Age=100;
	double TS_land = TS;
	if(TS_land<600)
		TS_land=600;
		
	//Calculate productive value for bare land
	pv_land = pow(-18.881+0.039*TS_land+16.681*IR-65.995*log(IR)+0.547*Price_log+1.738*Price_pulp-0.129*Price_log*IR
			-0.166*Price_pulp*IR-12.06*SC3,2);
	pv_land1 = pow(-18.881+0.039*TS_land+16.681*1-65.995*log(1.)+0.547*Price_log+1.738*Price_pulp-0.129*Price_log*1
			-0.166*Price_pulp*1-12.06*SC3,2);
	pv_land2 = pow(-18.881+0.039*TS_land+16.681*2-65.995*log(2.)+0.547*Price_log+1.738*Price_pulp-0.129*Price_log*2
			-0.166*Price_pulp*2-12.06*SC3,2);
	pv_land3 = pow(-18.881+0.039*TS_land+16.681*3-65.995*log(3.)+0.547*Price_log+1.738*Price_pulp-0.129*Price_log*3
			-0.166*Price_pulp*3-12.06*SC3,2);
	pv_land4 = pow(-18.881+0.039*TS_land+16.681*4-65.995*log(4.)+0.547*Price_log+1.738*Price_pulp-0.129*Price_log*4
			-0.166*Price_pulp*4-12.06*SC3,2);
	pv_land5 = 0.6*pv_land4;
	pv_land6 = 0.4*pv_land4;
	pv_land7 = 0.25*pv_land4;
	// Check logicality of values
	if (pv_land2>pv_land1)
		pv_land2 = pv_land1;
	if (pv_land3>pv_land2)
		pv_land3 = pv_land2;
	if (pv_land4>pv_land3)
		pv_land4 = pv_land3;
	if (pv_land5>pv_land4)
		pv_land5 = pv_land4;
	if (pv_land6>pv_land5)
		pv_land6 = pv_land5;
	if (pv_land7>pv_land6)
		pv_land7 = pv_land6;

	if (H_gM>=3 && BA>0)
    	{
		//Calculate productive value for growing stock
		pv = pow(31.152+0.0482*TS-0.000729*TS*D_g+14.832*IR-61.643*log(IR)+0.413*log(IR)*D_g+0.000347*Price_log*TS
				+1.548*Price_pulp-0.328*Price_log*log(IR)-0.268*Price_pulp*log(IR)+1.952*D_g+0.0263*Price_log*D_g
				-0.0169*Price_pulp*D_g+1.944*BA-7.55*log(BA)-0.00753*N-0.676*Age-73.16*D_g/Age-5.534*SC3,2);
		pv1 = pow(31.152+0.0482*TS-0.000729*TS*D_g+14.832*1-61.643*log(1.)+0.413*log(1.)*D_g+0.000347*Price_log*TS
				+1.548*Price_pulp-0.328*Price_log*log(1.)-0.268*Price_pulp*log(1.)+1.952*D_g+0.0263*Price_log*D_g
				-0.0169*Price_pulp*D_g+1.944*BA-7.55*log(BA)-0.00753*N-0.676*Age-73.16*D_g/Age-5.534*SC3,2);
		pv2 = pow(31.152+0.0482*TS-0.000729*TS*D_g+14.832*2-61.643*log(2.)+0.413*log(2.)*D_g+0.000347*Price_log*TS
				+1.548*Price_pulp-0.328*Price_log*log(2.)-0.268*Price_pulp*log(2.)+1.952*D_g+0.0263*Price_log*D_g
				-0.0169*Price_pulp*D_g+1.944*BA-7.55*log(BA)-0.00753*N-0.676*Age-73.16*D_g/Age-5.534*SC3,2);
		pv3 = pow(31.152+0.0482*TS-0.000729*TS*D_g+14.832*3-61.643*log(3.)+0.413*log(3.)*D_g+0.000347*Price_log*TS
				+1.548*Price_pulp-0.328*Price_log*log(3.)-0.268*Price_pulp*log(3.)+1.952*D_g+0.0263*Price_log*D_g
				-0.0169*Price_pulp*D_g+1.944*BA-7.55*log(BA)-0.00753*N-0.676*Age-73.16*D_g/Age-5.534*SC3,2);
		pv4 = pow(31.152+0.0482*TS-0.000729*TS*D_g+14.832*4-61.643*log(4.)+0.413*log(4.)*D_g+0.000347*Price_log*TS
				+1.548*Price_pulp-0.328*Price_log*log(4.)-0.268*Price_pulp*log(4.)+1.952*D_g+0.0263*Price_log*D_g
				-0.0169*Price_pulp*D_g+1.944*BA-7.55*log(BA)-0.00753*N-0.676*Age-73.16*D_g/Age-5.534*SC3,2);
		pv5 = 0.6*pv4;
		pv6 = 0.4*pv4;
		pv7 = 0.25*pv4;
		// Check logicality of values
		if (pv2>pv1)
			pv2 = pv1;
		if (pv3>pv2)
			pv3 = pv2;
		if (pv4>pv3)
			pv4 = pv3;
		if (pv5>pv4)
			pv5 = pv4;
		if (pv6>pv5)
			pv6 = pv5;
		if (pv7>pv6)
			pv7 = pv6;
	}
    	else
    	//If hgm lt 3 m bare land productive value is used
    	{
	        pv=pv_land;
	        pv1=pv_land1;
	        pv2=pv_land2;
	        pv3=pv_land3;
	        pv4=pv_land4;
	        pv5=pv_land5;
	        pv6=pv_land6;
	  	pv7=pv_land7;
    	}
	
	pv_b[0].PV = pv;
	pv_b[0].PV_land = pv_land;
	pv_b[0].PV_1 = pv1;
	pv_b[0].PV_2 = pv2;
	pv_b[0].PV_3 = pv3;
	pv_b[0].PV_4 = pv4;
	pv_b[0].PV_5 = pv5;
	pv_b[0].PV_6 = pv6;
	pv_b[0].PV_7 = pv7;
	pv_b[0].PV_land_1 = pv_land1;
	pv_b[0].PV_land_2 = pv_land2;
	pv_b[0].PV_land_3 = pv_land3;
	pv_b[0].PV_land_4 = pv_land4;
	pv_b[0].PV_land_5 = pv_land5;
	pv_b[0].PV_land_6 = pv_land6;
	pv_b[0].PV_land_7 = pv_land7;

}

int Number_of_trees_JKK (double BA, double D_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	*modelresult =BA/(pow(D_gM/200,2)*PI);
	return ret;

}

int Number_of_trees_Niemisto (double BA, double D_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	double res;

	if(D_gM<36)
		res = exp(9.29+0.952*(log(BA))+0.0033*pow(D_ari,2)-8.963*(acos(sqrt((40-D_ari)/40))));
	else
		res = BA/(pow(D_gM/200,2)*PI);

	*modelresult = res;
	return ret;

}

int Basal_area_Niemisto (double N, double D_gM,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	//Avoid negative values for arithmetical mean diameter
	double D_ari;
	if(D_gM<2)
		D_ari = D_gM * 0.1625;
	else
		D_ari= ((D_gM*10 - 16.87)/0.963)/10;

	double res;

	if(D_gM<36)
		res = exp(log(N)/0.952-9.7584-0.003466*pow(D_ari,2)+9.415*(acos(sqrt((40-D_ari)/40))));
	else
		res = PI*pow(D_gM/200,2)*N;

	*modelresult = res;
	return ret;

}


int Age_at_breast_height_JKK (double Age, double SC, double SP,
						int *nres, double *modelresult, char *errors, int errorCheckMode,
						double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double SP1, SP2, SP3, SP4_9;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SP1 = 7.9;
			SP2 = 12.2;
			SP3 = 6;
			SP4_9 = 7.2;
			break;
		case 2:
			SP1 = 7.9;
			SP2 = 12.2;
			SP3 = 6;
			SP4_9 = 7.2;
			break;
		case 3:
			SP1 = 8.4;
			SP2 = 12.9;
			SP3 = 7.5;
			SP4_9 = 8.2;
			break;
		case 4:
			SP1 = 8.9;
			SP2 = 13.8;
			SP3 = 9;
			SP4_9 = 9.2;
			break;
		case 5:
			SP1 = 10.2;
			SP2 = 14.8;
			SP3 = 10.2;
			SP4_9 = 10.2;
			break;
		case 6:
			SP1 = 12;
			SP2 = 14;
			SP3 = 12;
			SP4_9 = 12;
			break;
		case 7:
			SP1 = 10.2;
			SP2 = 14;
			SP3 = 10.2;
			SP4_9 = 10.2;
			break;
		case 8:
			SP1 = 10.2;
			SP2 = 14;
			SP3 = 10.2;
			SP4_9 = 10.2;
			break;
	}

	*nres = 1;

	double res;

	if(SP==1)
		res =Age-SP1;
	else
		if(SP==2)
			res =Age-SP2;
	else
		if(SP==3)
			res =Age-SP3;
	else
		res =Age-SP4_9;

	*modelresult = res * rectFactor;
	return ret;
}


int Age_at_breast_height_MELA_JKK (double Age, double t_h,
						int *nres, double *modelresult, char *errors, int errorCheckMode,
						double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	*nres = 1;

	double res;

	res = Age - t_h;

	*modelresult = res * rectFactor;
	return ret;
}


int Age_at_breast_height_MELA_tree_JKK (double Age, double t_h,
						int *nres, double *modelresult, char *errors, int errorCheckMode,
						double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	double res;

	res = Age - t_h;


	*modelresult = res * rectFactor;
	return ret;
}


int Age_at_stump_height_JKK (double Age, double SC,
						int *nres, double *modelresult, char *errors, int errorCheckMode,
						double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double x;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			x=2;
			break;
		case 2:
			x=2;
			break;
		case 3:
			x=2;
			break;
		case 4:
			x=3;
			break;
		case 5:
			x=3;
			break;
		case 6:
			x=4;
			break;
		case 7:
			x=4;
			break;
		case 8:
			x=4;
			break;
	}

	*nres = 1;

	double res =Age-x;


	*modelresult = res * rectFactor;
	return ret;
}


int Site_index_pubescent_birch_JKK (double SC,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if(SC==1)
	     *modelresult =22;
	if(SC==2)
		 *modelresult =19;
	if(SC==3)
		 *modelresult =18;
	if(SC==4)
		 *modelresult =16;
	if(SC==5)
		 *modelresult =14;
	if(SC>=6)
		 *modelresult =12;
	return ret;

}


int Basal_area_before_growing_JKK (double BA,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	double res = BA;
	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_JKK (double BA, double BA_old, double D_gM, double N,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Check for fatal errors
	char errorStr[100] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//How much the increase of BA is in D_gM when using average values
	//Estimation of stem number using Kuusela's formula if N not yet exists
	if(N<=0)
		N = 12732.4/D_gM;

	//Check for fatal errors
	if (N<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "N<=0(N=",N);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Theoretical mean diameter before growing
	double D_old = 2*sqrt(((BA_old*10000)/N)/PI);

	//Check for fatal errors
	if (D_old<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_old<=0(D_old=",D_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Theoretical mean diameter after growing
	double D = 2*sqrt(((BA*10000)/N)/PI);
	//Increase of theoretical mean diameter
	double D_change = (D-D_old)/D_old;

	*nres = 1;
	double res = D_gM*(1+D_change);

	//Restriction to annual mean diameter change
	//if((res-D_gM)>0.55)
	//	res=D_gM+0.55;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_diameter_after_thinning_JKK (double D_gM, double DgM_theor,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	double res;

	if(DgM_theor-D_gM<=1.5)
		res=DgM_theor;
	else
		res=D_gM+1.5;

	*modelresult = res * rectFactor;
	return ret;

}


int Site_index_white_birch_JKK (double SC,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if(SC==1)
	     *modelresult =28;
	if(SC==2)
		 *modelresult =26;
	if(SC==3)
		 *modelresult =24;
	if(SC==4)
		 *modelresult =20;
	if(SC==5)
		 *modelresult =16;
	if(SC>=6)
		 *modelresult =12;
	return ret;

}


int Site_index_spruce_JKK (double SC,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if(SC==1)
	     *modelresult =33;
	if(SC==2)
		 *modelresult =27;
	if(SC==3)
		 *modelresult =24;
	if(SC==4)
		 *modelresult =21;
	if(SC==5)
		 *modelresult =18;
	if(SC>=6)
		 *modelresult =12;
	return ret;

}


int Site_index_pine_JKK (double SC,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if(SC==1)
	     *modelresult =30;
	if(SC==2)
		 *modelresult =27;
	if(SC==3)
		 *modelresult =24;
	if(SC==4)
		 *modelresult =21;
	if(SC==5)
		 *modelresult =18;
	if(SC>=6)
		 *modelresult =15;
	return ret;

}


int Regeneration_limits_Tapio (double lower_dgm, double upper_dgm, double lower_age, double upper_age,
							   double REGEN_DGM_LEVEL, double REGEN_AGE_LEVEL,
							   int *nres, struct regen_limits *regen, char *errors,
							   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;

	double difference_dgm = upper_dgm - lower_dgm;
	double regen_dgm = lower_dgm + (difference_dgm * REGEN_DGM_LEVEL);

	double difference_age = upper_age - lower_age;
	double regen_age = lower_age + (difference_age * REGEN_AGE_LEVEL);

	regen[0].regen_dgm = regen_dgm;
	regen[0].regen_age = regen_age;

	return ret;
}

int Regeneration_statutory_limit_JKK (double orig_dgm, double REGENERATION_LIMIT_SUBTRACTION,
							   int *nres, double *modelresult, char *errors,
							   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;

	double regen_dgm = orig_dgm - REGENERATION_LIMIT_SUBTRACTION;

	*modelresult = regen_dgm;
	return ret;
}

int Regeneration_statutory_limit_in_protection_forest_area_JKK (double regen_dgm, double regen_age, double Municipal_id,
							   int *nres, struct Statutory_regen_limits *limits, char *errors,
							   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;

	if (Municipal_id==47||Municipal_id==148||Municipal_id==261||Municipal_id==498||Municipal_id==732||Municipal_id==742||Municipal_id==758||Municipal_id==890)
	{
		regen_dgm = regen_dgm - 1;
		regen_age = regen_age + 10;
	}

	limits[0].reg_dgm = regen_dgm;
	limits[0].reg_age = regen_age;

	return ret;
}


int Regeneration_limits_old_SF_Tapio (double SC, double SP, double D_gM, double Age,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;

    int iSC = (int) SC;
	int iSP = (int) SP;
	switch ( iSC )
	{
	    case 1:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 28 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 28 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 25 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 2:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 28 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 28 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 25 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 3:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 29 || Age >= 80)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 26 || Age >= 90)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 26 || Age >= 70)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 25 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 4:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 27 || Age >= 90)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 27 || Age >= 90)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 26 || Age >= 90)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 26 || Age >= 70)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 25 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		default:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 25 || Age >= 110)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 25 || Age >= 110)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 26 || Age >= 90)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 26 || Age >= 70)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 25 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
	}
	return ret;

}


int Regeneration_limits_old_NF_Tapio (double SC, double SP, double D_gM, double Age,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;
	*nres = 1;

    int iSC = (int) SC;
	int iSP = (int) SP;
	switch ( iSC )
	{
	    case 1:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 23 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 2:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 23 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 3:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 26 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 23 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		case 4:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 24 || Age >= 110)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 24 || Age >= 110)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 23 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
		default:
            switch (iSP)
			{
			case 1:
				if (D_gM >= 23 || Age >= 130)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 8:
				if (D_gM >= 23 || Age >= 130)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 2:
				if (D_gM >= 23 || Age >= 100)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
			case 3:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            default:
				if (D_gM >= 23 || Age >= 60)
					*modelresult = 1;
				else
					*modelresult = 0;
				break;
            }
			break;
	}
	return ret;
}


int Basal_area_removed_from_logging_roads (double N, double road_width, double road_distance,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
	int ret = 1;

	//Check for fatal errors
	char errorStr[100] = "";
	if (road_distance<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "road_distance<=0(road_distance=",road_distance);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double BA_remove_from_road;
	double BA_remove_max=((100/road_distance)*road_width*100)/10000*100;

	//If number of stems is greater than 1500 the maximum BA-removed value is removed from every diameter classes
	//When N is 500 BA-removed value is always 3 %.
	if (N>1500)
		BA_remove_from_road=BA_remove_max;
	else
		BA_remove_from_road=3+(((100/road_distance)*road_width*100)/10000*100-3)/1000*(N-500);

	double res =BA_remove_from_road;

	//To avoid negative values

	if (res<0)
		res=0;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_check_JKK (double D_gM, double H_gM, double PEAT, double SP,
							int *nres, double *modelresult, char *errors,
							int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;

	*nres = 1;
	double res;

	//Check if relation between D_gM and H_gM is realistic:
	if(H_gM<D_gM&&D_gM>1.5&&H_gM<2)  //For very young stratums (H_gM will be about 1,3 m) with quite big mean diameter (>1,5 cm)
		res=D_gM;
	else if(H_gM<D_gM&&D_gM>3&&H_gM<2)  //For very young stratums (H_gM will be about 1,3 m) with quite big mean diameter (>1,5 cm)
		res=0.85*D_gM;
	else if(PEAT==0&&H_gM<0.65*D_gM&&H_gM>=2)	//For stratums with very thick trees on mineral soil
		res=0.65*D_gM;
	else if(PEAT==1&&H_gM<0.55*D_gM&&H_gM>=2)	//For stratums with very thick trees on peatland
		res=0.55*D_gM;
	else if((SP==1||SP==2||SP==8)&&H_gM>1.3*D_gM&&D_gM>1.5)	//For stratums with very thin coniferous trees
		res=1.3*D_gM;
	else if((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&H_gM>1.5*D_gM&&D_gM>1.5)	//For stratums with very thin deciduous trees
		res=1.5*D_gM;
	else
		res=H_gM;

	//Check if mean height is below breast height
	if(res<=1.3001)
		res=1.3002;

	*modelresult = res * rectFactor;
	return ret;

}


int Mean_height_seedling_MELA_JKK (double Age, double t_h,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	double res;


	res = Age*(1.3/t_h);

	*modelresult = res * rectFactor;
	return ret;

}

int Development_class_JKK (double Age, double H_gM, double D_gM, double SP, double Age_difference,
						   double BA, double N, double Two_st, double MAIN_GROUP, double regen_dgm,
						   double regen_age, double OLD_DEVEL_CLASS, double dgm_upper, double Hdom,
						   int *nres, double *modelresult, char *errors,
						   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

	*nres = 1;
	double res;

	//OBS!! Open area site (DEVEL_CLASS=1) is already defined in model chain!
	if (OLD_DEVEL_CLASS<=5||OLD_DEVEL_CLASS==9)
	{
		if (OLD_DEVEL_CLASS<=2||OLD_DEVEL_CLASS==9)
		{
			res = 2;
			if((H_gM>=1.3&&(D_gM<8||(((SP==1||SP==2||SP==8)&&Hdom<7))||((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Hdom<9))))
				res = 3;
			if(D_gM>=8&&D_gM<16&&(((SP==1||SP==2||SP==8)&&Hdom>=7)||((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Hdom>=9)))
				res = 4;
			if(D_gM>=16&&D_gM<regen_dgm&&Age<regen_age)
				res = 5;
			if(D_gM>=regen_dgm||Age>=regen_age)
				res = 6;
		}
		else if (OLD_DEVEL_CLASS==3)
		{
			res = 3;
			if((H_gM>=1.3&&(D_gM<8||(((SP==1||SP==2||SP==8)&&Hdom<7))||((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Hdom<9))))
				res = 3;
			if(D_gM>=8&&D_gM<16&&(((SP==1||SP==2||SP==8)&&Hdom>=7)||((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Hdom>=9)))
				res = 4;
			if(D_gM>=16&&D_gM<regen_dgm&&Age<regen_age)
				res = 5;
			if(D_gM>=regen_dgm||Age>=regen_age)
				res = 6;
		}
		else if (OLD_DEVEL_CLASS==4)
		{
			res = 4;
			if(D_gM>=8&&D_gM<16&&(((SP==1||SP==2||SP==8)&&Hdom>=7)||((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Hdom>=9)))
				res = 4;
			if(D_gM>=16&&D_gM<regen_dgm&&Age<regen_age)
				res = 5;
			if(D_gM>=regen_dgm||Age>=regen_age)
				res = 6;
		}
		else if (OLD_DEVEL_CLASS==5)
		{
			res = 5;
			if(D_gM>=16&&D_gM<regen_dgm&&Age<regen_age)
				res = 5;
			if(D_gM>=regen_dgm||Age>=regen_age)
				res = 6;
		}
		//Define seed and shelter tree stand and fix if problems with basal area
		if(Two_st==0)
		{
			if(SP==1||SP==8)
		    	{
		    		if(MAIN_GROUP==1&&res==6&&BA<10&&N<150)
		    			res = 7;
		    	}
		    	else if(SP==2)
		    	{
		    		if(MAIN_GROUP==1&&res==6&&BA<10&&N<250)
		    			res = 8;
		    	}
		    	else if(SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)
		    	{
		    		if(MAIN_GROUP==1&&res==6&&BA<7&&N<80)
		    			res = 7;
		    	}
	        }
		//Define seed and shelter tree stand with seedling storey and fix if
		//problems with basal area
		else if(Two_st==1)
		{
			if(OLD_DEVEL_CLASS!=9)
			{
			       	if((SP==1||SP==8)&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<10)
			       	{
			   		if(Age_difference>60)
			    			res = 9;
			    	}
			    	if(SP==2&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<10)
			    	{
			    		if(Age_difference>60)
			    			res = 9;
			    	}
			    	if((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<7)
			    	{
			    		if(Age_difference>40)
			    			res = 9;
			    	}
			}
			else
			{
			 	res = 9;
                	}
		}
	}
	else if (OLD_DEVEL_CLASS==6)
	{
		res = 6;
		//Define seed and shelter tree stand and fix if problems with basal area
		if(Two_st==0)
		{
			if(MAIN_GROUP==1&&H_gM<1.3)
				res = 2;
			if(MAIN_GROUP==1&&H_gM>=1.3&&D_gM<8)
				res = 3;
	    		if(SP==1||SP==8)
	    		{
	    			if(MAIN_GROUP==1&&res==6&&BA<10&&N<150)
	    				res = 7;
	    		}
	    		else if(SP==2)
	    		{
	    			if(MAIN_GROUP==1&&res==6&&BA<10&&N<250)
	    				res = 8;
	    		}
	    		else if(SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)
	    		{
	    			if(MAIN_GROUP==1&&res==6&&BA<7&&N<80)
	    				res = 7;
	    		}
        	}
		//Define seed and shelter tree stand with seedling storey and fix if
	    	//problems with basal area
		else if(Two_st==1)
		{
	        	if((SP==1||SP==8)&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<10)
	        	{
	    			if(Age_difference>60)
	    				res = 9;
	    		}
	    		if(SP==2&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<10)
	    		{
	    			if(Age_difference>60)
	    				res = 9;
	    		}
	    		if((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&MAIN_GROUP==1&&(res==6||dgm_upper>=16)&&BA<7)
	    		{
	    			if(Age_difference>40)
	    				res = 9;
	    		}
		}
	}
	else if (OLD_DEVEL_CLASS==7)
	{
		res = 7;
		//Define seed and shelter tree stand and fix if problems with basal area
		if(Two_st==0)
		{
    			if(MAIN_GROUP==1&&H_gM<1.3)
				res = 2;
			if(MAIN_GROUP==1&&H_gM>=1.3&&D_gM<8)
				res = 3;
        	}
		//Define seed and shelter tree stand with seedling storey and fix if
	    	//problems with basal area
		else if(Two_st==1)
		{
	        	if((SP<=2||SP==8)&&Age_difference>60)
	        	{
			    	res = 9;
                	}
          		if((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Age_difference>40)
          		{
			        res = 9;
		        }
		}
	}
	else if (OLD_DEVEL_CLASS==8)
	{
		res = 8;
		//Define seed and shelter tree stand and fix if problems with basal area
		if(Two_st==0)
		{
    			if(MAIN_GROUP==1&&H_gM<1.3)
				res = 2;
			if(MAIN_GROUP==1&&H_gM>=1.3&&D_gM<8)
				res = 3;
        	}
		//Define seed and shelter tree stand with seedling storey and fix if
	    	//problems with basal area
		else if(Two_st==1)
		{
	        	if((SP<=2||SP==8)&&Age_difference>60)
	        	{
			    	res = 9;
                	}
          		if((SP==3||SP==4||SP==5||SP==6||SP==7||SP==9)&&Age_difference>40)
          		{
			        res = 9;
		        }
		}
	}
	//No change if uneven aged forest
	else if (OLD_DEVEL_CLASS==10)
	{
		res = 10;
	}

	*modelresult = res * rectFactor;
	return ret;
}

int Age_class_JKK (double Age,
						   int *nres, double *modelresult, char *errors,
						   int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

	*nres = 1;
	double res;

	//HUOM!! Open area site (stratum is missing) is already defined in model chain!
	//HUOM!! In that case AGE_CLASS=0!

	//Define AGE_CLASS to zero as default value
	res = 0;
	if (Age>0&&Age<=10)
		res = 1;
	if (Age>10&&Age<=20)
		res = 2;
	if (Age>20&&Age<=30)
		res = 3;
	if (Age>30&&Age<=40)
		res = 4;
	if (Age>40&&Age<=50)
		res = 5;
	if (Age>50&&Age<=60)
		res = 6;
	if (Age>60&&Age<=70)
		res = 7;
	if (Age>70&&Age<=80)
		res = 8;
	if (Age>80&&Age<=90)
		res = 9;
	if (Age>90&&Age<=100)
		res = 10;
	if (Age>100&&Age<=110)
		res = 11;
	if (Age>110&&Age<=120)
		res = 12;
	if (Age>120)
		res = 13;

	*modelresult = res;
	return ret;
}

int Volume_increase_m3_stratum_JKK (double V, double V_old, double timeStep,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	//Check for fatal errors
	char errorStr[200] = "";
	if (V_old==0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_old==0(V_old=",V_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	*nres = 1;

	res = V - V_old;
	if (timeStep > 1)
	{
       res = res / timeStep;
    }

	*modelresult = res;
	return ret;

}

int Volume_increase_percent_stratum_JKK (double V, double V_old, double timeStep,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	//Check for fatal errors
	char errorStr[200] = "";
	if (V_old==0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_old==0(V_old=",V_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	*nres = 1;

	double i_Vm3;
	i_Vm3 = V - V_old;

	if (timeStep > 1)
	{
       i_Vm3 = i_Vm3 / timeStep;
    }

	res = i_Vm3 / V_old * 100;

	*modelresult = res;
	return ret;

}

int Volume_increase_m3_stand_JKK (double V, double V_old, double timeStep,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	//Check for fatal errors
	char errorStr[200] = "";
	if (V_old==0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_old==0(V_old=",V_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	*nres = 1;

	res = V - V_old;

	if (timeStep > 1)
	{
       res = res / timeStep;
    }

	*modelresult = res;
	return ret;

}

int Volume_increase_percent_stand_JKK (double V, double V_old, double timeStep,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	//Check for fatal errors
	char errorStr[200] = "";
	if (V_old==0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_old==0(V_old=",V_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	*nres = 1;

	double i_Vm3;
	i_Vm3 = V - V_old;

	if (timeStep > 1)
	{
       i_Vm3 = i_Vm3 / timeStep;
    }

	res = i_Vm3 / V_old * 100;

	*modelresult = res;
	return ret;

}

int V_value_JKK (double stand_value, double value_growth_abs, double value_land, double IR,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	*nres = 1;

	res = value_growth_abs - IR/100*(stand_value+value_land);

	*modelresult = res;
	return ret;

}

int Indicating_percent_JKK (double stand_value, double value_growth_abs, double value_land,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res;

	*nres = 1;

	res = (100*value_growth_abs)/(stand_value+value_land);

	*modelresult = res;
	return ret;

}

int Soil_carbon_Liski (double TS, double SC, double MAIN_GROUP,
			int *nres, struct carbon_soil_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res_cwl,res_fwl,res_nwl,carbon_A_cwl,carbon_W_cwl,carbon_E_cwl,carbon_N_cwl,carbon_H_cwl,
    carbon_A_fwl,carbon_W_fwl,carbon_E_fwl,carbon_N_fwl,carbon_H_fwl,
    carbon_A_nwl,carbon_W_nwl,carbon_E_nwl,carbon_N_nwl,carbon_H_nwl;
    *nres = 1;

    //change kg/m2 to kg/ha at the same time
    if(SC==1 || SC==2)
    {
   	res_cwl = 0.15636937 * 10000 * (2.2 + 0.00266*TS + 4.09);
   	res_fwl = 0.073343612 * 10000 * (2.2 + 0.00266*TS + 4.09);
   	res_nwl = 0.77028704 * 10000 * (2.2 + 0.00266*TS + 4.09);

   	carbon_A_cwl = res_cwl * 0.16546553149955845;
	carbon_W_cwl = res_cwl * 0.020294094064772117;
	carbon_E_cwl = res_cwl * 0.0019512124261242896;
	carbon_N_cwl = res_cwl * 0.61865692181066212;
	carbon_H_cwl = res_cwl * 0.19363224019888303;
	carbon_A_fwl = res_fwl * 0.095156556069372789;
	carbon_W_fwl = res_fwl * 0.01173315440437177;
	carbon_E_fwl = res_fwl * 0.0025828206740232682;
	carbon_N_fwl = res_fwl * 0.47175520178257946;
	carbon_H_fwl = res_fwl * 0.41877226706965276;
	carbon_A_nwl = res_nwl * 0.080841868756340973;
	carbon_W_nwl = res_nwl * 0.010478732972659124;
	carbon_E_nwl = res_nwl * 0.01336751644542299;
	carbon_N_nwl = res_nwl * 0.51211059437594941;
	carbon_H_nwl = res_nwl * 0.38320128744962756;
    }
    else if(SC==3)
    {
	res_cwl = 0.16341063 * 10000 * (1.91 + 0.00266*TS + 2.69);
	res_fwl = 0.073589325 * 10000 * (1.91 + 0.00266*TS + 2.69);
	res_nwl = 0.76300001 * 10000 * (1.91 + 0.00266*TS + 2.69);

	carbon_A_cwl = res_cwl * 0.16142053392883679;
	carbon_W_cwl = res_cwl * 0.019830750546606591;
	carbon_E_cwl = res_cwl * 0.002635398762612469;
	carbon_N_cwl = res_cwl * 0.62642272248187936;
	carbon_H_cwl = res_cwl * 0.18969059428006477;
	carbon_A_fwl = res_fwl * 0.096456545128537921;
	carbon_W_fwl = res_fwl * 0.011893218290192867;
	carbon_E_fwl = res_fwl * 0.0026012300041466786;
	carbon_N_fwl = res_fwl * 0.46557883451506604;
	carbon_H_fwl = res_fwl * 0.42347017206205645;
	carbon_A_nwl = res_nwl * 0.08343169145570016;
	carbon_W_nwl = res_nwl * 0.010868707289457685;
	carbon_E_nwl = res_nwl * 0.014941857373408626;
	carbon_N_nwl = res_nwl * 0.4961423685486262;
	carbon_H_nwl = res_nwl * 0.39461537533280738;
    }
    else if(SC==4)
    {
	res_cwl = 0.12014822 * 10000 * (1.9 + 0.00266*TS + 0.79);
	res_fwl = 0.054312821 * 10000 * (1.9 + 0.00266*TS + 0.79);
	res_nwl = 0.82553899 * 10000 * (1.9 + 0.00266*TS + 0.79);

	carbon_A_cwl = res_cwl * 0.16142052262731479;
	carbon_W_cwl = res_cwl * 0.019830751472620946;
	carbon_E_cwl = res_cwl * 0.0026353988081651227;
	carbon_N_cwl = res_cwl * 0.62642273323452013;
	carbon_H_cwl = res_cwl * 0.18969059385737908;
	carbon_A_fwl = res_fwl * 0.096456539636422478;
	carbon_W_fwl = res_fwl * 0.011893218789475695;
	carbon_E_fwl = res_fwl * 0.0026012300259618203;
	carbon_N_fwl = res_fwl * 0.46557883410162315;
	carbon_H_fwl = res_fwl * 0.42347017744651688;
	carbon_A_nwl = res_nwl * 0.083431693228045237;
	carbon_W_nwl = res_nwl * 0.010868707185952802;
	carbon_E_nwl = res_nwl * 0.014941856838106241;
	carbon_N_nwl = res_nwl * 0.49614235176169097;
	carbon_H_nwl = res_nwl * 0.39461539098620468;
    }
    else
    {
    	if(SC==5 || SC==6)
	{
		res_cwl = 0.10517204496893229 * 10000 * (1.2 + 0.00266*TS + 0.218);
		res_fwl = 0.043853639457526974 * 10000 * (1.2 + 0.00266*TS + 0.218);
		res_nwl = 0.85097431557354075 * 10000 * (1.2 + 0.00266*TS + 0.218);
	}
    	else
    	{
	    	if (MAIN_GROUP == 1)
	    	{
		    	res_cwl = 0.10517204496893229 * 10000 * (1.2 + 0.00266*TS + 0.218);
		    	res_fwl = 0.043853639457526974 * 10000 * (1.2 + 0.00266*TS + 0.218);
		    	res_nwl = 0.85097431557354075 * 10000 * (1.2 + 0.00266*TS + 0.218);
		}
		else
		{
		    	res_cwl = 0.10517204496893229 * 10000 * 1.2;
		    	res_fwl = 0.043853639457526974 * 10000 * 1.2;
		    	res_nwl = 0.85097431557354075 * 10000 * 1.2;
		}
	}
	carbon_A_cwl = res_cwl * 0.15333051538723241;
	carbon_W_cwl = res_cwl * 0.018904063583079656;
	carbon_E_cwl = res_cwl * 0.0040037713393488942;
	carbon_N_cwl = res_cwl * 0.64195437105335784;
	carbon_H_cwl = res_cwl * 0.18180727863698115;
	carbon_A_fwl = res_fwl * 0.099056523697516705;
	carbon_W_fwl = res_fwl * 0.012213346094633003;
	carbon_E_fwl = res_fwl * 0.002638049281648448;
	carbon_N_fwl = res_fwl * 0.45322611727375156;
	carbon_H_fwl = res_fwl * 0.43286596365245028;
	carbon_A_nwl = res_nwl * 0.088611327762966346;
	carbon_W_nwl = res_nwl * 0.011648657849837155;
	carbon_E_nwl = res_nwl * 0.018090538065019215;
	carbon_N_nwl = res_nwl * 0.46420588519770678;
	carbon_H_nwl = res_nwl * 0.41744359112447055;
    }

    components[0].Carbon_A_cwl_soil = carbon_A_cwl;
    components[0].Carbon_W_cwl_soil = carbon_W_cwl;
    components[0].Carbon_E_cwl_soil = carbon_E_cwl;
    components[0].Carbon_N_cwl_soil = carbon_N_cwl;
    components[0].Carbon_H_cwl_soil = carbon_H_cwl;
    components[0].Carbon_A_fwl_soil = carbon_A_fwl;
    components[0].Carbon_W_fwl_soil = carbon_W_fwl;
    components[0].Carbon_E_fwl_soil = carbon_E_fwl;
    components[0].Carbon_N_fwl_soil = carbon_N_fwl;
    components[0].Carbon_H_fwl_soil = carbon_H_fwl;
    components[0].Carbon_A_nwl_soil = carbon_A_nwl;
    components[0].Carbon_W_nwl_soil = carbon_W_nwl;
    components[0].Carbon_E_nwl_soil = carbon_E_nwl;
    components[0].Carbon_N_nwl_soil = carbon_N_nwl;
    components[0].Carbon_H_nwl_soil = carbon_H_nwl;

    return ret;

}

int Soil_carbon_JKK (double County_id, double SC, double Age,
			int *nres, struct carbon_soil_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double res_cwl,res_fwl,res_nwl,carbon_A_cwl,carbon_W_cwl,carbon_E_cwl,carbon_N_cwl,carbon_H_cwl,
    carbon_A_fwl,carbon_W_fwl,carbon_E_fwl,carbon_N_fwl,carbon_H_fwl,
    carbon_A_nwl,carbon_W_nwl,carbon_E_nwl,carbon_N_nwl,carbon_H_nwl;
    *nres = 1;

    if(SC==1 || SC==2)
    {
    	if(County_id==5) //Kanta-Häme
    	{
    		if(Age>75)
			Age=75;
   		res_cwl = 21159; // vakio, jotta uudistulksen piikki ei kohoa yli
		res_fwl = 18476; // vakio, jotta uudistulksen piikki ei kohoa yli
		res_nwl = 0.0091*pow(Age,4) - 1.8049*pow(Age,3) + 123.73*pow(Age,2) - 2959.3*Age + 100500;
   	}
   	else if(County_id==6) //Pirkanmaa
    	{
    		if(Age>79)
			Age=79;
   		res_cwl = 20705;
   		res_fwl = 18053;
   		res_nwl = 0.0067*pow(Age,4) - 1.4797*pow(Age,3) + 110.1*pow(Age,2) - 2773.8*Age + 100000;
   	}
   	else if(County_id == 13) //Keski-Suomi
    	{
    		if(Age>79)
			Age=79;
    		res_cwl = 17517;
   		res_fwl = 15738;
   		res_nwl = 0.0058*pow(Age,4) - 1.2619*pow(Age,3) + 97.254*pow(Age,2) - 2664.4*Age + 97500;
    	}

   	carbon_A_cwl = res_cwl * 0.16546553149955845;
	carbon_W_cwl = res_cwl * 0.020294094064772117;
	carbon_E_cwl = res_cwl * 0.0019512124261242896;
	carbon_N_cwl = res_cwl * 0.61865692181066212;
	carbon_H_cwl = res_cwl * 0.19363224019888303;
	carbon_A_fwl = res_fwl * 0.095156556069372789;
	carbon_W_fwl = res_fwl * 0.01173315440437177;
	carbon_E_fwl = res_fwl * 0.0025828206740232682;
	carbon_N_fwl = res_fwl * 0.47175520178257946;
	carbon_H_fwl = res_fwl * 0.41877226706965276;
	carbon_A_nwl = res_nwl * 0.080841868756340973;
	carbon_W_nwl = res_nwl * 0.010478732972659124;
	carbon_E_nwl = res_nwl * 0.01336751644542299;
	carbon_N_nwl = res_nwl * 0.51211059437594941;
	carbon_H_nwl = res_nwl * 0.38320128744962756;
    }
    else if(SC==3)
    {
    	if(County_id==5) //Kanta-Häme
    	{
    		if(Age>79)
			Age=79;
   		res_cwl = 16334;
		res_fwl = 13413;
		res_nwl = 0.0073*pow(Age,4) - 1.4422*pow(Age,3) + 100.18*pow(Age,2) - 2467*Age+ 83000;
   	}
   	else if(County_id==6) //Pirkanmaa
    	{
    		if(Age>79)
			Age=79;
		res_cwl = 16201;
		res_fwl = 13351;
		res_nwl = 0.006*pow(Age,4) - 1.2582*pow(Age,3) + 92.708*pow(Age,2) - 2364.2*Age + 83000;
	}
	else if(County_id == 13) //Keski-Suomi
    	{
    		if(Age>79)
			Age=79;
    		res_cwl = 14016;
   		res_fwl = 12159;
   		res_nwl = 0.0023*pow(Age,4) - 0.6898*pow(Age,3) + 67.43*pow(Age,2) - 2184.4*Age + 86000;
	}

	carbon_A_cwl = res_cwl * 0.16142053392883679;
	carbon_W_cwl = res_cwl * 0.019830750546606591;
	carbon_E_cwl = res_cwl * 0.002635398762612469;
	carbon_N_cwl = res_cwl * 0.62642272248187936;
	carbon_H_cwl = res_cwl * 0.18969059428006477;
	carbon_A_fwl = res_fwl * 0.096456545128537921;
	carbon_W_fwl = res_fwl * 0.011893218290192867;
	carbon_E_fwl = res_fwl * 0.0026012300041466786;
	carbon_N_fwl = res_fwl * 0.46557883451506604;
	carbon_H_fwl = res_fwl * 0.42347017206205645;
	carbon_A_nwl = res_nwl * 0.08343169145570016;
	carbon_W_nwl = res_nwl * 0.010868707289457685;
	carbon_E_nwl = res_nwl * 0.014941857373408626;
	carbon_N_nwl = res_nwl * 0.4961423685486262;
	carbon_H_nwl = res_nwl * 0.39461537533280738;
    }
    else if(SC==4)
    {
    	if(County_id==5) //Kanta-Häme
    	{
    		if(Age>80)
			Age=80;
   		res_cwl = 9399;
		res_fwl = 7801;
		res_nwl = 0.0028*pow(Age,4) - 0.6438*pow(Age,3) + 49.868*pow(Age,2) - 1311.2*Age+ 54500;
   	}
   	else if(County_id==6) //Pirkanmaa
    	{
    		if(Age>80)
			Age=80;
		res_cwl = 9527;
		res_fwl = 8097;
		res_nwl = 0.0027*pow(Age,4) - 0.6408*pow(Age,3) + 50.841*pow(Age,2) - 1397.5*Age + 57000;
	}
	else if(County_id == 13) //Keski-Suomi
    	{
    		if(Age>80)
			Age=80;
    		res_cwl = 8184;
   		res_fwl = 6778;
   		res_nwl = 0.0016*pow(Age,4) - 0.4109*pow(Age,3) + 36.774*pow(Age,2) - 1105.8*Age + 52000;
    	}

	carbon_A_cwl = res_cwl * 0.16142052262731479;
	carbon_W_cwl = res_cwl * 0.019830751472620946;
	carbon_E_cwl = res_cwl * 0.0026353988081651227;
	carbon_N_cwl = res_cwl * 0.62642273323452013;
	carbon_H_cwl = res_cwl * 0.18969059385737908;
	carbon_A_fwl = res_fwl * 0.096456539636422478;
	carbon_W_fwl = res_fwl * 0.011893218789475695;
	carbon_E_fwl = res_fwl * 0.0026012300259618203;
	carbon_N_fwl = res_fwl * 0.46557883410162315;
	carbon_H_fwl = res_fwl * 0.42347017744651688;
	carbon_A_nwl = res_nwl * 0.083431693228045237;
	carbon_W_nwl = res_nwl * 0.010868707185952802;
	carbon_E_nwl = res_nwl * 0.014941856838106241;
	carbon_N_nwl = res_nwl * 0.49614235176169097;
	carbon_H_nwl = res_nwl * 0.39461539098620468;
    }
    else
    {
    	if(County_id==5) //Kanta-Häme
    	{
    		if(Age>89)
			Age=89;
   		res_cwl = 7023;
		res_fwl = 5351;
		res_nwl = 0.0014*pow(Age,4) - 0.3796*pow(Age,3) + 35.091*pow(Age,2) - 1105.6*Age+ 49000;
   	}
   	else if(County_id==6) //Pirkanmaa
    	{
    		if(Age>90)
			Age=90;
    		res_cwl = 6804;
		res_fwl = 5204;
		res_nwl = 0.0011*pow(Age,4) - 0.3072*pow(Age,3) + 30.143*pow(Age,2) - 984.98*Age + 48000;
	}
	else if(County_id == 13) //Keski-Suomi
    	{
    		if(Age>100)
			Age=100;
    		res_cwl = 5525;
   		res_fwl = 4336;
   		res_nwl = 0.0014*pow(Age,4) - 0.3738*pow(Age,3) + 34.678*pow(Age,2) - 1138.6*Age + 47000;
	}

	carbon_A_cwl = res_cwl * 0.15333051538723241;
	carbon_W_cwl = res_cwl * 0.018904063583079656;
	carbon_E_cwl = res_cwl * 0.0040037713393488942;
	carbon_N_cwl = res_cwl * 0.64195437105335784;
	carbon_H_cwl = res_cwl * 0.18180727863698115;
	carbon_A_fwl = res_fwl * 0.099056523697516705;
	carbon_W_fwl = res_fwl * 0.012213346094633003;
	carbon_E_fwl = res_fwl * 0.002638049281648448;
	carbon_N_fwl = res_fwl * 0.45322611727375156;
	carbon_H_fwl = res_fwl * 0.43286596365245028;
	carbon_A_nwl = res_nwl * 0.088611327762966346;
	carbon_W_nwl = res_nwl * 0.011648657849837155;
	carbon_E_nwl = res_nwl * 0.018090538065019215;
	carbon_N_nwl = res_nwl * 0.46420588519770678;
	carbon_H_nwl = res_nwl * 0.41744359112447055;
    }

    components[0].Carbon_A_cwl_soil = carbon_A_cwl;
    components[0].Carbon_W_cwl_soil = carbon_W_cwl;
    components[0].Carbon_E_cwl_soil = carbon_E_cwl;
    components[0].Carbon_N_cwl_soil = carbon_N_cwl;
    components[0].Carbon_H_cwl_soil = carbon_H_cwl;
    components[0].Carbon_A_fwl_soil = carbon_A_fwl;
    components[0].Carbon_W_fwl_soil = carbon_W_fwl;
    components[0].Carbon_E_fwl_soil = carbon_E_fwl;
    components[0].Carbon_N_fwl_soil = carbon_N_fwl;
    components[0].Carbon_H_fwl_soil = carbon_H_fwl;
    components[0].Carbon_A_nwl_soil = carbon_A_nwl;
    components[0].Carbon_W_nwl_soil = carbon_W_nwl;
    components[0].Carbon_E_nwl_soil = carbon_E_nwl;
    components[0].Carbon_N_nwl_soil = carbon_N_nwl;
    components[0].Carbon_H_nwl_soil = carbon_H_nwl;

    return ret;

}

int Carbon_cwl_Liski (double BM_stems, double BM_stumps, double D_gM, double SP, double natural_removal, double V,
			double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double time_step,
			int *nres, struct carbon_cwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_stem,c_stump,carbon_A,carbon_W,carbon_E,carbon_N,prop_of_dead_trees,prop_of_dead_trees_not_collected,dimension;

    //Calculate proportion of dead trees from the growing stock
    prop_of_dead_trees = natural_removal/V;
    prop_of_dead_trees_not_collected = (natural_removal * (1-BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL))/V;

    //Carbon (kg/ha) is assumed to be half of biomass
    c_stem = carbon_in_biomass * time_step * BM_stems * prop_of_dead_trees_not_collected;
    c_stump = carbon_in_biomass * time_step * BM_stumps * prop_of_dead_trees;

    *nres = 1;

    if (SP == 1 || SP == 8)
    {
    	carbon_A = c_stump * 0.69 + c_stem * 0.69;
    	carbon_W = c_stump * 0.015 + c_stem * 0.015;
    	carbon_E = c_stump * 0.015 + c_stem * 0.015;
    	carbon_N = c_stump * 0.28 + c_stem * 0.28;
    }
    else if (SP == 2)
    {
    	carbon_A = c_stump * 0.69 + c_stem * 0.69;
    	carbon_W = c_stump * 0.005 + c_stem * 0.005;
    	carbon_E = c_stump * 0.005 + c_stem * 0.005;
    	carbon_N = c_stump * 0.3 + c_stem * 0.3;
    }
    else
    {
    	carbon_A = c_stump * 0.77 + c_stem * 0.77;
    	carbon_W = c_stump * 0.005 + c_stem * 0.005;
    	carbon_E = c_stump * 0.005 + c_stem * 0.005;
    	carbon_N = c_stump * 0.22 + c_stem * 0.22;
    }
    //Carbon mean dimension predicted using stratum's mean diameter
    //dimension = D_gM;
    dimension = 10.;

    components[0].Carbon_A_cwl = carbon_A;
    components[0].Carbon_W_cwl = carbon_W;
    components[0].Carbon_E_cwl = carbon_E;
    components[0].Carbon_N_cwl = carbon_N;
    components[0].Carbon_dimension_cwl = dimension;

    return ret;

}

int Carbon_fwl_Liski (double D_gM, double BM_branches, double BM_roots, double SP, double natural_removal, double V,
			double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double time_step,
			int *nres, struct carbon_fwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_branches_roots,carbon_A,carbon_W,carbon_E,carbon_N,branch_litter_prop,prop_of_dead_trees,prop_of_dead_trees_not_collected,dimension;
    double BM_branches_roots = BM_branches + BM_roots;

    //Calculate proportion of dead trees from the growing stock
    prop_of_dead_trees = natural_removal/V;
    prop_of_dead_trees_not_collected = (natural_removal * (1-BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL))/V;

    //Carbon (kg/ha) is assumed to be half of biomass
    if(SP==1||SP==8)
    {
    	branch_litter_prop = 0.0574 * exp(-0.00482*pow(BM_branches,2)) + 0.00648;
    	c_branches_roots = carbon_in_biomass * (time_step * BM_branches_roots * branch_litter_prop + time_step * BM_roots * prop_of_dead_trees + time_step * BM_branches * prop_of_dead_trees_not_collected);
    }
    else if(SP==2)
    {
    	c_branches_roots = carbon_in_biomass * (time_step * BM_branches_roots * 0.0125 + time_step * BM_roots * prop_of_dead_trees + time_step * BM_branches * prop_of_dead_trees_not_collected);
    }
    else
    {
    //Deciduos tree species
    	c_branches_roots = carbon_in_biomass * (time_step * BM_branches_roots * 0.0135 + time_step * BM_roots * prop_of_dead_trees + time_step * BM_branches * prop_of_dead_trees_not_collected);
    }

    *nres = 1;

    if (SP == 1 || SP == 8)
    {
    	carbon_A = c_branches_roots * 0.66;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.31;
    }
    else if (SP == 2)
    {
    	carbon_A = c_branches_roots * 0.61;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.36;
    }
    else
    {
    	carbon_A = c_branches_roots * 0.65;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.32;
    }
    //Carbon mean dimension predicted using stratum's mean diameter
    //dimension = 0.4 + 0.1 * D_gM;
    dimension = 2.;

    components[0].Carbon_A_fwl = carbon_A;
    components[0].Carbon_W_fwl = carbon_W;
    components[0].Carbon_E_fwl = carbon_E;
    components[0].Carbon_N_fwl = carbon_N;
    components[0].Carbon_dimension_fwl = dimension;

    return ret;

}

int Carbon_nwl_Liski (double MANAGEMENT_REGION, double MAIN_SP, double D_gM, double BM_foliage,
			double BM_stump_bark, double BM_stem_bark, double BM_fine_roots, double SP, double natural_removal, double V,
			double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL, double carbon_in_biomass, double PEAT, double BMASS_COLLECT_PROP_grass, double time_step,
			int *nres, struct carbon_nwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_foliage,c_stump_bark,c_stem_bark,c_fine_roots,carbon_A,carbon_W,carbon_E,carbon_N,prop_of_dead_trees,prop_of_dead_trees_not_collected;

     //Calculate proportion of dead trees from the growing stock
    prop_of_dead_trees = natural_removal/V;
    prop_of_dead_trees_not_collected = (natural_removal * (1-BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL))/V;

    //Carbon (kg/ha) is assumed to be half of biomass
    if(SP==1||SP==8)
    {
    	if(PEAT == 0)
    	{
    		if(MANAGEMENT_REGION==1 || MANAGEMENT_REGION==2)
		//Southern Finland or Middle Finland
    			c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.22 + time_step * BM_foliage * prop_of_dead_trees_not_collected);
    		else
    		//Northern Finland
    			c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.1 + time_step * BM_foliage * prop_of_dead_trees_not_collected);
    	}
    	else
    	// peatland
    	{
    		c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.33 + time_step * BM_foliage * prop_of_dead_trees_not_collected);
    	}
    	c_stump_bark = carbon_in_biomass * time_step * BM_stump_bark * 0.003; //stump biomass includes proportion of natural_removal
    	c_stem_bark = carbon_in_biomass * time_step * BM_stem_bark * 0.0052; //stem biomass includes proportion of natural_removal
    	c_fine_roots = carbon_in_biomass * (time_step *BM_fine_roots * 0.868 + time_step * BM_fine_roots * prop_of_dead_trees);
    }
    else if(SP==2)
    {
    	if(MANAGEMENT_REGION==1 || MANAGEMENT_REGION==2)
	//Southern Finland or Middle Finland
		c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.1 + time_step * BM_foliage * prop_of_dead_trees_not_collected);
	else
    	//Northern Finland
    		c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.05 + time_step * BM_foliage * prop_of_dead_trees);
    	c_stump_bark = 0; //stump biomass includes proportion of natural_removal
    	c_stem_bark = carbon_in_biomass * time_step * BM_stem_bark * 0.0027; //stem biomass includes proportion of natural_removal
    	c_fine_roots = carbon_in_biomass * (time_step * BM_fine_roots * 0.811 + time_step * BM_fine_roots * prop_of_dead_trees);
    }
    else
    {
    //Deciduos tree species
    	c_foliage = carbon_in_biomass * (time_step * BM_foliage * 0.78 + time_step * BM_foliage * prop_of_dead_trees_not_collected);
    	c_stump_bark = carbon_in_biomass * time_step * BM_stump_bark * 0.0001; //stump biomass includes proportion of natural_removal
    	c_stem_bark = carbon_in_biomass * time_step * BM_stem_bark * 0.0029; //stem biomass includes proportion of natural_removal
    	c_fine_roots = carbon_in_biomass * (time_step * BM_fine_roots + time_step * BM_fine_roots * prop_of_dead_trees);
    }

    //Biomass of ground vegetation kg/ha
    double BM_mosses, BM_lichens, BM_dwarf_shrubs, BM_herbs_grasses;
    double c_mosses, c_lichens, c_dwarf_shrubs, c_herbs_grasses;

    if(PEAT == 0)
    {
    	if (MAIN_SP == 1 || MAIN_SP == 8)
    	{
    		BM_mosses = pow(27.329+0.138*D_gM-0.0005*pow(D_gM,2),2)-0.5+0.14;
    		BM_lichens = pow(7.975-0.0002*pow(D_gM,2),2)-0.5+0.04;
    		BM_dwarf_shrubs = pow(16.68+0.129*D_gM-0.0004*pow(D_gM,2),2)-0.5+0.27;
    		BM_herbs_grasses = pow(11.725-0.098*D_gM+0.0002*pow(D_gM,2),2)-0.5-0.41;
    		c_mosses = carbon_in_biomass * time_step * BM_mosses * 0.33 * (1-BMASS_COLLECT_PROP_grass);
    		c_lichens = carbon_in_biomass * time_step * BM_lichens * 0.1 * (1-BMASS_COLLECT_PROP_grass);
    		c_dwarf_shrubs = carbon_in_biomass * time_step * BM_dwarf_shrubs * 0.25 * (1-BMASS_COLLECT_PROP_grass);
    		c_herbs_grasses = carbon_in_biomass * time_step * BM_herbs_grasses * (1-BMASS_COLLECT_PROP_grass);
    	}
    	else //spruce and deciduous tree species
    	{
    		BM_mosses = pow(19.282+0.164*D_gM-0.000001*pow(D_gM,3),2)-0.5+0.35;
    		BM_dwarf_shrubs = pow(10.375-0.033*D_gM+0.001*pow(D_gM,2)-0.000004*pow(D_gM,3),2)-0.5+0.42;
    		BM_herbs_grasses = pow(15.058-0.113*D_gM+0.0003*pow(D_gM,2),2)-0.5-0.31;
    		c_mosses = carbon_in_biomass * time_step * BM_mosses * 0.33 * (1-BMASS_COLLECT_PROP_grass);
    		c_dwarf_shrubs = carbon_in_biomass * time_step * BM_dwarf_shrubs * 0.25 * (1-BMASS_COLLECT_PROP_grass);
    		c_herbs_grasses = carbon_in_biomass * time_step * BM_herbs_grasses * (1-BMASS_COLLECT_PROP_grass);
    	}
    }
    else
    //peatland
    {
    	c_mosses = time_step * 1012 * (1-BMASS_COLLECT_PROP_grass);
    	c_dwarf_shrubs = time_step * (50 + 568) * (1-BMASS_COLLECT_PROP_grass);
    	c_herbs_grasses = time_step * (131 + 537) * (1-BMASS_COLLECT_PROP_grass);
    }

    *nres = 1;

    if (SP == 1 || SP == 8)
    {
    	carbon_A = c_foliage * 0.51 + c_stump_bark * 0.69 + c_stem_bark * 0.69 + c_fine_roots * 0.51 + c_mosses * 0.25 + c_lichens * 0.25 + c_dwarf_shrubs * 0.25 + c_herbs_grasses * 0.25;
    	carbon_W = c_foliage * 0.135 + c_stump_bark * 0.015 + c_stem_bark * 0.015 + c_fine_roots * 0.135 + c_mosses * 0.25 + c_lichens * 0.25 + c_dwarf_shrubs * 0.25 + c_herbs_grasses * 0.25;
    	carbon_E = c_foliage * 0.135 + c_stump_bark * 0.015 + c_stem_bark * 0.015 + c_fine_roots * 0.135 + c_mosses * 0.25 + c_lichens * 0.25 + c_dwarf_shrubs * 0.25 + c_herbs_grasses * 0.25;
    	carbon_N = c_foliage * 0.22 + c_stump_bark * 0.28 + c_stem_bark * 0.28 + c_fine_roots * 0.22 + c_mosses * 0.25 + c_lichens * 0.25 + c_dwarf_shrubs * 0.25 + c_herbs_grasses * 0.25;
    }
    else if (SP == 2)
    {
    	carbon_A = c_foliage * 0.54 + c_stump_bark * 0.69 + c_stem_bark * 0.69 + c_fine_roots * 0.54 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_W = c_foliage * 0.03 + c_stump_bark * 0.005 + c_stem_bark * 0.005 + c_fine_roots * 0.03 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_E = c_foliage * 0.03 + c_stump_bark * 0.005 + c_stem_bark * 0.005 + c_fine_roots * 0.03 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_N = c_foliage * 0.4 + c_stump_bark * 0.3 + c_stem_bark * 0.3 + c_fine_roots * 0.4 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    }
    else
    {
    	carbon_A = c_foliage * 0.36 + c_stump_bark * 0.77 + c_stem_bark * 0.77 + c_fine_roots * 0.36 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_W = c_foliage * 0.19 + c_stump_bark * 0.005 + c_stem_bark * 0.005 + c_fine_roots * 0.19 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_E = c_foliage * 0.19 + c_stump_bark * 0.005 + c_stem_bark * 0.005 + c_fine_roots * 0.19 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    	carbon_N = c_foliage * 0.26 + c_stump_bark * 0.22 + c_stem_bark * 0.22 + c_fine_roots * 0.26 + c_mosses * 0.33 + c_dwarf_shrubs * 0.33 + c_herbs_grasses * 0.33;
    }

    components[0].Carbon_A_nwl = carbon_A;
    components[0].Carbon_W_nwl = carbon_W;
    components[0].Carbon_E_nwl = carbon_E;
    components[0].Carbon_N_nwl = carbon_N;

    return ret;

}

int Carbon_nwl_agri_Liski (double Municipal_id,
			int *nres, struct carbon_nwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_annual_crop, c_perennial_crop, c_manure, total_percent;

    double coef_ww, coef_sw, coef_r, coef_b, coef_o, coef_ra, coef_h, AREA_cropland;
    double N_dairycattle, N_cattle, N_calf, N_sheep, N_horse, N_swine, N_poultry;

    if (Municipal_id == 49) //espoo
    {
	coef_ww = 0.0169;
	coef_sw = 0.2743;
	coef_r = 0.0295;
	coef_b = 0.0435+0.0717;
	coef_o = 0.1026+0.027;
	coef_ra = 0.0374+0.0555;
	coef_h = 0.1184+0.2038;
	total_percent = 0.9562;
	//kesanto? 0.2038 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 1923;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 1447;
	N_swine = 0;
	N_poultry = 50;
    }
    else if (Municipal_id == 91) //helsinki
    {
	coef_ww = 0.0;
	coef_sw = 0.267;
	coef_r = 0.0;
	coef_b = 0.0;
	coef_o = 0.0616+0.1603;
	coef_ra = 0.0;
	coef_h = 0.2235+0.1896;
	total_percent = 0.9021;
	//kesanto? 0.1896 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 936;
	N_dairycattle = 79;
	N_cattle = 57;
	N_calf = 50;
	N_sheep = 0;
	N_horse = 1980;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 92) //vantaa
    {
	coef_ww = 0.0291;
	coef_sw = 0.3548;
	coef_r = 0.0391;
	coef_b = 0.0294+0.12;
	coef_o = 0.0688+0.0013;
	coef_ra = 0.0071+0.0247;
	coef_h = 0.1076+0.2023;
	total_percent = 0.9841;
	//kesanto? 0.2023 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 3499;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 833;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 398) //lahti
    {
	coef_ww = 0.0793;
	coef_sw = 0.0994;
	coef_r = 0.0655;
	coef_b = 0.2149;
	coef_o = 0.0971+0.0029;
	coef_ra = 0.0;
	coef_h = 0.1193+0.2016;
	total_percent = 0.9070;
	//kesanto? 0.2016 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 1266;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 696;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 853) //turku
    {
	coef_ww = 0.0139;
	coef_sw = 0.2629;
	coef_r = 0.0126;
	coef_b = 0.0401+0.1228;
	coef_o = 0.0976+0.0195+0.002;
	coef_ra = 0.0378+0.0421;
	coef_h = 0.0991+0.2121;
	total_percent = 0.9624;
	//kesanto? 0.2121 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 4286;
	N_dairycattle = 159;
	N_cattle = 0;
	N_calf = 76;
	N_sheep = 82;
	N_horse = 930;
	N_swine = 0;
	N_poultry = 26952;
    }

    //Average annual dry matter manure excretion per head of species IPPC 2000
    double M_exc_dairycattle, M_exc_cattle, M_exc_calf, M_exc_sheep, M_exc_horse, M_exc_swine, M_exc_poultry;
    M_exc_dairycattle = 5.1*365;
    M_exc_cattle = 2.6*365;
    M_exc_calf = 1.3*365; //half of cattle
    M_exc_sheep = 0.4*365;
    M_exc_horse = 2.13*365;
    M_exc_swine = 0.3*365;
    M_exc_poultry = 0.01*365;

    //Carbon is assumed to be 45 % of yearly manure biomass
    //Carbon of manure / hectare on average
    c_manure = ((N_dairycattle*M_exc_dairycattle+N_cattle*M_exc_cattle+N_calf*M_exc_calf+N_sheep*M_exc_sheep+N_horse*M_exc_horse+N_swine*M_exc_swine+N_poultry*M_exc_poultry)*0.45)/AREA_cropland;

    double cyield_ww, cyield_sw, cyield_r, cyield_b, cyield_o, cyield_ra, cyield_h;
    //yield Kasvener table Muu_jako
    // cyield_... = yield*dd*cc
    cyield_ww = 3688*0.86*0.45;
    cyield_sw = 3569*0.86*0.45;
    cyield_r = 2426*0.86*0.45;
    cyield_b = 3551*0.86*0.45;
    cyield_o = 3532*0.86*0.45;
    cyield_ra = 2011*0.92*0.45;
    cyield_h = 3252*0.86*0.45; //heinn tuotosta ei lydy mistn - oma arvio

    //Carbon of above ground / hectare on average
    // ..._ag = cyield_...*((1-hi)/hi)
    double c_annual_crop_ag = (1/total_percent)*(cyield_ww*((1-0.42)/0.42)*coef_ww+cyield_sw*((1-0.42)/0.42)*coef_sw+cyield_r*((1-0.4)/0.4)*coef_r+cyield_b*((1-0.53)/0.53)*coef_b+cyield_o*((1-0.46)/0.46)*coef_o+cyield_ra*((1-0.35)/0.35)*coef_ra);
    double c_perennial_crop_ag = (1/total_percent)*(cyield_h*((1-0.84)/0.84)*coef_h);
    //Carbon of roots
    double c_annual_crop_roots = (1/total_percent)*(cyield_ww*(1/(0.42*5.6))*coef_ww+cyield_sw*(1/(0.42*5.6))*coef_sw+cyield_r*(1/(0.4*5.6))*coef_r+cyield_b*(1/(0.53*5.6))*coef_b+cyield_o*(1/(0.46*5.6))*coef_o+cyield_ra*(1/(0.35*5.1))*coef_ra);
    double c_perennial_crop_roots = (1/total_percent)*((4036/3.5)*0.45*coef_h);
    //Carbon of rhizo deposition
    double c_annual_crop_rhizo = (1/total_percent)*(0.41*c_annual_crop_roots);
    double c_perennial_crop_rhizo = (1/total_percent)*(0.41*4036*0.45*coef_h);

    c_annual_crop = c_annual_crop_ag + c_annual_crop_roots + c_annual_crop_rhizo;
    c_perennial_crop = c_perennial_crop_ag + c_perennial_crop_roots + c_perennial_crop_rhizo;

    *nres = 1;

    double carbon_A = c_annual_crop * 0.74 + c_perennial_crop * 0.451 + c_manure * 0.645;
    double carbon_W = c_annual_crop * 0.094 + c_perennial_crop * 0.358 + c_manure * 0.123;
    double carbon_E = c_annual_crop * 0.021 + c_perennial_crop * 0.034 + c_manure * 0.072;
    double carbon_N = c_annual_crop * 0.144 + c_perennial_crop * 0.157 + c_manure * 0.161;

    components[0].Carbon_A_nwl = carbon_A;
    components[0].Carbon_W_nwl = carbon_W;
    components[0].Carbon_E_nwl = carbon_E;
    components[0].Carbon_N_nwl = carbon_N;

    return ret;

}

int Carbon_nwl_agri_plants_Liski (double Municipal_id, double cult_plant,
			int *nres, struct carbon_nwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_annual_crop, c_perennial_crop, c_manure;

    double coef_ww, coef_sw, coef_r, coef_b, coef_o, coef_ra, coef_h, AREA_cropland;
    double N_dairycattle, N_cattle, N_calf, N_sheep, N_horse, N_swine, N_poultry;

    if (Municipal_id == 49) //espoo
    {
	coef_ww = 0.0169;
	coef_sw = 0.2743;
	coef_r = 0.0295;
	coef_b = 0.0435+0.0717;
	coef_o = 0.1026+0.027;
	coef_ra = 0.0374+0.0555;
	coef_h = 0.1184+0.1924;
	//kesanto? 0.1924 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 1923;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 1447;
	N_swine = 0;
	N_poultry = 50;
    }
    else if (Municipal_id == 91) //helsinki
    {
	coef_ww = 0.0;
	coef_sw = 0.267;
	coef_r = 0.0;
	coef_b = 0.0;
	coef_o = 0.0616+0.1603;
	coef_ra = 0.0;
	coef_h = 0.2235+0.1896;
	//kesanto? 0.1896 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 936;
	N_dairycattle = 79;
	N_cattle = 57;
	N_calf = 50;
	N_sheep = 0;
	N_horse = 1980;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 92) //vantaa
    {
	coef_ww = 0.0291;
	coef_sw = 0.3548;
	coef_r = 0.0391;
	coef_b = 0.0294+0.12;
	coef_o = 0.0688+0.0013;
	coef_ra = 0.0071+0.0247;
	coef_h = 0.1076+0.2009;
	//kesanto? 0.2009 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 3499;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 833;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 398) //lahti
    {
	coef_ww = 0.0793;
	coef_sw = 0.0994;
	coef_r = 0.0655;
	coef_b = 0.2149;
	coef_o = 0.0971+0.0029;
	coef_ra = 0.0;
	coef_h = 0.1193+0.2016;
	//kesanto? 0.2016 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 1266;
	N_dairycattle = 0;
	N_cattle = 0;
	N_calf = 0;
	N_sheep = 0;
	N_horse = 696;
	N_swine = 0;
	N_poultry = 0;
    }
    else if (Municipal_id == 853) //turku
    {
	coef_ww = 0.0139;
	coef_sw = 0.2629;
	coef_r = 0.0126;
	coef_b = 0.0401+0.1228;
	coef_o = 0.0976+0.0195+0.002;
	coef_ra = 0.0378+0.0421;
	coef_h = 0.0991+0.1835;
	//kesanto? 0.1835 nyt yhdistetty heinn
	//coef_s =
	AREA_cropland = 4286;
	N_dairycattle = 159;
	N_cattle = 0;
	N_calf = 76;
	N_sheep = 82;
	N_horse = 930;
	N_swine = 0;
	N_poultry = 26952;
    }

    //Average annual dry matter manure excretion per head of species IPPC 2000
    double M_exc_dairycattle, M_exc_cattle, M_exc_calf, M_exc_sheep, M_exc_horse, M_exc_swine, M_exc_poultry;
    M_exc_dairycattle = 5.1*365;
    M_exc_cattle = 2.6*365;
    M_exc_calf = 1.3*365; //half of cattle
    M_exc_sheep = 0.4*365;
    M_exc_horse = 2.13*365;
    M_exc_swine = 0.3*365;
    M_exc_poultry = 0.01*365;

    //Carbon is assumed to be 45 % of yearly manure biomass
    //Carbon of manure / hectare on average
    c_manure = ((N_dairycattle*M_exc_dairycattle+N_cattle*M_exc_cattle+N_calf*M_exc_calf+N_sheep*M_exc_sheep+N_horse*M_exc_horse+N_swine*M_exc_swine+N_poultry*M_exc_poultry)*0.45)/AREA_cropland;

    double cyield_ww, cyield_sw, cyield_r, cyield_b, cyield_o, cyield_ra, cyield_h;
    //yield Kasvener table Muu_jako
    // cyield_... = yield*dd*cc
    cyield_ww = 3688*0.86*0.45;
    cyield_sw = 3569*0.86*0.45;
    cyield_r = 2426*0.86*0.45;
    cyield_b = 3551*0.86*0.45;
    cyield_o = 3532*0.86*0.45;
    cyield_ra = 2011*0.92*0.45;
    cyield_h = 3252*0.86*0.45;

    //Carbon of above ground / hectare on average
    // ..._ag = cyield_...*((1-hi)/hi)
    double c_annual_crop_ag = 0;
    double c_perennial_crop_ag = 0;
    //Carbon of roots
    double c_annual_crop_roots = 0;
    double c_perennial_crop_roots = 0;
    //Carbon of rhizo deposition
    double c_annual_crop_rhizo = 0;
    double c_perennial_crop_rhizo = 0;

    if (cult_plant==1)
    {
    	c_annual_crop_ag = cyield_ww*((1-0.42)/0.42);
     	c_annual_crop_roots = cyield_ww*(1/(0.42*5.6));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else if (cult_plant==2)
    {
    	c_annual_crop_ag = cyield_sw*((1-0.42)/0.42);
    	c_annual_crop_roots = cyield_sw*(1/(0.42*5.6));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else if (cult_plant==3)
    {
    	c_annual_crop_ag = cyield_r*((1-0.4)/0.4);
    	c_annual_crop_roots = cyield_r*(1/(0.4*5.6));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else if (cult_plant==4)
    {
    	c_annual_crop_ag = cyield_b*((1-0.53)/0.53);
    	c_annual_crop_roots = cyield_b*(1/(0.53*5.6));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else if (cult_plant==5)
    {
    	c_annual_crop_ag = cyield_o*((1-0.46)/0.46);
    	c_annual_crop_roots = cyield_o*(1/(0.46*5.6));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else if (cult_plant==6)
    {
    	c_annual_crop_ag = cyield_ra*((1-0.35)/0.35);
    	c_annual_crop_roots = cyield_ra*(1/(0.35*5.1));
    	c_annual_crop_rhizo = 0.41*c_annual_crop_roots;
    }
    else
    {
    	c_perennial_crop_ag = cyield_h*((1-0.84)/0.84);
     	c_perennial_crop_roots = (4036/3.5)*0.45;
    	c_perennial_crop_rhizo = 0.41*4036*0.45;
    }

    c_annual_crop = c_annual_crop_ag + c_annual_crop_roots + c_annual_crop_rhizo;
    c_perennial_crop = c_perennial_crop_ag + c_perennial_crop_roots + c_perennial_crop_rhizo;

    *nres = 1;

    double carbon_A = c_annual_crop * 0.74 + c_perennial_crop * 0.451 + c_manure * 0.645;
    double carbon_W = c_annual_crop * 0.094 + c_perennial_crop * 0.358 + c_manure * 0.123;
    double carbon_E = c_annual_crop * 0.021 + c_perennial_crop * 0.034 + c_manure * 0.072;
    double carbon_N = c_annual_crop * 0.144 + c_perennial_crop * 0.157 + c_manure * 0.161;

    components[0].Carbon_A_nwl = carbon_A;
    components[0].Carbon_W_nwl = carbon_W;
    components[0].Carbon_E_nwl = carbon_E;
    components[0].Carbon_N_nwl = carbon_N;

    return ret;

}

int Carbon_cwl_after_harvest_Liski (double Carbon_A_cwl, double Carbon_W_cwl, double Carbon_E_cwl, double Carbon_N_cwl, double Carbon_dimension_cwl,
			double Harvest_BM_stumps_to_soil, double Harvest_BM_scrapstems_to_soil, double SP, double carbon_in_biomass,
			int *nres, struct carbon_cwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_stem, c_stump, carbon_A, carbon_W, carbon_E, carbon_N;

    //Carbon (kg/ha) is assumed to be half of biomass
    //At the same time Harvest_ is changed from 1000 kg/ha to kg/ha
    c_stem = carbon_in_biomass * Harvest_BM_scrapstems_to_soil * 1000;
    c_stump = carbon_in_biomass * Harvest_BM_stumps_to_soil * 1000;

    *nres = 1;

    if (SP == 1 || SP == 8)
    {
    	carbon_A = c_stump * 0.69 + c_stem * 0.69;
    	carbon_W = c_stump * 0.015 + c_stem * 0.015;
    	carbon_E = c_stump * 0.015 + c_stem * 0.015;
    	carbon_N = c_stump * 0.28 + c_stem * 0.28;
    }
    else if (SP == 2)
    {
    	carbon_A = c_stump * 0.69 + c_stem * 0.69;
    	carbon_W = c_stump * 0.005 + c_stem * 0.005;
    	carbon_E = c_stump * 0.005 + c_stem * 0.005;
    	carbon_N = c_stump * 0.3 + c_stem * 0.3;
    }
    else
    {
    	carbon_A = c_stump * 0.77 + c_stem * 0.77;
    	carbon_W = c_stump * 0.005 + c_stem * 0.005;
    	carbon_E = c_stump * 0.005 + c_stem * 0.005;
    	carbon_N = c_stump * 0.22 + c_stem * 0.22;
    }

    components[0].Carbon_A_cwl = Carbon_A_cwl + carbon_A;
    components[0].Carbon_W_cwl = Carbon_W_cwl + carbon_W;
    components[0].Carbon_E_cwl = Carbon_E_cwl + carbon_E;
    components[0].Carbon_N_cwl = Carbon_N_cwl + carbon_N;
    components[0].Carbon_dimension_cwl = Carbon_dimension_cwl;

    return ret;

}

int Carbon_fwl_after_harvest_Liski (double Carbon_A_fwl, double Carbon_W_fwl, double Carbon_E_fwl, double Carbon_N_fwl, double Carbon_dimension_fwl,
			double Harvest_BM_branches_to_soil, double Harvest_BM_roots_to_soil, double Clearing_BM, double Tending_BM, double SP, double carbon_in_biomass,
			int *nres, struct carbon_fwl_components *components, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double c_branches_roots, carbon_A, carbon_W, carbon_E, carbon_N;;
    double BM_branches_roots = Harvest_BM_branches_to_soil + Harvest_BM_roots_to_soil + Clearing_BM + Tending_BM;


    //Carbon (kg/ha) is assumed to be half of biomass
    c_branches_roots = carbon_in_biomass * BM_branches_roots * 1000;

    *nres = 1;

    if (SP == 1 || SP == 8)
    {
    	carbon_A = c_branches_roots * 0.66;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.31;
    }
    else if (SP == 2)
    {
    	carbon_A = c_branches_roots * 0.61;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.36;
    }
    else
    {
    	carbon_A = c_branches_roots * 0.65;
    	carbon_W = c_branches_roots * 0.015;
    	carbon_E = c_branches_roots * 0.015;
    	carbon_N = c_branches_roots * 0.32;
    }

    components[0].Carbon_A_fwl = Carbon_A_fwl + carbon_A;
    components[0].Carbon_W_fwl = Carbon_W_fwl + carbon_W;
    components[0].Carbon_E_fwl = Carbon_E_fwl + carbon_E;
    components[0].Carbon_N_fwl = Carbon_N_fwl + carbon_N;
    components[0].Carbon_dimension_fwl = Carbon_dimension_fwl;

    return ret;

}

int Mean_height_JKK (double D_gM, double H_gM, double PEAT, double SP, double Age, double TS, double SC, double NF,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	double H_gM_orig = H_gM;
	//Check if relation between D_gM and H_gM is realistic
	ret = Mean_height_check_JKK(D_gM, H_gM, PEAT, SP, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);

    //Calculate H_gM if H_gM was changed in model Mean_height_check_JKK, i.e. relation between D_gM and H_gM was not correct
   	if (*modelresult!=H_gM_orig)
    {
    	//Calculate H_gM to Southern and Middle Finland
        if (NF==0)
    	{
    		//Mineral soil
    		if (PEAT==0)
    		{
    			//Scots pine and other conifers
    			if (SP==1||SP==8)
    			{
    				ret = Mean_height_pine_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Norway spruce
    			else if (SP==2)
    			{
    				ret = Mean_height_spruce_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Birches and other deciduous species
    			else
    			{
    				ret = Mean_height_birch_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}

    			if (ret == 0)
    				return ret;

    		}
    		//Peatland
    		else if (PEAT==1)
    		{
    			//Scots pine and other conifers
    			if (SP==1||SP==8)
    			{
    				ret = Mean_height_pine_peatland_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Norway spruce
    			else if (SP==2)
    			{
    				ret = Mean_height_spruce_peatland_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Birches and other deciduous species
    			else
    			{
    				ret = Mean_height_birch_peatland_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}

    			if (ret == 0)
    				return ret;
    		}
    		//Check if relation between D_gM and H_gM is now realistic
    		ret = Mean_height_check_JKK(D_gM, *modelresult, PEAT, SP, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    	}
    	//Calculate H_gM to Northern Finland (NF==1)
        else
    	{
    		//Mineral soil
    		if (PEAT==0)
    		{
    			//Scots pine and other conifers
    			if (SP==1||SP==8)
    			{
    				ret = Mean_height_pine_peatland_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Norway spruce
    			else if (SP==2)
    			{
    				ret = Mean_height_spruce_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Birches and other deciduous species
    			else
    			{
    				ret = Mean_height_birch_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}

    			if (ret == 0)
    				return ret;

    		}
    		//Peatland
    		else if (PEAT==1)
    		{
    			//Scots pine and other conifers
    			if (SP==1||SP==8)
    			{
    				ret = Mean_height_pine_peatland_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Norway spruce
    			else if (SP==2)
    			{
    				ret = Mean_height_spruce_peatland_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}
    			//Birches and other deciduous species
    			else
    			{
    				ret = Mean_height_birch_peatland_entire_country_Nuutinen(Age, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    			}

    			if (ret == 0)
    				return ret;
    		}
    		//Check if relation between D_gM and H_gM is now realistic
    		ret = Mean_height_check_JKK(D_gM, *modelresult, PEAT, SP, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    	}
     }

	*nres = 1;
	return ret;

}

int Dominant_height_JKK (double H_gM, double SP,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate dominant height
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Dominant_height_pine_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Dominant_height_spruce_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_birch_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	*nres = 1;
	return ret;

}

int Dominant_height_comp_unit_JKK (double H_gM, double MAIN_SP,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate dominant height
	//Scots pine and other conifers
	if (MAIN_SP==1||MAIN_SP==8)
	{
		ret = Dominant_height_pine_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (MAIN_SP==2)
	{
		ret = Dominant_height_spruce_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_birch_JKK(H_gM, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	*nres = 1;
	return ret;

}

int Basal_area_JKK (double Age, double D_gM, double N, double SP,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate basal area, when D_gM and N are known and greater than 0
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_pine_Nissinen(Age, D_gM, N, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_spruce_Nissinen(Age, D_gM, N, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Basal_area_birch_Nissinen(D_gM, N, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	*nres = 1;
	return ret;

}

int Mean_age_JKK (double D_gM, double SP, double t_href, double H_gM, double TS, double SC,
                double PEAT, double SCRUB, double WASTE, double NF,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double Age;

	//Calculate biological mean age
	//First calculate age at breast height
	//Scots pine and other conifers
	if (D_gM>0)
	{
        if (SP==1||SP==8)
	    {
		    //Southern Finland
		    if (NF==0)
		    {
			    ret = Age_pine_SF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }
		    //Northern Finland (NF==1)
		    else
		    {
			    ret = Age_pine_NF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }

		    if (ret == 0)
			    return ret;

	    }
	    //Norway spruce
	    else if (SP==2)
	    {
		    //Southern Finland
		    if (NF==0)
		    {
			    ret = Age_spruce_SF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }
		    //Northern Finland (NF==1)
		    else
		    {
			    ret = Age_spruce_NF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }

		    if (ret == 0)
			    return ret;

	    }
	    //Birches and other deciduous species
	    else
	    {
		    //Southern Finland
		    if (NF==0)
		    {
			    ret = Age_birch_SF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }
		    //Northern Finland (NF==1)
		    else
		    {
			    ret = Age_birch_NF_JKK(D_gM, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		    }

		    if (ret == 0)
			    return ret;
	    }
	    //Calculate biological age
	    Age = *modelresult + t_href;
    }
    else
    {
        if(H_gM>0&&H_gM<=1.3)
        {
            Age = (H_gM/1.3) * t_href;
        }
        else
        {
            Age = 10;
        }
    }

	*nres = 1;
	*modelresult = Age;
	return ret;

}

int Site_index_JKK (double SP, double TS, double ALT, double LAKE, double SEA, double SC,
						double STONY, double PALU, double HUMUS, double RDF, double PLANT, double PUB, double PEAT,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate site index to stratum
	//Scots pine and other conifers
	//Fixed value for RDF = 0,75

	if (SP==1||SP==8)
	{
		ret = Site_index_pine_Hynynenym(TS, ALT, LAKE, SEA, SC, STONY, PALU, HUMUS, 0.75,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Site_index_spruce_Hynynenym(TS, LAKE, SEA, SC, STONY, PALU, HUMUS,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Site_index_birch_Hynynenym(TS, LAKE, PLANT, SC, PUB, PALU, HUMUS,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	*nres = 1;
	return ret;

}

int Age_at_breast_height_aggr_JKK (double Age, double A, double t_href, double D_gM, double SC,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Update age at breast height
	//Calculate using biological age if not exists
	if (A==0)
	{
		A = Age - t_href;
	}
	//If A has now negative value
	if (A<0)
	{
		A = 0;
	}
	//Fix A if it's value is zero, but D_gM is greater than zero
	if (A==0&&D_gM>0)
	{
		if(SC<=3)
		{
			if(D_gM<=2)
			{
				A =Age*0.1;
			}
			else if(D_gM>2&&D_gM<=5)
			{
				A =Age*0.2;
			}
			else if(D_gM>5&&D_gM<=10)
			{
				A =Age*0.5;
			}
			else
			{
				A = Age*0.6;
			}
		}
		else {
			if(D_gM<=2)
			{
				A =Age*0.2;
			}
			else if(D_gM>2&&D_gM<=5)
			{
				A =Age*0.3;
			}
			else if(D_gM>5&&D_gM<=10)
			{
				A =Age*0.5;
			}
			else
			{
				A = Age*0.6;
			}
		}
	}

	*nres = 1;
	*modelresult = A;
	return ret;

}

int Update_age_at_breast_height_JKK (double Age, double A, double t_href, double D_gM, double SC, double timeStep,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Update age at breast height
	//Calculate using biological age if not exists
	if (A==0)
	{
		A = Age - t_href;
	}
	//Update age
	A = A + timeStep;
	//If A has now negative value
	if (A<0)
	{
		A = 0;
	}
	//Fix A if it's value is zero, but D_gM is greater than zero
	if (A==0&&D_gM>0&&Age+1!=t_href)
	{
		if(SC<=3)
		{
			if(D_gM<=2)
			{
				A =Age*0.1;
			}
			else if(D_gM>2&&D_gM<=5)
			{
				A =Age*0.2;
			}
			else if(D_gM>5&&D_gM<=10)
			{
				A =Age*0.5;
			}
			else
			{
				A = Age*0.6;
			}
		}
		else {
			if(D_gM<=2)
			{
				A =Age*0.2;
			}
			else if(D_gM>2&&D_gM<=5)
			{
				A =Age*0.3;
			}
			else if(D_gM>5&&D_gM<=10)
			{
				A =Age*0.5;
			}
			else
			{
				A = Age*0.6;
			}
		}
	}

	*nres = 1;
	*modelresult = A;
	return ret;

}

int Value_growth_JKK (double stand_value, double stand_value_old, double value_growth_rel_1, double value_growth_rel_2,
					double value_growth_rel_3, double value_growth_rel_4, double value_growth_rel_5,
					double value_growth_abs_1, double value_growth_abs_2, double value_growth_abs_3,
					double value_growth_abs_4, double value_growth_abs_5, double timeStep,
					double Age, int *nres, struct Value_growth *value_growth, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate absolute and relative value growth
	double value_growth_abs, value_growth_rel;
	if (stand_value_old>10&&stand_value>10)
	//stand value is set to be bigger than 10 , because very little values cause problems (huge values) to value_growth calculation
	{
	        if (timeStep > 1)
	        {
	           value_growth_abs = (stand_value - stand_value_old)/timeStep;
	        }
	        else
	        {
	           value_growth_abs = stand_value - stand_value_old;
	        }
	        value_growth_rel = value_growth_abs/stand_value_old*100;
	}
	else
	{
		value_growth_abs = 0;
		value_growth_rel = 0;
	}


	//Assign old value growth value to value of value_growth_rel_#_old variable
	double value_growth_rel_1_old, value_growth_rel_2_old, value_growth_rel_3_old, value_growth_rel_4_old, value_growth_rel_5_old,
	value_growth_abs_1_old, value_growth_abs_2_old, value_growth_abs_3_old, value_growth_abs_4_old, value_growth_abs_5_old;
	value_growth_rel_1_old = value_growth_rel_1;
	value_growth_rel_2_old = value_growth_rel_2;
	value_growth_rel_3_old = value_growth_rel_3;
	value_growth_rel_4_old = value_growth_rel_4;
	value_growth_rel_5_old = value_growth_rel_5;
	value_growth_abs_1_old = value_growth_abs_1;
	value_growth_abs_2_old = value_growth_abs_2;
	value_growth_abs_3_old = value_growth_abs_3;
	value_growth_abs_4_old = value_growth_abs_4;
	value_growth_abs_5_old = value_growth_abs_5;

	//Move values when a new value have been calculated if value is not 0
    if (value_growth_rel != 0)
    {
        value_growth_rel_1 = value_growth_rel;
        value_growth_rel_2 = value_growth_rel_1_old;
        value_growth_rel_3 = value_growth_rel_2_old;
        value_growth_rel_4 = value_growth_rel_3_old;
        value_growth_rel_5 = value_growth_rel_4_old;
        value_growth_abs_1 = value_growth_abs;
        value_growth_abs_2 = value_growth_abs_1_old;
        value_growth_abs_3 = value_growth_abs_2_old;
        value_growth_abs_4 = value_growth_abs_3_old;
        value_growth_abs_5 = value_growth_abs_4_old;
    }

    //Calculate mean_value_growth_rel if 5 values exists
    double sum_value_growth_rel, mean_value_growth_rel, sum_value_growth_abs, mean_value_growth_abs;
    if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999&&value_growth_rel_4>-999&&value_growth_rel_5>-999)
    {
            if(timeStep==1)
            {
		    sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3+value_growth_rel_4+value_growth_rel_5;
		    mean_value_growth_rel = sum_value_growth_rel/5;
		    sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3+value_growth_abs_4+value_growth_abs_5;
		    mean_value_growth_abs = sum_value_growth_abs/5;
   	    }
   	    else if (timeStep==2)
   	    {
		    sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3;
		    mean_value_growth_rel = sum_value_growth_rel/3;
		    sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3;
		    mean_value_growth_abs = sum_value_growth_abs/3;
   	    }
   	    else if (timeStep==3)
   	    {
		    sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2;
		    mean_value_growth_rel = sum_value_growth_rel/2;
		    sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2;
		    mean_value_growth_abs = sum_value_growth_abs/2;
   	    }
   	    else if (timeStep>=4)
   	    {
		    mean_value_growth_rel = value_growth_rel_1;
		    mean_value_growth_abs = value_growth_abs_1;
   	    }
    }
    else if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999&&value_growth_rel_4>-999)
    {
	if (timeStep==1)
        {
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3+value_growth_rel_4;
		mean_value_growth_rel = sum_value_growth_rel/4;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3+value_growth_abs_4;
	        mean_value_growth_abs = sum_value_growth_abs/4;
	}
	else if (timeStep==2)
        {
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3;
		mean_value_growth_rel = sum_value_growth_rel/3;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3;
	        mean_value_growth_abs = sum_value_growth_abs/3;
	}
	else if (timeStep==3)
   	{
	        sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2;
	        mean_value_growth_rel = sum_value_growth_rel/2;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2;
		mean_value_growth_abs = sum_value_growth_abs/2;
   	}
   	else if (timeStep>=4)
   	{
	        mean_value_growth_rel = value_growth_rel_1;
		mean_value_growth_abs = value_growth_abs_1;
   	}
    }
    else if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999)
    {
    	if (timeStep<=2)
        {
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3;
		mean_value_growth_rel = sum_value_growth_rel/3;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3;
	    	mean_value_growth_abs = sum_value_growth_abs/3;
	}
	else if (timeStep==3)
   	{
	        sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2;
	        mean_value_growth_rel = sum_value_growth_rel/2;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2;
		mean_value_growth_abs = sum_value_growth_abs/2;
   	}
   	else if (timeStep>=4)
   	{
	        mean_value_growth_rel = value_growth_rel_1;
	        mean_value_growth_abs = value_growth_abs_1;
	}
    }
    else if (value_growth_rel_1>-999&&value_growth_rel_2>-999)
    {
    	if (timeStep<=3)
        {
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2;
		mean_value_growth_rel = sum_value_growth_rel/2;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2;
	    	mean_value_growth_abs = sum_value_growth_abs/2;
	}
	else if (timeStep>=4)
   	{
	        mean_value_growth_rel = value_growth_rel_1;
	        mean_value_growth_abs = value_growth_abs_1;
	}
    }
    else if (value_growth_rel_1>-999)
    {
        	mean_value_growth_rel = value_growth_rel_1;
	        mean_value_growth_abs = value_growth_abs_1;
    }
    else
    {
        	mean_value_growth_rel = value_growth_rel;
        	mean_value_growth_abs = value_growth_abs;
    }

	//Store the new values
	value_growth[0].value_growth_abs = value_growth_abs;
	value_growth[0].value_growth_rel = value_growth_rel;
	value_growth[0].mean_value_growth_abs = mean_value_growth_abs;
	value_growth[0].mean_value_growth_rel = mean_value_growth_rel;
	value_growth[0].value_growth_rel_1 = value_growth_rel_1;
	value_growth[0].value_growth_rel_2 = value_growth_rel_2;
	value_growth[0].value_growth_rel_3 = value_growth_rel_3;
	value_growth[0].value_growth_rel_4 = value_growth_rel_4;
	value_growth[0].value_growth_rel_5 = value_growth_rel_5;
	value_growth[0].value_growth_abs_1 = value_growth_abs_1;
	value_growth[0].value_growth_abs_2 = value_growth_abs_2;
	value_growth[0].value_growth_abs_3 = value_growth_abs_3;
	value_growth[0].value_growth_abs_4 = value_growth_abs_4;
	value_growth[0].value_growth_abs_5 = value_growth_abs_5;

	*nres = 1;
	return ret;

}

int Value_growth_standsimulator_JKK (double stand_value, double stand_value_old, double value_growth_rel_1, double value_growth_rel_2,
					double value_growth_rel_3, double value_growth_rel_4, double value_growth_rel_5,
					double value_growth_abs_1, double value_growth_abs_2, double value_growth_abs_3,
					double value_growth_abs_4, double value_growth_abs_5, double timeStep,
					double Age, int *nres, struct Value_growth *value_growth, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate absolute and relative value growth
	double value_growth_abs, value_growth_rel;
	if (stand_value_old>10&&stand_value>10)
	//stand value is set to be bigger than 10 , because very little values cause problems (huge values) to value_growth calculation
	{
        if (timeStep > 1)
        {
           value_growth_abs = (stand_value - stand_value_old)/timeStep;
        }
        else
        {
           value_growth_abs = stand_value - stand_value_old;
        }
        value_growth_rel = value_growth_abs/stand_value_old*100;
    }
	else
	{
		value_growth_abs = 0;
		value_growth_rel = 0;
	}


	//Assign old value growth value to value of value_growth_rel_#_old variable
	double value_growth_rel_1_old, value_growth_rel_2_old, value_growth_rel_3_old, value_growth_rel_4_old, value_growth_rel_5_old,
	value_growth_abs_1_old, value_growth_abs_2_old, value_growth_abs_3_old, value_growth_abs_4_old, value_growth_abs_5_old;
	value_growth_rel_1_old = value_growth_rel_1;
	value_growth_rel_2_old = value_growth_rel_2;
	value_growth_rel_3_old = value_growth_rel_3;
	value_growth_rel_4_old = value_growth_rel_4;
	value_growth_rel_5_old = value_growth_rel_5;
	value_growth_abs_1_old = value_growth_abs_1;
	value_growth_abs_2_old = value_growth_abs_2;
	value_growth_abs_3_old = value_growth_abs_3;
	value_growth_abs_4_old = value_growth_abs_4;
	value_growth_abs_5_old = value_growth_abs_5;

	//Move values when a new value have been calculated if value is not 0
    if (value_growth_rel != 0)
    {
        value_growth_rel_1 = value_growth_rel;
        value_growth_rel_2 = value_growth_rel_1_old;
        value_growth_rel_3 = value_growth_rel_2_old;
        value_growth_rel_4 = value_growth_rel_3_old;
        value_growth_rel_5 = value_growth_rel_4_old;
        value_growth_abs_1 = value_growth_abs;
        value_growth_abs_2 = value_growth_abs_1_old;
        value_growth_abs_3 = value_growth_abs_2_old;
        value_growth_abs_4 = value_growth_abs_3_old;
        value_growth_abs_5 = value_growth_abs_4_old;
    }

    //Calculate mean_value_growth_rel if 5 values exists
    double sum_value_growth_rel, mean_value_growth_rel, sum_value_growth_abs, mean_value_growth_abs;
    if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999&&value_growth_rel_4>-999&&value_growth_rel_5>-999)
    {
	    sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3+value_growth_rel_4+value_growth_rel_5;
	    mean_value_growth_rel = sum_value_growth_rel/5;
	    sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3+value_growth_abs_4+value_growth_abs_5;
	    mean_value_growth_abs = sum_value_growth_abs/5;
    }
    else if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999&&value_growth_rel_4>-999)
	{
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3+value_growth_rel_4;
		mean_value_growth_rel = sum_value_growth_rel/4;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3+value_growth_abs_4;
	        mean_value_growth_abs = sum_value_growth_abs/4;
	}
	else if (value_growth_rel_1>-999&&value_growth_rel_2>-999&&value_growth_rel_3>-999)
	{
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2+value_growth_rel_3;
		mean_value_growth_rel = sum_value_growth_rel/3;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2+value_growth_abs_3;
	    	mean_value_growth_abs = sum_value_growth_abs/3;
	}
	else if (value_growth_rel_1>-999&&value_growth_rel_2>-999)
	{
		sum_value_growth_rel = value_growth_rel_1+value_growth_rel_2;
		mean_value_growth_rel = sum_value_growth_rel/2;
		sum_value_growth_abs = value_growth_abs_1+value_growth_abs_2;
	    	mean_value_growth_abs = sum_value_growth_abs/2;
	}
	else if (value_growth_rel_1>-999)
        {
        	mean_value_growth_rel = value_growth_rel_1;
	        mean_value_growth_abs = value_growth_abs_1;
	}
	else if (value_growth_rel_1>-999)
    	{
         	if (value_growth_rel_1<1||value_growth_rel_1>20||(value_growth_rel_1>10&&Age>50)||(value_growth_rel_1>5&&Age>70)||(value_growth_rel_1>3&&Age>80))
         	{
             if (Age < 25)
             {
                mean_value_growth_rel = 15;
             }
             else if (Age >= 25&& Age < 40)
             {
                mean_value_growth_rel = 8;
             }
             else if (Age >= 40&& Age < 55)
             {
                mean_value_growth_rel = 5;
             }
             else if (Age >= 55&& Age < 70)
             {
                mean_value_growth_rel = 3;
             }
             else
             {
                mean_value_growth_rel = 1;
             }
             value_growth_rel_1 = mean_value_growth_rel;
         }

         else
         {
	         mean_value_growth_rel = value_growth_rel_1;
         }
	}
	else
	{
          mean_value_growth_rel = value_growth_rel;
          mean_value_growth_abs = value_growth_abs;
	}

	//Store the new values
	value_growth[0].value_growth_abs = value_growth_abs;
	value_growth[0].value_growth_rel = value_growth_rel;
	value_growth[0].mean_value_growth_abs = mean_value_growth_abs;
	value_growth[0].mean_value_growth_rel = mean_value_growth_rel;
	value_growth[0].value_growth_rel_1 = value_growth_rel_1;
	value_growth[0].value_growth_rel_2 = value_growth_rel_2;
	value_growth[0].value_growth_rel_3 = value_growth_rel_3;
	value_growth[0].value_growth_rel_4 = value_growth_rel_4;
	value_growth[0].value_growth_rel_5 = value_growth_rel_5;
	value_growth[0].value_growth_abs_1 = value_growth_abs_1;
	value_growth[0].value_growth_abs_2 = value_growth_abs_2;
	value_growth[0].value_growth_abs_3 = value_growth_abs_3;
	value_growth[0].value_growth_abs_4 = value_growth_abs_4;
	value_growth[0].value_growth_abs_5 = value_growth_abs_5;

	*nres = 1;
	return ret;

}

int Productive_value_JKK (double TS, double SC, double D_g, double H_gM, double BA, double Age, double N,
                       double BA_Sp, double BA_Ns, double BA_b, double BA_od, double N_Sp, double N_Ns, double N_b, double N_od, double PEAT,
                       double MAIN_SP, double MAIN_GROUP, double IR, double log_price_Sp, double log_price_Ns, double log_price_b, double log_price_od,
                       double pulp_price_Sp, double pulp_price_Ns, double pulp_price_b, double pulp_price_od, double seedtree_stand, double Peatland_reduction,
		       int *nres, struct PVs *pv_values, char *errors,
		       int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    //Initial values for PV variables
    double PV_Sp=0;
    double PV_Ns=0;
    double PV_b=0;
    double PV_od=0;
    double PVland_Sp=0;
    double PVland_Ns=0;
    double PVland_b=0;
    double PVland_od=0;
    double PV_1_Sp=0;
    double PV_1_Ns=0;
    double PV_1_b=0;
    double PV_1_od=0;
    double PVland_1_Sp=0;
    double PVland_1_Ns=0;
    double PVland_1_b=0;
    double PVland_1_od=0;
    double PV_2_Sp=0;
    double PV_2_Ns=0;
    double PV_2_b=0;
    double PV_2_od=0;
    double PVland_2_Sp=0;
    double PVland_2_Ns=0;
    double PVland_2_b=0;
    double PVland_2_od=0;
    double PV_3_Sp=0;
    double PV_3_Ns=0;
    double PV_3_b=0;
    double PV_3_od=0;
    double PVland_3_Sp=0;
    double PVland_3_Ns=0;
    double PVland_3_b=0;
    double PVland_3_od=0;
    double PV_4_Sp=0;
    double PV_4_Ns=0;
    double PV_4_b=0;
    double PV_4_od=0;
    double PVland_4_Sp=0;
    double PVland_4_Ns=0;
    double PVland_4_b=0;
    double PVland_4_od=0;
    double PV_5_Sp=0;
    double PV_5_Ns=0;
    double PV_5_b=0;
    double PV_5_od=0;
    double PVland_5_Sp=0;
    double PVland_5_Ns=0;
    double PVland_5_b=0;
    double PVland_5_od=0;
    double PV_6_Sp=0;
    double PV_6_Ns=0;
    double PV_6_b=0;
    double PV_6_od=0;
    double PVland_6_Sp=0;
    double PVland_6_Ns=0;
    double PVland_6_b=0;
    double PVland_6_od=0;
    double PV_7_Sp=0;
    double PV_7_Ns=0;
    double PV_7_b=0;
    double PV_7_od=0;
    double PVland_7_Sp=0;
    double PVland_7_Ns=0;
    double PVland_7_b=0;
    double PVland_7_od=0;
    double res=0;
    double res_1=0;
    double res_2=0;
    double res_3=0;
    double res_4=0;
    double res_5=0;
    double res_6=0;
    double res_7=0;
    double res_land=0;
    double res_land_1=0;
    double res_land_2=0;
    double res_land_3=0;
    double res_land_4=0;
    double res_land_5=0;
    double res_land_6=0;
    double res_land_7=0;
    
    //Restictions for Pukkala's models to avoid non-logical behaviour of model at seedling stage 
    if (D_g<5)
       D_g = 5;
    if (IR>=0&&IR<1)
       IR=1.0;
         
    //Calculate productive value for the land and growing stock

    //Check for fatal errors
	char errorStr[100] = "";
	if (IR<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "IR<=0(IR=",IR);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

    //Scots pine and other conifers        
    Productive_value_of_pine_Pukkala (TS, SC, D_g, BA, Age, IR, log_price_Sp, pulp_price_Sp, H_gM, pv_values);
    PV_Sp=pv_values[0].PV;
    PVland_Sp=pv_values[0].PV_land;
    PV_1_Sp=pv_values[0].PV_1;
    PV_2_Sp=pv_values[0].PV_2;
    PV_3_Sp=pv_values[0].PV_3;
    PV_4_Sp=pv_values[0].PV_4;
    PV_5_Sp=pv_values[0].PV_5;
    PV_6_Sp=pv_values[0].PV_6;
    PV_7_Sp=pv_values[0].PV_7;
    PVland_1_Sp=pv_values[0].PV_land_1;
    PVland_2_Sp=pv_values[0].PV_land_2;
    PVland_3_Sp=pv_values[0].PV_land_3;
    PVland_4_Sp=pv_values[0].PV_land_4;
    PVland_5_Sp=pv_values[0].PV_land_5;
    PVland_6_Sp=pv_values[0].PV_land_6;
    PVland_7_Sp=pv_values[0].PV_land_7;
    
    //For spruce
    Productive_value_of_spruce_Pukkala (TS, SC, D_g, BA, Age, IR, log_price_Ns, pulp_price_Ns, H_gM, pv_values);
    PV_Ns=pv_values[0].PV;
    PVland_Ns=pv_values[0].PV_land;
    PV_1_Ns=pv_values[0].PV_1;
    PV_2_Ns=pv_values[0].PV_2;
    PV_3_Ns=pv_values[0].PV_3;
    PV_4_Ns=pv_values[0].PV_4;
    PV_5_Ns=pv_values[0].PV_5;
    PV_6_Ns=pv_values[0].PV_6;
    PV_7_Ns=pv_values[0].PV_7;
    PVland_1_Ns=pv_values[0].PV_land_1;
    PVland_2_Ns=pv_values[0].PV_land_2;
    PVland_3_Ns=pv_values[0].PV_land_3;
    PVland_4_Ns=pv_values[0].PV_land_4;
    PVland_5_Ns=pv_values[0].PV_land_5;
    PVland_6_Ns=pv_values[0].PV_land_6;
    PVland_7_Ns=pv_values[0].PV_land_7;
   
    //For birch
    Productive_value_of_white_birch_Pukkala (TS, SC, D_g, BA, Age, N, IR, log_price_b, pulp_price_b, H_gM, pv_values);
    PV_b=pv_values[0].PV;
    PVland_b=pv_values[0].PV_land;
    PV_1_b=pv_values[0].PV_1;
    PV_2_b=pv_values[0].PV_2;
    PV_3_b=pv_values[0].PV_3;
    PV_4_b=pv_values[0].PV_4;
    PV_5_b=pv_values[0].PV_5;
    PV_6_b=pv_values[0].PV_6;
    PV_7_b=pv_values[0].PV_7;
    PVland_1_b=pv_values[0].PV_land_1;
    PVland_2_b=pv_values[0].PV_land_2;
    PVland_3_b=pv_values[0].PV_land_3;
    PVland_4_b=pv_values[0].PV_land_4;
    PVland_5_b=pv_values[0].PV_land_5;
    PVland_6_b=pv_values[0].PV_land_6;
    PVland_7_b=pv_values[0].PV_land_7;
    
    //For other deciduous species
    Productive_value_of_white_birch_Pukkala (TS, SC, D_g, BA, Age, N, IR, log_price_od, pulp_price_od, H_gM, pv_values);
    PV_od=0.5*pv_values[0].PV;
    PVland_od=pv_values[0].PV_land;
    PV_1_od=0.5*pv_values[0].PV_1;
    PV_2_od=0.5*pv_values[0].PV_2;
    PV_3_od=0.5*pv_values[0].PV_3;
    PV_4_od=0.5*pv_values[0].PV_4;
    PV_5_od=0.5*pv_values[0].PV_5;
    PV_6_od=0.5*pv_values[0].PV_6;
    PV_7_od=0.5*pv_values[0].PV_7;
    PVland_1_od=pv_values[0].PV_land_1;
    PVland_2_od=pv_values[0].PV_land_2;
    PVland_3_od=pv_values[0].PV_land_3;
    PVland_4_od=pv_values[0].PV_land_4;
    PVland_5_od=pv_values[0].PV_land_5;
    PVland_6_od=pv_values[0].PV_land_6;
    PVland_7_od=pv_values[0].PV_land_7;
    
    //Calculate productive value weighted with basal area or stem number of tree species
    //For mineral land
    *nres = 1;

    if (PEAT==0)
    {
       if (H_gM>=3 && BA > 0)
       {
          res_land = (BA_Sp/BA)*PVland_Sp+(BA_Ns/BA)*PVland_Ns+(BA_b/BA)*PVland_b+(BA_od/BA)*PVland_od;
          res = (BA_Sp/BA)*PV_Sp+(BA_Ns/BA)*PV_Ns+(BA_b/BA)*PV_b+(BA_od/BA)*PV_od;
          res_land_1 = (BA_Sp/BA)*PVland_1_Sp+(BA_Ns/BA)*PVland_1_Ns+(BA_b/BA)*PVland_1_b+(BA_od/BA)*PVland_1_od;
          res_land_2 = (BA_Sp/BA)*PVland_2_Sp+(BA_Ns/BA)*PVland_2_Ns+(BA_b/BA)*PVland_2_b+(BA_od/BA)*PVland_2_od;
          res_land_3 = (BA_Sp/BA)*PVland_3_Sp+(BA_Ns/BA)*PVland_3_Ns+(BA_b/BA)*PVland_3_b+(BA_od/BA)*PVland_3_od;
          res_land_4 = (BA_Sp/BA)*PVland_4_Sp+(BA_Ns/BA)*PVland_4_Ns+(BA_b/BA)*PVland_4_b+(BA_od/BA)*PVland_4_od;
          res_land_5 = (BA_Sp/BA)*PVland_5_Sp+(BA_Ns/BA)*PVland_5_Ns+(BA_b/BA)*PVland_5_b+(BA_od/BA)*PVland_5_od;
          res_land_6 = (BA_Sp/BA)*PVland_6_Sp+(BA_Ns/BA)*PVland_6_Ns+(BA_b/BA)*PVland_6_b+(BA_od/BA)*PVland_6_od;
          res_land_7 = (BA_Sp/BA)*PVland_7_Sp+(BA_Ns/BA)*PVland_7_Ns+(BA_b/BA)*PVland_7_b+(BA_od/BA)*PVland_7_od;
          res_1 = (BA_Sp/BA)*PV_1_Sp+(BA_Ns/BA)*PV_1_Ns+(BA_b/BA)*PV_1_b+(BA_od/BA)*PV_1_od;
          res_2 = (BA_Sp/BA)*PV_2_Sp+(BA_Ns/BA)*PV_2_Ns+(BA_b/BA)*PV_2_b+(BA_od/BA)*PV_2_od;
          res_3 = (BA_Sp/BA)*PV_3_Sp+(BA_Ns/BA)*PV_3_Ns+(BA_b/BA)*PV_3_b+(BA_od/BA)*PV_3_od;
          res_4 = (BA_Sp/BA)*PV_4_Sp+(BA_Ns/BA)*PV_4_Ns+(BA_b/BA)*PV_4_b+(BA_od/BA)*PV_4_od;
          res_5 = (BA_Sp/BA)*PV_5_Sp+(BA_Ns/BA)*PV_5_Ns+(BA_b/BA)*PV_5_b+(BA_od/BA)*PV_5_od;
          res_6 = (BA_Sp/BA)*PV_6_Sp+(BA_Ns/BA)*PV_6_Ns+(BA_b/BA)*PV_6_b+(BA_od/BA)*PV_6_od;
          res_7 = (BA_Sp/BA)*PV_7_Sp+(BA_Ns/BA)*PV_7_Ns+(BA_b/BA)*PV_7_b+(BA_od/BA)*PV_7_od;
       }
       else if (N > 0)
       {
          res_land = (N_Sp/N)*PVland_Sp+(N_Ns/N)*PVland_Ns+(N_b/N)*PVland_b+(N_od/N)*PVland_od;
          res = (N_Sp/N)*PV_Sp+(N_Ns/N)*PV_Ns+(N_b/N)*PV_b+(N_od/N)*PV_od;
          res_land_1 = (N_Sp/N)*PVland_1_Sp+(N_Ns/N)*PVland_1_Ns+(N_b/N)*PVland_1_b+(N_od/N)*PVland_1_od;
          res_land_2 = (N_Sp/N)*PVland_2_Sp+(N_Ns/N)*PVland_2_Ns+(N_b/N)*PVland_2_b+(N_od/N)*PVland_2_od;
          res_land_3 = (N_Sp/N)*PVland_3_Sp+(N_Ns/N)*PVland_3_Ns+(N_b/N)*PVland_3_b+(N_od/N)*PVland_3_od;
          res_land_4 = (N_Sp/N)*PVland_4_Sp+(N_Ns/N)*PVland_4_Ns+(N_b/N)*PVland_4_b+(N_od/N)*PVland_4_od;
          res_land_5 = (N_Sp/N)*PVland_5_Sp+(N_Ns/N)*PVland_5_Ns+(N_b/N)*PVland_5_b+(N_od/N)*PVland_5_od;
          res_land_6 = (N_Sp/N)*PVland_6_Sp+(N_Ns/N)*PVland_6_Ns+(N_b/N)*PVland_6_b+(N_od/N)*PVland_6_od;
          res_land_7 = (N_Sp/N)*PVland_7_Sp+(N_Ns/N)*PVland_7_Ns+(N_b/N)*PVland_7_b+(N_od/N)*PVland_7_od;
          res_1 = (N_Sp/N)*PV_1_Sp+(N_Ns/N)*PV_1_Ns+(N_b/N)*PV_1_b+(N_od/N)*PV_1_od;
          res_2 = (N_Sp/N)*PV_2_Sp+(N_Ns/N)*PV_2_Ns+(N_b/N)*PV_2_b+(N_od/N)*PV_2_od;
          res_3 = (N_Sp/N)*PV_3_Sp+(N_Ns/N)*PV_3_Ns+(N_b/N)*PV_3_b+(N_od/N)*PV_3_od;
          res_4 = (N_Sp/N)*PV_4_Sp+(N_Ns/N)*PV_4_Ns+(N_b/N)*PV_4_b+(N_od/N)*PV_4_od;
          res_5 = (N_Sp/N)*PV_5_Sp+(N_Ns/N)*PV_5_Ns+(N_b/N)*PV_5_b+(N_od/N)*PV_5_od;
          res_6 = (N_Sp/N)*PV_6_Sp+(N_Ns/N)*PV_6_Ns+(N_b/N)*PV_6_b+(N_od/N)*PV_6_od;
          res_7 = (N_Sp/N)*PV_7_Sp+(N_Ns/N)*PV_7_Ns+(N_b/N)*PV_7_b+(N_od/N)*PV_7_od;
       }
       else
       {
       if (MAIN_SP == 1 || MAIN_SP == 8)
       {
          res_land = PVland_Sp;
          res_land_1 = PVland_1_Sp;
          res_land_2 = PVland_2_Sp;
          res_land_3 = PVland_3_Sp;
          res_land_4 = PVland_4_Sp;
          res_land_5 = PVland_5_Sp;
          res_land_6 = PVland_6_Sp;
          res_land_7 = PVland_7_Sp;
       }
       else if (MAIN_SP == 2)
       {
          res_land = PVland_Ns;
          res_land_1 = PVland_1_Ns;
          res_land_2 = PVland_2_Ns;
          res_land_3 = PVland_3_Ns;
          res_land_4 = PVland_4_Ns;
          res_land_5 = PVland_5_Ns;
          res_land_6 = PVland_6_Ns;
          res_land_7 = PVland_7_Ns;
       }
       else if (MAIN_SP == 3 || MAIN_SP == 4)
       {
          res_land = PVland_b;
          res_land_1 = PVland_1_b;
          res_land_2 = PVland_2_b;
          res_land_3 = PVland_3_b;
          res_land_4 = PVland_4_b;
          res_land_5 = PVland_5_b;
          res_land_6 = PVland_6_b;
          res_land_7 = PVland_7_b;
       }
       else
       {
          res_land = PVland_od;
          res_land_1 = PVland_1_od;
          res_land_2 = PVland_2_od;
          res_land_3 = PVland_3_od;
          res_land_4 = PVland_4_od;
          res_land_5 = PVland_5_od;
          res_land_6 = PVland_6_od;
          res_land_7 = PVland_7_od;
       }
       }
    }
    //For peatland
    else
    {
        if (H_gM>=3 && BA > 0)
        {
           res_land = Peatland_reduction*((BA_Sp/BA)*PVland_Sp+(BA_Ns/BA)*PVland_Ns+(BA_b/BA)*PVland_b+(BA_od/BA)*PVland_od);
           res = Peatland_reduction*((BA_Sp/BA)*PV_Sp+(BA_Ns/BA)*PV_Ns+(BA_b/BA)*PV_b+(BA_od/BA)*PV_od);
           res_land_1 = Peatland_reduction*((BA_Sp/BA)*PVland_1_Sp+(BA_Ns/BA)*PVland_1_Ns+(BA_b/BA)*PVland_1_b+(BA_od/BA)*PVland_1_od);
           res_land_2 = Peatland_reduction*((BA_Sp/BA)*PVland_2_Sp+(BA_Ns/BA)*PVland_2_Ns+(BA_b/BA)*PVland_2_b+(BA_od/BA)*PVland_2_od);
           res_land_3 = Peatland_reduction*((BA_Sp/BA)*PVland_3_Sp+(BA_Ns/BA)*PVland_3_Ns+(BA_b/BA)*PVland_3_b+(BA_od/BA)*PVland_3_od);
           res_land_4 = Peatland_reduction*((BA_Sp/BA)*PVland_4_Sp+(BA_Ns/BA)*PVland_4_Ns+(BA_b/BA)*PVland_4_b+(BA_od/BA)*PVland_4_od);
           res_land_5 = Peatland_reduction*((BA_Sp/BA)*PVland_5_Sp+(BA_Ns/BA)*PVland_5_Ns+(BA_b/BA)*PVland_5_b+(BA_od/BA)*PVland_5_od);
           res_land_6 = Peatland_reduction*((BA_Sp/BA)*PVland_6_Sp+(BA_Ns/BA)*PVland_6_Ns+(BA_b/BA)*PVland_6_b+(BA_od/BA)*PVland_6_od);
           res_land_7 = Peatland_reduction*((BA_Sp/BA)*PVland_7_Sp+(BA_Ns/BA)*PVland_7_Ns+(BA_b/BA)*PVland_7_b+(BA_od/BA)*PVland_7_od);
           res_1 = Peatland_reduction*((BA_Sp/BA)*PV_1_Sp+(BA_Ns/BA)*PV_1_Ns+(BA_b/BA)*PV_1_b+(BA_od/BA)*PV_1_od);
           res_2 = Peatland_reduction*((BA_Sp/BA)*PV_2_Sp+(BA_Ns/BA)*PV_2_Ns+(BA_b/BA)*PV_2_b+(BA_od/BA)*PV_2_od);
           res_3 = Peatland_reduction*((BA_Sp/BA)*PV_3_Sp+(BA_Ns/BA)*PV_3_Ns+(BA_b/BA)*PV_3_b+(BA_od/BA)*PV_3_od);
           res_4 = Peatland_reduction*((BA_Sp/BA)*PV_4_Sp+(BA_Ns/BA)*PV_4_Ns+(BA_b/BA)*PV_4_b+(BA_od/BA)*PV_4_od);
           res_5 = Peatland_reduction*((BA_Sp/BA)*PV_5_Sp+(BA_Ns/BA)*PV_5_Ns+(BA_b/BA)*PV_5_b+(BA_od/BA)*PV_5_od);
           res_6 = Peatland_reduction*((BA_Sp/BA)*PV_6_Sp+(BA_Ns/BA)*PV_6_Ns+(BA_b/BA)*PV_6_b+(BA_od/BA)*PV_6_od);
           res_7 = Peatland_reduction*((BA_Sp/BA)*PV_7_Sp+(BA_Ns/BA)*PV_7_Ns+(BA_b/BA)*PV_7_b+(BA_od/BA)*PV_7_od);
        }
        else if (N > 0)
        {
           res_land = Peatland_reduction*((N_Sp/N)*PVland_Sp+(N_Ns/N)*PVland_Ns+(N_b/N)*PVland_b+(N_od/N)*PVland_od);
           res = Peatland_reduction*((N_Sp/N)*PV_Sp+(N_Ns/N)*PV_Ns+(N_b/N)*PV_b+(N_od/N)*PV_od);
           res_land_1 = Peatland_reduction*((N_Sp/N)*PVland_1_Sp+(N_Ns/N)*PVland_1_Ns+(N_b/N)*PVland_1_b+(N_od/N)*PVland_1_od);
           res_land_2 = Peatland_reduction*((N_Sp/N)*PVland_2_Sp+(N_Ns/N)*PVland_2_Ns+(N_b/N)*PVland_2_b+(N_od/N)*PVland_2_od);
           res_land_3 = Peatland_reduction*((N_Sp/N)*PVland_3_Sp+(N_Ns/N)*PVland_3_Ns+(N_b/N)*PVland_3_b+(N_od/N)*PVland_3_od);
           res_land_4 = Peatland_reduction*((N_Sp/N)*PVland_4_Sp+(N_Ns/N)*PVland_4_Ns+(N_b/N)*PVland_4_b+(N_od/N)*PVland_4_od);
           res_land_5 = Peatland_reduction*((N_Sp/N)*PVland_5_Sp+(N_Ns/N)*PVland_5_Ns+(N_b/N)*PVland_5_b+(N_od/N)*PVland_5_od);
           res_land_6 = Peatland_reduction*((N_Sp/N)*PVland_6_Sp+(N_Ns/N)*PVland_6_Ns+(N_b/N)*PVland_6_b+(N_od/N)*PVland_6_od);
           res_land_7 = Peatland_reduction*((N_Sp/N)*PVland_7_Sp+(N_Ns/N)*PVland_7_Ns+(N_b/N)*PVland_7_b+(N_od/N)*PVland_7_od);
           res_1 = Peatland_reduction*((N_Sp/N)*PV_1_Sp+(N_Ns/N)*PV_1_Ns+(N_b/N)*PV_1_b+(N_od/N)*PV_1_od);
           res_2 = Peatland_reduction*((N_Sp/N)*PV_2_Sp+(N_Ns/N)*PV_2_Ns+(N_b/N)*PV_2_b+(N_od/N)*PV_2_od);
           res_3 = Peatland_reduction*((N_Sp/N)*PV_3_Sp+(N_Ns/N)*PV_3_Ns+(N_b/N)*PV_3_b+(N_od/N)*PV_3_od);
           res_4 = Peatland_reduction*((N_Sp/N)*PV_4_Sp+(N_Ns/N)*PV_4_Ns+(N_b/N)*PV_4_b+(N_od/N)*PV_4_od);
           res_5 = Peatland_reduction*((N_Sp/N)*PV_5_Sp+(N_Ns/N)*PV_5_Ns+(N_b/N)*PV_5_b+(N_od/N)*PV_5_od);
           res_6 = Peatland_reduction*((N_Sp/N)*PV_6_Sp+(N_Ns/N)*PV_6_Ns+(N_b/N)*PV_6_b+(N_od/N)*PV_6_od);
           res_7 = Peatland_reduction*((N_Sp/N)*PV_7_Sp+(N_Ns/N)*PV_7_Ns+(N_b/N)*PV_7_b+(N_od/N)*PV_7_od);
        }
        else
        {
        if (MAIN_SP == 1 || MAIN_SP == 8)
        {
          res_land = PVland_Sp;
          res_land_1 = PVland_1_Sp;
          res_land_2 = PVland_2_Sp;
          res_land_3 = PVland_3_Sp;
          res_land_4 = PVland_4_Sp;
          res_land_5 = PVland_5_Sp;
          res_land_6 = PVland_6_Sp;
          res_land_7 = PVland_7_Sp;
        }
       else if (MAIN_SP == 2)
       {
          res_land = PVland_Ns;
          res_land_1 = PVland_1_Ns;
          res_land_2 = PVland_2_Ns;
          res_land_3 = PVland_3_Ns;
          res_land_4 = PVland_4_Ns;
          res_land_5 = PVland_5_Ns;
          res_land_6 = PVland_6_Ns;
          res_land_7 = PVland_7_Ns;
       }
       else if (MAIN_SP == 3 || MAIN_SP == 4)
       {
          res_land = PVland_b;
          res_land_1 = PVland_1_b;
          res_land_2 = PVland_2_b;
          res_land_3 = PVland_3_b;
          res_land_4 = PVland_4_b;
          res_land_5 = PVland_5_b;
          res_land_6 = PVland_6_b;
          res_land_7 = PVland_7_b;
       }
       else
       {
          res_land = PVland_od;
          res_land_1 = PVland_1_od;
          res_land_2 = PVland_2_od;
          res_land_3 = PVland_3_od;
          res_land_4 = PVland_4_od;
          res_land_5 = PVland_5_od;
          res_land_6 = PVland_6_od;
          res_land_7 = PVland_7_od;
       }
       }
    }

    //If PV_land is greater than PV or Main_group is scrub or waste land or upper seedtree layer exists the value of PV_land is used also for PV
    if (res<res_land || MAIN_GROUP>1 || seedtree_stand==1)
    {
    	res = res_land;
    	res_1 = res_land_1;
    	res_2 = res_land_2;
    	res_3 = res_land_3;
    	res_4 = res_land_4;
    	res_5 = res_land_5;
    	res_6 = res_land_6;
    	res_7 = res_land_7;
    }
    
    //Store the result
    pv_values[0].PV = res;		//with given IR
    pv_values[0].PV_land = res_land;	//with given IR
    pv_values[0].PV_1 = res_1;
    pv_values[0].PV_2 = res_2;
    pv_values[0].PV_3 = res_3;
    pv_values[0].PV_4 = res_4;
    pv_values[0].PV_5 = res_5;
    pv_values[0].PV_6 = res_6;
    pv_values[0].PV_7 = res_7;
    pv_values[0].PV_land_1 = res_land_1;
    pv_values[0].PV_land_2 = res_land_2;
    pv_values[0].PV_land_3 = res_land_3;
    pv_values[0].PV_land_4 = res_land_4;
    pv_values[0].PV_land_5 = res_land_5;
    pv_values[0].PV_land_6 = res_land_6;
    pv_values[0].PV_land_7 = res_land_7;
    
    return ret;

}

int Carbon_flux_tending_energy_JKK (double carbon_in_biomass, double mwh_to_kg_c, double Tending_BM,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double ENERGY_FROM_MASS1 = 0.35;
    double ENERGY_FROM_MASS2 = 0.00314;

    double energy, c_ref, c_rel, c_rel_energy;

    //Carbon release from energy production from the wood material mass -- mass used in energy production
    energy = Tending_BM * (1. + ENERGY_FROM_MASS1) * ENERGY_FROM_MASS2;
    c_ref = energy * mwh_to_kg_c;
    c_rel = carbon_in_biomass * Tending_BM;
    c_rel_energy = c_rel - c_ref;

    *nres = 1;
    *modelresult = c_rel_energy;
    return ret;

}

int Carbon_flux_natural_removal_JKK (double BM_stems, double BM_branches, double BM_foliage,
			double natural_removal, double V, double BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL,
			double carbon_in_biomass, double mwh_to_kg_c,
			int *nres, struct Carbon_flux *values, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double prop_of_dead_trees_collected, BM, energy, c_ref, c_rel, c_rel_energy;

    double ENERGY_FROM_MASS1 = 0.35;
    double ENERGY_FROM_MASS2 = 0.00314;

    //Check for fatal errors
	char errorStr[200] = "";
	if (V<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V<=0(V=",V);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

     //Calculate proportion of dead trees from the growing stock
    prop_of_dead_trees_collected = (natural_removal * BIOMASS_COLLECTION_PROPORTION_NATURAL_REMOVAL)/V;

    BM = (BM_stems+BM_branches+BM_foliage) * prop_of_dead_trees_collected;

     //Carbon release from energy production from the wood material mass -- mass used in energy production
    energy = BM * (1. + ENERGY_FROM_MASS1) * ENERGY_FROM_MASS2;
    c_ref = energy * mwh_to_kg_c;
    c_rel = carbon_in_biomass * BM;
    c_rel_energy = c_rel - c_ref;

    *nres = 1;

    values[0].Carbon_flux_natural_removal = c_rel;
    values[0].Carbon_flux_natural_removal_energy = c_rel_energy;

    return ret;

}

int Site_index_increment_JKK (double SI_50, double climate_change_scenario, double time_step,
		int *nres, double *modelresult, char *errors,
		int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate site index increment due to climate change
	double change;

	// A1
	if (climate_change_scenario==1)
	{
		change = time_step*(-0.00418*SI_50+0.1135);
	}
	//A2
	else if (climate_change_scenario==2)
	{
		change = time_step*(-0.006*SI_50+0.157);
	}
	//B1 climate_change_scenario==3
	else
	{
		change = time_step*(-0.00257*SI_50+0.069);
	}
	*nres = 1;
    	*modelresult = change;
    	return ret;

}

int Site_index_present_state_JKK (double SI_50, double climate_change_scenario, double year,
		int *nres, double *modelresult, char *errors,
		int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate site index increment due to climate change
	//present state, base SI_50 value from year 2000
	double change;

	// A1
	if (climate_change_scenario==1)
	{
		change = (year-2000)*(-0.00418*SI_50+0.1135);
	}
	//A2
	else if (climate_change_scenario==2)
	{
		change = (year-2000)*(-0.006*SI_50+0.157);
	}
	//B1 climate_change_scenario==3
	else
	{
		change = (year-2000)*(-0.00257*SI_50+0.069);
	}
	*nres = 1;
    	*modelresult = SI_50+change;
    	return ret;

}

int Mean_volume_increase_m3_stratum_JKK (double V, double V_old, double i_V_1, double i_V_2, double i_V_3, double i_V_4, double i_V_5, double timeStep,
					int *nres, struct Mean_growth *mean_growth, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate absolute growth
	double i_V;
		
	//Check for fatal errors
	char errorStr[200] = "";
	if (V_old==0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "V_old==0(V_old=",V_old);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	*nres = 1;
	
	i_V = V - V_old;
	if (timeStep > 1)
	{
              	i_V = i_V / timeStep;
        }

	//Assign old i_V value to value of i_V#_old variable
	double i_V_1_old, i_V_2_old, i_V_3_old, i_V_4_old, i_V_5_old;
	i_V_1_old = i_V_1;
	i_V_2_old = i_V_2;
	i_V_3_old = i_V_3;
	i_V_4_old = i_V_4;
	i_V_5_old = i_V_5;

	//Move values when a new value have been calculated if value is not 0
    if (i_V != 0)
    {
        i_V_1 = i_V;
        i_V_2 = i_V_1_old;
        i_V_3 = i_V_2_old;
        i_V_4 = i_V_3_old;
        i_V_5 = i_V_4_old;
    }

    //Calculate mean_i_V if 5 values exists
    double sum_i_V, mean_i_V;
    if (i_V_1>-999&&i_V_2>-999&&i_V_3>-999&&i_V_4>-999&&i_V_5>-999)
    {
            if(timeStep==1)
            {
		    sum_i_V = i_V_1+i_V_2+i_V_3+i_V_4+i_V_5;
		    mean_i_V = sum_i_V/5;
   	    }
   	    else if (timeStep==2)
   	    {
		    sum_i_V = i_V_1+i_V_2+i_V_3;
		    mean_i_V = sum_i_V/3;
   	    }
   	    else if (timeStep==3)
   	    {
		    sum_i_V = i_V_1+i_V_2;
		    mean_i_V = sum_i_V/2;
   	    }
   	    else if (timeStep>=4)
   	    {
		    mean_i_V = i_V_1;
   	    }
    }
    else if (i_V_1>-999&&i_V_2>-999&&i_V_3>-999&&i_V_4>-999)
    {
	if (timeStep==1)
        {		
		sum_i_V = i_V_1+i_V_2+i_V_3+i_V_4;
	        mean_i_V = sum_i_V/4;
	}
	else if (timeStep==2)
        {		
		sum_i_V = i_V_1+i_V_2+i_V_3;
	        mean_i_V = sum_i_V/3;
	}
	else if (timeStep==3)
   	{
	        sum_i_V = i_V_1+i_V_2;
		mean_i_V = sum_i_V/2;
   	}
   	else if (timeStep>=4)
   	{
	        mean_i_V = i_V_1;
   	}
    }
    else if (i_V_1>-999&&i_V_2>-999&&i_V_3>-999)
    {
    	if (timeStep<=2)
        {	
		sum_i_V = i_V_1+i_V_2+i_V_3;
	    	mean_i_V = sum_i_V/3;
	}
	else if (timeStep==3)
   	{
	        sum_i_V = i_V_1+i_V_2;
		mean_i_V = sum_i_V/2;
   	}
   	else if (timeStep>=4)
   	{
	        mean_i_V = i_V_1;
	}
    }
    else if (i_V_1>-999&&i_V_2>-999)
    {
    	if (timeStep<=3)
        {
		sum_i_V = i_V_1+i_V_2;
	    	mean_i_V = sum_i_V/2;
	}
	else if (timeStep>=4)
   	{
	        mean_i_V = i_V_1;
	}
    }
    else if (i_V_1>-999)
    {
        	mean_i_V = i_V_1;
    }
    else
    {
        	mean_i_V = i_V;
    }
	
	//Store the new values
	mean_growth[0].mean_i_Vm3 = mean_i_V;
	mean_growth[0].i_Vm3_1 = i_V_1;
	mean_growth[0].i_Vm3_2 = i_V_2;
	mean_growth[0].i_Vm3_3 = i_V_3;
	mean_growth[0].i_Vm3_4 = i_V_4;
	mean_growth[0].i_Vm3_5 = i_V_5;

	*nres = 1;
	return ret;

}