// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  TreegrowthmodelLibrary.cpp
//
// 	Tree level growth Models - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "TreegrowthmodelLibrary.h"
#include "SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


int Height_growth_pine_Hynynenym (double d, double D_domvmi, double i_Hdom5, double cr, double CR_domvmi, double RDF_L,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (D_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_domvmi<=0(D_domvmi=",D_domvmi);
	}
	if (CR_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "CR_domvmi<=0(CR_domvmi=",CR_domvmi);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid problems caused by very little diameter minimum diameter for pine is 0.1 cm!
	if (d<0.1)
		d=0.1;

	*nres = 1;
	double res =i_Hdom5*pow((d/D_domvmi),(0.80171*pow(i_Hdom5,0.4353)-0.49724*(cr/CR_domvmi)-0.64383*cr+0.08193*RDF_L));


	*modelresult = res * rectFactor;
	return ret;

}


int Height_growth_spruce_Hynynenym (double d, double D_domvmi, double i_Hdom5, double cr, double CR_domvmi, double RDF_L,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (D_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_domvmi<=0(D_domvmi=",D_domvmi);
	}
	if (CR_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "CR_domvmi<=0(CR_domvmi=",CR_domvmi);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little diameter minimum diameter for spruce is 1.1 cm!
	if (d<0.1)
		d=0.1;
	
	*nres = 1;
	double res =i_Hdom5*pow((d/D_domvmi),(1.78842*pow(i_Hdom5,0.2544)-0.38878*(cr/CR_domvmi)-1.96189*cr+0.53034*RDF_L));


	*modelresult = res * rectFactor;
	return ret;

}


int Height_growth_birch_Hynynenym (double d, double D_domvmi, double i_Hdom5, double PLANT, double RDF_L,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_domvmi<=0(D_domvmi=",D_domvmi);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little diameter minimum diameter for birch is 1.1 cm!
	if (d<0.1)
		d=0.1;

	*nres = 1;
	double res =i_Hdom5*pow((d/D_domvmi),((1.54405-0.81202*PLANT)*RDF_L));

	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_pine_Hynynenym (double d, double D_gM, double BA, double LAT, double SC, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;

	double varcor;
	//SQ24 is categorical variable referring to middle nutrious (mesotrophic, oligo-mesotrophic or oligotrophic) site
	int SQ24;

	if(SC>=2&&SC<=4)
		SQ24=1;
	else
		SQ24=0;

	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<3.5) {
		if(BA<2.7) {
			BA=2.7;
			D_gM=3.5;
		}
	}
	if(d<=3&&BA>8)
		BA=8;
	if(d>3&&BA>(8+(d-3)*4))
		BA=8+(d-3)*4;
	if(BA>36.5)
		BA=36.5;

	//Variance correction:
	//If maximum diameter is less than 9 cm, then d=9:

	if(d<=9) {
		varcor=0.021764931;
	} else {
		//else variance correction is calculated using maximum diameter:
		varcor=((pow(0.155145,2)+2*0.155145*0.6554*0.07345*(((pow(10,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)+pow(0.07345,2)*
				pow(((((pow(10,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)),2))+(pow(0.071937,2)+pow(1.389244,2)*(pow(d,-2))))/2;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(exp(5.369+0.3566*log(D_gM)+0.1177*log(BA)-0.02638*(LAT/100.0)+0.05115*THIN_0_5+0.0984*SQ24+0.3748*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1+
		0.141*((log(D_gM))*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)-0.01263*((BA)*(((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1))+varcor)+13)/10;


	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_pine_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double SC, double TS,
									double ALT, double SEA, double SCRUB, double UNDRAINED, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 && SINCE_THINNING <=5)
		THIN_0_5 = 1;

	double varcor;
	//SQ24 is categorical variable referring to middle nutrious (mesotrophic, oligo-mesotrophic or oligotrophic) site
	int SQ24;

	if(SC>=2&&SC<=4)
		SQ24=1;
	else
		SQ24=0;

	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<3.5) {
		if(BA<2.7) {
			BA=2.7;
			D_gM=3.5;
		}
	}
	if(d<=3&&BA>8)
		BA=8;
	if(d>3&&BA>(8+(d-3)*4))
		BA=8+(d-3)*4;
	if(BA>36.5)
		BA=36.5;

	int SC2, SC3, SC5, SC6;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC2 = 0;
				SC3 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 2:
				SC2 = 1;
				SC3 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 3:
				SC2 = 0;
				SC3 = 1;
				SC5 = 0;
				SC6 = 0;
				break;
		case 4:
				SC2 = 0;
				SC3 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 5:
				SC2 = 0;
				SC3 = 0;
				SC5 = 1;
				SC6 = 0;
				break;
		case 6:
				SC2 = 0;
				SC3 = 0;
				SC5 = 0;
				SC6 = 1;
				break;
	};

	//Variance correction:
	//If maximum diameter is less than 9 cm, then d=9:

	if(d<=9) {
		varcor=0.021764931;
	} else {
		//else variance correction is calculated using maximum diameter:
		varcor=((pow(0.155145,2)+2*0.155145*0.6554*0.07345*(((pow(10,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)+pow(0.07345,2)*
				pow(((((pow(10,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)),2))+(pow(0.071937,2)+pow(1.389244,2)*(pow(d,-2))))/2;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Bias correction model:
	double bias = -0.08262+0.70792*(TS/1000.0)-0.45964*pow(TS/1000.0,2)-0.00043*ALT-0.47783*SEA
			-0.02611*SC2-0.01712*SC3+0.0273*SC5-0.0466*SC6-0.02487*SCRUB+0.04334*UNDRAINED;

	//Variance correction term:
	double c = 1.00458;

	*nres = 1;
	double res =(c*((exp(5.369+0.3566*log(D_gM)+0.1177*log(BA)-0.02638*(LAT/100.0)+0.05115*THIN_0_5+0.0984*SQ24+0.3748*((pow(d,-0.4)-
		pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1+0.141*((log(D_gM))*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)-
		0.01263*((BA)*(((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*(-1)))+varcor)+13)/10)*exp(bias))-1;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_spruce_Hynynenym (double d, double D_gM, double BA, double LAT,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;

	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<2)
		D_gM=2;
	if(d<1) {
		if(D_gM<3) {
			D_gM=3;
		}
		if(BA<3) {
			BA=3;
		}
	}
	if(BA>45.1)
		BA=45.1;

	double varcor;

	//Variance correction:
	varcor=0.015391832;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(exp(5.789+0.1889*log(D_gM)+0.1665*log(BA)-0.02381*(LAT/100.0)+0.4502*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*(-1)+
		0.1622*((log(D_gM))*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)+varcor)+13)/10;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_spruce_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double TS, double ALT, double LAKE, double SC, double SCRUB,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<2)
		D_gM=2;
	if(d<1) {
		if(D_gM<3) {
			D_gM=3;
		}
		if(BA<3) {
			BA=3;
		}
	}
	if(BA>45.1)
		BA=45.1;

	int SC4, SC5;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC4 = 0;
				SC5 = 0;
				break;
		case 2:
				SC4 = 0;
				SC5 = 0;
				break;
		case 3:
				SC4 = 0;
				SC5 = 0;
				break;
		case 4:
				SC4 = 1;
				SC5 = 0;
				break;
		case 5:
				SC4 = 0;
				SC5 = 1;
				break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
				SC4 = 0;
				SC5 = 1;
				break;
	};

	double varcor;
		
	//Variance correction:
	varcor=0.015391832;

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Bias correction model:
	double d30;
	if(d<=30)
		d30 = d;
	if(d>30)
		d30 = 30;
	double bias = -0.25968+0.00375*d30-0.00031*pow(d30,2)+0.81172*(TS/1000.0)-0.37828*pow(TS/1000.0,2)-0.00044*ALT+0.16576*LAKE-0.02569*SC4-0.05291*SC5+0.03949*SCRUB;

	//Variance correction term:
	double c = 1.0067457;

	*nres = 1;
	double res =(c*((exp(5.789+0.1889*log(D_gM)+0.1665*log(BA)-0.02381*(LAT/100.0)+0.4502*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1+
		0.1622*((log(D_gM))*((pow(d,-0.4)-pow(30,-0.4))/(pow(10,-0.4)-pow(30,-0.4)))*-1)+varcor)+13)/10)*exp(bias))-1;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_birch_Hynynenym (double d, double D_gM, double BA, double LAT, double ALT, double BA_prosb, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;


	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<5)
		D_gM=5;
	if(d<=3) {
		if(BA>5) {
			BA=5;
		}
	}
	if(d>3) {
		if(BA>(5+(d-3)*5)) {
			BA=5+(d-3)*5;
		}
	}
	if(BA>45)
		BA=45;

	double varcor;

	//Variance correction:
	//If maximum diameter is less than 9 cm, then d=9:

	if(d<=9) {
		varcor=0.015597043;
	} else {
		//else variance correction is calculated using maximum diameter:
		varcor=((pow(0.133116,2)+2*0.133116*0.579*0.004265*(((pow(10,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)+
			pow(0.004265,2)*pow(((((pow(10,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)),2))+(pow(0.059958,2)+pow(0.923038,2)*(pow(d,-2))))/2;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = (exp(6.465+0.4743*log(D_gM)+0.1339*log(BA_prosb/100.0+1)-0.0433*(LAT/100.0)-0.000436*ALT+0.06447*THIN_0_5-0.05942*((pow(d,-0.7)-
		pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1+0.2123*((log(D_gM))*((pow(d,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)-
		0.004922*((BA)*(((pow(d,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1))+varcor)+13)/10;


	*modelresult = res * rectFactor;
	return ret;

}


int Height_peatland_birch_with_bias_corr_Hynynenym (double d, double D_gM, double BA, double LAT, double ALT, double BA_prosb,
									double TS, double LAKE, double SC, double SCRUB, double UNDRAINED, double SP, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 && SINCE_THINNING <=5)
		THIN_0_5 = 1;


	//To avoid extrapolation: Minimum and maximum values
	if(d<0.1)
		d=0.1;
	if(D_gM<5)
		D_gM=5;
	if(d<=3) {
		if(BA>5) {
			BA=5;
		}
	}
	if(d>3) {
		if(BA>(5+(d-3)*5)) {
			BA=5+(d-3)*5;
		}
	}
	if(BA>45)
		BA=45;

	int SC1, SC2, SC4, SC5;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC1 = 1;
				SC2 = 0;
				SC4 = 0;
				SC5 = 0;
				break;
		case 2:
				SC1 = 0;
				SC2 = 1;
				SC4 = 0;
				SC5 = 0;
				break;
		case 3:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				SC5 = 0;
				break;
		case 4:
				SC1 = 0;
				SC2 = 0;
				SC4 = 1;
				SC5 = 0;
				break;
		case 5:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				SC5 = 1;
				break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				SC5 = 1;
				break;
	};

	double varcor;

	//Variance correction:
	//If maximum diameter is less than 9 cm, then d=9:

	if(d<=9) {
		varcor=0.015597043;
	} else {
		//else variance correction is calculated using maximum diameter:
		varcor=((pow(0.133116,2)+2*0.133116*0.579*0.004265*(((pow(10,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)+
			pow(0.004265,2)*pow(((((pow(10,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)),2))+(pow(0.059958,2)+pow(0.923038,2)*(pow(d,-2))))/2;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Bias and variance correction models:
	double bias, c;
	if(SP==4) {
		double d20;
		if(d<=20)
			d20 = d;
		if(d>20)
			d20 = 20;
		bias = 0.29346-0.07106*d20+0.00251*pow(d20,2)+0.12696*(TS/1000.0)-0.07549*SC1+0.02106*SC2-0.03481*SC4-0.18407*SC5-0.07961*SCRUB+0.06948*UNDRAINED;

		//Variance correction term:
		c = 1.0041588;
	}
	if((SP==3||SP>=5)) {
		int sp3, sp5, sp6, sp7, sp89;
		int isp = (int) SP;
		switch(isp)
		{
			case 3:
				sp3 = 1;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 4:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 5:
				sp3 = 0;
				sp5 = 1;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 6:
				sp3 = 0;
				sp5 = 0;
				sp6 = 1;
				sp7 = 0;
				sp89 = 0;
				break;
			case 7:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 1;
				sp89 = 0;
				break;
			case 8:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			case 9:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			// defaults to sp==4, sensible?
			default:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
		};
		double d20;
		if(d<=20)
			d20 = d;
		if(d>20)
			d20 = 20;
		bias = -0.12688-0.07132*d20+0.00246*pow(d20,2)+0.97358*(TS/1000.0)-0.40511*pow(TS/1000.0,2)+0.11083*LAKE+0.02745*SC2-0.03001*SC4-0.16052*SC5
				-0.11331*SCRUB+0.03865*UNDRAINED+0.08913*sp3+0.08222*sp5-0.1107*sp6-0.3171*sp7+0.07686*sp89;

		//Variance correction term:
		c = 1.00384;
	}

	*nres = 1;
	double res = (c*((exp(6.465+0.4743*log(D_gM)+0.1339*log(BA_prosb/100.0+1)-0.0433*(LAT/100.0)-0.000436*ALT+0.06447*THIN_0_5-0.05942*((pow(d,-0.7)-
		pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1+0.2123*((log(D_gM))*((pow(d,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1)-
		0.004922*((BA)*(((pow(d,-0.7)-pow(30,-0.7))/(pow(10,-0.7)-pow(30,-0.7)))*-1))+varcor)+13)/10)*exp(bias))-1;


	*modelresult = res * rectFactor;
	return ret;

}



int Basal_area_growth_pine_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns,
									double RDF_decid, double TS, double H_domvmi, double SC, double h, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5, THIN_5_10;
	THIN_0_5 = 0;
	THIN_5_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;
	if(SINCE_THINNING > 5 &&SINCE_THINNING <=10)
		THIN_5_10 = 1;


	int SC1_2, SC3, SC5_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF_Sp<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Sp<=0(RDF_Sp=",RDF_Sp);
	}
	if (RDF_Ns<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Ns<=0(RDF_Ns=",RDF_Ns);
	}
	if (RDF_decid<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_decid<=0(RDF_decid=",RDF_decid);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for pine is 1.0 cm and max 29 cm! (Hynynen ym.)
	if (d<1)
		d=1;
	if (d>29)
		d=29;
	if (H_domvmi<4)
		H_domvmi=4;

	*nres = 1;
	double res = exp(-2.74332+0.75638*log(SI_50)+0.11721*SC1_2+0.05213*SC3-0.13549*SC5_8+6.46997*(1/H_domvmi)-4.22774*(1/pow(H_domvmi,2))+
		1.57123*log(d)-0.00088*pow(d,2)+1.09423*(1/(d+0.1)+0.56858*log(cr)-0.40647*RDF_L-0.89384*log(RDF_Sp+1)-0.97311*log(RDF_Ns+1)-
		0.94614*log(RDF_decid+1)+0.38051*((cr*TS)/1000.0)+0.11976*THIN_0_5+0.0806*THIN_5_10)+(pow(0.2907,2)+pow(0.44,2))/2);

	if(h < 5) {
		if(res < 15) {
			res = -3 + 0.015 * TS;
		}
	}
	
	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_pine_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, 
									double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h,
									double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5, THIN_5_10;
	THIN_0_5 = 0;
	THIN_5_10 = 0;
	if(SINCE_THINNING >= 0 && SINCE_THINNING <=5)
		THIN_0_5 = 1;
	if(SINCE_THINNING > 5 && SINCE_THINNING <=10)
		THIN_5_10 = 1;


	int SC1_2, SC3, SC5_8, SC1_b, SC2_b, SC3_b, SC5_b, SC7_b, SC8_b;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			SC1_b = 1;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			SC1_b = 0;
			SC2_b = 1;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 1;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 1;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 1;
			SC8_b = 0;
			break;
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC1_b = 0;
			SC2_b = 0;
			SC3_b = 0;
			SC5_b = 0;
			SC7_b = 0;
			SC8_b = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF_Sp<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Sp<=0(RDF_Sp=",RDF_Sp);
	}
	if (RDF_Ns<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Ns<=0(RDF_Ns=",RDF_Ns);
	}
	if (RDF_decid<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_decid<=0(RDF_decid=",RDF_decid);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Very low productive land
	if(WASTE==1)
		SCRUB=1;

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for pine is 1.0 cm and max 29 cm! (Hynynen ym.)
	if (d<1)
		d=1;
	if (d>29)
		d=29;
	if (H_domvmi<4)
		H_domvmi=4;

	//To avoid extrapolation
	if(TS>1340)
		TS=1340;

	//Model for bias correction. 
	//Original value of parameter was +0.31621 for variable SC8. To ensure logic of the model it was changed to -0.3.
	//Original value of parameter was -0.14729 for variable SCRUB. To ensure logic of the model it was changed to -0.5.
	//Bias correction model:
	double bias = -1.88386+1.6542*TS/1000.0-0.28099*pow(TS/1000.0,2)+0.00145*ALT+0.1818*SEA+0.0004*SC1_b*pow(d,2)
				+0.00043*SC2_b*pow(d,2)+0.0521*SC3_b+0.08097*SC5_b-0.20553*SC7_b-0.3*SC8_b-0.5*SCRUB;

	//Variance correction:
	double c;
	if(d<5)
		c=1.36562;
	if((d>=5&&d<10))
		c=1.11041;
	if((d>=10&&d<20))
		c=1.04251;
	if((d>=20&&d<30))
		c=1.01429;
	if(d>=30)
		c=1.1516;

	*nres = 1;
	//Without calibration
	double res1 =exp(-2.74332+0.75638*log(SI_50)+0.11721*SC1_2+0.05213*SC3-0.13549*SC5_8+6.46997*(1/H_domvmi)-4.22774*(1/pow(H_domvmi,2))+
		1.57123*log(d)-0.00088*pow(d,2)+1.09423*(1/(d+0.1)+0.56858*log(cr)-0.40647*RDF_L-0.89384*log(RDF_Sp+1)-0.97311*log(RDF_Ns+1)-
		0.94614*log(RDF_decid+1)+0.38051*((cr*TS)/1000.0)+0.11976*THIN_0_5+0.0806*THIN_5_10)+(pow(0.2907,2)+pow(0.44,2))/2);
	//With calibration
	double res2 =(c*(exp(-2.74332+0.75638*log(SI_50)+0.11721*SC1_2+0.05213*SC3-0.13549*SC5_8+6.46997*(1/H_domvmi)-4.22774*(1/pow(H_domvmi,2))+
		1.57123*log(d)-0.00088*pow(d,2)+1.09423*(1/(d+0.1)+0.56858*log(cr)-0.40647*RDF_L-0.89384*log(RDF_Sp+1)-0.97311*log(RDF_Ns+1)-
		0.94614*log(RDF_decid+1)+0.38051*((cr*TS)/1000.0)+0.11976*THIN_0_5+0.0806*THIN_5_10)+(pow(0.2907,2)+pow(0.44,2))/2)+1)*exp(bias))-1;

	double res = res2;
	//Difference of calibrated and not calibrated predictions:
	//If difference is too big calibration will not be used!
	double diff = (res2/res1)*100;
	if(diff<50)
		res = 0.5*res1;
	if(diff>150)
		res = 1.5*res1;
	//Growth for little tree
	if(d<2)
		res=res1;
	if(d>=1&&h<5) {   
		if(res<15) {
			res=-3+0.015*TS;
		}
	}
	if(WASTE==1)
		res=0.5*res;
	
	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_spruce_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, double RDF_Ns,
									double RDF_decid, double TS, double H_domvmi, double SC, double h, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;


	int SC1, SC2, SC4_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF_Sp<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Sp<0(RDF_Sp=",RDF_Sp);
	}
	if (RDF_Ns<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Ns<0(RDF_Ns=",RDF_Ns);
	}
	if (RDF_decid<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_decid<0(RDF_decid=",RDF_decid);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}


	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for spruce is 1.1 cm and max 41 cm! (Hynynen ym.)
	if (d<1.1)
		d=1.1;
	if (d>41)
		d=41;
	if (H_domvmi<4)
		H_domvmi=4;

	*nres = 1;
	double res =exp(-5.08998+0.66849*log(SI_50)+0.14526*(SC1)+0.11475*SC2-0.10130*SC4_8+7.10363*(1/log(H_domvmi))-25.3565*(1/pow(H_domvmi,2))+
		0.61162*log(d)-0.00063*pow(d,2)+0.19257*pow(log(d),2)-0.35418*log(cr)-0.42997*RDF_L-0.67742*log(RDF_Sp+1)-0.78516*log(RDF_Ns+1)-
		0.30546*log(RDF_decid+1)+1.71107*((cr*TS)/1000.0)+0.11804*THIN_0_5+(pow(0.2936,2)+pow(0.4709,2))/2);

	if(h<5) {
		if(res<15) {
			res=-3+0.015*TS;
		}
	}

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_spruce_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF_Sp, 
									double RDF_Ns, double RDF_decid, double TS, double H_domvmi, double SC, double h, 
									double ALT, double SCRUB, double WASTE, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 && SINCE_THINNING <=5)
		THIN_0_5 = 1;


	int SC1, SC2, SC4_8, SC2_b, SC48_b;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			SC2_b = 0;
			SC48_b = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			SC2_b = 1;
			SC48_b = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC2_b = 0;
			SC48_b = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC2_b = 0;
			SC48_b = 1;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC2_b = 0;
			SC48_b = 1;
			break;
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC2_b = 0;
			SC48_b = 1;
			break;
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC2_b = 0;
			SC48_b = 1;
			break;
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC2_b = 0;
			SC48_b = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC2_b = 0;
			SC48_b = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF_Sp<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Sp<0(RDF_Sp=",RDF_Sp);
	}
	if (RDF_Ns<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_Ns<0(RDF_Ns=",RDF_Ns);
	}
	if (RDF_decid<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_decid<0(RDF_decid=",RDF_decid);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	if(WASTE==1)
		SCRUB=1;

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for spruce is 1.1 cm and max 41 cm! (Hynynen ym.)
	if (d<1.1)
		d=1.1;
	if (d>41)
		d=41;
	if (H_domvmi<4)
		H_domvmi=4;
	
	//To avoid extrapolation
	if(TS>1300)
		TS=1300;

	//Bias correction model:
	double bias = -1.22666+0.7629*(TS/1000.0)+0.00155*ALT+0.00288*d*SC2_b+0.19087*(1/d)*SC48_b-0.52581*SCRUB;

	//Variance correction:
	double c;
	if(d<5)
		c=1.13425;
	if(d>=5&&d<10)
		c=1.17624;
	if(d>=10&&d<20)
		c=1.10393;
	if(d>=20&&d<30)
		c=1.03971;
	if(d>=30&&d<35)
		c=1.08988;
	if(d>=35)
		c=1.24713;

	*nres = 1;
	//Without calibration
	double res1 =exp(-5.08998+0.66849*log(SI_50)+0.14526*(SC1)+0.11475*SC2-0.10130*SC4_8+7.10363*(1/log(H_domvmi))-25.3565*(1/pow(H_domvmi,2))+
		0.61162*log(d)-0.00063*pow(d,2)+0.19257*pow(log(d),2)-0.35418*log(cr)-0.42997*RDF_L-0.67742*log(RDF_Sp+1)-0.78516*log(RDF_Ns+1)-
		0.30546*log(RDF_decid+1)+1.71107*((cr*TS)/1000.0)+0.11804*THIN_0_5+(pow(0.2936,2)+pow(0.4709,2))/2);
	//With calibration
	double res2 =(c*(exp(-5.08998+0.66849*log(SI_50)+0.14526*(SC1)+0.11475*SC2-0.10130*SC4_8+7.10363*(1/log(H_domvmi))-25.3565*(1/pow(H_domvmi,2))+
		0.61162*log(d)-0.00063*pow(d,2)+0.19257*pow(log(d),2)-0.35418*log(cr)-0.42997*RDF_L-0.67742*log(RDF_Sp+1)-0.78516*log(RDF_Ns+1)-
		0.30546*log(RDF_decid+1)+1.71107*((cr*TS)/1000.0)+0.11804*THIN_0_5+(pow(0.2936,2)+pow(0.4709,2))/2)+1)*exp(bias))-1;

	double res = res2;
	//Difference of calibrated and not calibrated pedictions:
	//If difference is too big calibration will not be used!
	double diff = (res2/res1)*100;
	if(diff<50)
		res = 0.5*res1;
	if(diff>150)
		res = 1.5*res1;
	//Growth for little tree
	if(d<2)
		res=res1;
	if(d>1&&h<5) {
		if(res<15) {
			res=-3+0.015*TS;
		}
	}

	if(WASTE==1)
		res=0.5*res;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_birch_Hynynenym (double d, double cr, double SI_50, double RDF_L, double RDF, double TS,
									double H_domvmi, double PLANT, double SP, double h, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=10)
		THIN_0_10 = 1;


	int PUB;
	PUB = 0;
	if(SP==4) 
		PUB=1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for birch is 1.1 cm and max 40 cm! (Hynynen ym.)
	if (d<1.1)
		d=1.1;
	if (d>40)
		d=40;
	if (H_domvmi<4)
		H_domvmi=4;

	*nres = 1;
	double res =exp(-3.98207+0.76027*log(SI_50)+14.97845*(1/H_domvmi)-32.1554*(1/pow(H_domvmi,2))+1.314906*log(d+1)-0.00053*pow(d,2)+
		0.37444*log(cr)-0.96221*RDF_L-0.59253*log(RDF+1)+1.16996*(TS/1000.0)+0.36701*PLANT+0.19411*THIN_0_10-0.16571*PUB+(pow(0.33827,2)+pow(0.50537,2))/2);

	if(h<5) {
		if(res<15) {
			res=-3+0.015*TS;
		}
	}

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_birch_with_bias_corr_Hynynenym (double d, double cr, double SI_50, double RDF_L, 
									double RDF, double TS, double H_domvmi, double PLANT,
									double SP, double h, double SC,	double LAKE, double WASTE, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning dummy variables

	double THIN_0_10 = 0;
	if(SINCE_THINNING >= 0 && SINCE_THINNING <=10)
		THIN_0_10 = 1;


	int PUB = 0;
	if(SP == 4) 
		PUB = 1;

	int SC1, SC2, SC4, SC5, SC6, SC7, SC8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 1;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 1;
			SC7 = 0;
			SC8 = 0;
			break;
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 1;
			SC8 = 0;
			break;
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
	};


	//Check for fatal errors
	char errorStr[200] = "";
	if (SI_50<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "SI_50<=0(SI_50=",SI_50);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (cr<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "cr<=0(cr=",cr);
	}
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for birch is 1.1 cm and max 40 cm! (Hynynen ym.)
	if (d < 1.1)
		d = 1.1;
	if (d > 40)
		d = 40;
	if (H_domvmi < 4)
		H_domvmi = 4;

	//To avoid extrapolation
	if(TS>1350)
		TS=1350;

	//Bias and variance correction models by tree species:
	double bias, c;
	if(SP == 3){
		//Model for bias correction. 
		//Original value of parameter was none for variable SC5. To ensure logic of the model it was changed to -0.15.
		//Original value of parameter was none for variable SC6. To ensure logic of the model it was changed to -0.15.
		//Original value of parameter was none for variable SC8. To ensure logic of the model it was changed to -0.20.
		bias = -1.97241 + 0.3267 * (TS / 1000) + 0.11402 * H_domvmi -
				0.00196 * pow(H_domvmi, 2) + 0.4816 * SC1 + 0.36118 *
				SC2 - 0.1158 * SC4 -0.15 * SC5 -0.15 * SC6 - 0.17282 * SC7 - 0.2 * SC8;
		if(d < 5)
			c = 1.26255;
		if(d >= 5 && d < 10)
			c = 1.15526;
		if(d >= 10 && d < 15)
			c = 1.11487;
		if(d >= 15 && d < 20)
			c = 1.05096;
		if(d >= 20 && d < 25)
			c = 1.03475;
		if(d >= 25 && d < 30)
			c = 1.05407;
		if(d >= 30 && d < 35)
			c = 1.17978;
		if(d >= 35)
			c = 1.37698; 
	}
	if(SP == 4){
		//Model for bias correction. 
		//Original value of parameter was none for variable SC6. To ensure logic of the model it was changed to -0.45.
		//Original value of parameter was + 0.10357 for variable SC8. To ensure logic of the model it was changed to -0.20.
		bias = 0.27428 + 0.07849 * H_domvmi - 0.0012 * pow(H_domvmi, 2) -
				2.67933 * (TS / 1000) + 1.15406 * pow(TS / 1000, 2) + 0.43532 *
				SC1 + 0.11509 * SC2 - 0.40626 * SC5 -0.45 * SC6 - 0.15671 * SC7 -0.2 * 
				SC8 + 0.45273 * LAKE;
		if(d < 5)
			c = 1.15147;
		if(d >= 5 && d < 10)
			c = 1.22573;
		if(d >= 10 && d < 15)
			c = 1.13414;
		if(d >= 15 && d < 20)
			c = 0.9738;
		if(d >= 20 && d < 25)
			c = 0.94264;
		if(d >= 25)
			c = 1.07059; 
	}
	if(SP >= 5){

		int sp5, sp6, sp7, sp89;
		int isp = (int) SP;
		switch(isp)
		{
			case 5:
				sp5 = 1;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 6:
				sp5 = 0;
				sp6 = 1;
				sp7 = 0;
				sp89 = 0;
				break;
			case 7:
				sp5 = 0;
				sp6 = 0;
				sp7 = 1;
				sp89 = 0;
				break;
			case 8:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			case 9:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			// defaults to sp==3 or 4, sensible?
			default:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
		};

		//Model for bias correction. 
		//Original value of parameter was none for variable SC6. To ensure logic of the model it was changed to -0.15.
		bias =	0.27428 + 0.03468 * H_domvmi - 2.38949 * (TS / 1000) + 1.02036 * 
				pow(TS / 1000, 2) + 0.43532 * SC1 + 0.11509 * SC2 - 0.40626 * SC5 -0.15 * SC6 -
				0.15671 * SC7 - 0.10357 * SC8 + 0.2918 * sp5 + 0.20955 * sp6 + 
				0.01914 * sp7 + 0.37152 * sp89;
		if(d < 5)
			c = 1.15147;
		else if(d >= 5 && d < 10)
			c = 1.22573;
		else if(d >= 10 && d < 15)
			c = 1.13414;
		else if(d >= 15 && d < 20)
			c = 0.9738;
		else if(d >= 20 && d < 25)
			c = 0.94264;
		else if(d >= 25)
			c = 1.07059; 
	}


	*nres = 1;
	//Without calibration
	double res1 = exp(-3.98207 + 0.76027 * log(SI_50) + 14.97845 * (1 / H_domvmi) -
				32.1554 * (1 / pow(H_domvmi, 2)) + 1.314906 * log(d + 1) - 0.00053 * pow(d, 2) +
				0.37444 * log(cr)- 0.96221 * RDF_L - 0.59253 * log(RDF + 1) + 1.16996 * (TS / 1000) +
				0.36701 * PLANT + 0.19411 * THIN_0_10 - 0.16571 * PUB + (pow(0.33827, 2) + pow(0.50537,2)) / 2);
	 
	//With calibration
	double res2 = ((exp(-3.98207 + 0.76027 * log(SI_50) + 14.97845 * (1 / H_domvmi) - 32.1554 *
				(1 / pow(H_domvmi, 2)) + 1.314906 * log(d + 1) - 0.00053 * pow(d, 2) +
				0.37444 * log(cr) - 0.96221 * RDF_L - 0.59253 * log(RDF + 1) + 1.16996 * 
				(TS / 1000) + 0.36701 * PLANT + 0.19411 * THIN_0_10 - 0.16571 * PUB +
				(pow(0.33827, 2) + pow(0.50537, 2)) / 2) + 1) * exp(bias)) -1;

	double res = res2;
	//Difference of calibrated and not calibrated pedictions:
	//If difference is too big calibration will not be used!
	double diff = (res2 / res1)*100;
	if(diff < 50)
		res = 0.5 * res1;
	if(diff > 150)
		res = 1.5 * res1;
	//Growth for little tree
	if(d < 2)
		res = res1;
	if(d>1&&h<5) {
		if(res < 15) {
			res = -3 + 0.015 * TS;
		}
	}

	if(WASTE == 1)
		res = 0.5 * res;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_pine_Hynynenym (double d, double BA, double BA_L, double SC, double TS, double PDR,
									double FUSC, double SUBGROUP, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_11_25;
	DR_0_5 = 0;
	DR_11_25 = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 10 &&SINCE_DRAINAGE <= 25)
		DR_11_25 = 1;


	int PDR_d, Y1, Y2, Y3, Y4, Y24;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
		Y24 = 0;
		Y4 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 3:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
		};
	}

	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (TS*sqrt(d)<=0)
	{
		ret = 0;
		constructErrorMessage2val (errorStr, "TS*sqrt(d)<=0(TS=",TS, ",d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for pine is 1.0 cm and max 60 cm! (Hynynen ym.)
	if (d<1)
		d=1;
	if (d>60)
		d=60;

	*nres = 1;
	double res =exp(-1.217101-0.00187*PI/4*pow(d,2)-0.00892*BA_L-0.00153*pow(BA_L,2)-0.2468*log(BA)+0.06915*sqrt(TS*sqrt(d))-
		0.6115*Y1+0.301425*Y2+0.664339*(Y1*log(d))+0.314616*((Y24)*log(d))+0.388967*(Y3*log(d))-0.23774*DR_0_5+
		0.093963*DR_11_25-0.155569*PDR_d+0.12766*THIN_0_5+0.258215*FUSC+(pow(0.410967,2)+pow(0.576062,2))/2)-1;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr;
	//Southern Finland (forest districts 0-15)
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2)
	{
		if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.881;
		if(Y3==1)
			fert_corr=0.91;
		if(Y4==1)
			fert_corr=0.971;
	}
	else //Northern Finland (forest districts 15-)
	{
		if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.93;
		if(Y3==1)
			fert_corr=0.9434;
		if(Y4==1)
			fert_corr=0.971;
	}

	res=fert_corr*res;

	if(d<5) {
		if(res<6) {
			res=6;
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_pine_with_bias_corr_Hynynenym (double d, double BA, double BA_L, double SC, double TS,
									double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double SCRUB, 
									double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_11_25;
	DR_0_5 = 0;
	DR_11_25 = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 10 &&SINCE_DRAINAGE <= 25)
		DR_11_25 = 1;


	int PDR_d, Y1, Y2, Y3, Y4, Y24, SC1, SC4, SC5, SC6;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
		Y24 = 0;
		Y4 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 3:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
		};
	}

	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC1 = 1;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 2:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 3:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 4:
				SC1 = 0;
				SC4 = 1;
				SC5 = 0;
				SC6 = 0;
				break;
		case 5:
				SC1 = 0;
				SC4 = 0;
				SC5 = 1;
				SC6 = 0;
				break;
		case 6:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 1;
				break;
	};

	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (TS*sqrt(d)<=0)
	{
		ret = 0;
		constructErrorMessage2val (errorStr, "TS*sqrt(d)<=0(TS=",TS, ",d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for pine is 1.0 cm and max 60 cm! (Hynynen ym.)
	if (d<1)
		d=1;
	if (d>60)
		d=60;

	//Bias correction model:
	double bias = -3.89138+5.76914*(TS/1000.0)-2.40611*pow(TS/1000.0,2)+0.00293*ALT+0.82861*SEA
			-0.29352*SC1-0.26141*SC4-0.24866*SC5-0.3955*SC6-0.32354*SCRUB-0.27118*UNDRAINED;

	//Variance correction:
	double c;
	if(d<5)
		c=1.24647;
	if(d>=5&&d<10)
		c=1.11459;
	if(d>=10&&d<20)
		c=1.06898;
	if(d>=20)
		c=1.04918;

	*nres = 1;
	//Without calibration
	double res1 =exp(-1.217101-0.00187*PI/4*pow(d,2)-0.00892*BA_L-0.00153*pow(BA_L,2)-0.2468*log(BA)+0.06915*sqrt(TS*sqrt(d))-
		0.6115*Y1+0.301425*Y2+0.664339*(Y1*log(d))+0.314616*((Y24)*log(d))+0.388967*(Y3*log(d))-0.23774*DR_0_5+
		0.093963*DR_11_25-0.155569*PDR_d+0.12766*THIN_0_5+0.258215*FUSC+(pow(0.410967,2)+pow(0.576062,2))/2)-1;

	//With calibration
	double res2 =(c*((exp(-1.217101-0.00187*PI/4*pow(d,2)-0.00892*BA_L-0.00153*pow(BA_L,2)-0.2468*log(BA)+0.06915*sqrt(TS*sqrt(d))-
		0.6115*Y1+0.301425*Y2+0.664339*(Y1*log(d))+0.314616*((Y24)*log(d))+0.388967*(Y3*log(d))-0.23774*DR_0_5+
		0.093963*DR_11_25-0.155569*PDR_d+0.12766*THIN_0_5+0.258215*FUSC+(pow(0.410967,2)+pow(0.576062,2))/2)-1)+1)*exp(bias))-1;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr;
	//Southern Finland (forest districts 0-15)
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2)
	{	if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.881;
		if(Y3==1)
			fert_corr=0.91;
		if(Y4==1)
			fert_corr=0.971;
	}
	else //Northern Finland (forest districts 15-)
	{
		if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.93;
		if(Y3==1)
			fert_corr=0.9434;
		if(Y4==1)
			fert_corr=0.971;
	}
	res1=fert_corr*res1;
	res2=fert_corr*res2;


	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<6) {
				res=6.0;
			}
		}
		if(WASTE == 1)
		res = 0.5 * res;
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=6.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=6+(((15-d)/5)*(res-6));
			if(d>15)
				res=6.0;	
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_spruce_Hynynenym (double d, double BA_prosNs, double BA_L, double SC, double TS,
									double PDR, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_25_;
	DR_0_5 = 0;
	DR_25_ = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 25)
		DR_25_ = 1;


	int PDR_d, Y1, Y2;
	int iSC = (int) SC;
	switch (iSC)
		{
		case 1:
			Y1 = 1;
			Y2 = 0;
			break;
		case 2:
			Y1 = 1;
			Y2 = 0;
			break;
		case 3:
			Y1 = 1;
			Y2 = 0;
			break;
		case 4:
			Y1 = 0;
			Y2 = 1;
			break;
		case 5:
			Y1 = 0;
			Y2 = 1;
			break;
		case 6:
			Y1 = 0;
			Y2 = 1;
			break;
	};


	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little diameter minimum diameter for spruce is 1.3 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;

	*nres = 1;
	double res =exp(1.223-0.00001747*pow(d,2.9)-0.01503*BA_L-0.00441*(BA_prosNs/100.0)+0.0008156*TS+0.5141*(Y1*sqrt(d))+0.4279*((Y2)*sqrt(d))-
		0.2639*DR_0_5-0.04874*DR_25_-0.06852*PDR_d+0.1621*THIN_0_5+(pow(0.08562,2)+pow(0.1241,2))/2)-7;
	
	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	//Southern Finland (forest districts 0-15)
	double fert_corr;
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2)
		fert_corr=0.9852;
	else //Northern Finland (forest districts 15-)
		fert_corr=0.995;

	res=fert_corr*res;

	if(d<5) {
		if(res<6) {
			res=6;
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;
	
	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_spruce_with_bias_corr_Hynynenym (double d, double BA_prosNs, double BA_L, double SC, double TS,
									double PDR, double H_domvmi, double ALT, double LAKE, double UNDRAINED, 
									double WASTE, double SUBGROUP, double SCRUB, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_25_;
	DR_0_5 = 0;
	DR_25_ = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 25)
		DR_25_ = 1;


	int PDR_d, Y1, Y2, SC1, SC2, SC4, SC5;
	int iSC = (int) SC;
	switch (iSC)
		{
		case 1:
			Y1 = 1;
			Y2 = 0;
			SC1 = 1;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 2:
			Y1 = 1;
			Y2 = 0;
			SC1 = 0;
			SC2 = 1;
			SC4 = 0;
			SC5 = 0;
			break;
		case 3:
			Y1 = 1;
			Y2 = 0;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 4:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			SC5 = 0;
			break;
		case 5:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
	};


	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	//To avoid extrapolation: problems caused by very little diameter minimum diameter for spruce is 1.3 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;

	//Bias correction model:
	double bias = -0.04965+0.29286*log(d)-0.62972*log(H_domvmi)+0.70113*(TS/1000.0)+0.0017*ALT+0.32627*LAKE
				+0.65389*SC1+0.28405*SC2+0.10779*SC4-0.20786*SC5;

	//Variance correction:
	double c = 1.15823;

	*nres = 1;
	//Without calibration
	double res1 =exp(1.223-0.00001747*pow(d,2.9)-0.01503*BA_L-0.00441*(BA_prosNs/100.0)+0.0008156*TS+0.5141*(Y1*sqrt(d))+0.4279*((Y2)*sqrt(d))-
		0.2639*DR_0_5-0.04874*DR_25_-0.06852*PDR_d+0.1621*THIN_0_5+(pow(0.08562,2)+pow(0.1241,2))/2)-7;

	//With calibration
	double res2 =(c*((exp(1.223-0.00001747*pow(d,2.9)-0.01503*BA_L-0.00441*(BA_prosNs/100.0)+0.0008156*TS+0.5141*(Y1*sqrt(d))+0.4279*((Y2)*sqrt(d))-
		0.2639*DR_0_5-0.04874*DR_25_-0.06852*PDR_d+0.1621*THIN_0_5+(pow(0.08562,2)+pow(0.1241,2))/2)-7)+1)*exp(bias))-1;
	
	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	//Southern Finland (forest districts 0-15)
	double fert_corr;
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2) 
		fert_corr=0.9852;
	else //Northern Finland (forest districts 15-)
		fert_corr=0.995;
	res1=fert_corr*res1;
	res2=fert_corr*res2;

	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<6) {
				res=6.0;
			}
		}
		if(SCRUB==1||WASTE==1) {
             res = 0.5 * res;
        }     
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=6.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=6+(((15-d)/5)*(res-6));
			if(d>15)
				res=6.0;	
		}
	}
	
	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_birch_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BA_L, double SC,
									double TS, double PDR, double SEA, double SUBGROUP, double SINCE_THINNING, double SINCE_DRAINAGE,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_11_25;
	DR_0_5 = 0;
	DR_11_25 = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 10 &&SINCE_DRAINAGE <= 25)
		DR_11_25 = 1;


	int PDR_d, Y1, Y2, Y3;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 3:
				Y1 = 1;
				Y2 = 0;
				Y3 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
		};
	}

	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for birch is 1.3 cm and max 45 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;
	if (d>45)
		d=45;

	*nres = 1;
	double res =exp(0.6394-0.00074*pow(d,2)-0.03533*BA_L+0.0005473*TS-0.003167*(BA_prosNs/100.0)-0.003626*(BA_prosb/100.0)+0.005155*SEA+1.016*(Y1*log(d))+
		1.048*(Y2*log(d))+0.9442*(Y3*log(d))-0.1955*DR_0_5+0.06322*DR_11_25-0.1028*PDR_d+0.1777*THIN_0_5+(pow(0.1052,2)+pow(0.2413,2))/2)-3;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr=0.978; 
	
	res=fert_corr*res;

	if(d<5) {
		if(res<4) {
			res=4;
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BA_L, double SC,
									double TS, double PDR, double SEA, double SUBGROUP, double ALT, double SCRUB, double UNDRAINED,
									double SP, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	double DR_0_5, DR_11_25;
	DR_0_5 = 0;
	DR_11_25 = 0;
	if(SINCE_DRAINAGE >= 0 &&SINCE_DRAINAGE <= 5)
		DR_0_5 = 1;
	if(SINCE_DRAINAGE > 10 &&SINCE_DRAINAGE <= 25)
		DR_11_25 = 1;


	int PDR_d, Y1, Y2, Y3, SC1, SC2;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 3:
				Y1 = 1;
				Y2 = 0;
				Y3 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
		};
	}

	if(SC==1){
		SC1=1;
		SC2=0;
	}
	if(SC==2){
		SC1=0;
		SC2=1;
	}
	if((SC!=1&&SC!=2)){
		SC1=0;
		SC2=0;
	}

	if(PDR>=0.5)
		PDR_d=1;
	else
		PDR_d=0;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for birch is 1.3 cm and max 45 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;
	if (d>45)
		d=45;

	double bias, c;

	//Bias and variance correction models:
	if(SP==4) {
		bias = -1.49202+1.11213*(TS/1000.0)+0.00228*ALT+0.18397*SC1+0.09348*SC2-0.28639*SCRUB-0.34648*UNDRAINED;
		if(d<5)
			c=1.15435;
		if(d>=5&&d<10)
			c=1.05338;
		if(d>=10&&d<20)
			c=1.11601;
		if(d>=20)
			c=1.35653;
	}
	if((SP==3||SP>=5)) {

		int sp3, sp5, sp6, sp7, sp89;
		int isp = (int) SP;
		switch(isp)
		{
			case 3:
				sp3 = 1;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 5:
				sp3 = 0;
				sp5 = 1;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 6:
				sp3 = 0;
				sp5 = 0;
				sp6 = 1;
				sp7 = 0;
				sp89 = 0;
				break;
			case 7:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 1;
				sp89 = 0;
				break;
			case 8:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			case 9:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			// defaults to sp==4, sensible?
			default:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
		};

		bias = -2.67024+3.39029*(TS/1000.0)-1.08965*pow(TS/1000.0,2)+0.00231*ALT+0.09608*SC2-0.2579*SCRUB-0.34513*UNDRAINED
			+0.47066*sp3+0.45852*sp5-0.13055*sp6-0.19422*sp7+0.40588*sp89;

		if(d<5)
			c=1.16468;
		if(d>=5&&d<10)
			c=1.05695;
		if(d>=10&&d<20)
			c=1.10804;
		if(d>=20)
			c=1.34524;
	}

	*nres = 1;
	//Without calibration
	double res1 =exp(0.6394-0.00074*pow(d,2)-0.03533*BA_L+0.0005473*TS-0.003167*(BA_prosNs/100.0)-0.003626*(BA_prosb/100.0)+0.005155*SEA+1.016*(Y1*log(d))+
		1.048*(Y2*log(d))+0.9442*(Y3*log(d))-0.1955*DR_0_5+0.06322*DR_11_25-0.1028*PDR_d+0.1777*THIN_0_5+(pow(0.1052,2)+pow(0.2413,2))/2)-3;
	//With calibration
	double res2 =(c*((exp(0.6394-0.00074*pow(d,2)-0.03533*BA_L+0.0005473*TS-0.003167*(BA_prosNs/100.0)-0.003626*(BA_prosb/100.0)+0.005155*SEA+1.016*(Y1*log(d))+
		1.048*(Y2*log(d))+0.9442*(Y3*log(d))-0.1955*DR_0_5+0.06322*DR_11_25-0.1028*PDR_d+0.1777*THIN_0_5+(pow(0.1052,2)+pow(0.2413,2))/2)-3)+1)*exp(bias))-1;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr=0.978; 
	res1=fert_corr*res1;
	res2=fert_corr*res2;


	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<4) {
				res=4.0;
			}
		}
		if (WASTE == 1)
		   res = 0.5 * res;
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=4.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=4+(((15-d)/5)*(res-4));
			if(d>15)
				res=4.0;	
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}

int Grow_ba_peatland_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT,
						double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, 
						double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE,
						double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    
	//Calculate tree basal area growth on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_peatland_pine_with_bias_corr_Hynynenym(d, BA, BA_L, SC, TS, PDR, FUSC, SUBGROUP, ALT, SEA, SCRUB, UNDRAINED, WASTE,
						SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_peatland_spruce_with_bias_corr_Hynynenym(d, BA_prosNs, BA_L, SC, TS, PDR, H_dom, ALT, LAKE, UNDRAINED, WASTE,
						SUBGROUP, SCRUB, SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Birches and other deciduous species
	else
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
						SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (SP == 3)
                {
			if (BA>BA_max_sb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_sb*0.85)
				*modelresult = *modelresult*0.6;
		}
		else
		{
			if (BA>BA_max_pb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_pb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_pb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_pb*0.85)
				*modelresult = *modelresult*0.6;
		}
	}

	if (ret == 0)
		return ret;

	*nres = 1;	
	return ret;

}

int Grow_ba_peatland_spruce_with_mineralsoil_model_JKK (double d, double BA, double BA_L, double BA_prosb, double BA_prosNs, double SP, double H_dom, double LAT,
						double SC, double TS, double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double LAKE, double SCRUB, double UNDRAINED, 
						double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE,
						double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb, double MANAGEMENT_REGION, double cr, double h, double RDF_L,
						double RDF_Sp, double RDF_Ns, double RDF_decid, double SI_50,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    
	//Calculate tree basal area growth on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_peatland_pine_with_bias_corr_Hynynenym(d, BA, BA_L, SC, TS, PDR, FUSC, SUBGROUP, ALT, SEA, SCRUB, UNDRAINED, WASTE,
						SINCE_THINNING, SINCE_DRAINAGE, MANAGEMENT_REGION, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_spruce_with_bias_corr_Hynynenym(d, cr, SI_50, RDF_L, RDF_Sp, RDF_Ns, RDF_decid, TS, H_dom, SC, h,
						ALT, SCRUB, WASTE, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Birches and other deciduous species
	else
	{
		ret = Basal_area_growth_peatland_birch_with_bias_corr_Hynynenym(d, BA_prosNs, BA_prosb, BA_L, SC, TS, PDR, SEA, SUBGROUP, ALT, SCRUB, UNDRAINED, SP, WASTE,
						SINCE_THINNING, SINCE_DRAINAGE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (SP == 3)
                {
			if (BA>BA_max_sb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_sb*0.85)
				*modelresult = *modelresult*0.6;
		}
		else
		{
			if (BA>BA_max_pb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_pb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_pb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_pb*0.85)
				*modelresult = *modelresult*0.6;
		}
	}

	if (ret == 0)
		return ret;

	*nres = 1;	
	return ret;
	
}

int Grow_h_mineral_soil_JKK (double d, double cr, double SP, double i_Hdom5,  
						double CR_dom, double RDF_L, double PLANT, double D_dom, double h,
						double h_max_Sp, double h_max_Ns, double h_max_sb, double h_max_pb,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    
	//Calculate tree height growth on mineral soil
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Height_growth_pine_Hynynenym(d, D_dom, i_Hdom5, cr, CR_dom, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the height growth
		if (h>h_max_Sp)
			*modelresult = *modelresult*0.01;
		else if (h>h_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (h>h_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (h>h_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Height_growth_spruce_Hynynenym(d, D_dom, i_Hdom5, cr, CR_dom, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the height growth
		if (h>h_max_Ns)
			*modelresult = *modelresult*0.01;
		else if (h>h_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (h>h_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (h>h_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Birches and other deciduous species
	else
	{
		ret = Height_growth_birch_Hynynenym(d, D_dom, i_Hdom5, PLANT, RDF_L, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the height growth
		if (SP == 3)
                {
			if (h>h_max_sb)
				*modelresult = *modelresult*0.01;
			else if (h>h_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (h>h_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (h>h_max_sb*0.85)
				*modelresult = *modelresult*0.6;
		}
		else
		{
			if (h>h_max_pb)
				*modelresult = *modelresult*0.01;
			else if (h>h_max_pb*0.95)
				*modelresult = *modelresult*0.1;
			else if (h>h_max_pb*0.9)
				*modelresult = *modelresult*0.3;
			else if (h>h_max_pb*0.85)
				*modelresult = *modelresult*0.6;
		}
		
	}

	if (ret == 0)
		return ret;

	*nres = 1;
	//Restrict if result is impossible
	if (*modelresult<=0.01)
	{
         *modelresult = 0.01;
    } 
	
	return ret;

}


int Grow_ba_mineral_soil_JKK (double d, double cr, double h, double SP, double H_dom, double RDF, double RDF_L, double PLANT, double RDF_Sp,
						double RDF_Ns, double RDF_decid, double SI_50, double TS, double SC, double D_dom,
						double ALT, double SEA, double SCRUB, double WASTE, double SINCE_THINNING, double BA,
						double BA_max_Sp, double BA_max_Ns, double BA_max_sb, double BA_max_pb,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

	//Calculate tree basal area growth on mineral soil
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Basal_area_growth_pine_with_bias_corr_Hynynenym(d, cr, SI_50, RDF_L, RDF_Sp, RDF_Ns, RDF_decid, TS, H_dom, SC, h,
						ALT, SEA, SCRUB, WASTE, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Sp)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Sp*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Sp*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Sp*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Basal_area_growth_spruce_with_bias_corr_Hynynenym(d, cr, SI_50, RDF_L, RDF_Sp, RDF_Ns, RDF_decid, TS, H_dom, SC, h,
						ALT, SCRUB, WASTE, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
		//Brakes for the diameter growth
		if (BA>BA_max_Ns)
			*modelresult = *modelresult*0.03;
		else if (BA>BA_max_Ns*0.95)
			*modelresult = *modelresult*0.1;
		else if (BA>BA_max_Ns*0.9)
			*modelresult = *modelresult*0.3;
		else if (BA>BA_max_Ns*0.85)
			*modelresult = *modelresult*0.6;
	}
	//Birches and other deciduous species
	//Bias correction not used because it causes problems
	else
	{
		ret = Basal_area_growth_birch_Hynynenym(d, cr, SI_50, RDF_L, RDF, TS, H_dom, PLANT, SP, h,
                        SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
                //Brakes for the diameter growth
                if (SP == 3)
                {
			if (BA>BA_max_sb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_sb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_sb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_sb*0.85)
				*modelresult = *modelresult*0.6;
		}
		else
		{
			if (BA>BA_max_pb)
				*modelresult = *modelresult*0.03;
			else if (BA>BA_max_pb*0.95)
				*modelresult = *modelresult*0.1;
			else if (BA>BA_max_pb*0.9)
				*modelresult = *modelresult*0.3;
			else if (BA>BA_max_pb*0.85)
				*modelresult = *modelresult*0.6;
		}
	}

	if (ret == 0)
		return ret;

	*nres = 1;
	return ret;

}

int Tree_height_before_growing_d_peatland_JKK (double d, double D_gM, double BA, double SP, double LAT, double SC,
						double SINCE_THINNING, double ALT, double BA_prosb,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate tree height on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Height_peatland_birch_Hynynenym(d, D_gM, BA, LAT, ALT, BA_prosb, SINCE_THINNING, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}

	if (ret == 0)
		return ret;

	*nres = 1;
	return ret;

}

int Tree_height_after_growing_d_peatland_JKK (double d, double h, double h_before, double D_gM, double BA, double SP, double LAT, double SC,
						double SINCE_THINNING, double ALT, double BA_prosb,
						int *nres, struct Tree_h_peatland *h_peatland, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    double *h_temp = (double *) malloc(1*sizeof(double));
	//Calculate tree height on peatland
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Height_peatland_pine_Hynynenym(d, D_gM, BA, LAT, SC, SINCE_THINNING, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Height_peatland_spruce_Hynynenym(d, D_gM, BA, LAT, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Height_peatland_birch_Hynynenym(d, D_gM, BA, LAT, ALT, BA_prosb, SINCE_THINNING, nres, h_temp, errors, errorCheckMode, allowedRiskLevel, 1);
	}

	if (ret == 0)
	{
        free(h_temp);    
		return ret;
    }

	double h_after, i_h;
	h_after = *h_temp;
	i_h = h_after - h_before;
	
	//Restrict if result is too small
	if (i_h<=0.01)
	{
         i_h = 0.01;
    }
	
	//Update h
	h = h + i_h;

	*nres = 1;
	h_peatland[0].i_h=i_h;
	h_peatland[0].h=h;
	
	free(h_temp);

	return ret;

}

int H_dom_growth_JKK (double SP, double A, double H_gM, double TS, double LAKE, double PLANT, double SC,
						double PUB, double PALU, double HUMUS, double SEA, double ALT,
						double SCRUB, double WASTE, double STONY, double RDF,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	double H_dom_potential, H_dom_a5y_potential;
	double i_Hdom5;
	//Calculate stratum's dominant height
	//First calculate H_dom at the beginning of the period
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Dominant_height_pine_with_bias_corr_Hynynenym(A, TS, ALT, LAKE, SEA, SC, STONY, PALU, HUMUS, RDF, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Dominant_height_spruce_with_bias_corr_Hynynenym(A, TS, LAKE, SEA, SC, STONY, PALU, HUMUS, ALT, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_birch_with_bias_corr_Hynynenym(A, TS, LAKE, PLANT, SC, SP, PUB, PALU, HUMUS, SEA, ALT, SCRUB, WASTE, STONY,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
    H_dom_potential = *modelresult;

	if (ret == 0)
		return ret;
	
	//Calculate H_dom after 5 years
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Dominant_height_after_5_years_pine_with_bias_corr_Hynynenym(A, TS, ALT, LAKE, SEA, SC, STONY, PALU, HUMUS, RDF, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Dominant_height_after_5_years_spruce_with_bias_corr_Hynynenym(A, TS, LAKE, SEA, SC, STONY, PALU, HUMUS, ALT, SCRUB, WASTE,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	//Birches and other deciduous species
	else
	{
		ret = Dominant_height_after_5_years_birch_with_bias_corr_Hynynenym(A, TS, LAKE, PLANT, SC, SP, PUB, PALU, HUMUS, SEA, ALT, SCRUB, WASTE, STONY,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, rectFactor);
	}
	H_dom_a5y_potential = *modelresult;

	if (ret == 0)
		return ret;

	//Calculate H_dom growth during 5 years
	i_Hdom5 = H_dom_a5y_potential - H_dom_potential;
	
	//Restrict Hdom growth if it's too high
	if (SP==3||SP==5)
	{
            if (H_gM>30)
               i_Hdom5 = i_Hdom5*0.2;
            else if (H_gM>26)
               i_Hdom5 = i_Hdom5*0.5;
            else
                i_Hdom5 = i_Hdom5*0.8;  
            if (i_Hdom5>5)
               i_Hdom5 = 5;
    }  
	else if (SP==4||SP==6||SP==7||SP==9)
    {
            if (H_gM>28)
               i_Hdom5 = i_Hdom5*0.1;
            else if (H_gM>24)
               i_Hdom5 = i_Hdom5*0.5;
            else
                i_Hdom5 = i_Hdom5*0.8; 
            if (i_Hdom5>3.5)
               i_Hdom5 = 3.5;
    }

	*nres = 1;
	*modelresult = i_Hdom5;
	return ret;

}

int Dominant_height_pine_Hynynenym (double A, double TS,
					double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC3, SC5_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4 or SC==1, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term was not behaved as expected!
	*nres = 1;
	double res =exp(3.578+1.453*(TS/1000)+0.0875*(ALT/100)+0.1647*LAKE-0.4558*SEA+0.1385*SC1_2+0.0782*SC3-0.1363*SC5_8-0.1013*STONY-0.0365*PALU-0.1718*HUMUS+
			0.1308*sqrt(RDF)-6.054*pow(A,-0.2)+(pow(0.178,2)+pow(0.094,2)+pow(0.0323,2))/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_pine_with_bias_corr_Hynynenym (double A, double TS,
					double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC3, SC5_8, SC4, SC5, SC7, SC8, SC14, SC38;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 1;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 1;
			SC7 = 0;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 1;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 1;
			SC14 = 0;
			SC38 = 1;
			break;
		// defaults to SC==4 or SC==1, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 1;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
	};

	if(SCRUB==1) {
		SC8 = 1;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid division by zero, when computing H_dom for seedling
	if(A==0)
		A=0.1;

	//To avoid extrapolation
	/*if(TS>1340)
		TS=1340;*/

	//Model for bias correction.
	double bias = 0.149241+0.68*(TS/1000)-0.562*pow(TS/1000,2)-0.000194*ALT-0.358054*LAKE+0.176556*SEA+0.020707*SC4+0.068819*SC5-0.170542*SC7-0.402331*SC8-
					0.7*WASTE-0.252109*SC14*STONY-0.10647*SC14*PALU-0.70677*SC38*PALU;

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term was not behaved as expected!
	*nres = 1;
	double res =exp(3.578+1.453*(TS/1000)+0.0875*(ALT/100)+0.1647*LAKE-0.4558*SEA+0.1385*SC1_2+0.0782*SC3-0.1363*SC5_8-0.1013*STONY-0.0365*PALU-0.1718*HUMUS+
			0.1308*sqrt(RDF)-6.054*pow(A,-0.2)+(pow(0.178,2)+pow(0.094,2)+pow(0.0323,2))/2+bias)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_pine_Hynynenym (double A, double TS,
					double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC3, SC5_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			break;
		// defaults to SC==4 or SC==1, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.578+1.453*(TS/1000)+0.0875*(ALT/100)+0.1647*LAKE-0.4558*SEA+0.1385*SC1_2+0.0782*SC3-0.1363*SC5_8-0.1013*STONY-0.0365*PALU-0.1718*HUMUS+
			0.1308*sqrt(RDF)-6.054*pow(A,-0.2)+(pow(0.178,2)+pow(0.094,2)+pow(0.0323,2))/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_pine_with_bias_corr_Hynynenym (double A, double TS,
					double ALT, double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double RDF, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC3, SC5_8, SC4, SC5, SC7, SC8, SC14, SC38;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC3 = 1;
			SC5_8 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
		case 4:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 1;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 1;
			SC7 = 0;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 1;
			SC8 = 0;
			SC14 = 0;
			SC38 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 1;
			SC14 = 0;
			SC38 = 1;
			break;
		// defaults to SC==4 or SC==1, sensible?
		default:
			SC1_2 = 0;
			SC3 = 0;
			SC5_8 = 0;
			SC4 = 1;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			SC14 = 1;
			SC38 = 1;
			break;
	};

	if(SCRUB==1) {
		SC8 = 1;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation
	/*if(TS>1340)
		TS=1340;*/

	//Model for bias correction.
	double bias = 0.149241+0.68*(TS/1000)-0.562*pow(TS/1000,2)-0.000194*ALT-0.358054*LAKE+0.176556*SEA+0.020707*SC4+0.068819*SC5-0.170542*SC7-0.402331*SC8-
					0.7*WASTE-0.252109*SC14*STONY-0.10647*SC14*PALU-0.70677*SC38*PALU;

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term was not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.578+1.453*(TS/1000)+0.0875*(ALT/100)+0.1647*LAKE-0.4558*SEA+0.1385*SC1_2+0.0782*SC3-0.1363*SC5_8-0.1013*STONY-0.0365*PALU-0.1718*HUMUS+
			0.1308*sqrt(RDF)-6.054*pow(A,-0.2)+(pow(0.178,2)+pow(0.094,2)+pow(0.0323,2))/2+bias)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_spruce_Hynynenym (double A, double TS,
					double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 5 treated here as SC4
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
	};


	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	*nres = 1;
	double res =exp(3.418+1.48*(TS/1000)+0.08331*LAKE-0.7855*SEA+0.2112*SC1+0.1874*SC2-0.2005*SC4_8-0.0434*STONY-0.0892*PALU-0.1631*HUMUS
			-6.337*pow(A,-0.25)+(pow(0.176,2)+pow(0.0738,2)+pow(0.0323,2))/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_spruce_with_bias_corr_Hynynenym (double A, double TS,
					double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double ALT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4_8, SC4, SC6, SC7, SC8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 1;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 5 treated here as SC4
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 1;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 1;
			SC8 = 0;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
	};

	//To avoid division by zero, when computing H_dom for seedling
	if(A==0)
		A=0.1;

	//To avoid extrapolation
	/*if(TS>1340)
		TS=1340;*/

	//Model for bias correction.
	//Original value of parameter was +0.196336 for variable SC7_b. To ensure logic of the model it was changed to -0.1.
	//Original value of parameter was +0.187935 for variable SCRUB. To ensure logic of the model it was changed to -0.2.
	//Original value of parameter was -0.2 for variable WASTE. To ensure logic of the model it was changed to -0.4.
	double bias = -2.055512+3.98*(TS/1000)-1.839*pow(TS/1000,2)+0.00097*ALT-0.037887*LAKE+0.664343*SEA+0.01729*SC1-0.070675*SC2+0.019196*SC4
		-0.1*SC6-0.1*SC7-0.187849*SC8-0.2*SCRUB-0.4*WASTE-0.074049*STONY-0.099341*PALU-0.08391*HUMUS;

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	*nres = 1;
	double res =exp(3.418+1.48*(TS/1000)+0.08331*LAKE-0.7855*SEA+0.2112*SC1+0.1874*SC2-0.2005*SC4_8-0.0434*STONY-0.0892*PALU-0.1631*HUMUS
			-6.337*pow(A,-0.25)+(pow(0.176,2)+pow(0.0738,2)+pow(0.0323,2))/2+bias)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_spruce_Hynynenym (double A, double TS,
					double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4_8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 5 treated here as SC4
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			break;
	};


	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.418+1.48*(TS/1000)+0.08331*LAKE-0.7855*SEA+0.2112*SC1+0.1874*SC2-0.2005*SC4_8-0.0434*STONY-0.0892*PALU-0.1631*HUMUS
			-6.337*pow(A,-0.25)+(pow(0.176,2)+pow(0.0738,2)+pow(0.0323,2))/2)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_spruce_with_bias_corr_Hynynenym (double A, double TS,
					double LAKE, double SEA, double SC, double STONY, double PALU, double HUMUS, double ALT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4_8, SC4, SC6, SC7, SC8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 1;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 5 treated here as SC4
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 1;
			SC7 = 0;
			SC8 = 0;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 1;
			SC8 = 0;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 1;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4_8 = 0;
			SC4 = 0;
			SC6 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
	};

	//To avoid extrapolation
	/*if(TS>1340)
		TS=1340;*/

	//Model for bias correction.
	//Original value of parameter was +0.196336 for variable SC7_b. To ensure logic of the model it was changed to -0.1.
	//Original value of parameter was +0.187935 for variable SCRUB. To ensure logic of the model it was changed to -0.2.
	//Original value of parameter was -0.2 for variable WASTE. To ensure logic of the model it was changed to -0.4.
	double bias = -2.055512+3.98*(TS/1000)-1.839*pow(TS/1000,2)+0.00097*ALT-0.037887*LAKE+0.664343*SEA+0.01729*SC1-0.070675*SC2+0.019196*SC4
		-0.1*SC6-0.1*SC7-0.187849*SC8-0.2*SCRUB-0.4*WASTE-0.074049*STONY-0.099341*PALU-0.08391*HUMUS;

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.418+1.48*(TS/1000)+0.08331*LAKE-0.7855*SEA+0.2112*SC1+0.1874*SC2-0.2005*SC4_8-0.0434*STONY-0.0892*PALU-0.1631*HUMUS
			-6.337*pow(A,-0.25)+(pow(0.176,2)+pow(0.0738,2)+pow(0.0323,2))/2+bias)+1.3;
	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_birch_Hynynenym (double A, double TS,
					double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1, SC2;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC1 = 0;
			SC2 = 0;
			break;
		// defaults to SC==3 or SC=4 or SC=5, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			break;
	};


	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	*nres = 1;
	double res =exp(3.155+0.6528*(TS/1000)-0.05292*LAKE+0.03088*SC1+0.01246*SC2+0.1871*PALU-0.445*HUMUS
			+0.3229*PLANT-0.175*PUB-6.569*pow(A,-0.5)+(pow(0.0745,2)+pow(0.0519,2)+pow(0.0439,2))/2)+1.3;
	//In some extreme cases model gives unbelievable dominant height as a result (for instance 47 m)
	//To avoid such values, a constraint is added:
	if(res>36)
		res=36;

	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_birch_with_bias_corr_Hynynenym (double A, double TS,
					double LAKE, double PLANT, double SC, double SP, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4, SC5, SC6, SC78;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4 =1;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =1;
			SC6 =0;
			SC78 =0;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =1;
			SC78 =0;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =1;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
	};

	//To avoid extrapolation
	/*if(TS>1280)
		TS=1280;*/

	//Model for bias correction.
	//Original value of parameter was +0.234759 for variable SC4. To ensure logic of the model it was changed to -0.1.
	//Original value of parameter was +0.59783 for variable SC5. To ensure logic of the model it was changed to -0.25.
	//Original value of parameter was -0.1 for variable SC6. To ensure logic of the model it was changed to -0.35.
	//Original value of parameter was +0.169286 for variable SC78. To ensure logic of the model it was changed to -0.5.
	double bias;
	if((SP==3||SP==4)) {
		bias = -1.557652+0.063115*PUB+1.986*(TS/1000)-0.513*pow(TS/1000,2)+0.00108*ALT-0.110713*LAKE-0.108478*SEA-0.027724*SC1+0.075057*SC2
			-0.1*SC4-0.25*SC5-0.35*SC6-0.5*SC78-0.249864*SCRUB-0.3*WASTE-0.067949*STONY+0.01218*STONY*PUB+0.27985*PALU
			-0.433211*PALU*PUB+0.246037*HUMUS;
	}
	if(SP>=5) {
		int SP4, SP5, SP6, SP7, SP8, SP9;
		int iSP = (int) SP;
		switch(iSP)
		{
			case 5:
				SP4 = 0;
				SP5 = 1;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 0;
				break;
			case 6:
				SP4 = 0;
				SP5 = 0;
				SP6 = 1;
				SP7 = 0;
				SP8 = 0;
				SP9 = 0;
				break;
			case 7:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 1;
				SP8 = 0;
				SP9 = 0;
				break;
			case 8:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 1;
				SP9 = 0;
				break;
			case 9:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 1;
				break;
			// defaults to SP==9 (other deciduous tree species)
			default:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 1;
				break;
		};

		//Model for bias correction.
		//Original value of parameter was +0.257001 for variable SC4. To ensure logic of the model it was changed to -0.1.
		//Original value of parameter was +0.599669 for variable SC5. To ensure logic of the model it was changed to -0.25.
		//Original value of parameter was -0.1 for variable SC6. To ensure logic of the model it was changed to -0.35.
		//Original value of parameter was +0.163237 for variable SC78. To ensure logic of the model it was changed to -0.5.
		//Original value of parameter was +0.316935 for variable SCRUB. To ensure logic of the model it was changed to -0.15.
		bias = -1.426887+0.0582216*SP4+0.144658*SP5-0.080327*SP6+0.0916*SP7-0.2*SP8-0.41*SP9+1.838*(TS/1000)-0.54*pow(TS/1000,2)
				+0.00086*ALT-0.082521*LAKE-0.390489*SEA+0.012887*SC1+0.064706*SC2-0.1*SC4-0.25*SC5-0.35*SC6-0.5*SC78
				-0.15*SCRUB-0.2*WASTE-0.53425*STONY-0.14812*PALU;
	}

	//To avoid division by zero, when computing H_dom for seedling
	if(A==0)
		A=0.1;

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	*nres = 1;
	double res =exp(3.155+0.6528*(TS/1000)-0.05292*LAKE+0.03088*SC1+0.01246*SC2+0.1871*PALU-0.445*HUMUS
			+0.3229*PLANT-0.175*PUB-6.569*pow(A,-0.5)+(pow(0.0745,2)+pow(0.0519,2)+pow(0.0439,2))/2+bias)+1.3;
	//In some extreme cases model gives unbelievable dominant height as a result (for instance 47 m)
	//To avoid such values, a constraint is added:
	if(res>36)
		res=36;

	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_birch_Hynynenym (double A, double TS,
					double LAKE, double PLANT, double SC, double PUB, double PALU, double HUMUS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1, SC2;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC1 = 0;
			SC2 = 0;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC1 = 0;
			SC2 = 0;
			break;
		// defaults to SC==3 or SC=4 or SC=5, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			break;
	};


	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.155+0.6528*(TS/1000)-0.05292*LAKE+0.03088*SC1+0.01246*SC2+0.1871*PALU-0.445*HUMUS
			+0.3229*PLANT-0.175*PUB-6.569*pow(A,-0.5)+(pow(0.0745,2)+pow(0.0519,2)+pow(0.0439,2))/2)+1.3;
	//In some extreme cases model gives unbelievable dominant height as a result (for instance 47 m)
	//To avoid such values, a constraint is added:
	if(res>36)
		res=36;

	*modelresult = res * rectFactor;
	return ret;

}


int Dominant_height_after_5_years_birch_with_bias_corr_Hynynenym (double A, double TS,
					double LAKE, double PLANT, double SC, double SP, double PUB, double PALU, double HUMUS, double SEA, double ALT, double SCRUB, double WASTE, double STONY,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int SC1, SC2, SC4, SC5, SC6, SC78;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC4 =1;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =1;
			SC6 =0;
			SC78 =0;
			break;
		// SC 6 not present in original publication, treated here as SC3
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =1;
			SC78 =0;
			break;
		// SC 7 not present in original publication, treated here as SC3
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =1;
			break;
		// SC 8 not present in original publication, treated here as SC3
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =1;
			break;
		// defaults to SC==3 or SC=4 or SC=5, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC4 =0;
			SC5 =0;
			SC6 =0;
			SC78 =0;
			break;
	};

	//To avoid extrapolation
	/*if(TS>1280)
		TS=1280;*/

	//Model for bias correction.
	//Original value of parameter was +0.234759 for variable SC4. To ensure logic of the model it was changed to -0.1.
	//Original value of parameter was +0.59783 for variable SC5. To ensure logic of the model it was changed to -0.25.
	//Original value of parameter was -0.1 for variable SC6. To ensure logic of the model it was changed to -0.35.
	//Original value of parameter was +0.169286 for variable SC78. To ensure logic of the model it was changed to -0.5.
	double bias;
	if((SP==3||SP==4)) {
		bias = -1.557652+0.063115*PUB+1.986*(TS/1000)-0.513*pow(TS/1000,2)+0.00108*ALT-0.110713*LAKE-0.108478*SEA-0.027724*SC1+0.075057*SC2
			-0.1*SC4-0.25*SC5-0.35*SC6-0.5*SC78-0.249864*SCRUB-0.3*WASTE-0.067949*STONY+0.01218*STONY*PUB+0.27985*PALU
			-0.433211*PALU*PUB+0.246037*HUMUS;
	}
	if(SP>=5) {
		int SP4, SP5, SP6, SP7, SP8, SP9;
		int iSP = (int) SP;
		switch(iSP)
		{
			case 5:
				SP4 = 0;
				SP5 = 1;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 0;
				break;
			case 6:
				SP4 = 0;
				SP5 = 0;
				SP6 = 1;
				SP7 = 0;
				SP8 = 0;
				SP9 = 0;
				break;
			case 7:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 1;
				SP8 = 0;
				SP9 = 0;
				break;
			case 8:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 1;
				SP9 = 0;
				break;
			case 9:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 1;
				break;
			// defaults to SP==9 (other deciduous tree species)
			default:
				SP4 = 0;
				SP5 = 0;
				SP6 = 0;
				SP7 = 0;
				SP8 = 0;
				SP9 = 1;
				break;
		};

		//Model for bias correction.
		//Original value of parameter was +0.257001 for variable SC4. To ensure logic of the model it was changed to -0.1.
		//Original value of parameter was +0.599669 for variable SC5. To ensure logic of the model it was changed to -0.25.
		//Original value of parameter was -0.1 for variable SC6. To ensure logic of the model it was changed to -0.35.
		//Original value of parameter was +0.163237 for variable SC78. To ensure logic of the model it was changed to -0.5.
		//Original value of parameter was +0.316935 for variable SCRUB. To ensure logic of the model it was changed to -0.15.
		bias = -1.426887+0.0582216*SP4+0.144658*SP5-0.080327*SP6+0.0916*SP7-0.2*SP8-0.41*SP9+1.838*(TS/1000)-0.54*pow(TS/1000,2)
				+0.00086*ALT-0.082521*LAKE-0.390489*SEA+0.012887*SC1+0.064706*SC2-0.1*SC4-0.25*SC5-0.35*SC6-0.5*SC78
				-0.15*SCRUB-0.2*WASTE-0.53425*STONY-0.14812*PALU;
	}

	//OBS! Anomaly term D_domvmi/d is removed because Risto Ojansuu told to do so - the term is not behaved as expected!
	//Age after 5 years!
	A=A+5;
	*nres = 1;
	double res =exp(3.155+0.6528*(TS/1000)-0.05292*LAKE+0.03088*SC1+0.01246*SC2+0.1871*PALU-0.445*HUMUS
			+0.3229*PLANT-0.175*PUB-6.569*pow(A,-0.5)+(pow(0.0745,2)+pow(0.0519,2)+pow(0.0439,2))/2+bias)+1.3;
	//In some extreme cases model gives unbelievable dominant height as a result (for instance 47 m)
	//To avoid such values, a constraint is added:
	if(res>36)
		res=36;

	*modelresult = res * rectFactor;
	return ret;

}

int Basal_area_growth_peatland_pine_with_bias_corr_fixed_Hynynenym (double d, double BA, double BA_L, double SC, double TS,
									double PDR, double FUSC, double SUBGROUP, double ALT, double SEA, double SCRUB, 
									double UNDRAINED, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	// More simple growth effect for ditching condition
	double DR_25, coeff_DR25;
	DR_25 = 0;
	coeff_DR25 = 0.0;
	if(SINCE_DRAINAGE > 25)
		DR_25 = 1;


	int PDR_d, Y1, Y2, Y3, Y4, Y24, SC1, SC4, SC5, SC6;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
		Y24 = 0;
		Y4 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				Y24 = 1;
				Y4 = 0;
				break;
			case 3:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				Y24 = 0;
				Y4 = 0;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 0;
				Y24 = 1;
				Y4 = 1;
				break;
		};
	}

	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
				SC1 = 1;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 2:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 3:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 0;
				break;
		case 4:
				SC1 = 0;
				SC4 = 1;
				SC5 = 0;
				SC6 = 0;
				break;
		case 5:
				SC1 = 0;
				SC4 = 0;
				SC5 = 1;
				SC6 = 0;
				break;
		case 6:
				SC1 = 0;
				SC4 = 0;
				SC5 = 0;
				SC6 = 1;
				break;
	};

	// PDR growth effect removed
	PDR_d=0;

	//Check for fatal errors
	char errorStr[200] = "";
	if (BA<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "BA<=0(BA=",BA);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (TS*sqrt(d)<=0)
	{
		ret = 0;
		constructErrorMessage2val (errorStr, "TS*sqrt(d)<=0(TS=",TS, ",d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for pine is 1.0 cm and max 60 cm! (Hynynen ym.)
	if (d<1)
		d=1;
	if (d>60)
		d=60;

	//Bias correction model:
	double bias = -3.89138+5.76914*(TS/1000.0)-2.40611*pow(TS/1000.0,2)+0.00293*ALT+0.82861*SEA
			-0.29352*SC1-0.26141*SC4-0.24866*SC5-0.3955*SC6-0.32354*SCRUB-0.27118*UNDRAINED;

	//Variance correction:
	double c;
	if(d<5)
		c=1.24647;
	if(d>=5&&d<10)
		c=1.11459;
	if(d>=10&&d<20)
		c=1.06898;
	if(d>=20)
		c=1.04918;

	*nres = 1;
	//Without calibration
	double res1 =exp(-1.217101-0.00187*PI/4*pow(d,2)-0.00892*BA_L-0.00153*pow(BA_L,2)-0.2468*log(BA)+0.06915*sqrt(TS*sqrt(d))-
		0.6115*Y1+0.301425*Y2+0.664339*(Y1*log(d))+0.314616*((Y24)*log(d))+0.388967*(Y3*log(d))+coeff_DR25*DR_25-
		0.155569*PDR_d+0.12766*THIN_0_5+0.258215*FUSC+(pow(0.410967,2)+pow(0.576062,2))/2)-1;

	//With calibration
	double res2 =(c*((exp(-1.217101-0.00187*PI/4*pow(d,2)-0.00892*BA_L-0.00153*pow(BA_L,2)-0.2468*log(BA)+0.06915*sqrt(TS*sqrt(d))-
		0.6115*Y1+0.301425*Y2+0.664339*(Y1*log(d))+0.314616*((Y24)*log(d))+0.388967*(Y3*log(d))+coeff_DR25*DR_25-
		0.155569*PDR_d+0.12766*THIN_0_5+0.258215*FUSC+(pow(0.410967,2)+pow(0.576062,2))/2)-1)+1)*exp(bias))-1;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr;
	//Southern Finland (forest districts 0-15)
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2)
	{	if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.881;
		if(Y3==1)
			fert_corr=0.91;
		if(Y4==1)
			fert_corr=0.971;
	}
	else //Northern Finland (forest districts 15-)
	{
		if(Y1==1)
			fert_corr=0.9852;
		if(Y2==1)
			fert_corr=0.93;
		if(Y3==1)
			fert_corr=0.9434;
		if(Y4==1)
			fert_corr=0.971;
	}
	res1=fert_corr*res1;
	res2=fert_corr*res2;


	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<6) {
				res=6.0;
			}
		}
		if(WASTE == 1)
		res = 0.5 * res;
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=6.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=6+(((15-d)/5)*(res-6));
			if(d>15)
				res=6.0;	
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}

int Basal_area_growth_peatland_spruce_with_bias_corr_fixed_Hynynenym (double d, double BA_prosNs, double BA_L, double SC, double TS,
									double PDR, double H_domvmi, double ALT, double LAKE, double UNDRAINED, 
									double WASTE, double SUBGROUP, double SCRUB, double SINCE_THINNING, double SINCE_DRAINAGE, double MANAGEMENT_REGION,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	// More simple growth effect for ditching condition
	double DR_25, coeff_DR25;
	DR_25 = 0;
	coeff_DR25 = 0.0;
	if(SINCE_DRAINAGE > 25)
		DR_25 = 1;


	int PDR_d, Y1, Y2, SC1, SC2, SC4, SC5;
	int iSC = (int) SC;
	switch (iSC)
		{
		case 1:
			Y1 = 1;
			Y2 = 0;
			SC1 = 1;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 2:
			Y1 = 1;
			Y2 = 0;
			SC1 = 0;
			SC2 = 1;
			SC4 = 0;
			SC5 = 0;
			break;
		case 3:
			Y1 = 1;
			Y2 = 0;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 4:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 1;
			SC5 = 0;
			break;
		case 5:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			Y1 = 0;
			Y2 = 1;
			SC1 = 0;
			SC2 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
	};


	// PDR growth effect removed
	PDR_d=0;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}
	
	//To avoid extrapolation: problems caused by very little diameter minimum diameter for spruce is 1.3 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;

	//Bias correction model:
	double bias = -0.04965+0.29286*log(d)-0.62972*log(H_domvmi)+0.70113*(TS/1000.0)+0.0017*ALT+0.32627*LAKE
				+0.65389*SC1+0.28405*SC2+0.10779*SC4-0.20786*SC5;

	//Variance correction:
	double c = 1.15823;

	*nres = 1;
	//Without calibration
	double res1 =exp(1.223-0.00001747*pow(d,2.9)-0.01503*BA_L-0.00441*(BA_prosNs/100.0)+0.0008156*TS+0.5141*(Y1*sqrt(d))+0.4279*((Y2)*sqrt(d))+coeff_DR25*DR_25-0.06852*PDR_d+0.1621*THIN_0_5+(pow(0.08562,2)+pow(0.1241,2))/2)-7;

	//With calibration
	double res2 =(c*((exp(1.223-0.00001747*pow(d,2.9)-0.01503*BA_L-0.00441*(BA_prosNs/100.0)+0.0008156*TS+0.5141*(Y1*sqrt(d))+0.4279*((Y2)*sqrt(d))+coeff_DR25*DR_25-0.06852*PDR_d+0.1621*THIN_0_5+(pow(0.08562,2)+pow(0.1241,2))/2)-7)+1)*exp(bias))-1;
	
	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	//Southern Finland (forest districts 0-15)
	double fert_corr;
	if (MANAGEMENT_REGION == 1 || MANAGEMENT_REGION == 2) 
		fert_corr=0.9852;
	else //Northern Finland (forest districts 15-)
		fert_corr=0.995;
	res1=fert_corr*res1;
	res2=fert_corr*res2;

	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<6) {
				res=6.0;
			}
		}
		if(SCRUB==1||WASTE==1) {
             res = 0.5 * res;
        }     
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=6.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=6+(((15-d)/5)*(res-6));
			if(d>15)
				res=6.0;	
		}
	}
	
	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}

int Basal_area_growth_peatland_birch_with_bias_corr_fixed_Hynynenym (double d, double BA_prosNs, double BA_prosb, double BA_L, double SC,
									double TS, double PDR, double SEA, double SUBGROUP, double ALT, double SCRUB, double UNDRAINED,
									double SP, double WASTE, double SINCE_THINNING, double SINCE_DRAINAGE,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {

    int ret = 1;


	//Assign time since thinning and drainage dummy variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <= 5)
		THIN_0_5 = 1;

	// More simple growth effect for ditching condition
	double DR_25, coeff_DR25;
	DR_25 = 0;
	coeff_DR25 = 0.0;
	if(SINCE_DRAINAGE > 25)
		DR_25 = 1;


	int PDR_d, Y1, Y2, Y3, SC1, SC2;

	if(SUBGROUP==2||SUBGROUP==5) {
		Y1 = 1;
		Y2 = 0;
		Y3 = 0;
	}
	if(SUBGROUP==3||SUBGROUP==4) {
		int iSC = (int) SC;
		switch (iSC)
		{
			case 1:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 2:
				Y1 = 0;
				Y2 = 1;
				Y3 = 0;
				break;
			case 3:
				Y1 = 1;
				Y2 = 0;
				Y3 = 0;
				break;
			case 4:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 5:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
			case 6:
				Y1 = 0;
				Y2 = 0;
				Y3 = 1;
				break;
		};
	}

	if(SC==1){
		SC1=1;
		SC2=0;
	}
	if(SC==2){
		SC1=0;
		SC2=1;
	}
	if((SC!=1&&SC!=2)){
		SC1=0;
		SC2=0;
	}

	// PDR growth effect removed
	PDR_d=0;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//To avoid extrapolation: problems caused by very little or large diameter minimum diameter for birch is 1.3 cm and max 45 cm! (Hynynen ym.)
	if (d<1.3)
		d=1.3;
	if (d>45)
		d=45;

	double bias, c;

	//Bias and variance correction models:
	if(SP==4) {
		bias = -1.49202+1.11213*(TS/1000.0)+0.00228*ALT+0.18397*SC1+0.09348*SC2-0.28639*SCRUB-0.34648*UNDRAINED;
		if(d<5)
			c=1.15435;
		if(d>=5&&d<10)
			c=1.05338;
		if(d>=10&&d<20)
			c=1.11601;
		if(d>=20)
			c=1.35653;
	}
	if((SP==3||SP>=5)) {

		int sp3, sp5, sp6, sp7, sp89;
		int isp = (int) SP;
		switch(isp)
		{
			case 3:
				sp3 = 1;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 5:
				sp3 = 0;
				sp5 = 1;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 6:
				sp3 = 0;
				sp5 = 0;
				sp6 = 1;
				sp7 = 0;
				sp89 = 0;
				break;
			case 7:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 1;
				sp89 = 0;
				break;
			case 8:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			case 9:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			// defaults to sp==4, sensible?
			default:
				sp3 = 0;
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
		};

		bias = -2.67024+3.39029*(TS/1000.0)-1.08965*pow(TS/1000.0,2)+0.00231*ALT+0.09608*SC2-0.2579*SCRUB-0.34513*UNDRAINED
			+0.47066*sp3+0.45852*sp5-0.13055*sp6-0.19422*sp7+0.40588*sp89;

		if(d<5)
			c=1.16468;
		if(d>=5&&d<10)
			c=1.05695;
		if(d>=10&&d<20)
			c=1.10804;
		if(d>=20)
			c=1.34524;
	}

	*nres = 1;
	//Without calibration
	double res1 =exp(0.6394-0.00074*pow(d,2)-0.03533*BA_L+0.0005473*TS-0.003167*(BA_prosNs/100.0)-0.003626*(BA_prosb/100.0)+0.005155*SEA+1.016*(Y1*log(d))+
		1.048*(Y2*log(d))+0.9442*(Y3*log(d))+coeff_DR25*DR_25-0.1028*PDR_d+0.1777*THIN_0_5+(pow(0.1052,2)+pow(0.2413,2))/2)-3;
	//With calibration
	double res2 =(c*((exp(0.6394-0.00074*pow(d,2)-0.03533*BA_L+0.0005473*TS-0.003167*(BA_prosNs/100.0)-0.003626*(BA_prosb/100.0)+0.005155*SEA+1.016*(Y1*log(d))+
		1.048*(Y2*log(d))+0.9442*(Y3*log(d))+coeff_DR25*DR_25-0.1028*PDR_d+0.1777*THIN_0_5+(pow(0.1052,2)+pow(0.2413,2))/2)-3)+1)*exp(bias))-1;

	//Fertilization correction (modelling data of basal area growth models on peatlands included also fertilized stands):
	double fert_corr=0.978; 
	res1=fert_corr*res1;
	res2=fert_corr*res2;


	//Calibrated basal area growth is assumed to be the default value!
	double res=res2;
	//Basal area growth for little tree
	if(d<2)
		res=res1;

	if(UNDRAINED==0){
		//Difference of calibrated and not calibrated predictions:
		//If difference is too big calibration will not be used!
		double diff = (res2/res1)*100;
		if(diff<50)
			res = 0.5*res1;
		if(diff>150)
			res = 1.5*res1;
		if(d<5) {
			if(res<4) {
				res=4.0;
			}
		}
		if (WASTE == 1)
		   res = 0.5 * res;
	}

	//If stand is undrained, then basal area growth is calculated as follows
	//Undrained peatland correction:
	if(UNDRAINED==1){
		double und_corr;
		int iSC = (int) SC;
		if(SUBGROUP==2||SUBGROUP==5) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 1.0;
						break;
					case 2:
						und_corr = 1.0;
						break;
					default:
						und_corr = 0.7;
						break;
				}
			}
			if(SCRUB==1||WASTE==1){
				und_corr = 0.7;
			}
		}
		if(SUBGROUP==3||SUBGROUP==4) {
			if(SCRUB==0&&WASTE==0){
				switch (iSC)
				{
					case 1:
						und_corr = 0.5;
						break;
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.6;
						break;
				}
			}
			if(SCRUB==1){
				switch (iSC)
				{
					case 6:
						und_corr = 0.4;
						break;
					default:
						und_corr = 0.5;
						break;
				}
			}
			if(WASTE==1){
				und_corr = 0.4;
			}
		}
		res=und_corr*res;
		if(d>25)
			res=4.0;
		if((SCRUB==1||WASTE==1||SC==6)) {
			if((d>10&&d<=15))
				res=4+(((15-d)/5)*(res-4));
			if(d>15)
				res=4.0;	
		}
	}

	//Negative basal area growth is not allowed (it is possible for tiny trees). In that case garowth value (0,01 cm2) is given!
	if(res<=0)
		res=0.01;

	*modelresult = res * rectFactor;
	return ret;

}