// Copyright (c) 2007 Simosol Oy. Distributed under the GNU General Public License 2.0.

//////////////////////////////////////////////////////////////////////////////
//
//  TreemodelLibrary.cpp
//
//	Tree model library - Source file
//
//	Here are the conventional calls of the functions

#include <stdio.h>
#include "TreemodelLibrary.h"
#include "SeedlingmodelLibrary.h"
#include "SimoUtils.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


int Height_pine_Veltheim (double d, double Age, double BA, double D_gM, double TS, double ALT, double SC,
						int *nres, double *modelresult, char *errors, int errorCheckMode,
						double allowedRiskLevel, double rectFactor)
{

	int ret = 1;

	/*if (errorCheckMode == 2)
	{
		// Evaluate the riskfunction for the model
		// Palauttaako riskimalli jonkin arvon mallin kutsujalle?
		// Jos palauttaa, voisiko viestin vlitt warnings tai
		// errors osoittimissa?
	}*/

	int SC1_2, SC4, SC5, SC6_7;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4 = 1;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 1;
			SC6_7 = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 6/7
		case 8:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = 1.3 + exp(3.1077-18.4669/(d+5)-20.8014*(1/(Age+10))+0.00927384*BA-0.13927*(d/D_gM)+166.004/(TS/10)-12278.5/pow((TS/10.0),2)
		+0.00443761*(ALT/10)+0.0079877*(SC1_2)-0.0474632*(SC4)-0.117699*(SC5)-0.20109*(SC6_7)+pow(0.144925, 2)/2);
	*modelresult = res * rectFactor;
	return ret;
}


int Height_spruce_Veltheim (double d, double Age, double BA, double D_gM, double TS, double ALT, double SC,
							int *nres, double *modelresult, char *errors, int errorCheckMode,
							double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	// Check for the validity of the parameters

	int SC1_2, SC4_7;
	int iSC = (int) SC;

	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(3.28747-27.8738/(d+5)+19.6503/pow((d+5),2)+0.000596058*Age+0.00758229*BA-0.145859*(d/D_gM)+0.0039764*(TS/10)
		-0.00200392*ALT/10-0.013007*(SC1_2)-0.0544321*(SC4_7)+pow(0.137671, 2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_white_birch_Veltheim (double d, double Age, double BA, double D_gM, double TS, double ALT, double SC,
								int *nres, double *modelresult, char *errors, int errorCheckMode,
								double allowedRiskLevel, double rectFactor)
{
	int ret = 1;


	int SC1_2, SC4_7;

	int iSC = (int) SC;

	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.3 + exp(0.0924547-15.9087/(d+5)-16.1394/pow((d+5),2)+0.00236413*Age+0.0152315*BA-0.000157243*pow(BA,2)-0.0752347*(d/D_gM)
		+0.0424791*(TS/10)-0.000145416*pow((TS/10),2)+0.0097927*ALT/10+0.0245652*(SC1_2)-0.0656305*(SC4_7)+pow(0.135363,2)/2);
	*modelresult = res * rectFactor;
	return ret;
}



int Height_pubescent_birch_Veltheim (double d, double Age, double BA, double D_gM, double TS, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor)
{
	int ret = 1;


	int SC1_2, SC4_7;

	int iSC = (int) SC;

	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.3 + exp(2.34571-15.4407/(d+5)-7.23954/pow((d+5),2)-10.8662/(Age+10)+128.857/pow((Age+10),2)+0.00800176*BA-0.0292937*(d/D_gM)
		+288.903/(TS/10)-19611/pow((TS/10),2)+0.0229426*(SC1_2)-0.0435982*(SC4_7)+pow(0.156964,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_pine_peatland_Veltheim (double d, double Age, double BA, double D_gM, double TS, double ALT, double PEAT_transf, double SC,
								int *nres, double *modelresult, char *errors, int errorCheckMode,
								double allowedRiskLevel, double rectFactor)
{
	int ret = 1;



	int K1_3_R1_3, R5_6;

	int iSC = (int) SC;

	switch ( iSC )
	{
		case 1:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 2:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 3:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 4:
			K1_3_R1_3 = 0;
			R5_6 = 0;
			break;
		case 5:
			K1_3_R1_3 = 0;
			R5_6 = 1;
			break;
		case 6:
			K1_3_R1_3 = 0;
			R5_6 = 1;
			break;
		// defaults to SC==4 (K4 or R4), sensible?
		default:
			K1_3_R1_3 = 0;
			R5_6 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.3 + exp(3.17438-12.3585/(d+5)-44.4167/pow((d+5),2)-20.4126/(Age+10)+266.011/pow((Age+10),2)+0.0291349*BA-0.000319285*pow(BA,2)
 		-0.0609424*(d/D_gM)-37.5727/(TS/10)+0.00497042*ALT/10+0.0541735*(K1_3_R1_3)-0.0740786*(R5_6)+0.0474278*PEAT_transf+pow(0.172876,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_spruce_peatland_Veltheim (double d, double BA, double D_gM, double TS, double PEAT_transf, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor)
{
	int ret = 1;


	int K1_2_R1_2, K4_R4_6;

	int iSC = (int) SC;

	switch ( iSC )
	{
		case 1:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 2:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 3:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
		case 4:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 5:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 6:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		// defaults to SC==3 (K3 or R3), sensible?
		default:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.3 + exp(1.51318-22.6924/(d+5)-21.9424/pow((d+5),2)+0.0213751*BA-0.000305414*pow(BA,2)-0.104563*(d/D_gM)+0.0292778*(TS/10)
		-0.000110495*pow((TS/10),2)-0.00238941*(K1_2_R1_2)-0.0641081*(K4_R4_6)+0.0128418*PEAT_transf+pow(0.152065,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Height_birch_peatland_Veltheim (double d, double BA, double D_gM, double TS, double ALT, double PEAT_transf, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor)
{
	int ret = 1;


	int K1_2_R1_2, K4_R4_6;

	int iSC = (int) SC;

	switch ( iSC )
	{
		case 1:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 2:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 3:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
		case 4:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 5:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 6:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		// defaults to SC==3 (K3 or R3), sensible?
		default:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (D_gM<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "D_gM<=0(D_gM=",D_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =1.3 + exp(1.36109-16.1552/(d+5)+0.0179231*BA-0.000181031*pow(BA,2)-0.0531461*(d/D_gM)+0.0269223*(TS/10)-0.000100319*pow((TS/10),2)
		-0.00283863*ALT/10+0.032024*(K1_2_R1_2)-0.0730828*(K4_R4_6)+0.0263218*PEAT_transf+pow(0.172894,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Height_reduced_pine_Veltheim (double d, double Age, double BA, double TS, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC4, SC5, SC6_7;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4 = 1;
			SC5 = 0;
			SC6_7 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 1;
			SC6_7 = 0;
			break;
		case 6:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 6/7
		case 8:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4 = 0;
			SC5 = 0;
			SC6_7 = 0;
			break;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}


	*nres = 1;

	double res =1.3 + exp(2.90511-14.6143/(d+5)-14.9627/pow(d+5,2)-24.2827/(Age+10)+0.00975625*BA+161.142/(TS/10)-11639.6/pow(TS/10,2)
		+0.0242836*(SC1_2)-0.0493153*(SC4)-0.12775*(SC5)-0.220231*(SC6_7)+pow(0.149037,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Height_reduced_spruce_Veltheim (double d, double Age, double BA, double TS, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int SC1_2, SC4_7;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=", Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = 1.3 + exp(4.15505-23.0225/(d+5)-8.34469/(Age+10)+0.00750035*BA-67.5744/(TS/10)-0.00809273*(SC1_2)-0.0563144*(SC4_7)+pow(0.139924,2)/2);
	*modelresult = res * rectFactor;
	return ret;
}



int Height_reduced_white_birch_Veltheim (double d, double Age, double BA, double TS, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int SC1_2, SC4_7;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(0.136217-14.8584/(d+5)-18.6099/pow(d+5,2)+0.00300693*Age+0.0149697*BA-0.000153457*pow(BA,2)+0.0446155*(TS/10)
		-0.000169799*pow(TS/10,2)+0.0223578*(SC1_2)-0.073238*(SC4_7)+pow(0.140603,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_reduced_pubescent_birch_Veltheim (double d, double Age, double BA, double TS, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int SC1_2, SC4_7;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC4_7 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 5:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		case 7:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC 4/7
		case 8:
			SC1_2 = 0;
			SC4_7 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1_2 = 0;
			SC4_7 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(2.26508-14.9897/(d+5)-7.72103/pow(d+5,2)-11.6023/(Age+10)+121.728/pow(Age+10,2)+0.00808788*BA+298.582/(TS/10)-20157.8/pow(TS/10,2)
		+0.02318*(SC1_2)-0.0457852*(SC4_7)+pow(0.15716,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_reduced_pine_peatland_Veltheim (double d, double Age, double BA, double TS, double PEAT_transf, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int K1_3_R1_3, R5_6;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 2:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 3:
			K1_3_R1_3 = 1;
			R5_6 = 0;
			break;
		case 4:
			K1_3_R1_3 = 0;
			R5_6 = 0;
			break;
		case 5:
			K1_3_R1_3 = 0;
			R5_6 = 1;
			break;
		case 6:
			K1_3_R1_3 = 0;
			R5_6 = 1;
			break;
		// defaults to SC==4 (K4 or R4), sensible?
		default:
			K1_3_R1_3 = 0;
			R5_6 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (Age<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "Age<=0(Age=",Age);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(3.00806-11.5802/(d+5)-44.656/pow(d+5,2)-19.6658/(Age+10)+200.452/pow(Age+10,2)+0.0303168*BA-0.000354874*pow(BA,2)-25.7898/(TS/10)
		+0.0505343*(K1_3_R1_3)-0.0788385*(R5_6)+0.0549434*PEAT_transf+pow(0.17554,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Height_reduced_spruce_peatland_Veltheim (double d, double BA, double TS, double PEAT_transf, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int K1_2_R1_2, K4_R4_6;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 2:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 3:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
		case 4:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 5:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 6:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		// defaults to SC==3 (K3 or R3), sensible?
		default:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(1.30706-20.2395/(d+5)-30.4779/pow(d+5,2)+0.025524*BA-0.000380616*pow(BA,2)+0.0282449*(TS/10)-0.000105419*pow(TS/10,2)
		-0.00204873*(K1_2_R1_2)-0.0653818*(K4_R4_6)+0.0152586*PEAT_transf+pow(0.152586,2)/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Height_reduced_birch_peatland_Veltheim (double d, double BA, double TS, double PEAT_transf, double SC,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	int K1_2_R1_2, K4_R4_6;
	int iSC = (int) SC;
	switch ( iSC )
	{
		case 1:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 2:
			K1_2_R1_2 = 1;
			K4_R4_6 = 0;
			break;
		case 3:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
		case 4:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 5:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		case 6:
			K1_2_R1_2 = 0;
			K4_R4_6 = 1;
			break;
		// defaults to SC==3 (K3 or R3), sensible?
		default:
			K1_2_R1_2 = 0;
			K4_R4_6 = 0;
			break;
	}
	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1.3 + exp(0.7246-14.2489/(d+5)-7.7885/pow(d+5,2)+0.0214434*BA-0.000241903*pow(BA,2)+0.0338719*(TS/10)-0.000128162*pow(TS/10,2)
		+0.03493*(K1_2_R1_2)-0.067976*(K4_R4_6)+0.0252868*PEAT_transf+pow(0.174201,2)/2);

	*modelresult = res * rectFactor;
	return ret;

}


int Height_pine_Siipilehto (double d, double D_gM, double H_gM,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double a0, a1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	a1=0.2908+0.001341*D_gM-0.006337*H_gM;
	a0=(D_gM/pow(H_gM-1.3,(1.0/2.0)))-a1*D_gM;

	*nres = 1;
	double res =pow(d,2)/pow((a0+a1*d),2)+1.3;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_spruce_Siipilehto (double d, double D_gM, double H_gM,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double a0, a1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	a1=0.3834+0.002992*D_gM-0.00807*H_gM;
	a0=(D_gM/pow(H_gM-1.3,(1.0/3.0)))-a1*D_gM;

	*nres = 1;
	double res =pow(d,3)/pow(a0+a1*d,3)+1.3;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_birch_Siipilehto (double d, double D_gM, double H_gM,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	double a0, a1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (H_gM<=1.3)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_gM<=1.3(H_gM=",H_gM);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	a1=0.2754-0.004176*H_gM;
	a0=(D_gM/pow(H_gM-1.3,(1.0/2.0)))-a1*D_gM;

	*nres = 1;
	double res =pow(d,2)/pow(a0+a1*d,2)+1.3;

	*modelresult = res * rectFactor;
	return ret;

}


int Height_grey_alder_Karki (double d, double H_dom,
									int *nres, double *modelresult, char *errors, int errorCheckMode,
									double allowedRiskLevel, double rectFactor) {

	int ret = 1;




	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.4223-5.5862*(1/d)+0.0343*H_dom)+1.3;

	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_hemib_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(2.075+0.416*log(10*h)+0.1/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_pine_southb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.306+0.525*log(10*h)+0.163/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Age_pine_middleb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.306+0.586*log(10*h)+0.152/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_pine_northb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.206+0.671*log(10*h)+0.15/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_spruce_hemib_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.884+0.430*log(10*h)+0.098/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_spruce_southb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(2.113+0.394*log(10*h)+0.085/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_spruce_middleb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(2.072+0.467*log(10*h)+0.133/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_spruce_northb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(2.154+0.546*log(10*h)+0.138/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_birch_hemib_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.752+0.615*log(10*h)+0.053/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_birch_southb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.858+0.592*log(10*h)+0.121/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_birch_middleb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(0.991+0.613*log(10*h)+0.121/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_birch_northb_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;



	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	double res =exp(1.166+0.678*log(10*h)+0.118/2);

	*modelresult = res * rectFactor;
	return ret;
}



int Age_pine_entire_country_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.376+0.556*log(10*h)+0.189/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_spruce_entire_country_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.085+0.429*log(10*h)+0.142/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Age_birch_entire_country_h_KalliovirtaTokola (double h, int *nres, double *modelresult, char *errors,
										 int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.252+0.544*log(10*h)+0.155/2);

	*modelresult = res * rectFactor;
	return ret;
}


int Log_volume_reduction_pine_Mehtatalo (double a, double d, double LAT, double ALT, double AHV, double PEAT, double SC,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC1_2, SC5_6;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1_2 = 1;
			SC5_6 = 0;
			break;
		case 2:
			SC1_2 = 1;
			SC5_6 = 0;
			break;
		case 3:
			SC1_2 = 0;
			SC5_6 = 0;
			break;
		case 4:
			SC1_2 = 0;
			SC5_6 = 0;
			break;
		case 5:
			SC1_2 = 0;
			SC5_6 = 1;
			break;
		case 6:
			SC1_2 = 0;
			SC5_6 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5_6
		case 7:
			SC1_2 = 0;
			SC5_6 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5_6
		case 8:
			SC1_2 = 0;
			SC5_6 = 1;
			break;
		// defaults to SC==3 or SC==4, sensible?
		default:
			SC1_2 = 0;
			SC5_6 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (LAT<=6600)
	{
		ret = 0;
		constructErrorMessage (errorStr, "LAT<=6600(LAT=",LAT);
	}
	if (ALT<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "ALT<=0(ALT=",ALT);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c =-7.301+0.03814*a-3.568*log(a)-0.2184*d+0.00309*pow(d,2)+0.004176*LAT-0.7974*log(LAT-6600)-0.4549*log(ALT+1)+1.59*AHV+0.2832*PEAT+0.4607*(SC1_2)+0.6323*(SC5_6);

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res = exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_pine_north_Finland_Mehtatalo (double a, double d, double LONG,
							double ALT, double TS, double PEAT, double SC, double LAT,
							int *nres, double *modelresult, char *errors,
							int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	double ALTat;
	double tykky;
	int SC1_2;

	tykky=(11./80.)*LAT-741.25;
	if(ALT<tykky) {
		ALTat=0;
	} else {
		ALTat=ALT-tykky;
	}

	if(SC==1||SC==2) {
		SC1_2=1;
	} else {
		SC1_2=0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (d<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<0(d=",d);
	}
	if (ALT<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "ALT<=0(ALT=",ALT);
	}
	if (TS<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "TS<=0(TS=",TS);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c=-21.95+5.144*log(a)+98.19/sqrt(a)+0.1714*d+131.9/d-0.00128*LONG-0.2909*log(ALT+1)+0.02053*ALTat-2.939*log(TS)+0.5456*PEAT+0.6029*(SC1_2);

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res =exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_pine_scrub_land_Mehtatalo (double a, double d, double ALT, double LAT,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double ALTat;
	double tykky;

	tykky=(11./80.)*LAT-741.25;
	if(ALT<tykky) {
		ALTat=0;
	} else {
		ALTat=ALT-tykky;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c=-12.77+2.227*log(a)+206.5/a+0.07324*d-0.004133*ALT+0.01416*ALTat;

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res =exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_spruce_Mehtatalo (double a, double d, double PLANT,
							double ALT, double AHV, double TS, double PEAT, double SC,
							int *nres, double *modelresult, char *errors,
							int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double minalt;
	int SC1_2;

	if(ALT<60) {
		minalt=ALT;
	} else {
		minalt=60;
	}

	if(SC==1||SC==2) {
		SC1_2=1;
	} else {
		SC1_2=0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c=20.61-0.004853*a+0.0000556*pow(a,2)+0.1893*d-8.186*log(d)+1.493*(d/a)+0.2884*PLANT-0.016*minalt+0.7182*AHV+0.1538*PEAT
		+3.638*(SC1_2)-0.002493*(TS*(SC1_2))-0.003826*(ALT*(SC1_2));

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res =exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_spruce_north_Finland_Mehtatalo (double a, double d,
						double LONG, double ALT, double TS, double PEAT, double LAT,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double ALTat;
	double tykky;

	tykky=(11./80.)*LAT-741.25;
	if(ALT<tykky) {
		ALTat=0;
	} else {
		ALTat=ALT-tykky;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c=6.841+0.01229*a+96.45/a+8.786*log(d)+298.1/d-0.01602*LONG-3407/LONG-0.005684*ALT+0.01629*ALTat-5.066*log(TS)+0.3361*PEAT;

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res =exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_spruce_scrub_land_north_Finland_Mehtatalo (double a, double d,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c = 47.76+2.668*log(a)+0.8221*d-25.11*log(d);

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res =exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_white_birch_Mehtatalo (double a, double d, double LAT,
						double ALT, double AHV, double POHJ, double SC,
					    int *nres, double *modelresult, char *errors,
					    int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	double minalt;
	int SC5_6;

	if(ALT<120) {
		minalt=ALT;
	} else {
		minalt=120;
	}

	if(SC==5||SC==6) {
		SC5_6=1;
	} else {
		SC5_6=0;
	}

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c =-13.21-0.02215*a+0.0001727*pow(a,2)+0.1046*d+73.49/d+0.001167*LAT+0.006862*ALT-0.01092*minalt+1.401*AHV+0.613*POHJ+2.239*(SC5_6);

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res = exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_pubescent_birch_Mehtatalo (double a, double d, double LAT, double AHV,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (a<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "a<=0(a=",a);
	}
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (LAT<=6600)
	{
		ret = 0;
		constructErrorMessage (errorStr, "LAT<=6600(LAT=",LAT);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c = -34.42+0.0532*a-3.675*log(a)+0.146*d+81.25/d+0.00661*LAT-1.106*log(LAT-6600)+1.871*AHV;
	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res = exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Log_volume_reduction_aspen_Mehtatalo (double a, double d, double LAT,
						double LONG, double ALT, double POHJ, double PEAT,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (LAT<=6600)
	{
		ret = 0;
		constructErrorMessage (errorStr, "LAT<=6600(LAT=",LAT);
	}
	if (LONG<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "LONG<=0(LONG=",LONG);
	}
	if (ALT<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "ALT<=0(ALT=",ALT);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	double c = -6.205+0.01137*a+0.02394*d+0.7191*log(LAT-6600)+0.7884*log(LONG)+0.01686*ALT-1.186*log(ALT+1)+0.5157*POHJ-0.7854*PEAT;

	*nres = 1;
	//Relative reduction of total log volume caused by any kind of defects
	double res = exp(c)/(1+exp(c));

	*modelresult = res * rectFactor;
	return ret;

}


int Volume_pine_Laasasenaho (double d, double h,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	//Volume m3
	double res = (exp(-3.2890 +1.9995*log(d) +2.1395*log(h)-1.1411*log(h-1.3) -0.002847*d))/1000;
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_spruce_Laasasenaho (double d, double h,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	//Volume m3
	double res =(exp(-3.7544 +1.8960*log(d) +2.8979*log(h)-1.6020*log(h-1.3) -0.007827*d))/1000;
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_birch_Laasasenaho (double d, double h,
			int *nres, double *modelresult, char *errors,
			int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;

	//Volume m3
	double res =(exp(-4.4759 +2.0851*log(d) +4.0691*log(h)-2.7375*log(h-1.3) -0.013311*d))/1000;
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_pine_upper_height_Laasasenaho (double d, double d6, double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;
	double res = 0.268621*pow(d,2)-0.0145543*pow(d,2)*h-0.0000478628*pow(d,3)*h+0.000334101*pow(d,2)*pow(h,2)+0.0973148*(pow(d,2)+d*d6+d6)+0.0440716*pow(d6,2)*(h-6);
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_spruce_upper_height_Laasasenaho (double d, double d6, double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;
	double res =0.208043*pow(d,2)-0.0149567*pow(d,2)*h -0.000114406*pow(d,3)*h+0.000436781*pow(d,2)*pow(h,2)+0.133947*(pow(d,2)+d*d6+d6)+0.0374599*pow(d6,2)*(h-6);
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_birch_upper_height_Laasasenaho (double d, double d6, double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;
	double res = 0.226547*pow(d,2)-0.0104691*pow(d,2)*h-0.000122258*pow(d,3)*h+0.000438033*pow(d,2)*pow(h,2)+0.0991620*(pow(d,2)+d*d6+d6)+0.0334836*pow(d6,2)*(h-6);
	*modelresult = res * rectFactor;
	return ret;

}


int Volume_grey_alder_Karki (double d, double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	//Volume m3
	double res =(exp(0.6042-0.4641*d+0.0046*pow(d,2)+1.2356*pow(log(d),2)+0.0557*pow(log(h),3)+0.0085/2))/1000;
	*modelresult = res * rectFactor;
	return ret;

}



int Age_pine_Vahasaari (double d, double SC,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor)
{

	int ret = 1;


	int SC3, SC4, SC5;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 2:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
		case 3:
			SC3 = 1;
			SC4 = 0;
			SC5 = 0;
			break;
		case 4:
			SC3 = 0;
			SC4 = 1;
			SC5 = 0;
			break;
		case 5:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC5
		case 6:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC5
		case 7:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC5
		case 8:
			SC3 = 0;
			SC4 = 0;
			SC5 = 1;
			break;
		// defaults to SC==1 or SC=2, sensible?
		default:
			SC3 = 0;
			SC4 = 0;
			SC5 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res = exp(2.259+0.8456*log(d)-1.047*SC3-0.6458*SC4-0.5051*SC5);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_Korhonen (double d, double LGP, double SC,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	int SC2, SC3, SC4;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC2 = 0;
			SC3 = 0;
			SC4 = 0;
			break;
		case 2:
			SC2 = 1;
			SC3 = 0;
			SC4 = 0;
			break;
		case 3:
			SC2 = 0;
			SC3 = 1;
			SC4 = 0;
			break;
		case 4:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 5 not present in original publication, treated here as SC4
		case 5:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 6 not present in original publication, treated here as SC4
		case 6:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 7 not present in original publication, treated here as SC4
		case 7:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
		// SC 8 not present in original publication, treated here as SC4
		case 8:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
		// defaults to SC==1 or SC=2, sensible?
		default:
			SC2 = 0;
			SC3 = 0;
			SC4 = 1;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(17.2+1.1241*log(d)-0.03061*d+0.0004144*pow(d,2)+0.01663*SC2+0.01127*SC3+0.009978*SC4-0.1804*LGP+0.0005197*pow(LGP,2)+pow(0.4189,2)/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(3.393+0.200*log(d_crm)+0.114/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.326+0.475*log(d_crm)+0.108/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.655+0.329*log(d_crm)+0.091/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.902+0.263*log(d_crm)+0.264/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.380+0.457*log(d_crm)+0.118/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.064+0.468*log(d_crm)+0.138/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.806+0.356*log(d_crm)+0.22/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.476+0.513*log(d_crm)+0.175/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.284+0.446*log(d_crm)+0.127/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.870+0.415*log(d_crm)+0.192/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(3.064+0.463*log(d_crm)+0.168/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.165+0.572*log(d_crm)+0.115/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.840+0.332*log(d_crm)+0.252/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.459+0.474*log(d_crm)+0.173/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.225+0.454*log(d_crm)+0.146/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.407+0.520*log(10*h)+0.040*log(d_crm)+0.192/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.019+0.398*log(10*h)+0.061*log(d_crm)+0.143/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.264+0.371*log(10*h)+0.232*log(d_crm)+0.144/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.150+0.346*log(10*h)+0.075*log(d_crm)+0.099/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.705+0.338*log(10*h)+0.180*log(d_crm)+0.093/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(0.788+0.590*log(10*h)+0.025*log(d_crm)+0.052/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.282+0.544*log(10*h)-0.020*log(d_crm)+0.162/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.994+0.335*log(10*h)+0.115*log(d_crm)+0.085/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(0.896+0.452*log(10*h)+0.180*log(d_crm)+0.117/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.319+0.569*log(10*h)+0.019*log(d_crm)+0.153/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.015+0.441*log(10*h)+0.053*log(d_crm)+0.135/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.016+0.455*log(10*h)+0.207*log(d_crm)+0.116/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.303+0.554*log(10*h)+0.130*log(d_crm)+0.15/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.095+0.530*log(10*h)+0.038*log(d_crm)+0.139/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (d_crm<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<=0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.014+0.508*log(10*h)+0.260*log(d_crm)+0.108/2);
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.155+0.730*sqrt(10*h)+1.323*sqrt(d_crm),2)+0.869)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.214+0.861*sqrt(10*h)+1.016*sqrt(d_crm),2)+0.948)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_entire_country_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.341+0.700*sqrt(10*h)+1.143*sqrt(d_crm),2)+1.00)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.324+0.910*sqrt(10*h)+1.029*sqrt(d_crm),2)+1.011)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-5.512+0.800*sqrt(10*h)+1.512*sqrt(d_crm),2)+0.801)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_hemib_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-6.978+0.972*sqrt(10*h)+1.271*sqrt(d_crm),2)+1.019)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.524+0.729*sqrt(10*h)+1.345*sqrt(d_crm),2)+0.811)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.835+0.860*sqrt(10*h)+1.079*sqrt(d_crm),2)+0.742)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_southb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-4.250+0.804*sqrt(10*h)+1.028*sqrt(d_crm),2)+0.78)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.306+0.743*sqrt(10*h)+1.334*sqrt(d_crm),2)+0.723)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.739+0.920*sqrt(10*h)+0.868*sqrt(d_crm),2)+0.741)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_middleb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.420+0.741*sqrt(10*h)+1.107*sqrt(d_crm),2)+0.787)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.734+0.797*sqrt(10*h)+1.230*sqrt(d_crm),2)+1.05)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.948+1.013*sqrt(10*h)+0.962*sqrt(d_crm),2)+0.858)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_northb_h_d_crm_KalliovirtaTokola (double h, double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.770+0.886*sqrt(10*h)+1.108*sqrt(d_crm),2)+1.185)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_entire_country_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.801+1.204*sqrt(10*h),2)+2.057)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_entire_country_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.524+1.145*sqrt(10*h),2)+1.408)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_entire_country_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.591+1.153*sqrt(10*h),2)+1.854)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_hemib_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.775+1.314*sqrt(10*h),2)+1.829)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_hemib_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.740+1.228*sqrt(10*h),2)+1.94)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_hemib_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-5.533+1.475*sqrt(10*h),2)+2.14)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_southb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.785+1.177*sqrt(10*h),2)+1.986)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_southb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.960+1.168*sqrt(10*h),2)+1.316)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_southb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.938+1.226*sqrt(10*h),2)+1.39)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_middleb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.948+1.218*sqrt(10*h),2)+1.833)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_middleb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.457+1.161*sqrt(10*h),2)+1.008)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_middleb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.984+1.206*sqrt(10*h),2)+1.524)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_northb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.854+1.389*sqrt(10*h),2)+2.267)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_northb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.343+1.275*sqrt(10*h),2)+1.247)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_northb_h_KalliovirtaTokola (double h,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.480+1.327*sqrt(10*h),2)+1.893)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(0.548+2.065*sqrt(d_crm),2)+2.309)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.600+2.719*sqrt(d_crm),2)+3.056)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_entire_country_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.982+2.019*sqrt(d_crm),2)+1.77)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(2.628+1.796*sqrt(d_crm),2)+2.844)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.444+2.774*sqrt(d_crm),2)+2.418)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_hemib_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.867+2.419*sqrt(d_crm),2)+3.144)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.444+2.226*sqrt(d_crm),2)+2.597)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-4.088+2.769*sqrt(d_crm),2)+3.071)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_southb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.492+2.106*sqrt(d_crm),2)+1.89)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.547+2.221*sqrt(d_crm),2)+2.088)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.841+2.594*sqrt(d_crm),2)+2.911)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_middleb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.363+1.896*sqrt(d_crm),2)+1.485)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(1.177+1.957*sqrt(d_crm),2)+1.966)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.005+2.422*sqrt(d_crm),2)+2.966)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_northb_d_crm_KalliovirtaTokola (double d_crm,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.721+1.982*sqrt(d_crm),2)+2.061)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_entire_country_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double LAT,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-11.934+0.752*sqrt(10*h)+1.311*sqrt(d_crm)+0.00122*LAT,2)+0.806)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_entire_country_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double TS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(0.088+0.876*sqrt(10*h)+1.033*sqrt(d_crm)-0.00312*TS,2)+0.738)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_entire_country_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double TS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-0.656+0.805*sqrt(10*h)+1.056*sqrt(d_crm)-0.00302*TS,2)+0.838)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_hemib_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double Age,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-3.913+0.890*sqrt(10*h)+0.985*sqrt(d_crm)+0.016*Age,2)+0.875)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_hemib_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double BA,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-4.201+0.802*sqrt(10*h)+1.490*sqrt(d_crm)-0.047*BA,2)+0.702)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_hemib_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double D_gM,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-6.863+0.965*sqrt(10*h)+1.051*sqrt(d_crm)+0.058*D_gM,2)+0.986)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_southb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.978+0.721*sqrt(10*h)+1.279*sqrt(d_crm)-0.379*dcl,2)+0.801)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_southb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.836+0.803*sqrt(10*h)+1.059*sqrt(d_crm)-0.493*dcl,2)+0.694)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_southb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.859+0.731*sqrt(10*h)+0.981*sqrt(d_crm)-0.738*dcl,2)+0.681)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_middleb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.311+0.712*sqrt(10*h)+1.233*sqrt(d_crm)-0.538*dcl,2)+0.684)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_middleb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double TS,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(1.469+0.934*sqrt(10*h)+0.888*sqrt(d_crm)-0.00430*TS,2)+0.611)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_middleb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-1.688+0.666*sqrt(10*h)+0.996*sqrt(d_crm)-0.789*dcl,2)+0.648)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_pine_northb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double Age,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.728+0.751*sqrt(10*h)+1.234*sqrt(d_crm)-0.005*Age,2)+0.964)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_spruce_northb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double BA,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.287+1.042*sqrt(10*h)+0.949*sqrt(d_crm)-0.053*BA,2)+0.699)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_birch_northb_h_d_crm_c_KalliovirtaTokola (double h, double d_crm, double dcl,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<0(h=",h);
	}
	if (d_crm<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d_crm<0(d_crm=",d_crm);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =(pow(-2.141+0.745*sqrt(10*h)+1.134*sqrt(d_crm)-0.806*dcl,2)+1.052)/10;
	*modelresult = res * rectFactor;
	return ret;

}


int Diameter_at_stump_height_Laasasenaho (double d, int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	*nres = 1;
	double res;
	if (d>0)
		res = 2+1.25*d;
	else
		res = 0;
	*modelresult = res * rectFactor;
	return ret;

}

int Crown_ratio_pine_Hynynenym (double d, double H_domvmi, double RDF, double RDF_L, double TS, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Assign time since thinning variables

	double THIN_0_5, THIN_5_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;
	if(SINCE_THINNING > 5  &&SINCE_THINNING <=10)
		THIN_5_10 = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF_L<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_L<0(RDF_L=",RDF_L);
	}
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1-exp(-(4.90911-0.28807*THIN_0_5-0.21357*THIN_5_10)*pow(H_domvmi,-0.76199)*pow(d,0.14438)*
		exp(-0.118*RDF_L)*pow((TS/1000),-0.47057)*exp(-0.2949*RDF));
	*modelresult = res * rectFactor;
	return ret;

}


int Crown_ratio_pine_with_bias_corr_Hynynenym (double d, double H_domvmi, double RDF, double RDF_L, double TS,
									double ALT, double SEA, double LAKE, double SI_50, double BA, double SC, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Assign time since thinning variables

	double THIN_0_5 = 0;
	double THIN_5_10 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;
	if(SINCE_THINNING > 5  &&SINCE_THINNING <=10)
		THIN_5_10 = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF_L<0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF_L<0(RDF_L=",RDF_L);
	}
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	int SC2, SC6, SC8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC2 = 0;
			SC6 = 0;
			SC8 = 0;
			break;
		case 2:
			SC2 = 1;
			SC6 = 0;
			SC8 = 0;
			break;
		case 3:
			SC2 = 0;
			SC6 = 0;
			SC8 = 0;
			break;
		case 4:
			SC2 = 0;
			SC6 = 0;
			SC8 = 0;
			break;
		case 5:
			SC2 = 0;
			SC6 = 0;
			SC8 = 0;
			break;
		case 6:
			SC2 = 0;
			SC6 = 1;
			SC8 = 0;
			break;
		case 7:
			SC2 = 0;
			SC6 = 0;
			SC8 = 0;
			break;
		case 8:
			SC2 = 0;
			SC6 = 0;
			SC8 = 1;
			break;
	};


	//Bias correction model:
	double bias = -0.40141-0.7908*(TS/1000)+0.47168*pow(TS/1000,2)+0.00022*ALT+0.06588*SEA+0.06236*LAKE+0.02304*H_domvmi
			-0.0004*pow(H_domvmi,2)+0.06649*SI_50-0.00191*pow(SI_50,2)-0.00967*BA+0.00013*pow(BA,2)+0.01594*SC2+0.06116*SC6+0.09373*SC8;

	*nres = 1;
	//Without calibration
	double res1 =1-exp(-(4.90911-0.28807*THIN_0_5-0.21357*THIN_5_10)*pow(H_domvmi,-0.76199)*pow(d,0.14438)*
			exp(-0.118*RDF_L)*pow((TS/1000),-0.47057)*exp(-0.2949*RDF));
	//With calibration
	double res2 =1-exp(-(4.90911-0.28807*THIN_0_5-0.21357*THIN_5_10)*pow(H_domvmi,-0.76199)*pow(d,0.14438)*
			exp(-0.118*RDF_L)*pow((TS/1000),-0.47057)*exp(-0.2949*RDF))+bias;

	//Calibrated value as default
	double res=res2;
	if(res2<0.5*res1)
		res=0.5*res1;
	if(res2>1.5*res1)
		res=1.5*res1;

	//If crown ratio is greater than 1:
	if(res>1)
		res=1;

	*modelresult = res * rectFactor;
	return ret;

}


int Crown_ratio_spruce_Hynynenym (double d, double H_domvmi, double RDF, double TS, double SI_50, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Assign time since thinning variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1-exp(-(2.05788-0.16547*THIN_0_5)*pow(H_domvmi,-0.71419)*pow(d,0.25755)*exp(-0.38242*RDF)*
		pow((TS/1000),-0.54008)*pow(SI_50,0.42485));
	*modelresult = res * rectFactor;
	return ret;

}


int Crown_ratio_spruce_with_bias_corr_Hynynenym (double d, double H_domvmi, double RDF, double TS, double SI_50,
									double SEA, double LAKE, double BA_prosb, double BA, double h, double SC, double SCRUB, double SINCE_THINNING,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Assign time since thinning variables

	double THIN_0_5 = 0;
	if(SINCE_THINNING >= 0 &&SINCE_THINNING <=5)
		THIN_0_5 = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (RDF<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "RDF<=0(RDF=",RDF);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	int SC12, SC4, SC5, SC7, SC8;
	int iSC = (int) SC;
	switch(iSC)
	{
		case 1:
			SC12 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 2:
			SC12 = 1;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 3:
			SC12 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 4:
			SC12 = 0;
			SC4 = 1;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 5:
			SC12 = 0;
			SC4 = 0;
			SC5 = 1;
			SC7 = 0;
			SC8 = 0;
			break;
		case 6:
			SC12 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 0;
			break;
		case 7:
			SC12 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 1;
			SC8 = 0;
			break;
		case 8:
			SC12 = 0;
			SC4 = 0;
			SC5 = 0;
			SC7 = 0;
			SC8 = 1;
			break;
	};

	//Bias correction model:
	double bias = 0.05697-0.45311*(TS/1000)+0.26348*pow(TS/1000,2)+0.02065*SEA+0.0511*LAKE+0.07898*(BA_prosb/100)+0.01377*H_domvmi
			-0.0001*pow(H_domvmi,2)-0.01333*log(BA)-0.00358*h-0.00866*SC12+0.0581*SC4+0.10011*SC5+0.07718*SC7+0.09683*SC8-0.10671*SCRUB;

	*nres = 1;
	//Without calibration
	double res1 =1-exp(-(2.05788-0.16547*THIN_0_5)*pow(H_domvmi,-0.71419)*pow(d,0.25755)*exp(-0.38242*RDF)*
		pow((TS/1000),-0.54008)*pow(SI_50,0.42485));
	//With calibration
	double res2 =1-exp(-(2.05788-0.16547*THIN_0_5)*pow(H_domvmi,-0.71419)*pow(d,0.25755)*exp(-0.38242*RDF)*
		pow((TS/1000),-0.54008)*pow(SI_50,0.42485))+bias;

	//Calibrated value as default
	double res=res2;
	if(res2<0.5*res1)
		res=0.5*res1;
	if(res2>1.5*res1)
		res=1.5*res1;

	//If crown ratio is greater than 1:
	if(res>1)
		res=1;

	*modelresult = res * rectFactor;
	return ret;

}


int Crown_ratio_deciduous_Hynynenym (double d, double h, double H_domvmi, double RDF, double PLANT,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =1-exp(-(2.02622+1.7165*PLANT)*pow(log(H_domvmi),-0.62383)*pow(d,0.54674)*pow(log(h),-1.67423)*
		exp(-(0.32816+0.6403*PLANT)*RDF));
	*modelresult = res * rectFactor;
	return ret;

}


int Crown_ratio_deciduous_with_bias_corr_Hynynenym (double d, double h, double H_domvmi, double RDF, double PLANT,
									double ALT, double LAKE, double SEA, double BA, double SC, double SCRUB,
									double TS, double BA_prosb, double SP,
									int *nres, double *modelresult, char *errors,
									int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (h<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "h<=0(h=",h);
	}
	if (H_domvmi<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "H_domvmi<=0(H_domvmi=",H_domvmi);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	//Bias correction models:
	double bias;

	if(SP==3) {

		int SC1, SC2, SC4;
		int iSC = (int) SC;
		switch(iSC)
		{
			case 1:
				SC1 = 1;
				SC2 = 0;
				SC4 = 0;
				break;
			case 2:
				SC1 = 0;
				SC2 = 1;
				SC4 = 0;
				break;
			case 3:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				break;
			case 4:
				SC1 = 0;
				SC2 = 0;
				SC4 = 1;
				break;
			case 5:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				break;
			case 6:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				break;
			case 7:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				break;
			case 8:
				SC1 = 0;
				SC2 = 0;
				SC4 = 0;
				break;
		};
		bias = -0.21277-0.00067*ALT+0.17392*LAKE-0.1008*SEA+0.01476*H_domvmi-0.00034*pow(H_domvmi,2)+0.00503*BA+0.04756*SC1
				+0.05121*SC2+0.05415*SC4-0.09836*SCRUB;
	}
	if(SP==4) {
		bias = -0.34492-0.37472*(TS/1000)+0.27911*pow(TS/1000,2)+0.00012*ALT+0.1174*LAKE+0.01*H_domvmi-0.00012*pow(H_domvmi,2)
				+0.0468*(BA_prosb/100)-0.00222*BA+0.00014*pow(BA,2)+0.03358*h-0.00138*pow(h,2)-0.11774*SCRUB;
	}
	if(SP>=5) {
		int sp5, sp6, sp7, sp89;
		int isp = (int) SP;
		switch(isp)
		{
			case 5:
				sp5 = 1;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
			case 6:
				sp5 = 0;
				sp6 = 1;
				sp7 = 0;
				sp89 = 0;
				break;
			case 7:
				sp5 = 0;
				sp6 = 0;
				sp7 = 1;
				sp89 = 0;
				break;
			case 8:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			case 9:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 1;
				break;
			// defaults to sp==3 or 4, sensible?
			default:
				sp5 = 0;
				sp6 = 0;
				sp7 = 0;
				sp89 = 0;
				break;
		};
		bias = -0.39508-0.21996*(TS/1000)+0.20236*pow(TS/1000,2)+0.0001*ALT+0.05426*LAKE+0.01376*H_domvmi-0.0002*pow(H_domvmi,2)+0.01907*(BA_prosb/100)
				-0.00359*BA+0.00014*pow(BA,2)+0.02664*h-0.00101*pow(h,2)-0.08463*SCRUB-0.04507*sp5-0.04709*sp6-0.1012*sp7+0.05698*sp89;
	}

	*nres = 1;
	//Without calibration
	double res1 =1-exp(-(2.02622+1.7165*PLANT)*pow(log(H_domvmi),-0.62383)*pow(d,0.54674)*pow(log(h),-1.67423)*
			exp(-(0.32816+0.6403*PLANT)*RDF));
	//With calibration
	double res2 =1-exp(-(2.02622+1.7165*PLANT)*pow(log(H_domvmi),-0.62383)*pow(d,0.54674)*pow(log(h),-1.67423)*
			exp(-(0.32816+0.6403*PLANT)*RDF))+bias;

	//Calibrated value as default
	double res=res2;
	if(res2<0.5*res1)
		res=0.5*res1;
	if(res2>1.5*res1)
		res=1.5*res1;

	//If crown ratio is greater than 1:
	if(res>1)
		res=1;

	*modelresult = res * rectFactor;
	return ret;

}


int Basal_area_of_tree_m2_JKK (double d,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	*nres = 1;

	if (d < 0.001) {
		d = 0.001;
	}

	d = d / 100;

	*modelresult = PI * pow(d / 2, 2);
	return ret;

}


int Basal_area_of_tree_JKK (double d,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	if (d < 0.001) {
		d = 0.001;
	}

	*nres = 1;

	*modelresult = PI * pow(d / 2, 2);
	return ret;

}

int Diameter_under_bark_Melkas_Laasasenaho (double d, double sp,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;


	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double bark;

	if(sp==1||sp==8) {
		bark = exp(-2.37+1.03*log(d));
	}
	else if(sp==2) {
		bark = exp(-1.66+0.65*log(d));
	}
	else {
		bark = exp(-2.56+0.97*log(d));
	}

	double res = d - bark;

	*modelresult = res * rectFactor;
	return ret;

}


int Age_pine_SF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;

	int SC12, SC3, SC5, SC678;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC12 = 1;
			SC3 = 0;
			SC5 = 0;
			SC678 = 0;
			break;
		case 2:
			SC12 = 1;
			SC3 = 0;
			SC5 = 0;
			SC678 = 0;
			break;
		case 3:
			SC12 = 0;
			SC3 = 1;
			SC5 = 0;
			SC678 = 0;
			break;
		case 4:
			SC12 = 0;
			SC3 = 0;
			SC5 = 0;
			SC678 = 0;
			break;
		case 5:
			SC12 = 0;
			SC3 = 0;
			SC5 = 1;
			SC678 = 0;
			break;
		case 6:
			SC12 = 0;
			SC3 = 0;
			SC5 = 0;
			SC678 = 1;
			break;
		case 7:
			SC12 = 0;
			SC3 = 0;
			SC5 = 0;
			SC678 = 1;
			break;
		case 8:
			SC12 = 0;
			SC3 = 0;
			SC5 = 0;
			SC678 = 1;
			break;
		// defaults to SC==4, sensible?
		default:
			SC12 = 0;
			SC3 = 0;
			SC5 = 0;
			SC678 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(1.5959+0.9185*log(d)-0.0002*TS-0.3166*SC12-0.1654*SC3+0.1514*SC5+0.2599*SC678
				+0.2496*PEAT+0.3948*LOW);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_SF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;


	int SC1, SC2, SC5678;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC5678 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC5678 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 0;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 1;
			break;
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 1;
			break;
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 1;
			break;
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 1;
			break;
		// defaults to SC==4 or SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC5678 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(3.3602+0.6446*log(d)-0.0010*TS-0.3505*SC1-0.1929*SC2+0.1867*SC5678
				+0.1207*PEAT+0.3874*LOW);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_SF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;


	int SC1, SC2, SC45678;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC1 = 1;
			SC2 = 0;
			SC45678 = 0;
			break;
		case 2:
			SC1 = 0;
			SC2 = 1;
			SC45678 = 0;
			break;
		case 3:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 0;
			break;
		case 4:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 1;
			break;
		case 5:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 1;
			break;
		case 6:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 1;
			break;
		case 7:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 1;
			break;
		case 8:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC1 = 0;
			SC2 = 0;
			SC45678 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.6504+0.7004*log(d)-0.0006*TS-0.3480*SC1-0.1740*SC2+0.0532*SC45678
				+0.0984*PEAT+0.2195*LOW);
	*modelresult = res * rectFactor;
	return ret;

}

int Age_pine_NF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;

	int SC123;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC123 = 1;
			break;
		case 2:
			SC123 = 1;
			break;
		case 3:
			SC123 = 1;
			break;
		case 4:
			SC123 = 0;
			break;
		case 5:
			SC123 = 0;
			break;
		case 6:
			SC123 = 0;
			break;
		case 7:
			SC123 = 0;
			break;
		case 8:
			SC123 = 0;
			break;
		// defaults to SC>=4, sensible?
		default:
			SC123 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(2.6871+0.9107*log(d)-0.0012*TS-0.1813*SC123+0.2254*PEAT+0.3646*LOW);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_spruce_NF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;


	int SC12;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC12 = 1;
			break;
		case 2:
			SC12 = 1;
			break;
		case 3:
			SC12 = 0;
			break;
		case 4:
			SC12 = 0;
			break;
		case 5:
			SC12 = 0;
			break;
		case 6:
			SC12 = 0;
			break;
		case 7:
			SC12 = 0;
			break;
		case 8:
			SC12 = 0;
			break;
		// defaults to SC>=3, sensible?
		default:
			SC12 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(3.5221+0.7638*log(d)-0.0013*TS-0.2104*SC12+0.2272*PEAT+0.1180*LOW);
	*modelresult = res * rectFactor;
	return ret;

}


int Age_birch_NF_JKK (double d, double TS, double SC, double PEAT, double SCRUB, double WASTE,
					int *nres, double *modelresult, char *errors,
					int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;

	int LOW=0;

	if (SCRUB==1||WASTE==1)
		LOW=1;


	int SC12, SC45678;
	int iSC = (int) SC;
	switch (iSC)
	{
		case 1:
			SC12 = 1;
			SC45678 = 0;
			break;
		case 2:
			SC12 = 1;
			SC45678 = 0;
			break;
		case 3:
			SC12 = 0;
			SC45678 = 0;
			break;
		case 4:
			SC12 = 0;
			SC45678 = 1;
			break;
		case 5:
			SC12 = 0;
			SC45678 = 1;
			break;
		case 6:
			SC12 = 0;
			SC45678 = 1;
			break;
		case 7:
			SC12 = 0;
			SC45678 = 1;
			break;
		case 8:
			SC12 = 0;
			SC45678 = 1;
			break;
		// defaults to SC==3, sensible?
		default:
			SC12 = 0;
			SC45678 = 0;
			break;
	};

	//Check for fatal errors
	char errorStr[200] = "";
	if (d<=0)
	{
		ret = 0;
		constructErrorMessage (errorStr, "d<=0(d=",d);
	}
	if (ret == 0) {
		strcat(errors, errorStr);
		return 0;
	}

	*nres = 1;
	double res =exp(3.7912+0.8048*log(d)-0.0019*TS-0.1074*SC12+0.1136*SC45678+0.1302*PEAT+0.1591*LOW);
	*modelresult = res * rectFactor;
	return ret;

}

int Stem_number_plot_JKK (double d, double inventory_method, double relascope_factor, double radius,
					   int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor) {

	int ret = 1;
    double res;

    //relascope plot
    if (inventory_method==0)
    {
        res = 10000*relascope_factor/(PI*pow((d/2),2));
    }
    //ympyrkoeala
    else
    {
        res = 10000/(PI*pow(radius,2));
    }

	*nres = 1;

	*modelresult = res;
	return ret;

}

int Tree_height_JKK (double d, double Age, double BA, double D_gM, double H_dom, double SP,
				double TS, double ALT, double SC, double PEAT, double PEAT_transf,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

	//Mineral soil
	if (PEAT==0)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Silver birch
		else if (SP==3)
		{
			ret = Height_white_birch_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Downy birch, aspen and other deciduous species
		else if (SP==4||SP==5||SP==9)
		{
			ret = Height_pubescent_birch_Veltheim(d, Age, BA, D_gM, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
			if (ret == 0)
			return ret;
		}
	//Peatland
	if (PEAT==1)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_peatland_Veltheim(d, Age, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_peatland_Veltheim(d, BA, D_gM, TS, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Birches and other deciduous species
		else if (SP==3||SP==4||SP==5||SP==9)
		{
			ret = Height_birch_peatland_Veltheim(d, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		if (ret == 0)
			return ret;

	}

	*nres = 1;
	return ret;

}

int Tree_volume_JKK (double d, double h, double SP,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate tree volume
	//Calculate volume for seedlings (h less than 1,3 m)
	if (h<1.3)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Volume_seedling_pine_Tomppoym(h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Volume_seedling_spruce_Tomppoym(h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		else
		{
			ret = Volume_seedling_birch_Tomppoym(h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

	}
	//Calculate volume for small and big trees (h greater or equal as 1,3 m)
	else
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			if (h<3)
			{
				ret = Volume_small_tree_pine_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}
			else
			{
				ret = Volume_pine_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}

			if (ret == 0)
			return ret;

		}
		//Norway spruce
		else if (SP==2)
		{
			if (h<3)
			{
				ret = Volume_small_tree_spruce_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}
			else
			{
				ret = Volume_spruce_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}

			if (ret == 0)
			return ret;

		}
		//Birches and other deciduous species
		else if (SP==3||SP==4||SP==5||SP==9||((SP==6||SP==7)&&(d>30||h>25)))
		{
			if (h<4)
			{
				ret = Volume_small_tree_birch_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}
			else
			{
				ret = Volume_birch_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}

			if (ret == 0)
			return ret;

		}
		//alder
		else
		{
			if (h<4)
			{
				ret = Volume_small_tree_birch_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}
			else
			{
				ret = Volume_grey_alder_Karki(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}

			if (ret == 0)
			return ret;

		}
	}

	*nres = 1;
	return ret;

}
int Tree_volume_plot_JKK (double d, double d6, double h, double SP,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate tree volume for tree, when d exists
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		if (h<3)
		{
			ret = Volume_small_tree_pine_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Upper height exists
		else if (d6>0)
       	{
    		ret = Volume_pine_upper_height_Laasasenaho(d, d6, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
    	}
        else
        {
    		ret = Volume_pine_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
       	}

		if (ret == 0)
	    	return ret;
    }
	//Norway spruce
	else if (SP==2)
	{
		if (h<3)
		{
			ret = Volume_small_tree_spruce_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Upper height exists
		else if (d6>0)
       	{
            ret = Volume_spruce_upper_height_Laasasenaho(d, d6, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		else
		{
			ret = Volume_spruce_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;
	}
	//Birches and other deciduous species
	else if (SP==3||SP==4||SP==5||SP==9||((SP==6||SP==7)&&(d>30||h>25)))
	{
		if (h<4)
		{
			ret = Volume_small_tree_birch_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Upper height exists
		else if (d6>0)
       	{
       	    ret = Volume_birch_upper_height_Laasasenaho(d, d6, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		else
		{
			ret = Volume_birch_Laasasenaho(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

   }
		//alder
		else
		{
			if (h<4)
			{
				ret = Volume_small_tree_birch_Tomppoym(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}
			else
			{
				ret = Volume_grey_alder_Karki(d, h, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
			}

			if (ret == 0)
			return ret;

		}

	*nres = 1;
	return ret;

}

int Crown_ratio_JKK (double d, double h, double SP, double BA, double BA_prosb, double H_dom, double RDF,
						double RDF_L, double TS, double ALT, double LAKE, double SEA, double SCRUB,
						double SC, double SI_50, double STONY, double PALU, double HUMUS, double SINCE_THINNING, double PLANT,
						int *nres, double *modelresult, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Calculate site index to stratum
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		ret = Crown_ratio_pine_with_bias_corr_Hynynenym(d, H_dom, RDF, RDF_L, TS, ALT, SEA, LAKE, SI_50, BA, SC, SINCE_THINNING,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Norway spruce
	else if (SP==2)
	{
		ret = Crown_ratio_spruce_with_bias_corr_Hynynenym(d, H_dom, RDF, TS, SI_50, SEA, LAKE, BA_prosb, BA, h, SC, SCRUB, SINCE_THINNING,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}
	//Birches and other deciduous species
	else
	{
		ret = Crown_ratio_deciduous_with_bias_corr_Hynynenym(d, h, H_dom, RDF, PLANT, ALT, LAKE, SEA, BA, SC, SCRUB, TS, BA_prosb, SP,
						nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
	}

	*nres = 1;
	return ret;

}

int Tree_height_with_calibration_JKK (double d, double Age, double BA, double D_gM, double H_dom, double SP,
				double TS, double ALT, double SC, double Calibration_height, double PEAT, double PEAT_transf,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;

	//Mineral soil
	if (PEAT==0)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Silver birch
		else if (SP==3)
		{
			ret = Height_white_birch_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Downy birch, aspen and other deciduous species
		else if (SP==4||SP==5||SP==9)
		{
			ret = Height_pubescent_birch_Veltheim(d, Age, BA, D_gM, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
			if (ret == 0)
			return ret;
		}
	//Peatland
	if (PEAT==1)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_peatland_Veltheim(d, Age, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_peatland_Veltheim(d, BA, D_gM, TS, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Birches and other deciduous species
		else if (SP==3||SP==4||SP==5||SP==9)
		{
			ret = Height_birch_peatland_Veltheim(d, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		if (ret == 0)
			return ret;

	}

	//Calibrate height. Calibration height is 1, if calibration is not used

	*nres = 1;
	*modelresult = *modelresult * Calibration_height;
	//Restrict if result is impossible
	if (*modelresult>36)
	{
         *modelresult = 36;
    }
    else if (*modelresult<=1.3001)
    {
         *modelresult = 1.3002;
    }
	return ret;

}

int Biological_tree_age_JKK (double d, double SP, double t_href, double Calibration_age,
				double TS, double SC, double PEAT, double SCRUB, double WASTE, double NF,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
    // To avoid problems with seedlings
    if (d==0)
    {
    	d=0.1;
    }
	//Calculate biological tree age
	//First calculate age at breast height
	//Scots pine and other conifers
	if (SP==1||SP==8)
	{
		//Southern Finland
		if (NF==0)
		{
			ret = Age_pine_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Northern Finland (NF==1)
		else
		{
			ret = Age_pine_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

	}
	//Norway spruce
	else if (SP==2)
	{
		//Southern Finland
		if (NF==0)
		{
			ret = Age_spruce_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Northern Finland (NF==1)
		else
		{
			ret = Age_spruce_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

	}
	//Birches and other deciduous species
	else
	{
		//Southern Finland
		if (NF==0)
		{
			ret = Age_birch_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Northern Finland (NF==1)
		else
		{
			ret = Age_birch_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

	}
	//Calibrate age. Calibration age is 1, if calibration is not used
	//Calculate biological age
	double age = *modelresult + t_href;
	age = age*Calibration_age;

	*nres = 1;
	*modelresult = age;
	return ret;

}

int Tree_age_JKK (double age, double a, double t_href, double timeStep,
						int *nres, struct Tree_age *tree_age, char *errors,
						int errorCheckMode, double allowedRiskLevel, double rectFactor)
{
    int ret = 1;
	//Update tree's age at breast height

	//Calculate using biological age if a is zero
	if (a==0&&age!=t_href)
	{
		a = age - t_href;
	}

	//Update a
	a = a + timeStep;
	//If a has now negative value
	if (a<0)
	{
		a = 0;
	}

	//Update age
	age = age + timeStep;

	*nres = 1;
	tree_age[0].a=a;
	tree_age[0].age=age;
	return ret;

}

int Log_volume_reduction_JKK (double MANAGEMENT_REGION, double SCRUB, double WASTE, double SP, double a, double d, double LAT, double ALT, double AHV, double PEAT, double SC,
					   double LONG, double TS, double PLANT, double POHJ, int *nres, double *modelresult, char *errors,
					   int errorCheckMode, double allowedRiskLevel, double rectFactor)

{
	*nres = 1;
    	int ret = 1;

    	//To avoid problems in biomass calculations when age at breast height is calculated
    	//using models. In that case d is greater than zero and still t_href is greater than age -> a is assign to zero
    	//Assign to 1 if less than 1
    	if (a<1)
       		a=1;
    	//Assing a using d value
	if (d<2.0 && a<2)
	    	a=2;
	else if (d>=2.0 && d<3.0 && a<3)
	    	a=3;
	else if (d>=3.0 && d<4.0 && a<4)
	    	a=4;
	else if (d>=4.0 && d<5.0 && a<5)
	    	a=5;
	else if (d>=5.0 && d<6.0 && a<6)
	    	a=6;
	else if (d>=6.0 && d<7.0 && a<7)
	    	a=7;
	else if (d>=7.0 && d<8.0 && a<8)
	    	a=8;
	else if (d>=8.0 && d<9.0 && a<9)
	    	a=9;
	else if (d>=9.0 && d<10.0 && a<10)
	    	a=10;
	else if (d>=10.0 && d<11.0 && a<11)
	    	a=11;
	else if (d>=11.0 && d<13.0 && a<13)
	    	a=13;
	else if (d>=13.0 && d<15.0 && a<16)
	    	a=16;
	else if (d>=15.0 && d<17.0 && a<19)
	    	a=19;
	else if (d>=17.0 && a<23)
	    	a=23;

	//Calculate log volume reduction for trees
	//Pine
	if (SP==1||SP==8)
	{
    		//For northern Finland
    		if (MANAGEMENT_REGION==3)
   		{
                ret = Log_volume_reduction_pine_north_Finland_Mehtatalo(a, d, LONG, ALT, TS, PEAT, SC, LAT, nres,
                                                                        modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
   		//For scrubland
   		else if (SCRUB==1||WASTE==1)
   		{
                   ret = Log_volume_reduction_pine_scrub_land_Mehtatalo(a, d, ALT, LAT, nres,
                   							modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
   		//For all the rest pine stands
   		else
   		{
        	   ret = Log_volume_reduction_pine_Mehtatalo(a, d, LAT, ALT, AHV, PEAT, SC, nres,
        	   						modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
		if (ret == 0)
   			return ret;
   	}
   	//Spruce
	else if (SP==2)
    	{
            	//For scrubland in northern Finland
    		if (MANAGEMENT_REGION==3&&(SCRUB==1||WASTE==1))
   		{
                	ret = Log_volume_reduction_spruce_scrub_land_north_Finland_Mehtatalo(a, d, nres,
                										modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
   			//For northern Finland
   		else if (MANAGEMENT_REGION==3)
   		{
                	ret = Log_volume_reduction_spruce_north_Finland_Mehtatalo(a, d, LONG, ALT, TS, PEAT, LAT, nres,
                								  modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
   		//For all the rest spruce stands
   		else
   		{
                	ret = Log_volume_reduction_spruce_Mehtatalo(a, d, PLANT, ALT, AHV, TS, PEAT, SC, nres,
                							modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
   		}
		if (ret == 0)
   			return ret;
    	}
	//White birch
    	else if (SP==3)
    	{
         	ret = Log_volume_reduction_white_birch_Mehtatalo(a, d, LAT, ALT, AHV, POHJ, SC, nres,
         							modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
 	 	if (ret == 0)
   			return ret;
     	}
    //Aspen
    else if (SP==5)
	{
         	ret = Log_volume_reduction_aspen_Mehtatalo(a, d, LAT, LONG, ALT, POHJ, PEAT, nres,
         						   modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
 	     	if (ret == 0)
   		    return ret;
     	}
	//Pubescent birch and other deciduous dominated stand
    else
    {
         ret = Log_volume_reduction_pubescent_birch_Mehtatalo(a, d, LAT, AHV, nres,
         						      modelresult, errors, errorCheckMode, allowedRiskLevel, 1.0);
 	 if (ret == 0)
   	 	return ret;
     }

    *modelresult = 1.0 - *modelresult;

    return ret;

}

int Calibration_age_JKK (double Age, double SP, double D_gM, double t_href, double TS, double SC, double PEAT, double SCRUB, double WASTE, double NF,
							int *nres, double *modelresult, char *errors,
							int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;

	double d=D_gM;

	*nres = 1;

	if (SP==1||SP==8) {
        //Scots pine in southern Finland
		if(NF==0) {
            Age_pine_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Scots pine in northern Finland
		else {
			Age_pine_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}
	else if(SP==2) {
        //Norway spruce in southern Finland
		if(NF==0) {
			Age_spruce_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce in northern Finland
		else {
            Age_spruce_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}
	else {
        //Birch or other deciduous in southern Finland
		if(NF==0) {
			Age_birch_SF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Birch or other deciduous in northern Finland
		else {
			Age_birch_NF_JKK(d, TS, SC, PEAT, SCRUB, WASTE, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
	}

	*modelresult = Age/(*modelresult+t_href);
	return ret;
}

int Calibration_height_JKK (double Age, double BA, double D_gM, double H_gM, double H_dom, double SP,
				double TS, double ALT, double SC, double PEAT, double PEAT_transf,
				int *nres, double *modelresult, char *errors,
				int errorCheckMode, double allowedRiskLevel, double rectFactor) {
    int ret = 1;

	double d=D_gM;

	*nres = 1;

	//Mineral soil
	if (PEAT==0)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Silver birch
		else if (SP==3)
		{
			ret = Height_white_birch_Veltheim(d, Age, BA, D_gM, TS, ALT, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Downy birch, aspen and other deciduous species
		else if (SP==4||SP==5||SP==9)
		{
			ret = Height_pubescent_birch_Veltheim(d, Age, BA, D_gM, TS, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
			if (ret == 0)
			return ret;
		}
	//Peatland
	else if (PEAT==1)
	{
		//Scots pine and other conifers
		if (SP==1||SP==8)
		{
			ret = Height_pine_peatland_Veltheim(d, Age, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Norway spruce
		else if (SP==2)
		{
			ret = Height_spruce_peatland_Veltheim(d, BA, D_gM, TS, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Birches and other deciduous species
		else if (SP==3||SP==4||SP==5||SP==9)
		{
			ret = Height_birch_peatland_Veltheim(d, BA, D_gM, TS, ALT, PEAT_transf, SC, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}
		//Alder
		else
		{
			ret = Height_grey_alder_Karki(d, H_dom, nres, modelresult, errors, errorCheckMode, allowedRiskLevel, 1);
		}

		if (ret == 0)
			return ret;

	}

	*modelresult = H_gM / *modelresult;
	return ret;
}
